@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2022 Mario Luzeiro <mrluzeiro@ua.pt>
- * Copyright (C) 2015-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN
+ * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,9 +42,10 @@
 #include <footprint.h>
 #include <fp_lib_table.h>
 #include <eda_3d_viewer_frame.h>
+#include <project_pcb.h>
 
 #include <base_units.h>
-#include <profile.h>        // To use GetRunningMicroSecs or another profiling utility
+#include <core/profile.h>        // To use GetRunningMicroSecs or another profiling utility
 
 /**
  * Perform an interpolation step to easy control the transparency based on the
@@ -234,7 +236,9 @@ void RENDER_3D_RAYTRACE::createItemsFromContainer( const BVH_CONTAINER_2D* aCont
     if( aContainer2d == nullptr )
         return;
 
-    const LIST_OBJECT2D& listObject2d = aContainer2d->GetList();
+    EDA_3D_VIEWER_SETTINGS::RENDER_SETTINGS& cfg = m_boardAdapter.m_Cfg->m_Render;
+    bool                                     isSilk = aLayer_id == B_SilkS || aLayer_id == F_SilkS;
+    const LIST_OBJECT2D&                     listObject2d = aContainer2d->GetList();
 
     if( listObject2d.size() == 0 )
         return;
@@ -273,11 +277,8 @@ void RENDER_3D_RAYTRACE::createItemsFromContainer( const BVH_CONTAINER_2D* aCont
             // layer and the flag is set, then clip the silk at the outer edge of the annular ring,
             // rather than the at the outer edge of the copper plating.
             const BVH_CONTAINER_2D& throughHoleOuter =
-                    m_boardAdapter.m_Cfg->m_Render.clip_silk_on_via_annulus
-                        && m_boardAdapter.m_Cfg->m_Render.realistic
-                        && ( aLayer_id == B_SilkS || aLayer_id == F_SilkS ) ?
-                                                    m_boardAdapter.GetThroughHoleAnnularRings() :
-                                                    m_boardAdapter.GetThroughHoleOds();
+                        cfg.clip_silk_on_via_annuli && isSilk ? m_boardAdapter.GetViaAnnuli()
+                                                              : m_boardAdapter.GetTH_ODs();
 
             if( !throughHoleOuter.GetList().empty() )
             {
@@ -303,14 +304,13 @@ void RENDER_3D_RAYTRACE::createItemsFromContainer( const BVH_CONTAINER_2D* aCont
 
         const MAP_CONTAINER_2D_BASE& mapLayers = m_boardAdapter.GetLayerMap();
 
-        if( m_boardAdapter.m_Cfg->m_Render.subtract_mask_from_silk
-            && m_boardAdapter.m_Cfg->m_Render.realistic
+        if( cfg.subtract_mask_from_silk
             && (    ( aLayer_id == B_SilkS && mapLayers.find( B_Mask ) != mapLayers.end() )
                  || ( aLayer_id == F_SilkS && mapLayers.find( F_Mask ) != mapLayers.end() ) ) )
         {
-            const PCB_LAYER_ID layerMask_id = ( aLayer_id == B_SilkS ) ? B_Mask : F_Mask;
+            const PCB_LAYER_ID maskLayer = ( aLayer_id == B_SilkS ) ? B_Mask : F_Mask;
 
-            const BVH_CONTAINER_2D* containerMaskLayer2d = mapLayers.at( layerMask_id );
+            const BVH_CONTAINER_2D* containerMaskLayer2d = mapLayers.at( maskLayer );
 
             CONST_LIST_OBJECT2D intersecting;
 
@@ -330,8 +330,8 @@ void RENDER_3D_RAYTRACE::createItemsFromContainer( const BVH_CONTAINER_2D* aCont
         if( ( object2d_B == CSGITEM_EMPTY ) && ( object2d_C == CSGITEM_FULL ) )
         {
             LAYER_ITEM* objPtr = new LAYER_ITEM( object2d_A,
-                    m_boardAdapter.GetLayerBottomZPos( aLayer_id ) - aLayerZOffset,
-                    m_boardAdapter.GetLayerTopZPos( aLayer_id ) + aLayerZOffset );
+                                    m_boardAdapter.GetLayerBottomZPos( aLayer_id ) - aLayerZOffset,
+                                    m_boardAdapter.GetLayerTopZPos( aLayer_id ) + aLayerZOffset );
             objPtr->SetMaterial( aMaterialLayer );
             objPtr->SetColor( ConvertSRGBToLinear( aLayerColor ) );
             m_objectContainer.Add( objPtr );
@@ -343,8 +343,8 @@ void RENDER_3D_RAYTRACE::createItemsFromContainer( const BVH_CONTAINER_2D* aCont
             m_containerWithObjectsToDelete.Add( itemCSG2d );
 
             LAYER_ITEM* objPtr = new LAYER_ITEM( itemCSG2d,
-                    m_boardAdapter.GetLayerBottomZPos( aLayer_id ) - aLayerZOffset,
-                    m_boardAdapter.GetLayerTopZPos( aLayer_id ) + aLayerZOffset );
+                                    m_boardAdapter.GetLayerBottomZPos( aLayer_id ) - aLayerZOffset,
+                                    m_boardAdapter.GetLayerTopZPos( aLayer_id ) + aLayerZOffset );
 
             objPtr->SetMaterial( aMaterialLayer );
             objPtr->SetColor( ConvertSRGBToLinear( aLayerColor ) );
@@ -368,7 +368,7 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     OBJECT_2D_STATS::Instance().ResetStats();
     OBJECT_3D_STATS::Instance().ResetStats();
 
-    unsigned stats_startReloadTime = GetRunningMicroSecs();
+    int64_t stats_startReloadTime = GetRunningMicroSecs();
 
     if( !aOnlyLoadCopperAndShapes )
     {
@@ -393,6 +393,8 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     m_outlineBoard2dObjects     = new CONTAINER_2D;
     m_antioutlineBoard2dObjects = new BVH_CONTAINER_2D;
 
+    std::bitset<LAYER_3D_END> layerFlags = m_boardAdapter.GetVisibleLayers();
+
     if( !aOnlyLoadCopperAndShapes )
     {
         const int outlineCount = m_boardAdapter.GetBoardPoly().OutlineCount();
@@ -427,14 +429,14 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
 
             boardPolyCopy.Fracture( SHAPE_POLY_SET::PM_FAST );
 
-            for( int ii = 0; ii < outlineCount; ii++ )
+            for( int ii = 0; ii < boardPolyCopy.OutlineCount(); ii++ )
             {
                 ConvertPolygonToBlocks( boardPolyCopy, *m_outlineBoard2dObjects,
                                         m_boardAdapter.BiuTo3dUnits(), divFactor,
                                         *m_boardAdapter.GetBoard(), ii );
             }
 
-            if( m_boardAdapter.m_Cfg->m_Render.show_board_body )
+            if( layerFlags.test( LAYER_3D_BOARD ) )
             {
                 const LIST_OBJECT2D& listObjects = m_outlineBoard2dObjects->GetList();
 
@@ -443,9 +445,9 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
                     std::vector<const OBJECT_2D*>* object2d_B = new std::vector<const OBJECT_2D*>();
 
                     // Check if there are any THT that intersects this outline object part
-                    if( !m_boardAdapter.GetThroughHoleOds().GetList().empty() )
+                    if( !m_boardAdapter.GetTH_ODs().GetList().empty() )
                     {
-                        const BVH_CONTAINER_2D& throughHoles = m_boardAdapter.GetThroughHoleOds();
+                        const BVH_CONTAINER_2D& throughHoles = m_boardAdapter.GetTH_ODs();
                         CONST_LIST_OBJECT2D     intersecting;
 
                         throughHoles.GetIntersectingObjects( object2d_A->GetBBox(), intersecting );
@@ -509,9 +511,9 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
                 // a polygon or dummy block) it will cut also the render of the hole.
                 // So this will add a full hole.
                 // In fact, that is not need if the hole have copper.
-                if( !m_boardAdapter.GetThroughHoleOds().GetList().empty() )
+                if( !m_boardAdapter.GetTH_ODs().GetList().empty() )
                 {
-                    const LIST_OBJECT2D& holeList = m_boardAdapter.GetThroughHoleOds().GetList();
+                    const LIST_OBJECT2D& holeList = m_boardAdapter.GetTH_ODs().GetList();
 
                     for( const OBJECT_2D* hole2d : holeList )
                     {
@@ -586,40 +588,33 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
         case B_Paste:
         case F_Paste:
             materialLayer = &m_materials.m_Paste;
-
-            if( m_boardAdapter.m_Cfg->m_Render.realistic )
-                layerColor = m_boardAdapter.m_SolderPasteColor;
-            else
-                layerColor = m_boardAdapter.GetLayerColor( layer_id );
-
+            layerColor = m_boardAdapter.m_SolderPasteColor;
             break;
 
         case B_SilkS:
             materialLayer = &m_materials.m_SilkS;
-
-            if( m_boardAdapter.m_Cfg->m_Render.realistic )
-                layerColor = m_boardAdapter.m_SilkScreenColorBot;
-            else
-                layerColor = m_boardAdapter.GetLayerColor( layer_id );
-
+            layerColor = m_boardAdapter.m_SilkScreenColorBot;
             break;
 
         case F_SilkS:
             materialLayer = &m_materials.m_SilkS;
-
-            if( m_boardAdapter.m_Cfg->m_Render.realistic )
-                layerColor = m_boardAdapter.m_SilkScreenColorTop;
-            else
-                layerColor = m_boardAdapter.GetLayerColor( layer_id );
-
+            layerColor = m_boardAdapter.m_SilkScreenColorTop;
             break;
 
         case Dwgs_User:
+            layerColor = m_boardAdapter.m_UserDrawingsColor;
+            break;
+
         case Cmts_User:
+            layerColor = m_boardAdapter.m_UserCommentsColor;
+            break;
+
         case Eco1_User:
+            layerColor = m_boardAdapter.m_ECO1Color;
+            break;
+
         case Eco2_User:
-        case Edge_Cuts:
-        case Margin:
+            layerColor = m_boardAdapter.m_ECO2Color;
             break;
 
         case B_CrtYd:
@@ -631,22 +626,15 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
             break;
 
         default:
-            if( m_boardAdapter.m_Cfg->m_Render.realistic )
+            if( m_boardAdapter.m_Cfg->m_Render.differentiate_plated_copper )
             {
-                if( m_boardAdapter.m_Cfg->m_Render.renderPlatedPadsAsPlated )
-                {
-                    layerColor = SFVEC3F( 184.0f / 255.0f, 115.0f / 255.0f, 50.0f / 255.0f );
-                    materialLayer = &m_materials.m_NonPlatedCopper;
-                }
-                else
-                {
-                    layerColor = m_boardAdapter.m_CopperColor;
-                    materialLayer = &m_materials.m_Copper;
-                }
+                layerColor = SFVEC3F( 184.0f / 255.0f, 115.0f / 255.0f, 50.0f / 255.0f );
+                materialLayer = &m_materials.m_NonPlatedCopper;
             }
             else
             {
-                layerColor = m_boardAdapter.GetLayerColor( layer_id );
+                layerColor = m_boardAdapter.m_CopperColor;
+                materialLayer = &m_materials.m_Copper;
             }
 
             break;
@@ -656,8 +644,7 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     } // for each layer on map
 
     // Create plated copper
-    if( m_boardAdapter.m_Cfg->m_Render.renderPlatedPadsAsPlated
-            && m_boardAdapter.m_Cfg->m_Render.realistic )
+    if( m_boardAdapter.m_Cfg->m_Render.differentiate_plated_copper )
     {
         createItemsFromContainer( m_boardAdapter.GetPlatedPadsFront(), F_Cu, &m_materials.m_Copper,
                                   m_boardAdapter.m_CopperColor,
@@ -674,8 +661,9 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
         // Solder mask layers are "negative" layers so the elements that we have in the container
         // should remove the board outline. We will check for all objects in the outline if it
         // intersects any object in the layer container and also any hole.
-        if( m_boardAdapter.m_Cfg->m_Render.show_soldermask
-                && !m_outlineBoard2dObjects->GetList().empty() )
+        if( ( layerFlags.test( LAYER_3D_SOLDERMASK_TOP )
+                || layerFlags.test( LAYER_3D_SOLDERMASK_BOTTOM ) )
+            && !m_outlineBoard2dObjects->GetList().empty() )
         {
             const MATERIAL* materialLayer = &m_materials.m_SolderMask;
 
@@ -689,22 +677,22 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
                     continue;
 
                 // Only get the Solder mask layers (and only if the board has them)
-                if( !( layer_id == B_Mask || layer_id == F_Mask ) )
+                if( layer_id == F_Mask && !layerFlags.test( LAYER_3D_SOLDERMASK_TOP ) )
+                    continue;
+
+                if( layer_id == B_Mask && !layerFlags.test( LAYER_3D_SOLDERMASK_BOTTOM ) )
+                    continue;
+
+                // Only Mask layers are processed here because they are negative layers
+                if( layer_id != F_Mask && layer_id != B_Mask )
                     continue;
 
                 SFVEC3F layerColor;
 
-                if( m_boardAdapter.m_Cfg->m_Render.realistic )
-                {
-                    if( layer_id == B_Mask )
-                        layerColor = m_boardAdapter.m_SolderMaskColorBot;
-                    else
-                        layerColor = m_boardAdapter.m_SolderMaskColorTop;
-                }
+                if( layer_id == B_Mask )
+                    layerColor = m_boardAdapter.m_SolderMaskColorBot;
                 else
-                {
-                    layerColor = m_boardAdapter.GetLayerColor( layer_id );
-                }
+                    layerColor = m_boardAdapter.m_SolderMaskColorTop;
 
                 const float zLayerMin = m_boardAdapter.GetLayerBottomZPos( layer_id );
                 const float zLayerMax = m_boardAdapter.GetLayerTopZPos( layer_id );
@@ -715,9 +703,9 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
                     std::vector<const OBJECT_2D*>* object2d_B = new std::vector<const OBJECT_2D*>();
 
                     // Check if there are any THT that intersects this outline object part
-                    if( !m_boardAdapter.GetThroughHoleOds().GetList().empty() )
+                    if( !m_boardAdapter.GetTH_ODs().GetList().empty() )
                     {
-                        const BVH_CONTAINER_2D& throughHoles = m_boardAdapter.GetThroughHoleOds();
+                        const BVH_CONTAINER_2D& throughHoles = m_boardAdapter.GetTH_ODs();
                         CONST_LIST_OBJECT2D     intersecting;
 
                         throughHoles.GetIntersectingObjects( object2d_A->GetBBox(), intersecting );
@@ -783,8 +771,8 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     }
 
 #ifdef PRINT_STATISTICS_3D_VIEWER
-    unsigned stats_endConvertTime        = GetRunningMicroSecs();
-    unsigned stats_startLoad3DmodelsTime = stats_endConvertTime;
+    int64_t stats_endConvertTime = GetRunningMicroSecs();
+    int64_t stats_startLoad3DmodelsTime = stats_endConvertTime;
 #endif
 
     if( aStatusReporter )
@@ -793,7 +781,7 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     load3DModels( m_objectContainer, aOnlyLoadCopperAndShapes );
 
 #ifdef PRINT_STATISTICS_3D_VIEWER
-    unsigned stats_endLoad3DmodelsTime = GetRunningMicroSecs();
+    int64_t stats_endLoad3DmodelsTime = GetRunningMicroSecs();
 #endif
 
     if( !aOnlyLoadCopperAndShapes )
@@ -931,10 +919,33 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     {
         float ratio =
                 std::max( 1.0f, m_objectContainer.GetBBox().GetMaxDimension() / RANGE_SCALE_3D );
-        m_camera.SetMaxZoom( CAMERA::DEFAULT_MAX_ZOOM * ratio );
 
-        m_camera.SetMinZoom( static_cast<float>( MIN_DISTANCE_IU * m_boardAdapter.BiuTo3dUnits()
-                                                 / -m_camera.GetCameraInitPos().z ) );
+        float max_zoom = CAMERA::DEFAULT_MAX_ZOOM * ratio;
+        float min_zoom = static_cast<float>( MIN_DISTANCE_IU * m_boardAdapter.BiuTo3dUnits()
+                                             / -m_camera.GetCameraInitPos().z );
+
+        if( min_zoom > max_zoom )
+            std::swap( min_zoom, max_zoom );
+
+        float zoom_ratio = max_zoom / min_zoom;
+
+        // Set the minimum number of zoom 'steps' between max and min.
+        int steps = 3 * 3;
+        steps -= static_cast<int>( ceil( log( zoom_ratio ) / log( 1.26f ) ) );
+        steps = std::max( steps, 0 );
+
+        // Resize max and min zoom to accomplish the number of steps.
+        float increased_zoom = pow( 1.26f, steps / 2 );
+        max_zoom *= increased_zoom;
+        min_zoom /= increased_zoom;
+
+        if( steps & 1 )
+            min_zoom /= 1.26f;
+
+        min_zoom = std::min( min_zoom, 1.0f );
+
+        m_camera.SetMaxZoom( max_zoom );
+        m_camera.SetMinZoom( min_zoom );
     }
 
     // Create an accelerator
@@ -944,7 +955,7 @@ void RENDER_3D_RAYTRACE::Reload( REPORTER* aStatusReporter, REPORTER* aWarningRe
     if( aStatusReporter )
     {
         // Calculation time in seconds
-        double calculation_time = (double) GetRunningMicroSecs() - stats_startReloadTime / 1e6;
+        double calculation_time = (double) ( GetRunningMicroSecs() - stats_startReloadTime ) / 1e6;
 
         aStatusReporter->Report( wxString::Format( _( "Reload time %.3f s" ), calculation_time ) );
     }
@@ -976,15 +987,7 @@ void RENDER_3D_RAYTRACE::insertHole( const PCB_VIA* aVia )
     LAYER_ITEM* objPtr = new LAYER_ITEM( ring, topZ, botZ );
 
     objPtr->SetMaterial( &m_materials.m_Copper );
-
-    if( m_boardAdapter.m_Cfg->m_Render.realistic )
-        objPtr->SetColor( ConvertSRGBToLinear( m_boardAdapter.m_CopperColor ) );
-    else if( aVia->GetViaType() == VIATYPE::MICROVIA )
-        objPtr->SetColor( ConvertSRGBToLinear( m_boardAdapter.GetItemColor( LAYER_VIA_MICROVIA ) ) );
-    else if( aVia->GetViaType() == VIATYPE::BLIND_BURIED )
-        objPtr->SetColor( ConvertSRGBToLinear( m_boardAdapter.GetItemColor( LAYER_VIA_BBLIND ) ) );
-    else
-        objPtr->SetColor( ConvertSRGBToLinear( m_boardAdapter.GetItemColor( LAYER_VIAS ) ) );
+    objPtr->SetColor( ConvertSRGBToLinear( m_boardAdapter.m_CopperColor ) );
 
     m_objectContainer.Add( objPtr );
 }
@@ -994,13 +997,7 @@ void RENDER_3D_RAYTRACE::insertHole( const PAD* aPad )
 {
     const OBJECT_2D* object2d_A = nullptr;
 
-    SFVEC3F objColor;
-
-    if( m_boardAdapter.m_Cfg->m_Render.realistic )
-        objColor = m_boardAdapter.m_CopperColor;
-    else
-        objColor = m_boardAdapter.GetItemColor( LAYER_PADS_TH );
-
+    SFVEC3F        objColor = m_boardAdapter.m_CopperColor;
     const VECTOR2I drillsize = aPad->GetDrillSize();
     const bool     hasHole = drillsize.x && drillsize.y;
 
@@ -1119,12 +1116,11 @@ void RENDER_3D_RAYTRACE::insertHole( const PAD* aPad )
 
         // Check if there are any other THT that intersects this hole
         // It will use the non inflated holes
-        if( !m_boardAdapter.GetThroughHoleIds().GetList().empty() )
+        if( !m_boardAdapter.GetTH_IDs().GetList().empty() )
         {
             CONST_LIST_OBJECT2D intersecting;
 
-            m_boardAdapter.GetThroughHoleIds().GetIntersectingObjects( object2d_A->GetBBox(),
-                                                                       intersecting );
+            m_boardAdapter.GetTH_IDs().GetIntersectingObjects( object2d_A->GetBBox(), intersecting );
 
             for( const OBJECT_2D* hole2d : intersecting )
             {
@@ -1247,8 +1243,6 @@ void RENDER_3D_RAYTRACE::load3DModels( CONTAINER_3D& aDstContainer, bool aSkipMa
                     fpMatrix, SFVEC3F( modelunit_to_3d_units_factor, modelunit_to_3d_units_factor,
                                        modelunit_to_3d_units_factor ) );
 
-            BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( fp );
-
             // Get the list of model files for this model
             S3D_CACHE* cacheMgr = m_boardAdapter.Get3dCacheManager();
             auto       sM       = fp->Models().begin();
@@ -1263,8 +1257,8 @@ void RENDER_3D_RAYTRACE::load3DModels( CONTAINER_3D& aDstContainer, bool aSkipMa
                 {
                     // FindRow() can throw an exception
                     const FP_LIB_TABLE_ROW* fpRow =
-                            m_boardAdapter.GetBoard()->GetProject()->PcbFootprintLibs()->FindRow(
-                                    libraryName, false );
+                        PROJECT_PCB::PcbFootprintLibs( m_boardAdapter.GetBoard()->GetProject() )
+                                ->FindRow( libraryName, false );
 
                     if( fpRow )
                         footprintBasePath = fpRow->GetFullURI( true );
@@ -1308,7 +1302,7 @@ void RENDER_3D_RAYTRACE::load3DModels( CONTAINER_3D& aDstContainer, bool aSkipMa
                                 SFVEC3F( sM->m_Scale.x, sM->m_Scale.y, sM->m_Scale.z ) );
 
                         addModels( aDstContainer, modelPtr, modelMatrix, (float) sM->m_Opacity,
-                                   aSkipMaterialInformation, boardItem );
+                                   aSkipMaterialInformation, fp );
                     }
                 }
 
