@@ -4,7 +4,7 @@
  * Copyright (C) 2016 Mario Luzeiro <mrluzeiro@ua.pt>
  * Copyright (C) 2015 Cirilo Bernardo <cirilo.bernardo@gmail.com>
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2015-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
  */
 
 #include "panel_preview_3d_model.h"
+#include <dialogs/dialog_unit_entry.h>
 #include <3d_canvas/eda_3d_canvas.h>
 #include <tool/tool_manager.h>
 #include <tool/tool_dispatcher.h>
@@ -34,26 +35,29 @@
 #include <bitmaps.h>
 #include <board.h>
 #include <common_ogl/ogl_attr_list.h>
-#include <gal/dpi_scaling.h>
+#include <dpi_scaling_common.h>
 #include <pgm_base.h>
+#include <project_pcb.h>
 #include <settings/common_settings.h>
 #include <settings/settings_manager.h>
 #include <widgets/wx_infobar.h>
 #include <eda_3d_viewer_settings.h>
 #include <board_design_settings.h>
 
+#include <3d_navlib/nl_footprint_properties_plugin.h>
+
 PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAME* aFrame,
                                                 FOOTPRINT* aFootprint,
                                                 std::vector<FP_3DMODEL>* aParentModelList ) :
         PANEL_PREVIEW_3D_MODEL_BASE( aParent, wxID_ANY ),
+        m_parentFrame( aFrame ),
         m_previewPane( nullptr ),
         m_infobar( nullptr ),
         m_boardAdapter(),
         m_currentCamera( m_trackBallCamera ),
-        m_trackBallCamera( 2 * RANGE_SCALE_3D ),
-        m_boardThickness( aFrame, nullptr, m_boardThicknessCtrl, m_boardThicknessUnits )
+        m_trackBallCamera( 2 * RANGE_SCALE_3D )
 {
-    m_userUnits = aFrame->GetUserUnits();
+    m_userUnits = m_parentFrame->GetUserUnits();
 
     m_dummyBoard = new BOARD();
 
@@ -62,6 +66,8 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     // This board will only be used to hold a footprint for viewing
     m_dummyBoard->SetBoardUse( BOARD_USE::FPHOLDER );
 
+    m_bodyStyleShowAll = true;
+
     BOARD_DESIGN_SETTINGS parent_bds = aFrame->GetDesignSettings();
     BOARD_DESIGN_SETTINGS& dummy_bds = m_dummyBoard->GetDesignSettings();
     dummy_bds.SetBoardThickness( parent_bds.GetBoardThickness() );
@@ -75,14 +81,16 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     m_previewLabel->SetFont( KIUI::GetStatusFont( this ) );
 
     // Set the bitmap of 3D view buttons:
-    m_bpvTop->SetBitmap( KiBitmap( BITMAPS::axis3d_top ) );
-    m_bpvFront->SetBitmap( KiBitmap( BITMAPS::axis3d_front ) );
-    m_bpvBack->SetBitmap( KiBitmap( BITMAPS::axis3d_back ) );
-    m_bpvLeft->SetBitmap( KiBitmap( BITMAPS::axis3d_left ) );
-    m_bpvRight->SetBitmap( KiBitmap( BITMAPS::axis3d_right ) );
-    m_bpvBottom->SetBitmap( KiBitmap( BITMAPS::axis3d_bottom ) );
-    m_bpvISO->SetBitmap( KiBitmap( BITMAPS::ortho ) );
-    m_bpUpdate->SetBitmap( KiBitmap( BITMAPS::reload ) );
+    m_bpvTop->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_top ) );
+    m_bpvFront->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_front ) );
+    m_bpvBack->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_back ) );
+    m_bpvLeft->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_left ) );
+    m_bpvRight->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_right ) );
+    m_bpvBottom->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_bottom ) );
+    m_bpvISO->SetBitmap( KiBitmapBundle( BITMAPS::ortho ) );
+    m_bpvBodyStyle->SetBitmap( KiBitmapBundle( BITMAPS::axis3d ) );
+    m_bpUpdate->SetBitmap( KiBitmapBundle( BITMAPS::reload ) );
+    m_bpSettings->SetBitmap( KiBitmapBundle( BITMAPS::options_3drender ) );
 
     // Set the min and max values of spin buttons (mandatory on Linux)
     // They are not used, so they are set to min and max 32 bits int values
@@ -103,13 +111,24 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
 
     m_dummyFootprint = new FOOTPRINT( *aFootprint );
     m_dummyFootprint->SetParentGroup( nullptr );
+
+    // Ensure the footprint is shown like in Fp editor: rot 0, not flipped
+    // to avoid mistakes when setting the3D shape position/rotation
+    if( m_dummyFootprint->IsFlipped() )
+        m_dummyFootprint->Flip( m_dummyFootprint->GetPosition(), false );
+
+    m_dummyFootprint->SetOrientation( ANGLE_0 );
+
+
     m_dummyBoard->Add( m_dummyFootprint );
 
     // Create the 3D canvas
     m_previewPane = new EDA_3D_CANVAS( this,
                                        OGL_ATT_LIST::GetAttributesList( ANTIALIASING_MODE::AA_8X ),
-                                       m_boardAdapter, m_currentCamera,
-                                       aFrame->Prj().Get3DCacheManager() );
+                                       m_boardAdapter, m_currentCamera, PROJECT_PCB::Get3DCacheManager( &aFrame->Prj() ) );
+
+    m_spaceMouse = new NL_FOOTPRINT_PROPERTIES_PLUGIN( m_previewPane );
+    m_spaceMouse->SetFocus( true );
 
     m_boardAdapter.SetBoard( m_dummyBoard );
     m_boardAdapter.m_IsBoardView = false;
@@ -144,9 +163,12 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
                  this );
     }
 
-    aFrame->Connect( UNITS_CHANGED, wxCommandEventHandler( PANEL_PREVIEW_3D_MODEL::onUnitsChanged ),
+    aFrame->Connect( EDA_EVT_UNITS_CHANGED,
+                     wxCommandEventHandler( PANEL_PREVIEW_3D_MODEL::onUnitsChanged ),
                      nullptr, this );
 
+    Bind( wxCUSTOM_PANEL_SHOWN_EVENT, &PANEL_PREVIEW_3D_MODEL::onPanelShownEvent, this );
+
 #ifdef __WXOSX__
     // Call layout once to get the proper button sizes after the bitmaps have been set
     Layout();
@@ -164,12 +186,18 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     m_bpvBottom->SetMinSize( m_bpvBottom->GetSize() + borderFix );
     m_bpvISO->SetMinSize( m_bpvISO->GetSize() + borderFix );
     m_bpUpdate->SetMinSize( m_bpUpdate->GetSize() + borderFix );
+    m_bpSettings->SetMinSize( m_bpSettings->GetSize() + borderFix );
 #endif
 }
 
 
 PANEL_PREVIEW_3D_MODEL::~PANEL_PREVIEW_3D_MODEL()
 {
+    // Restore the 3D viewer Render settings, that can be modified by the panel tools
+    if( m_boardAdapter.m_Cfg )
+        m_boardAdapter.m_Cfg->m_Render = m_initialRender;
+
+    delete m_spaceMouse;
     delete m_dummyBoard;
     delete m_previewPane;
 }
@@ -190,45 +218,30 @@ void PANEL_PREVIEW_3D_MODEL::loadSettings()
 
     COMMON_SETTINGS* settings = Pgm().GetCommonSettings();
 
-    const DPI_SCALING dpi{ settings, this };
-    m_previewPane->SetScaleFactor( dpi.GetScaleFactor() );
-
     // TODO(JE) use all control options
     m_boardAdapter.m_MousewheelPanning = settings->m_Input.scroll_modifier_zoom != 0;
 
-    COLOR_SETTINGS* colors = Pgm().GetSettingsManager().GetColorSettings();
-
-    if( colors )
-    {
-        auto set =
-                [] ( const COLOR4D& aColor, SFVEC4F& aTarget )
-                {
-                    aTarget.r = aColor.r;
-                    aTarget.g = aColor.g;
-                    aTarget.b = aColor.b;
-                    aTarget.a = aColor.a;
-                };
-
-        set( colors->GetColor( LAYER_3D_BACKGROUND_BOTTOM ), m_boardAdapter.m_BgColorBot );
-        set( colors->GetColor( LAYER_3D_BACKGROUND_TOP ), m_boardAdapter.m_BgColorTop );
-        set( colors->GetColor( LAYER_3D_BOARD ), m_boardAdapter.m_BoardBodyColor );
-        set( colors->GetColor( LAYER_3D_COPPER ), m_boardAdapter.m_CopperColor );
-        set( colors->GetColor( LAYER_3D_SILKSCREEN_BOTTOM ), m_boardAdapter.m_SilkScreenColorBot );
-        set( colors->GetColor( LAYER_3D_SILKSCREEN_TOP ), m_boardAdapter.m_SilkScreenColorTop );
-        set( colors->GetColor( LAYER_3D_SOLDERMASK_BOTTOM ), m_boardAdapter.m_SolderMaskColorBot );
-        set( colors->GetColor( LAYER_3D_SOLDERMASK_TOP ), m_boardAdapter.m_SolderMaskColorTop );
-        set( colors->GetColor( LAYER_3D_SOLDERPASTE ), m_boardAdapter.m_SolderPasteColor );
-    }
-
-    EDA_3D_VIEWER_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>();
+    auto* cfg = Pgm().GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>( "3d_viewer" );
 
     if( cfg )
     {
+        // Save the 3D viewer render settings, to restore it after closing the preview
+        m_initialRender = cfg->m_Render;
+
         m_boardAdapter.m_Cfg = cfg;
 
         m_previewPane->SetAnimationEnabled( cfg->m_Camera.animation_enabled );
         m_previewPane->SetMovingSpeedMultiplier( cfg->m_Camera.moving_speed_multiplier );
         m_previewPane->SetProjectionMode( cfg->m_Camera.projection_mode );
+
+        // Ensure the board body is always shown, and do not use the settings of the 3D viewer
+        cfg->m_Render.show_copper_top = m_bodyStyleShowAll;
+        cfg->m_Render.show_copper_bottom = m_bodyStyleShowAll;
+        cfg->m_Render.show_soldermask_top = m_bodyStyleShowAll;
+        cfg->m_Render.show_soldermask_bottom = m_bodyStyleShowAll;
+        cfg->m_Render.show_solderpaste = m_bodyStyleShowAll;
+        cfg->m_Render.show_zones = m_bodyStyleShowAll;
+        cfg->m_Render.show_board_body = m_bodyStyleShowAll;
     }
 }
 
@@ -327,9 +340,6 @@ void PANEL_PREVIEW_3D_MODEL::SetSelectedModel( int idx )
 
         m_opacity->SetValue( 100 );
     }
-
-    BOARD_DESIGN_SETTINGS dummy_bds = m_dummyBoard->GetDesignSettings();
-    m_boardThickness.ChangeValue( dummy_bds.GetBoardThickness() );
 }
 
 
@@ -382,16 +392,49 @@ void PANEL_PREVIEW_3D_MODEL::onOpacitySlider( wxCommandEvent& event )
 }
 
 
-void PANEL_PREVIEW_3D_MODEL::updateBoardThickness( wxCommandEvent& event )
+void PANEL_PREVIEW_3D_MODEL::setBodyStyleView( wxCommandEvent& event )
 {
-    BOARD_DESIGN_SETTINGS dummy_bds = m_dummyBoard->GetDesignSettings();
-    dummy_bds.SetBoardThickness( m_boardThickness.GetValue() );
+    // turn ON or OFF options to show the board body if OFF, soder paste, soldermask
+    // and board body are hidden, to allows a good view of the 3D model and its pads.
+    EDA_3D_VIEWER_SETTINGS* cfg = m_boardAdapter.m_Cfg;
 
-    BOARD_STACKUP& dummy_board_stackup = m_dummyBoard->GetDesignSettings().GetStackupDescriptor();
-    dummy_board_stackup.RemoveAll();
-    dummy_board_stackup.BuildDefaultStackupList( &dummy_bds, 2 );
+    if( !cfg )
+        return;
+
+    m_bodyStyleShowAll = !m_bodyStyleShowAll;
+
+    cfg->m_Render.show_soldermask_top = m_bodyStyleShowAll;
+    cfg->m_Render.show_soldermask_bottom = m_bodyStyleShowAll;
+    cfg->m_Render.show_solderpaste = m_bodyStyleShowAll;
+    cfg->m_Render.show_zones = m_bodyStyleShowAll;
+    cfg->m_Render.show_board_body = m_bodyStyleShowAll;
+
+    m_previewPane->ReloadRequest();
+    m_previewPane->Refresh();
+}
+
+
+void PANEL_PREVIEW_3D_MODEL::View3DSettings( wxCommandEvent& event )
+{
+    BOARD_DESIGN_SETTINGS bds = m_dummyBoard->GetDesignSettings();
+    int                   thickness = bds.GetBoardThickness();
+
+    WX_UNIT_ENTRY_DIALOG dlg( m_parentFrame, _( "3D Preview Options" ), _( "Board thickness:" ),
+                              thickness );
+
+    if( dlg.ShowModal() != wxID_OK )
+        return;
+
+    bds.SetBoardThickness( dlg.GetValue() );
+
+    BOARD_STACKUP& boardStackup = m_dummyBoard->GetDesignSettings().GetStackupDescriptor();
+    boardStackup.RemoveAll();
+    boardStackup.BuildDefaultStackupList( &bds, 2 );
 
     UpdateDummyFootprint( true );
+
+    m_previewPane->ReloadRequest();
+    m_previewPane->Refresh();
 }
 
 
@@ -406,10 +449,15 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementScale( wxSpinEvent& event, double aSign
     else if( spinCtrl == m_spinZscale )
         textCtrl = zscale;
 
+    double step = SCALE_INCREMENT;
+
+    if( wxGetMouseState().ShiftDown( ) )
+        step = SCALE_INCREMENT_FINE;
+
     double curr_value = EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, EDA_UNITS::UNSCALED,
                                                                    textCtrl->GetValue() );
 
-    curr_value += ( SCALE_INCREMENT * aSign );
+    curr_value += ( step * aSign );
     curr_value = std::max( 1/MAX_SCALE, curr_value );
     curr_value = std::min( curr_value, MAX_SCALE );
 
@@ -427,10 +475,15 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementRotation( wxSpinEvent& aEvent, double aS
     else if( spinCtrl == m_spinZrot )
         textCtrl = zrot;
 
+    double step = ROTATION_INCREMENT;
+
+    if( wxGetMouseState().ShiftDown( ) )
+        step = ROTATION_INCREMENT_FINE;
+
     double curr_value = EDA_UNIT_UTILS::UI::DoubleValueFromString( unityScale, EDA_UNITS::DEGREES,
                                                                    textCtrl->GetValue() );
 
-    curr_value += ( ROTATION_INCREMENT * aSign );
+    curr_value += ( step * aSign );
     curr_value = std::max( -MAX_ROTATION, curr_value );
     curr_value = std::min( curr_value, MAX_ROTATION );
 
@@ -450,15 +503,22 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementOffset( wxSpinEvent& event, double aSign
         textCtrl = zoff;
 
     double step_mm = OFFSET_INCREMENT_MM;
-    double curr_value_mm =
-            EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, m_userUnits, textCtrl->GetValue() )
-            / pcbIUScale.IU_PER_MM;
+
+    if( wxGetMouseState().ShiftDown( ) )
+        step_mm = OFFSET_INCREMENT_MM_FINE;
 
     if( m_userUnits == EDA_UNITS::MILS || m_userUnits == EDA_UNITS::INCHES )
     {
         step_mm = 25.4*OFFSET_INCREMENT_MIL/1000;
+
+        if( wxGetMouseState().ShiftDown( ) )
+            step_mm = 25.4*OFFSET_INCREMENT_MIL_FINE/1000;;
     }
 
+    double curr_value_mm = EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, m_userUnits,
+                                                                      textCtrl->GetValue() )
+                           / pcbIUScale.IU_PER_MM;
+
     curr_value_mm += ( step_mm * aSign );
     curr_value_mm = std::max( -MAX_OFFSET, curr_value_mm );
     curr_value_mm = std::min( curr_value_mm, MAX_OFFSET );
@@ -494,10 +554,10 @@ void PANEL_PREVIEW_3D_MODEL::onMouseWheelRot( wxMouseEvent& event )
 {
     wxTextCtrl* textCtrl = (wxTextCtrl*) event.GetEventObject();
 
-    double step = ROTATION_INCREMENT_WHEEL;
+    double step = ROTATION_INCREMENT;
 
     if( event.ShiftDown( ) )
-        step = ROTATION_INCREMENT_WHEEL_FINE;
+        step = ROTATION_INCREMENT_FINE;
 
     if( event.GetWheelRotation() >= 0 )
         step = -step;
@@ -568,6 +628,17 @@ void PANEL_PREVIEW_3D_MODEL::onUnitsChanged( wxCommandEvent& aEvent )
 }
 
 
+void PANEL_PREVIEW_3D_MODEL::onPanelShownEvent( wxCommandEvent& aEvent )
+{
+    if( m_spaceMouse != nullptr )
+    {
+        m_spaceMouse->SetFocus( static_cast<bool>( aEvent.GetInt() ) );
+    }
+
+    aEvent.Skip();
+}
+
+
 void PANEL_PREVIEW_3D_MODEL::UpdateDummyFootprint( bool aReloadRequired )
 {
     m_dummyFootprint->Models().clear();
