@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,9 +22,9 @@
  */
 
 #include <advanced_config.h>
-
+#include <trace_helpers.h>
 #include <config_params.h>
-#include <settings/settings_manager.h>
+#include <paths.h>
 
 #include <wx/app.h>
 #include <wx/config.h>
@@ -56,145 +56,63 @@ namespace AC_STACK
 /**
  * List of known keys for advanced configuration options.
  *
- * Set these options in the file `kicad_advanced` in the
- * KiCad config directory.
+ * Set these options in the file `kicad_advanced` in the KiCad configuration directory.
  */
 namespace AC_KEYS
 {
 
-/**
- * When filling zones, we add an extra amount of clearance to each zone to ensure that rounding
- * errors do not overrun minimum clearance distances.  This is the extra in mm.
- */
+static const wxChar IncrementalConnectivity[] = wxT( "IncrementalConnectivity" );
+static const wxChar Use3DConnexionDriver[] = wxT( "3DConnexionDriver" );
 static const wxChar ExtraFillMargin[] = wxT( "ExtraFillMargin" );
-
-/**
- * A fudge factor for DRC.  Required to prevent false positives due to rounding errors, errors
- * in polygonalization, etc.
- * Previous versions hard-coded various values from 0.000005mm to 0.002mm.
- */
 static const wxChar DRCEpsilon[] = wxT( "DRCEpsilon" );
-
-/**
- * Angle and width tolerances for copper and solder mask sliver detection.
- */
 static const wxChar DRCSliverWidthTolerance[] = wxT( "DRCSliverWidthTolerance" );
 static const wxChar DRCSliverMinimumLength[] = wxT( "DRCSliverMinimumLength" );
 static const wxChar DRCSliverAngleTolerance[] = wxT( "DRCSliverAngleTolerance" );
-
-/**
- * Used to calculate the actual hole size from the finish hole size.
- * IPC-6012 says 0.015-0.018mm; Cadence says at least 0.020mm for a Class 2 board and at least
- * 0.025mm for Class 3.
- */
 static const wxChar HoleWallThickness[] = wxT( "HoleWallPlatingThickness" );
-
-/**
- * Configure the coroutine stack size in bytes.  This should be allocated in multiples of
- * the system page size (n*4096 is generally safe)
- */
 static const wxChar CoroutineStackSize[] = wxT( "CoroutineStackSize" );
-
-/**
- * Show PNS router debug graphics while routing
- */
 static const wxChar ShowRouterDebugGraphics[] = wxT( "ShowRouterDebugGraphics" );
-
-/**
- * When set to true, this will wrap polygon point sets at 4 points per line rather
- * than a single point per line.  Single point per line helps with version control systems
- */
+static const wxChar EnableRouterDump[] = wxT( "EnableRouterDump" );
+static const wxChar HyperZoom[] = wxT( "HyperZoom" );
 static const wxChar CompactFileSave[] = wxT( "CompactSave" );
-
-/**
- * For drawsegments - arcs.
- * Distance from an arc end point and the estimated end point,
- * when rotating from the start point to the end point.
- * 0 will not allow any approximate result, and the arc will not show.
- * Squared value for performances, in system unit.
- */
 static const wxChar DrawArcAccuracy[] = wxT( "DrawArcAccuracy" );
-
-/**
- * For drawsegments - arcs.
- * When drawing an arc, the angle ( center - start ) - ( start - end ),
- * can be limited to avoid extremely high radii.
- * The value is the tan( angle )
- */
 static const wxChar DrawArcCenterStartEndMaxAngle[] = wxT( "DrawArcCenterStartEndMaxAngle" );
-
-/**
- * For arc track interactive drag-resizing
- * Maximum angle between the tangent line of an arc track and a connected straight track
- * in order to commence arc dragging. Units are degrees.
- */
 static const wxChar MaxTangentTrackAngleDeviation[] = wxT( "MaxTangentTrackAngleDeviation" );
-
-/**
- * For arc track interactive drag-resizing
- * Maximum track length to keep after doing an arc track resizing operation. Units are mm.
- */
 static const wxChar MaxTrackLengthToKeep[] = wxT( "MaxTrackLengthToKeep" );
-
-/**
- * When true, GAL will stroke the triangulations (only used in OpenGL) with a visible color
- */
 static const wxChar StrokeTriangulation[] = wxT( "StrokeTriangulation" );
-
-/**
- * When true, a third zone-display-mode is included which strokes the filled areas and fracture
- * boundaries.
- */
 static const wxChar ExtraZoneDisplayModes[] = wxT( "ExtraZoneDisplayModes" );
-
-/**
- * Absolute minimum pen width to send to the plotter.  PDF seems happy enough with 0.0212mm
- * (which equates to 1px @ 1200dpi).
- */
 static const wxChar MinPlotPenWidth[] = wxT( "MinPlotPenWidth" );
-
 static const wxChar DebugZoneFiller[] = wxT( "DebugZoneFiller" );
-
 static const wxChar DebugPDFWriter[] = wxT( "DebugPDFWriter" );
-
-/**
- * The diameter of the drill marks on print and plot outputs (in mm),
- * when the "Drill marks" option is set to "Small mark"
- */
 static const wxChar SmallDrillMarkSize[] = wxT( "SmallDrillMarkSize" );
-
 static const wxChar HotkeysDumper[] = wxT( "HotkeysDumper" );
-
 static const wxChar DrawBoundingBoxes[] = wxT( "DrawBoundingBoxes" );
-
 static const wxChar ShowPcbnewExportNetlist[] = wxT( "ShowPcbnewExportNetlist" );
-
 static const wxChar Skip3DModelFileCache[] = wxT( "Skip3DModelFileCache" );
-
 static const wxChar Skip3DModelMemoryCache[] = wxT( "Skip3DModelMemoryCache" );
-
 static const wxChar HideVersionFromTitle[] = wxT( "HideVersionFromTitle" );
-
 static const wxChar TraceMasks[] = wxT( "TraceMasks" );
-
 static const wxChar ShowRepairSchematic[] = wxT( "ShowRepairSchematic" );
-
 static const wxChar ShowEventCounters[] = wxT( "ShowEventCounters" );
-
 static const wxChar AllowManualCanvasScale[] = wxT( "AllowManualCanvasScale" );
-
 static const wxChar UpdateUIEventInterval[] = wxT( "UpdateUIEventInterval" );
-
-static const wxChar ShowPropertiesPanel[] = wxT( "ShowPropertiesPanel" );
-
 static const wxChar V3DRT_BevelHeight_um[] = wxT( "V3DRT_BevelHeight_um" );
-
 static const wxChar V3DRT_BevelExtentFactor[] = wxT( "V3DRT_BevelExtentFactor" );
-
 static const wxChar UseClipper2[] = wxT( "UseClipper2" );
+static const wxChar EnableGenerators[] = wxT( "EnableGenerators" );
+static const wxChar EnableGit[] = wxT( "EnableGit" );
+static const wxChar EnableEeschemaPrintCairo[] = wxT( "EnableEeschemaPrintCairo" );
+static const wxChar DisambiguationTime[] = wxT( "DisambiguationTime" );
+static const wxChar PcbSelectionVisibilityRatio[] = wxT( "PcbSelectionVisibilityRatio" );
+static const wxChar MinimumSegmentLength[] = wxT( "MinimumSegmentLength" );
+static const wxChar OcePluginLinearDeflection[] = wxT( "OcePluginLinearDeflection" );
+static const wxChar OcePluginAngularDeflection[] = wxT( "OcePluginAngularDeflection" );
+static const wxChar TriangulateSimplificationLevel[] = wxT( "TriangulateSimplificationLevel" );
+static const wxChar TriangulateMinimumArea[] = wxT( "TriangulateMinimumArea" );
+static const wxChar EnableCacheFriendlyFracture[] = wxT( "EnableCacheFriendlyFracture" );
+static const wxChar MinParallelAngle[] = wxT( "MinParallelAngle" );
+static const wxChar ResolveTextRecursionDepth[] = wxT( "ResolveTextRecursionDepth" );
 } // namespace KEYS
 
-
 /**
  * List of known groups for advanced configuration options.
  *
@@ -268,7 +186,7 @@ static void dumpCfg( const std::vector<PARAM_CFG*>& aArray )
 static wxFileName getAdvancedCfgFilename()
 {
     const static wxString cfg_filename{ wxS( "kicad_advanced" ) };
-    return wxFileName( SETTINGS_MANAGER::GetUserSettingsPath(), cfg_filename );
+    return wxFileName( PATHS::GetUserSettingsPath(), cfg_filename );
 }
 
 
@@ -280,6 +198,8 @@ ADVANCED_CFG::ADVANCED_CFG()
     // then the values will remain as set here.
     m_CoroutineStackSize        = AC_STACK::default_stack;
     m_ShowRouterDebugGraphics   = false;
+    m_EnableRouterDump          = false;
+    m_HyperZoom                 = false;
     m_DrawArcAccuracy           = 10.0;
     m_DrawArcCenterMaxAngle     = 50.0;
     m_MaxTangentAngleDeviation  = 1.0;
@@ -291,7 +211,7 @@ ADVANCED_CFG::ADVANCED_CFG()
     m_DRCEpsilon                = 0.0005;   // 0.5um is small enough not to materially violate
                                             // any constraints.
     m_SliverWidthTolerance      = 0.08;
-    m_SliverMinimumLength       = 0.0005;
+    m_SliverMinimumLength       = 0.0008;
     m_SliverAngleTolerance      = 20.0;
 
     m_HoleWallThickness         = 0.020;    // IPC-6012 says 15-18um; Cadence says at least
@@ -314,13 +234,38 @@ ADVANCED_CFG::ADVANCED_CFG()
     m_CompactSave               = false;
     m_UpdateUIEventInterval     = 0;
     m_ShowRepairSchematic       = false;
-    m_ShowPropertiesPanel       = false;
+    m_EnableGenerators          = false;
+    m_EnableGit                 = false;
+
+    m_EnableEeschemaPrintCairo  = true;
 
     m_3DRT_BevelHeight_um       = 30;
     m_3DRT_BevelExtentFactor    = 1.0 / 16.0;
 
     m_UseClipper2               = true;
 
+    m_Use3DConnexionDriver      = true;
+
+    m_IncrementalConnectivity   = true;
+
+    m_DisambiguationMenuDelay   = 500;
+
+    m_PcbSelectionVisibilityRatio = 1.0;
+
+    m_MinimumSegmentLength      = 50;
+
+    m_OcePluginLinearDeflection = 0.14;
+    m_OcePluginAngularDeflection = 30;
+
+    m_TriangulateSimplificationLevel = 50;
+    m_TriangulateMinimumArea = 1000;
+
+    m_EnableCacheFriendlyFracture = true;
+
+    m_MinParallelAngle = 0.001;
+
+    m_ResolveTextRecursionDepth = 3;
+
     loadFromConfigFile();
 }
 
@@ -351,7 +296,8 @@ void ADVANCED_CFG::loadFromConfigFile()
         return;
     }
 
-    wxLogTrace( AdvancedConfigMask, wxS( "Loading advanced config from: %s" ), k_advanced.GetFullPath() );
+    wxLogTrace( AdvancedConfigMask, wxS( "Loading advanced config from: %s" ),
+                k_advanced.GetFullPath() );
 
     wxFileConfig file_cfg( wxS( "" ), wxS( "" ), k_advanced.GetFullPath() );
     loadSettings( file_cfg );
@@ -363,56 +309,76 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
     std::vector<PARAM_CFG*> configParams;
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::ExtraFillMargin,
-                                                  &m_ExtraClearance, m_ExtraClearance, 0.0, 1.0 ) );
+                                                  &m_ExtraClearance,
+                                                  m_ExtraClearance, 0.0, 1.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DRCEpsilon,
                                                   &m_DRCEpsilon, m_DRCEpsilon, 0.0, 1.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DRCSliverWidthTolerance,
-                                                  &m_SliverWidthTolerance, m_SliverWidthTolerance, 0.01, 0.25 ) );
+                                                  &m_SliverWidthTolerance, m_SliverWidthTolerance,
+                                                  0.01, 0.25 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DRCSliverMinimumLength,
-                                                  &m_SliverMinimumLength, m_SliverMinimumLength, 1e-9, 10 ) );
+                                                  &m_SliverMinimumLength, m_SliverMinimumLength,
+                                                  1e-9, 10 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DRCSliverAngleTolerance,
-                                                  &m_SliverAngleTolerance, m_SliverAngleTolerance, 1.0, 90.0 ) );
+                                                  &m_SliverAngleTolerance, m_SliverAngleTolerance,
+                                                  1.0, 90.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::HoleWallThickness,
-                                                  &m_HoleWallThickness, m_HoleWallThickness, 0.0, 1.0 ) );
+                                                  &m_HoleWallThickness, m_HoleWallThickness,
+                                                  0.0, 1.0 ) );
 
     configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::CoroutineStackSize,
                                                &m_CoroutineStackSize, AC_STACK::default_stack,
                                                AC_STACK::min_stack, AC_STACK::max_stack ) );
 
     configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::UpdateUIEventInterval,
-                                               &m_UpdateUIEventInterval, m_UpdateUIEventInterval, -1, 100000 ) );
+                                               &m_UpdateUIEventInterval, m_UpdateUIEventInterval,
+                                               -1, 100000 ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::ShowRouterDebugGraphics,
-                                                &m_ShowRouterDebugGraphics, m_ShowRouterDebugGraphics ) );
+                                                &m_ShowRouterDebugGraphics,
+                                                m_ShowRouterDebugGraphics ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::EnableRouterDump,
+                                                &m_EnableRouterDump, m_EnableRouterDump ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::HyperZoom,
+                                                &m_HyperZoom, m_HyperZoom ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::CompactFileSave,
                                                 &m_CompactSave, m_CompactSave ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DrawArcAccuracy,
-                                                  &m_DrawArcAccuracy, m_DrawArcAccuracy, 0.0, 100000.0 ) );
+                                                  &m_DrawArcAccuracy, m_DrawArcAccuracy,
+                                                  0.0, 100000.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::DrawArcCenterStartEndMaxAngle,
-                                                  &m_DrawArcCenterMaxAngle, m_DrawArcCenterMaxAngle, 0.0, 100000.0 ) );
+                                                  &m_DrawArcCenterMaxAngle,
+                                                  m_DrawArcCenterMaxAngle, 0.0, 100000.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::MaxTangentTrackAngleDeviation,
-                                                  &m_MaxTangentAngleDeviation, m_MaxTangentAngleDeviation, 0.0, 90.0 ) );
+                                                  &m_MaxTangentAngleDeviation,
+                                                  m_MaxTangentAngleDeviation, 0.0, 90.0 ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::MaxTrackLengthToKeep,
-                                                  &m_MaxTrackLengthToKeep, m_MaxTrackLengthToKeep, 0.0, 1.0 ) );
+                                                  &m_MaxTrackLengthToKeep, m_MaxTrackLengthToKeep,
+                                                  0.0, 1.0 ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::ExtraZoneDisplayModes,
-                                                &m_ExtraZoneDisplayModes, m_ExtraZoneDisplayModes ) );
+                                                &m_ExtraZoneDisplayModes,
+                                                m_ExtraZoneDisplayModes ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::StrokeTriangulation,
-                                                &m_DrawTriangulationOutlines, m_DrawTriangulationOutlines ) );
+                                                &m_DrawTriangulationOutlines,
+                                                m_DrawTriangulationOutlines ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::MinPlotPenWidth,
-                                                  &m_MinPlotPenWidth, m_MinPlotPenWidth, 0.0, 1.0 ) );
+                                                  &m_MinPlotPenWidth, m_MinPlotPenWidth,
+                                                  0.0, 1.0 ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::DebugZoneFiller,
                                                 &m_DebugZoneFiller, m_DebugZoneFiller ) );
@@ -421,7 +387,8 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
                                                 &m_DebugPDFWriter, m_DebugPDFWriter ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::SmallDrillMarkSize,
-                                                  &m_SmallDrillMarkSize, m_SmallDrillMarkSize, 0.0, 3.0 ) );
+                                                  &m_SmallDrillMarkSize, m_SmallDrillMarkSize,
+                                                  0.0, 3.0 ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::HotkeysDumper,
                                                 &m_HotkeysDumper, m_HotkeysDumper ) );
@@ -430,7 +397,8 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
                                                 &m_DrawBoundingBoxes, m_DrawBoundingBoxes ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::ShowPcbnewExportNetlist,
-                                                &m_ShowPcbnewExportNetlist, m_ShowPcbnewExportNetlist ) );
+                                                &m_ShowPcbnewExportNetlist,
+                                                m_ShowPcbnewExportNetlist ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::Skip3DModelFileCache,
                                                 &m_Skip3DModelFileCache, m_Skip3DModelFileCache ) );
@@ -448,7 +416,8 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
                                                 &m_ShowEventCounters, m_ShowEventCounters ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::AllowManualCanvasScale,
-                                                &m_AllowManualCanvasScale, m_AllowManualCanvasScale ) );
+                                                &m_AllowManualCanvasScale,
+                                                m_AllowManualCanvasScale ) );
 
     configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::V3DRT_BevelHeight_um,
                                                &m_3DRT_BevelHeight_um, m_3DRT_BevelHeight_um,
@@ -456,22 +425,76 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
                                                AC_GROUPS::V3D_RayTracing ) );
 
     configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::V3DRT_BevelExtentFactor,
-                                                  &m_3DRT_BevelExtentFactor, m_3DRT_BevelExtentFactor,
-                                                  0.0, 100.0,
+                                                  &m_3DRT_BevelExtentFactor,
+                                                  m_3DRT_BevelExtentFactor, 0.0, 100.0,
                                                   AC_GROUPS::V3D_RayTracing ) );
 
     configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::UseClipper2,
                                                 &m_UseClipper2, m_UseClipper2 ) );
 
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::Use3DConnexionDriver,
+                                                &m_Use3DConnexionDriver, m_Use3DConnexionDriver ) );
 
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::IncrementalConnectivity,
+                                                &m_IncrementalConnectivity,
+                                                m_IncrementalConnectivity ) );
+
+    configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::DisambiguationTime,
+                                               &m_DisambiguationMenuDelay,
+                                               m_DisambiguationMenuDelay,
+                                               50, 10000 ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::EnableGenerators,
+                                                &m_EnableGenerators, m_EnableGenerators ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::EnableGit,
+                                                &m_EnableGit, m_EnableGit ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::EnableEeschemaPrintCairo,
+                                                &m_EnableEeschemaPrintCairo,
+                                                m_EnableEeschemaPrintCairo ) );
+
+    configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::PcbSelectionVisibilityRatio,
+                                                  &m_PcbSelectionVisibilityRatio,
+                                                  m_PcbSelectionVisibilityRatio, 0.0, 1.0 ) );
+
+    configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::MinimumSegmentLength,
+                                                  &m_MinimumSegmentLength,
+                                                  m_MinimumSegmentLength, 10, 1000 ) );
+
+    configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::OcePluginLinearDeflection,
+                                                    &m_OcePluginLinearDeflection,
+                                                    m_OcePluginLinearDeflection, 0.01, 1.0 ) );
+
+    configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::OcePluginAngularDeflection,
+                                                    &m_OcePluginAngularDeflection,
+                                                    m_OcePluginAngularDeflection, 0.01, 360.0 ) );
+
+    configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::TriangulateSimplificationLevel,
+                                                    &m_TriangulateSimplificationLevel,
+                                                    m_TriangulateSimplificationLevel, 0, 1000 ) );
+
+    configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::TriangulateMinimumArea,
+                                                    &m_TriangulateMinimumArea,
+                                                    m_TriangulateMinimumArea, 0, 100000 ) );
+
+    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::EnableCacheFriendlyFracture,
+                                                &m_EnableCacheFriendlyFracture,
+                                                m_EnableCacheFriendlyFracture ) );
+
+    configParams.push_back( new PARAM_CFG_DOUBLE( true, AC_KEYS::MinParallelAngle,
+                                                  &m_MinParallelAngle, m_MinParallelAngle,
+                                                  0.0, 45.0 ) );
+
+    configParams.push_back( new PARAM_CFG_INT( true, AC_KEYS::ResolveTextRecursionDepth,
+                                                  &m_ResolveTextRecursionDepth,
+                                                  m_ResolveTextRecursionDepth, 0, 10 ) );
 
     // Special case for trace mask setting...we just grab them and set them immediately
     // Because we even use wxLogTrace inside of advanced config
     wxString traceMasks;
-    configParams.push_back( new PARAM_CFG_WXSTRING( true, AC_KEYS::TraceMasks, &traceMasks, wxS( "" ) ) );
-
-    configParams.push_back( new PARAM_CFG_BOOL( true, AC_KEYS::ShowPropertiesPanel,
-                                                &m_ShowPropertiesPanel, false ) );
+    configParams.push_back( new PARAM_CFG_WXSTRING( true, AC_KEYS::TraceMasks, &traceMasks,
+                                                    wxS( "" ) ) );
 
     // Load the config from file
     wxConfigLoadSetups( &aCfg, configParams );
@@ -489,6 +512,8 @@ void ADVANCED_CFG::loadSettings( wxConfigBase& aCfg )
 
     for( PARAM_CFG* param : configParams )
         delete param;
+
+    wxLogTrace( kicadTraceCoroutineStack, wxT( "Using coroutine stack size %d" ), m_CoroutineStackSize );
 }
 
 
