@@ -27,13 +27,17 @@
 #include <bitmaps.h>
 #include <confirm.h>
 #include <core/arraydim.h>
+#include <core/kicad_algo.h>
 #include <dialog_shim.h>
+#include <dialogs/hotkey_cycle_popup.h>
 #include <eda_draw_frame.h>
 #include <file_history.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <id.h>
 #include <kiface_base.h>
 #include <lockfile.h>
 #include <macros.h>
+#include <math/vector2wx.h>
 #include <page_info.h>
 #include <paths.h>
 #include <pgm_base.h>
@@ -44,6 +48,7 @@
 #include <settings/settings_manager.h>
 #include <title_block.h>
 #include <tool/actions.h>
+#include <tool/action_toolbar.h>
 #include <tool/common_tools.h>
 #include <tool/grid_menu.h>
 #include <tool/selection_conditions.h>
@@ -55,9 +60,9 @@
 #include <view/view.h>
 #include <drawing_sheet/ds_draw_item.h>
 #include <widgets/msgpanel.h>
+#include <widgets/properties_panel.h>
 #include <wx/event.h>
 #include <wx/snglinst.h>
-#include <dialogs/dialog_grid_settings.h>
 #include <widgets/ui_common.h>
 #include <widgets/search_pane.h>
 #include <wx/dirdlg.h>
@@ -87,9 +92,9 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
                                 long aStyle, const wxString& aFrameName,
                                 const EDA_IU_SCALE& aIuScale ) :
     KIWAY_PLAYER( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName, aIuScale ),
-    m_socketServer( nullptr )
+    m_socketServer( nullptr ),
+    m_lastToolbarIconSize( 0 )
 {
-    m_socketServer        = nullptr;
     m_mainToolBar         = nullptr;
     m_drawToolBar         = nullptr;
     m_optionsToolBar      = nullptr;
@@ -109,65 +114,72 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
     m_drawBgColor         = COLOR4D( BLACK );   // the background color of the draw canvas:
                                                 // BLACK for Pcbnew, BLACK or WHITE for Eeschema
     m_colorSettings       = nullptr;
-    m_msgFrameHeight      = EDA_MSG_PANEL::GetRequiredHeight( this );
     m_polarCoords         = false;
     m_findReplaceData     = std::make_unique<EDA_SEARCH_DATA>();
+    m_hotkeyPopup         = nullptr;
+    m_propertiesPanel     = nullptr;
 
     SetUserUnits( EDA_UNITS::MILLIMETRES );
 
     m_auimgr.SetFlags( wxAUI_MGR_DEFAULT );
 
-    CreateStatusBar( 8 )->SetDoubleBuffered( true );
+    if( ( aStyle & wxFRAME_NO_TASKBAR ) == 0 )
+    {
+        CreateStatusBar( 8 )->SetDoubleBuffered( true );
 
     // set the size of the status bar subwindows:
 
-    wxWindow* stsbar = GetStatusBar();
-    int       spacer = KIUI::GetTextSize( wxT( "M" ), stsbar ).x * 2;
+        wxWindow* stsbar = GetStatusBar();
+        int       spacer = KIUI::GetTextSize( wxT( "M" ), stsbar ).x * 2;
 
-    int dims[] = {
+        int dims[] =
+        {
+            // remainder of status bar on far left is set to a default or whatever is left over.
+            -1,
 
-        // remainder of status bar on far left is set to a default or whatever is left over.
-        -1,
+            // When using GetTextSize() remember the width of character '1' is not the same
+            // as the width of '0' unless the font is fixed width, and it usually won't be.
 
-        // When using GetTextSize() remember the width of character '1' is not the same
-        // as the width of '0' unless the font is fixed width, and it usually won't be.
+            // zoom:
+            KIUI::GetTextSize( wxT( "Z 762000" ), stsbar ).x,
 
-        // zoom:
-        KIUI::GetTextSize( wxT( "Z 762000" ), stsbar ).x + spacer,
+            // cursor coords
+            KIUI::GetTextSize( wxT( "X 1234.1234  Y 1234.1234" ), stsbar ).x,
 
-        // cursor coords
-        KIUI::GetTextSize( wxT( "X 1234.1234  Y 1234.1234" ), stsbar ).x + spacer,
+            // delta distances
+            KIUI::GetTextSize( wxT( "dx 1234.1234  dy 1234.1234  dist 1234.1234" ), stsbar ).x,
 
-        // delta distances
-        KIUI::GetTextSize( wxT( "dx 1234.1234  dy 1234.1234  dist 1234.1234" ), stsbar ).x + spacer,
+            // grid size
+            KIUI::GetTextSize( wxT( "grid X 1234.1234  Y 1234.1234" ), stsbar ).x,
 
-        // grid size
-        KIUI::GetTextSize( wxT( "grid X 1234.1234  Y 1234.1234" ), stsbar ).x + spacer,
+            // units display, Inches is bigger than mm
+            KIUI::GetTextSize( _( "Inches" ), stsbar ).x,
 
-        // units display, Inches is bigger than mm
-        KIUI::GetTextSize( _( "Inches" ), stsbar ).x + spacer,
+            // Size for the "Current Tool" panel; longest string from SetTool()
+            KIUI::GetTextSize( wxT( "Add layer alignment target" ), stsbar ).x,
 
-        // Size for the "Current Tool" panel; longest string from SetTool()
-        KIUI::GetTextSize( wxT( "Add layer alignment target" ), stsbar ).x + spacer,
+            // constraint mode
+            KIUI::GetTextSize( _( "Constrain to H, V, 45" ), stsbar ).x
+        };
 
-        // constraint mode
-        KIUI::GetTextSize( _( "Constrain to H, V, 45" ), stsbar ).x + spacer
-    };
+        for( size_t ii = 1; ii < arrayDim( dims ); ii++ )
+            dims[ii] += spacer;
 
-    SetStatusWidths( arrayDim( dims ), dims );
-    stsbar->SetFont( KIUI::GetStatusFont( this ) );
+        SetStatusWidths( arrayDim( dims ), dims );
+        stsbar->SetFont( KIUI::GetStatusFont( this ) );
+    }
+
+    m_messagePanel = new EDA_MSG_PANEL( this, -1, wxPoint( 0, m_frameSize.y ), wxDefaultSize );
+    m_messagePanel->SetBackgroundColour( COLOR4D( LIGHTGRAY ).ToColour() );
+    m_msgFrameHeight = m_messagePanel->GetBestSize().y;
 
     // Create child subwindows.
     GetClientSize( &m_frameSize.x, &m_frameSize.y );
     m_framePos.x   = m_framePos.y = 0;
     m_frameSize.y -= m_msgFrameHeight;
 
-    m_messagePanel  = new EDA_MSG_PANEL( this, -1, wxPoint( 0, m_frameSize.y ),
-                                         wxSize( m_frameSize.x, m_msgFrameHeight ) );
+    m_messagePanel->SetSize( m_frameSize.x, m_msgFrameHeight );
 
-    m_messagePanel->SetBackgroundColour( COLOR4D( LIGHTGRAY ).ToColour() );
-
-#if wxCHECK_VERSION( 3, 1, 3 )
     Bind( wxEVT_DPI_CHANGED,
           [&]( wxDPIChangedEvent& )
           {
@@ -178,26 +190,19 @@ EDA_DRAW_FRAME::EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrame
               // especially important even for first launches as the constructor of the window
               // here usually doesn't have the correct dpi awareness yet
               m_frameSize.y += m_msgFrameHeight;
-              m_msgFrameHeight = EDA_MSG_PANEL::GetRequiredHeight( this );
+              m_msgFrameHeight = m_messagePanel->GetBestSize().y;
               m_frameSize.y -= m_msgFrameHeight;
 
               m_messagePanel->SetPosition( wxPoint( 0, m_frameSize.y ) );
               m_messagePanel->SetSize( m_frameSize.x, m_msgFrameHeight );
+
+              // Don't skip, otherwise the frame gets too big
           } );
-#endif
 }
 
 
 EDA_DRAW_FRAME::~EDA_DRAW_FRAME()
 {
-    delete m_socketServer;
-
-    for( wxSocketBase* socket : m_sockets )
-    {
-        socket->Shutdown();
-        socket->Destroy();
-    }
-
     saveCanvasTypeSetting( m_canvasType );
 
     delete m_actions;
@@ -216,15 +221,31 @@ EDA_DRAW_FRAME::~EDA_DRAW_FRAME()
 
 void EDA_DRAW_FRAME::ReleaseFile()
 {
-    m_file_checker = nullptr;
+    if( m_file_checker.get() != nullptr )
+        m_file_checker->UnlockFile();
 }
 
 
 bool EDA_DRAW_FRAME::LockFile( const wxString& aFileName )
 {
-    m_file_checker = ::LockFile( aFileName );
+    // We need to explicitly reset here to get the deletion before
+    // we create a new unique_ptr that may be for the same file
+    m_file_checker.reset();
 
-    return m_file_checker && !m_file_checker->IsAnotherRunning();
+    m_file_checker = std::make_unique<LOCKFILE>( aFileName );
+
+    if( !m_file_checker->Valid() && m_file_checker->IsLockedByMe() )
+    {
+        // If we cannot acquire the lock but we appear to be the one who
+        // locked it, check to see if there is another KiCad instance running.
+        // If there is not, then we can override the lock.  This could happen if
+        // KiCad crashed or was interrupted
+        if( !Pgm().SingleInstance()->IsAnotherRunning() )
+            m_file_checker->OverrideLock();
+    }
+    // If the file is valid, return true.  This could mean that the file is
+    // locked or it could mean that the file is read-only
+    return m_file_checker->Valid();
 }
 
 
@@ -272,10 +293,11 @@ void EDA_DRAW_FRAME::unitsChangeRefresh()
 {
     // Notify all tools the units have changed
     if( m_toolManager )
-        m_toolManager->RunAction( ACTIONS::updateUnits, true );
+        m_toolManager->RunAction( ACTIONS::updateUnits );
 
     UpdateStatusBar();
     UpdateMsgPanel();
+    UpdateProperties();
 }
 
 
@@ -292,7 +314,7 @@ void EDA_DRAW_FRAME::ToggleUserUnits()
                                                           : EDA_UNITS::INCHES );
         unitsChangeRefresh();
 
-        wxCommandEvent e( UNITS_CHANGED );
+        wxCommandEvent e( EDA_EVT_UNITS_CHANGED );
         ProcessEventLocally( e );
     }
 }
@@ -322,13 +344,23 @@ void EDA_DRAW_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
 
     m_galDisplayOptions.ReadCommonConfig( *settings, this );
 
+    GetToolManager()->RunAction( ACTIONS::gridPreset, config()->m_Window.grid.last_size_idx );
+    UpdateGridSelectBox();
+
+    if( m_lastToolbarIconSize == 0
+        || m_lastToolbarIconSize != settings->m_Appearance.toolbar_icon_size )
+    {
+        OnToolbarSizeChanged();
+        m_lastToolbarIconSize = settings->m_Appearance.toolbar_icon_size;
+    }
+
 #ifndef __WXMAC__
     resolveCanvasType();
 
     if( m_canvasType != GetCanvas()->GetBackend() )
     {
         // Try to switch (will automatically fallback if necessary)
-        GetCanvas()->SwitchBackend( m_canvasType );
+        SwitchCanvas( m_canvasType );
         EDA_DRAW_PANEL_GAL::GAL_TYPE newGAL = GetCanvas()->GetBackend();
         bool                         success = newGAL == m_canvasType;
 
@@ -342,7 +374,7 @@ void EDA_DRAW_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
 
     // Notify all tools the preferences have changed
     if( m_toolManager )
-        m_toolManager->RunAction( ACTIONS::updatePreferences, true );
+        m_toolManager->RunAction( ACTIONS::updatePreferences );
 }
 
 
@@ -373,7 +405,7 @@ void EDA_DRAW_FRAME::UpdateGridSelectBox()
         m_gridSelectBox->Append( grid );
 
     m_gridSelectBox->Append( wxT( "---" ) );
-    m_gridSelectBox->Append( _( "Edit User Grid..." ) );
+    m_gridSelectBox->Append( _( "Edit Grids..." ) );
 
     m_gridSelectBox->SetSelection( config()->m_Window.grid.last_size_idx );
 }
@@ -389,7 +421,7 @@ void EDA_DRAW_FRAME::OnUpdateSelectGrid( wxUpdateUIEvent& aEvent )
     wxCHECK( config(), /* void */ );
 
     int idx = config()->m_Window.grid.last_size_idx;
-    idx = std::max( 0, std::min( idx, (int) m_gridSelectBox->GetCount() - 1 ) );
+    idx = alg::clamp( 0, idx, (int) m_gridSelectBox->GetCount() - 1 );
 
     if( idx != m_gridSelectBox->GetSelection() )
         m_gridSelectBox->SetSelection( idx );
@@ -460,11 +492,11 @@ void EDA_DRAW_FRAME::OnSelectGrid( wxCommandEvent& event )
         // (Only matters on some versions of GTK.)
         wxSafeYield();
 
-        m_toolManager->RunAction( ACTIONS::gridProperties, true );
+        m_toolManager->RunAction( ACTIONS::gridProperties );
     }
     else
     {
-        m_toolManager->RunAction( ACTIONS::gridPreset, true, static_cast<intptr_t>( idx ) );
+        m_toolManager->RunAction( ACTIONS::gridPreset, idx );
     }
 
     UpdateStatusBar();
@@ -475,18 +507,6 @@ void EDA_DRAW_FRAME::OnSelectGrid( wxCommandEvent& event )
 }
 
 
-void EDA_DRAW_FRAME::OnGridSettings( wxCommandEvent& aEvent )
-{
-    DIALOG_GRID_SETTINGS dlg( this );
-
-    if( dlg.ShowModal() == wxID_OK )
-    {
-        UpdateStatusBar();
-        GetCanvas()->Refresh();
-    }
-}
-
-
 bool EDA_DRAW_FRAME::IsGridVisible() const
 {
     wxCHECK( config(), true );
@@ -517,6 +537,22 @@ void EDA_DRAW_FRAME::SetGridVisibility( bool aVisible )
 }
 
 
+bool EDA_DRAW_FRAME::IsGridOverridden() const
+{
+    wxCHECK( config(), false );
+
+    return config()->m_Window.grid.overrides_enabled;
+}
+
+
+void EDA_DRAW_FRAME::SetGridOverrides( bool aOverride )
+{
+    wxCHECK( config(), /* void */ );
+
+    config()->m_Window.grid.overrides_enabled = aOverride;
+}
+
+
 void EDA_DRAW_FRAME::UpdateZoomSelectBox()
 {
     if( m_zoomSelectBox == nullptr )
@@ -551,7 +587,7 @@ void EDA_DRAW_FRAME::OnSelectZoom( wxCommandEvent& event )
     if( id < 0 || !( id < (int)m_zoomSelectBox->GetCount() ) )
         return;
 
-    m_toolManager->RunAction( ACTIONS::zoomPreset, true, static_cast<intptr_t>( id ) );
+    m_toolManager->RunAction( ACTIONS::zoomPreset, id );
     UpdateStatusBar();
     m_canvas->Refresh();
     // Needed on Windows because clicking on m_zoomSelectBox remove the focus from m_canvas
@@ -613,8 +649,11 @@ void EDA_DRAW_FRAME::DisplayGridMsg()
 {
     wxString msg;
 
-    msg.Printf( _( "grid %s" ), MessageTextFromValue( GetCanvas()->GetGAL()->GetGridSize().x,
-                                                      false ) );
+    GRID_SETTINGS& gridSettings = m_toolManager->GetSettings()->m_Window.grid;
+    int            currentIdx = m_toolManager->GetSettings()->m_Window.grid.last_size_idx;
+
+    msg.Printf( _( "grid %s" ),
+                gridSettings.grids[currentIdx].UserUnitsMessageText( this, false ) );
 
     SetStatusText( msg, 4 );
 }
@@ -693,6 +732,8 @@ void EDA_DRAW_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 
     for( const wxString& s : aCfg->m_FindReplace.replace_history )
         m_replaceStringHistoryList.Add( s );
+
+    m_lastToolbarIconSize = cmnCfg->m_Appearance.toolbar_icon_size;
 }
 
 
@@ -741,21 +782,21 @@ void EDA_DRAW_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 void EDA_DRAW_FRAME::AppendMsgPanel( const wxString& aTextUpper, const wxString& aTextLower,
                                      int aPadding )
 {
-    if( m_messagePanel )
+    if( m_messagePanel && !m_isClosing )
         m_messagePanel->AppendMessage( aTextUpper, aTextLower, aPadding );
 }
 
 
 void EDA_DRAW_FRAME::ClearMsgPanel()
 {
-    if( m_messagePanel )
+    if( m_messagePanel && !m_isClosing )
         m_messagePanel->EraseMsgBox();
 }
 
 
 void EDA_DRAW_FRAME::SetMsgPanel( const std::vector<MSG_PANEL_ITEM>& aList )
 {
-    if( m_messagePanel )
+    if( m_messagePanel && !m_isClosing )
     {
         m_messagePanel->EraseMsgBox();
 
@@ -768,10 +809,9 @@ void EDA_DRAW_FRAME::SetMsgPanel( const std::vector<MSG_PANEL_ITEM>& aList )
 void EDA_DRAW_FRAME::SetMsgPanel( const wxString& aTextUpper, const wxString& aTextLower,
                                   int aPadding )
 {
-    if( m_messagePanel )
+    if( m_messagePanel && !m_isClosing )
     {
         m_messagePanel->EraseMsgBox();
-
         m_messagePanel->AppendMessage( aTextUpper, aTextLower, aPadding );
     }
 }
@@ -808,7 +848,7 @@ void EDA_DRAW_FRAME::SwitchCanvas( EDA_DRAW_PANEL_GAL::GAL_TYPE aCanvasType )
 }
 
 
-EDA_DRAW_PANEL_GAL::GAL_TYPE EDA_DRAW_FRAME::loadCanvasTypeSetting()
+EDA_DRAW_PANEL_GAL::GAL_TYPE EDA_DRAW_FRAME::loadCanvasTypeSetting(  APP_SETTINGS_BASE* aCfg )
 {
 #ifdef __WXMAC__
     // Cairo renderer doesn't handle Retina displays so there's really only one game
@@ -817,7 +857,7 @@ EDA_DRAW_PANEL_GAL::GAL_TYPE EDA_DRAW_FRAME::loadCanvasTypeSetting()
 #endif
 
     EDA_DRAW_PANEL_GAL::GAL_TYPE canvasType = EDA_DRAW_PANEL_GAL::GAL_TYPE_NONE;
-    APP_SETTINGS_BASE* cfg = Kiface().KifaceSettings();
+    APP_SETTINGS_BASE* cfg = aCfg ? aCfg : Kiface().KifaceSettings();
 
     if( cfg )
         canvasType = static_cast<EDA_DRAW_PANEL_GAL::GAL_TYPE>( cfg->m_Graphics.canvas_type );
@@ -842,26 +882,15 @@ bool EDA_DRAW_FRAME::saveCanvasTypeSetting( EDA_DRAW_PANEL_GAL::GAL_TYPE aCanvas
     // Not all classes derived from EDA_DRAW_FRAME can save the canvas type, because some
     // have a fixed type, or do not have a option to set the canvas type (they inherit from
     // a parent frame)
-    FRAME_T allowed_frames[] =
-    {
-        FRAME_SCH, FRAME_SCH_SYMBOL_EDITOR,
-        FRAME_PCB_EDITOR, FRAME_FOOTPRINT_EDITOR,
-        FRAME_GERBER,
-        FRAME_PL_EDITOR
-    };
+    static std::vector<FRAME_T> s_allowedFrames =
+            {
+                FRAME_SCH, FRAME_SCH_SYMBOL_EDITOR,
+                FRAME_PCB_EDITOR, FRAME_FOOTPRINT_EDITOR,
+                FRAME_GERBER,
+                FRAME_PL_EDITOR
+            };
 
-    bool allow_save = false;
-
-    for( unsigned ii = 0; ii < arrayDim( allowed_frames ); ii++ )
-    {
-        if( m_ident == allowed_frames[ii] )
-        {
-            allow_save = true;
-            break;
-        }
-    }
-
-    if( !allow_save )
+    if( !alg::contains( s_allowedFrames, m_ident ) )
         return false;
 
     if( aCanvasType < EDA_DRAW_PANEL_GAL::GAL_TYPE_NONE
@@ -871,9 +900,7 @@ bool EDA_DRAW_FRAME::saveCanvasTypeSetting( EDA_DRAW_PANEL_GAL::GAL_TYPE aCanvas
         return false;
     }
 
-    APP_SETTINGS_BASE* cfg = Kiface().KifaceSettings();
-
-    if( cfg )
+    if( APP_SETTINGS_BASE* cfg = Kiface().KifaceSettings() )
         cfg->m_Graphics.canvas_type = static_cast<int>( aCanvasType );
 
     return false;
@@ -883,7 +910,7 @@ bool EDA_DRAW_FRAME::saveCanvasTypeSetting( EDA_DRAW_PANEL_GAL::GAL_TYPE aCanvas
 VECTOR2I EDA_DRAW_FRAME::GetNearestGridPosition( const VECTOR2I& aPosition ) const
 {
     const VECTOR2I& gridOrigin = GetGridOrigin();
-    VECTOR2D       gridSize = GetCanvas()->GetGAL()->GetGridSize();
+    VECTOR2D        gridSize = GetCanvas()->GetGAL()->GetGridSize();
 
     double xOffset = fmod( gridOrigin.x, gridSize.x );
     int    x = KiROUND( (aPosition.x - xOffset) / gridSize.x );
@@ -897,7 +924,7 @@ VECTOR2I EDA_DRAW_FRAME::GetNearestGridPosition( const VECTOR2I& aPosition ) con
 VECTOR2I EDA_DRAW_FRAME::GetNearestHalfGridPosition( const VECTOR2I& aPosition ) const
 {
     const VECTOR2I& gridOrigin = GetGridOrigin();
-    VECTOR2D       gridSize = GetCanvas()->GetGAL()->GetGridSize() / 2.0;
+    VECTOR2D        gridSize = GetCanvas()->GetGAL()->GetGridSize() / 2.0;
 
     double xOffset = fmod( gridOrigin.x, gridSize.x );
     int    x = KiROUND( (aPosition.x - xOffset) / gridSize.x );
@@ -922,7 +949,7 @@ void EDA_DRAW_FRAME::HardRedraw()
 
 void EDA_DRAW_FRAME::Zoom_Automatique( bool aWarpPointer )
 {
-    m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+    m_toolManager->RunAction( ACTIONS::zoomFitScreen );
 }
 
 
@@ -956,8 +983,8 @@ void EDA_DRAW_FRAME::FocusOnLocation( const VECTOR2I& aPos )
 
     for( wxWindow* dialog : findDialogs() )
     {
-        dialogScreenRects.emplace_back( GetCanvas()->ScreenToClient( dialog->GetScreenPosition() ),
-                                        dialog->GetSize() );
+        dialogScreenRects.emplace_back( ToVECTOR2D( GetCanvas()->ScreenToClient( dialog->GetScreenPosition() ) ),
+                                        ToVECTOR2D( dialog->GetSize() ) );
     }
 
     // Center if we're behind an obscuring dialog, or within 10% of its edge
@@ -996,10 +1023,10 @@ void PrintDrawingSheet( const RENDER_SETTINGS* aSettings, const PAGE_INFO& aPage
                         const wxString& aPageNumber, double aMils2Iu, const PROJECT* aProject,
                         const wxString& aSheetLayer, bool aIsFirstPage )
 {
-    DS_DRAW_ITEM_LIST drawList;
+    DS_DRAW_ITEM_LIST drawList( unityScale );
 
     drawList.SetDefaultPenSize( aSettings->GetDefaultPenWidth() );
-    drawList.SetMilsToIUfactor( aMils2Iu );
+    drawList.SetPlotterMilsToIUfactor( aMils2Iu );
     drawList.SetPageNumber( aPageNumber );
     drawList.SetSheetCount( aSheetCount );
     drawList.SetFileName( aFileName );
@@ -1134,6 +1161,26 @@ void EDA_DRAW_FRAME::RecreateToolbars()
 
     if( m_auxiliaryToolBar )    // Additional tools under main toolbar
        ReCreateAuxiliaryToolbar();
+
+
+}
+
+
+void EDA_DRAW_FRAME::OnToolbarSizeChanged()
+{
+    if( m_mainToolBar )
+        m_auimgr.GetPane( m_mainToolBar ).MaxSize( m_mainToolBar->GetSize() );
+
+    if( m_drawToolBar )
+        m_auimgr.GetPane( m_drawToolBar ).MaxSize( m_drawToolBar->GetSize() );
+
+    if( m_optionsToolBar )
+        m_auimgr.GetPane( m_optionsToolBar ).MaxSize( m_optionsToolBar->GetSize() );
+
+    if( m_auxiliaryToolBar )
+        m_auimgr.GetPane( m_auxiliaryToolBar ).MaxSize( m_auxiliaryToolBar->GetSize() );
+
+    m_auimgr.Update();
 }
 
 
@@ -1143,8 +1190,34 @@ void EDA_DRAW_FRAME::ShowChangedLanguage()
 
     if( m_searchPane )
     {
+        wxAuiPaneInfo& search_pane_info = m_auimgr.GetPane( m_searchPane );
+        search_pane_info.Caption( _( "Search" ) );
+
         m_searchPane->OnLanguageChange();
     }
+
+    if( m_propertiesPanel )
+    {
+        wxAuiPaneInfo& properties_pane_info = m_auimgr.GetPane( m_propertiesPanel );
+        properties_pane_info.Caption( _( "Properties" ) );
+
+        m_propertiesPanel->OnLanguageChanged();
+    }
+}
+
+
+void EDA_DRAW_FRAME::UpdateProperties()
+{
+    if( !m_propertiesPanel || !m_propertiesPanel->IsShownOnScreen() )
+        return;
+
+    m_propertiesPanel->UpdateData();
+}
+
+
+void EDA_DRAW_FRAME::CreateHotkeyPopup()
+{
+    m_hotkeyPopup = new HOTKEY_CYCLE_POPUP( this );
 }
 
 
@@ -1234,3 +1307,37 @@ void EDA_DRAW_FRAME::onActivate( wxActivateEvent& aEvent )
 
     aEvent.Skip();
 }
+
+
+bool EDA_DRAW_FRAME::SaveCanvasImageToFile( const wxString& aFileName,
+                                            BITMAP_TYPE aBitmapType )
+{
+    bool retv = true;
+
+    // Make a screen copy of the canvas:
+    wxSize image_size = GetCanvas()->GetClientSize();
+
+    wxClientDC dc( GetCanvas() );
+    wxBitmap   bitmap( image_size.x, image_size.y );
+    wxMemoryDC memdc;
+
+    memdc.SelectObject( bitmap );
+    memdc.Blit( 0, 0, image_size.x, image_size.y, &dc, 0, 0 );
+    memdc.SelectObject( wxNullBitmap );
+
+    wxImage image = bitmap.ConvertToImage();
+
+    wxBitmapType type = wxBITMAP_TYPE_PNG;
+    switch( aBitmapType )
+    {
+    case BITMAP_TYPE::PNG: type = wxBITMAP_TYPE_PNG; break;
+    case BITMAP_TYPE::BMP: type = wxBITMAP_TYPE_BMP; break;
+    case BITMAP_TYPE::JPG: type = wxBITMAP_TYPE_JPEG; break;
+    }
+
+    if( !image.SaveFile( aFileName, type ) )
+        retv = false;
+
+    image.Destroy();
+    return retv;
+}
