@@ -2,9 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2021 Ola Rinta-Koski <gitlab@rinta-koski.net>
- * Copyright (C) 2021-2022 Kicad Developers, see AUTHORS.txt for contributors.
- *
- * Outline font class
+ * Copyright (C) 2021-2024 Kicad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,20 +23,16 @@
  */
 
 #include <limits>
-#include <pgm_base.h>
-#include <settings/settings_manager.h>
 #include <harfbuzz/hb.h>
 #include <harfbuzz/hb-ft.h>
 #include <bezier_curves.h>
 #include <geometry/shape_poly_set.h>
-#include <eda_text.h>
 #include <font/fontconfig.h>
 #include <font/outline_font.h>
 #include FT_GLYPH_H
 #include FT_BBOX_H
 #include <trigo.h>
-#include <font/fontconfig.h>
-#include <convert_basic_shapes_to_polygon.h>
+#include <core/utf8.h>
 
 using namespace KIFONT;
 
@@ -50,7 +44,8 @@ OUTLINE_FONT::OUTLINE_FONT() :
         m_face(NULL),
         m_faceSize( 16 ),
         m_fakeBold( false ),
-        m_fakeItal( false )
+        m_fakeItal( false ),
+        m_forDrawingSheet( false )
 {
     std::lock_guard<std::mutex> guard( m_freeTypeMutex );
 
@@ -59,41 +54,8 @@ OUTLINE_FONT::OUTLINE_FONT() :
 }
 
 
-wxString OUTLINE_FONT::FreeTypeVersion()
-{
-    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
-
-    if( !m_freeType )
-        FT_Init_FreeType( &m_freeType );
-
-    FT_Int major = 0;
-    FT_Int minor = 0;
-    FT_Int patch = 0;
-    FT_Library_Version( m_freeType, &major, &minor, &patch );
-
-    return wxString::Format( "%d.%d.%d", major, minor, patch );
-}
-
-
-wxString OUTLINE_FONT::HarfBuzzVersion()
-{
-    return wxString::FromUTF8( HB_VERSION_STRING );
-}
-
-
-wxString OUTLINE_FONT::FontConfigVersion()
-{
-    return fontconfig::FONTCONFIG::Version();
-}
-
-
-wxString OUTLINE_FONT::FontLibraryVersion()
-{
-    return wxString::Format( "FreeType %s HarfBuzz %s", FreeTypeVersion(), HarfBuzzVersion() );
-}
-
-
-OUTLINE_FONT* OUTLINE_FONT::LoadFont( const wxString& aFontName, bool aBold, bool aItalic )
+OUTLINE_FONT* OUTLINE_FONT::LoadFont( const wxString& aFontName, bool aBold, bool aItalic,
+                                      bool aForDrawingSheet )
 {
     std::unique_ptr<OUTLINE_FONT> font = std::make_unique<OUTLINE_FONT>();
 
@@ -117,6 +79,7 @@ OUTLINE_FONT* OUTLINE_FONT::LoadFont( const wxString& aFontName, bool aBold, boo
 
     font->m_fontName = aFontName;       // Keep asked-for name, even if we substituted.
     font->m_fontFileName = fontFile;
+    font->m_forDrawingSheet = aForDrawingSheet;
 
     return font.release();
 }
@@ -126,8 +89,6 @@ FT_Error OUTLINE_FONT::loadFace( const wxString& aFontFileName, int aFaceIndex )
 {
     std::lock_guard<std::mutex> guard( m_freeTypeMutex );
 
-    // TODO: check that going from wxString to char* with UTF-8
-    // conversion for filename makes sense on any/all platforms
     FT_Error e = FT_New_Face( m_freeType, aFontFileName.mb_str( wxConvUTF8 ), aFaceIndex, &m_face );
 
     if( !e )
@@ -146,47 +107,18 @@ FT_Error OUTLINE_FONT::loadFace( const wxString& aFontFileName, int aFaceIndex )
 }
 
 
-/**
- * Compute the vertical position of an overbar.  This is the distance between the text
- * baseline and the overbar.
- */
-double OUTLINE_FONT::ComputeOverbarVerticalPosition( double aGlyphHeight ) const
-{
-    // The overbar on actual text is positioned above the bounding box of the glyphs.  However,
-    // that's expensive to calculate so we use an estimation here (as this is only used for
-    // calculating bounding boxes).
-    return aGlyphHeight * m_outlineFontSizeCompensation;
-}
-
-
-/**
- * Compute the vertical position of an underline.  This is the distance between the text
- * baseline and the underline.
- */
-double OUTLINE_FONT::ComputeUnderlineVerticalPosition( double aGlyphHeight ) const
-{
-    return aGlyphHeight * m_underlineOffsetScaler;
-}
-
-
 /**
  * Compute the distance (interline) between 2 lines of text (for multiline texts).  This is
  * the distance between baselines, not the space between line bounding boxes.
  */
-double OUTLINE_FONT::GetInterline( double aGlyphHeight, double aLineSpacing ) const
+double OUTLINE_FONT::GetInterline( double aGlyphHeight, const METRICS& aFontMetrics ) const
 {
-    double pitch = INTERLINE_PITCH_RATIO;
+    double glyphToFontHeight = 1.0;
 
     if( GetFace()->units_per_EM )
-        pitch = GetFace()->height / GetFace()->units_per_EM;
+        glyphToFontHeight = GetFace()->height / GetFace()->units_per_EM;
 
-    double interline = aLineSpacing * aGlyphHeight * pitch * m_outlineFontSizeCompensation;
-
-    // FONT TODO this hack is an attempt to fix interline spacing by eyeballing it
-    static constexpr double interlineHackMultiplier = 1.2;
-    interline *= interlineHackMultiplier;
-
-    return interline;
+    return aFontMetrics.GetInterline( aGlyphHeight * glyphToFontHeight );
 }
 
 
@@ -241,7 +173,8 @@ BOX2I OUTLINE_FONT::getBoundingBox( const std::vector<std::unique_ptr<GLYPH>>& a
 
 void OUTLINE_FONT::GetLinesAsGlyphs( std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
                                      const wxString& aText, const VECTOR2I& aPosition,
-                                     const TEXT_ATTRIBUTES& aAttrs ) const
+                                     const TEXT_ATTRIBUTES& aAttrs,
+                                     const METRICS& aFontMetrics ) const
 {
     wxArrayString         strings;
     std::vector<VECTOR2I> positions;
@@ -251,12 +184,12 @@ void OUTLINE_FONT::GetLinesAsGlyphs( std::vector<std::unique_ptr<GLYPH>>* aGlyph
     if( aAttrs.m_Italic )
         textStyle |= TEXT_STYLE::ITALIC;
 
-    getLinePositions( aText, aPosition, strings, positions, extents, aAttrs );
+    getLinePositions( aText, aPosition, strings, positions, extents, aAttrs, aFontMetrics );
 
     for( size_t i = 0; i < strings.GetCount(); i++ )
     {
         (void) drawMarkup( nullptr, aGlyphs, strings.Item( i ), positions[i], aAttrs.m_Size,
-                           aAttrs.m_Angle, aAttrs.m_Mirrored, aPosition, textStyle );
+                           aAttrs.m_Angle, aAttrs.m_Mirrored, aPosition, textStyle, aFontMetrics );
     }
 }
 
@@ -313,22 +246,6 @@ VECTOR2I OUTLINE_FONT::GetTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_pt
 }
 
 
-double OUTLINE_FONT::getOverbarOffset( int ascender, int height, int thickness ) const
-{
-    double thicknessRatio = abs( (double) thickness ) / (double) height;
-    double ascenderRatio = (double) ascender / (double) height;
-
-    if( thicknessRatio < 0.05 )
-        return 0.04;
-    else if( ascenderRatio < 0.78 )
-        return 0.00;
-    else if( ascenderRatio < 0.80 )
-        return -0.03;
-    else
-        return -0.06;
-}
-
-
 VECTOR2I OUTLINE_FONT::getTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
                                         const wxString& aText, const VECTOR2I& aSize,
                                         const VECTOR2I& aPosition, const EDA_ANGLE& aAngle,
@@ -338,23 +255,70 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_pt
     std::lock_guard<std::mutex> guard( m_freeTypeMutex );
 
     return getTextAsGlyphsUnlocked( aBBox, aGlyphs, aText, aSize, aPosition, aAngle, aMirror,
-            aOrigin, aTextStyle );
+                                    aOrigin, aTextStyle );
 }
 
-VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
-        const wxString& aText, const VECTOR2I& aSize,
-        const VECTOR2I& aPosition, const EDA_ANGLE& aAngle,
-        bool aMirror, const VECTOR2I& aOrigin,
-        TEXT_STYLE_FLAGS aTextStyle ) const
+
+struct GLYPH_CACHE_KEY {
+    FT_Face        face;
+    hb_codepoint_t codepoint;
+    double         scaler;
+    bool           forDrawingSheet;
+    bool           fakeItalic;
+    bool           fakeBold;
+    bool           mirror;
+    bool           supersub;
+    EDA_ANGLE      angle;
+
+    bool operator==(const GLYPH_CACHE_KEY& rhs ) const
+    {
+        return face == rhs.face
+                && codepoint == rhs.codepoint
+                && scaler == rhs.scaler
+                && forDrawingSheet == rhs.forDrawingSheet
+                && fakeItalic == rhs.fakeItalic
+                && fakeBold == rhs.fakeBold
+                && mirror == rhs.mirror
+                && supersub == rhs.supersub
+                && angle == rhs.angle;
+    }
+};
+
+namespace std
+{
+    template <>
+    struct hash<GLYPH_CACHE_KEY>
+    {
+        std::size_t operator()( const GLYPH_CACHE_KEY& k ) const
+        {
+            return hash<const void*>()( k.face )
+                        ^ hash<unsigned>()( k.codepoint )
+                        ^ hash<double>()( k.scaler )
+                        ^ hash<double>()( k.forDrawingSheet )
+                        ^ hash<int>()( k.fakeItalic )
+                        ^ hash<int>()( k.fakeBold )
+                        ^ hash<int>()( k.mirror )
+                        ^ hash<int>()( k.supersub )
+                        ^ hash<int>()( k.angle.AsTenthsOfADegree() );
+        }
+    };
+}
+
+
+VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox,
+                                                std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
+                                                const wxString& aText, const VECTOR2I& aSize,
+                                                const VECTOR2I& aPosition, const EDA_ANGLE& aAngle,
+                                                bool aMirror, const VECTOR2I& aOrigin,
+                                                TEXT_STYLE_FLAGS aTextStyle ) const
 {
     VECTOR2D glyphSize = aSize;
     FT_Face  face = m_face;
     double   scaler = faceSize();
+    bool     supersub = IsSuperscript( aTextStyle ) || IsSubscript( aTextStyle );
 
-    if( IsSubscript( aTextStyle ) || IsSuperscript( aTextStyle ) )
-    {
+    if( supersub )
         scaler = subscriptSize();
-    }
 
     // set glyph resolution so that FT_Load_Glyph() results are good enough for decomposing
     FT_Set_Char_Size( face, 0, scaler, GLYPH_RESOLUTION, 0 );
@@ -364,19 +328,26 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::u
     hb_buffer_guess_segment_properties( buf );  // guess direction, script, and language based on
                                                 // contents
 
-    unsigned int         glyphCount;
-    hb_glyph_info_t*     glyphInfo = hb_buffer_get_glyph_infos( buf, &glyphCount );
-    hb_glyph_position_t* glyphPos = hb_buffer_get_glyph_positions( buf, &glyphCount );
-
     hb_font_t* referencedFont = hb_ft_font_create_referenced( face );
     hb_ft_font_set_funcs( referencedFont );
     hb_shape( referencedFont, buf, nullptr, 0 );
 
+    unsigned int         glyphCount;
+    hb_glyph_info_t*     glyphInfo = hb_buffer_get_glyph_infos( buf, &glyphCount );
+    hb_glyph_position_t* glyphPos = hb_buffer_get_glyph_positions( buf, &glyphCount );
+
     VECTOR2D scaleFactor( glyphSize.x / faceSize(), -glyphSize.y / faceSize() );
     scaleFactor = scaleFactor * m_outlineFontSizeCompensation;
 
     VECTOR2I cursor( 0, 0 );
 
+    if( aGlyphs )
+        aGlyphs->reserve( glyphCount );
+
+    // GLYPH_DATA is a collection of all outlines in the glyph; for example the 'o' glyph
+    // generally contains 2 contours, one for the glyph outline and one for the hole
+    static std::unordered_map<GLYPH_CACHE_KEY, GLYPH_DATA> s_glyphCache;
+
     for( unsigned int i = 0; i < glyphCount; i++ )
     {
         // Don't process glyphs that were already included in a previous cluster
@@ -385,38 +356,72 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::u
 
         if( aGlyphs )
         {
-            if( m_fakeItal )
+            GLYPH_CACHE_KEY key = { face, glyphInfo[i].codepoint, scaler, m_forDrawingSheet,
+                                    m_fakeItal, m_fakeBold, aMirror, supersub, aAngle };
+            GLYPH_DATA&     glyphData = s_glyphCache[ key ];
+
+            if( glyphData.m_Contours.empty() )
             {
-                FT_Matrix matrix;
-                // Create a 12 degree slant
-                const float angle = (float)( -M_PI * 12.0f ) / 180.0f;
-                matrix.xx = (FT_Fixed) ( cos( angle ) * 0x10000L );
-                matrix.xy = (FT_Fixed) ( -sin( angle ) * 0x10000L );
-                matrix.yx = (FT_Fixed) ( 0 * 0x10000L );  // Don't rotate in the y direction
-                matrix.yy = (FT_Fixed) ( 1 * 0x10000L );
+                if( m_fakeItal )
+                {
+                    FT_Matrix matrix;
+                    // Create a 12 degree slant
+                    const float angle = (float)( -M_PI * 12.0f ) / 180.0f;
+                    matrix.xx = (FT_Fixed) ( cos( angle ) * 0x10000L );
+                    matrix.xy = (FT_Fixed) ( -sin( angle ) * 0x10000L );
+                    matrix.yx = (FT_Fixed) ( 0 * 0x10000L );  // Don't rotate in the y direction
+                    matrix.yy = (FT_Fixed) ( 1 * 0x10000L );
 
-                FT_Set_Transform(face, &matrix,0);
+                    FT_Set_Transform( face, &matrix, nullptr );
+                }
+
+                FT_Load_Glyph( face, glyphInfo[i].codepoint, FT_LOAD_NO_BITMAP );
+
+                if( m_fakeBold )
+                    FT_Outline_Embolden( &face->glyph->outline, 1 << 6 );
+
+                OUTLINE_DECOMPOSER decomposer( face->glyph->outline );
+
+                if( !decomposer.OutlineToSegments( &glyphData.m_Contours ) )
+                {
+                    double  hb_advance = glyphPos[i].x_advance * GLYPH_SIZE_SCALER;
+                    BOX2D   tofuBox( { scaler * 0.03, 0.0 },
+                                     { hb_advance - scaler * 0.02, scaler * 0.72 } );
+
+                    glyphData.m_Contours.clear();
+
+                    CONTOUR outline;
+                    outline.m_Winding = 1;
+                    outline.m_Orientation = FT_ORIENTATION_TRUETYPE;
+                    outline.m_Points.push_back( tofuBox.GetPosition() );
+                    outline.m_Points.push_back( { tofuBox.GetSize().x, tofuBox.GetPosition().y } );
+                    outline.m_Points.push_back( tofuBox.GetSize() );
+                    outline.m_Points.push_back( { tofuBox.GetPosition().x, tofuBox.GetSize().y } );
+                    glyphData.m_Contours.push_back( outline );
+
+                    CONTOUR hole;
+                    tofuBox.Move( { scaler * 0.06, scaler * 0.06 } );
+                    tofuBox.SetSize( { tofuBox.GetWidth() - scaler * 0.06,
+                                       tofuBox.GetHeight() - scaler * 0.06 } );
+                    hole.m_Winding = 1;
+                    hole.m_Orientation = FT_ORIENTATION_NONE;
+                    hole.m_Points.push_back( tofuBox.GetPosition() );
+                    hole.m_Points.push_back( { tofuBox.GetSize().x, tofuBox.GetPosition().y } );
+                    hole.m_Points.push_back( tofuBox.GetSize() );
+                    hole.m_Points.push_back( { tofuBox.GetPosition().x, tofuBox.GetSize().y } );
+                    glyphData.m_Contours.push_back( hole );
+                }
             }
 
-            FT_Load_Glyph( face, glyphInfo[i].codepoint, FT_LOAD_NO_BITMAP );
-
-            if( m_fakeBold )
-                FT_Outline_Embolden( &face->glyph->outline, 1 << 6 );
-
-            // contours is a collection of all outlines in the glyph; for example the 'o' glyph
-            // generally contains 2 contours, one for the glyph outline and one for the hole
-            CONTOURS contours;
-
-            OUTLINE_DECOMPOSER decomposer( face->glyph->outline );
-            decomposer.OutlineToSegments( &contours );
-
             std::unique_ptr<OUTLINE_GLYPH> glyph = std::make_unique<OUTLINE_GLYPH>();
             std::vector<SHAPE_LINE_CHAIN>  holes;
 
-            for( CONTOUR& c : contours )
+            for( CONTOUR& c : glyphData.m_Contours )
             {
-                GLYPH_POINTS     points = c.m_Points;
-                SHAPE_LINE_CHAIN shape;
+                std::vector<VECTOR2D> points = c.m_Points;
+                SHAPE_LINE_CHAIN      shape;
+
+                shape.ReservePoints( points.size() );
 
                 for( const VECTOR2D& v : points )
                 {
@@ -449,6 +454,8 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::u
 
             for( SHAPE_LINE_CHAIN& hole : holes )
             {
+                bool added_hole = false;
+
                 if( hole.PointCount() )
                 {
                     for( int ii = 0; ii < glyph->OutlineCount(); ++ii )
@@ -456,18 +463,27 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::u
                         if( glyph->Outline( ii ).PointInside( hole.GetPoint( 0 ) ) )
                         {
                             glyph->AddHole( std::move( hole ), ii );
+                            added_hole = true;
                             break;
                         }
                     }
+
+                    // Some lovely TTF fonts decided that winding didn't matter for outlines that
+                    // don't have holes, so holes that don't fit in any outline are added as outlines
+                    if( !added_hole )
+                        glyph->AddOutline( std::move( hole ) );
                 }
             }
 
-            // FONT TODO we might not want to do Fracture() here;
-            // knockout text (eg. silkscreen labels with a background) will
-            // need to do that after the contours have been turned into holes
-            // and vice versa
-            if( glyph->HasHoles() )
-                glyph->Fracture( SHAPE_POLY_SET::PM_FAST ); // FONT TODO verify aFastMode
+            if( glyphData.m_TriangulationData.empty() )
+            {
+                glyph->CacheTriangulation( false, false );
+                glyphData.m_TriangulationData = glyph->GetTriangulationData();
+            }
+            else
+            {
+                glyph->CacheTriangulation( glyphData.m_TriangulationData );
+            }
 
             aGlyphs->push_back( std::move( glyph ) );
         }
@@ -478,75 +494,9 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::u
     }
 
     int      ascender = abs( face->size->metrics.ascender * GLYPH_SIZE_SCALER );
-    int      height = abs( face->size->metrics.height * GLYPH_SIZE_SCALER );
     int      descender = abs( face->size->metrics.descender * GLYPH_SIZE_SCALER );
     VECTOR2I extents( cursor.x * scaleFactor.x, ( ascender + descender ) * abs( scaleFactor.y ) );
 
-    // Font metrics don't include all descenders and diacriticals, so beef them up just a little.
-    extents.y *= 1.05;
-
-    if( IsOverbar( aTextStyle ) )
-    {
-        std::vector<std::unique_ptr<GLYPH>> underscoreGlyphs;
-
-        getTextAsGlyphsUnlocked( nullptr, &underscoreGlyphs, wxT( "_" ), aSize, { 0, 0 }, ANGLE_0, false,
-                         { 0, 0 }, aTextStyle & ~TEXT_STYLE::OVERBAR );
-
-        OUTLINE_GLYPH* underscoreGlyph = static_cast<OUTLINE_GLYPH*>( underscoreGlyphs[0].get() );
-        BOX2I          underscoreBBox;
-
-        for( const VECTOR2I& pt : underscoreGlyph->Outline( 0 ).CPoints() )
-            underscoreBBox.Merge( pt );
-
-        int barThickness = underscoreBBox.GetHeight();
-
-        // Shorten the bar a little so its rounded ends don't make it over-long
-        double barTrim = barThickness / 2.0;
-        double barOffset = getOverbarOffset( ascender, height, barThickness / scaleFactor.y );
-
-        VECTOR2I topLeft( aPosition );
-        VECTOR2I topRight( aPosition );
-
-        topLeft.y += ascender * scaleFactor.y * ( 1.0 + barOffset );
-        topRight.y = topLeft.y;
-
-        topLeft.x += barTrim;
-        topRight.x += extents.x - barTrim;
-
-        extents.y *= ( 1.0 + barOffset + barOffset );
-        extents.x += barTrim;
-
-        if( IsItalic() )
-        {
-            topLeft.x += aSize.y * ITALIC_TILT;
-            topRight.x += aSize.y * ITALIC_TILT;
-            extents.x += aSize.y * ITALIC_TILT;
-        }
-
-        if( aMirror )
-        {
-            topLeft.x = aOrigin.x - ( topLeft.x - aOrigin.x );
-            topRight.x = aOrigin.x - ( topRight.x - aOrigin.x );
-        }
-
-        if( !aAngle.IsZero() )
-        {
-            RotatePoint( topLeft, aOrigin, aAngle );
-            RotatePoint( topRight, aOrigin, aAngle );
-        }
-
-        if( aGlyphs )
-        {
-            int            maxError = KiROUND( barThickness / 48 );
-            SHAPE_POLY_SET poly;
-
-            TransformOvalToPolygon( poly, topLeft, topRight, barThickness, maxError, ERROR_INSIDE );
-
-            std::unique_ptr<OUTLINE_GLYPH> overbarGlyph = std::make_unique<OUTLINE_GLYPH>( poly );
-            aGlyphs->push_back( std::move( overbarGlyph ) );
-        }
-    }
-
     hb_buffer_destroy( buf );
     hb_font_destroy( referencedFont );
 
@@ -575,7 +525,7 @@ void OUTLINE_FONT::RenderToOpenGLCanvas( KIGFX::OPENGL_GAL& aGal, const wxString
     unsigned int         glyphCount;
     hb_glyph_info_t*     glyphInfo = hb_buffer_get_glyph_infos( buf, &glyphCount );
     hb_glyph_position_t* glyphPos = hb_buffer_get_glyph_positions( buf, &glyphCount );
-    
+
     std::lock_guard<std::mutex> guard( m_freeTypeMutex );
 
     hb_font_t*           referencedFont = hb_ft_font_create_referenced( m_face );
