@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright 2012 Torsten Hueter, torstenhtr <at> gmx.de
- * Copyright 2017-2021 Kicad Developers, see AUTHORS.txt for contributors.
+ * Copyright 2017-2023 Kicad Developers, see AUTHORS.txt for contributors.
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -29,6 +29,7 @@
 #include <i18n_utility.h>
 #include <wx/crt.h>
 #include <math/util.h>
+#include <core/kicad_algo.h>
 
 using namespace KIGFX;
 
@@ -109,7 +110,6 @@ COLOR4D::COLOR4D( EDA_COLOR_T aColor )
 }
 
 
-#ifdef WX_COMPATIBILITY
 COLOR4D::COLOR4D( const wxString& aColorStr )
 {
     if( !SetFromHexString( aColorStr ) )
@@ -180,7 +180,7 @@ bool COLOR4D::SetFromHexString( const wxString& aColorString )
     str.Trim( true );
     str.Trim( false );
 
-    if( str.length() < 7 || str.GetChar( 0 ) != '#' )
+    if( str.length() < 7 || !str.StartsWith( '#' ) )
         return false;
 
     unsigned long tmp;
@@ -227,8 +227,6 @@ wxColour COLOR4D::ToColour() const
     return colour;
 }
 
-#endif
-
 
 COLOR4D COLOR4D::LegacyMix( const COLOR4D& aColor ) const
 {
@@ -246,38 +244,21 @@ COLOR4D COLOR4D::LegacyMix( const COLOR4D& aColor ) const
 }
 
 
-unsigned int COLOR4D::ToU32() const
-{
-    return ToColour().GetRGB();
-}
-
-
-void COLOR4D::FromU32( unsigned int aPackedColor )
-{
-    wxColour c;
-    c.SetRGB( aPackedColor );
-    r = c.Red() / 255.0;
-    g = c.Green() / 255.0;
-    b = c.Blue() / 255.0;
-    a = c.Alpha() / 255.0;
-}
-
-
 namespace KIGFX {
 
-const bool operator==( const COLOR4D& lhs, const COLOR4D& rhs )
+bool operator==( const COLOR4D& lhs, const COLOR4D& rhs )
 {
     return lhs.a == rhs.a && lhs.r == rhs.r && lhs.g == rhs.g && lhs.b == rhs.b;
 }
 
 
-const bool operator!=( const COLOR4D& lhs, const COLOR4D& rhs )
+bool operator!=( const COLOR4D& lhs, const COLOR4D& rhs )
 {
     return !( lhs == rhs );
 }
 
 
-const bool operator<( const COLOR4D& lhs, const COLOR4D& rhs )
+bool operator<( const COLOR4D& lhs, const COLOR4D& rhs )
 {
     if( lhs.r < rhs.r )
         return true;
@@ -595,10 +576,28 @@ EDA_COLOR_T COLOR4D::FindNearestLegacyColor( int aR, int aG, int aB )
 
 COLOR4D& COLOR4D::FromCSSRGBA( int aRed, int aGreen, int aBlue, double aAlpha )
 {
-    r = std::max( 0, std::min( 255, aRed ) ) / 255.0;
-    g = std::max( 0, std::min( 255, aGreen ) ) / 255.0;
-    b = std::max( 0, std::min( 255, aBlue ) ) / 255.0;
-    a = std::max( 0.0, std::min( 1.0, aAlpha ) );
+    r = alg::clamp( 0, aRed, 255 ) / 255.0;
+    g = alg::clamp( 0, aGreen, 255 ) / 255.0;
+    b = alg::clamp( 0, aBlue, 255 ) / 255.0;
+    a = alg::clamp( 0.0, aAlpha, 1.0 );
 
     return *this;
 }
+
+
+int COLOR4D::Compare( const COLOR4D& aRhs ) const
+{
+    if( r != aRhs.r )
+        return ( r < aRhs.r ) ? -1 : 1;
+
+    if( g != aRhs.g )
+        return ( g < aRhs.g ) ? -1 : 1;
+
+    if( b != aRhs.b )
+        return ( b < aRhs.b ) ? -1 : 1;
+
+    if( a != aRhs.a )
+        return ( a < aRhs.a ) ? -1 : 1;
+
+    return 0;
+}
