@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Mark Roszko <mark.roszko@gmail.com>
- * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2015-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,14 +33,16 @@
 #include <cstddef>
 #include <exception>
 #include <sstream>
+#include <shared_mutex>
 #include <wx/app.h>
 
 #include <build_version.h>
 #include <ki_exception.h>   // THROW_IO_ERROR
 #include <kiplatform/app.h>
 #include <kiplatform/environment.h>
-#include <pgm_base.h>
 
+#include <kiplatform/policy.h>
+#include <policy_keys.h>
 
 struct CURL_PROGRESS
 {
@@ -82,10 +84,14 @@ static size_t stream_write_callback( void* aContents, size_t aSize, size_t aNmem
     return realsize;
 }
 
+#if LIBCURL_VERSION_NUM >= 0x072000 // 7.32.0
 
 static int xferinfo( void* aProgress, curl_off_t aDLtotal, curl_off_t aDLnow, curl_off_t aULtotal,
                      curl_off_t aULnow )
 {
+    if( KICAD_CURL::IsShuttingDown() )
+        return 1; // Should abort the operation
+
     CURL_PROGRESS* progress = static_cast<CURL_PROGRESS*>( aProgress );
     curl_off_t     curtime  = 0;
 
@@ -100,17 +106,20 @@ static int xferinfo( void* aProgress, curl_off_t aDLtotal, curl_off_t aDLnow, cu
     return CURLE_OK;
 }
 
-#if LIBCURL_VERSION_NUM < 0x072000 // 7.32.0
+#else
+
 static int progressinfo( void* aProgress, double aDLtotal, double aDLnow, double aULtotal, double aULnow )
 {
     return xferinfo( aProgress, static_cast<curl_off_t>( aDLtotal ), static_cast<curl_off_t>( aDLnow ),
                      static_cast<curl_off_t>( aULtotal ), static_cast<curl_off_t>( aULnow ) );
 }
+
 #endif
 
 
 KICAD_CURL_EASY::KICAD_CURL_EASY() :
-        m_headers( nullptr )
+        m_headers( nullptr ),
+        m_curlSharedLock( KICAD_CURL::Mutex() )
 {
     m_CURL = curl_easy_init();
 
@@ -127,13 +136,31 @@ KICAD_CURL_EASY::KICAD_CURL_EASY() :
     curl_easy_setopt( m_CURL, CURLOPT_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS );
 #endif
 
-    #ifdef _WIN32
+#ifdef _WIN32
+    long sslOpts = CURLSSLOPT_NATIVE_CA;
+
+    POLICY_CURL_SSL_REVOKE policyState = KIPLATFORM::POLICY::GetPolicyEnum<POLICY_CURL_SSL_REVOKE>( POLICY_KEY_REQUESTS_CURL_REVOKE );
+    if( policyState == POLICY_CURL_SSL_REVOKE::BEST_EFFORT )
+    {
+        sslOpts |= CURLSSLOPT_REVOKE_BEST_EFFORT;
+    }
+    else if( policyState == POLICY_CURL_SSL_REVOKE::NONE )
+    {
+        sslOpts |= CURLSSLOPT_NO_REVOKE;
+    }
+
     // We need this to use the Windows Certificate store
-    curl_easy_setopt( m_CURL, CURLOPT_SSL_OPTIONS, CURLSSLOPT_NATIVE_CA );
-    #endif
+    curl_easy_setopt( m_CURL, CURLOPT_SSL_OPTIONS, sslOpts );
+#endif
+
+    if( wxGetEnv( wxT( "KICAD_CURL_VERBOSE" ), nullptr ) )
+    {
+        // note: curl verbose will end up in stderr
+        curl_easy_setopt( m_CURL, CURLOPT_VERBOSE, 1L );
+    }
 
     wxPlatformInfo platformInfo;
-    wxString application( Pgm().App().GetAppName() );
+    wxString application( wxS( "KiCad" ) );
     wxString version( GetBuildVersion() );
     wxString platform = wxS( "(" ) + wxGetOsDescription() + wxS( ";" ) + GetPlatformGetBitnessName();
 
@@ -207,6 +234,37 @@ bool KICAD_CURL_EASY::SetUserAgent( const std::string& aAgent )
 }
 
 
+bool KICAD_CURL_EASY::SetPostFields(
+        const std::vector<std::pair<std::string, std::string>>& aFields )
+{
+    std::string postfields;
+
+    for( size_t i = 0; i < aFields.size(); i++ )
+    {
+        if( i > 0 )
+            postfields += "&";
+
+        postfields += Escape( aFields[i].first );
+        postfields += "=";
+        postfields += Escape( aFields[i].second );
+    }
+
+    if( setOption<const char*>( CURLOPT_COPYPOSTFIELDS, postfields.c_str() ) != CURLE_OK )
+        return false;
+
+    return true;
+}
+
+
+bool KICAD_CURL_EASY::SetPostFields( const std::string& aField )
+{
+    if( setOption<const char*>( CURLOPT_COPYPOSTFIELDS, aField.c_str() ) != CURLE_OK )
+        return false;
+
+    return true;
+}
+
+
 bool KICAD_CURL_EASY::SetURL( const std::string& aURL )
 {
     if( setOption<const char*>( CURLOPT_URL, aURL.c_str() ) == CURLE_OK )
@@ -296,3 +354,12 @@ int KICAD_CURL_EASY::GetTransferTotal( uint64_t& aDownloadedBytes ) const
 #endif
     return result;
 }
+
+
+int KICAD_CURL_EASY::GetResponseStatusCode()
+{
+    long http_code = 0;
+    curl_easy_getinfo( m_CURL, CURLINFO_RESPONSE_CODE, &http_code );
+
+    return static_cast<int>( http_code );
+}
\ No newline at end of file
