@@ -3,7 +3,8 @@
  *
  * Copyright (C) 2017 Chris Pavlina <pavlina.chris@gmail.com>
  * Copyright (C) 2014 Henner Zeller <h.zeller@acm.org>
- * Copyright (C) 2014-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN
+ * Copyright (C) 2014-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -22,18 +23,96 @@
 #include <eda_base_frame.h>
 #include <eda_pattern_match.h>
 #include <kiface_base.h>
+#include <kiplatform/ui.h>
 #include <lib_tree_model_adapter.h>
 #include <project/project_file.h>
 #include <settings/app_settings.h>
 #include <widgets/ui_common.h>
 #include <wx/tokenzr.h>
 #include <wx/wupdlock.h>
+#include <wx/settings.h>
+#include <wx/dc.h>
 #include <string_utils.h>
 
 
 static const int kDataViewIndent = 20;
 
 
+class LIB_TREE_RENDERER : public wxDataViewCustomRenderer
+{
+public:
+    LIB_TREE_RENDERER() :
+            m_canvasItem( false )
+    {}
+
+    wxSize GetSize() const override
+    {
+        return wxSize( GetOwner()->GetWidth(), GetTextExtent( m_text ).y + 2 );
+    }
+
+    bool GetValue( wxVariant& aValue ) const override
+    {
+        aValue = m_text;
+        return true;
+    }
+
+    bool SetValue( const wxVariant& aValue ) override
+    {
+        m_text = aValue.GetString();
+        return true;
+    }
+
+    void SetAttr( const wxDataViewItemAttr& aAttr ) override
+    {
+        // Use strikethrough as a proxy for is-canvas-item
+        m_canvasItem = aAttr.GetStrikethrough();
+
+        wxDataViewItemAttr realAttr = aAttr;
+        realAttr.SetStrikethrough( false );
+
+        wxDataViewCustomRenderer::SetAttr( realAttr );
+    }
+
+    bool Render( wxRect aRect, wxDC *dc, int aState ) override
+    {
+        RenderBackground( dc, aRect );
+
+        if( m_canvasItem )
+        {
+            wxPoint points[6];
+            points[0] = aRect.GetTopLeft();
+            points[1] = aRect.GetTopRight() + wxPoint( -4, 0 );
+            points[2] = aRect.GetTopRight() + wxPoint( 0, aRect.GetHeight() / 2 );
+            points[3] = aRect.GetBottomRight() + wxPoint( -4, 1 );
+            points[4] = aRect.GetBottomLeft() + wxPoint( 0, 1 );
+            points[5] = aRect.GetTopLeft();
+
+            dc->SetPen( KIPLATFORM::UI::IsDarkTheme() ? *wxWHITE_PEN : *wxBLACK_PEN );
+            dc->DrawLines( 6, points );
+        }
+
+        aRect.Deflate( 1 );
+
+#ifdef __WXOSX__
+        // We should be able to pass wxDATAVIEW_CELL_SELECTED into RenderText() and have it do
+        // the right thing -- but it picks wxSYS_COLOUR_HIGHLIGHTTEXT on MacOS (instead
+        // of wxSYS_COLOUR_LISTBOXHIGHLIGHTTEXT).
+        if( aState & wxDATAVIEW_CELL_SELECTED )
+            dc->SetTextForeground( wxSystemSettings::GetColour( wxSYS_COLOUR_LISTBOXHIGHLIGHTTEXT ) );
+
+        RenderText( m_text, 0, aRect, dc, 0 );
+#else
+        RenderText( m_text, 0, aRect, dc, aState );
+#endif
+        return true;
+    }
+
+private:
+    bool     m_canvasItem;
+    wxString m_text;
+};
+
+
 wxDataViewItem LIB_TREE_MODEL_ADAPTER::ToItem( const LIB_TREE_NODE* aNode )
 {
     return wxDataViewItem( const_cast<void*>( static_cast<void const*>( aNode ) ) );
@@ -46,38 +125,21 @@ LIB_TREE_NODE* LIB_TREE_MODEL_ADAPTER::ToNode( wxDataViewItem aItem )
 }
 
 
-unsigned int LIB_TREE_MODEL_ADAPTER::IntoArray( const LIB_TREE_NODE& aNode,
-                                                wxDataViewItemArray& aChildren )
-{
-    unsigned int n = 0;
-
-    for( std::unique_ptr<LIB_TREE_NODE> const& child: aNode.m_Children )
-    {
-        if( child->m_Score > 0 )
-        {
-            aChildren.Add( ToItem( &*child ) );
-            ++n;
-        }
-    }
-
-    return n;
-}
-
-
 LIB_TREE_MODEL_ADAPTER::LIB_TREE_MODEL_ADAPTER( EDA_BASE_FRAME* aParent,
                                                 const wxString& aPinnedKey ) :
+        m_widget( nullptr ),
         m_parent( aParent ),
-        m_filter( SYM_FILTER_NONE ),
+        m_sort_mode( BEST_MATCH ),
         m_show_units( true ),
         m_preselect_unit( 0 ),
         m_freeze( 0 ),
-        m_widget( nullptr )
+        m_filter( nullptr )
 {
     // Default column widths.  Do not translate these names.
-    m_colWidths[ wxT( "Item" ) ] = 300;
-    m_colWidths[ wxT( "Description" ) ] = 600;
+    m_colWidths[ _HKI( "Item" ) ] = 300;
+    m_colWidths[ _HKI( "Description" ) ] = 600;
 
-    m_availableColumns = { wxT( "Item" ), wxT( "Description" ) };
+    m_availableColumns = { _HKI( "Item" ), _HKI( "Description" ) };
 
     APP_SETTINGS_BASE* cfg = Kiface().KifaceSettings();
 
@@ -87,10 +149,10 @@ LIB_TREE_MODEL_ADAPTER::LIB_TREE_MODEL_ADAPTER( EDA_BASE_FRAME* aParent,
     m_shownColumns = cfg->m_LibTree.columns;
 
     if( m_shownColumns.empty() )
-        m_shownColumns = {  wxT( "Item" ), wxT( "Description" ) };
+        m_shownColumns = {  _HKI( "Item" ), _HKI( "Description" ) };
 
-    if( m_shownColumns[0] != wxT( "Item" ) )
-        m_shownColumns.insert( m_shownColumns.begin(), wxT( "Item" ) );
+    if( m_shownColumns[0] != _HKI( "Item" ) )
+        m_shownColumns.insert( m_shownColumns.begin(), _HKI( "Item" ) );
 }
 
 
@@ -98,6 +160,38 @@ LIB_TREE_MODEL_ADAPTER::~LIB_TREE_MODEL_ADAPTER()
 {}
 
 
+std::vector<wxString> LIB_TREE_MODEL_ADAPTER::GetOpenLibs() const
+{
+    std::vector<wxString> openLibs;
+    wxDataViewItem        rootItem( nullptr );
+    wxDataViewItemArray   children;
+
+    GetChildren( rootItem, children );
+
+    for( const wxDataViewItem& child : children )
+    {
+        if( m_widget->IsExpanded( child ) )
+            openLibs.emplace_back( ToNode( child )->m_LibId.GetLibNickname().wx_str() );
+    }
+
+    return openLibs;
+}
+
+
+void LIB_TREE_MODEL_ADAPTER::OpenLibs( const std::vector<wxString>& aLibs )
+{
+    wxWindowUpdateLocker updateLock( m_widget );
+
+    for( const wxString& lib : aLibs )
+    {
+        wxDataViewItem item = FindItem( LIB_ID( lib, wxEmptyString ) );
+
+        if( item.IsOk() )
+            m_widget->Expand( item );
+    }
+}
+
+
 void LIB_TREE_MODEL_ADAPTER::SaveSettings()
 {
     if( m_widget )
@@ -109,16 +203,12 @@ void LIB_TREE_MODEL_ADAPTER::SaveSettings()
 
         for( const std::pair<const wxString, wxDataViewColumn*>& pair : m_colNameMap )
             cfg->m_LibTree.column_widths[pair.first] = pair.second->GetWidth();
+
+        cfg->m_LibTree.open_libs = GetOpenLibs();
     }
 }
 
 
-void LIB_TREE_MODEL_ADAPTER::SetFilter( SYM_FILTER_TYPE aFilter )
-{
-    m_filter = aFilter;
-}
-
-
 void LIB_TREE_MODEL_ADAPTER::ShowUnits( bool aShow )
 {
     m_show_units = aShow;
@@ -132,10 +222,11 @@ void LIB_TREE_MODEL_ADAPTER::SetPreselectNode( const LIB_ID& aLibId, int aUnit )
 }
 
 
-LIB_TREE_NODE_LIB& LIB_TREE_MODEL_ADAPTER::DoAddLibraryNode( const wxString& aNodeName,
-                                                             const wxString& aDesc, bool pinned )
+LIB_TREE_NODE_LIBRARY& LIB_TREE_MODEL_ADAPTER::DoAddLibraryNode( const wxString& aNodeName,
+                                                                 const wxString& aDesc,
+                                                                 bool pinned )
 {
-    LIB_TREE_NODE_LIB& lib_node = m_tree.AddLib( aNodeName, aDesc );
+    LIB_TREE_NODE_LIBRARY& lib_node = m_tree.AddLib( aNodeName, aDesc );
 
     lib_node.m_Pinned = pinned;
 
@@ -147,7 +238,7 @@ void LIB_TREE_MODEL_ADAPTER::DoAddLibrary( const wxString& aNodeName, const wxSt
                                            const std::vector<LIB_TREE_ITEM*>& aItemList,
                                            bool pinned, bool presorted )
 {
-    LIB_TREE_NODE_LIB& lib_node = DoAddLibraryNode( aNodeName, aDesc, pinned );
+    LIB_TREE_NODE_LIBRARY& lib_node = DoAddLibraryNode( aNodeName, aDesc, pinned );
 
     for( LIB_TREE_ITEM* item: aItemList )
         lib_node.AddItem( item );
@@ -190,50 +281,58 @@ void LIB_TREE_MODEL_ADAPTER::UpdateSearchString( const wxString& aSearch, bool a
 
         m_tree.ResetScore();
 
-        wxStringTokenizer tokenizer( aSearch );
+        // Don't cause KiCad to hang if someone accidentally pastes the PCB or schematic into
+        // the search box.
+        constexpr int MAX_TERMS = 100;
 
-        while( tokenizer.HasMoreTokens() )
+        wxStringTokenizer tokenizer( aSearch );
+        int               termCount = 0;
+
+        while( tokenizer.HasMoreTokens() && termCount < MAX_TERMS )
         {
-            wxString lib;
-            wxString term = tokenizer.GetNextToken().Lower();
+            // First search for the full token, in case it appears in a search string
+            wxString             term = tokenizer.GetNextToken().Lower();
+            EDA_COMBINED_MATCHER termMatcher( term, CTX_LIBITEM );
+
+            m_tree.UpdateScore( &termMatcher, wxEmptyString, m_filter );
+            termCount++;
 
             if( term.Contains( ":" ) )
             {
-                lib = term.BeforeFirst( ':' );
-                term = term.AfterFirst( ':' );
+                // Next search for the library:item_name
+                wxString             lib = term.BeforeFirst( ':' );
+                wxString             itemName = term.AfterFirst( ':' );
+                EDA_COMBINED_MATCHER itemNameMatcher( itemName, CTX_LIBITEM );
+
+                m_tree.UpdateScore( &itemNameMatcher, lib, nullptr );
             }
-
-            EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
-
-            m_tree.UpdateScore( matcher, lib );
         }
 
-        m_tree.SortNodes();
+        if( termCount == 0 )
+        {
+            // No terms processed; just run the filter
+            m_tree.UpdateScore( nullptr, wxEmptyString, m_filter );
+        }
+
+        m_tree.SortNodes( m_sort_mode == BEST_MATCH );
         AfterReset();
         Thaw();
     }
 
-    LIB_TREE_NODE* bestMatch = ShowResults();
+    const LIB_TREE_NODE* firstMatch = ShowResults();
 
-    if( !bestMatch )
-        bestMatch = ShowPreselect();
-
-    if( !bestMatch )
-        bestMatch = ShowSingleLibrary();
-
-    if( bestMatch )
+    if( firstMatch )
     {
-        wxDataViewItem item = wxDataViewItem( bestMatch );
+        wxDataViewItem item = ToItem( firstMatch );
         m_widget->Select( item );
 
-        // Make sure the *parent* item is visible. The selected item is the
-        // first (shown) child of the parent. So it's always right below the parent,
-        // and this way the user can also see what library the selected part belongs to,
-        // without having a case where the selection is off the screen (unless the
-        // window is a single row high, which is unlikely)
+        // Make sure the *parent* item is visible. The selected item is the first (shown) child
+        // of the parent. So it's always right below the parent, and this way the user can also
+        // see what library the selected part belongs to, without having a case where the selection
+        // is off the screen (unless the window is a single row high, which is unlikely).
         //
-        // This also happens to circumvent https://bugs.launchpad.net/kicad/+bug/1804400
-        // which appears to be a GTK+3 bug.
+        // This also happens to circumvent https://bugs.launchpad.net/kicad/+bug/1804400 which
+        // appears to be a GTK+3 bug.
         {
             wxDataViewItem parent = GetParent( item );
 
@@ -279,7 +378,7 @@ void LIB_TREE_MODEL_ADAPTER::resortTree()
     Freeze();
     BeforeReset();
 
-    m_tree.SortNodes();
+    m_tree.SortNodes( m_sort_mode == BEST_MATCH );
 
     AfterReset();
     Thaw();
@@ -317,18 +416,20 @@ wxDataViewColumn* LIB_TREE_MODEL_ADAPTER::doAddColumn( const wxString& aHeader,
     if( !m_colWidths.count( aHeader ) || m_colWidths[aHeader] < headerMinWidth.x )
         m_colWidths[aHeader] = headerMinWidth.x;
 
-    int index = m_columns.size();
+    int index = (int) m_columns.size();
 
-    wxDataViewColumn* ret = m_widget->AppendTextColumn( translatedHeader, index,
-                                                        wxDATAVIEW_CELL_INERT,
-                                                        m_colWidths[aHeader] );
-    ret->SetMinWidth( headerMinWidth.x );
+    wxDataViewColumn* col = new wxDataViewColumn(
+            translatedHeader, new LIB_TREE_RENDERER(), index, m_colWidths[aHeader], wxALIGN_NOT,
+            wxDATAVIEW_CELL_INERT | static_cast<int>( wxDATAVIEW_COL_RESIZABLE ) );
+    m_widget->AppendColumn( col );
 
-    m_columns.emplace_back( ret );
-    m_colNameMap[aHeader] = ret;
+    col->SetMinWidth( headerMinWidth.x );
+
+    m_columns.emplace_back( col );
+    m_colNameMap[aHeader] = col;
     m_colIdxMap[m_columns.size() - 1] = aHeader;
 
-    return ret;
+    return col;
 }
 
 
@@ -357,13 +458,7 @@ void LIB_TREE_MODEL_ADAPTER::SetShownColumns( const std::vector<wxString>& aColu
 LIB_ID LIB_TREE_MODEL_ADAPTER::GetAliasFor( const wxDataViewItem& aSelection ) const
 {
     const LIB_TREE_NODE* node = ToNode( aSelection );
-
-    LIB_ID emptyId;
-
-    if( !node )
-        return emptyId;
-
-    return node->m_LibId;
+    return node ? node->m_LibId : LIB_ID();
 }
 
 
@@ -422,22 +517,33 @@ wxDataViewItem LIB_TREE_MODEL_ADAPTER::FindItem( const LIB_ID& aLibId )
 }
 
 
+wxDataViewItem LIB_TREE_MODEL_ADAPTER::GetCurrentDataViewItem()
+{
+    return FindItem( m_preselect_lib_id );
+}
+
+
 unsigned int LIB_TREE_MODEL_ADAPTER::GetChildren( const wxDataViewItem&   aItem,
                                                   wxDataViewItemArray&    aChildren ) const
 {
     const LIB_TREE_NODE* node = ( aItem.IsOk() ? ToNode( aItem ) : &m_tree );
+    unsigned int         count = 0;
 
     if( node->m_Type == LIB_TREE_NODE::TYPE::ROOT
-            || node->m_Type == LIB_TREE_NODE::LIB
-            || ( m_show_units && node->m_Type == LIB_TREE_NODE::TYPE::LIBID ) )
+            || node->m_Type == LIB_TREE_NODE::LIBRARY
+            || ( m_show_units && node->m_Type == LIB_TREE_NODE::TYPE::ITEM ) )
     {
-        return IntoArray( *node, aChildren );
-    }
-    else
-    {
-        return 0;
+        for( std::unique_ptr<LIB_TREE_NODE> const& child: node->m_Children )
+        {
+            if( child->m_Score > 0 )
+            {
+                aChildren.Add( ToItem( &*child ) );
+                ++count;
+            }
+        }
     }
 
+    return count;
 }
 
 
@@ -451,7 +557,7 @@ void LIB_TREE_MODEL_ADAPTER::FinishTreeInitialization()
     for( ; idx < m_columns.size() - 1; idx++ )
     {
         wxASSERT( m_colIdxMap.count( idx ) );
-    
+
         col    = m_columns[idx];
         header = m_colIdxMap[idx];
 
@@ -468,12 +574,6 @@ void LIB_TREE_MODEL_ADAPTER::FinishTreeInitialization()
 }
 
 
-void LIB_TREE_MODEL_ADAPTER::OnSize( wxSizeEvent& aEvent )
-{
-    aEvent.Skip();
-}
-
-
 void LIB_TREE_MODEL_ADAPTER::RefreshTree()
 {
     // Yes, this is an enormous hack.  But it works on all platforms, it doesn't suffer
@@ -560,15 +660,16 @@ void LIB_TREE_MODEL_ADAPTER::GetValue( wxVariant&              aVariant,
     }
 
     LIB_TREE_NODE* node = ToNode( aItem );
-    wxASSERT( node );
+    wxCHECK( node, /* void */ );
+    wxString valueStr;
 
     switch( aCol )
     {
     case NAME_COL:
         if( node->m_Pinned )
-            aVariant = GetPinningSymbol() + UnescapeString( node->m_Name );
+            valueStr = GetPinningSymbol() + UnescapeString( node->m_Name );
         else
-            aVariant = UnescapeString( node->m_Name );
+            valueStr = UnescapeString( node->m_Name );
 
         break;
 
@@ -577,16 +678,20 @@ void LIB_TREE_MODEL_ADAPTER::GetValue( wxVariant&              aVariant,
         {
             const wxString& key = m_colIdxMap.at( aCol );
 
-            if( node->m_Fields.count( key ) )
-                aVariant = node->m_Fields.at( key );
-            else if( key == wxT( "Description" ) )
-                aVariant = node->m_Desc;
+            if( key == wxT( "Description" ) )
+                valueStr = UnescapeString( node->m_Desc );
+            else if( node->m_Fields.count( key ) )
+                valueStr = UnescapeString( node->m_Fields.at( key ) );
             else
-                aVariant = wxEmptyString;
+                valueStr = wxEmptyString;
         }
 
         break;
     }
+
+    valueStr.Replace( wxS( "\n" ), wxS( " " ) ); // Clear line breaks
+
+    aVariant = valueStr;
 }
 
 
@@ -598,114 +703,119 @@ bool LIB_TREE_MODEL_ADAPTER::GetAttr( const wxDataViewItem&   aItem,
         return false;
 
     LIB_TREE_NODE* node = ToNode( aItem );
-    wxASSERT( node );
+    wxCHECK( node, false );
 
-    if( node->m_Type != LIB_TREE_NODE::LIBID )
+    if( node->m_Type == LIB_TREE_NODE::ITEM )
     {
-        // Currently only aliases are formatted at all
-        return false;
+        if( !node->m_IsRoot && aCol == 0 )
+        {
+            // Names of non-root aliases are italicized
+            aAttr.SetItalic( true );
+            return true;
+        }
     }
 
-    if( !node->m_IsRoot && aCol == 0 )
-    {
-        // Names of non-root aliases are italicized
-        aAttr.SetItalic( true );
-        return true;
-    }
-    else
-    {
-        return false;
-    }
+    return false;
 }
 
 
-void LIB_TREE_MODEL_ADAPTER::Find( LIB_TREE_NODE& aNode,
-                                            std::function<bool( const LIB_TREE_NODE* )> aFunc,
-                                            LIB_TREE_NODE** aHighScore )
+void recursiveDescent( LIB_TREE_NODE& aNode, const std::function<int( const LIB_TREE_NODE* )>& f )
 {
     for( std::unique_ptr<LIB_TREE_NODE>& node: aNode.m_Children )
     {
-        if( aFunc( &*node ) )
+        int r = f( node.get() );
+
+        if( r == 0 )
+            break;
+        else if( r == -1 )
+            continue;
+
+        recursiveDescent( *node, f );
+    }
+}
+
+
+const LIB_TREE_NODE* LIB_TREE_MODEL_ADAPTER::ShowResults()
+{
+    const LIB_TREE_NODE* firstMatch = nullptr;
+
+    // Expand parents of leaf nodes with some level of matching
+    recursiveDescent( m_tree,
+            [&]( const LIB_TREE_NODE* n )
+            {
+                if( n->m_Type == LIB_TREE_NODE::TYPE::ITEM && n->m_Score > 1 )
+                {
+                    if( !firstMatch )
+                        firstMatch = n;
+                    else if( n->m_Score > firstMatch->m_Score )
+                        firstMatch = n;
+
+                    m_widget->ExpandAncestors( ToItem( n ) );
+                }
+
+                return 1; // keep going to expand ancestors of all found items
+            } );
+
+    // If no matches, find and show the preselect node
+    if( !firstMatch && m_preselect_lib_id.IsValid() )
+    {
+        recursiveDescent( m_tree,
+                [&]( const LIB_TREE_NODE* n )
+                {
+                    // Don't match the recent and already placed libraries
+                    if( n->m_Name.StartsWith( "-- " ) )
+                        return -1; // Skip this node and its children
+
+                    if( n->m_Type == LIB_TREE_NODE::ITEM
+                              && ( n->m_Children.empty() || !m_preselect_unit )
+                              && m_preselect_lib_id == n->m_LibId )
+                    {
+                        firstMatch = n;
+                        m_widget->ExpandAncestors( ToItem( n ) );
+                        return 0;
+                    }
+                    else if( n->m_Type == LIB_TREE_NODE::UNIT
+                              && ( m_preselect_unit && m_preselect_unit == n->m_Unit )
+                              && m_preselect_lib_id == n->m_Parent->m_LibId )
+                    {
+                        firstMatch = n;
+                        m_widget->ExpandAncestors( ToItem( n ) );
+                        return 0;
+                    }
+
+                    return 1;
+                } );
+    }
+
+    // If still no matches expand a single library if there is only one
+    if( !firstMatch )
+    {
+        int libraries = 0;
+
+        for( const std::unique_ptr<LIB_TREE_NODE>& child : m_tree.m_Children )
         {
-            if( !(*aHighScore) || node->m_Score > (*aHighScore)->m_Score )
-                (*aHighScore) = &*node;
+            if( !child->m_Name.StartsWith( "-- " ) )
+                 libraries++;
         }
 
-        Find( *node, aFunc, aHighScore );
+        if( libraries != 1 )
+            return nullptr;
+
+        recursiveDescent( m_tree,
+                [&]( const LIB_TREE_NODE* n )
+                {
+                    if( n->m_Type == LIB_TREE_NODE::TYPE::ITEM )
+                    {
+                        firstMatch = n;
+                        m_widget->ExpandAncestors( ToItem( n ) );
+                        return 0;
+                    }
+
+                    return 1;
+                } );
     }
+
+    return firstMatch;
 }
 
 
-LIB_TREE_NODE* LIB_TREE_MODEL_ADAPTER::ShowResults()
-{
-    LIB_TREE_NODE* highScore = nullptr;
-
-    Find( m_tree,
-                   []( LIB_TREE_NODE const* n )
-                   {
-                       // return leaf nodes with some level of matching
-                       return n->m_Type == LIB_TREE_NODE::TYPE::LIBID && n->m_Score > 1;
-                   },
-                   &highScore );
-
-    if( highScore)
-    {
-        wxDataViewItem item = wxDataViewItem( highScore );
-        m_widget->ExpandAncestors( item );
-    }
-
-    return highScore;
-}
-
-
-LIB_TREE_NODE* LIB_TREE_MODEL_ADAPTER::ShowPreselect()
-{
-    LIB_TREE_NODE* highScore = nullptr;
-
-    if( !m_preselect_lib_id.IsValid() )
-        return highScore;
-
-    Find( m_tree,
-            [&]( LIB_TREE_NODE const* n )
-            {
-                if( n->m_Type == LIB_TREE_NODE::LIBID && ( n->m_Children.empty() ||
-                                                           !m_preselect_unit ) )
-                    return m_preselect_lib_id == n->m_LibId;
-                else if( n->m_Type == LIB_TREE_NODE::UNIT && m_preselect_unit )
-                    return m_preselect_lib_id == n->m_Parent->m_LibId &&
-                            m_preselect_unit == n->m_Unit;
-                else
-                    return false;
-            },
-            &highScore );
-
-    if( highScore)
-    {
-        wxDataViewItem item = wxDataViewItem( highScore );
-        m_widget->ExpandAncestors( item );
-    }
-
-    return highScore;
-}
-
-
-LIB_TREE_NODE* LIB_TREE_MODEL_ADAPTER::ShowSingleLibrary()
-{
-    LIB_TREE_NODE* highScore = nullptr;
-
-    Find( m_tree,
-                   []( LIB_TREE_NODE const* n )
-                   {
-                       return n->m_Type == LIB_TREE_NODE::TYPE::LIBID &&
-                              n->m_Parent->m_Parent->m_Children.size() == 1;
-                   },
-                   &highScore );
-
-    if( highScore)
-    {
-        wxDataViewItem item = wxDataViewItem( highScore );
-        m_widget->ExpandAncestors( item );
-    }
-
-    return highScore;
-}
