@@ -7,7 +7,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 1992-2012 Lorenzo Marcantonio, l.marcantonio@logossrl.com
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023, 2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,15 +28,19 @@
  */
 
 #include <algorithm>
+#include <cstdio> // snprintf
 
 #include <wx/filename.h>
 #include <wx/mstream.h>
 #include <wx/zstream.h>
+#include <wx/wfstream.h>
+#include <wx/datstrm.h>
 
 #include <advanced_config.h>
-#include <eda_text.h> // for IsGotoPageHref
+#include <common.h>               // ResolveUriByEnvVars
+#include <eda_text.h>             // for IsGotoPageHref
 #include <font/font.h>
-#include <ignore.h>
+#include <core/ignore.h>
 #include <macros.h>
 #include <trigo.h>
 #include <string_utils.h>
@@ -93,11 +97,12 @@ std::string PDF_PLOTTER::encodeStringForPlotter( const wxString& aText )
     {
         result = "<FEFF";
 
+
         for( size_t ii = 0; ii < aText.Len(); ii++ )
         {
             unsigned int code = aText[ii];
             char buffer[16];
-            sprintf( buffer, "%4.4X", code );
+            std::snprintf( buffer, sizeof( buffer ), "%4.4X", code );
             result += buffer;
 
         }
@@ -179,29 +184,29 @@ void PDF_PLOTTER::emitSetRGBColor( double r, double g, double b, double a )
 }
 
 
-void PDF_PLOTTER::SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle )
+void PDF_PLOTTER::SetDash( int aLineWidth, LINE_STYLE aLineStyle )
 {
     wxASSERT( m_workFile );
 
     switch( aLineStyle )
     {
-    case PLOT_DASH_TYPE::DASH:
+    case LINE_STYLE::DASH:
         fprintf( m_workFile, "[%d %d] 0 d\n",
                 (int) GetDashMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ) );
         break;
 
-    case PLOT_DASH_TYPE::DOT:
+    case LINE_STYLE::DOT:
         fprintf( m_workFile, "[%d %d] 0 d\n",
                 (int) GetDotMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ) );
         break;
 
-    case PLOT_DASH_TYPE::DASHDOT:
+    case LINE_STYLE::DASHDOT:
         fprintf( m_workFile, "[%d %d %d %d] 0 d\n",
                 (int) GetDashMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ),
                 (int) GetDotMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ) );
         break;
 
-    case PLOT_DASH_TYPE::DASHDOTDOT:
+    case LINE_STYLE::DASHDOTDOT:
         fprintf( m_workFile, "[%d %d %d %d %d %d] 0 d\n",
                 (int) GetDashMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ),
                 (int) GetDotMarkLenIU( aLineWidth ), (int) GetDashGapLenIU( aLineWidth ),
@@ -218,6 +223,9 @@ void PDF_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill, int
 {
     wxASSERT( m_workFile );
 
+    if( fill == FILL_T::NO_FILL && width <= 0 )
+        return;
+
     SetCurrentLineWidth( width );
 
     VECTOR2I size = p2 - p1;
@@ -225,8 +233,6 @@ void PDF_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill, int
     if( size.x == 0 && size.y == 0 )
     {
         // Can't draw zero-sized rectangles
-        SetCurrentLineWidth( width );
-
         MoveTo( VECTOR2I( p1.x, p1.y ) );
         FinishTo( VECTOR2I( p1.x, p1.y ) );
 
@@ -252,14 +258,25 @@ void PDF_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill, int
     VECTOR2D p1_dev = userToDeviceCoordinates( p1 );
     VECTOR2D p2_dev = userToDeviceCoordinates( p2 );
 
+    char paintOp;
+
+    if( fill == FILL_T::NO_FILL )
+        paintOp = 'S';
+    else
+        paintOp = width > 0 ? 'B' : 'f';
+
     fprintf( m_workFile, "%g %g %g %g re %c\n", p1_dev.x, p1_dev.y, p2_dev.x - p1_dev.x,
-             p2_dev.y - p1_dev.y, fill == FILL_T::NO_FILL ? 'S' : 'B' );
+             p2_dev.y - p1_dev.y, paintOp );
 }
 
 
 void PDF_PLOTTER::Circle( const VECTOR2I& pos, int diametre, FILL_T aFill, int width )
 {
     wxASSERT( m_workFile );
+
+    if( aFill == FILL_T::NO_FILL && width <= 0 )
+        return;
+
     VECTOR2D pos_dev = userToDeviceCoordinates( pos );
     double   radius = userToDeviceSize( diametre / 2.0 );
 
@@ -311,64 +328,8 @@ void PDF_PLOTTER::Circle( const VECTOR2I& pos, int diametre, FILL_T aFill, int w
 }
 
 
-void PDF_PLOTTER::Arc( const VECTOR2I& aCenter, const VECTOR2I& aStart, const VECTOR2I& aEnd,
-                       FILL_T aFill, int aWidth, int aMaxError )
-{
-    wxASSERT( m_workFile );
-
-    /*
-     * Arcs are not so easily approximated by beziers (in the general case), so we approximate
-     * them in the old way
-     */
-    EDA_ANGLE startAngle( aStart - aCenter );
-    EDA_ANGLE endAngle( aEnd - aCenter );
-    int       radius = ( aStart - aCenter ).EuclideanNorm();
-    int       numSegs = GetArcToSegmentCount( radius, aMaxError, FULL_CIRCLE );
-    EDA_ANGLE delta = ANGLE_360 / std::max( 8, numSegs );
-    VECTOR2I  start( aStart );
-    VECTOR2I  end( aEnd );
-    VECTOR2I  pt;
-
-    if( startAngle > endAngle )
-    {
-        if( endAngle < ANGLE_0 )
-            endAngle.Normalize();
-        else
-            startAngle = startAngle.Normalize() - ANGLE_360;
-    }
-
-    SetCurrentLineWidth( aWidth );
-    VECTOR2D pos_dev = userToDeviceCoordinates( start );
-    fprintf( m_workFile, "%g %g m ", pos_dev.x, pos_dev.y );
-
-    for( EDA_ANGLE ii = delta; startAngle + ii < endAngle; ii += delta )
-    {
-        pt = start;
-        RotatePoint( pt, aCenter, -ii );
-
-        pos_dev = userToDeviceCoordinates( pt );
-        fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
-    }
-
-    pos_dev = userToDeviceCoordinates( end );
-    fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
-
-    // The arc is drawn... if not filled we stroke it, otherwise we finish
-    // closing the pie at the center
-    if( aFill == FILL_T::NO_FILL )
-    {
-        fputs( "S\n", m_workFile );
-    }
-    else
-    {
-        pos_dev = userToDeviceCoordinates( aCenter );
-        fprintf( m_workFile, "%g %g l b\n", pos_dev.x, pos_dev.y );
-    }
-}
-
-
 void PDF_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
-                       const EDA_ANGLE& aEndAngle, double aRadius, FILL_T aFill, int aWidth )
+                       const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill, int aWidth )
 {
     wxASSERT( m_workFile );
 
@@ -382,36 +343,33 @@ void PDF_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
      * Arcs are not so easily approximated by beziers (in the general case), so we approximate
      * them in the old way
      */
-    EDA_ANGLE       startAngle( aStartAngle );
-    EDA_ANGLE       endAngle( aEndAngle );
+    EDA_ANGLE       startAngle = -aStartAngle;
+    EDA_ANGLE       endAngle = startAngle - aAngle;
     VECTOR2I        start;
     VECTOR2I        end;
     const EDA_ANGLE delta( 5, DEGREES_T );   // increment to draw circles
 
     if( startAngle > endAngle )
-    {
         std::swap( startAngle, endAngle );
-        std::swap( start, end );
-    }
 
     SetCurrentLineWidth( aWidth );
 
     // Usual trig arc plotting routine...
-    start.x = aCenter.x + KiROUND( aRadius * (-startAngle).Cos() );
-    start.y = aCenter.y + KiROUND( aRadius * (-startAngle).Sin() );
+    start.x = KiROUND( aCenter.x + aRadius * ( -startAngle ).Cos() );
+    start.y = KiROUND( aCenter.y + aRadius * ( -startAngle ).Sin() );
     VECTOR2D pos_dev = userToDeviceCoordinates( start );
     fprintf( m_workFile, "%g %g m ", pos_dev.x, pos_dev.y );
 
     for( EDA_ANGLE ii = startAngle + delta; ii < endAngle; ii += delta )
     {
-        end.x = aCenter.x + KiROUND( aRadius * (-ii).Cos() );
-        end.y = aCenter.y + KiROUND( aRadius * (-ii).Sin() );
+        end.x = KiROUND( aCenter.x + aRadius * ( -ii ).Cos() );
+        end.y = KiROUND( aCenter.y + aRadius * ( -ii ).Sin() );
         pos_dev = userToDeviceCoordinates( end );
         fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
     }
 
-    end.x = aCenter.x + KiROUND( aRadius * (-endAngle).Cos() );
-    end.y = aCenter.y + KiROUND( aRadius * (-endAngle).Sin() );
+    end.x = KiROUND( aCenter.x + aRadius * ( -endAngle ).Cos() );
+    end.y = KiROUND( aCenter.y + aRadius * ( -endAngle ).Sin() );
     pos_dev = userToDeviceCoordinates( end );
     fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
 
@@ -434,18 +392,21 @@ void PDF_PLOTTER::PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T aFi
 {
     wxASSERT( m_workFile );
 
+    if( aFill == FILL_T::NO_FILL && aWidth <= 0 )
+        return;
+
     if( aCornerList.size() <= 1 )
         return;
 
     SetCurrentLineWidth( aWidth );
 
     VECTOR2D pos = userToDeviceCoordinates( aCornerList[0] );
-    fprintf( m_workFile, "%g %g m\n", pos.x, pos.y );
+    fprintf( m_workFile, "%f %f m\n", pos.x, pos.y );
 
     for( unsigned ii = 1; ii < aCornerList.size(); ii++ )
     {
         pos = userToDeviceCoordinates( aCornerList[ii] );
-        fprintf( m_workFile, "%g %g l\n", pos.x, pos.y );
+        fprintf( m_workFile, "%f %f l\n", pos.x, pos.y );
     }
 
     // Close path and stroke and/or fill
@@ -478,7 +439,7 @@ void PDF_PLOTTER::PenTo( const VECTOR2I& pos, char plume )
     if( m_penState != plume || pos != m_penLastpos )
     {
         VECTOR2D pos_dev = userToDeviceCoordinates( pos );
-        fprintf( m_workFile, "%g %g %c\n",
+        fprintf( m_workFile, "%f %f %c\n",
                  pos_dev.x, pos_dev.y,
                  ( plume=='D' ) ? 'l' : 'm' );
     }
@@ -500,6 +461,53 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
     VECTOR2I start( aPos.x - drawsize.x / 2, aPos.y + drawsize.y / 2 );
     VECTOR2D dev_start = userToDeviceCoordinates( start );
 
+    // Deduplicate images
+    auto findHandleForImage = [&]( const wxImage& aCurrImage ) -> int
+    {
+        for( const auto& [imgHandle, image] : m_imageHandles )
+        {
+            if( image.IsSameAs( aCurrImage ) )
+                return imgHandle;
+
+            if( image.GetWidth() != aCurrImage.GetWidth() )
+                continue;
+
+            if( image.GetHeight() != aCurrImage.GetHeight() )
+                continue;
+
+            if( image.GetType() != aCurrImage.GetType() )
+                continue;
+
+            if( image.HasAlpha() != aCurrImage.HasAlpha() )
+                continue;
+
+            if( image.HasMask() != aCurrImage.HasMask() || image.GetMaskRed() != aCurrImage.GetMaskRed()
+                || image.GetMaskGreen() != aCurrImage.GetMaskGreen()
+                || image.GetMaskBlue() != aCurrImage.GetMaskBlue() )
+                continue;
+
+            int pixCount = image.GetWidth() * image.GetHeight();
+
+            if( memcmp( image.GetData(), aCurrImage.GetData(), pixCount * 3 ) != 0 )
+                continue;
+
+            if( image.HasAlpha() && memcmp( image.GetAlpha(), aCurrImage.GetAlpha(), pixCount ) != 0 )
+                continue;
+
+            return imgHandle;
+        }
+
+        return -1;
+    };
+
+    int imgHandle = findHandleForImage( aImage );
+
+    if( imgHandle == -1 )
+    {
+        imgHandle = allocPdfObject();
+        m_imageHandles.emplace( imgHandle, aImage );
+    }
+
     /* PDF has an uhm... simplified coordinate system handling. There is
        *one* operator to do everything (the PS concat equivalent). At least
        they kept the matrix stack to save restore environments. Also images
@@ -515,71 +523,8 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
              userToDeviceSize( drawsize.y ),
              dev_start.x, dev_start.y );
 
-    /* An inline image is a cross between a dictionary and a stream.
-       A real ugly construct (compared with the elegance of the PDF
-       format). Also it accepts some 'abbreviations', which is stupid
-       since the content stream is usually compressed anyway... */
-    fprintf( m_workFile,
-             "BI\n"
-             "  /BPC 8\n"
-             "  /CS %s\n"
-             "  /W %d\n"
-             "  /H %d\n"
-             "ID\n", m_colorMode ? "/RGB" : "/G", pix_size.x, pix_size.y );
-
-    /* Here comes the stream (in binary!). I *could* have hex or ascii84
-       encoded it, but who cares? I'll go through zlib anyway */
-    for( int y = 0; y < pix_size.y; y++ )
-    {
-        for( int x = 0; x < pix_size.x; x++ )
-        {
-            unsigned char r = aImage.GetRed( x, y ) & 0xFF;
-            unsigned char g = aImage.GetGreen( x, y ) & 0xFF;
-            unsigned char b = aImage.GetBlue( x, y ) & 0xFF;
-
-            // PDF inline images don't support alpha, so premultiply against white background
-            if( aImage.HasAlpha() )
-            {
-                unsigned char alpha = aImage.GetAlpha( x, y ) & 0xFF;
-
-                if( alpha < 0xFF )
-                {
-                    float d = alpha / 255.0;
-                    float a = 1.0 - d;
-                    r = std::min( KiROUND( r * d + ( a * 0xFF ) ), 0xFF );
-                    g = std::min( KiROUND( g * d + ( a * 0xFF ) ), 0xFF );
-                    b = std::min( KiROUND( b * d + ( a * 0xFF ) ), 0xFF );
-                }
-            }
-
-            if( aImage.HasMask() )
-            {
-                if( r == aImage.GetMaskRed() && g == aImage.GetMaskGreen()
-                  && b == aImage.GetMaskBlue() )
-                {
-                    r = 0xFF;
-                    g = 0xFF;
-                    b = 0xFF;
-                }
-            }
-
-            // As usual these days, stdio buffering has to suffeeeeerrrr
-            if( m_colorMode )
-            {
-                putc( r, m_workFile );
-                putc( g, m_workFile );
-                putc( b, m_workFile );
-            }
-            else
-            {
-                // Greyscale conversion (CIE 1931)
-                unsigned char grey = KiROUND( r * 0.2126 + g * 0.7152 + b * 0.0722 );
-                putc( grey, m_workFile );
-            }
-        }
-    }
-
-    fputs( "EI Q\n", m_workFile ); // Finish step 2 and do step 3
+    fprintf( m_workFile, "/Im%d Do\n", imgHandle );
+    fputs( "Q\n", m_workFile );
 }
 
 
@@ -737,6 +682,82 @@ void PDF_PLOTTER::StartPage( const wxString& aPageNumber, const wxString& aPageN
 }
 
 
+void WriteImageStream( const wxImage& aImage, wxDataOutputStream& aOut, wxColor bg, bool colorMode )
+{
+    int w = aImage.GetWidth();
+    int h = aImage.GetHeight();
+
+    for( int y = 0; y < h; y++ )
+    {
+        for( int x = 0; x < w; x++ )
+        {
+            unsigned char r = aImage.GetRed( x, y ) & 0xFF;
+            unsigned char g = aImage.GetGreen( x, y ) & 0xFF;
+            unsigned char b = aImage.GetBlue( x, y ) & 0xFF;
+
+            if( aImage.HasMask() )
+            {
+                if( r == aImage.GetMaskRed() && g == aImage.GetMaskGreen()
+                    && b == aImage.GetMaskBlue() )
+                {
+                    r = bg.Red();
+                    g = bg.Green();
+                    b = bg.Blue();
+                }
+            }
+
+            if( colorMode )
+            {
+                aOut.Write8( r );
+                aOut.Write8( g );
+                aOut.Write8( b );
+            }
+            else
+            {
+                // Greyscale conversion (CIE 1931)
+                unsigned char grey = KiROUND( r * 0.2126 + g * 0.7152 + b * 0.0722 );
+
+                aOut.Write8( grey );
+            }
+        }
+    }
+}
+
+
+void WriteImageSMaskStream( const wxImage& aImage, wxDataOutputStream& aOut )
+{
+    int w = aImage.GetWidth();
+    int h = aImage.GetHeight();
+
+    if( aImage.HasMask() )
+    {
+        for( int y = 0; y < h; y++ )
+        {
+            for( int x = 0; x < w; x++ )
+            {
+                unsigned char a = 255;
+                unsigned char r = aImage.GetRed( x, y );
+                unsigned char g = aImage.GetGreen( x, y );
+                unsigned char b = aImage.GetBlue( x, y );
+
+                if( r == aImage.GetMaskRed() && g == aImage.GetMaskGreen()
+                    && b == aImage.GetMaskBlue() )
+                {
+                    a = 0;
+                }
+
+                aOut.Write8( a );
+            }
+        }
+    }
+    else if( aImage.HasAlpha() )
+    {
+        int size = w * h;
+        aOut.Write8( aImage.GetAlpha(), size );
+    }
+}
+
+
 void PDF_PLOTTER::ClosePage()
 {
     wxASSERT( m_workFile );
@@ -756,6 +777,14 @@ void PDF_PLOTTER::ClosePage()
                 VECTOR2D retval = VECTOR2D( aCoord ) * PTsPERMIL / ( m_IUsPerDecimil * 10 );
                 // PDF y=0 is at bottom of page, invert coordinate
                 retval.y = psPaperSize.y - retval.y;
+
+                // The pdf plot can be mirrored (from left to right). So mirror the
+                // x coordinate if m_plotMirror is set
+                if( m_plotMirror )
+                {
+                    retval.x = ( psPaperSize.x - retval.x );
+                }
+
                 return retval;
             };
 
@@ -832,11 +861,13 @@ void PDF_PLOTTER::ClosePage()
              "/Parent %d 0 R\n"
              "/Resources <<\n"
              "    /ProcSet [/PDF /Text /ImageC /ImageB]\n"
-             "    /Font %d 0 R >>\n"
+             "    /Font %d 0 R\n"
+             "    /XObject %d 0 R >>\n"
              "/MediaBox [0 0 %g %g]\n"
              "/Contents %d 0 R\n",
              m_pageTreeHandle,
              m_fontResDictHandle,
+             m_imgResDictHandle,
              psPaperSize.x,
              psPaperSize.y,
              m_pageStreamHandle );
@@ -934,6 +965,10 @@ bool PDF_PLOTTER::StartPlot( const wxString& aPageNumber, const wxString& aPageN
        (it *could* be inherited via the Pages tree */
     m_fontResDictHandle = allocPdfObject();
 
+    m_imgResDictHandle = allocPdfObject();
+
+    m_jsNamesHandle = allocPdfObject();
+
     /* Now, the PDF is read from the end, (more or less)... so we start
        with the page stream for page 1. Other more important stuff is written
        at the end */
@@ -1005,31 +1040,32 @@ void PDF_PLOTTER::emitOutlineNode( OUTLINE_NODE* node, int parentHandle, int nex
         startPdfObject( nodeHandle );
 
         fprintf( m_outputFile,
-                 "<< /Title %s\n"
-                 "   /Parent %d 0 R\n",
+                 "<<\n"
+                 "/Title %s\n"
+                 "/Parent %d 0 R\n",
                  encodeStringForPlotter(node->title ).c_str(),
                  parentHandle);
 
         if( nextNode > 0 )
         {
-            fprintf( m_outputFile, "   /Next %d 0 R\n", nextNode );
+            fprintf( m_outputFile, "/Next %d 0 R\n", nextNode );
         }
 
         if( prevNode > 0 )
         {
-            fprintf( m_outputFile, "   /Prev %d 0 R\n", prevNode );
+            fprintf( m_outputFile, "/Prev %d 0 R\n", prevNode );
         }
 
         if( node->children.size() > 0 )
         {
-            fprintf( m_outputFile, "   /Count %zd\n", -1 * node->children.size() );
-            fprintf( m_outputFile, "   /First %d 0 R\n", node->children.front()->entryHandle );
-            fprintf( m_outputFile, "   /Last %d 0 R\n", node->children.back()->entryHandle );
+            fprintf( m_outputFile, "/Count %zd\n", -1 * node->children.size() );
+            fprintf( m_outputFile, "/First %d 0 R\n", node->children.front()->entryHandle );
+            fprintf( m_outputFile, "/Last %d 0 R\n", node->children.back()->entryHandle );
         }
 
         if( node->actionHandle != -1 )
         {
-            fprintf( m_outputFile, "   /A %d 0 R\n", node->actionHandle );
+            fprintf( m_outputFile, "/A %d 0 R\n", node->actionHandle );
         }
 
         fputs( ">>\n", m_outputFile );
@@ -1131,6 +1167,112 @@ bool PDF_PLOTTER::EndPlot()
     fputs( ">>\n", m_outputFile );
     closePdfObject();
 
+    // Named image dictionary (was allocated, now we emit it)
+    startPdfObject( m_imgResDictHandle );
+    fputs( "<<\n", m_outputFile );
+
+    for( const auto& [imgHandle, image] : m_imageHandles )
+    {
+        fprintf( m_outputFile, "    /Im%d %d 0 R\n", imgHandle, imgHandle );
+    }
+
+    fputs( ">>\n", m_outputFile );
+    closePdfObject();
+
+    // Emit images with optional SMask for transparency
+    for( const auto& [imgHandle, image] : m_imageHandles )
+    {
+        // Init wxFFile so wxFFileOutputStream won't close file in dtor.
+        wxFFile outputFFile( m_outputFile );
+
+        // Image
+        startPdfObject( imgHandle );
+        int imgLenHandle = allocPdfObject();
+        int smaskHandle = ( image.HasAlpha() || image.HasMask() ) ? allocPdfObject() : -1;
+
+        fprintf( m_outputFile,
+                 "<<\n"
+                 "/Type /XObject\n"
+                 "/Subtype /Image\n"
+                 "/BitsPerComponent 8\n"
+                 "/ColorSpace %s\n"
+                 "/Width %d\n"
+                 "/Height %d\n"
+                 "/Filter /FlateDecode\n"
+                 "/Length %d 0 R\n", // Length is deferred
+                 m_colorMode ? "/DeviceRGB" : "/DeviceGray", image.GetWidth(), image.GetHeight(),
+                 imgLenHandle );
+
+        if( smaskHandle != -1 )
+            fprintf( m_outputFile, "/SMask %d 0 R\n", smaskHandle );
+
+        fputs( ">>\n", m_outputFile );
+        fputs( "stream\n", m_outputFile );
+
+        long imgStreamStart = ftell( m_outputFile );
+
+        {
+            wxFFileOutputStream ffos( outputFFile );
+            wxZlibOutputStream  zos( ffos, wxZ_BEST_COMPRESSION, wxZLIB_ZLIB );
+            wxDataOutputStream  dos( zos );
+
+            WriteImageStream( image, dos, m_renderSettings->GetBackgroundColor().ToColour(),
+                              m_colorMode );
+        }
+
+        long imgStreamSize = ftell( m_outputFile ) - imgStreamStart;
+
+        fputs( "\nendstream\n", m_outputFile );
+        closePdfObject();
+
+        startPdfObject( imgLenHandle );
+        fprintf( m_outputFile, "%ld\n", imgStreamSize );
+        closePdfObject();
+
+        if( smaskHandle != -1 )
+        {
+            // SMask
+            startPdfObject( smaskHandle );
+            int smaskLenHandle = allocPdfObject();
+
+            fprintf( m_outputFile,
+                     "<<\n"
+                     "/Type /XObject\n"
+                     "/Subtype /Image\n"
+                     "/BitsPerComponent 8\n"
+                     "/ColorSpace /DeviceGray\n"
+                     "/Width %d\n"
+                     "/Height %d\n"
+                     "/Length %d 0 R\n"
+                     "/Filter /FlateDecode\n"
+                     ">>\n", // Length is deferred
+                     image.GetWidth(), image.GetHeight(), smaskLenHandle );
+
+            fputs( "stream\n", m_outputFile );
+
+            long smaskStreamStart = ftell( m_outputFile );
+
+            {
+                wxFFileOutputStream ffos( outputFFile );
+                wxZlibOutputStream  zos( ffos, wxZ_BEST_COMPRESSION, wxZLIB_ZLIB );
+                wxDataOutputStream  dos( zos );
+
+                WriteImageSMaskStream( image, dos );
+            }
+
+            long smaskStreamSize = ftell( m_outputFile ) - smaskStreamStart;
+
+            fputs( "\nendstream\n", m_outputFile );
+            closePdfObject();
+
+            startPdfObject( smaskLenHandle );
+            fprintf( m_outputFile, "%u\n", (unsigned) smaskStreamSize );
+            closePdfObject();
+        }
+
+        outputFFile.Detach(); // Don't close it
+    }
+
     for( const auto& [ linkHandle, linkPair ] : m_hyperlinkHandles )
     {
         const BOX2D&    box = linkPair.first;
@@ -1139,10 +1281,11 @@ bool PDF_PLOTTER::EndPlot()
         startPdfObject( linkHandle );
 
         fprintf( m_outputFile,
-                 "<< /Type /Annot\n"
-                 "   /Subtype /Link\n"
-                 "   /Rect [%g %g %g %g]\n"
-                 "   /Border [16 16 0]\n",
+                 "<<\n"
+                 "/Type /Annot\n"
+                 "/Subtype /Link\n"
+                 "/Rect [%g %g %g %g]\n"
+                 "/Border [16 16 0]\n",
                  box.GetLeft(), box.GetBottom(), box.GetRight(), box.GetTop() );
 
         wxString pageNumber;
@@ -1155,7 +1298,7 @@ bool PDF_PLOTTER::EndPlot()
                 if( m_pageNumbers[ii] == pageNumber )
                 {
                     fprintf( m_outputFile,
-                             "   /Dest [%d 0 R /FitB]\n"
+                             "/Dest [%d 0 R /FitB]\n"
                              ">>\n",
                              m_pageHandles[ii] );
 
@@ -1167,15 +1310,14 @@ bool PDF_PLOTTER::EndPlot()
             if( !pageFound )
             {
                 // destination page is not being plotted, assign the NOP action to the link
-                fprintf( m_outputFile,
-                         "   /A << /Type /Action /S /NOP >>\n"
-                         ">>\n" );
+                fprintf( m_outputFile, "/A << /Type /Action /S /NOP >>\n"
+                                       ">>\n" );
             }
         }
         else
         {
             fprintf( m_outputFile,
-                     "   /A << /Type /Action /S /URI /URI %s >>\n"
+                     "/A << /Type /Action /S /URI /URI %s >>\n"
                      ">>\n",
                      encodeStringForPlotter( url ).c_str() );
         }
@@ -1187,7 +1329,7 @@ bool PDF_PLOTTER::EndPlot()
     {
         const BOX2D&                 box = menuPair.first;
         const std::vector<wxString>& urls = menuPair.second;
-        wxString                     js = wxT( "var aParams = [ " );
+        wxString                     js = wxT( "ShM([\n" );
 
         for( const wxString& url : urls )
         {
@@ -1199,21 +1341,40 @@ bool PDF_PLOTTER::EndPlot()
                 {
                     wxString href = property.substr( property.Find( "http:" ) );
 
-                    js += wxString::Format( wxT( "{ cName: '%s', cReturn: '%s' }, " ),
-                                            EscapeString( property, CTX_JS_STR ),
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
+                    js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
+                                            EscapeString( href, CTX_JS_STR ),
                                             EscapeString( href, CTX_JS_STR ) );
                 }
                 else if( property.Find( "https:" ) >= 0 )
                 {
                     wxString href = property.substr( property.Find( "https:" ) );
 
-                    js += wxString::Format( wxT( "{ cName: '%s', cReturn: '%s' }, " ),
-                                            EscapeString( property, CTX_JS_STR ),
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
+                    js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
+                                            EscapeString( href, CTX_JS_STR ),
+                                            EscapeString( href, CTX_JS_STR ) );
+                }
+                else if( property.Find( "file:" ) >= 0 )
+                {
+                    wxString href = property.substr( property.Find( "file:" ) );
+
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
+                    href = NormalizeFileUri( href );
+
+                    js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
+                                            EscapeString( href, CTX_JS_STR ),
                                             EscapeString( href, CTX_JS_STR ) );
                 }
                 else
                 {
-                    js += wxString::Format( wxT( "{ cName: '%s', cReturn: null }, " ),
+                    js += wxString::Format( wxT( "[\"%s\"],\n" ),
                                             EscapeString( property, CTX_JS_STR ) );
                 }
             }
@@ -1227,44 +1388,79 @@ bool PDF_PLOTTER::EndPlot()
                     {
                         wxString menuText = wxString::Format( _( "Show Page %s" ), pageNumber );
 
-                        js += wxString::Format( wxT( "{ cName: '%s', cReturn: '#%d' }, " ),
+                        js += wxString::Format( wxT( "[\"%s\", \"#%d\"],\n" ),
                                                 EscapeString( menuText, CTX_JS_STR ),
                                                 static_cast<int>( ii ) );
                         break;
                     }
                 }
             }
-            else if( url.StartsWith( "http:" ) || url.StartsWith( "https:" ) )
+            else if( url.StartsWith( "http:" ) || url.StartsWith( "https:" )
+                   || url.StartsWith( "file:" ) )
             {
-                wxString menuText = wxString::Format( _( "Open %s" ), url );
+                wxString href = url;
 
-                js += wxString::Format( wxT( "{ cName: '%s', cReturn: '%s' }, " ),
-                                        EscapeString( menuText, CTX_JS_STR ),
-                                        EscapeString( url, CTX_JS_STR ) );
+                if( m_project )
+                    href = ResolveUriByEnvVars( url, m_project );
+
+                if( url.StartsWith( "file:" ) )
+                    href = NormalizeFileUri( href );
+
+                wxString menuText = wxString::Format( _( "Open %s" ), href );
+
+                js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
+                                        EscapeString( href, CTX_JS_STR ),
+                                        EscapeString( href, CTX_JS_STR ) );
             }
         }
 
-        js += wxT( "]; " );
-
-        js += wxT( "var cChoice = app.popUpMenuEx.apply\\( app, aParams \\); " );
-        js += wxT( "if\\( cChoice != null && cChoice.substring\\( 0, 1 \\) == '#' \\)"
-                   "    this.pageNum = parseInt\\( cChoice.slice\\( 1 \\) \\); " );
-        js += wxT( "else if\\( cChoice != null && cChoice.substring\\( 0, 4 \\) == 'http' \\)"
-                   "    app.launchURL\\( cChoice \\);" );
+        js += wxT( "]);" );
 
         startPdfObject( menuHandle );
 
         fprintf( m_outputFile,
-                 "<< /Type /Annot\n"
-                 "   /Subtype /Link\n"
-                 "   /Rect [%g %g %g %g]\n"
-                 "   /Border [16 16 0]\n",
+                 "<<\n"
+                 "/Type /Annot\n"
+                 "/Subtype /Link\n"
+                 "/Rect [%g %g %g %g]\n"
+                 "/Border [16 16 0]\n",
                  box.GetLeft(), box.GetBottom(), box.GetRight(), box.GetTop() );
 
         fprintf( m_outputFile,
-                 "   /A << /Type /Action /S /JavaScript /JS (%s) >>\n"
+                 "/A << /Type /Action /S /JavaScript /JS %s >>\n"
                  ">>\n",
-                 js.ToStdString().c_str() );
+                 encodeStringForPlotter( js ).c_str() );
+
+        closePdfObject();
+    }
+
+    {
+        startPdfObject( m_jsNamesHandle );
+
+        wxString js = R"JS(
+function ShM(aEntries) {
+    var aParams = [];
+    for (var i in aEntries) {
+        aParams.push({
+            cName: aEntries[i][0],
+            cReturn: aEntries[i][1]
+        })
+    }
+
+    var cChoice = app.popUpMenuEx.apply(app, aParams);
+    if (cChoice != null && cChoice.substring(0, 1) == '#') this.pageNum = parseInt(cChoice.slice(1));
+    else if (cChoice != null && cChoice.substring(0, 4) == 'http') app.launchURL(cChoice);
+    else if (cChoice != null && cChoice.substring(0, 4) == 'file') app.openDoc(cChoice.substring(7));
+}
+)JS";
+
+        fprintf( m_outputFile,
+                 "<< /JavaScript\n"
+                 " << /Names\n"
+                 "    [ (JSInit) << /Type /Action /S /JavaScript /JS %s >> ]\n"
+                 " >>\n"
+                 ">>\n",
+                 encodeStringForPlotter( js ).c_str() );
 
         closePdfObject();
     }
@@ -1328,10 +1524,12 @@ bool PDF_PLOTTER::EndPlot()
                  "/Version /1.5\n"
                  "/PageMode /UseOutlines\n"
                  "/Outlines %d 0 R\n"
+                 "/Names %d 0 R\n"
                  "/PageLayout /SinglePage\n"
                  ">>\n",
                  m_pageTreeHandle,
-                 outlineHandle );
+                 outlineHandle,
+                 m_jsNamesHandle );
     }
     else
     {
@@ -1378,19 +1576,20 @@ bool PDF_PLOTTER::EndPlot()
 }
 
 
-void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
-                        const COLOR4D&              aColor,
-                        const wxString&             aText,
-                        const EDA_ANGLE&            aOrient,
-                        const VECTOR2I&             aSize,
-                        enum GR_TEXT_H_ALIGN_T      aH_justify,
-                        enum GR_TEXT_V_ALIGN_T      aV_justify,
-                        int                         aWidth,
-                        bool                        aItalic,
-                        bool                        aBold,
-                        bool                        aMultilineAllowed,
-                        KIFONT::FONT*               aFont,
-                        void*                       aData )
+void PDF_PLOTTER::Text( const VECTOR2I&        aPos,
+                        const COLOR4D&         aColor,
+                        const wxString&        aText,
+                        const EDA_ANGLE&       aOrient,
+                        const VECTOR2I&        aSize,
+                        enum GR_TEXT_H_ALIGN_T aH_justify,
+                        enum GR_TEXT_V_ALIGN_T aV_justify,
+                        int                    aWidth,
+                        bool                   aItalic,
+                        bool                   aBold,
+                        bool                   aMultilineAllowed,
+                        KIFONT::FONT*          aFont,
+                        const KIFONT::METRICS& aFontMetrics,
+                        void*                  aData )
 {
     // PDF files do not like 0 sized texts which create broken files.
     if( aSize.x == 0 || aSize.y == 0 )
@@ -1409,10 +1608,11 @@ void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
     double wideningFactor, heightFactor;
 
     VECTOR2I t_size( std::abs( aSize.x ), std::abs( aSize.y ) );
+    bool     textMirrored = aSize.x < 0;
 
-    computeTextParameters( aPos, aText, aOrient, t_size, m_plotMirror, aH_justify,
-                           aV_justify, aWidth, aItalic, aBold, &wideningFactor, &ctm_a,
-                           &ctm_b, &ctm_c, &ctm_d, &ctm_e, &ctm_f, &heightFactor );
+    computeTextParameters( aPos, aText, aOrient, t_size, textMirrored, aH_justify, aV_justify,
+                           aWidth, aItalic, aBold, &wideningFactor, &ctm_a, &ctm_b, &ctm_c, &ctm_d,
+                           &ctm_e, &ctm_f, &heightFactor );
 
     SetColor( aColor );
     SetCurrentLineWidth( aWidth, aData );
@@ -1423,7 +1623,12 @@ void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
     if( !aFont )
         aFont = KIFONT::FONT::GetFont();
 
-    VECTOR2I full_box( aFont->StringBoundaryLimits( aText, t_size, aWidth, aBold, aItalic ) );
+    VECTOR2I full_box( aFont->StringBoundaryLimits( aText, t_size, aWidth, aBold, aItalic,
+                                                    aFontMetrics ) );
+
+    if( textMirrored )
+        full_box.x *= -1;
+
     VECTOR2I box_x( full_box.x, 0 );
     VECTOR2I box_y( 0, full_box.y );
 
@@ -1444,13 +1649,18 @@ void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
     {
         wxString word = str_tok.GetNextToken();
 
-        computeTextParameters( pos, word, aOrient, t_size, m_plotMirror, GR_TEXT_H_ALIGN_LEFT,
-                               GR_TEXT_V_ALIGN_BOTTOM, aWidth, aItalic, aBold, &wideningFactor, &ctm_a,
-                               &ctm_b, &ctm_c, &ctm_d, &ctm_e, &ctm_f, &heightFactor );
+        computeTextParameters( pos, word, aOrient, t_size, textMirrored, GR_TEXT_H_ALIGN_LEFT,
+                               GR_TEXT_V_ALIGN_BOTTOM, aWidth, aItalic, aBold, &wideningFactor,
+                               &ctm_a, &ctm_b, &ctm_c, &ctm_d, &ctm_e, &ctm_f, &heightFactor );
 
         // Extract the changed width and rotate by the orientation to get the offset for the
         // next word
-        VECTOR2I bbox( aFont->StringBoundaryLimits( word, t_size, aWidth, aBold, aItalic ).x, 0 );
+        VECTOR2I bbox( aFont->StringBoundaryLimits( word, t_size, aWidth,
+                                                    aBold, aItalic, aFontMetrics ).x, 0 );
+
+        if( textMirrored )
+            bbox.x *= -1;
+
         RotatePoint( bbox, aOrient );
         pos += bbox;
 
@@ -1462,7 +1672,7 @@ void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
            coordinate system will be used for the overlining. Also the %f
            for the trig part of the matrix to avoid %g going in exponential
            format (which is not supported) */
-        fprintf( m_workFile, "q %f %f %f %f %g %g cm BT %s %g Tf %d Tr %g Tz ",
+        fprintf( m_workFile, "q %f %f %f %f %f %f cm BT %s %g Tf %d Tr %g Tz ",
                  ctm_a, ctm_b, ctm_c, ctm_d, ctm_e, ctm_f,
                  fontname, heightFactor, render_mode, wideningFactor * 100 );
 
@@ -1474,15 +1684,17 @@ void PDF_PLOTTER::Text( const VECTOR2I&             aPos,
 
     // Plot the stroked text (if requested)
     PLOTTER::Text( aPos, aColor, aText, aOrient, aSize, aH_justify, aV_justify, aWidth, aItalic,
-                   aBold, aMultilineAllowed, aFont );
+                   aBold, aMultilineAllowed, aFont, aFontMetrics );
 }
 
 
-void PDF_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
-                    const wxString& aText,
-                    const TEXT_ATTRIBUTES& aAttributes,
-                    KIFONT::FONT* aFont,
-                    void* aData )
+void PDF_PLOTTER::PlotText( const VECTOR2I&        aPos,
+                            const COLOR4D&         aColor,
+                            const wxString&        aText,
+                            const TEXT_ATTRIBUTES& aAttributes,
+                            KIFONT::FONT*          aFont,
+                            const KIFONT::METRICS& aFontMetrics,
+                            void*                  aData )
 {
     VECTOR2I size = aAttributes.m_Size;
 
@@ -1493,12 +1705,9 @@ void PDF_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
     if( aAttributes.m_Mirrored )
         size.x = -size.x;
 
-    PDF_PLOTTER::Text( aPos, aColor, aText, aAttributes.m_Angle, size,
-                       aAttributes.m_Halign, aAttributes.m_Valign,
-                       aAttributes.m_StrokeWidth,
-                       aAttributes.m_Italic, aAttributes.m_Bold,
-                       aAttributes.m_Multiline,
-                       aFont, aData );
+    PDF_PLOTTER::Text( aPos, aColor, aText, aAttributes.m_Angle, size, aAttributes.m_Halign,
+                       aAttributes.m_Valign, aAttributes.m_StrokeWidth, aAttributes.m_Italic,
+                       aAttributes.m_Bold, aAttributes.m_Multiline, aFont, aFontMetrics, aData );
 }
 
 
