@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,7 +86,7 @@
  * The center of ellipse is automatically calculated.
  */
 
-#include <base64.h>
+#include <core/base64.h>
 #include <eda_shape.h>
 #include <string_utils.h>
 #include <font/font.h>
@@ -152,7 +152,9 @@ static wxString XmlEsc( const wxString& aStr, bool isAttribute = false )
                 }
             }
             else
+            {
                 escaped.append(c);
+            }
         }
     }
 
@@ -160,17 +162,17 @@ static wxString XmlEsc( const wxString& aStr, bool isAttribute = false )
 }
 
 
-SVG_PLOTTER::SVG_PLOTTER()
+SVG_PLOTTER::SVG_PLOTTER( const PROJECT* aProject ) :
+    PSLIKE_PLOTTER( aProject )
 {
     m_graphics_changed = true;
     SetTextMode( PLOT_TEXT_MODE::STROKE );
     m_fillMode        = FILL_T::NO_FILL; // or FILLED_SHAPE or FILLED_WITH_BG_BODYCOLOR
-    m_pen_rgb_color   = 0;          // current color value (black)
-    m_brush_rgb_color = 0;          // current color value (black)
+    m_pen_rgb_color   = 0;               // current color value (black)
+    m_brush_rgb_color = 0;               // current color value with alpha(black)
     m_brush_alpha     = 1.0;
-    m_dashed          = PLOT_DASH_TYPE::SOLID;
-    m_useInch         = false;      // millimeters are always the svg unit
-    m_precision       = 4;          // default: 4 digits in mantissa.
+    m_dashed          = LINE_STYLE::SOLID;
+    m_precision       = 4;               // default: 4 digits in mantissa.
 }
 
 
@@ -204,15 +206,6 @@ void SVG_PLOTTER::SetSvgCoordinatesFormat( unsigned aPrecision )
 }
 
 
-void SVG_PLOTTER::SetColor( const COLOR4D& color )
-{
-    PSLIKE_PLOTTER::SetColor( color );
-
-    if( m_graphics_changed )
-        setSVGPlotStyle( GetCurrentLineWidth() );
-}
-
-
 void SVG_PLOTTER::setFillMode( FILL_T fill )
 {
     if( m_fillMode != fill )
@@ -228,68 +221,77 @@ void SVG_PLOTTER::setSVGPlotStyle( int aLineWidth, bool aIsGroup, const std::str
     if( aIsGroup )
         fputs( "</g>\n<g ", m_outputFile );
 
-    // output the background fill color
-    fprintf( m_outputFile, "style=\"fill:#%6.6lX; ", m_brush_rgb_color );
+    fputs( "style=\"", m_outputFile );
 
-    switch( m_fillMode )
+    if( m_fillMode == FILL_T::NO_FILL )
     {
-    case FILL_T::NO_FILL:
-        fputs( "fill-opacity:0.0; ", m_outputFile );
-        break;
+        fputs( "fill:none; ", m_outputFile );
+    }
+    else
+    {
+        // output the background fill color
+        fprintf( m_outputFile, "fill:#%6.6lX; ", m_brush_rgb_color );
 
-    case FILL_T::FILLED_SHAPE:
-    case FILL_T::FILLED_WITH_BG_BODYCOLOR:
-    case FILL_T::FILLED_WITH_COLOR:
-        fprintf( m_outputFile, "fill-opacity:%.*f; ", m_precision, m_brush_alpha );
-        break;
+        switch( m_fillMode )
+        {
+        case FILL_T::FILLED_SHAPE:
+        case FILL_T::FILLED_WITH_BG_BODYCOLOR:
+        case FILL_T::FILLED_WITH_COLOR:
+            fprintf( m_outputFile, "fill-opacity:%.*f; ", m_precision, m_brush_alpha );
+            break;
+        default: break;
+        }
     }
 
     double pen_w = userToDeviceSize( aLineWidth );
 
-    if( pen_w < 0.0 )   // Ensure pen width validity
-        pen_w = 0.0;
-
-    // Fix a strange issue found in Inkscape: aWidth < 100 nm create issues on degrouping objects
-    // So we use only 4 digits in mantissa for stroke-width.
-    // TODO: perhaps used only 3 or 4 digits in mantissa for all values in mm, because some
-    // issues were previously reported reported when using nm as integer units
-
-    fprintf( m_outputFile, "\nstroke:#%6.6lX; stroke-width:%.*f; stroke-opacity:1; \n",
-             m_pen_rgb_color, m_precision, pen_w  );
-    fputs( "stroke-linecap:round; stroke-linejoin:round;", m_outputFile );
-
-    //set any extra attributes for non-solid lines
-    switch( m_dashed )
+    if( pen_w <= 0 )
     {
-    case PLOT_DASH_TYPE::DASH:
-        fprintf( m_outputFile, "stroke-dasharray:%.*f,%.*f;",
-                 m_precision, GetDashMarkLenIU( aLineWidth ),
-                 m_precision, GetDashGapLenIU( aLineWidth ) );
-        break;
+        fputs( "stroke:none;", m_outputFile );
+    }
+    else
+    {
+        // Fix a strange issue found in Inkscape: aWidth < 100 nm create issues on degrouping objects
+        // So we use only 4 digits in mantissa for stroke-width.
+        // TODO: perhaps used only 3 or 4 digits in mantissa for all values in mm, because some
+        // issues were previously reported reported when using nm as integer units
 
-    case PLOT_DASH_TYPE::DOT:
-        fprintf( m_outputFile, "stroke-dasharray:%f,%f;",
-                 GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ) );
-        break;
+        fprintf( m_outputFile, "\nstroke:#%6.6lX; stroke-width:%.*f; stroke-opacity:1; \n",
+                 m_pen_rgb_color, m_precision, pen_w );
+        fputs( "stroke-linecap:round; stroke-linejoin:round;", m_outputFile );
 
-    case PLOT_DASH_TYPE::DASHDOT:
-        fprintf( m_outputFile, "stroke-dasharray:%f,%f,%f,%f;",
-                 GetDashMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ),
-                 GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ) );
-        break;
+        //set any extra attributes for non-solid lines
+        switch( m_dashed )
+        {
+        case LINE_STYLE::DASH:
+            fprintf( m_outputFile, "stroke-dasharray:%.*f,%.*f;", m_precision,
+                     GetDashMarkLenIU( aLineWidth ), m_precision, GetDashGapLenIU( aLineWidth ) );
+            break;
 
-    case PLOT_DASH_TYPE::DASHDOTDOT:
-        fprintf( m_outputFile, "stroke-dasharray:%f,%f,%f,%f,%f,%f;",
-                 GetDashMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ),
-                 GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ),
-                 GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ) );
-        break;
+        case LINE_STYLE::DOT:
+            fprintf( m_outputFile, "stroke-dasharray:%f,%f;", GetDotMarkLenIU( aLineWidth ),
+                     GetDashGapLenIU( aLineWidth ) );
+            break;
 
-    case PLOT_DASH_TYPE::DEFAULT:
-    case PLOT_DASH_TYPE::SOLID:
-    default:
-        //do nothing
-        break;
+        case LINE_STYLE::DASHDOT:
+            fprintf( m_outputFile, "stroke-dasharray:%f,%f,%f,%f;", GetDashMarkLenIU( aLineWidth ),
+                     GetDashGapLenIU( aLineWidth ), GetDotMarkLenIU( aLineWidth ),
+                     GetDashGapLenIU( aLineWidth ) );
+            break;
+
+        case LINE_STYLE::DASHDOTDOT:
+            fprintf( m_outputFile, "stroke-dasharray:%f,%f,%f,%f,%f,%f;",
+                     GetDashMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ),
+                     GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ),
+                     GetDotMarkLenIU( aLineWidth ), GetDashGapLenIU( aLineWidth ) );
+            break;
+
+        case LINE_STYLE::DEFAULT:
+        case LINE_STYLE::SOLID:
+        default:
+            //do nothing
+            break;
+        }
     }
 
     if( aExtraStyle.length() )
@@ -323,30 +325,19 @@ void SVG_PLOTTER::SetCurrentLineWidth( int aWidth, void* aData )
         m_graphics_changed  = true;
         m_currentPenWidth   = aWidth;
     }
-
-    if( m_graphics_changed )
-        setSVGPlotStyle( aWidth );
 }
 
 
 void SVG_PLOTTER::StartBlock( void* aData )
 {
-    std::string* idstr = reinterpret_cast<std::string*>( aData );
-
-    fputs( "<g ", m_outputFile );
-
-    if( idstr )
-        fprintf( m_outputFile, "id=\"%s\"", idstr->c_str() );
-
-    fprintf( m_outputFile, ">\n" );
+    // We can't use <g></g> for blocks because we're already using it for graphics context, and
+    // our graphics context handling is lazy (ie: it leaves the last group open until the context
+    // changes).
 }
 
 
 void SVG_PLOTTER::EndBlock( void* aData )
 {
-    fprintf( m_outputFile, "</g>\n" );
-
-    m_graphics_changed = true;
 }
 
 
@@ -369,16 +360,13 @@ void SVG_PLOTTER::emitSetRGBColor( double r, double g, double b, double a )
 }
 
 
-void SVG_PLOTTER::SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle )
+void SVG_PLOTTER::SetDash( int aLineWidth, LINE_STYLE aLineStyle )
 {
     if( m_dashed != aLineStyle )
     {
         m_graphics_changed = true;
         m_dashed = aLineStyle;
     }
-
-    if( m_graphics_changed )
-        setSVGPlotStyle( aLineWidth );
 }
 
 
@@ -400,6 +388,9 @@ void SVG_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill, int
     setFillMode( fill );
     SetCurrentLineWidth( width );
 
+    if( m_graphics_changed )
+        setSVGPlotStyle( GetCurrentLineWidth() );
+
     // Rectangles having a 0 size value for height or width are just not drawn on Inkscape,
     // so use a line when happens.
     if( rect_dev.GetSize().x == 0.0 || rect_dev.GetSize().y == 0.0 )    // Draw a line
@@ -428,6 +419,9 @@ void SVG_PLOTTER::Circle( const VECTOR2I& pos, int diametre, FILL_T fill, int wi
     setFillMode( fill );
     SetCurrentLineWidth( width );
 
+    if( m_graphics_changed )
+        setSVGPlotStyle( GetCurrentLineWidth() );
+
     // If diameter is less than width, switch to filled mode
     if( fill == FILL_T::NO_FILL && diametre < width )
     {
@@ -444,7 +438,7 @@ void SVG_PLOTTER::Circle( const VECTOR2I& pos, int diametre, FILL_T fill, int wi
 
 
 void SVG_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
-                       const EDA_ANGLE& aEndAngle, double aRadius, FILL_T aFill, int aWidth )
+                       const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill, int aWidth )
 {
     /* Draws an arc of a circle, centered on (xc,yc), with starting point (x1, y1) and ending
      * at (x2, y2). The current pen is used for the outline and the current brush for filling
@@ -459,10 +453,10 @@ void SVG_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
         return;
     }
 
-    EDA_ANGLE startAngle( aStartAngle );
-    EDA_ANGLE endAngle( aEndAngle );
+    EDA_ANGLE startAngle = -aStartAngle;
+    EDA_ANGLE endAngle = startAngle - aAngle;
 
-    if( startAngle > endAngle )
+    if( endAngle < startAngle )
         std::swap( startAngle, endAngle );
 
     // Calculate start point.
@@ -525,6 +519,9 @@ void SVG_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
         setFillMode( aFill );
         SetCurrentLineWidth( 0 );
 
+        if( m_graphics_changed )
+            setSVGPlotStyle( GetCurrentLineWidth() );
+
         fprintf( m_outputFile, "<path d=\"M%.*f %.*f A%.*f %.*f 0.0 %d %d %.*f %.*f L %.*f %.*f Z\" />\n",
                  m_precision, start.x, m_precision, start.y,
                  m_precision, radius_device, m_precision, radius_device,
@@ -535,6 +532,10 @@ void SVG_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
 
     setFillMode( FILL_T::NO_FILL );
     SetCurrentLineWidth( aWidth );
+
+    if( m_graphics_changed )
+        setSVGPlotStyle( GetCurrentLineWidth() );
+
     fprintf( m_outputFile, "<path d=\"M%.*f %.*f A%.*f %.*f 0.0 %d %d %.*f %.*f\" />\n",
              m_precision, start.x, m_precision, start.y,
              m_precision, radius_device, m_precision, radius_device,
@@ -551,6 +552,9 @@ void SVG_PLOTTER::BezierCurve( const VECTOR2I& aStart, const VECTOR2I& aControl1
     setFillMode( FILL_T::NO_FILL );
     SetCurrentLineWidth( aLineThickness );
 
+    if( m_graphics_changed )
+        setSVGPlotStyle( GetCurrentLineWidth() );
+
     VECTOR2D start  = userToDeviceCoordinates( aStart );
     VECTOR2D ctrl1  = userToDeviceCoordinates( aControl1 );
     VECTOR2D ctrl2  = userToDeviceCoordinates( aControl2 );
@@ -634,7 +638,9 @@ void SVG_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
         wxMemoryOutputStream img_stream;
 
         if( m_colorMode )
+        {
             aImage.SaveFile( img_stream, wxBITMAP_TYPE_PNG );
+        }
         else    // Plot in B&W
         {
             wxImage image = aImage.ConvertToGreyscale();
@@ -687,10 +693,10 @@ void SVG_PLOTTER::PenTo( const VECTOR2I& pos, char plume )
         // Ensure we do not use a fill mode when moving the pen,
         // in SVG mode (i;e. we are plotting only basic lines, not a filled area
         if( m_fillMode != FILL_T::NO_FILL )
-        {
             setFillMode( FILL_T::NO_FILL );
+
+        if( m_graphics_changed )
             setSVGPlotStyle( GetCurrentLineWidth() );
-        }
 
         fprintf( m_outputFile, "<path d=\"M%.*f %.*f\n",
                  m_precision, pos_dev.x,
@@ -698,6 +704,9 @@ void SVG_PLOTTER::PenTo( const VECTOR2I& pos, char plume )
     }
     else if( m_penState != plume || pos != m_penLastpos )
     {
+        if( m_graphics_changed )
+            setSVGPlotStyle( GetCurrentLineWidth() );
+
         VECTOR2D pos_dev = userToDeviceCoordinates( pos );
 
         fprintf( m_outputFile, "L%.*f %.*f\n",
@@ -778,24 +787,28 @@ bool SVG_PLOTTER::EndPlot()
 }
 
 
-void SVG_PLOTTER::Text( const VECTOR2I&             aPos,
-                        const COLOR4D&              aColor,
-                        const wxString&             aText,
-                        const EDA_ANGLE&            aOrient,
-                        const VECTOR2I&             aSize,
-                        enum GR_TEXT_H_ALIGN_T      aH_justify,
-                        enum GR_TEXT_V_ALIGN_T      aV_justify,
-                        int                         aWidth,
-                        bool                        aItalic,
-                        bool                        aBold,
-                        bool                        aMultilineAllowed,
-                        KIFONT::FONT*               aFont,
-                        void*                       aData )
+void SVG_PLOTTER::Text( const VECTOR2I&        aPos,
+                        const COLOR4D&         aColor,
+                        const wxString&        aText,
+                        const EDA_ANGLE&       aOrient,
+                        const VECTOR2I&        aSize,
+                        enum GR_TEXT_H_ALIGN_T aH_justify,
+                        enum GR_TEXT_V_ALIGN_T aV_justify,
+                        int                    aWidth,
+                        bool                   aItalic,
+                        bool                   aBold,
+                        bool                   aMultilineAllowed,
+                        KIFONT::FONT*          aFont,
+                        const KIFONT::METRICS& aFontMetrics,
+                        void*                  aData )
 {
     setFillMode( FILL_T::NO_FILL );
     SetColor( aColor );
     SetCurrentLineWidth( aWidth );
 
+    if( m_graphics_changed )
+        setSVGPlotStyle( GetCurrentLineWidth() );
+
     VECTOR2I    text_pos = aPos;
     const char* hjust = "start";
 
@@ -817,7 +830,7 @@ void SVG_PLOTTER::Text( const VECTOR2I&             aPos,
 
     // aSize.x or aSize.y is < 0 for mirrored texts.
     // The actual text size value is the absolute value
-    text_size.x = std::abs( GraphicTextWidth( aText, aFont, aSize, aWidth, aBold, aItalic ) );
+    text_size.x = std::abs( GRTextWidth( aText, aFont, aSize, aWidth, aBold, aItalic, aFontMetrics ) );
     text_size.y = std::abs( aSize.x * 4/3 ); // Hershey font height to em size conversion
     VECTOR2D anchor_pos_dev = userToDeviceCoordinates( aPos );
     VECTOR2D text_pos_dev = userToDeviceCoordinates( text_pos );
@@ -839,7 +852,7 @@ void SVG_PLOTTER::Text( const VECTOR2I&             aPos,
 
     fprintf( m_outputFile,
              "textLength=\"%.*f\" font-size=\"%.*f\" lengthAdjust=\"spacingAndGlyphs\"\n"
-             "text-anchor=\"%s\" opacity=\"0\">%s</text>\n",
+             "text-anchor=\"%s\" opacity=\"0\" stroke-opacity=\"0\">%s</text>\n",
              m_precision, sz_dev.x, m_precision, sz_dev.y, hjust, TO_UTF8( XmlEsc( aText ) ) );
 
     if( !aOrient.IsZero() )
@@ -849,27 +862,26 @@ void SVG_PLOTTER::Text( const VECTOR2I&             aPos,
              TO_UTF8( XmlEsc( aText ) ) );
 
     PLOTTER::Text( aPos, aColor, aText, aOrient, aSize, aH_justify, aV_justify, aWidth, aItalic,
-                   aBold, aMultilineAllowed, aFont );
+                   aBold, aMultilineAllowed, aFont, aFontMetrics );
 
     fputs( "</g>", m_outputFile );
 }
 
 
-void SVG_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
-                    const wxString& aText,
-                    const TEXT_ATTRIBUTES& aAttributes,
-                    KIFONT::FONT* aFont,
-                    void* aData )
+void SVG_PLOTTER::PlotText( const VECTOR2I&        aPos,
+                            const COLOR4D&         aColor,
+                            const wxString&        aText,
+                            const TEXT_ATTRIBUTES& aAttributes,
+                            KIFONT::FONT*          aFont,
+                            const KIFONT::METRICS& aFontMetrics,
+                            void*                  aData )
 {
     VECTOR2I size = aAttributes.m_Size;
 
     if( aAttributes.m_Mirrored )
         size.x = -size.x;
 
-    SVG_PLOTTER::Text( aPos, aColor, aText, aAttributes.m_Angle, size,
-                       aAttributes.m_Halign, aAttributes.m_Valign,
-                       aAttributes.m_StrokeWidth,
-                       aAttributes.m_Italic, aAttributes.m_Bold,
-                       aAttributes.m_Multiline,
-                       aFont, aData );
+    SVG_PLOTTER::Text( aPos, aColor, aText, aAttributes.m_Angle, size, aAttributes.m_Halign,
+                       aAttributes.m_Valign, aAttributes.m_StrokeWidth, aAttributes.m_Italic,
+                       aAttributes.m_Bold, aAttributes.m_Multiline, aFont, aFontMetrics, aData );
 }
