@@ -19,7 +19,6 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <config_params.h>
 #include <project.h>
 #include <project/net_settings.h>
 #include <settings/json_settings_internals.h>
@@ -42,7 +41,8 @@ PROJECT_FILE::PROJECT_FILE( const wxString& aFullPath ) :
         m_BoardSettings(),
         m_sheets(),
         m_boards(),
-        m_project( nullptr )
+        m_project( nullptr ),
+        m_wasMigrated( false )
 {
     // Keep old files around
     m_deleteLegacyAfterMigration = false;
@@ -51,39 +51,50 @@ PROJECT_FILE::PROJECT_FILE( const wxString& aFullPath ) :
 
     m_params.emplace_back( new PARAM_LIST<FILE_INFO_PAIR>( "boards", &m_boards, {} ) );
 
-    m_params.emplace_back( new PARAM_WXSTRING_MAP( "text_variables", &m_TextVars, {} ) );
+    m_params.emplace_back( new PARAM_WXSTRING_MAP( "text_variables",
+            &m_TextVars, {}, false, true /* array behavior, even though stored as a map */ ) );
 
-    m_params.emplace_back(
-            new PARAM_LIST<wxString>( "libraries.pinned_symbol_libs", &m_PinnedSymbolLibs, {} ) );
+    m_params.emplace_back( new PARAM_LIST<wxString>( "libraries.pinned_symbol_libs",
+            &m_PinnedSymbolLibs, {} ) );
 
-    m_params.emplace_back( new PARAM_LIST<wxString>(
-            "libraries.pinned_footprint_libs", &m_PinnedFootprintLibs, {} ) );
+    m_params.emplace_back( new PARAM_LIST<wxString>( "libraries.pinned_footprint_libs",
+            &m_PinnedFootprintLibs, {} ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH_LIST( "cvpcb.equivalence_files", &m_EquivalenceFiles, {} ) );
+    m_params.emplace_back( new PARAM_PATH_LIST( "cvpcb.equivalence_files",
+            &m_EquivalenceFiles, {} ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.page_layout_descr_file", &m_BoardDrawingSheetFile, "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.page_layout_descr_file",
+            &m_BoardDrawingSheetFile, "" ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.last_paths.netlist", &m_PcbLastPath[LAST_PATH_NETLIST], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.netlist",
+            &m_PcbLastPath[LAST_PATH_NETLIST], "" ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.last_paths.step", &m_PcbLastPath[LAST_PATH_STEP], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.step",
+            &m_PcbLastPath[LAST_PATH_STEP], "" ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.last_paths.idf", &m_PcbLastPath[LAST_PATH_IDF], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.idf",
+            &m_PcbLastPath[LAST_PATH_IDF], "" ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.last_paths.vrml", &m_PcbLastPath[LAST_PATH_VRML], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.vrml",
+            &m_PcbLastPath[LAST_PATH_VRML], "" ) );
 
-    m_params.emplace_back( new PARAM_PATH(
-            "pcbnew.last_paths.specctra_dsn", &m_PcbLastPath[LAST_PATH_SPECCTRADSN], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.specctra_dsn",
+            &m_PcbLastPath[LAST_PATH_SPECCTRADSN], "" ) );
 
-    m_params.emplace_back(
-            new PARAM_PATH( "pcbnew.last_paths.gencad", &m_PcbLastPath[LAST_PATH_GENCAD], "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.gencad",
+            &m_PcbLastPath[LAST_PATH_GENCAD], "" ) );
 
-    m_params.emplace_back( new PARAM<wxString>( "schematic.legacy_lib_dir", &m_LegacyLibDir, "" ) );
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.pos_files",
+            &m_PcbLastPath[LAST_PATH_POS_FILES], "" ) );
+
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.svg",
+            &m_PcbLastPath[LAST_PATH_SVG], "" ) );
+
+    m_params.emplace_back( new PARAM_PATH( "pcbnew.last_paths.plot",
+            &m_PcbLastPath[LAST_PATH_PLOT], "" ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "schematic.legacy_lib_dir",
+            &m_LegacyLibDir, "" ) );
 
     m_params.emplace_back( new PARAM_LAMBDA<nlohmann::json>( "schematic.legacy_lib_list",
             [&]() -> nlohmann::json
@@ -113,6 +124,21 @@ PROJECT_FILE::PROJECT_FILE( const wxString& aFullPath ) :
     m_params.emplace_back( new PARAM_VIEWPORT( "board.viewports", &m_Viewports ) );
 
     m_params.emplace_back( new PARAM_VIEWPORT3D( "board.3dviewports", &m_Viewports3D ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "board.ipc2581.internal_id",
+            &m_IP2581Bom.id, wxEmptyString ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "board.ipc2581.mpn",
+            &m_IP2581Bom.MPN, wxEmptyString ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "board.ipc2581.mfg",
+            &m_IP2581Bom.mfg, wxEmptyString ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "board.ipc2581.distpn",
+            &m_IP2581Bom.distPN, wxEmptyString ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "board.ipc2581.dist",
+            &m_IP2581Bom.dist, wxEmptyString ) );
 }
 
 
@@ -561,17 +587,61 @@ bool PROJECT_FILE::SaveToFile( const wxString& aDirectory, bool aForce )
 {
     wxASSERT( m_project );
 
-    Set( "meta.filename", m_project->GetProjectName() + "." + ProjectFileExtension );
+    Set( "meta.filename", m_project->GetProjectName() + "." + FILEEXT::ProjectFileExtension );
 
-    return JSON_SETTINGS::SaveToFile( aDirectory, aForce );
+    // Even if parameters were not modified, we should resave after migration
+    bool force = aForce || m_wasMigrated;
+
+    // If we're actually going ahead and doing the save, the flag that keeps code from doing the save
+    // should be cleared at this point
+    m_wasMigrated = false;
+
+    return JSON_SETTINGS::SaveToFile( aDirectory, force );
 }
 
 
 bool PROJECT_FILE::SaveAs( const wxString& aDirectory, const wxString& aFile )
 {
-    Set( "meta.filename", aFile + "." + ProjectFileExtension );
+    wxFileName oldFilename( GetFilename() );
+    wxString   oldProjectName = oldFilename.GetName();
+    wxString   oldProjectPath = oldFilename.GetPath();
+
+    Set( "meta.filename", aFile + "." + FILEEXT::ProjectFileExtension );
     SetFilename( aFile );
 
+    auto updatePath =
+            [&]( wxString& aPath )
+            {
+                if( aPath.StartsWith( oldProjectName + wxS( "." ) ) )
+                    aPath.Replace( oldProjectName, aFile, false );
+                else if( aPath.StartsWith( oldProjectPath + wxS( "/" ) ) )
+                    aPath.Replace( oldProjectPath, aDirectory, false );
+            };
+
+    updatePath( m_BoardDrawingSheetFile );
+
+    for( int ii = LAST_PATH_FIRST; ii < (int) LAST_PATH_SIZE; ++ii )
+        updatePath( m_PcbLastPath[ ii ] );
+
+    auto updatePathByPtr =
+            [&]( const std::string& aPtr )
+            {
+                if( std::optional<wxString> path = Get<wxString>( aPtr ) )
+                {
+                    updatePath( path.value() );
+                    Set( aPtr, path.value() );
+                }
+            };
+
+    updatePathByPtr( "schematic.page_layout_descr_file" );
+    updatePathByPtr( "schematic.plot_directory" );
+    updatePathByPtr( "schematic.ngspice.workbook_filename" );
+    updatePathByPtr( "pcbnew.page_layout_descr_file" );
+
+    // If we're actually going ahead and doing the save, the flag that keeps code from doing the save
+    // should be cleared at this point
+    m_wasMigrated = false;
+
     // While performing Save As, we have already checked that we can write to the directory
     // so don't carry the previous flag
     SetReadOnly( false );
@@ -581,13 +651,13 @@ bool PROJECT_FILE::SaveAs( const wxString& aDirectory, const wxString& aFile )
 
 wxString PROJECT_FILE::getFileExt() const
 {
-    return ProjectFileExtension;
+    return FILEEXT::ProjectFileExtension;
 }
 
 
 wxString PROJECT_FILE::getLegacyFileExt() const
 {
-    return LegacyProjectFileExtension;
+    return FILEEXT::LegacyProjectFileExtension;
 }
 
 
