@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2020-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,9 +19,13 @@
  */
 
 #include <set>
+#include <fstream>
+#include <sstream>
 
+#include <env_vars.h>
 #include <paths.h>
 #include <search_stack.h>
+#include <settings/settings_manager.h>
 #include <settings/common_settings.h>
 #include <settings/json_settings_internals.h>
 #include <settings/parameters.h>
@@ -29,20 +33,14 @@
 #include <trace_helpers.h>
 #include <wx/config.h>
 #include <wx/log.h>
+#include <wx/regex.h>
 
 
 ///! The following environment variables will never be migrated from a previous version
-const std::set<wxString> envVarBlacklist =
-        {
-            wxT( "KICAD6_SYMBOL_DIR" ),
-            wxT( "KICAD6_FOOTPRINT_DIR" ),
-            wxT( "KICAD6_TEMPLATES_DIR" ),
-            wxT( "KICAD6_3DMODEL_DIR" )
-        };
-
+const wxRegEx versionedEnvVarRegex( wxS( "KICAD[0-9]+_[A-Z0-9_]+(_DIR)?" ) );
 
 ///! Update the schema version whenever a migration is required
-const int commonSchemaVersion = 2;
+const int commonSchemaVersion = 3;
 
 COMMON_SETTINGS::COMMON_SETTINGS() :
         JSON_SETTINGS( "kicad_common", SETTINGS_LOC::USER, commonSchemaVersion ),
@@ -53,7 +51,9 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
         m_Graphics(),
         m_Session(),
         m_System(),
-        m_NetclassPanel()
+        m_DoNotShowAgain(),
+        m_NetclassPanel(),
+        m_PackageManager()
 {
     /*
      * Automatic dark mode detection works fine on Mac.
@@ -65,17 +65,6 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     m_Appearance.icon_theme = ICON_THEME::AUTO;
 #endif
 
-    /*
-     * Automatic icon scaling works fine on Mac.  It works mostly fine on MSW, but perhaps not
-     * uniformly enough to exclude the explicit controls there.
-     */
-#if defined( __WXGTK__ ) || defined( __WXMSW__ )
-    m_params.emplace_back( new PARAM<int>( "appearance.icon_scale",
-            &m_Appearance.icon_scale, 0 ) );
-#else
-    m_Appearance.icon_scale = 0.0;
-#endif
-
     /*
    	 * Automatic canvas scaling works fine on all supported platforms, so it's no longer exposed as
      * a configuration option.
@@ -96,12 +85,19 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     /*
      * Font scaling hacks are only needed on GTK under wxWidgets 3.0.
      */
-#if defined( __WXGTK__ ) && !wxCHECK_VERSION( 3, 1, 0 )
-    m_params.emplace_back( new PARAM<bool>( "appearance.apply_icon_scale_to_fonts",
-            &m_Appearance.apply_icon_scale_to_fonts, false ) );
-#else
     m_Appearance.apply_icon_scale_to_fonts = false;
-#endif
+
+    m_params.emplace_back( new PARAM<bool>( "appearance.show_scrollbars",
+            &m_Appearance.show_scrollbars, false ) );
+
+    m_params.emplace_back( new PARAM<double>( "appearance.hicontrast_dimming_factor",
+            &m_Appearance.hicontrast_dimming_factor, 0.8f ) );
+
+    m_params.emplace_back( new PARAM<int>( "appearance.text_editor_zoom",
+            &m_Appearance.text_editor_zoom, 0 ) );
+
+    m_params.emplace_back( new PARAM<int>( "appearance.toolbar_icon_size",
+            &m_Appearance.toolbar_icon_size, 24, 16, 64 ) );
 
     m_params.emplace_back( new PARAM<bool>( "auto_backup.enabled", &m_Backup.enabled, true ) );
 
@@ -120,7 +116,7 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     m_params.emplace_back( new PARAM<int>( "auto_backup.min_interval",
             &m_Backup.min_interval, 300 ) );
 
-    m_params.emplace_back( new PARAM_LAMBDA<nlohmann::json>( "environment.vars",
+    auto envVarsParam = m_params.emplace_back( new PARAM_LAMBDA<nlohmann::json>( "environment.vars",
             [&]() -> nlohmann::json
             {
                 nlohmann::json ret = {};
@@ -135,7 +131,7 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
                     if( var.IsDefault() )
                     {
                         wxLogTrace( traceEnvVars,
-                                    "COMMON_SETTINGS: Env var %s skipping save (default)",
+                                    wxS( "COMMON_SETTINGS: Env var %s skipping save (default)" ),
                                     var.GetKey() );
                         continue;
                     }
@@ -149,26 +145,27 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
                         if( var.GetDefinedInSettings() )
                         {
                             wxLogTrace( traceEnvVars,
-                                        "COMMON_SETTINGS: Env var %s was overridden externally, "
-                                        "saving previously-loaded value %s",
+                                        wxS( "COMMON_SETTINGS: Env var %s was overridden "
+                                             "externally, saving previously-loaded value %s" ),
                                         var.GetKey(), var.GetSettingsValue() );
                             value = var.GetSettingsValue();
                         }
                         else
                         {
                             wxLogTrace( traceEnvVars,
-                                        "COMMON_SETTINGS: Env var %s skipping save (external)",
+                                        wxS( "COMMON_SETTINGS: Env var %s skipping save "
+                                             "(external)" ),
                                         var.GetKey() );
                             continue;
                         }
                     }
 
                     wxLogTrace( traceEnvVars,
-                                "COMMON_SETTINGS: Saving env var %s = %s",
+                                wxS( "COMMON_SETTINGS: Saving env var %s = %s" ),
                                 var.GetKey(), value);
 
                     std::string key( var.GetKey().ToUTF8() );
-                    ret[key] = value;
+                    ret[ std::move( key ) ] = value;
                 }
 
                 return ret;
@@ -187,7 +184,8 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
                     {
                         if( m_Env.vars[key].GetDefinedExternally() )
                         {
-                            wxLogTrace( traceEnvVars, "COMMON_SETTINGS: %s is defined externally",
+                            wxLogTrace( traceEnvVars,
+                                        wxS( "COMMON_SETTINGS: %s is defined externally" ),
                                         key );
                             m_Env.vars[key].SetDefinedInSettings();
                             m_Env.vars[key].SetSettingsValue( val );
@@ -195,14 +193,16 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
                         }
                         else
                         {
-                            wxLogTrace( traceEnvVars, "COMMON_SETTINGS: Updating %s: %s -> %s",
+                            wxLogTrace( traceEnvVars,
+                                        wxS( "COMMON_SETTINGS: Updating %s: %s -> %s"),
                                         key, m_Env.vars[key].GetValue(), val );
                             m_Env.vars[key].SetValue( val );
                         }
                     }
                     else
                     {
-                        wxLogTrace( traceEnvVars, "COMMON_SETTINGS: Loaded new var: %s = %s",
+                        wxLogTrace( traceEnvVars,
+                                    wxS( "COMMON_SETTINGS: Loaded new var: %s = %s" ),
                                     key, val );
                         m_Env.vars[key] = ENV_VAR_ITEM( key, val );
                     }
@@ -212,6 +212,10 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
                 }
             },
             {} ) );
+    envVarsParam->SetClearUnknownKeys();
+
+    m_params.emplace_back( new PARAM<bool>( "input.focus_follow_sch_pcb",
+            &m_Input.focus_follow_sch_pcb, false ) );
 
     m_params.emplace_back( new PARAM<bool>( "input.auto_pan", &m_Input.auto_pan, false ) );
 
@@ -230,6 +234,9 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     m_params.emplace_back( new PARAM<bool>( "input.horizontal_pan",
             &m_Input.horizontal_pan, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "input.hotkey_feedback",
+            &m_Input.hotkey_feedback, true ) );
+
     m_params.emplace_back( new PARAM<bool>( "input.zoom_acceleration",
             &m_Input.zoom_acceleration, false ) );
 
@@ -254,6 +261,9 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     m_params.emplace_back( new PARAM<int>( "input.scroll_modifier_pan_v",
             &m_Input.scroll_modifier_pan_v, WXK_SHIFT ) );
 
+    m_params.emplace_back( new PARAM<bool>( "input.reverse_scroll_pan_h",
+            &m_Input.reverse_scroll_pan_h, false ) );
+
     m_params.emplace_back( new PARAM_ENUM<MOUSE_DRAG_ACTION>( "input.mouse_left",
             &m_Input.drag_left, MOUSE_DRAG_ACTION::DRAG_SELECTED, MOUSE_DRAG_ACTION::DRAG_ANY,
             MOUSE_DRAG_ACTION::SELECT ) );
@@ -267,7 +277,7 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
             MOUSE_DRAG_ACTION::NONE ) );
 
     m_params.emplace_back( new PARAM<int>( "graphics.opengl_antialiasing_mode",
-            &m_Graphics.opengl_aa_mode, 0, 0, 2 ) );
+            &m_Graphics.opengl_aa_mode, 1, 0, 2 ) );
 
     m_params.emplace_back( new PARAM<int>( "graphics.cairo_antialiasing_mode",
             &m_Graphics.cairo_aa_mode, 0, 0, 2 ) );
@@ -277,26 +287,26 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
 
 #ifdef __WXMAC__
     m_params.emplace_back( new PARAM<wxString>( "system.text_editor",
-            &m_System.text_editor, "/usr/bin/open -e" ) );
+            &m_System.text_editor, wxS( "/usr/bin/open -e" ) ) );
 #else
     m_params.emplace_back( new PARAM<wxString>( "system.text_editor",
-            &m_System.text_editor, "" ) );
+            &m_System.text_editor, wxS( "" ) ) );
 #endif
 
     m_params.emplace_back( new PARAM<int>( "system.file_history_size",
             &m_System.file_history_size, 9 ) );
 
     m_params.emplace_back( new PARAM<wxString>( "system.language",
-            &m_System.language, "Default" ) );
+            &m_System.language, wxS( "Default" ) ) );
 
     m_params.emplace_back( new PARAM<wxString>( "system.pdf_viewer_name",
-            &m_System.pdf_viewer_name, "" ) );
+            &m_System.pdf_viewer_name, wxS( "" ) ) );
 
     m_params.emplace_back( new PARAM<bool>( "system.use_system_pdf_viewer",
             &m_System.use_system_pdf_viewer, true ) );
 
     m_params.emplace_back( new PARAM<wxString>( "system.working_dir",
-            &m_System.working_dir, "" ) );
+            &m_System.working_dir, wxS( "" ) ) );
 
     m_params.emplace_back( new PARAM<int>( "system.clear_3d_cache_interval",
             &m_System.clear_3d_cache_interval, 30 ) );
@@ -310,17 +320,86 @@ COMMON_SETTINGS::COMMON_SETTINGS() :
     m_params.emplace_back( new PARAM<bool>( "do_not_show_again.scaled_3d_models_warning",
             &m_DoNotShowAgain.scaled_3d_models_warning, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "do_not_show_again.data_collection_prompt",
+            &m_DoNotShowAgain.data_collection_prompt, false ) );
+
+    m_params.emplace_back( new PARAM<bool>( "do_not_show_again.update_check_prompt",
+            &m_DoNotShowAgain.update_check_prompt, false ) );
+
     m_params.emplace_back( new PARAM<bool>( "session.remember_open_files",
             &m_Session.remember_open_files, false ) );
 
+    m_params.emplace_back( new PARAM_LIST<wxString>( "session.pinned_symbol_libs",
+            &m_Session.pinned_symbol_libs, {} ) );
+
+    m_params.emplace_back( new PARAM_LIST<wxString>( "session.pinned_fp_libs",
+            &m_Session.pinned_fp_libs, {} ) );
+
     m_params.emplace_back( new PARAM<int>( "netclass_panel.sash_pos",
             &m_NetclassPanel.sash_pos, 160 ) );
 
     m_params.emplace_back( new PARAM<int>( "package_manager.sash_pos",
             &m_PackageManager.sash_pos, 380 ) );
 
+    m_params.emplace_back( new PARAM_LAMBDA<nlohmann::json>( "git.repositories",
+            [&]() -> nlohmann::json
+            {
+                nlohmann::json ret = {};
+
+                for( const GIT_REPOSITORY& repo : m_Git.repositories )
+                {
+                    nlohmann::json repoJson = {};
+
+                    repoJson["name"] = repo.name;
+                    repoJson["path"] = repo.path;
+                    repoJson["authType"] = repo.authType;
+                    repoJson["username"] = repo.username;
+                    repoJson["ssh_path"] = repo.ssh_path;
+                    repoJson["active"] = repo.active;
+
+                    ret.push_back( repoJson );
+                }
+
+                return ret;
+            },
+            [&]( const nlohmann::json& aJson )
+            {
+                if( !aJson.is_array() )
+                    return;
+
+                m_Git.repositories.clear();
+
+                for( const auto& repoJson : aJson )
+                {
+                    GIT_REPOSITORY repo;
+
+                    repo.name = repoJson["name"].get<wxString>();
+                    repo.path = repoJson["path"].get<wxString>();
+                    repo.authType = repoJson["authType"].get<wxString>();
+                    repo.username = repoJson["username"].get<wxString>();
+                    repo.ssh_path = repoJson["ssh_path"].get<wxString>();
+                    repo.active = repoJson["active"].get<bool>();
+                    repo.checkValid = true;
+
+                    m_Git.repositories.push_back( repo );
+                }
+            },
+            {} ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "git.authorName",
+            &m_Git.authorName, wxS( "" ) ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "git.authorEmail",
+            &m_Git.authorEmail, wxS( "" ) ) );
+
+    m_params.emplace_back( new PARAM<bool>( "git.useDefaultAuthor",
+            &m_Git.useDefaultAuthor, true ) );
+
+
+
     registerMigration( 0, 1, std::bind( &COMMON_SETTINGS::migrateSchema0to1, this ) );
     registerMigration( 1, 2, std::bind( &COMMON_SETTINGS::migrateSchema1to2, this ) );
+    registerMigration( 2, 3, std::bind( &COMMON_SETTINGS::migrateSchema2to3, this ) );
 }
 
 
@@ -343,7 +422,7 @@ bool COMMON_SETTINGS::migrateSchema0to1()
     }
     catch( ... )
     {
-        wxLogTrace( traceSettings, "COMMON_SETTINGS::Migrate 0->1: mousewheel_pan not found" );
+        wxLogTrace( traceSettings, wxT( "COMMON_SETTINGS::Migrate 0->1: mousewheel_pan not found" ) );
     }
 
     if( mwp )
@@ -380,7 +459,7 @@ bool COMMON_SETTINGS::migrateSchema1to2()
     }
     catch( ... )
     {
-        wxLogTrace( traceSettings, "COMMON_SETTINGS::Migrate 1->2: prefer_select_to_drag not found" );
+        wxLogTrace( traceSettings, wxT( "COMMON_SETTINGS::Migrate 1->2: prefer_select_to_drag not found" ) );
     }
 
     if( prefer_selection )
@@ -392,6 +471,52 @@ bool COMMON_SETTINGS::migrateSchema1to2()
 }
 
 
+bool COMMON_SETTINGS::migrateSchema2to3()
+{
+    wxFileName cfgpath;
+    cfgpath.AssignDir( PATHS::GetUserSettingsPath() );
+    cfgpath.AppendDir( wxT( "3d" ) );
+    cfgpath.SetFullName( wxS( "3Dresolver.cfg" ) );
+    cfgpath.MakeAbsolute();
+
+    std::vector<LEGACY_3D_SEARCH_PATH> legacyPaths;
+    readLegacy3DResolverCfg( cfgpath.GetFullPath(), legacyPaths );
+
+    // env variables have a limited allowed character set for names
+    wxRegEx nonValidCharsRegex( wxS( "[^A-Z0-9_]+" ), wxRE_ADVANCED );
+
+    for( const LEGACY_3D_SEARCH_PATH& path : legacyPaths )
+    {
+        wxString key = path.m_Alias;
+        const wxString& val = path.m_Pathvar;
+
+        // The 3d alias config didnt use the same naming restrictions as real env variables
+        // We need to sanitize them
+
+        // upper case only
+        key.MakeUpper();
+        // logically swap - with _
+        key.Replace( wxS( "-" ), wxS( "_" ) );
+
+        // remove any other chars
+        nonValidCharsRegex.Replace( &key, wxEmptyString );
+
+        if( !m_Env.vars.count( key ) )
+        {
+            wxLogTrace( traceEnvVars, wxS( "COMMON_SETTINGS: Loaded new var: %s = %s" ), key, val );
+            m_Env.vars[key] = ENV_VAR_ITEM( key, val );
+        }
+    }
+
+    if( cfgpath.FileExists() )
+    {
+        wxRemoveFile( cfgpath.GetFullPath() );
+    }
+
+    return true;
+}
+
+
 bool COMMON_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
 {
     bool ret = true;
@@ -413,9 +538,10 @@ bool COMMON_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
 
                 while( aCfg->GetNextEntry( key, index ) )
                 {
-                    if( envVarBlacklist.count( key ) )
+                    if( versionedEnvVarRegex.Matches( key ) )
                     {
-                        wxLogTrace( traceSettings, "Migrate Env: %s is blacklisted; skipping.", key );
+                        wxLogTrace( traceSettings,
+                                    wxT( "Migrate Env: %s is blacklisted; skipping." ), key );
                         continue;
                     }
 
@@ -425,7 +551,8 @@ bool COMMON_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
                     {
                         ptr.push_back( key.ToStdString() );
 
-                        wxLogTrace( traceSettings, "Migrate Env: %s=%s", ptr.to_string(), value );
+                        wxLogTrace( traceSettings, wxT( "Migrate Env: %s=%s" ),
+                                    ptr.to_string(), value );
                         ( *m_internals )[ptr] = value.ToUTF8();
 
                         ptr.pop_back();
@@ -454,7 +581,7 @@ bool COMMON_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
     ret &= fromLegacy<bool>( aCfg, "ZoomNoCenter",              "input.center_on_zoom" );
 
     // This was stored inverted in legacy config
-    if( OPT<bool> value = Get<bool>( "input.center_on_zoom" ) )
+    if( std::optional<bool> value = Get<bool>( "input.center_on_zoom" ) )
         Set( "input.center_on_zoom", !( *value ) );
 
     ret &= fromLegacy<int>( aCfg, "OpenGLAntialiasingMode", "graphics.opengl_antialiasing_mode" );
@@ -486,12 +613,13 @@ void COMMON_SETTINGS::InitializeEnvironment()
                 m_Env.vars[aKey].SetValue( envValue );
                 m_Env.vars[aKey].SetDefinedExternally();
                 wxLogTrace( traceEnvVars,
-                            "InitializeEnvironment: Entry %s defined externally as %s", aKey,
+                            wxS( "InitializeEnvironment: Entry %s defined externally as %s" ), aKey,
                             envValue );
             }
             else
             {
-                wxLogTrace( traceEnvVars, "InitializeEnvironment: Setting entry %s to default %s",
+                wxLogTrace( traceEnvVars, wxS( "InitializeEnvironment: Setting entry %s to "
+                                               "default %s" ),
                             aKey, aDefault );
             }
         };
@@ -500,56 +628,218 @@ void COMMON_SETTINGS::InitializeEnvironment()
 
     wxFileName path( basePath );
     path.AppendDir( wxT( "footprints" ) );
-    addVar( wxT( "KICAD6_FOOTPRINT_DIR" ), path.GetFullPath() );
+    addVar( ENV_VAR::GetVersionedEnvVarName( wxS( "FOOTPRINT_DIR" ) ), path.GetFullPath() );
 
     path = basePath;
     path.AppendDir( wxT( "3dmodels" ) );
-    addVar( wxT( "KICAD6_3DMODEL_DIR" ), path.GetFullPath() );
+    addVar( ENV_VAR::GetVersionedEnvVarName( wxS( "3DMODEL_DIR" ) ), path.GetFullPath() );
 
-    // We don't have just one default template path, so use this logic that originally was in
-    // PGM_BASE::InitPgm to determine the best default template path
-    {
-        // Attempt to find the best default template path.
-        SEARCH_STACK bases;
-        SEARCH_STACK templatePaths;
-
-        SystemDirsAppend( &bases );
-
-        for( unsigned i = 0; i < bases.GetCount(); ++i )
-        {
-            wxFileName fn( bases[i], wxEmptyString );
-
-            // Add KiCad template file path to search path list.
-            fn.AppendDir( "template" );
-
-            // Only add path if exists and can be read by the user.
-            if( fn.DirExists() && fn.IsDirReadable() )
-            {
-                wxLogTrace( tracePathsAndFiles, "Checking template path '%s' exists",
-                            fn.GetPath() );
-                templatePaths.AddPaths( fn.GetPath() );
-            }
-        }
-
-        if( templatePaths.IsEmpty() )
-        {
-            path = basePath;
-            path.AppendDir( "template" );
-        }
-        else
-        {
-            // Take the first one.  There may be more but this will likely be the best option.
-            path.AssignDir( templatePaths[0] );
-        }
-
-        addVar( wxT( "KICAD6_TEMPLATE_DIR" ), path.GetFullPath() );
-    }
+    addVar( ENV_VAR::GetVersionedEnvVarName( wxS( "TEMPLATE_DIR" ) ),
+            PATHS::GetStockTemplatesPath() );
 
     addVar( wxT( "KICAD_USER_TEMPLATE_DIR" ), PATHS::GetUserTemplatesPath() );
 
-    addVar( wxT( "KICAD6_3RD_PARTY" ), PATHS::GetDefault3rdPartyPath() );
+    addVar( ENV_VAR::GetVersionedEnvVarName( wxS( "3RD_PARTY" ) ),
+            PATHS::GetDefault3rdPartyPath() );
 
     path = basePath;
     path.AppendDir( wxT( "symbols" ) );
-    addVar( wxT( "KICAD6_SYMBOL_DIR" ), path.GetFullPath() );
+    addVar( ENV_VAR::GetVersionedEnvVarName( wxS( "SYMBOL_DIR" ) ), path.GetFullPath() );
+}
+
+
+bool COMMON_SETTINGS::readLegacy3DResolverCfg( const wxString&                   path,
+                                               std::vector<LEGACY_3D_SEARCH_PATH>& aSearchPaths )
+{
+    wxFileName cfgpath( path );
+
+    // This should be the same as wxWidgets 3.0 wxPATH_NORM_ALL which is deprecated in 3.1.
+    // There are known issues with environment variable expansion so maybe we should be using
+    // our own ExpandEnvVarSubstitutions() here instead.
+    cfgpath.Normalize( FN_NORMALIZE_FLAGS | wxPATH_NORM_ENV_VARS );
+    wxString cfgname = cfgpath.GetFullPath();
+
+    std::ifstream cfgFile;
+    std::string   cfgLine;
+
+    if( !wxFileName::Exists( cfgname ) )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = "no 3D configuration file";
+        ostr << " * " << errmsg.ToUTF8() << " '";
+        ostr << cfgname.ToUTF8() << "'";
+        wxLogTrace( traceSettings, "%s\n", ostr.str().c_str() );
+        return false;
+    }
+
+    cfgFile.open( cfgname.ToUTF8() );
+
+    if( !cfgFile.is_open() )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "Could not open configuration file" );
+        ostr << " * " << errmsg.ToUTF8() << " '" << cfgname.ToUTF8() << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+        return false;
+    }
+
+    int         lineno = 0;
+    LEGACY_3D_SEARCH_PATH al;
+    size_t      idx;
+    int         vnum = 0; // version number
+
+    while( cfgFile.good() )
+    {
+        cfgLine.clear();
+        std::getline( cfgFile, cfgLine );
+        ++lineno;
+
+        if( cfgLine.empty() )
+        {
+            if( cfgFile.eof() )
+                break;
+
+            continue;
+        }
+
+        if( 1 == lineno && cfgLine.compare( 0, 2, "#V" ) == 0 )
+        {
+            // extract the version number and parse accordingly
+            if( cfgLine.size() > 2 )
+            {
+                std::istringstream istr;
+                istr.str( cfgLine.substr( 2 ) );
+                istr >> vnum;
+            }
+
+            continue;
+        }
+
+        idx = 0;
+
+        if( !getLegacy3DHollerith( cfgLine, idx, al.m_Alias ) )
+            continue;
+
+        // Don't add KICADn_3DMODEL_DIR, one of its legacy equivalents, or KIPRJMOD from a
+        // config file.  They're system variables which are defined at runtime.
+        wxString versionedPath = wxString::Format( wxS( "${%s}" ),
+                                       ENV_VAR::GetVersionedEnvVarName( wxS( "3DMODEL_DIR" ) ) );
+
+        if( al.m_Alias == versionedPath || al.m_Alias == wxS( "${KIPRJMOD}" )
+            || al.m_Alias == wxS( "$(KIPRJMOD)" ) || al.m_Alias == wxS( "${KISYS3DMOD}" )
+            || al.m_Alias == wxS( "$(KISYS3DMOD)" ) )
+        {
+            continue;
+        }
+
+        if( !getLegacy3DHollerith( cfgLine, idx, al.m_Pathvar ) )
+            continue;
+
+        if( !getLegacy3DHollerith( cfgLine, idx, al.m_Description ) )
+            continue;
+
+        aSearchPaths.push_back( al );
+    }
+
+    cfgFile.close();
+
+    return true;
+}
+
+
+bool COMMON_SETTINGS::getLegacy3DHollerith( const std::string& aString, size_t& aIndex,
+                                            wxString& aResult )
+{
+    aResult.clear();
+
+    if( aIndex >= aString.size() )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "bad Hollerith string on line" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    size_t i2 = aString.find( '"', aIndex );
+
+    if( std::string::npos == i2 )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "missing opening quote mark in config file" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    ++i2;
+
+    if( i2 >= aString.size() )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "invalid entry (unexpected end of line)" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    std::string tnum;
+
+    while( aString[i2] >= '0' && aString[i2] <= '9' )
+        tnum.append( 1, aString[i2++] );
+
+    if( tnum.empty() || aString[i2++] != ':' )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "bad Hollerith string on line" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    std::istringstream istr;
+    istr.str( tnum );
+    size_t nchars;
+    istr >> nchars;
+
+    if( ( i2 + nchars ) >= aString.size() )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "invalid entry (unexpected end of line)" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    if( nchars > 0 )
+    {
+        aResult = wxString::FromUTF8( aString.substr( i2, nchars ).c_str() );
+        i2 += nchars;
+    }
+
+    if( i2 >= aString.size() || aString[i2] != '"' )
+    {
+        std::ostringstream ostr;
+        ostr << __FILE__ << ": " << __FUNCTION__ << ": " << __LINE__ << "\n";
+        wxString errmsg = wxS( "missing closing quote mark in config file" );
+        ostr << " * " << errmsg.ToUTF8() << "\n'" << aString << "'";
+        wxLogTrace( traceSettings, wxS( "%s\n" ), ostr.str().c_str() );
+
+        return false;
+    }
+
+    aIndex = i2 + 1;
+    return true;
 }
