@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2021, 2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -46,6 +46,7 @@
 #include <settings/json_settings_internals.h>
 #include <settings/settings_manager.h>
 #include <wildcards_and_files_ext.h>
+#include <env_vars.h>
 
 
 SETTINGS_MANAGER::SETTINGS_MANAGER( bool aHeadless ) :
@@ -70,6 +71,15 @@ SETTINGS_MANAGER::SETTINGS_MANAGER( bool aHeadless ) :
     // Create the built-in color settings
     // Here to allow the Python API to access the built-in colors
     registerBuiltinColorSettings();
+
+    wxFileName commonSettings( GetPathForSettingsFile( m_common_settings ),
+                               m_common_settings->GetFullFilename() );
+
+    if( !wxFileExists( commonSettings.GetFullPath() ) )
+    {
+        m_common_settings->Load();
+        Save( m_common_settings );
+    }
 }
 
 SETTINGS_MANAGER::~SETTINGS_MANAGER()
@@ -94,7 +104,8 @@ JSON_SETTINGS* SETTINGS_MANAGER::registerSettings( JSON_SETTINGS* aSettings, boo
 
     ptr->SetManager( this );
 
-    wxLogTrace( traceSettings, wxT( "Registered new settings object <%s>" ), ptr->GetFullFilename() );
+    wxLogTrace( traceSettings, wxT( "Registered new settings object <%s>" ),
+                ptr->GetFullFilename() );
 
     if( aLoadNow )
         ptr->LoadFromFile( GetPathForSettingsFile( ptr.get() ) );
@@ -109,8 +120,17 @@ void SETTINGS_MANAGER::Load()
     // TODO(JE) We should check for dirty settings here and write them if so, because
     // Load() could be called late in the application lifecycle
 
-    for( auto&& settings : m_settings )
-        settings->LoadFromFile( GetPathForSettingsFile( settings.get() ) );
+    std::vector<JSON_SETTINGS*> toLoad;
+
+    // Cache a copy of raw pointers; m_settings may be modified during the load loop
+    std::transform( m_settings.begin(), m_settings.end(), std::back_inserter( toLoad ),
+                    []( std::unique_ptr<JSON_SETTINGS>& aSettings )
+                    {
+                        return aSettings.get();
+                    } );
+
+    for( JSON_SETTINGS* settings : toLoad )
+        settings->LoadFromFile( GetPathForSettingsFile( settings ) );
 }
 
 
@@ -135,6 +155,13 @@ void SETTINGS_MANAGER::Save()
         if( dynamic_cast<COLOR_SETTINGS*>( settings.get() ) )
             continue;
 
+        // Never automatically save project settings, caller should use SaveProject or UnloadProject
+        if( dynamic_cast<PROJECT_FILE*>( settings.get() )
+            || dynamic_cast<PROJECT_LOCAL_SETTINGS*>( settings.get() ) )
+        {
+            continue;
+        }
+
         settings->SaveToFile( GetPathForSettingsFile( settings.get() ) );
     }
 }
@@ -228,7 +255,8 @@ COLOR_SETTINGS* SETTINGS_MANAGER::loadColorSettingsByName( const wxString& aName
 
     if( !fn.IsOk() || !fn.Exists() )
     {
-        wxLogTrace( traceSettings, wxT( "Theme file %s.json not found, falling back to user" ), aName );
+        wxLogTrace( traceSettings, wxT( "Theme file %s.json not found, falling back to user" ),
+                    aName );
         return nullptr;
     }
 
@@ -323,7 +351,7 @@ void SETTINGS_MANAGER::loadAllColorSettings()
 
     wxFileName third_party_path;
     const ENV_VAR_MAP& env = Pgm().GetLocalEnvVariables();
-    auto               it = env.find( wxS( "KICAD7_3RD_PARTY" ) );
+    auto               it = env.find( ENV_VAR::GetVersionedEnvVarName( wxS( "3RD_PARTY" ) ) );
 
     if( it != env.end() && !it->second.GetValue().IsEmpty() )
         third_party_path.SetPath( it->second.GetValue() );
@@ -332,8 +360,14 @@ void SETTINGS_MANAGER::loadAllColorSettings()
 
     third_party_path.AppendDir( wxS( "colors" ) );
 
+    // PCM-managed themes
     wxDir third_party_colors_dir( third_party_path.GetFullPath() );
-    wxString color_settings_path = GetColorSettingsPath();
+
+    // System-installed themes
+    wxDir system_colors_dir( PATHS::GetStockDataPath( false ) + "/colors" );
+
+    // User-created themes
+    wxDir colors_dir( GetColorSettingsPath() );
 
     // Search for and load any other settings
     JSON_DIR_TRAVERSER loader( [&]( const wxFileName& aFilename )
@@ -341,22 +375,21 @@ void SETTINGS_MANAGER::loadAllColorSettings()
                                    registerColorSettings( aFilename.GetName() );
                                } );
 
-    JSON_DIR_TRAVERSER thirdPartyLoader(
+    JSON_DIR_TRAVERSER readOnlyLoader(
             [&]( const wxFileName& aFilename )
             {
                 COLOR_SETTINGS* settings = registerColorSettings( aFilename.GetFullPath(), true );
                 settings->SetReadOnly( true );
             } );
 
-    wxDir colors_dir( color_settings_path );
+    if( system_colors_dir.IsOpened() )
+        system_colors_dir.Traverse( readOnlyLoader );
+
+    if( third_party_colors_dir.IsOpened() )
+       third_party_colors_dir.Traverse( readOnlyLoader );
 
     if( colors_dir.IsOpened() )
-    {
-        if( third_party_colors_dir.IsOpened() )
-           third_party_colors_dir.Traverse( thirdPartyLoader );
-
         colors_dir.Traverse( loader );
-    }
 }
 
 
@@ -414,7 +447,7 @@ wxString SETTINGS_MANAGER::GetPathForSettingsFile( JSON_SETTINGS* aSettings )
     switch( aSettings->GetLocation() )
     {
     case SETTINGS_LOC::USER:
-        return GetUserSettingsPath();
+        return PATHS::GetUserSettingsPath();
 
     case SETTINGS_LOC::PROJECT:
         // TODO: MDI support
@@ -512,13 +545,14 @@ public:
 
 bool SETTINGS_MANAGER::MigrateIfNeeded()
 {
-    wxFileName path( GetUserSettingsPath(), wxS( "" ) );
+    wxFileName path( PATHS::GetUserSettingsPath(), wxS( "" ) );
     wxLogTrace( traceSettings, wxT( "Using settings path %s" ), path.GetFullPath() );
 
     if( m_headless )
     {
         // Special case namely for cli
-        // Ensure the settings directory at least exists to prevent additional loading errors from subdirectories
+        // Ensure the settings directory at least exists to prevent additional loading errors
+        // from subdirectories.
         // TODO review headless (unit tests) vs cli needs, this should be fine for unit tests though
         if( !path.DirExists() )
         {
@@ -589,7 +623,11 @@ bool SETTINGS_MANAGER::MigrateIfNeeded()
             wxT( "KICAD7_SYMBOL_DIR" ),
             wxT( "KICAD7_3DMODEL_DIR" ),
             wxT( "KICAD7_FOOTPRINT_DIR" ),
-            wxT( "KICAD7_TEMPLATE_DIR" ), // Stores the default library table to be copied
+            wxT( "KICAD7_TEMPLATE_DIR" ),
+            wxT( "KICAD8_SYMBOL_DIR" ),
+            wxT( "KICAD8_3DMODEL_DIR" ),
+            wxT( "KICAD8_FOOTPRINT_DIR" ),
+            wxT( "KICAD8_TEMPLATE_DIR" ),
 
             // Deprecated keys
             wxT( "KICAD_PTEMPLATES" ),
@@ -617,11 +655,12 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
     wxDir dir;
     std::vector<wxFileName> base_paths;
 
-    base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false ), wxS( "" ) ) );
+    base_paths.emplace_back( wxFileName( PATHS::CalculateUserSettingsPath( false ), wxS( "" ) ) );
 
     // If the env override is set, also check the default paths
     if( wxGetEnv( wxT( "KICAD_CONFIG_HOME" ), nullptr ) )
-        base_paths.emplace_back( wxFileName( calculateUserSettingsPath( false, false ), wxS( "" ) ) );
+        base_paths.emplace_back( wxFileName( PATHS::CalculateUserSettingsPath( false, false ),
+                                             wxS( "" ) ) );
 
 #ifdef __WXGTK__
     // When running inside FlatPak, KIPLATFORM::ENV::GetUserConfigPath() will return a sandboxed
@@ -700,19 +739,47 @@ bool SETTINGS_MANAGER::GetPreviousVersionPaths( std::vector<wxString>* aPaths )
         }
     }
 
+    alg::delete_if( *aPaths, []( const wxString& aPath ) -> bool
+    {
+        wxFileName fulldir = wxFileName::DirName( aPath );
+        const wxArrayString& dirs = fulldir.GetDirs();
+
+        if( dirs.empty() || !fulldir.IsDirReadable() )
+            return true;
+
+        std::string ver = dirs.back().ToStdString();
+
+        if( !extractVersion( ver ) )
+            return true;
+
+        return false;
+    } );
+
     std::sort( aPaths->begin(), aPaths->end(),
                [&]( const wxString& a, const wxString& b ) -> bool
                {
-                   wxString verA = wxFileName::DirName( a ).GetDirs().back();
-                   wxString verB = wxFileName::DirName( b ).GetDirs().back();
+                   wxFileName aPath = wxFileName::DirName( a );
+                   wxFileName bPath = wxFileName::DirName( b );
 
-                   if( !extractVersion( verA.ToStdString() )
-                       || !extractVersion( verB.ToStdString() ) )
-                   {
+                   const wxArrayString& aDirs = aPath.GetDirs();
+                   const wxArrayString& bDirs = bPath.GetDirs();
+
+                   if( aDirs.empty() )
                        return false;
-                   }
 
-                   return compareVersions( verA.ToStdString(), verB.ToStdString() ) >= 0;
+                   if( bDirs.empty() )
+                       return true;
+
+                   std::string verA = aDirs.back().ToStdString();
+                   std::string verB = bDirs.back().ToStdString();
+
+                   if( !extractVersion( verA ) )
+                       return false;
+
+                   if( !extractVersion( verB ) )
+                       return true;
+
+                   return compareVersions( verA, verB ) >= 0;
                } );
 
     return aPaths->size() > 0;
@@ -736,7 +803,7 @@ wxString SETTINGS_MANAGER::GetColorSettingsPath()
 {
     wxFileName path;
 
-    path.AssignDir( GetUserSettingsPath() );
+    path.AssignDir( PATHS::GetUserSettingsPath() );
     path.AppendDir( wxS( "colors" ) );
 
     if( !path.DirExists() )
@@ -753,43 +820,6 @@ wxString SETTINGS_MANAGER::GetColorSettingsPath()
 }
 
 
-wxString SETTINGS_MANAGER::GetUserSettingsPath()
-{
-    static wxString user_settings_path;
-
-    if( user_settings_path.empty() )
-        user_settings_path = calculateUserSettingsPath();
-
-    return user_settings_path;
-}
-
-
-wxString SETTINGS_MANAGER::calculateUserSettingsPath( bool aIncludeVer, bool aUseEnv )
-{
-    wxFileName cfgpath;
-
-    // http://docs.wxwidgets.org/3.0/classwx_standard_paths.html#a7c7cf595d94d29147360d031647476b0
-
-    wxString envstr;
-    if( aUseEnv && wxGetEnv( wxT( "KICAD_CONFIG_HOME" ), &envstr ) && !envstr.IsEmpty() )
-    {
-        // Override the assignment above with KICAD_CONFIG_HOME
-        cfgpath.AssignDir( envstr );
-    }
-    else
-    {
-        cfgpath.AssignDir( KIPLATFORM::ENV::GetUserConfigPath() );
-
-        cfgpath.AppendDir( TO_STR( KICAD_CONFIG_DIR ) );
-    }
-
-    if( aIncludeVer )
-        cfgpath.AppendDir( GetSettingsVersion() );
-
-    return cfgpath.GetPath();
-}
-
-
 std::string SETTINGS_MANAGER::GetSettingsVersion()
 {
     // CMake computes the major.minor string for us.
@@ -806,7 +836,8 @@ int SETTINGS_MANAGER::compareVersions( const std::string& aFirst, const std::str
 
     if( !extractVersion( aFirst, &a_maj, &a_min ) || !extractVersion( aSecond, &b_maj, &b_min ) )
     {
-        wxLogTrace( traceSettings, wxT( "compareSettingsVersions: bad input (%s, %s)" ), aFirst, aSecond );
+        wxLogTrace( traceSettings, wxT( "compareSettingsVersions: bad input (%s, %s)" ),
+                    aFirst, aSecond );
         return -1;
     }
 
@@ -871,8 +902,8 @@ bool SETTINGS_MANAGER::LoadProject( const wxString& aFullPath, bool aSetActive )
     // Normalize path to new format even if migrating from a legacy file
     wxFileName path( aFullPath );
 
-    if( path.GetExt() == LegacyProjectFileExtension )
-        path.SetExt( ProjectFileExtension );
+    if( path.GetExt() == FILEEXT::LegacyProjectFileExtension )
+        path.SetExt( FILEEXT::ProjectFileExtension );
 
     wxString fullPath = path.GetFullPath();
 
@@ -883,9 +914,9 @@ bool SETTINGS_MANAGER::LoadProject( const wxString& aFullPath, bool aSetActive )
         return true;
 
     bool readOnly = false;
-    std::unique_ptr<wxSingleInstanceChecker> lockFile = ::LockFile( fullPath );
+    LOCKFILE lockFile( fullPath );
 
-    if( !lockFile )
+    if( !lockFile.Valid() )
     {
         wxLogTrace( traceSettings, wxT( "Project %s is locked; opening read-only" ), fullPath );
         readOnly = true;
@@ -913,14 +944,22 @@ bool SETTINGS_MANAGER::LoadProject( const wxString& aFullPath, bool aSetActive )
     std::unique_ptr<PROJECT> project = std::make_unique<PROJECT>();
     project->setProjectFullName( fullPath );
 
+    if( aSetActive )
+    {
+        // until multiple projects are in play, set an environment variable for the
+        // the project pointer.
+        wxFileName projectPath( fullPath );
+        wxSetEnv( PROJECT_VAR_NAME, projectPath.GetPath() );
+    }
+
     bool success = loadProjectFile( *project );
 
     if( success )
     {
         project->SetReadOnly( readOnly || project->GetProjectFile().IsReadOnly() );
 
-        if( lockFile )
-            m_project_lock.reset( lockFile.release() );
+        if( lockFile && aSetActive )
+            m_project_lock.reset( new LOCKFILE( std::move( lockFile ) ) );
     }
 
     m_projects_list.push_back( std::move( project ) );
@@ -956,6 +995,8 @@ bool SETTINGS_MANAGER::UnloadProject( PROJECT* aProject, bool aSave )
     wxLogTrace( traceSettings, wxT( "Unload project %s" ), projectPath );
 
     PROJECT* toRemove = m_projects.at( projectPath );
+    bool wasActiveProject = m_projects_list.begin()->get() == toRemove;
+
     auto it = std::find_if( m_projects_list.begin(), m_projects_list.end(),
                             [&]( const std::unique_ptr<PROJECT>& ptr )
                             {
@@ -967,19 +1008,22 @@ bool SETTINGS_MANAGER::UnloadProject( PROJECT* aProject, bool aSave )
 
     m_projects.erase( projectPath );
 
-    // Immediately reload a null project; this is required until the rest of the application
-    // is refactored to not assume that Prj() always works
-    if( m_projects.empty() )
-        LoadProject( "" );
+    if( wasActiveProject )
+    {
+        // Immediately reload a null project; this is required until the rest of the application
+        // is refactored to not assume that Prj() always works
+        if( m_projects_list.empty() )
+            LoadProject( "" );
 
-    // Remove the reference in the environment to the previous project
-    wxSetEnv( PROJECT_VAR_NAME, wxS( "" ) );
+        // Remove the reference in the environment to the previous project
+        wxSetEnv( PROJECT_VAR_NAME, wxS( "" ) );
 
-    // Release lock on the file, in case we had one
-    m_project_lock = nullptr;
+        // Release lock on the file, in case we had one
+        m_project_lock = nullptr;
 
-    if( m_kiway )
-        m_kiway->ProjectChanged();
+        if( m_kiway )
+            m_kiway->ProjectChanged();
+    }
 
     return true;
 }
@@ -1141,6 +1185,9 @@ bool SETTINGS_MANAGER::unloadProjectFile( PROJECT* aProject, bool aSave )
 
     PROJECT_FILE* file = m_project_files[name];
 
+    if( !file->ShouldAutoSave() )
+        aSave = false;
+
     auto it = std::find_if( m_settings.begin(), m_settings.end(),
                             [&file]( const std::unique_ptr<JSON_SETTINGS>& aPtr )
                             {
@@ -1151,7 +1198,9 @@ bool SETTINGS_MANAGER::unloadProjectFile( PROJECT* aProject, bool aSave )
     {
         wxString projectPath = GetPathForSettingsFile( it->get() );
 
-        FlushAndRelease( &aProject->GetLocalSettings(), aSave );
+        bool saveLocalSettings = aSave && aProject->GetLocalSettings().ShouldAutoSave();
+
+        FlushAndRelease( &aProject->GetLocalSettings(), saveLocalSettings );
 
         if( aSave )
             ( *it )->SaveToFile( projectPath );
@@ -1184,11 +1233,12 @@ bool SETTINGS_MANAGER::BackupProject( REPORTER& aReporter ) const
     wxFileName target;
     target.SetPath( GetProjectBackupsPath() );
     target.SetName( fileName );
-    target.SetExt( ArchiveFileExtension );
+    target.SetExt( FILEEXT::ArchiveFileExtension );
 
     if( !target.DirExists() && !wxMkdir( target.GetPath() ) )
     {
-        wxLogTrace( traceSettings, wxT( "Could not create project backup path %s" ), target.GetPath() );
+        wxLogTrace( traceSettings, wxT( "Could not create project backup path %s" ),
+                    target.GetPath() );
         return false;
     }
 
@@ -1378,3 +1428,9 @@ bool SETTINGS_MANAGER::TriggerBackupIfNeeded( REPORTER& aReporter ) const
 
     return BackupProject( aReporter );
 }
+
+
+wxString SETTINGS_MANAGER::GetUserSettingsPath()
+{
+    return PATHS::GetUserSettingsPath();
+}
