@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
- * Copyright (C) 2020 CERN
+ * Copyright (C) 2020-2023 CERN
  * Copyright (C) 2020-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -24,6 +24,7 @@
 #include <eda_base_frame.h>
 #include <eda_item.h>
 #include <import_export.h>
+#include <pgm_base.h>
 #include <properties/pg_cell_renderer.h>
 
 #include <algorithm>
@@ -34,7 +35,10 @@
 #include <wx/propgrid/advprops.h>
 
 
+// This is provided by wx >3.3.0
+#if !wxCHECK_VERSION( 3, 3, 0 )
 extern APIIMPORT wxPGGlobalVarsClass* wxPGGlobalVars;
+#endif
 
 PROPERTIES_PANEL::PROPERTIES_PANEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame ) :
         wxPanel( aParent ),
@@ -62,22 +66,40 @@ PROPERTIES_PANEL::PROPERTIES_PANEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame )
         wxPGEditor_DatePickerCtrl = nullptr;
     }
 
-    delete wxPGGlobalVars->m_defaultRenderer;
-    wxPGGlobalVars->m_defaultRenderer = new PG_CELL_RENDERER();
+    if( !Pgm().m_PropertyGridInitialized )
+    {
+        delete wxPGGlobalVars->m_defaultRenderer;
+        wxPGGlobalVars->m_defaultRenderer = new PG_CELL_RENDERER();
+        Pgm().m_PropertyGridInitialized = true;
+    }
 
     m_caption = new wxStaticText( this, wxID_ANY, _( "No objects selected" ) );
     mainSizer->Add( m_caption, 0, wxALL | wxEXPAND, 5 );
 
-    m_grid = new wxPropertyGrid( this, wxID_ANY, wxDefaultPosition, wxSize( 300, 400 ),
-                                 wxPG_DEFAULT_STYLE );
+    m_grid = new wxPropertyGrid( this );
     m_grid->SetUnspecifiedValueAppearance( wxPGCell( wxT( "<...>" ) ) );
     m_grid->SetExtraStyle( wxPG_EX_HELP_AS_TOOLTIPS );
+
+#if wxCHECK_VERSION( 3, 3, 0 )
+    m_grid->SetValidationFailureBehavior( wxPGVFBFlags::MarkCell );
+#else
     m_grid->SetValidationFailureBehavior( wxPG_VFB_MARK_CELL );
+#endif
+
+#if wxCHECK_VERSION( 3, 3, 0 )
+    m_grid->AddActionTrigger( wxPGKeyboardActions::NextProperty, WXK_RETURN );
+    m_grid->AddActionTrigger( wxPGKeyboardActions::NextProperty, WXK_NUMPAD_ENTER );
+    m_grid->AddActionTrigger( wxPGKeyboardActions::NextProperty, WXK_DOWN );
+    m_grid->AddActionTrigger( wxPGKeyboardActions::PrevProperty, WXK_UP );
+    m_grid->AddActionTrigger( wxPGKeyboardActions::Edit, WXK_SPACE );
+#else
     m_grid->AddActionTrigger( wxPG_ACTION_NEXT_PROPERTY, WXK_RETURN );
     m_grid->AddActionTrigger( wxPG_ACTION_NEXT_PROPERTY, WXK_NUMPAD_ENTER );
     m_grid->AddActionTrigger( wxPG_ACTION_NEXT_PROPERTY, WXK_DOWN );
     m_grid->AddActionTrigger( wxPG_ACTION_PREV_PROPERTY, WXK_UP );
     m_grid->AddActionTrigger( wxPG_ACTION_EDIT, WXK_SPACE );
+#endif
+
     m_grid->DedicateKey( WXK_RETURN );
     m_grid->DedicateKey( WXK_NUMPAD_ENTER );
     m_grid->DedicateKey( WXK_DOWN );
@@ -113,7 +135,7 @@ PROPERTIES_PANEL::PROPERTIES_PANEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame )
     Bind( wxEVT_SIZE,
           [&]( wxSizeEvent& aEvent )
           {
-              CallAfter( [&]()
+              CallAfter( [this]()
                          {
                             RecalculateSplitterPos();
                          } );
@@ -124,6 +146,12 @@ PROPERTIES_PANEL::PROPERTIES_PANEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame )
 
 void PROPERTIES_PANEL::OnLanguageChanged()
 {
+    if( m_grid->IsEditorFocused() )
+        m_grid->CommitChangesFromEditor();
+
+    m_grid->Clear();
+    m_displayed.clear();
+
     UpdateData();
 }
 
@@ -146,6 +174,14 @@ void PROPERTIES_PANEL::rebuildProperties( const SELECTION& aSelection )
         reset();
         return;
     }
+    else if( aSelection.Size() == 1 )
+    {
+        m_caption->SetLabel( aSelection.Front()->GetFriendlyName() );
+    }
+    else
+    {
+        m_caption->SetLabel( wxString::Format( _( "%d objects selected" ), aSelection.Size() ) );
+    }
 
     // Get all the selected types
     std::set<TYPE_ID> types;
@@ -155,12 +191,10 @@ void PROPERTIES_PANEL::rebuildProperties( const SELECTION& aSelection )
 
     wxCHECK( !types.empty(), /* void */ );
 
-    PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
-    propMgr.SetUnits( m_frame->GetUserUnits() );
-    propMgr.SetTransforms( &m_frame->GetOriginTransforms() );
-
+    PROPERTY_MANAGER&        propMgr = PROPERTY_MANAGER::Instance();
     std::set<PROPERTY_BASE*> commonProps;
-    const PROPERTY_LIST& allProperties = propMgr.GetProperties( *types.begin() );
+    const PROPERTY_LIST&     allProperties = propMgr.GetProperties( *types.begin() );
+
     copy( allProperties.begin(), allProperties.end(), inserter( commonProps, commonProps.begin() ) );
 
     PROPERTY_DISPLAY_ORDER displayOrder = propMgr.GetDisplayOrder( *types.begin() );
@@ -201,11 +235,15 @@ void PROPERTIES_PANEL::rebuildProperties( const SELECTION& aSelection )
     }
 
     EDA_ITEM* firstItem = aSelection.Front();
+    bool isFootprintEditor = m_frame->IsType( FRAME_FOOTPRINT_EDITOR );
 
     // Find a set of properties that is common to all selected items
     for( PROPERTY_BASE* property : commonProps )
     {
-        if( property->IsInternal() )
+        if( property->IsHiddenFromPropertiesManager() )
+            continue;
+
+        if( isFootprintEditor && property->IsHiddenFromLibraryEditors() )
             continue;
 
         if( propMgr.IsAvailableFor( TYPE_HASH( *firstItem ), property, firstItem ) )
@@ -233,7 +271,7 @@ void PROPERTIES_PANEL::rebuildProperties( const SELECTION& aSelection )
         existingProps.insert( property );
     }
 
-    if( existingProps == availableProps )
+    if( !existingProps.empty() && existingProps == availableProps )
         return;
 
     // Some difference exists:  start from scratch
@@ -262,15 +300,6 @@ void PROPERTIES_PANEL::rebuildProperties( const SELECTION& aSelection )
         }
     }
 
-    if( aSelection.Size() > 1 )
-    {
-        m_caption->SetLabel( wxString::Format( _( "%d objects selected" ), aSelection.Size() ) );
-    }
-    else
-    {
-        m_caption->SetLabel( aSelection.Front()->GetFriendlyName() );
-    }
-
     const wxString unspecifiedGroupCaption = _( "Basic Properties" );
 
     for( const wxString& groupName : groupDisplayOrder )
@@ -330,11 +359,8 @@ bool PROPERTIES_PANEL::extractValueAndWritability( const SELECTION& aSelection,
                                                    wxVariant& aValue, bool& aWritable )
 {
     PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
-    propMgr.SetUnits( m_frame->GetUserUnits() );
-    propMgr.SetTransforms( &m_frame->GetOriginTransforms() );
-
-    bool different = false;
-    wxVariant commonVal;
+    bool              different = false;
+    wxVariant         commonVal;
 
     aWritable = true;
 
@@ -343,8 +369,11 @@ bool PROPERTIES_PANEL::extractValueAndWritability( const SELECTION& aSelection,
         if( !propMgr.IsAvailableFor( TYPE_HASH( *item ), aProperty, item ) )
             return false;
 
+        if( aProperty->IsHiddenFromPropertiesManager() )
+            return false;
+
         // If read-only for any of the selection, read-only for the whole selection.
-        if( !aProperty->Writeable( item ) )
+        if( !propMgr.IsWriteableFor( TYPE_HASH( *item ), aProperty, item ) )
             aWritable = false;
 
         wxVariant value;
@@ -377,14 +406,19 @@ void PROPERTIES_PANEL::onShow( wxShowEvent& aEvent )
 {
     if( aEvent.IsShown() )
         UpdateData();
+
+    aEvent.Skip();
 }
 
 
 void PROPERTIES_PANEL::onCharHook( wxKeyEvent& aEvent )
 {
-    if( aEvent.GetKeyCode() == WXK_TAB && !aEvent.ShiftDown() )
+    if( aEvent.GetKeyCode() == WXK_TAB && !aEvent.ShiftDown() && m_grid->IsAnyModified() )
     {
         m_grid->CommitChangesFromEditor();
+
+        // Pass the tab key on so the default property grid tab behavior is honored.
+        aEvent.Skip();
         return;
     }
 
