@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2012 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,8 +38,10 @@
 #include <tools/ee_inspection_tool.h>
 #include <dialog_erc.h>
 #include <erc.h>
+#include <erc_report.h>
 #include <id.h>
 #include <confirm.h>
+#include <common.h>
 #include <widgets/wx_html_report_box.h>
 #include <wx/ffile.h>
 #include <wx/filedlg.h>
@@ -49,6 +51,10 @@
 #include <string_utils.h>
 #include <kiplatform/ui.h>
 
+
+wxDEFINE_EVENT( EDA_EVT_CLOSE_ERC_DIALOG, wxCommandEvent );
+
+
 // wxWidgets spends *far* too long calcuating column widths (most of it, believe it or
 // not, in repeatedly creating/destroying a wxDC to do the measurement in).
 // Use default column widths instead.
@@ -73,6 +79,7 @@ DIALOG_ERC::DIALOG_ERC( SCH_EDIT_FRAME* parent ) :
     m_currentSchematic = &parent->Schematic();
 
     SetName( DIALOG_ERC_WINDOW_NAME ); // Set a window name to be able to find it
+    KIPLATFORM::UI::SetFloatLevel( this );
 
     EESCHEMA_SETTINGS* settings = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
     m_severities = settings->m_Appearance.erc_severities;
@@ -146,7 +153,7 @@ void DIALOG_ERC::UpdateAnnotationWarning()
     if( m_parent->CheckAnnotate( []( ERCE_T, const wxString&, SCH_REFERENCE*, SCH_REFERENCE* )
                                  { } ) )
     {
-        if( !m_infoBar->IsShown() )
+        if( !m_infoBar->IsShownOnScreen() )
         {
             wxHyperlinkCtrl* button = new wxHyperlinkCtrl( m_infoBar, wxID_ANY,
                                                            _( "Show Annotation dialog" ),
@@ -168,7 +175,7 @@ void DIALOG_ERC::UpdateAnnotationWarning()
     }
     else
     {
-        if( m_infoBar->IsShown() )
+        if( m_infoBar->IsShownOnScreen() )
         {
             m_infoBar->RemoveAllButtons();
             m_infoBar->Hide();
@@ -177,8 +184,6 @@ void DIALOG_ERC::UpdateAnnotationWarning()
 }
 
 
-// PROGRESS_REPORTER calls
-
 bool DIALOG_ERC::updateUI()
 {
     // If ERC checks ever get slow enough we'll want a progress indicator...
@@ -273,7 +278,7 @@ void DIALOG_ERC::OnDeleteOneClick( wxCommandEvent& aEvent )
     if( m_notebook->GetSelection() == 0 )
     {
         // Clear the selection.  It may be the selected ERC marker.
-        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection, true );
+        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
 
         m_markerTreeModel->DeleteCurrentItem( true );
 
@@ -285,8 +290,6 @@ void DIALOG_ERC::OnDeleteOneClick( wxCommandEvent& aEvent )
 }
 
 
-/* Delete the old ERC markers, over the whole hierarchy
- */
 void DIALOG_ERC::OnDeleteAllClick( wxCommandEvent& event )
 {
     bool includeExclusions = false;
@@ -318,7 +321,6 @@ void DIALOG_ERC::OnDeleteAllClick( wxCommandEvent& event )
 }
 
 
-// This is a modeless dialog so we have to handle these ourselves.
 void DIALOG_ERC::OnCancelClick( wxCommandEvent& aEvent )
 {
     if( m_running )
@@ -329,15 +331,26 @@ void DIALOG_ERC::OnCancelClick( wxCommandEvent& aEvent )
 
     m_parent->FocusOnItem( nullptr );
 
-    Close();
+    aEvent.Skip();
 }
 
 
-void DIALOG_ERC::OnCloseErcDialog( wxCloseEvent& event )
+void DIALOG_ERC::OnCloseErcDialog( wxCloseEvent& aEvent )
 {
     m_parent->FocusOnItem( nullptr );
 
-    m_parent->GetToolManager()->GetTool<EE_INSPECTION_TOOL>()->DestroyERCDialog();
+    // Dialog is mode-less so let the parent know that it needs to be destroyed.
+    if( !IsModal() && !IsQuasiModal() )
+    {
+        wxCommandEvent* evt = new wxCommandEvent( EDA_EVT_CLOSE_ERC_DIALOG, wxID_ANY );
+
+        wxWindow* parent = GetParent();
+
+        if( parent )
+            wxQueueEvent( parent, evt );
+    }
+
+    aEvent.Skip();
 }
 
 
@@ -368,7 +381,7 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
 
     UpdateAnnotationWarning();
 
-    m_parent->RecordERCExclusions();
+    sch->RecordERCExclusions();
     deleteAllMarkers( true );
 
     std::vector<std::reference_wrapper<RC_ITEM>> violations = ERC_ITEM::GetItemsWithSeverities();
@@ -383,6 +396,9 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
         }
     }
 
+    m_ignoredList->SetColumnWidth( 0, m_ignoredList->GetParent()->GetClientSize().x - 20 );
+
+    m_cancelled = false;
     Raise();
 
     m_runningResultsBook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
@@ -436,14 +452,23 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
 
     if( !m_cancelled )
     {
+        m_sdbSizer1Cancel->SetDefault();
+        // wxWidgets has a tendency to keep both buttons highlighted without the following:
+        m_sdbSizer1OK->Enable( false );
+
         wxMilliSleep( 500 );
         m_runningResultsBook->ChangeSelection( 1 );
-        KIPLATFORM::UI::ForceFocus( m_markerDataView );
+        KIPLATFORM::UI::ForceFocus( m_notebook );
+
+        // now re-enable m_sdbSizerOK button
+        m_sdbSizer1OK->Enable( true );
     }
 
     m_ercRun = true;
     redrawDrawPanel();
     updateDisplayedCounts();
+    // set float level again, it can be lost due to window events during test run
+    KIPLATFORM::UI::SetFloatLevel( this );
 }
 
 
@@ -462,104 +487,14 @@ void DIALOG_ERC::testErc()
     SCHEMATIC* sch = &m_parent->Schematic();
 
     SCH_SCREENS screens( sch->Root() );
-    ERC_SETTINGS& settings = sch->ErcSettings();
     ERC_TESTER tester( sch );
 
-    // Test duplicate sheet names inside a given sheet.  While one can have multiple references
-    // to the same file, each must have a unique name.
-    if( settings.IsTestEnabled( ERCE_DUPLICATE_SHEET_NAME ) )
     {
-        AdvancePhase( _( "Checking sheet names..." ) );
-        tester.TestDuplicateSheetNames( true );
+        wxBusyCursor dummy;
+        tester.RunTests( m_parent->GetCanvas()->GetView()->GetDrawingSheet(), m_parent,
+                         m_parent->Kiway().KiFACE( KIWAY::FACE_CVPCB ), &m_parent->Prj(), this );
     }
 
-    if( settings.IsTestEnabled( ERCE_BUS_ALIAS_CONFLICT ) )
-    {
-        AdvancePhase( _( "Checking bus conflicts..." ) );
-        tester.TestConflictingBusAliases();
-    }
-
-    // The connection graph has a whole set of ERC checks it can run
-    AdvancePhase( _( "Checking conflicts..." ) );
-
-    // If we are using the new connectivity, make sure that we do a full-rebuild
-    if( ADVANCED_CFG::GetCfg().m_IncrementalConnectivity )
-        m_parent->RecalculateConnections( GLOBAL_CLEANUP );
-    else
-        m_parent->RecalculateConnections( NO_CLEANUP );
-
-    sch->ConnectionGraph()->RunERC();
-
-    AdvancePhase( _( "Checking units..." ) );
-
-    // Test is all units of each multiunit symbol have the same footprint assigned.
-    if( settings.IsTestEnabled( ERCE_DIFFERENT_UNIT_FP ) )
-    {
-        AdvancePhase( _( "Checking footprints..." ) );
-        tester.TestMultiunitFootprints();
-    }
-
-    if( settings.IsTestEnabled( ERCE_MISSING_UNIT )
-            || settings.IsTestEnabled( ERCE_MISSING_INPUT_PIN )
-            || settings.IsTestEnabled( ERCE_MISSING_POWER_INPUT_PIN )
-            || settings.IsTestEnabled( ERCE_MISSING_BIDI_PIN ) )
-    {
-        tester.TestMissingUnits();
-    }
-
-    AdvancePhase( _( "Checking pins..." ) );
-
-    if( settings.IsTestEnabled( ERCE_DIFFERENT_UNIT_NET ) )
-        tester.TestMultUnitPinConflicts();
-
-    // Test pins on each net against the pin connection table
-    if( settings.IsTestEnabled( ERCE_PIN_TO_PIN_ERROR )
-            || settings.IsTestEnabled( ERCE_POWERPIN_NOT_DRIVEN )
-            || settings.IsTestEnabled( ERCE_PIN_NOT_DRIVEN ) )
-    {
-         tester.TestPinToPin();
-    }
-
-    // Test similar labels (i;e. labels which are identical when
-    // using case insensitive comparisons)
-    if( settings.IsTestEnabled( ERCE_SIMILAR_LABELS ) )
-    {
-        AdvancePhase( _( "Checking labels..." ) );
-        tester.TestSimilarLabels();
-    }
-
-    if( settings.IsTestEnabled( ERCE_UNRESOLVED_VARIABLE ) )
-    {
-        AdvancePhase( _( "Checking for unresolved variables..." ) );
-        tester.TestTextVars( m_parent->GetCanvas()->GetView()->GetDrawingSheet() );
-    }
-
-    if( settings.IsTestEnabled( ERCE_SIMULATION_MODEL ) )
-    {
-        AdvancePhase( _( "Checking SPICE models..." ) );
-        tester.TestSimModelIssues();
-    }
-
-    if( settings.IsTestEnabled( ERCE_NOCONNECT_CONNECTED ) )
-    {
-        AdvancePhase( _( "Checking no connect pins for connections..." ) );
-        tester.TestNoConnectPins();
-    }
-
-    if( settings.IsTestEnabled( ERCE_LIB_SYMBOL_ISSUES ) )
-    {
-        AdvancePhase( _( "Checking for library symbol issues..." ) );
-        tester.TestLibSymbolIssues();
-    }
-
-    if( settings.IsTestEnabled( ERCE_ENDPOINT_OFF_GRID ) )
-    {
-        AdvancePhase( _( "Checking for off grid pins and wires..." ) );
-        tester.TestOffGridEndpoints( m_parent->GetCanvas()->GetView()->GetGAL()->GetGridSize().x );
-    }
-
-    m_parent->ResolveERCExclusions();
-
     // Update marker list:
     m_markerTreeModel->Update( m_markerProvider, m_severities );
 
@@ -578,7 +513,7 @@ void DIALOG_ERC::OnERCItemSelected( wxDataViewEvent& aEvent )
 {
     const KIID&     itemID = RC_TREE_MODEL::ToUUID( aEvent.GetItem() );
     SCH_SHEET_PATH  sheet;
-    SCH_ITEM*       item = m_parent->Schematic().GetSheets().GetItem( itemID, &sheet );
+    SCH_ITEM*       item = m_parent->Schematic().GetItem( itemID, &sheet );
 
     if( m_centerMarkerOnIdle )
     {
@@ -617,12 +552,12 @@ void DIALOG_ERC::OnERCItemSelected( wxDataViewEvent& aEvent )
 
         if( !sheet.empty() && sheet != m_parent->GetCurrentSheet() )
         {
-            m_parent->GetToolManager()->RunAction( ACTIONS::cancelInteractive, true );
-            m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection, true );
+            m_parent->GetToolManager()->RunAction( ACTIONS::cancelInteractive );
+            m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
 
             m_parent->SetCurrentSheet( sheet );
             m_parent->DisplayCurrentSheet();
-            m_parent->RedrawScreen( (wxPoint) m_parent->GetScreen()->m_ScrollCenter, false );
+            m_parent->RedrawScreen(  m_parent->GetScreen()->m_ScrollCenter, false );
         }
 
         m_parent->FocusOnItem( item );
@@ -681,7 +616,7 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
     menu.AppendSeparator();
 
     if( rcItem->GetErrorCode() == ERCE_PIN_TO_PIN_WARNING
-        || rcItem->GetErrorCode() == ERCE_PIN_TO_PIN_ERROR )
+      || rcItem->GetErrorCode() == ERCE_PIN_TO_PIN_ERROR )
     {
         // Pin to pin severities edited through pin conflict map
     }
@@ -689,13 +624,13 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
     {
         menu.Append( 4, wxString::Format( _( "Change severity to Error for all '%s' violations" ),
                                           rcItem->GetErrorText() ),
-                     _( "Violation severities can also be edited in the Board Setup... dialog" ) );
+                     _( "Violation severities can also be edited in the Schematic Setup... dialog" ) );
     }
     else
     {
         menu.Append( 5, wxString::Format( _( "Change severity to Warning for all '%s' violations" ),
                                           rcItem->GetErrorText() ),
-                     _( "Violation severities can also be edited in the Board Setup... dialog" ) );
+                     _( "Violation severities can also be edited in the Schematic Setup... dialog" ) );
     }
 
     menu.Append( 6, wxString::Format( _( "Ignore all '%s' violations" ), rcItem->GetErrorText() ),
@@ -714,6 +649,12 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
                      _( "Open the Schematic Setup... dialog" ) );
     }
 
+    if( rcItem->GetErrorCode() == ERCE_ENDPOINT_OFF_GRID )
+    {
+        menu.Append( 9, _( "Edit connection grid spacing..." ),
+                     _( "Open the Schematic Setup... dialog" ) );
+    }
+
     bool modified = false;
 
     switch( GetPopupMenuSelectionFromUser( menu ) )
@@ -793,6 +734,10 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
         if( rcItem->GetErrorCode() == ERCE_PIN_TO_PIN_ERROR )
             settings.SetSeverity( ERCE_PIN_TO_PIN_WARNING, RPT_SEVERITY_IGNORE );
 
+        // Clear the selection before deleting markers. It may be some selected ERC markers.
+        // Deleting a selected marker without deselecting it first generates a crash
+        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
+
         SCH_SCREENS ScreenList( m_parent->Schematic().Root() );
         ScreenList.DeleteMarkers( MARKER_BASE::MARKER_ERC, rcItem->GetErrorCode() );
 
@@ -809,6 +754,10 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
     case 8:
         m_parent->ShowSchematicSetupDialog( _( "Violation Severity" ) );
         break;
+
+    case 9:
+        m_parent->ShowSchematicSetupDialog( _( "Formatting" ) );
+        break;
     }
 
     if( modified )
@@ -900,18 +849,9 @@ void DIALOG_ERC::ExcludeMarker( SCH_MARKER* aMarker )
 }
 
 
-void DIALOG_ERC::OnIgnoreItemRClick( wxListEvent& event )
+void DIALOG_ERC::OnEditViolationSeverities( wxHyperlinkEvent& aEvent )
 {
-    wxMenu menu;
-
-    menu.Append( 1, _( "Edit ignored violations..." ), _( "Open the Schematic Setup... dialog" ) );
-
-    switch( GetPopupMenuSelectionFromUser( menu ) )
-    {
-    case 1:
-        m_parent->ShowSchematicSetupDialog( _( "Violation Severity" ) );
-        break;
-    }
+    m_parent->ShowSchematicSetupDialog( _( "Violation Severity" ) );
 }
 
 
@@ -945,21 +885,26 @@ void DIALOG_ERC::OnSeverity( wxCommandEvent& aEvent )
 void DIALOG_ERC::deleteAllMarkers( bool aIncludeExclusions )
 {
     // Clear current selection list to avoid selection of deleted items
-    m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection, true );
+    // Freeze to avoid repainting the dialog, which can cause a RePaint()
+    // of the screen as well
+    Freeze();
+    m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
 
     m_markerTreeModel->DeleteItems( false, aIncludeExclusions, false );
 
     SCH_SCREENS screens( m_parent->Schematic().Root() );
     screens.DeleteAllMarkers( MARKER_BASE::MARKER_ERC, aIncludeExclusions );
+    Thaw();
 }
 
 
 void DIALOG_ERC::OnSaveReport( wxCommandEvent& aEvent )
 {
-    wxFileName fn( wxS( "ERC." ) + ReportFileExtension );
+    wxFileName fn( wxS( "ERC." ) + FILEEXT::ReportFileExtension );
 
-    wxFileDialog dlg( this, _( "Save Report to File" ), Prj().GetProjectPath(), fn.GetFullName(),
-                      ReportFileWildcard(), wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
+    wxFileDialog dlg( this, _( "Save Report File" ), Prj().GetProjectPath(), fn.GetFullName(),
+                      FILEEXT::ReportFileWildcard() + wxS( "|" ) + FILEEXT::JsonFileWildcard(),
+                      wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
     if( dlg.ShowModal() != wxID_OK )
         return;
@@ -967,7 +912,7 @@ void DIALOG_ERC::OnSaveReport( wxCommandEvent& aEvent )
     fn = dlg.GetPath();
 
     if( fn.GetExt().IsEmpty() )
-        fn.SetExt( ReportFileExtension );
+        fn.SetExt( FILEEXT::ReportFileExtension );
 
     if( !fn.IsAbsolute() )
     {
@@ -975,7 +920,15 @@ void DIALOG_ERC::OnSaveReport( wxCommandEvent& aEvent )
         fn.MakeAbsolute( prj_path );
     }
 
-    if( writeReport( fn.GetFullPath() ) )
+    ERC_REPORT reportWriter( &m_parent->Schematic(), m_parent->GetUserUnits() );
+
+    bool success = false;
+    if( fn.GetExt() == FILEEXT::JsonFileExtension )
+        success = reportWriter.WriteJsonReport( fn.GetFullPath() );
+    else
+        success = reportWriter.WriteTextReport( fn.GetFullPath() );
+
+    if( success )
     {
         m_messages->Report( wxString::Format( _( "Report file '%s' created." ),
                                               fn.GetFullPath() ) );
@@ -985,66 +938,4 @@ void DIALOG_ERC::OnSaveReport( wxCommandEvent& aEvent )
         DisplayError( this, wxString::Format( _( "Failed to create file '%s'." ),
                                               fn.GetFullPath() ) );
     }
-}
-
-
-bool DIALOG_ERC::writeReport( const wxString& aFullFileName )
-{
-    wxFFile file( aFullFileName, wxT( "wt" ) );
-
-    if( !file.IsOpened() )
-        return false;
-
-    wxString msg = wxString::Format( _( "ERC report (%s, Encoding UTF8)\n" ), DateAndTime() );
-
-    std::map<KIID, EDA_ITEM*> itemMap;
-
-    int            err_count = 0;
-    int            warn_count = 0;
-    int            total_count = 0;
-    SCH_SHEET_LIST sheetList = m_parent->Schematic().GetSheets();
-
-    sheetList.FillItemMap( itemMap );
-
-    ERC_SETTINGS& settings = m_parent->Schematic().ErcSettings();
-
-    for( unsigned i = 0;  i < sheetList.size(); i++ )
-    {
-        msg << wxString::Format( _( "\n***** Sheet %s\n" ), sheetList[i].PathHumanReadable() );
-
-        for( SCH_ITEM* aItem : sheetList[i].LastScreen()->Items().OfType( SCH_MARKER_T ) )
-        {
-            const SCH_MARKER* marker = static_cast<const SCH_MARKER*>( aItem );
-            RC_ITEM*          item = marker->GetRCItem().get();
-            SEVERITY          severity = settings.GetSeverity( item->GetErrorCode() );
-
-            if( marker->GetMarkerType() != MARKER_BASE::MARKER_ERC )
-                continue;
-
-            total_count++;
-
-            switch( severity )
-            {
-            case RPT_SEVERITY_ERROR:   err_count++;  break;
-            case RPT_SEVERITY_WARNING: warn_count++; break;
-            default:                                 break;
-            }
-
-            msg << marker->GetRCItem()->ShowReport( m_parent, severity, itemMap );
-        }
-    }
-
-    msg << wxString::Format( _( "\n ** ERC messages: %d  Errors %d  Warnings %d\n" ),
-                             total_count, err_count, warn_count );
-
-    // Currently: write report using UTF8 (as usual in Kicad).
-    // TODO: see if we can use the current encoding page (mainly for Windows users),
-    // Or other format (HTML?)
-    file.Write( msg );
-
-    // wxFFile dtor will close the file.
-
-    return true;
-}
-
-
+}
\ No newline at end of file
