@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,7 +21,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <connection_graph.h>
 #include <dialog_global_edit_text_and_graphics_base.h>
 #include <string_utils.h>
 #include <sch_symbol.h>
@@ -33,7 +32,7 @@
 #include <sch_sheet.h>
 #include <sch_sheet_pin.h>
 #include <schematic.h>
-#include <advanced_config.h>
+#include <sch_commit.h>
 #include <tool/tool_manager.h>
 #include <tools/ee_selection_tool.h>
 #include <tools/sch_edit_tool.h>
@@ -78,8 +77,6 @@ class DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS : public DIALOG_GLOBAL_EDIT_TEXT_AND_
     UNIT_BINDER            m_lineWidth;
     UNIT_BINDER            m_junctionSize;
 
-    bool                   m_appendUndo;
-
 public:
     DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS( SCH_EDIT_FRAME* parent );
     ~DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS() override;
@@ -105,8 +102,8 @@ protected:
     bool TransferDataToWindow() override;
     bool TransferDataFromWindow() override;
 
-    void visitItem( const SCH_SHEET_PATH& aSheetPath, SCH_ITEM* aItem );
-    void processItem( const SCH_SHEET_PATH& aSheetPath, SCH_ITEM* aItem );
+    void visitItem( SCH_COMMIT* aCommit, const SCH_SHEET_PATH& aSheetPath, SCH_ITEM* aItem );
+    void processItem( SCH_COMMIT* aCommit, const SCH_SHEET_PATH& aSheetPath, SCH_ITEM* aItem );
 };
 
 
@@ -117,7 +114,6 @@ DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS( SCH_
         m_junctionSize( parent, m_dotSizeLabel, m_dotSizeCtrl, m_dotSizeUnits, true )
 {
     m_parent = parent;
-    m_appendUndo = false;
 
     m_lineStyle->Append( DEFAULT_STYLE );
     m_lineStyle->Append( INDETERMINATE_ACTION );
@@ -239,7 +235,8 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataToWindow()
 }
 
 
-void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aSheetPath,
+void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( SCH_COMMIT* aCommit,
+                                                        const SCH_SHEET_PATH& aSheetPath,
                                                         SCH_ITEM* aItem )
 {
     if( m_selectedFilterOpt->GetValue() )
@@ -248,14 +245,12 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aS
             return;
     }
 
-    m_parent->SaveCopyInUndoList( aSheetPath.LastScreen(), aItem, UNDO_REDO::CHANGED, m_appendUndo,
-                                  false );
-    m_appendUndo = true;
+    aCommit->Modify( aItem, aSheetPath.LastScreen() );
 
     if( EDA_TEXT* eda_text = dynamic_cast<EDA_TEXT*>( aItem ) )
     {
         if( !m_textSize.IsIndeterminate() )
-            eda_text->SetTextSize( wxSize( m_textSize.GetValue(), m_textSize.GetValue() ) );
+            eda_text->SetTextSize( VECTOR2I( m_textSize.GetValue(), m_textSize.GetValue() ) );
 
         if( m_setTextColor->GetValue() )
             eda_text->SetTextColor( m_textColorSwatch->GetSwatchColor() );
@@ -298,10 +293,11 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aS
         if( m_italic->Get3StateValue() != wxCHK_UNDETERMINED )
             eda_text->SetItalic( m_italic->GetValue() );
 
+        // Must come after SetTextSize()
         if( m_bold->Get3StateValue() != wxCHK_UNDETERMINED )
             eda_text->SetBold( m_bold->GetValue() );
 
-        // Must come after bold & italic
+        // Must come after SetBold() & SetItalic()
         if( m_fontCtrl->GetStringSelection() != INDETERMINATE_ACTION )
         {
             eda_text->SetFont( m_fontCtrl->GetFontSelection( eda_text->IsBold(),
@@ -319,10 +315,10 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aS
         }
     }
 
-    if( SCH_TEXT* sch_text = dynamic_cast<SCH_TEXT*>( aItem ) )
+    if( SCH_LABEL_BASE* sch_label = dynamic_cast<SCH_LABEL_BASE*>( aItem ) )
     {
         if( m_orientation->GetStringSelection() != INDETERMINATE_ACTION )
-            sch_text->SetTextSpinStyle( (TEXT_SPIN_STYLE::SPIN) m_orientation->GetSelection() );
+            sch_label->SetSpinStyle( (SPIN_STYLE::SPIN) m_orientation->GetSelection() );
     }
 
     if( SCH_FIELD* sch_field = dynamic_cast<SCH_FIELD*>( aItem ) )
@@ -341,9 +337,9 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aS
         if( m_lineStyle->GetStringSelection() != INDETERMINATE_ACTION )
         {
             if( m_lineStyle->GetStringSelection() == DEFAULT_STYLE )
-                stroke.SetPlotStyle( PLOT_DASH_TYPE::DEFAULT );
+                stroke.SetLineStyle( LINE_STYLE::DEFAULT );
             else
-                stroke.SetPlotStyle( (PLOT_DASH_TYPE) m_lineStyle->GetSelection() );
+                stroke.SetLineStyle( (LINE_STYLE) m_lineStyle->GetSelection() );
         }
 
         if( m_setColor->GetValue() )
@@ -375,7 +371,8 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( const SCH_SHEET_PATH& aS
     }
 }
 
-void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aSheetPath,
+void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( SCH_COMMIT* aCommit,
+                                                      const SCH_SHEET_PATH& aSheetPath,
                                                       SCH_ITEM* aItem )
 {
     if( m_netFilterOpt->GetValue() && !m_netFilter->GetValue().IsEmpty() )
@@ -422,15 +419,17 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
         }
     }
 
-    if( aItem->Type() == SCH_SYMBOL_T )
+    switch( aItem->Type() )
+    {
+    case SCH_SYMBOL_T:
     {
         SCH_SYMBOL* symbol = (SCH_SYMBOL*) aItem;
 
         if( m_references->GetValue() )
-            processItem( aSheetPath, symbol->GetField( REFERENCE_FIELD ) );
+            processItem( aCommit, aSheetPath, symbol->GetField( REFERENCE_FIELD ) );
 
         if( m_values->GetValue() )
-            processItem( aSheetPath, symbol->GetField( VALUE_FIELD ) );
+            processItem( aCommit, aSheetPath, symbol->GetField( VALUE_FIELD ) );
 
         if( m_otherFields->GetValue() )
         {
@@ -442,17 +441,20 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
                 if( !m_fieldnameFilterOpt->GetValue() || m_fieldnameFilter->GetValue().IsEmpty()
                         || WildCompareString( m_fieldnameFilter->GetValue(), fieldName, false ) )
                 {
-                    processItem( aSheetPath, &field );
+                    processItem( aCommit, aSheetPath, &field );
                 }
             }
         }
+
+        break;
     }
-    else if( aItem->Type() == SCH_SHEET_T )
+
+    case SCH_SHEET_T:
     {
         SCH_SHEET* sheet = static_cast<SCH_SHEET*>( aItem );
 
         if( m_sheetTitles->GetValue() )
-            processItem( aSheetPath, &sheet->GetFields()[SHEETNAME] );
+            processItem( aCommit, aSheetPath, &sheet->GetFields()[SHEETNAME] );
 
         if( m_sheetFields->GetValue() )
         {
@@ -466,7 +468,7 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
                 if( !m_fieldnameFilterOpt->GetValue() || m_fieldnameFilter->GetValue().IsEmpty()
                         || WildCompareString( m_fieldnameFilter->GetValue(), fieldName, false ) )
                 {
-                    processItem( aSheetPath, &field );
+                    processItem( aCommit, aSheetPath, &field );
                 }
             }
         }
@@ -486,26 +488,37 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
         if( m_sheetPins->GetValue() )
         {
             for( SCH_SHEET_PIN* pin : sheet->GetPins() )
-                processItem( aSheetPath, pin );
+                processItem( aCommit, aSheetPath, pin );
         }
+
+        break;
     }
-    else if( m_wires->GetValue() && aItem->IsType( { SCH_ITEM_LOCATE_WIRE_T,
-                                                     SCH_LABEL_LOCATE_WIRE_T } ) )
-    {
-        processItem( aSheetPath, aItem );
-    }
-    else if( m_buses->GetValue() && aItem->IsType( { SCH_ITEM_LOCATE_BUS_T,
-                                                     SCH_LABEL_LOCATE_BUS_T } ) )
-    {
-        processItem( aSheetPath, aItem );
-    }
-    else if( aItem->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
-    {
+
+    case SCH_LINE_T:
+        if( m_schTextAndGraphics->GetValue() && aItem->GetLayer() == LAYER_NOTES )
+            processItem( aCommit, aSheetPath, aItem );
+        else if( m_wires->GetValue() && aItem->GetLayer() == LAYER_WIRE )
+            processItem( aCommit, aSheetPath, aItem );
+        else if( m_buses->GetValue() && aItem->GetLayer() == LAYER_BUS )
+            processItem( aCommit, aSheetPath, aItem );
+
+        break;
+
+    case SCH_LABEL_T:
+    case SCH_GLOBAL_LABEL_T:
+    case SCH_HIER_LABEL_T:
+    case SCH_DIRECTIVE_LABEL_T:
+        if( m_wires->GetValue() && aItem->IsType( { SCH_LABEL_LOCATE_WIRE_T } ) )
+            processItem( aCommit, aSheetPath, aItem );
+
+        if( m_buses->GetValue() && aItem->IsType( { SCH_LABEL_LOCATE_BUS_T } ) )
+            processItem( aCommit, aSheetPath, aItem );
+
         if( m_globalLabels->GetValue() && aItem->Type() == SCH_GLOBAL_LABEL_T )
-            processItem( aSheetPath, aItem );
+            processItem( aCommit, aSheetPath, aItem );
 
         if( m_hierLabels->GetValue() && aItem->Type() == SCH_HIER_LABEL_T )
-            processItem( aSheetPath, aItem );
+            processItem( aCommit, aSheetPath, aItem );
 
         if( m_labelFields->GetValue() )
         {
@@ -516,12 +529,14 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
                 if( !m_fieldnameFilterOpt->GetValue() || m_fieldnameFilter->GetValue().IsEmpty()
                         || WildCompareString( m_fieldnameFilter->GetValue(), fieldName, false ) )
                 {
-                    processItem( aSheetPath, &field );
+                    processItem( aCommit, aSheetPath, &field );
                 }
             }
         }
-    }
-    else if( aItem->Type() == SCH_JUNCTION_T )
+
+        break;
+
+    case SCH_JUNCTION_T:
     {
         SCH_JUNCTION* junction = static_cast<SCH_JUNCTION*>( aItem );
 
@@ -529,21 +544,29 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( const SCH_SHEET_PATH& aShe
         {
             if( item->GetLayer() == LAYER_BUS && m_buses->GetValue() )
             {
-                processItem( aSheetPath, aItem );
+                processItem( aCommit, aSheetPath, aItem );
                 break;
             }
             else if( item->GetLayer() == LAYER_WIRE && m_wires->GetValue() )
             {
-                processItem( aSheetPath, aItem );
+                processItem( aCommit, aSheetPath, aItem );
                 break;
             }
         }
+
+        break;
     }
-    else if( m_schTextAndGraphics->GetValue() && aItem->IsType( { SCH_TEXT_T, SCH_TEXTBOX_T,
-                                                                  SCH_ITEM_LOCATE_GRAPHIC_LINE_T,
-                                                                  SCH_SHAPE_T } ) )
-    {
-        processItem( aSheetPath, aItem );
+
+    case SCH_TEXT_T:
+    case SCH_TEXTBOX_T:
+    case SCH_SHAPE_T:
+        if( m_schTextAndGraphics->GetValue() )
+            processItem( aCommit, aSheetPath, aItem );
+
+        break;
+
+    default:
+        break;
     }
 }
 
@@ -554,7 +577,7 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
         return false;
 
     SCH_SHEET_PATH currentSheet = m_parent->GetCurrentSheet();
-    m_appendUndo = false;
+    SCH_COMMIT     commit( m_parent );
 
     // Go through sheets
     for( const SCH_SHEET_PATH& sheetPath : m_parent->Schematic().GetSheets() )
@@ -566,18 +589,19 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
             m_parent->SetCurrentSheet( sheetPath );
 
             for( SCH_ITEM* item : screen->Items() )
-                visitItem( sheetPath, item );
+                visitItem( &commit, sheetPath, item );
         }
     }
 
-    if( m_appendUndo )
+    if( !commit.Empty() )
     {
-        m_parent->OnModify();
+        commit.Push( _( "Edit Text and Graphics" ) );
         m_parent->HardRedraw();
     }
 
     // Reset the view to where we left the user
     m_parent->SetCurrentSheet( currentSheet );
+    m_parent->Refresh();
 
     return true;
 }
