@@ -45,7 +45,7 @@ DIALOG_LIB_TEXT_PROPERTIES::DIALOG_LIB_TEXT_PROPERTIES( SYMBOL_EDIT_FRAME* aPare
     COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
 
     m_scintillaTricks = new SCINTILLA_TRICKS( m_StyledTextCtrl, wxT( "{}" ), false,
-            [this]()
+            [this]( wxKeyEvent& aEvent )
             {
                 wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
             } );
@@ -55,34 +55,34 @@ DIALOG_LIB_TEXT_PROPERTIES::DIALOG_LIB_TEXT_PROPERTIES( SYMBOL_EDIT_FRAME* aPare
     m_separator1->SetIsSeparator();
 
     m_horizontal->SetIsRadioButton();
-    m_horizontal->SetBitmap( KiBitmap( BITMAPS::text_horizontal ) );
+    m_horizontal->SetBitmap( KiBitmapBundle( BITMAPS::text_horizontal ) );
     m_vertical->SetIsRadioButton();
-    m_vertical->SetBitmap( KiBitmap( BITMAPS::text_vertical ) );
+    m_vertical->SetBitmap( KiBitmapBundle( BITMAPS::text_vertical ) );
 
     m_separator2->SetIsSeparator();
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator3->SetIsSeparator();
 
     m_hAlignLeft->SetIsRadioButton();
-    m_hAlignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_hAlignLeft->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
     m_hAlignCenter->SetIsRadioButton();
-    m_hAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_hAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_align_center ) );
     m_hAlignRight->SetIsRadioButton();
-    m_hAlignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+    m_hAlignRight->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
 
     m_separator4->SetIsSeparator();
 
     m_vAlignTop->SetIsRadioButton();
-    m_vAlignTop->SetBitmap( KiBitmap( BITMAPS::text_valign_top ) );
+    m_vAlignTop->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_top ) );
     m_vAlignCenter->SetIsRadioButton();
-    m_vAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_valign_center ) );
+    m_vAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_center ) );
     m_vAlignBottom->SetIsRadioButton();
-    m_vAlignBottom->SetBitmap( KiBitmap( BITMAPS::text_valign_bottom ) );
+    m_vAlignBottom->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_bottom ) );
 
     m_separator5->SetIsSeparator();
 
@@ -122,6 +122,8 @@ DIALOG_LIB_TEXT_PROPERTIES::~DIALOG_LIB_TEXT_PROPERTIES()
 
 bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataToWindow()
 {
+    wxCHECK( m_commonToAllUnits, false );
+
     LIB_SYMBOL* symbol = nullptr;
 
     if( m_graphicText )
@@ -141,9 +143,9 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataToWindow()
         m_bold->Check( m_graphicText->IsBold() );
 
         m_privateCheckbox->SetValue( m_graphicText->IsPrivate() );
-        m_CommonUnit->SetValue(
-                symbol && symbol->GetUnitCount() > 1 && m_graphicText->GetUnit() == 0 );
-        m_CommonConvert->SetValue( m_graphicText->GetConvert() == 0 );
+        m_commonToAllUnits->SetValue( symbol && symbol->GetUnitCount() > 1
+                                && m_graphicText->GetUnit() == 0 );
+        m_commonToAllBodyStyles->SetValue( m_graphicText->GetBodyStyle() == 0 );
 
         if( m_graphicText->GetTextAngle().IsHorizontal() )
             m_horizontal->Check();
@@ -170,11 +172,12 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataToWindow()
         auto* tools = m_parent->GetToolManager()->GetTool<SYMBOL_EDITOR_DRAWING_TOOLS>();
         symbol = m_parent->GetCurSymbol();
 
+        wxCHECK( cfg && symbol && tools, false );
+
         m_textSize.SetValue( schIUScale.MilsToIU( cfg->m_Defaults.text_size ) );
 
-        m_CommonUnit->SetValue(
-                symbol && symbol->GetUnitCount() > 1 && !tools->GetDrawSpecificUnit() );
-        m_CommonConvert->SetValue( !tools->GetDrawSpecificConvert() );
+        m_commonToAllUnits->SetValue( symbol->GetUnitCount() > 1 && !tools->GetDrawSpecificUnit() );
+        m_commonToAllBodyStyles->SetValue( !tools->GetDrawSpecificBodyStyle() );
 
         if( tools->GetLastTextAngle().IsHorizontal() )
             m_horizontal->Check();
@@ -182,7 +185,7 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataToWindow()
             m_vertical->Check();
     }
 
-    m_CommonUnit->Enable( symbol && symbol->GetUnitCount() > 1 );
+    m_commonToAllUnits->Enable( symbol && symbol->GetUnitCount() > 1 );
 
     return true;
 }
@@ -240,7 +243,7 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataFromWindow()
         }
 
         if( m_textSize.GetValue() != m_graphicText->GetTextWidth() )
-            m_graphicText->SetTextSize( wxSize( m_textSize.GetValue(), m_textSize.GetValue() ) );
+            m_graphicText->SetTextSize( VECTOR2I( m_textSize.GetValue(), m_textSize.GetValue() ) );
 
         if( m_horizontal->IsChecked() )
             m_graphicText->SetTextAngle( ANGLE_HORIZONTAL );
@@ -249,18 +252,19 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataFromWindow()
 
         m_graphicText->SetPrivate( m_privateCheckbox->GetValue() );
 
-        if( !m_CommonUnit->GetValue() )
+        if( !m_commonToAllUnits->GetValue() )
             m_graphicText->SetUnit( m_parent->GetUnit() );
         else
             m_graphicText->SetUnit( 0 );
 
-        if( !m_CommonConvert->GetValue() )
-            m_graphicText->SetConvert( m_parent->GetConvert() );
+        if( !m_commonToAllBodyStyles->GetValue() )
+            m_graphicText->SetBodyStyle( m_parent->GetBodyStyle() );
         else
-            m_graphicText->SetConvert( 0 );
+            m_graphicText->SetBodyStyle( 0 );
 
-        m_graphicText->SetItalic( m_italic->IsChecked() );
+        // Must come after SetTextSize()
         m_graphicText->SetBold( m_bold->IsChecked() );
+        m_graphicText->SetItalic( m_italic->IsChecked() );
         m_graphicText->SetTextColor( m_textColorSwatch->GetSwatchColor() );
 
         if( m_hAlignLeft->IsChecked() )
@@ -280,8 +284,8 @@ bool DIALOG_LIB_TEXT_PROPERTIES::TransferDataFromWindow()
         // Record settings used for next time:
         auto* tools = m_parent->GetToolManager()->GetTool<SYMBOL_EDITOR_DRAWING_TOOLS>();
         tools->SetLastTextAngle( m_graphicText->GetTextAngle() );
-        tools->SetDrawSpecificConvert( !m_CommonConvert->GetValue() );
-        tools->SetDrawSpecificUnit( !m_CommonUnit->GetValue() );
+        tools->SetDrawSpecificBodyStyle( !m_commonToAllBodyStyles->GetValue() );
+        tools->SetDrawSpecificUnit( !m_commonToAllUnits->GetValue() );
     }
 
     m_parent->SetMsgPanel( m_graphicText );
