@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2009 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2014-2023 KiCad Developers, see CHANGELOG.txt for contributors.
+ * Copyright (C) 2014-2024 KiCad Developers, see CHANGELOG.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,12 +27,14 @@
 #include <wx/string.h>
 #include <wx/log.h>
 #include <wx/tooltip.h>
+#include <common.h>
 #include <confirm.h>
 #include <validators.h>
 #include <wx_filename.h>
 #include <wildcards_and_files_ext.h>
 #include <widgets/std_bitmap_button.h>
 #include <kiplatform/ui.h>
+#include <sch_commit.h>
 #include <sch_edit_frame.h>
 #include <sch_sheet.h>
 #include <schematic.h>
@@ -42,15 +44,18 @@
 #include <trace_helpers.h>
 #include "panel_eeschema_color_settings.h"
 #include "wx/dcclient.h"
+#include "string_utils.h"
 
 DIALOG_SHEET_PROPERTIES::DIALOG_SHEET_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_SHEET* aSheet,
-                                                  bool* aClearAnnotationNewItems ) :
+                                                  bool* aClearAnnotationNewItems,
+                                                  bool* aUpdateHierarchyNavigator ) :
     DIALOG_SHEET_PROPERTIES_BASE( aParent ),
     m_frame( aParent ),
     m_clearAnnotationNewItems( aClearAnnotationNewItems ),
+    m_updateHierarchyNavigator( aUpdateHierarchyNavigator ),
     m_borderWidth( aParent, m_borderWidthLabel, m_borderWidthCtrl, m_borderWidthUnits ),
     m_dummySheet( *aSheet ),
-    m_dummySheetNameField( wxDefaultPosition, SHEETNAME, &m_dummySheet )
+    m_dummySheetNameField( VECTOR2I( -1, -1 ), SHEETNAME, &m_dummySheet )
 {
     m_sheet = aSheet;
     m_fields = new FIELDS_GRID_TABLE<SCH_FIELD>( this, aParent, m_grid, m_sheet );
@@ -87,10 +92,10 @@ DIALOG_SHEET_PROPERTIES::DIALOG_SHEET_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_S
     SetupStandardButtons();
 
     // Configure button logos
-    m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpMoveUp->SetBitmap( KiBitmap( BITMAPS::small_up ) );
-    m_bpMoveDown->SetBitmap( KiBitmap( BITMAPS::small_down ) );
+    m_bpAdd->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDelete->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpMoveUp->SetBitmap( KiBitmapBundle( BITMAPS::small_up ) );
+    m_bpMoveDown->SetBitmap( KiBitmapBundle( BITMAPS::small_down ) );
 
     // Set font sizes
     m_hierarchicalPathLabel->SetFont( KIUI::GetInfoFont( this ) );
@@ -208,9 +213,8 @@ bool DIALOG_SHEET_PROPERTIES::Validate()
     for( size_t i = SHEET_MANDATORY_FIELDS;  i < m_fields->size(); ++i )
     {
         SCH_FIELD& field = m_fields->at( i );
-        wxString   fieldName = field.GetName( false );
 
-        if( fieldName.IsEmpty() )
+        if( field.GetName( false ).empty() && !field.GetText().empty() )
         {
             DisplayErrorMessage( this, _( "Fields must have a name." ) );
 
@@ -257,9 +261,17 @@ static bool positioningChanged( FIELDS_GRID_TABLE<SCH_FIELD>* a, std::vector<SCH
 
 bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
 {
+    wxCHECK( m_sheet && m_frame, false );
+
     if( !wxDialog::TransferDataFromWindow() )  // Calls our Validate() method.
         return false;
 
+    SCH_COMMIT commit( m_frame );
+
+    commit.Modify( m_sheet, m_frame->GetScreen() );
+
+    bool isUndoable = true;
+
     // Sheet file names can be relative or absolute.
     wxString sheetFileName = m_fields->at( SHEETFILENAME ).GetText();
 
@@ -271,16 +283,19 @@ bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
         return false;
     }
 
-    // Ensure the filename extension is OK.  In normal use will be caught by grid validators,
-    // but unedited data from existing files can be bad.
-    wxFileName fn( sheetFileName );
+    // Ensure the filename extension is OK.  (In normal use will be caught by grid validators,
+    // but unedited data from existing files can be bad.)
+    sheetFileName = EnsureFileExtension( sheetFileName, FILEEXT::KiCadSchematicFileExtension );
 
-    if( fn.GetExt().CmpNoCase( KiCadSchematicFileExtension ) != 0 )
+    // Ensure sheetFileName is legal
+    if( !IsFullFileNameValid( sheetFileName ) )
     {
-        DisplayError( this, _( "Sheet file must have a '.kicad_sch' extension." ) );
+        DisplayError( this, _( "A sheet must have a valid file name." ) );
         return false;
     }
 
+    wxFileName fn( sheetFileName );
+
     wxString newRelativeFilename = fn.GetFullPath();
 
     // Inside Eeschema, filenames are stored using unix notation
@@ -333,13 +348,19 @@ bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
             }
         }
 
-        if( !onSheetFilenameChanged( newRelativeFilename ) )
+        if( !onSheetFilenameChanged( newRelativeFilename, &isUndoable ) )
         {
             if( clearFileName )
                 currentScreen->SetFileName( wxEmptyString );
+            else
+                m_fields->at( SHEETFILENAME ).SetText( oldFilename );
 
             return false;
         }
+        else if( m_updateHierarchyNavigator )
+        {
+            *m_updateHierarchyNavigator = true;
+        }
 
         if( clearFileName )
             currentScreen->SetFileName( wxEmptyString );
@@ -350,6 +371,9 @@ bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
 
     wxString newSheetname = m_fields->at( SHEETNAME ).GetText();
 
+    if( ( newSheetname != m_sheet->GetName() ) && m_updateHierarchyNavigator )
+        *m_updateHierarchyNavigator = true;
+
     if( newSheetname.IsEmpty() )
         newSheetname = _( "Untitled Sheet" );
 
@@ -367,7 +391,7 @@ bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
         SCH_FIELD&      field = m_fields->at( ii );
         const wxString& fieldName = field.GetCanonicalName();
 
-        if( fieldName.IsEmpty() && field.GetText().IsEmpty() )
+        if( field.IsEmpty() )
             m_fields->erase( m_fields->begin() + ii );
         else if( fieldName.IsEmpty() )
             field.SetName( _( "untitled" ) );
@@ -412,34 +436,35 @@ bool DIALOG_SHEET_PROPERTIES::TransferDataFromWindow()
     for( SCH_ITEM* item : m_frame->GetScreen()->Items().OfType( SCH_SHEET_T ) )
         m_frame->UpdateItem( item );
 
-    m_frame->OnModify();
+    if( isUndoable && !m_sheet->IsNew() )
+    {
+        commit.Push( _( "Edit Sheet Properties" ) );
+    }
+    else
+    {
+        if( m_updateHierarchyNavigator )
+            *m_updateHierarchyNavigator = true;
+
+        // If we are renaming files, the undo/redo list becomes invalid and must be cleared.
+        m_frame->ClearUndoRedoList();
+        m_frame->OnModify();
+    }
 
     return true;
 }
 
 
-bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilename )
+bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilename,
+                                                      bool* aIsUndoable )
 {
-    wxString msg;
+    wxCHECK( aIsUndoable, false );
+
+    wxString   msg;
+    wxFileName sheetFileName(
+            EnsureFileExtension( aNewFilename, FILEEXT::KiCadSchematicFileExtension ) );
 
     // Sheet file names are relative to the path of the current sheet.  This allows for
     // nesting of schematic files in subfolders.  Screen file names are always absolute.
-    wxFileName sheetFileName( aNewFilename );
-
-    if( sheetFileName.GetExt().IsEmpty() )
-    {
-        sheetFileName.SetExt( KiCadSchematicFileExtension );
-    }
-    else if( sheetFileName.GetExt().CmpNoCase( KiCadSchematicFileExtension ) != 0 )
-    {
-        msg = wxString::Format( _( "The file '%s' does not appear to be a valid schematic file." ),
-                                sheetFileName.GetFullName() );
-        wxMessageDialog badSchFileDialog( this, msg, _( "Invalid Schematic File" ),
-                                          wxOK | wxCENTRE | wxICON_EXCLAMATION );
-        badSchFileDialog.ShowModal();
-        return false;
-    }
-
     SCHEMATIC&                             schematic = m_frame->Schematic();
     SCH_SHEET_LIST                         fullHierarchy = schematic.GetFullHierarchy();
     wxFileName                             screenFileName( sheetFileName );
@@ -490,7 +515,7 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
 
     if( m_sheet->GetScreen() == nullptr )      // New just created sheet.
     {
-        if( !m_frame->AllowCaseSensitiveFileNameClashes( newAbsoluteFilename ) )
+        if( !m_frame->AllowCaseSensitiveFileNameClashes( m_sheet->GetFileName(), newAbsoluteFilename ) )
             return false;
 
         if( useScreen || loadFromFile )     // Load from existing file.
@@ -513,10 +538,9 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
     }
     else                                    // Existing sheet.
     {
-        bool isUndoable = true;
         isExistingSheet = true;
 
-        if( !m_frame->AllowCaseSensitiveFileNameClashes( newAbsoluteFilename ) )
+        if( !m_frame->AllowCaseSensitiveFileNameClashes( m_sheet->GetFileName(), newAbsoluteFilename ) )
             return false;
 
         // We are always using here a case insensitive comparison to avoid issues
@@ -534,11 +558,12 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
         if( newAbsoluteFilename.Cmp( oldAbsoluteFilename ) != 0 )
         {
             // Sheet file name changes cannot be undone.
-            isUndoable = false;
+            *aIsUndoable = false;
 
             if( useScreen || loadFromFile )           // Load from existing file.
             {
                 clearAnnotation = true;
+                oldScreen = m_sheet->GetScreen();
 
                 if( !IsOK( this, wxString::Format( _( "Change '%s' link from '%s' to '%s'?" ),
                                                    newAbsoluteFilename,
@@ -572,15 +597,9 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
             }
         }
 
-        // If we are renaming files, the undo/redo list becomes invalid and must be cleared
-        if( isUndoable )
-            m_frame->SaveCopyInUndoList( m_frame->GetScreen(), m_sheet, UNDO_REDO::CHANGED, false );
-        else
-            m_frame->ClearUndoRedoList();
-
         if( renameFile )
         {
-            SCH_PLUGIN::SCH_PLUGIN_RELEASER pi( SCH_IO_MGR::FindPlugin( SCH_IO_MGR::SCH_KICAD ) );
+            IO_RELEASER<SCH_IO> pi( SCH_IO_MGR::FindPlugin( SCH_IO_MGR::SCH_KICAD ) );
 
             // If the associated screen is shared by more than one sheet, do not
             // change the filename of the corresponding screen here.
@@ -591,7 +610,7 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
 
             try
             {
-                pi->Save( newAbsoluteFilename, m_sheet, &schematic );
+                pi->SaveSchematicFile( newAbsoluteFilename, m_sheet, &schematic );
             }
             catch( const IO_ERROR& ioe )
             {
@@ -633,6 +652,12 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
 
         // It's safe to set the sheet screen now.
         m_sheet->SetScreen( useScreen );
+
+        SCH_SHEET_LIST sheetHierarchy( m_sheet );  // The hierarchy of the loaded file.
+
+        sheetHierarchy.AddNewSymbolInstances( currentSheet, m_frame->Prj().GetProjectName() );
+        sheetHierarchy.AddNewSheetInstances( currentSheet,
+                                             fullHierarchy.GetLastVirtualPageNumber() );
     }
     else if( loadFromFile )
     {
@@ -668,6 +693,9 @@ bool DIALOG_SHEET_PROPERTIES::onSheetFilenameChanged( const wxString& aNewFilena
     if( m_clearAnnotationNewItems )
         *m_clearAnnotationNewItems = clearAnnotation;
 
+    // Rebuild the entire connection graph.
+    m_frame->RecalculateConnections( nullptr, GLOBAL_CLEANUP );
+
     return true;
 }
 
@@ -718,7 +746,7 @@ void DIALOG_SHEET_PROPERTIES::OnAddField( wxCommandEvent& event )
         return;
 
     int       fieldID = m_fields->size();
-    SCH_FIELD newField( wxPoint( 0, 0 ), fieldID, m_sheet,
+    SCH_FIELD newField( VECTOR2I( 0, 0 ), fieldID, m_sheet,
                         SCH_SHEET::GetDefaultFieldName( fieldID ) );
 
     newField.SetTextAngle( m_fields->at( SHEETNAME ).GetTextAngle() );
@@ -874,7 +902,7 @@ void DIALOG_SHEET_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
 
     m_dummySheet.SetFields( *m_fields );
     m_dummySheetNameField.SetText( sheetName );
-    path += m_dummySheetNameField.GetShownText();
+    path += m_dummySheetNameField.GetShownText( false );
 
     editor->DecRef();
 
