@@ -25,11 +25,10 @@
 
 #include <sim/kibis/kibis.h>
 #include <netlist_exporter_spice.h>
-#include <sim/ngspice_circuit_model.h>
+#include <sim/spice_circuit_model.h>
 #include <sim/sim_library_spice.h>
 #include <sim/sim_model_raw_spice.h>
 #include <sim/sim_model_ideal.h>
-#include <sim/spice_grammar.h>
 #include <common.h>
 #include <confirm.h>
 #include <pgm_base.h>
@@ -37,16 +36,17 @@
 #include <sim/sim_library.h>
 #include <sim/sim_library_kibis.h>
 #include <sim/sim_model_kibis.h>
-#include <sim/sim_model.h>
 #include <sch_screen.h>
 #include <sch_text.h>
 #include <sch_textbox.h>
 #include <string_utils.h>
+#include <ki_exception.h>
 
 #include <dialogs/html_message_box.h>
 #include <fmt/core.h>
 #include <paths.h>
 #include <wx/dir.h>
+#include <wx/log.h>
 #include <locale_io.h>
 #include "markup_parser.h"
 
@@ -105,34 +105,33 @@ NETLIST_EXPORTER_SPICE::NETLIST_EXPORTER_SPICE( SCHEMATIC_IFACE* aSchematic,
 bool NETLIST_EXPORTER_SPICE::WriteNetlist( const wxString& aOutFileName, unsigned aNetlistOptions,
                                            REPORTER& aReporter )
 {
-    m_libMgr.SetReporter( &aReporter );
     FILE_OUTPUTFORMATTER formatter( aOutFileName, wxT( "wt" ), '\'' );
-    return DoWriteNetlist( formatter, aNetlistOptions, aReporter );
+    return DoWriteNetlist( wxEmptyString, aNetlistOptions, formatter, aReporter );
 }
 
 
-bool NETLIST_EXPORTER_SPICE::DoWriteNetlist( OUTPUTFORMATTER& aFormatter, unsigned aNetlistOptions,
-                                             REPORTER& aReporter )
+bool NETLIST_EXPORTER_SPICE::DoWriteNetlist( const wxString& aSimCommand, unsigned aSimOptions,
+                                             OUTPUTFORMATTER& aFormatter, REPORTER& aReporter )
 {
     LOCALE_IO dummy;
 
    // Cleanup list to avoid duplicate if the netlist exporter is run more than once.
     m_rawIncludes.clear();
 
-    bool result = ReadSchematicAndLibraries( aNetlistOptions, aReporter );
+    bool result = ReadSchematicAndLibraries( aSimOptions, aReporter );
 
-    WriteHead( aFormatter, aNetlistOptions );
+    WriteHead( aFormatter, aSimOptions );
 
-    writeIncludes( aFormatter, aNetlistOptions );
+    writeIncludes( aFormatter, aSimOptions );
     writeModels( aFormatter );
 
     // Skip this if there is no netlist to avoid an ngspice segfault
     if( !m_items.empty() )
-        WriteDirectives( aFormatter, aNetlistOptions );
+        WriteDirectives( aSimCommand, aSimOptions, aFormatter );
 
     writeItems( aFormatter );
 
-    WriteTail( aFormatter, aNetlistOptions );
+    WriteTail( aFormatter, aSimOptions );
 
     return result;
 }
@@ -157,8 +156,6 @@ bool NETLIST_EXPORTER_SPICE::ReadSchematicAndLibraries( unsigned aNetlistOptions
     std::set<std::string> refNames; // Set of reference names to check for duplication.
     int                   ncCounter = 1;
 
-    m_libMgr.SetReporter( &aReporter );
-
     ReadDirectives( aNetlistOptions );
 
     m_nets.clear();
@@ -211,43 +208,41 @@ bool NETLIST_EXPORTER_SPICE::ReadSchematicAndLibraries( unsigned aNetlistOptions
         {
             SCH_SYMBOL* symbol = findNextSymbol( item, &sheet );
 
-            if( !symbol || symbol->GetFieldText( SIM_ENABLE_FIELD ) == wxT( "0" ) )
+            if( !symbol || symbol->GetExcludedFromSim() )
                 continue;
 
-            std::vector<PIN_INFO> pins = CreatePinList( symbol, &sheet, true );
-
-            SPICE_ITEM spiceItem;
-
-            for( int i = 0; i < symbol->GetFieldCount(); ++i )
+            try
             {
-                spiceItem.fields.emplace_back( VECTOR2I(), i, symbol,
-                                               symbol->GetFields()[ i ].GetName() );
+                SPICE_ITEM            spiceItem;
+                std::vector<PIN_INFO> pins = CreatePinList( symbol, &sheet, true );
 
-                if( i == REFERENCE_FIELD )
-                    spiceItem.fields.back().SetText( symbol->GetRef( &sheet ) );
-                else
-                    spiceItem.fields.back().SetText( symbol->GetFields()[i].GetShownText( 0, false ) );
+                for( const SCH_FIELD& field : symbol->GetFields() )
+                {
+                    spiceItem.fields.emplace_back( VECTOR2I(), -1, symbol, field.GetName() );
+
+                    if( field.GetId() == REFERENCE_FIELD )
+                        spiceItem.fields.back().SetText( symbol->GetRef( &sheet ) );
+                    else
+                        spiceItem.fields.back().SetText( field.GetShownText( &sheet, false ) );
+                }
+
+                readRefName( sheet, *symbol, spiceItem, refNames );
+                readModel( sheet, *symbol, spiceItem, aReporter );
+                readPinNumbers( *symbol, spiceItem, pins );
+                readPinNetNames( *symbol, spiceItem, pins, ncCounter );
+
+                // TODO: transmission line handling?
+
+                m_items.push_back( std::move( spiceItem ) );
+            }
+            catch( IO_ERROR& e )
+            {
+                aReporter.Report( e.What(), RPT_SEVERITY_ERROR );
             }
-
-            wxString deviceType;
-            wxString modelType;
-            wxString modelParams;
-            wxString pinMap;
-
-            readRefName( sheet, *symbol, spiceItem, refNames );
-            readModel( sheet, *symbol, spiceItem );
-            readPinNumbers( *symbol, spiceItem, pins );
-            readPinNetNames( *symbol, spiceItem, pins, ncCounter );
-
-            // TODO: transmission line handling?
-
-            m_items.push_back( std::move( spiceItem ) );
         }
     }
 
-    m_libMgr.SetReporter( nullptr );
-
-    return !aReporter.HasMessage();
+    return !aReporter.HasMessageOfSeverity( RPT_SEVERITY_UNDEFINED | RPT_SEVERITY_ERROR );
 }
 
 
@@ -285,7 +280,7 @@ void NETLIST_EXPORTER_SPICE::ConvertToSpiceMarkup( std::string& aNetName )
 
     convertMarkup( root );
 
-    // Remove ngspice-disallowed chars
+    // Replace all ngspice-disallowed chars in netnames by a '_'
     std::replace( converted.begin(), converted.end(), '%', '_' );
     std::replace( converted.begin(), converted.end(), '(', '_' );
     std::replace( converted.begin(), converted.end(), ')', '_' );
@@ -295,6 +290,7 @@ void NETLIST_EXPORTER_SPICE::ConvertToSpiceMarkup( std::string& aNetName )
     std::replace( converted.begin(), converted.end(), '<', '_' );
     std::replace( converted.begin(), converted.end(), '>', '_' );
     std::replace( converted.begin(), converted.end(), '~', '_' );
+    std::replace( converted.begin(), converted.end(), ' ', '_' );
 
     aNetName = converted;
 }
@@ -339,10 +335,13 @@ void NETLIST_EXPORTER_SPICE::ReadDirectives( unsigned aNetlistOptions )
     {
         for( SCH_ITEM* item : sheet.LastScreen()->Items() )
         {
+            if( item->GetExcludedFromSim() )
+                continue;
+
             if( item->Type() == SCH_TEXT_T )
-                text = static_cast<SCH_TEXT*>( item )->GetShownText();
+                text = static_cast<SCH_TEXT*>( item )->GetShownText( &sheet, false );
             else if( item->Type() == SCH_TEXTBOX_T )
-                text = static_cast<SCH_TEXTBOX*>( item )->GetShownText();
+                text = static_cast<SCH_TEXTBOX*>( item )->GetShownText( &sheet, false );
             else
                 continue;
 
@@ -450,9 +449,9 @@ void NETLIST_EXPORTER_SPICE::readRefName( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aS
 
 
 void NETLIST_EXPORTER_SPICE::readModel( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aSymbol,
-                                        SPICE_ITEM& aItem )
+                                        SPICE_ITEM& aItem, REPORTER& aReporter )
 {
-    SIM_LIBRARY::MODEL libModel = m_libMgr.CreateModel( &aSheet, aSymbol );
+    SIM_LIBRARY::MODEL libModel = m_libMgr.CreateModel( &aSheet, aSymbol, aReporter );
 
     aItem.baseModelName = libModel.name;
     aItem.model = &libModel.model;
@@ -481,15 +480,15 @@ void NETLIST_EXPORTER_SPICE::readModel( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aSym
 
         if( !cacheFile.IsOpened() )
         {
-            DisplayErrorMessage( m_dialogParent,
-                                 wxString::Format( _( "Could not open file '%s' to write "
-                                                      "IBIS model" ),
-                                                   cacheFn.GetFullPath() ) );
+            DisplayErrorMessage( m_dialogParent, wxString::Format( _( "Could not open file '%s' "
+                                                                      "to write IBIS model" ),
+                                                                   cacheFn.GetFullPath() ) );
         }
 
         auto spiceGenerator = static_cast<const SPICE_GENERATOR_KIBIS&>( kibisModel->SpiceGenerator() );
-        std::string modelData = spiceGenerator.IbisDevice( aItem, m_schematic->Prj(),
-                                                           cacheFn.GetPath( wxPATH_GET_SEPARATOR ) );
+        std::string modelData = spiceGenerator.IbisDevice(
+                aItem, m_schematic->Prj(),
+                cacheFn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR ), aReporter );
 
         cacheFile.Write( wxString( modelData ) );
         m_rawIncludes.insert( cacheFn.GetFullPath() );
@@ -536,9 +535,9 @@ void NETLIST_EXPORTER_SPICE::writeInclude( OUTPUTFORMATTER& aFormatter, unsigned
 
         if( fullPath.IsEmpty() )
         {
-            DisplayErrorMessage( m_dialogParent,
-                                 wxString::Format( _( "Could not find library file '%s'" ),
-                                                   expandedPath ) );
+            DisplayErrorMessage( m_dialogParent, wxString::Format( _( "Could not find library file "
+                                                                      "'%s'" ),
+                                                                   expandedPath ) );
             fullPath = expandedPath;
         }
         else if( wxFileName::GetPathSeparator() == '\\' )
@@ -593,15 +592,31 @@ void NETLIST_EXPORTER_SPICE::writeItems( OUTPUTFORMATTER& aFormatter )
 }
 
 
-void NETLIST_EXPORTER_SPICE::WriteDirectives( OUTPUTFORMATTER& aFormatter,
-                                              unsigned aNetlistOptions ) const
+void NETLIST_EXPORTER_SPICE::WriteDirectives( const wxString& aSimCommand, unsigned aSimOptions,
+                                              OUTPUTFORMATTER& aFormatter ) const
 {
-    if( aNetlistOptions & OPTION_SAVE_ALL_VOLTAGES )
+    if( aSimOptions & OPTION_SAVE_ALL_VOLTAGES )
         aFormatter.Print( 0, ".save all\n" );
 
-    if( aNetlistOptions & OPTION_SAVE_ALL_CURRENTS )
+    if( aSimOptions & OPTION_SAVE_ALL_CURRENTS )
         aFormatter.Print( 0, ".probe alli\n" );
 
+    if( aSimOptions & OPTION_SAVE_ALL_DISSIPATIONS )
+    {
+        for( const SPICE_ITEM& item : m_items )
+        {
+            // ngspice (v39) does not support power measurement for XSPICE devices
+            // XPSICE devices are marked with 'A'
+            std::string itemName = item.model->SpiceGenerator().ItemName( item );
+
+            if( ( item.model->GetPinCount() >= 2 ) && ( itemName.size() > 0 )
+                && ( itemName.c_str()[0] != 'A' ) )
+            {
+                aFormatter.Print( 0, ".probe p(%s)\n", itemName.c_str() );
+            }
+        }
+    }
+
     auto isSimCommand =
             []( const wxString& candidate, const wxString& dir )
             {
@@ -627,7 +642,7 @@ void NETLIST_EXPORTER_SPICE::WriteDirectives( OUTPUTFORMATTER& aFormatter,
                         || isSimCommand( candidate, wxS( ".TF" ) ) );
         }
 
-        if( !simCommand || ( aNetlistOptions & OPTION_SIM_COMMAND ) )
+        if( !simCommand || ( aSimOptions & OPTION_SIM_COMMAND ) )
             aFormatter.Print( 0, "%s\n", UTF8( directive ).c_str() );
     }
 }
