@@ -3,7 +3,8 @@
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN (www.cern.ch)
+ * Copyright (C) 2004-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,9 +33,9 @@
 #include <wx/event.h>
 #include <wx/gdicmn.h>
 #include <wx/string.h>
+#include <wx/treectrl.h>
 #include <wx/utils.h>
 
-#include <config_params.h>
 #include <core/typeinfo.h>
 #include <eda_base_frame.h>
 #include <eeschema_settings.h>
@@ -54,26 +55,14 @@ class SCH_SYMBOL;
 class SCH_FIELD;
 class SCH_JUNCTION;
 class SCHEMATIC;
+class SCH_COMMIT;
+class DIALOG_BOOK_REPORTER;
+class DIALOG_ERC;
+class DIALOG_SYMBOL_FIELDS_TABLE;
 class DIALOG_SCH_FIND;
-class wxFindReplaceData;
 class RESCUER;
 class HIERARCHY_PANE;
 
-// @todo Move this to transform alone with all of the transform manipulation code.
-/// enum used in RotationMiroir()
-enum SYMBOL_ORIENTATION_T
-{
-    SYM_NORMAL,                     // Normal orientation, no rotation or mirror
-    SYM_ROTATE_CLOCKWISE,           // Rotate -90
-    SYM_ROTATE_COUNTERCLOCKWISE,    // Rotate +90
-    SYM_ORIENT_0,                   // No rotation and no mirror id SYM_NORMAL
-    SYM_ORIENT_90,                  // Rotate 90, no mirror
-    SYM_ORIENT_180,                 // Rotate 180, no mirror
-    SYM_ORIENT_270,                 // Rotate -90, no mirror
-    SYM_MIRROR_X = 0x100,           // Mirror around X axis
-    SYM_MIRROR_Y = 0x200            // Mirror around Y axis
-};
-
 
 /// Schematic search type used by the socket link with Pcbnew
 enum SCH_SEARCH_T
@@ -91,6 +80,52 @@ enum SCH_CLEANUP_FLAGS
 };
 
 
+wxDECLARE_EVENT( EDA_EVT_SCHEMATIC_CHANGING, wxCommandEvent );
+wxDECLARE_EVENT( EDA_EVT_SCHEMATIC_CHANGED, wxCommandEvent );
+
+
+/**
+ * Tree view item data for the net navigator.
+ */
+class NET_NAVIGATOR_ITEM_DATA : public wxTreeItemData
+{
+public:
+    NET_NAVIGATOR_ITEM_DATA( const SCH_SHEET_PATH& aSheetPath, const SCH_ITEM* aItem ) :
+        m_sheetPath( aSheetPath ),
+        m_item( aItem )
+    {
+    }
+
+    NET_NAVIGATOR_ITEM_DATA() :
+        m_item( nullptr )
+    {
+    }
+
+    SCH_SHEET_PATH& GetSheetPath() { return m_sheetPath; }
+    const SCH_ITEM* GetItem() const { return m_item; }
+
+    bool operator==( const NET_NAVIGATOR_ITEM_DATA& aRhs ) const
+    {
+        return ( m_sheetPath == aRhs.m_sheetPath ) && ( m_item == aRhs.m_item );
+    }
+
+    NET_NAVIGATOR_ITEM_DATA& operator=( const NET_NAVIGATOR_ITEM_DATA& aItemData )
+    {
+        if( this == &aItemData )
+            return *this;
+
+        m_sheetPath = aItemData.m_sheetPath;
+        m_item = aItemData.m_item;
+
+        return *this;
+    }
+
+private:
+    SCH_SHEET_PATH m_sheetPath;
+    const SCH_ITEM* m_item;
+};
+
+
 /**
  * Schematic editor (Eeschema) main window.
  */
@@ -148,16 +183,6 @@ public:
      */
     void OnModify() override;
 
-    /**
-     * Scan existing markers and record data from any that are Excluded.
-     */
-    void RecordERCExclusions();
-
-    /**
-     * Update markers to match recorded exclusions.
-     */
-    void ResolveERCExclusions();
-
     SEVERITY GetSeverity( int aErrorCode ) const override;
 
     /**
@@ -167,11 +192,11 @@ public:
     wxString GetFullScreenDesc() const override;
 
     /**
-     * Execute a remote command sent by Pcbnew via a socket connection.
+     * Execute a remote command sent via a socket on port KICAD_SCH_PORT_SERVICE_NUMBER (which
+     * defaults to 4243).
      *
-     * When user selects a footprint or pin in Pcbnew, Eeschema shows that same
-     * symbol or pin and moves cursor on the item.  The socket port used
-     * is #KICAD_SCH_PORT_SERVICE_NUMBER which defaults to 4243.
+     * When user selects a footprint or pin in Pcbnew, Eeschema shows that same symbol or pin
+     * and moves cursor on the item.
      *
      * Valid commands are:
      * \li \c \$PART: \c "reference" Put cursor on symbol.
@@ -188,17 +213,16 @@ public:
 
     void KiwayMailIn( KIWAY_EXPRESS& aEvent ) override;
 
+    /**
+     * Refresh the display of any operaintg points.  Called after a .op simulation completes.
+     */
+    void RefreshOperatingPointDisplay();
+
     /**
      * Automatically set the rotation of an item (if the item supports it)
      */
     void AutoRotateItem( SCH_SCREEN* aScreen, SCH_ITEM* aItem );
 
-    /**
-     * Add an item to the schematic and adds the changes to the undo/redo container.
-     * @param aUndoAppend True if the action should be appended to the current undo record.
-     */
-    void AddItemToScreenAndUndoList( SCH_SCREEN* aScreen, SCH_ITEM* aItem, bool aUndoAppend );
-
     /**
      * Run the Find or Find & Replace dialog.
      */
@@ -209,6 +233,13 @@ public:
      */
     void UpdateHierarchyNavigator();
 
+    /**
+     * Update the hierarchy navigation tree labels.
+     * No change for the tree, only the labels are updated, after editing a sheet
+     * name or a sheet number.
+     */
+    void UpdateLabelsHierarchyNavigator();
+
     /**
      * Update the hierarchy navigation tree selection (cross-probe from schematic to hierarchy
      * pane).
@@ -225,39 +256,33 @@ public:
 
     /**
      * Break a single segment into two at the specified point.
-     *
-     * @note This always appends to the existing undo state.
-     *
+     * @param aCommit Transaction container used to record changes for undo/redo
      * @param aSegment Line segment to break
      * @param aPoint Point at which to break the segment
-     * @param aNewSegment Pointer to the newly created segment (if given and created)
-     * @param aScreen is the screen to examine, or nullptr to examine the current screen.
+     * @param aNewSegment Pointer to the newly created segment (if created)
+     * @param aScreen is the screen to examine
      */
-    void BreakSegment( SCH_LINE* aSegment, const VECTOR2I& aPoint,
-                       SCH_LINE** aNewSegment = nullptr, SCH_SCREEN* aScreen = nullptr );
+    void BreakSegment( SCH_COMMIT* aCommit, SCH_LINE* aSegment, const VECTOR2I& aPoint,
+                       SCH_LINE** aNewSegment, SCH_SCREEN* aScreen );
 
     /**
      * Check every wire and bus for a intersection at \a aPoint and break into two segments
      * at \a aPoint if an intersection is found.
-     *
-     * @note This always appends to the existing undo state.
-     *
+     * @param aCommit Transaction container used to record changes for undo/redo
      * @param aPoint Test this point for an intersection.
-     * @param aScreen is the screen to examine, or nullptr to examine the current screen.
+     * @param aScreen is the screen to examine.
      * @return True if any wires or buses were broken.
      */
-    bool BreakSegments( const VECTOR2I& aPoint, SCH_SCREEN* aScreen = nullptr );
+    bool BreakSegments( SCH_COMMIT* aCommit, const VECTOR2I& aPoint, SCH_SCREEN* aScreen );
 
     /**
      * Test all junctions and bus entries in the schematic for intersections with wires and
      * buses and breaks any intersections into multiple segments.
-     *
-     * @note This always appends to the existing undo state.
-     *
-     * @param aScreen is the screen to examine, or nullptr to examine the current screen
+     * @param aCommit Transaction container used to record changes for undo/redo
+     * @param aScreen is the screen to examine.
      * @return True if any wires or buses were broken.
      */
-    bool BreakSegmentsOnJunctions( SCH_SCREEN* aScreen = nullptr );
+    bool BreakSegmentsOnJunctions( SCH_COMMIT* aCommit, SCH_SCREEN* aScreen );
 
     /**
      * Test all of the connectable objects in the schematic for unused connection points.
@@ -266,16 +291,18 @@ public:
     void TestDanglingEnds();
 
     /**
-     * Sends items to Pcbnew for selection
+     * Send items to board editor for selection.
+     *
+     * This is used for when the eeschema user is using the cross-probe tool.
      *
      * @param aItems are the items to select
-     * @param aForce select the element in pcbnew whether or not the user has the select option chosen
-     * This is used for when the eeschema user is using the cross-probe tool
+     * @param aForce select the element in pcbnew whether or not the user has the select option
+     *        chosen
      */
     void SendSelectItemsToPcb( const std::vector<EDA_ITEM*>& aItems, bool aForce );
 
     /**
-     * Sends a net name to Pcbnew for highlighting
+     * Send a net name to Pcbnew for highlighting.
      *
      * @param aNetName is the name of a net, or empty string to clear highlight
      */
@@ -293,15 +320,13 @@ public:
      */
     void SendCrossProbeClearHighlight();
 
-    const SCH_CONNECTION* GetHighlightedConnection() const
+    const wxString& GetHighlightedConnection() const
     {
         return m_highlightedConn;
     }
 
-    void SetHighlightedConnection( const SCH_CONNECTION* aConnection )
-    {
-        m_highlightedConn = aConnection;
-    }
+    void SetHighlightedConnection( const wxString& aConnection,
+                                   const NET_NAVIGATOR_ITEM_DATA* aSelection = nullptr );
 
     /**
      * Check if we are ready to write a netlist file for the current schematic.
@@ -335,15 +360,14 @@ public:
      * Clear the current symbol annotation.
      *
      * @param aCurrentSheetOnly Where to clear the annotation. See #ANNOTATE_SCOPE_T
-     * @param appendUndo true to add the action to the previous undo list
      */
-    void DeleteAnnotation( ANNOTATE_SCOPE_T aAnnotateScope, bool aRecursive, bool* appendUndo );
+    void DeleteAnnotation( ANNOTATE_SCOPE_T aAnnotateScope, bool aRecursive );
 
     /**
      * Annotate the symbols in the schematic that are not currently annotated. Multi-unit symbols
      * are annotated together. E.g. if two symbols were R8A and R8B, they may become R3A and
      * R3B, but not R3A and R3C or R3C and R4D.
-     *
+     * @param aCommit Transaction container used to record changes for undo/redo
      * @param aAnnotateScope See #ANNOTATE_SCOPE_T
      * @param aSortOption Define the annotation order.  See #ANNOTATE_ORDER_T.
      * @param aAlgoOption Define the annotation style.  See #ANNOTATE_ALGO_T.
@@ -357,17 +381,14 @@ public:
      *                          used to handle annotation in complex hierarchies.
      * @param aReporter A sink for error messages.  Use NULL_REPORTER if you don't need errors.
      *
-     * @param appendUndo True if the annotation operation should be added to an existing undo,
-     *                   false if it should be separately undo-able.
-     *
      * When the sheet number is used in annotation, each sheet annotation starts from sheet
      * number * 100.  In other words the first sheet uses 100 to 199, the second sheet uses
      * 200 to 299, and so on.
      */
-    void AnnotateSymbols( ANNOTATE_SCOPE_T aAnnotateScope, ANNOTATE_ORDER_T aSortOption,
-                          ANNOTATE_ALGO_T aAlgoOption, bool aRecursive, int aStartNumber,
-                          bool aResetAnnotation, bool aRepairTimestamps, REPORTER& aReporter,
-                          bool appendUndo = false );
+    void AnnotateSymbols( SCH_COMMIT* aCommit, ANNOTATE_SCOPE_T aAnnotateScope,
+                          ANNOTATE_ORDER_T aSortOption, ANNOTATE_ALGO_T aAlgoOption,
+                          bool aRecursive, int aStartNumber, bool aResetAnnotation,
+                          bool aRepairTimestamps, REPORTER& aReporter );
 
     /**
      * Check for annotation errors.
@@ -479,31 +500,25 @@ public:
      */
     bool AskToSaveChanges();
 
-    SCH_JUNCTION* AddJunction( SCH_SCREEN* aScreen, const VECTOR2I& aPos, bool aAppendToUndo,
-                               bool aFinal = true );
+    SCH_JUNCTION* AddJunction( SCH_COMMIT* aCommit, SCH_SCREEN* aScreen, const VECTOR2I& aPos );
 
     /**
      * Perform routine schematic cleaning including breaking wire and buses and deleting
      * identical objects superimposed on top of each other.
-     *
-     * @note This always appends to the existing undo state.
-     *
+     * @param aCommit Transaction container used to record changes for undo/redo
      * @param aScreen is the screen to examine, or nullptr to examine the current screen
-     * @return True if any schematic clean up was performed.
      */
-    bool SchematicCleanUp( SCH_SCREEN* aScreen = nullptr );
+    void SchematicCleanUp( SCH_COMMIT* aCommit, SCH_SCREEN* aScreen = nullptr );
 
     /**
      * If any single wire passes through _both points_, remove the portion between the two points,
      * potentially splitting the wire into two.
-     *
-     * @note This always appends to the existing undo state.
-     *
+     * @param aCommit Transaction container used to record changes for undo/redo
      * @param aStart The starting point for trimmming
      * @param aEnd The ending point for trimming
      * @return True if any wires were changed by this operation
      */
-    bool TrimWire( const VECTOR2I& aStart, const VECTOR2I& aEnd );
+    bool TrimWire( SCH_COMMIT* aCommit, const VECTOR2I& aStart, const VECTOR2I& aEnd );
 
     void OnOpenPcbnew( wxCommandEvent& event );
     void OnOpenCvpcb( wxCommandEvent& event );
@@ -511,14 +526,13 @@ public:
     void OnAnnotate( wxCommandEvent& event );
 
     /**
-     * Verify that \a aSheet will not cause a recursion error in \a aHierarchy.
+     * Verify that \a aSheet will not cause a recursion error in \a aCurrentSheet.
      *
      * @param aSheet is the #SCH_SHEET object to test.
-     * @param aHierarchy is the #SCH_SHEET_PATH where \a aSheet is going to reside.
-     *
-     * @return true if \a aSheet will cause a recursion error in \a aHierarchy.
+     * @param aCurrentSheet is the #SCH_SHEET_PATH where \a aSheet is going to reside.
+     * @return true if \a aSheet will cause a recursion error in \a aCurrentSheet.
      */
-    bool CheckSheetForRecursion( SCH_SHEET* aSheet, SCH_SHEET_PATH* aHierarchy );
+    bool CheckSheetForRecursion( SCH_SHEET* aSheet, SCH_SHEET_PATH* aCurrentSheet );
 
     /**
      * Check \a aSchematicFileName for a potential file name case sensitivity clashes.
@@ -531,7 +545,7 @@ public:
      * @param aSchematicFileName is the absolute path and file name of the file to test.
      * @return true if the user accepts the potential file name clash risk.
      */
-    bool AllowCaseSensitiveFileNameClashes( const wxString& aSchematicFileName );
+    bool AllowCaseSensitiveFileNameClashes( const wxString& aOldName, const wxString& aSchematicFileName );
 
     /**
      * Edit an existing sheet or add a new sheet to the schematic.
@@ -560,9 +574,12 @@ public:
      * this sheet need to have their annotation cleared i.e. when an existing item list is used.
      * it can happens when the edited sheet used an existing file, or becomes a new instance
      * of a already existing sheet.
+     * @param aUpdateHierarchyNavigator is an optional flag to indicate the sheet changes require
+     *                                  the hierarchy navigator panel to be updated.
      */
     bool EditSheetProperties( SCH_SHEET* aSheet, SCH_SHEET_PATH* aHierarchy,
-                              bool* aClearAnnotationNewItems );
+                              bool* aClearAnnotationNewItems,
+                              bool* aUpdateHierarchyNavigator = nullptr );
 
     void InitSheet( SCH_SHEET* aSheet, const wxString& aNewFilename );
 
@@ -603,34 +620,30 @@ public:
      * - Refresh the symbol library links.
      *
      * @param aSheet is the sheet to either append or load the schematic.
-     * @param aHierarchy is the current position in the schematic hierarchy used to test for
-     *                   possible file recursion issues.
+     * @param aCurrentSheet is the current position in the schematic hierarchy used to test for
+     *                      possible file recursion issues.
      * @param aFileName is the file name to load.  The file name is expected to have an absolute
      *                  path.
      *
      * @return True if the schematic was imported properly.
      */
-    bool LoadSheetFromFile( SCH_SHEET* aSheet, SCH_SHEET_PATH* aHierarchy,
+    bool LoadSheetFromFile( SCH_SHEET* aSheet, SCH_SHEET_PATH* aCurrentSheet,
                             const wxString& aFileName );
 
     /**
      * Removes a given junction and heals any wire segments under the junction
      *
      * @param aItem The junction to delete
-     * @param aAppend True if we are updating an ongoing commit
      */
-    void DeleteJunction( SCH_ITEM* aItem, bool aAppend = false );
+    void DeleteJunction( SCH_COMMIT* aCommit, SCH_ITEM* aItem );
 
-    void ConvertPart( SCH_SYMBOL* aSymbol );
+    void FlipBodyStyle( SCH_SYMBOL* aSymbol );
 
     void SelectUnit( SCH_SYMBOL* aSymbol, int aUnit );
 
-    /* Undo - redo */
+    void SetAltPinFunction( SCH_PIN* aPin, const wxString& aFunction );
 
-    /**
-     * Create a new, blank stack for future Undo commands to be pushed to
-     */
-    void StartNewUndo();
+    /* Undo - redo */
 
     /**
      * Create a copy of the current schematic item, and put it in the undo list.
@@ -697,6 +710,15 @@ public:
         return m_items_to_repeat;
     }
 
+    /**
+     * Clear the list of items which are to be repeated with the insert key.
+     * These objects are owned by this container.
+     */
+    void ClearRepeatItemsList()
+    {
+        m_items_to_repeat.clear();
+    }
+
     EDA_ITEM* GetItem( const KIID& aId ) const override;
 
     /**
@@ -705,17 +727,6 @@ public:
      */
     void RollbackSchematicFromUndo();
 
-    /**
-     * Create a symbol library file with the name of the root document plus the '-cache' suffix,
-     *
-     * This file will contain all symbols used in the current schematic.
-     *
-     * @param aUseCurrentSheetFilename set to false to use the root sheet filename
-     *                                 (default) or true to use the currently opened sheet.
-     * @return true if the file was written successfully.
-     */
-    bool CreateArchiveLibraryCacheFile( bool aUseCurrentSheetFilename = false );
-
     /**
      * Create a library \a aFileName that contains all symbols used in the current schematic.
      *
@@ -756,7 +767,7 @@ public:
     /**
      * Generate the connection data for the entire schematic hierarchy.
      */
-    void RecalculateConnections( SCH_CLEANUP_FLAGS aCleanupFlags );
+    void RecalculateConnections( SCH_COMMIT* aCommit, SCH_CLEANUP_FLAGS aCleanupFlags );
 
     /**
      * Called after the preferences dialog is run.
@@ -803,12 +814,30 @@ public:
      */
     virtual void CheckForAutoSaveFile( const wxFileName& aFileName ) override;
 
+    virtual void DeleteAutoSaveFile( const wxFileName& aFileName ) override;
 
     /**
      * Toggle the show/hide state of the left side schematic navigation panel
      */
     void ToggleSchematicHierarchy();
 
+    /**
+     * Toggle the show/hide state of Search pane
+     */
+    void ToggleSearch();
+
+    void ToggleProperties() override;
+
+    DIALOG_BOOK_REPORTER* GetSymbolDiffDialog();
+
+    DIALOG_ERC* GetErcDialog();
+
+    DIALOG_SYMBOL_FIELDS_TABLE* GetSymbolFieldsTableDialog();
+
+    wxTreeCtrl* GetNetNavigator() { return m_netNavigator; }
+
+    const SCH_ITEM* GetSelectedNetNavigatorItem() const;
+
     /**
      * @return the name of the wxAuiPaneInfo managing the Hierarchy Navigator panel
      */
@@ -817,6 +846,42 @@ public:
         return wxT( "SchematicHierarchy" );
     }
 
+    /**
+     * @return the name of the wxAuiPaneInfo managing the Search panel
+     */
+    static const wxString SearchPaneName() { return wxT( "Search" ); }
+
+    /**
+     * Add \a aListener to post #EDA_EVT_SCHEMATIC_CHANGED command events to.
+     *
+     * @warning The caller is reponsible for removing any listeners that are no long valid.
+     *
+     * @note This only gets called when the schematic editor is in stand alone mode.  Changing
+     *       projects in the project manager closes the schematic editor when a new project is
+     *       loaded.
+     */
+    void AddSchematicChangeListener( wxEvtHandler* aListener );
+
+    /**
+     * Remove \a aListener to from the schematic changed listener list.
+     */
+    void RemoveSchematicChangeListener( wxEvtHandler* aListener );
+
+    static const wxString NetNavigatorPaneName()
+    {
+        return wxS( "NetNavigator" );
+    }
+
+    void RefreshNetNavigator( const NET_NAVIGATOR_ITEM_DATA* aSelection = nullptr );
+
+    void MakeNetNavigatorNode( const wxString& aNetName, wxTreeItemId aParentId,
+                               const NET_NAVIGATOR_ITEM_DATA* aSelection,
+                               bool aSingleSheetSchematic );
+
+    void SelectNetNavigatorItem( const NET_NAVIGATOR_ITEM_DATA* aSelection = nullptr );
+
+    void ToggleNetNavigator();
+
     DECLARE_EVENT_TABLE()
 
 protected:
@@ -838,10 +903,20 @@ protected:
 
     void saveProjectSettings() override;
 
+    void onCloseSymbolDiffDialog( wxCommandEvent& aEvent );
+
+    void onCloseErcDialog( wxCommandEvent& aEvent );
+
+    void onCloseSymbolFieldsTableDialog( wxCommandEvent& aEvent );
+
+    void unitsChangeRefresh() override;
+
 private:
     // Called when resizing the Hierarchy Navigator panel
     void OnResizeHierarchyNavigator( wxSizeEvent& aEvent );
 
+    void onResizeNetNavigator( wxSizeEvent& aEvent );
+
     // Sets up the tool framework
     void setupTools();
 
@@ -887,7 +962,8 @@ private:
      *  @param full filepath of file to be imported.
      *  @param aFileType SCH_FILE_T value for file type
      */
-    bool importFile( const wxString& aFileName, int aFileType );
+    bool importFile( const wxString& aFileName, int aFileType,
+                     const STRING_UTF8_MAP* aProperties = nullptr );
 
     /**
      * Save \a aSheet to a schematic file.
@@ -909,6 +985,12 @@ private:
 
     const wxString& getAutoSaveFileName() const;
 
+    wxTreeCtrl* createHighlightedNetNavigator();
+
+    void onNetNavigatorSelection( wxTreeEvent& aEvent );
+
+    void onNetNavigatorSelChanging( wxTreeEvent& aEvent );
+
 private:
     // The schematic editor control class should be able to access some internal
     // functions of the editor frame.
@@ -916,7 +998,7 @@ private:
     friend class SCH_FIND_REPLACE_TOOL;
 
     SCHEMATIC*              m_schematic;          ///< The currently loaded schematic
-    const SCH_CONNECTION*   m_highlightedConn;    ///< The highlighted net or bus, or nullptr
+    wxString                m_highlightedConn;    ///< The highlighted net or bus or empty string.
 
     wxPageSetupDialogData   m_pageSetupData;
     std::vector<std::unique_ptr<SCH_ITEM>> m_items_to_repeat;  ///< For the repeat-last-item cmd
@@ -927,10 +1009,21 @@ private:
                                                   ///< to call a custom net list generator.
 
     DIALOG_SCH_FIND*        m_findReplaceDialog;
-
+    DIALOG_ERC*             m_ercDialog;
+    DIALOG_BOOK_REPORTER*   m_diffSymbolDialog;
     HIERARCHY_PANE*         m_hierarchy;
+    DIALOG_SYMBOL_FIELDS_TABLE* m_symbolFieldsTableDialog;
+
+
+    wxTreeCtrl*             m_netNavigator;
 
 	bool m_syncingPcbToSchSelection; // Recursion guard when synchronizing selection from PCB
+
+    bool m_show_search;
+
+    bool m_highlightedConnChanged;
+
+    std::vector<wxEvtHandler*> m_schematicChangeListeners;
 };
 
 
