@@ -32,6 +32,8 @@
 
 #include <wx/log.h>
 #include <wx/menu.h>
+
+#include <advanced_config.h>
 #include <base_units.h>
 #include <common.h>     // for ExpandTextVars
 #include <eda_item.h>
@@ -55,18 +57,19 @@
 #include <tool/tool_manager.h>
 #include <tools/sch_navigate_tool.h>
 #include <font/outline_font.h>
+#include "sim/sim_lib_mgr.h"
 
 SCH_FIELD::SCH_FIELD( const VECTOR2I& aPos, int aFieldId, SCH_ITEM* aParent,
                       const wxString& aName ) :
     SCH_ITEM( aParent, SCH_FIELD_T ),
     EDA_TEXT( schIUScale, wxEmptyString ),
     m_id( 0 ),
-    m_name( aName ),
     m_showName( false ),
     m_allowAutoPlace( true ),
     m_renderCacheValid( false ),
     m_lastResolvedColor( COLOR4D::UNSPECIFIED )
 {
+    SetName( aName );
     SetTextPos( aPos );
     SetId( aFieldId );  // will also set the layer
     SetVisible( false );
@@ -87,13 +90,16 @@ SCH_FIELD::SCH_FIELD( const SCH_FIELD& aField ) :
     m_name           = aField.m_name;
     m_showName       = aField.m_showName;
     m_allowAutoPlace = aField.m_allowAutoPlace;
+    m_isNamedVariable = aField.m_isNamedVariable;
 
     m_renderCache.clear();
 
     for( const std::unique_ptr<KIFONT::GLYPH>& glyph : aField.m_renderCache )
     {
-        KIFONT::OUTLINE_GLYPH* outline_glyph = static_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() );
-        m_renderCache.emplace_back( std::make_unique<KIFONT::OUTLINE_GLYPH>( *outline_glyph ) );
+        if( KIFONT::OUTLINE_GLYPH* outline = dynamic_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() ) )
+            m_renderCache.emplace_back( std::make_unique<KIFONT::OUTLINE_GLYPH>( *outline ) );
+        else if( KIFONT::STROKE_GLYPH* stroke = dynamic_cast<KIFONT::STROKE_GLYPH*>( glyph.get() ) )
+            m_renderCache.emplace_back( std::make_unique<KIFONT::STROKE_GLYPH>( *stroke ) );
     }
 
     m_renderCacheValid = aField.m_renderCacheValid;
@@ -107,17 +113,20 @@ SCH_FIELD& SCH_FIELD::operator=( const SCH_FIELD& aField )
 {
     EDA_TEXT::operator=( aField );
 
-    m_id             = aField.m_id;
-    m_name           = aField.m_name;
-    m_showName       = aField.m_showName;
-    m_allowAutoPlace = aField.m_allowAutoPlace;
+    m_id              = aField.m_id;
+    m_name            = aField.m_name;
+    m_showName        = aField.m_showName;
+    m_allowAutoPlace  = aField.m_allowAutoPlace;
+    m_isNamedVariable = aField.m_isNamedVariable;
 
     m_renderCache.clear();
 
     for( const std::unique_ptr<KIFONT::GLYPH>& glyph : aField.m_renderCache )
     {
-        KIFONT::OUTLINE_GLYPH* outline_glyph = static_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() );
-        m_renderCache.emplace_back( std::make_unique<KIFONT::OUTLINE_GLYPH>( *outline_glyph ) );
+        if( KIFONT::OUTLINE_GLYPH* outline = dynamic_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() ) )
+            m_renderCache.emplace_back( std::make_unique<KIFONT::OUTLINE_GLYPH>( *outline ) );
+        else if( KIFONT::STROKE_GLYPH* stroke = dynamic_cast<KIFONT::STROKE_GLYPH*>( glyph.get() ) )
+            m_renderCache.emplace_back( std::make_unique<KIFONT::STROKE_GLYPH>( *stroke ) );
     }
 
     m_renderCacheValid = aField.m_renderCacheValid;
@@ -172,48 +181,84 @@ void SCH_FIELD::SetId( int aId )
 }
 
 
-wxString SCH_FIELD::GetShownText( int aDepth, bool aAllowExtraText ) const
+wxString SCH_FIELD::GetShownName() const
+{
+    return m_isNamedVariable ? GetTextVars( GetName() ) : GetName();
+}
+
+
+wxString SCH_FIELD::GetShownText( const SCH_SHEET_PATH* aPath, bool aAllowExtraText,
+                                  int aDepth ) const
 {
     std::function<bool( wxString* )> symbolResolver =
             [&]( wxString* token ) -> bool
             {
-                return static_cast<SCH_SYMBOL*>( m_parent )->ResolveTextVar( token, aDepth + 1 );
+                SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( m_parent );
+                return symbol->ResolveTextVar( aPath, token, aDepth + 1 );
+            };
+
+    std::function<bool( wxString* )> schematicResolver =
+            [&]( wxString* token ) -> bool
+            {
+                if( SCHEMATIC* schematic = Schematic() )
+                    return schematic->ResolveTextVar( aPath, token, aDepth + 1 );
+
+                return false;
             };
 
     std::function<bool( wxString* )> sheetResolver =
             [&]( wxString* token ) -> bool
             {
-                return static_cast<SCH_SHEET*>( m_parent )->ResolveTextVar( token, aDepth + 1 );
+                SCH_SHEET* sheet = static_cast<SCH_SHEET*>( m_parent );
+
+                SCHEMATIC* schematic = Schematic();
+                SCH_SHEET_PATH path = *aPath;
+                path.push_back( sheet );
+
+                bool retval = sheet->ResolveTextVar( &path, token, aDepth + 1 );
+
+                if( schematic )
+                    retval |= schematic->ResolveTextVar( &path, token, aDepth + 1 );
+
+                return retval;
             };
 
     std::function<bool( wxString* )> labelResolver =
             [&]( wxString* token ) -> bool
             {
-                return static_cast<SCH_LABEL_BASE*>( m_parent )->ResolveTextVar( token,
-                                                                                 aDepth + 1 );
+                SCH_LABEL_BASE* label = static_cast<SCH_LABEL_BASE*>( m_parent );
+                return label->ResolveTextVar( aPath, token, aDepth + 1 );
             };
 
-    wxString text = EDA_TEXT::GetShownText();
+    wxString text = EDA_TEXT::GetShownText( aAllowExtraText, aDepth );
 
-    if( IsNameShown() )
-        text = GetName() << wxS( ": " ) << text;
+    if( IsNameShown() && aAllowExtraText )
+        text = GetShownName() << wxS( ": " ) << text;
 
     if( text == wxS( "~" ) ) // Legacy placeholder for empty string
     {
         text = wxS( "" );
     }
-    else if( HasTextVars() )
+
+    // The iteration here it to allow for nested variables in the
+    // text strings (e.g. ${${VAR}}).  Although the symbols and sheets
+    // and labels recurse, text that is none of those types such as text
+    // boxes and labels do not.  This only loops if there is still a
+    // variable to resolve.
+    for( int ii = aDepth;
+         ii < ADVANCED_CFG::GetCfg().m_ResolveTextRecursionDepth && text.Contains( wxT( "${" ) );
+         ++ii )
     {
-        if( aDepth < 10 )
+        if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
+            text = ExpandTextVars( text, &symbolResolver );
+        else if( m_parent && m_parent->Type() == SCH_SHEET_T )
+            text = ExpandTextVars( text, &sheetResolver );
+        else if( m_parent && m_parent->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
+            text = ExpandTextVars( text, &labelResolver );
+        else if( Schematic() )
         {
-            if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
-                text = ExpandTextVars( text, &symbolResolver );
-            else if( m_parent && m_parent->Type() == SCH_SHEET_T )
-                text = ExpandTextVars( text, &sheetResolver );
-            else if( m_parent && m_parent->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
-                text = ExpandTextVars( text, &labelResolver );
-            else if( Schematic() )
-                text = ExpandTextVars( text, &Schematic()->Prj() );
+            text = ExpandTextVars( text, &Schematic()->Prj() );
+            text = ExpandTextVars( text, &schematicResolver );
         }
     }
 
@@ -229,19 +274,29 @@ wxString SCH_FIELD::GetShownText( int aDepth, bool aAllowExtraText ) const
             // For more than one part per package, we must add the part selection
             // A, B, ... or 1, 2, .. to the reference.
             if( parentSymbol->GetUnitCount() > 1 )
-                text << LIB_SYMBOL::SubReference( parentSymbol->GetUnit() );
+                text << parentSymbol->SubReference( parentSymbol->GetUnitSelection( aPath ) );
         }
     }
     else if( m_parent && m_parent->Type() == SCH_SHEET_T )
     {
-        if( m_id == SHEETFILENAME && aAllowExtraText )
-            text = _( "File:" ) + wxS( " " )+ text;
+        if( m_id == SHEETFILENAME && aAllowExtraText && !IsNameShown() )
+            text = _( "File:" ) + wxS( " " ) + text;
     }
 
     return text;
 }
 
 
+wxString SCH_FIELD::GetShownText( bool aAllowExtraText, int aDepth ) const
+{
+    if( SCHEMATIC* schematic = Schematic() )
+        return GetShownText( &schematic->CurrentSheet(), aAllowExtraText, aDepth );
+    else
+        return EDA_TEXT::GetShownText( aAllowExtraText, aDepth );
+}
+
+
+
 int SCH_FIELD::GetPenWidth() const
 {
     return GetEffectiveTextPenWidth();
@@ -290,7 +345,8 @@ SCH_FIELD::GetRenderCache( const wxString& forResolvedText, const VECTOR2I& forP
         {
             m_renderCache.clear();
 
-            outlineFont->GetLinesAsGlyphs( &m_renderCache, forResolvedText, forPosition, aAttrs );
+            outlineFont->GetLinesAsGlyphs( &m_renderCache, forResolvedText, forPosition, aAttrs,
+                                           GetFontMetrics() );
 
             m_renderCachePos = forPosition;
             m_renderCacheValid = true;
@@ -301,7 +357,12 @@ SCH_FIELD::GetRenderCache( const wxString& forResolvedText, const VECTOR2I& forP
             VECTOR2I delta = forPosition - m_renderCachePos;
 
             for( std::unique_ptr<KIFONT::GLYPH>& glyph : m_renderCache )
-                static_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() )->Move( delta );
+            {
+                if( glyph->IsOutline() )
+                    static_cast<KIFONT::OUTLINE_GLYPH*>( glyph.get() )->Move( delta );
+                else
+                    static_cast<KIFONT::STROKE_GLYPH*>( glyph.get() )->Move( delta );
+            }
 
             m_renderCachePos = forPosition;
         }
@@ -321,7 +382,7 @@ void SCH_FIELD::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset
     VECTOR2I textpos;
     int      penWidth = GetEffectiveTextPenWidth( aSettings->GetDefaultPenWidth() );
 
-    if( ( !IsVisible() && !IsForceVisible() ) || GetShownText().IsEmpty() )
+    if( ( !IsVisible() && !IsForceVisible() ) || GetShownText( true ).IsEmpty() )
         return;
 
     COLOR4D bg = aSettings->GetBackgroundColor();
@@ -374,8 +435,15 @@ void SCH_FIELD::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset
      */
     textpos = GetBoundingBox().Centre() + aOffset;
 
-    GRPrintText( DC, textpos, color, GetShownText(), orient, GetTextSize(), GR_TEXT_H_ALIGN_CENTER,
-                 GR_TEXT_V_ALIGN_CENTER, penWidth, IsItalic(), IsBold(), font );
+    if( GetParent() && GetParent()->Type() == SCH_GLOBAL_LABEL_T )
+    {
+        SCH_GLOBALLABEL* label = static_cast<SCH_GLOBALLABEL*>( GetParent() );
+        textpos += label->GetSchematicTextOffset( aSettings );
+    }
+
+    GRPrintText( DC, textpos, color, GetShownText( true ), orient, GetTextSize(),
+                 GR_TEXT_H_ALIGN_CENTER, GR_TEXT_V_ALIGN_CENTER, penWidth, IsItalic(), IsBold(),
+                 font, GetFontMetrics() );
 }
 
 
@@ -389,6 +457,8 @@ void SCH_FIELD::ImportValues( const LIB_FIELD& aSource )
 
 void SCH_FIELD::SwapData( SCH_ITEM* aItem )
 {
+    SCH_ITEM::SwapFlags( aItem );
+
     wxCHECK_RET( ( aItem != nullptr ) && ( aItem->Type() == SCH_FIELD_T ),
                  wxT( "Cannot swap field data with invalid item." ) );
 
@@ -397,6 +467,7 @@ void SCH_FIELD::SwapData( SCH_ITEM* aItem )
     std::swap( m_layer, item->m_layer );
     std::swap( m_showName, item->m_showName );
     std::swap( m_allowAutoPlace, item->m_allowAutoPlace );
+    std::swap( m_isNamedVariable, item->m_isNamedVariable );
     SwapText( *item );
     SwapAttributes( *item );
 
@@ -424,6 +495,21 @@ COLOR4D SCH_FIELD::GetFieldColor() const
 }
 
 
+void SCH_FIELD::ViewGetLayers( int aLayers[], int& aCount ) const
+{
+    aCount      = 2;
+
+    switch( m_id )
+    {
+    case REFERENCE_FIELD: aLayers[0] = LAYER_REFERENCEPART; break;
+    case VALUE_FIELD:     aLayers[0] = LAYER_VALUEPART;     break;
+    default:              aLayers[0] = LAYER_FIELDS;        break;
+    }
+
+    aLayers[1] = LAYER_SELECTION_SHADOWS;
+}
+
+
 EDA_ANGLE SCH_FIELD::GetDrawRotation() const
 {
     // Calculate the text orientation according to the symbol orientation.
@@ -579,7 +665,7 @@ bool SCH_FIELD::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) co
     {
     }
 
-    wxString text = GetShownText();
+    wxString text = UnescapeString( GetText() );
 
     if( !IsVisible() && !searchHiddenFields )
         return false;
@@ -596,13 +682,14 @@ bool SCH_FIELD::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) co
         // symbols with multiple parts.
         if( aAuxData )
         {
-            text = parentSymbol->GetRef((SCH_SHEET_PATH*) aAuxData );
+            SCH_SHEET_PATH* sheet = (SCH_SHEET_PATH*) aAuxData;
+            text = parentSymbol->GetRef( sheet );
 
             if( SCH_ITEM::Matches( text, aSearchData ) )
                 return true;
 
             if( parentSymbol->GetUnitCount() > 1 )
-                text << LIB_SYMBOL::SubReference( parentSymbol->GetUnit() );
+                text << parentSymbol->SubReference( parentSymbol->GetUnitSelection( sheet ) );
         }
     }
 
@@ -610,19 +697,127 @@ bool SCH_FIELD::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) co
 }
 
 
-bool SCH_FIELD::IsReplaceable() const
+void SCH_FIELD::OnScintillaCharAdded( SCINTILLA_TRICKS* aScintillaTricks,
+                                      wxStyledTextEvent &aEvent ) const
 {
-    if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
-    {
-        SCH_SYMBOL* parentSymbol = static_cast<SCH_SYMBOL*>( m_parent );
+    SCH_ITEM*  parent = dynamic_cast<SCH_ITEM*>( GetParent() );
+    SCHEMATIC* schematic = parent ? parent->Schematic() : nullptr;
 
-        if( m_id == VALUE_FIELD )
+    if( !schematic )
+        return;
+
+    wxStyledTextCtrl* scintilla = aScintillaTricks->Scintilla();
+    int               key = aEvent.GetKey();
+
+    wxArrayString autocompleteTokens;
+    int           pos = scintilla->GetCurrentPos();
+    int           start = scintilla->WordStartPosition( pos, true );
+    wxString      partial;
+
+    // Multi-line fields are not allowed. So remove '\n' if entered.
+    if( key == '\n' )
+    {
+        wxString text = scintilla->GetText();
+        int currpos = scintilla->GetCurrentPos();
+        text.Replace( wxS( "\n" ), wxS( "" ) );
+        scintilla->SetText( text );
+        scintilla->GotoPos( currpos-1 );
+        return;
+    }
+
+    auto textVarRef =
+            [&]( int pt )
+            {
+                return pt >= 2
+                        && scintilla->GetCharAt( pt - 2 ) == '$'
+                        && scintilla->GetCharAt( pt - 1 ) == '{';
+            };
+
+    // Check for cross-reference
+    if( start > 1 && scintilla->GetCharAt( start - 1 ) == ':' )
+    {
+        int refStart = scintilla->WordStartPosition( start - 1, true );
+
+        if( textVarRef( refStart ) )
         {
-            if( parentSymbol->GetLibSymbolRef() && parentSymbol->GetLibSymbolRef()->IsPower() )
-                return false;
+            partial = scintilla->GetRange( start, pos );
+
+            wxString ref = scintilla->GetRange( refStart, start - 1 );
+
+            if( ref == wxS( "OP" ) )
+            {
+                // SPICE operating points use ':' syntax for ports
+                if( SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( parent ) )
+                {
+                    NULL_REPORTER   devnull;
+                    SCH_SHEET_PATH& sheet = schematic->CurrentSheet();
+                    SIM_LIB_MGR     mgr( &schematic->Prj() );
+                    SIM_MODEL&      model = mgr.CreateModel( &sheet, *symbol, devnull ).model;
+
+                    for( wxString pin : model.GetPinNames() )
+                    {
+                        if( pin.StartsWith( '<' ) && pin.EndsWith( '>' ) )
+                            autocompleteTokens.push_back( pin.Mid( 1, pin.Length() - 2 ) );
+                        else
+                            autocompleteTokens.push_back( pin );
+                    }
+                }
+            }
+            else
+            {
+                SCH_REFERENCE_LIST refs;
+                SCH_SYMBOL*        refSymbol = nullptr;
+
+                schematic->GetUnorderedSheets().GetSymbols( refs );
+
+                for( size_t jj = 0; jj < refs.GetCount(); jj++ )
+                {
+                    if( refs[ jj ].GetSymbol()->GetRef( &refs[ jj ].GetSheetPath(), true ) == ref )
+                    {
+                        refSymbol = refs[ jj ].GetSymbol();
+                        break;
+                    }
+                }
+
+                if( refSymbol )
+                    refSymbol->GetContextualTextVars( &autocompleteTokens );
+            }
         }
     }
-    else if( m_parent && m_parent->Type() == SCH_SHEET_T )
+    else if( textVarRef( start ) )
+    {
+        partial = scintilla->GetTextRange( start, pos );
+
+        SCH_SYMBOL*     symbol = dynamic_cast<SCH_SYMBOL*>( parent );
+        SCH_SHEET*      sheet = dynamic_cast<SCH_SHEET*>( parent );
+        SCH_LABEL_BASE* label = dynamic_cast<SCH_LABEL_BASE*>( parent );
+
+        if( symbol )
+        {
+            symbol->GetContextualTextVars( &autocompleteTokens );
+
+            if( schematic->CurrentSheet().Last() )
+                schematic->CurrentSheet().Last()->GetContextualTextVars( &autocompleteTokens );
+        }
+
+        if( sheet )
+            sheet->GetContextualTextVars( &autocompleteTokens );
+
+        if( label )
+            label->GetContextualTextVars( &autocompleteTokens );
+
+        for( std::pair<wxString, wxString> entry : schematic->Prj().GetTextVars() )
+            autocompleteTokens.push_back( entry.first );
+    }
+
+    aScintillaTricks->DoAutocomplete( partial, autocompleteTokens );
+    scintilla->SetFocus();
+}
+
+
+bool SCH_FIELD::IsReplaceable() const
+{
+    if( m_parent && m_parent->Type() == SCH_SHEET_T )
     {
         // See comments in SCH_FIELD::Replace(), below.
         if( m_id == SHEETFILENAME )
@@ -652,7 +847,6 @@ bool SCH_FIELD::Replace( const EDA_SEARCH_DATA& aSearchData, void* aAuxData )
     }
 
     wxString text;
-    bool     resolve = false;    // Replace in source text, not shown text
     bool     isReplaced = false;
 
     if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
@@ -678,7 +872,7 @@ bool SCH_FIELD::Replace( const EDA_SEARCH_DATA& aSearchData, void* aAuxData )
         case VALUE_FIELD:
             wxCHECK_MSG( aAuxData, false, wxT( "Need sheetpath to replace in value field." ) );
 
-            text = parentSymbol->GetValueFieldText( resolve );
+            text = parentSymbol->GetField( VALUE_FIELD )->GetText();
             isReplaced = EDA_ITEM::Replace( aSearchData, text );
 
             if( isReplaced )
@@ -689,7 +883,7 @@ bool SCH_FIELD::Replace( const EDA_SEARCH_DATA& aSearchData, void* aAuxData )
         case FOOTPRINT_FIELD:
             wxCHECK_MSG( aAuxData, false, wxT( "Need sheetpath to replace in footprint field." ) );
 
-            text = parentSymbol->GetFootprintFieldText( resolve );
+            text = parentSymbol->GetField( FOOTPRINT_FIELD )->GetText();
             isReplaced = EDA_ITEM::Replace( aSearchData, text );
 
             if( isReplaced )
@@ -731,7 +925,7 @@ void SCH_FIELD::Rotate( const VECTOR2I& aCenter )
 
 wxString SCH_FIELD::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 {
-    return wxString::Format( "%s '%s'", GetName(), KIUI::EllipsizeMenuText( GetShownText() ) );
+    return wxString::Format( "%s '%s'", GetName(), KIUI::EllipsizeMenuText( GetText() ) );
 }
 
 
@@ -739,10 +933,10 @@ void SCH_FIELD::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_I
 {
     wxString msg;
 
-    aList.emplace_back( _( "Symbol Field" ), GetName() );
+    aList.emplace_back( _( "Symbol Field" ), UnescapeString( GetName() ) );
 
     // Don't use GetShownText() here; we want to show the user the variable references
-    aList.emplace_back( _( "Text" ), UnescapeString( GetText() ) );
+    aList.emplace_back( _( "Text" ), KIUI::EllipsizeStatusText( aFrame, GetText() ) );
 
     aList.emplace_back( _( "Visible" ), IsVisible() ? _( "Yes" ) : _( "No" ) );
 
@@ -811,12 +1005,37 @@ void SCH_FIELD::DoHypertextAction( EDA_DRAW_FRAME* aFrame ) const
 }
 
 
+void SCH_FIELD::SetName( const wxString& aName )
+{
+    m_name = aName;
+    m_isNamedVariable = m_name.StartsWith( wxT( "${" ) );
+
+    if( m_isNamedVariable )
+        EDA_TEXT::SetText( aName );
+}
+
+
+void SCH_FIELD::SetText( const wxString& aText )
+{
+    // Don't allow modification of text value when using named variables
+    // as field name.
+    if( m_isNamedVariable )
+        return;
+
+    // Mandatory fields should not have leading or trailing whitespace.
+    if( IsMandatory() )
+        EDA_TEXT::SetText( aText.Strip( wxString::both ) );
+    else
+        EDA_TEXT::SetText( aText );
+}
+
+
 wxString SCH_FIELD::GetName( bool aUseDefaultName ) const
 {
     if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
     {
         if( m_id >= 0 && m_id < MANDATORY_FIELDS )
-            return TEMPLATE_FIELDNAME::GetDefaultFieldName( m_id );
+            return GetCanonicalFieldName( m_id );
         else if( m_name.IsEmpty() && aUseDefaultName )
             return TEMPLATE_FIELDNAME::GetDefaultFieldName( m_id );
         else
@@ -847,14 +1066,10 @@ wxString SCH_FIELD::GetCanonicalName() const
 {
     if( m_parent && m_parent->Type() == SCH_SYMBOL_T )
     {
-        switch( m_id )
-        {
-        case  REFERENCE_FIELD: return wxT( "Reference" );
-        case  VALUE_FIELD:     return wxT( "Value" );
-        case  FOOTPRINT_FIELD: return wxT( "Footprint" );
-        case  DATASHEET_FIELD: return wxT( "Datasheet" );
-        default:               return m_name;
-        }
+        if( m_id >= 0 && m_id < MANDATORY_FIELDS )
+            return GetCanonicalFieldName( m_id );
+        else
+            return m_name;
     }
     else if( m_parent && m_parent->Type() == SCH_SHEET_T )
     {
@@ -869,13 +1084,19 @@ wxString SCH_FIELD::GetCanonicalName() const
     {
         // These should be stored in canonical format, but just in case:
         if( m_name == _( "Net Class" ) || m_name == wxT( "Net Class" ) )
+        {
             return wxT( "Netclass" );
+        }
         else if( m_name == _( "Sheet References" )
                  || m_name == wxT( "Sheet References" )
                  || m_name == wxT( "Intersheet References" ) )
+        {
             return wxT( "Intersheetrefs" );
+        }
         else
+        {
             return m_name;
+        }
     }
     else
     {
@@ -910,7 +1131,7 @@ BITMAPS SCH_FIELD::GetMenuImage() const
 bool SCH_FIELD::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
 {
     // Do not hit test hidden or empty fields.
-    if( !IsVisible() || GetShownText().IsEmpty() )
+    if( !IsVisible() || GetShownText( true ).IsEmpty() )
         return false;
 
     BOX2I rect = GetBoundingBox();
@@ -930,7 +1151,7 @@ bool SCH_FIELD::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
 bool SCH_FIELD::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) const
 {
     // Do not hit test hidden fields.
-    if( !IsVisible() || GetShownText().IsEmpty() )
+    if( !IsVisible() || GetShownText( true ).IsEmpty() )
         return false;
 
     BOX2I rect = aRect;
@@ -950,9 +1171,10 @@ bool SCH_FIELD::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) co
 }
 
 
-void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground ) const
+void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground,
+                      const SCH_PLOT_SETTINGS& aPlotSettings ) const
 {
-    if( GetShownText().IsEmpty() || aBackground )
+    if( GetShownText( true ).IsEmpty() || aBackground )
         return;
 
     RENDER_SETTINGS* settings = aPlotter->RenderSettings();
@@ -1032,7 +1254,7 @@ void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground ) const
     attrs.m_Angle = orient;
     attrs.m_Multiline = false;
 
-    aPlotter->PlotText( textpos, color, GetShownText(), attrs, font );
+    aPlotter->PlotText( textpos, color, GetShownText( true ), attrs, font, GetFontMetrics() );
 
     if( IsHypertext() )
     {
@@ -1097,6 +1319,15 @@ VECTOR2I SCH_FIELD::GetParentPosition() const
 }
 
 
+bool SCH_FIELD::IsMandatory() const
+{
+    if( m_parent && m_parent->Type() == SCH_SHEET_T )
+        return m_id >= 0 && m_id < SHEET_MANDATORY_FIELDS;
+    else
+        return m_id >= 0 && m_id < MANDATORY_FIELDS;
+}
+
+
 bool SCH_FIELD::operator <( const SCH_ITEM& aItem ) const
 {
     if( Type() != aItem.Type() )
@@ -1118,3 +1349,115 @@ bool SCH_FIELD::operator <( const SCH_ITEM& aItem ) const
 
     return GetName() < field->GetName();
 }
+
+bool SCH_FIELD::operator==( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return false;
+
+    const SCH_FIELD& field = static_cast<const SCH_FIELD&>( aOther );
+
+    if( GetId() != field.GetId() )
+        return false;
+
+    if( GetPosition() != field.GetPosition() )
+        return false;
+
+    if( IsNamedVariable() != field.IsNamedVariable() )
+        return false;
+
+    if( IsNameShown() != field.IsNameShown() )
+        return false;
+
+    if( CanAutoplace() != field.CanAutoplace() )
+        return false;
+
+    if( GetText() != field.GetText() )
+        return false;
+
+    return true;
+}
+
+
+double SCH_FIELD::Similarity( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return 0.0;
+
+    if( m_Uuid == aOther.m_Uuid )
+        return 1.0;
+
+    const SCH_FIELD& field = static_cast<const SCH_FIELD&>( aOther );
+
+    double similarity = 0.99; // The UUIDs are different, so we start with non-identity
+
+    if( GetId() != field.GetId() )
+    {
+        // We don't allow swapping of mandatory fields, so these cannot be the same item
+        if( GetId() < MANDATORY_FIELDS || field.GetId() < MANDATORY_FIELDS )
+            return 0.0;
+        else
+            similarity *= 0.5;
+    }
+
+    if( GetPosition() != field.GetPosition() )
+        similarity *= 0.5;
+
+    if( IsNamedVariable() != field.IsNamedVariable() )
+        similarity *= 0.5;
+
+    if( IsNameShown() != field.IsNameShown() )
+        similarity *= 0.5;
+
+    if( CanAutoplace() != field.CanAutoplace() )
+        similarity *= 0.5;
+
+    if( GetText() != field.GetText() )
+        similarity *= Levenshtein( field );
+
+    return similarity;
+}
+
+
+static struct SCH_FIELD_DESC
+{
+    SCH_FIELD_DESC()
+    {
+        PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
+        REGISTER_TYPE( SCH_FIELD );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_FIELD, SCH_ITEM> );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_FIELD, EDA_TEXT> );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_FIELD ), TYPE_HASH( SCH_ITEM ) );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_FIELD, bool>( _HKI( "Show Field Name" ),
+                &SCH_FIELD::SetNameShown, &SCH_FIELD::IsNameShown ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_FIELD, bool>( _HKI( "Allow Autoplacement" ),
+                &SCH_FIELD::SetCanAutoplace, &SCH_FIELD::CanAutoplace ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Hyperlink" ) );
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Thickness" ) );
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Mirrored" ) );
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Width" ) );
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Height" ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_FIELD, int>( _HKI( "Text Size" ),
+                &SCH_FIELD::SetSchTextSize, &SCH_FIELD::GetSchTextSize, PROPERTY_DISPLAY::PT_SIZE ),
+                _HKI( "Text Properties" ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Orientation" ) );
+
+        auto isNotNamedVariable =
+                []( INSPECTABLE* aItem ) -> bool
+                {
+                    if( SCH_FIELD* field = dynamic_cast<SCH_FIELD*>( aItem ) )
+                        return !field->IsNamedVariable();
+
+                    return true;
+                };
+
+        propMgr.OverrideWriteability( TYPE_HASH( SCH_FIELD ), TYPE_HASH( EDA_TEXT ), _HKI( "Text" ),
+                                      isNotNamedVariable );
+    }
+} _SCH_FIELD_DESC;
