@@ -4,7 +4,7 @@
  * Copyright (C) 1992-2018 Jean-Pierre Charras jp.charras at wanadoo.fr
  * Copyright (C) 1992-2010 Lorenzo Marcantonio
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023, 2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <wx/log.h>
 #include <common.h>
 #include <sch_plotter.h>
 #include <locale_io.h>
@@ -40,6 +41,13 @@
 #include <sch_screen.h>
 #include <settings/settings_manager.h>
 
+// Note:
+// We need to switch between sheets to plot a hierarchy and update references and sheet number
+// Use SCHEMATIC::SetCurrentSheet( xxx ) to switch to a sheet.
+// Do not use SCH_EDIT_FRAME::SetCurrentSheet( xxx ) to switch to a sheet, because the new sheet
+// is not displayed, but SCH_EDIT_FRAME::SetCurrentSheet() has side effects to the current VIEW
+// (clear some data used to show the sheet on screen) and does not fully restore the "old" screen
+
 
 static const wxChar* plot_sheet_list( HPGL_PAGE_SIZE aSize )
 {
@@ -63,7 +71,6 @@ static const wxChar* plot_sheet_list( HPGL_PAGE_SIZE aSize )
 
 
 SCH_PLOTTER::SCH_PLOTTER( SCHEMATIC* aSchematic ) :
-        m_schFrame( nullptr ),
         m_schematic( aSchematic )
 {
     m_colorSettings = nullptr;
@@ -71,7 +78,6 @@ SCH_PLOTTER::SCH_PLOTTER( SCHEMATIC* aSchematic ) :
 
 
 SCH_PLOTTER::SCH_PLOTTER( SCH_EDIT_FRAME* aFrame ) :
-        m_schFrame( aFrame ),
         m_schematic( &aFrame->Schematic() )
 {
     m_colorSettings = nullptr;
@@ -111,18 +117,25 @@ void SCH_PLOTTER::createPDFFile( const SCH_PLOT_SETTINGS& aPlotSettings,
      */
     SCH_SHEET_LIST sheetList;
 
-    if( aPlotSettings.m_plotAll )
+    if( aPlotSettings.m_plotAll || aPlotSettings.m_plotPages.size() > 0 )
     {
         sheetList.BuildSheetList( &m_schematic->Root(), true );
         sheetList.SortByPageNumbers();
+
+        // remove the non-selected pages if we are in plot pages mode
+        if( aPlotSettings.m_plotPages.size() > 0 )
+            sheetList.TrimToPageNumbers( aPlotSettings.m_plotPages );
     }
     else
     {
+        // in eeschema, this prints the current page
         sheetList.push_back( m_schematic->CurrentSheet() );
     }
 
+    wxCHECK( m_schematic, /* void */ );
+
     // Allocate the plotter and set the job level parameter
-    PDF_PLOTTER* plotter = new PDF_PLOTTER();
+    PDF_PLOTTER* plotter = new PDF_PLOTTER( &m_schematic->Prj() );
     plotter->SetRenderSettings( aRenderSettings );
     plotter->SetColorMode( !aPlotSettings.m_blackAndWhite );
     plotter->SetCreator( wxT( "Eeschema-PDF" ) );
@@ -135,16 +148,12 @@ void SCH_PLOTTER::createPDFFile( const SCH_PLOT_SETTINGS& aPlotSettings,
 
     for( unsigned i = 0; i < sheetList.size(); i++ )
     {
-        if( m_schFrame )
-            m_schFrame->SetCurrentSheet( sheetList[i] );
-        else
-            m_schematic->SetCurrentSheet( sheetList[i] );
-
+        m_schematic->SetCurrentSheet( sheetList[i] );
         m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
         m_schematic->SetSheetNumberAndCount();
 
         SCH_SCREEN* screen = m_schematic->CurrentSheet().LastScreen();
+        wxString    sheetName = sheetList[i].Last()->GetFields()[SHEETNAME].GetShownText( false );
 
         if( i == 0 )
         {
@@ -172,7 +181,8 @@ void SCH_PLOTTER::createPDFFile( const SCH_PLOT_SETTINGS& aPlotSettings,
 
                 // Open the plotter and do the first page
                 setupPlotPagePDF( plotter, screen, aPlotSettings );
-                plotter->StartPlot( sheetList[i].GetPageNumber(), _( "Root" ) );
+
+                plotter->StartPlot( sheetList[i].GetPageNumber(), sheetName );
             }
             catch( const IO_ERROR& e )
             {
@@ -193,8 +203,7 @@ void SCH_PLOTTER::createPDFFile( const SCH_PLOT_SETTINGS& aPlotSettings,
              *  reconfigure, and then start a new one */
             plotter->ClosePage();
             setupPlotPagePDF( plotter, screen, aPlotSettings );
-            plotter->StartPage( sheetList[i].GetPageNumber(),
-                                sheetList[i].Last()->GetFields()[SHEETNAME].GetShownText() );
+            plotter->StartPage( sheetList[i].GetPageNumber(), sheetName );
         }
 
         plotOneSheetPDF( plotter, screen, aPlotSettings );
@@ -218,9 +227,13 @@ void SCH_PLOTTER::plotOneSheetPDF( PLOTTER* aPlotter, SCH_SCREEN* aScreen,
     if( aPlotSettings.m_useBackgroundColor && aPlotter->GetColorMode() )
     {
         aPlotter->SetColor( aPlotter->RenderSettings()->GetBackgroundColor() );
-        wxPoint end( aPlotter->PageSettings().GetWidthIU( schIUScale.IU_PER_MILS ),
-                     aPlotter->PageSettings().GetHeightIU( schIUScale.IU_PER_MILS ) );
-        aPlotter->Rect( wxPoint( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
+
+        // Use page size selected in schematic to know the schematic bg area
+        const PAGE_INFO& actualPage = aScreen->GetPageSettings(); // page size selected in schematic
+        VECTOR2I end( actualPage.GetWidthIU( schIUScale.IU_PER_MILS ),
+                      actualPage.GetHeightIU( schIUScale.IU_PER_MILS ) );
+
+        aPlotter->Rect( VECTOR2I( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
     }
 
     if( aPlotSettings.m_plotDrawingSheet )
@@ -242,7 +255,7 @@ void SCH_PLOTTER::plotOneSheetPDF( PLOTTER* aPlotter, SCH_SCREEN* aScreen,
                           aScreen->GetFileName(), color, aScreen->GetVirtualPageNumber() == 1 );
     }
 
-    aScreen->Plot( aPlotter );
+    aScreen->Plot( aPlotter, aPlotSettings );
 }
 
 
@@ -278,7 +291,7 @@ void SCH_PLOTTER::setupPlotPagePDF( PLOTTER* aPlotter, SCH_SCREEN* aScreen,
     aPlotter->SetPageSettings( plotPage );
 
     // Currently, plot units are in decimil
-    aPlotter->SetViewport( wxPoint( 0, 0 ), schIUScale.IU_PER_MILS/10, scale, false );
+    aPlotter->SetViewport( VECTOR2I( 0, 0 ), schIUScale.IU_PER_MILS / 10, scale, false );
 }
 
 
@@ -301,6 +314,12 @@ void SCH_PLOTTER::createPSFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
     {
         sheetList.BuildSheetList( &m_schematic->Root(), true );
         sheetList.SortByPageNumbers();
+
+        // remove the non-selected pages if we are in plot pages mode
+        if( aPlotSettings.m_plotPages.size() > 0 )
+        {
+            sheetList.TrimToPageNumbers( aPlotSettings.m_plotPages );
+        }
     }
     else
     {
@@ -309,13 +328,8 @@ void SCH_PLOTTER::createPSFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
 
     for( unsigned i = 0; i < sheetList.size(); i++ )
     {
-        if( m_schFrame )
-            m_schFrame->SetCurrentSheet( sheetList[i] );
-        else
-            m_schematic->SetCurrentSheet( sheetList[i] );
-
+        m_schematic->SetCurrentSheet( sheetList[i] );
         m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
         m_schematic->SetSheetNumberAndCount();
 
         SCH_SCREEN* screen = m_schematic->CurrentSheet().LastScreen();
@@ -340,7 +354,7 @@ void SCH_PLOTTER::createPSFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
         double  scalex = (double) plotPage.GetWidthMils() / actualPage.GetWidthMils();
         double  scaley = (double) plotPage.GetHeightMils() / actualPage.GetHeightMils();
         double  scale = std::min( scalex, scaley );
-        wxPoint plot_offset;
+        VECTOR2I plot_offset;
 
         try
         {
@@ -393,20 +407,13 @@ void SCH_PLOTTER::createPSFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
         aReporter->ReportTail( _( "Done." ), RPT_SEVERITY_INFO );
     }
 
-    if( m_schFrame )
-        m_schFrame->SetCurrentSheet( oldsheetpath );
-    else
-        m_schematic->SetCurrentSheet( oldsheetpath );
-
-    m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
-    m_schematic->SetSheetNumberAndCount();
+    restoreEnvironment( nullptr, oldsheetpath );
 }
 
 
 bool SCH_PLOTTER::plotOneSheetPS( const wxString& aFileName, SCH_SCREEN* aScreen,
                                             RENDER_SETTINGS* aRenderSettings,
-                                            const PAGE_INFO& aPageInfo, const wxPoint& aPlot0ffset,
+                                            const PAGE_INFO& aPageInfo, const VECTOR2I& aPlot0ffset,
                                             double aScale, const SCH_PLOT_SETTINGS& aPlotSettings )
 {
     PS_PLOTTER* plotter = new PS_PLOTTER();
@@ -434,9 +441,12 @@ bool SCH_PLOTTER::plotOneSheetPS( const wxString& aFileName, SCH_SCREEN* aScreen
     {
         plotter->SetColor( plotter->RenderSettings()->GetLayerColor( LAYER_SCHEMATIC_BACKGROUND ) );
 
-        wxPoint end( plotter->PageSettings().GetWidthIU( schIUScale.IU_PER_MILS ),
-                     plotter->PageSettings().GetHeightIU( schIUScale.IU_PER_MILS ) );
-        plotter->Rect( wxPoint( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
+        // Use page size selected in schematic to know the schematic bg area
+        const PAGE_INFO& actualPage = aScreen->GetPageSettings(); // page size selected in schematic
+        VECTOR2I end( actualPage.GetWidthIU( schIUScale.IU_PER_MILS ),
+                      actualPage.GetHeightIU( schIUScale.IU_PER_MILS ) );
+
+        plotter->Rect( VECTOR2I( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
     }
 
     if( aPlotSettings.m_plotDrawingSheet )
@@ -453,7 +463,7 @@ bool SCH_PLOTTER::plotOneSheetPS( const wxString& aFileName, SCH_SCREEN* aScreen
                           aScreen->GetVirtualPageNumber() == 1 );
     }
 
-    aScreen->Plot( plotter );
+    aScreen->Plot( plotter, aPlotSettings );
 
     plotter->EndPlot();
     delete plotter;
@@ -473,22 +483,25 @@ void SCH_PLOTTER::createSVGFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
     {
         sheetList.BuildSheetList( &m_schematic->Root(), true );
         sheetList.SortByPageNumbers();
+
+        // remove the non-selected pages if we are in plot pages mode
+        if( aPlotSettings.m_plotPages.size() > 0 )
+        {
+            sheetList.TrimToPageNumbers( aPlotSettings.m_plotPages );
+        }
     }
     else
     {
+        // in eeschema, this prints the current page
         sheetList.push_back( m_schematic->CurrentSheet() );
     }
 
     for( unsigned i = 0; i < sheetList.size(); i++ )
     {
         SCH_SCREEN* screen;
-        if( m_schFrame )
-            m_schFrame->SetCurrentSheet( sheetList[i] );
-        else
-            m_schematic->SetCurrentSheet( sheetList[i] );
 
+        m_schematic->SetCurrentSheet( sheetList[i] );
         m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
         m_schematic->SetSheetNumberAndCount();
 
         screen = m_schematic->CurrentSheet().LastScreen();
@@ -547,14 +560,7 @@ void SCH_PLOTTER::createSVGFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
         aReporter->ReportTail( _( "Done" ), RPT_SEVERITY_INFO );
     }
 
-    if( m_schFrame )
-        m_schFrame->SetCurrentSheet( oldsheetpath );
-    else
-        m_schematic->SetCurrentSheet( oldsheetpath );
-
-    m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
-    m_schematic->SetSheetNumberAndCount();
+    restoreEnvironment( nullptr, oldsheetpath );
 }
 
 
@@ -588,7 +594,7 @@ bool SCH_PLOTTER::plotOneSheetSVG( const wxString& aFileName, SCH_SCREEN* aScree
     plotter->SetRenderSettings( aRenderSettings );
     plotter->SetPageSettings( plotPage );
     plotter->SetColorMode( aPlotSettings.m_blackAndWhite ? false : true );
-    wxPoint plot_offset;
+    VECTOR2I plot_offset;
 
     double  scalex = (double) plotPage.GetWidthMils() / actualPage.GetWidthMils();
     double  scaley = (double) plotPage.GetHeightMils() / actualPage.GetHeightMils();
@@ -613,9 +619,12 @@ bool SCH_PLOTTER::plotOneSheetSVG( const wxString& aFileName, SCH_SCREEN* aScree
     if( aPlotSettings.m_useBackgroundColor && plotter->GetColorMode() )
     {
         plotter->SetColor( plotter->RenderSettings()->GetLayerColor( LAYER_SCHEMATIC_BACKGROUND ) );
-        wxPoint end( plotter->PageSettings().GetWidthIU( schIUScale.IU_PER_MILS ),
-                     plotter->PageSettings().GetHeightIU( schIUScale.IU_PER_MILS ) );
-        plotter->Rect( wxPoint( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
+
+        // Use page size selected in schematic to know the schematic bg area
+        VECTOR2I end( actualPage.GetWidthIU( schIUScale.IU_PER_MILS ),
+                      actualPage.GetHeightIU( schIUScale.IU_PER_MILS ) );
+
+        plotter->Rect( VECTOR2I( 0, 0 ), end, FILL_T::FILLED_SHAPE, 1.0 );
     }
 
     if( aPlotSettings.m_plotDrawingSheet )
@@ -632,7 +641,7 @@ bool SCH_PLOTTER::plotOneSheetSVG( const wxString& aFileName, SCH_SCREEN* aScree
                           aScreen->GetVirtualPageNumber() == 1 );
     }
 
-    aScreen->Plot( plotter );
+    aScreen->Plot( plotter, aPlotSettings );
 
     plotter->EndPlot();
     delete plotter;
@@ -658,20 +667,23 @@ void SCH_PLOTTER::createHPGLFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
     {
         sheetList.BuildSheetList( &m_schematic->Root(), true );
         sheetList.SortByPageNumbers();
+
+        // remove the non-selected pages if we are in plot pages mode
+        if( aPlotSettings.m_plotPages.size() > 0 )
+        {
+            sheetList.TrimToPageNumbers( aPlotSettings.m_plotPages );
+        }
     }
     else
     {
+        // in eeschema, this prints the current page
         sheetList.push_back( m_schematic->CurrentSheet() );
     }
 
     for( unsigned i = 0; i < sheetList.size(); i++ )
     {
-        if( m_schFrame )
-            m_schFrame->SetCurrentSheet( sheetList[i] );
-        else
-            m_schematic->SetCurrentSheet( sheetList[i] );
+        m_schematic->SetCurrentSheet( sheetList[i] );
         m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
         m_schematic->SetSheetNumberAndCount();
 
         screen = m_schematic->CurrentSheet().LastScreen();
@@ -690,7 +702,7 @@ void SCH_PLOTTER::createHPGLFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
         double  plot_scale = (double) plotPage.GetWidthMils() / curPage.GetWidthMils();
 
         // Calculate offsets
-        wxPoint plotOffset;
+        VECTOR2I plotOffset;
         wxString msg;
 
         if( aPlotSettings.m_HPGLPlotOrigin == HPGL_PLOT_ORIGIN_AND_UNITS::PLOTTER_CENTER )
@@ -748,22 +760,15 @@ void SCH_PLOTTER::createHPGLFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
         aReporter->ReportTail( _( "Done." ), RPT_SEVERITY_INFO );
     }
 
-    if( m_schFrame )
-        m_schFrame->SetCurrentSheet( oldsheetpath );
-    else
-        m_schematic->SetCurrentSheet( oldsheetpath );
-
-    m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
-    m_schematic->SetSheetNumberAndCount();
+    restoreEnvironment( nullptr, oldsheetpath );
 }
 
 
-bool SCH_PLOTTER::plotOneSheetHpgl( const wxString& aFileName,
+bool SCH_PLOTTER::plotOneSheetHpgl( const wxString&   aFileName,
                                     SCH_SCREEN*       aScreen,
                                     const PAGE_INFO&  aPageInfo,
                                     RENDER_SETTINGS*  aRenderSettings,
-                                    const wxPoint&    aPlot0ffset,
+                                    const VECTOR2I&   aPlot0ffset,
                                     double            aScale,
                                     const SCH_PLOT_SETTINGS&   aPlotSettings )
 {
@@ -773,7 +778,6 @@ bool SCH_PLOTTER::plotOneSheetHpgl( const wxString& aFileName,
     plotter->SetPageSettings( aPageInfo );
     plotter->SetRenderSettings( aRenderSettings );
     plotter->RenderSettings()->LoadColors( m_colorSettings );
-    plotter->SetColorMode( !aPlotSettings.m_blackAndWhite );
     plotter->SetViewport( aPlot0ffset, schIUScale.IU_PER_MILS/10, aScale, false );
 
     // TODO this could be configurable
@@ -817,7 +821,7 @@ bool SCH_PLOTTER::plotOneSheetHpgl( const wxString& aFileName,
         wxString sheetName = m_schematic->CurrentSheet().Last()->GetName();
         wxString sheetPath = m_schematic->CurrentSheet().PathHumanReadable();
 
-        PlotDrawingSheet( plotter, &m_schematic->Prj(), 
+        PlotDrawingSheet( plotter, &m_schematic->Prj(),
                           aScreen->GetTitleBlock(),
                           aPageInfo,
                           aScreen->Schematic()->GetProperties(), aScreen->GetPageNumber(),
@@ -825,7 +829,7 @@ bool SCH_PLOTTER::plotOneSheetHpgl( const wxString& aFileName,
                           COLOR4D::BLACK, aScreen->GetVirtualPageNumber() == 1 );
     }
 
-    aScreen->Plot( plotter );
+    aScreen->Plot( plotter, aPlotSettings );
 
     plotter->EndPlot();
 
@@ -851,24 +855,27 @@ void SCH_PLOTTER::createDXFFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
     {
         sheetList.BuildSheetList( &m_schematic->Root(), true );
         sheetList.SortByPageNumbers();
+
+        // remove the non-selected pages if we are in plot pages mode
+        if( aPlotSettings.m_plotPages.size() > 0 )
+        {
+            sheetList.TrimToPageNumbers( aPlotSettings.m_plotPages );
+        }
     }
     else
     {
+        // in eeschema, this prints the current page
         sheetList.push_back( m_schematic->CurrentSheet() );
     }
 
     for( unsigned i = 0; i < sheetList.size(); i++ )
     {
-        if( m_schFrame )
-            m_schFrame->SetCurrentSheet( sheetList[i] );
-        else
-            m_schematic->SetCurrentSheet( sheetList[i] );
+        m_schematic->SetCurrentSheet( sheetList[i] );
         m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
         m_schematic->SetSheetNumberAndCount();
 
         SCH_SCREEN* screen = m_schematic->CurrentSheet().LastScreen();
-        wxPoint     plot_offset;
+        VECTOR2I    plot_offset;
         wxString    msg;
 
         try
@@ -914,13 +921,8 @@ void SCH_PLOTTER::createDXFFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
                 aReporter->Report( msg, RPT_SEVERITY_ERROR );
             }
 
-            if( m_schFrame )
-                m_schFrame->SetCurrentSheet( oldsheetpath );
-            else
-                m_schematic->SetCurrentSheet( oldsheetpath );
-
+            m_schematic->SetCurrentSheet( oldsheetpath );
             m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
             m_schematic->SetSheetNumberAndCount();
             return;
         }
@@ -929,19 +931,12 @@ void SCH_PLOTTER::createDXFFiles( const SCH_PLOT_SETTINGS& aPlotSettings,
     if( aReporter )
         aReporter->ReportTail( _( "Done." ), RPT_SEVERITY_INFO );
 
-    if( m_schFrame )
-        m_schFrame->SetCurrentSheet( oldsheetpath );
-    else
-        m_schematic->SetCurrentSheet( oldsheetpath );
-
-    m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
-    m_schematic->SetSheetNumberAndCount();
+    restoreEnvironment( nullptr, oldsheetpath );
 }
 
 
 bool SCH_PLOTTER::plotOneSheetDXF( const wxString& aFileName, SCH_SCREEN* aScreen,
-                                   RENDER_SETTINGS* aRenderSettings, const wxPoint& aPlotOffset,
+                                   RENDER_SETTINGS* aRenderSettings, const VECTOR2I& aPlotOffset,
                                    double aScale, const SCH_PLOT_SETTINGS& aPlotSettings )
 {
     aRenderSettings->LoadColors( m_colorSettings );
@@ -976,7 +971,7 @@ bool SCH_PLOTTER::plotOneSheetDXF( const wxString& aFileName, SCH_SCREEN* aScree
         wxString sheetPath = m_schematic->CurrentSheet().PathHumanReadable();
         COLOR4D  color = plotter->RenderSettings()->GetLayerColor( LAYER_SCHEMATIC_DRAWINGSHEET );
 
-        PlotDrawingSheet( plotter, &m_schematic->Prj(), 
+        PlotDrawingSheet( plotter, &m_schematic->Prj(),
                           aScreen->GetTitleBlock(),
                           pageInfo,
                           aScreen->Schematic()->GetProperties(), aScreen->GetPageNumber(),
@@ -985,7 +980,7 @@ bool SCH_PLOTTER::plotOneSheetDXF( const wxString& aFileName, SCH_SCREEN* aScree
                           aScreen->GetVirtualPageNumber() == 1 );
     }
 
-    aScreen->Plot( plotter );
+    aScreen->Plot( plotter, aPlotSettings );
 
     // finish
     plotter->EndPlot();
@@ -997,17 +992,15 @@ bool SCH_PLOTTER::plotOneSheetDXF( const wxString& aFileName, SCH_SCREEN* aScree
 
 void SCH_PLOTTER::restoreEnvironment( PDF_PLOTTER* aPlotter, SCH_SHEET_PATH& aOldsheetpath )
 {
-    aPlotter->EndPlot();
-    delete aPlotter;
-
-    // Restore the previous sheet
-    if( m_schFrame )
-        m_schFrame->SetCurrentSheet( aOldsheetpath );
-    else
-        m_schematic->SetCurrentSheet( aOldsheetpath );
+    if( aPlotter )
+    {
+        aPlotter->EndPlot();
+        delete aPlotter;
+    }
 
+    // Restore the initial sheet
+    m_schematic->SetCurrentSheet( aOldsheetpath );
     m_schematic->CurrentSheet().UpdateAllScreenReferences();
-
     m_schematic->SetSheetNumberAndCount();
 }
 
@@ -1069,4 +1062,4 @@ void SCH_PLOTTER::Plot( PLOT_FORMAT aPlotFormat, const SCH_PLOT_SETTINGS& aPlotS
     case PLOT_FORMAT::SVG: createSVGFiles( aPlotSettings, aRenderSettings, aReporter ); break;
     case PLOT_FORMAT::HPGL: createHPGLFiles( aPlotSettings, aRenderSettings, aReporter ); break;
     }
-}
\ No newline at end of file
+}
