@@ -24,7 +24,7 @@
  */
 
 /**
- * @file component_references_lister.cpp
+ * @file sch_reference_list.cpp
  * @brief functions to create a symbol flat list and to annotate schematic.
  */
 
@@ -155,26 +155,6 @@ bool SCH_REFERENCE_LIST::sortByTimeStamp( const SCH_REFERENCE& item1,
 }
 
 
-int SCH_REFERENCE_LIST::FindUnit( size_t aIndex, int aUnit, bool aIncludeNew ) const
-{
-    int NumRef = m_flatList[aIndex].m_numRef;
-
-    for( size_t ii = 0; ii < m_flatList.size(); ii++ )
-    {
-        if(  ( aIndex == ii )
-          || ( m_flatList[ii].m_isNew && !aIncludeNew )
-          || ( m_flatList[ii].m_numRef != NumRef )
-          || ( m_flatList[aIndex].CompareRef( m_flatList[ii] ) != 0 ) )
-            continue;
-
-        if( m_flatList[ii].m_unit == aUnit )
-            return (int) ii;
-    }
-
-    return -1;
-}
-
-
 int SCH_REFERENCE_LIST::FindRefByFullPath( const wxString& aFullPath ) const
 {
     for( size_t i = 0; i < m_flatList.size(); ++i )
@@ -564,6 +544,8 @@ void SCH_REFERENCE_LIST::Annotate( bool aUseSheetNum, int aSheetIntervalId, int
         if( aStartAtCurrent && ref_unit.m_numRef > 0 )
             minRefId = ref_unit.m_numRef;
 
+        wxCHECK( ref_unit.GetLibPart(), /* void */ );
+
         // Annotation of one part per package symbols (trivial case).
         if( ref_unit.GetLibPart()->GetUnitCount() <= 1 )
         {
@@ -573,6 +555,7 @@ void SCH_REFERENCE_LIST::Annotate( bool aUseSheetNum, int aSheetIntervalId, int
                 GetRefsInUse( first, idList, minRefId );
                 LastReferenceNumber = createFirstFreeRefId( idList, minRefId );
                 ref_unit.m_numRef = LastReferenceNumber;
+                ref_unit.m_numRefStr = wxString::Format( "%d", LastReferenceNumber );
             }
 
             ref_unit.m_flag  = 1;
@@ -591,6 +574,7 @@ void SCH_REFERENCE_LIST::Annotate( bool aUseSheetNum, int aSheetIntervalId, int
             {
                 LastReferenceNumber = FindFirstUnusedReference( ref_unit, minRefId, units );
                 ref_unit.m_numRef = LastReferenceNumber;
+                ref_unit.m_numRefStr = wxString::Format( "%d", LastReferenceNumber );
                 ref_unit.m_isNew = false;
                 ref_unit.m_flag = 1;
             }
@@ -628,6 +612,7 @@ void SCH_REFERENCE_LIST::Annotate( bool aUseSheetNum, int aSheetIntervalId, int
                     if( inUseRefs.find( ref_candidate ) == inUseRefs.end() )
                     {
                         m_flatList[jj].m_numRef = ref_unit.m_numRef;
+                        m_flatList[jj].m_numRefStr = ref_unit.m_numRefStr;
                         m_flatList[jj].m_isNew = false;
                         m_flatList[jj].m_flag = 1;
 
@@ -646,14 +631,14 @@ void SCH_REFERENCE_LIST::Annotate( bool aUseSheetNum, int aSheetIntervalId, int
             std::vector<int> units = { ref_unit.GetUnit() };
             LastReferenceNumber = FindFirstUnusedReference( ref_unit, minRefId, units );
             ref_unit.m_numRef = LastReferenceNumber;
+            ref_unit.m_numRefStr = wxString::Format( "%d", LastReferenceNumber );
             ref_unit.m_isNew = false;
             ref_unit.m_flag = 1;
         }
     }
 
     // Remove aAdditionalRefs references
-    for( size_t i = originalSize; i < ( aAdditionalRefs.GetCount() + originalSize ); i++ )
-        RemoveItem( originalSize );
+    m_flatList.erase( m_flatList.begin() + originalSize, m_flatList.end() );
 
     wxASSERT( originalSize == GetCount() ); // Make sure we didn't make a mistake
 }
@@ -680,7 +665,7 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
         if( m_flatList[ii].m_isNew )    // Not yet annotated
         {
             if( m_flatList[ii].m_numRef >= 0 )
-                tmp << m_flatList[ii].m_numRef;
+                tmp << m_flatList[ii].m_numRefStr;
             else
                 tmp = wxT( "?" );
 
@@ -708,14 +693,14 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
         if( std::max( m_flatList[ii].GetLibPart()->GetUnitCount(), 1 ) < m_flatList[ii].m_unit )
         {
             if( m_flatList[ii].m_numRef >= 0 )
-                tmp << m_flatList[ii].m_numRef;
+                tmp << m_flatList[ii].m_numRefStr;
             else
                 tmp = wxT( "?" );
 
             msg.Printf( _( "Error: symbol %s%s%s (unit %d) exceeds units defined (%d)" ),
                         m_flatList[ii].GetRef(),
                         tmp,
-                        LIB_SYMBOL::SubReference( m_flatList[ii].m_unit ),
+                        m_flatList[ii].GetSymbol()->SubReference( m_flatList[ii].GetUnit() ),
                         m_flatList[ii].m_unit,
                         m_flatList[ii].GetLibPart()->GetUnitCount() );
 
@@ -747,15 +732,15 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
         if( first.m_unit == second.m_unit )
         {
             if( first.m_numRef >= 0 )
-                tmp << first.m_numRef;
+                tmp << first.m_numRefStr;
             else
                 tmp = wxT( "?" );
 
             msg.Printf( _( "Duplicate items %s%s%s\n" ),
                         first.GetRef(),
                         tmp,
-                        first.GetLibPart()->GetUnitCount() > 1 ?
-                            LIB_SYMBOL::SubReference( first.m_unit ) : wxT( "" ) );
+                        first.GetLibPart()->GetUnitCount() > 1 ? first.GetSymbol()->SubReference( first.GetUnit() )
+                                                               : wxString( wxT( "" ) ) );
 
             aHandler( ERCE_DUPLICATE_REFERENCE, msg, &first, &m_flatList[ii+1] );
             error++;
@@ -767,22 +752,22 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
         if( first.GetLibPart()->GetUnitCount() != second.GetLibPart()->GetUnitCount() )
         {
             if( first.m_numRef >= 0 )
-                tmp << first.m_numRef;
+                tmp << first.m_numRefStr;
             else
                 tmp = wxT( "?" );
 
             if( second.m_numRef >= 0 )
-                tmp2 << second.m_numRef;
+                tmp2 << second.m_numRefStr;
             else
                 tmp2 = wxT( "?" );
 
             msg.Printf( _( "Differing unit counts for item %s%s%s and %s%s%s\n" ),
-                    first.GetRef(),
-                    tmp,
-                    LIB_SYMBOL::SubReference( first.m_unit ),
-                    second.GetRef(),
-                    tmp2,
-                    LIB_SYMBOL::SubReference( second.m_unit )  );
+                        first.GetRef(),
+                        tmp,
+                        first.GetSymbol()->SubReference( first.GetUnit() ),
+                        second.GetRef(),
+                        tmp2,
+                        first.GetSymbol()->SubReference( second.GetUnit() )  );
 
             aHandler( ERCE_DUPLICATE_REFERENCE, msg, &first, &second );
             error++;
@@ -795,11 +780,11 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
             msg.Printf( _( "Different values for %s%d%s (%s) and %s%d%s (%s)" ),
                         first.GetRef(),
                         first.m_numRef,
-                        LIB_SYMBOL::SubReference( first.m_unit ),
+                        first.GetSymbol()->SubReference( first.GetUnit() ),
                         first.m_value,
                         second.GetRef(),
                         second.m_numRef,
-                        LIB_SYMBOL::SubReference( second.m_unit ),
+                        first.GetSymbol()->SubReference( second.GetUnit() ),
                         second.m_value );
 
             aHandler( ERCE_DIFFERENT_UNIT_VALUE, msg, &first, &second );
@@ -811,16 +796,13 @@ int SCH_REFERENCE_LIST::CheckAnnotation( ANNOTATION_ERROR_HANDLER aHandler )
 }
 
 
-SCH_REFERENCE::SCH_REFERENCE( SCH_SYMBOL* aSymbol, LIB_SYMBOL* aLibSymbol,
-                              const SCH_SHEET_PATH& aSheetPath )
+SCH_REFERENCE::SCH_REFERENCE( SCH_SYMBOL* aSymbol, const SCH_SHEET_PATH& aSheetPath )
 {
     wxASSERT( aSymbol != nullptr );
 
     m_rootSymbol = aSymbol;
-    m_libPart    = aLibSymbol;     // Warning: can be nullptr for orphan symbols
-                                   // (i.e. with a symbol library not found)
     m_unit       = aSymbol->GetUnitSelection( &aSheetPath );
-    m_footprint  = aSymbol->GetFootprintFieldText( true );
+    m_footprint  = aSymbol->GetFootprintFieldText( true, &aSheetPath, false );
     m_sheetPath  = aSheetPath;
     m_isNew      = false;
     m_flag       = 0;
@@ -836,10 +818,10 @@ SCH_REFERENCE::SCH_REFERENCE( SCH_SYMBOL* aSymbol, LIB_SYMBOL* aLibSymbol,
 
     m_numRef = -1;
 
-    if( aSymbol->GetValueFieldText( false ).IsEmpty() )
+    if( aSymbol->GetValueFieldText( false, &aSheetPath, false ).IsEmpty() )
         aSymbol->SetValueFieldText( wxT( "~" ) );
 
-    m_value = aSymbol->GetValueFieldText( false );
+    m_value = aSymbol->GetValueFieldText( false, &aSheetPath, false );
 }
 
 
@@ -850,7 +832,7 @@ void SCH_REFERENCE::Annotate()
     else
         m_ref = TO_UTF8( GetRef() << GetRefNumber() );
 
-    m_rootSymbol->SetRef( &m_sheetPath, FROM_UTF8( m_ref.c_str() ) );
+    m_rootSymbol->SetRef( &m_sheetPath, From_UTF8( m_ref.c_str() ) );
     m_rootSymbol->SetUnit( m_unit );
     m_rootSymbol->SetUnitSelection( &m_sheetPath, m_unit );
 }
@@ -871,6 +853,7 @@ void SCH_REFERENCE::Split()
     std::string refText = GetRefStr();
 
     m_numRef = -1;
+    m_numRefStr.Clear();
 
     int ll = refText.length() - 1;
 
@@ -902,7 +885,8 @@ void SCH_REFERENCE::Split()
                     m_numRef = atoi( cp );
                 }
 
-                refText.erase( ll+1 );  // delete from ll+1 to end
+                m_numRefStr = std::string( refText, ll + 1 );
+                refText.erase( ll + 1 );
                 break;
             }
         }
@@ -922,7 +906,9 @@ bool SCH_REFERENCE::IsSplitNeeded()
 }
 
 
-wxString SCH_REFERENCE_LIST::Shorthand( std::vector<SCH_REFERENCE> aList )
+wxString SCH_REFERENCE_LIST::Shorthand( std::vector<SCH_REFERENCE> aList,
+                                        const wxString&            refDelimiter,
+                                        const wxString&            refRangeDelimiter )
 {
     wxString retVal;
     size_t   i = 0;
@@ -939,25 +925,28 @@ wxString SCH_REFERENCE_LIST::Shorthand( std::vector<SCH_REFERENCE> aList )
                && aList[ i + range ].m_numRef == int( numRef + range ) )
         {
             range++;
+
+            if( range == 2 && refRangeDelimiter.IsEmpty() )
+                break;
         }
 
         if( !retVal.IsEmpty() )
-            retVal << wxT( ", " );
+            retVal << refDelimiter;
 
         if( range == 1 )
         {
             retVal << ref << aList[ i ].GetRefNumber();
         }
-        else if( range == 2 )
+        else if( range == 2 || refRangeDelimiter.IsEmpty() )
         {
             retVal << ref << aList[ i ].GetRefNumber();
-            retVal << wxT( ", " );
+            retVal << refDelimiter;
             retVal << ref << aList[ i + 1 ].GetRefNumber();
         }
         else
         {
             retVal << ref << aList[ i ].GetRefNumber();
-            retVal << wxT( "-" );
+            retVal << refRangeDelimiter;
             retVal << ref << aList[ i + ( range - 1 ) ].GetRefNumber();
         }
 
@@ -966,3 +955,19 @@ wxString SCH_REFERENCE_LIST::Shorthand( std::vector<SCH_REFERENCE> aList )
 
     return retVal;
 }
+
+
+#if defined( DEBUG )
+void SCH_REFERENCE_LIST::Show( const char* aPrefix )
+{
+    printf( "%s\n", aPrefix );
+
+    for( unsigned i = 0; i < m_flatList.size(); ++i )
+    {
+        SCH_REFERENCE& schref = m_flatList[i];
+
+        printf( " [%-2d] ref:%-8s num:%-3d lib_part:%s\n", i, schref.m_ref.ToStdString().c_str(),
+                schref.m_numRef, TO_UTF8( schref.GetLibPart()->GetName() ) );
+    }
+}
+#endif
