@@ -23,10 +23,11 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <advanced_config.h>
 #include <base_units.h>
 #include <pgm_base.h>
 #include <sch_edit_frame.h>
-#include <plotters/plotter.h>
+#include <sch_plotter.h>
 #include <widgets/msgpanel.h>
 #include <bitmaps.h>
 #include <string_utils.h>
@@ -48,70 +49,6 @@
 using KIGFX::SCH_RENDER_SETTINGS;
 
 
-TEXT_SPIN_STYLE TEXT_SPIN_STYLE::RotateCW()
-{
-    SPIN newSpin = m_spin;
-
-    switch( m_spin )
-    {
-    case TEXT_SPIN_STYLE::LEFT:   newSpin = TEXT_SPIN_STYLE::UP;     break;
-    case TEXT_SPIN_STYLE::UP:     newSpin = TEXT_SPIN_STYLE::RIGHT;  break;
-    case TEXT_SPIN_STYLE::RIGHT:  newSpin = TEXT_SPIN_STYLE::BOTTOM; break;
-    case TEXT_SPIN_STYLE::BOTTOM: newSpin = TEXT_SPIN_STYLE::LEFT;   break;
-    }
-
-    return TEXT_SPIN_STYLE( newSpin );
-}
-
-
-TEXT_SPIN_STYLE TEXT_SPIN_STYLE::RotateCCW()
-{
-    SPIN newSpin = m_spin;
-
-    switch( m_spin )
-    {
-    case TEXT_SPIN_STYLE::LEFT:   newSpin = TEXT_SPIN_STYLE::BOTTOM; break;
-    case TEXT_SPIN_STYLE::BOTTOM: newSpin = TEXT_SPIN_STYLE::RIGHT;  break;
-    case TEXT_SPIN_STYLE::RIGHT:  newSpin = TEXT_SPIN_STYLE::UP;     break;
-    case TEXT_SPIN_STYLE::UP:     newSpin = TEXT_SPIN_STYLE::LEFT;   break;
-    }
-
-    return TEXT_SPIN_STYLE( newSpin );
-}
-
-
-TEXT_SPIN_STYLE TEXT_SPIN_STYLE::MirrorX()
-{
-    SPIN newSpin = m_spin;
-
-    switch( m_spin )
-    {
-    case TEXT_SPIN_STYLE::UP:     newSpin = TEXT_SPIN_STYLE::BOTTOM; break;
-    case TEXT_SPIN_STYLE::BOTTOM: newSpin = TEXT_SPIN_STYLE::UP;     break;
-    case TEXT_SPIN_STYLE::LEFT:                                      break;
-    case TEXT_SPIN_STYLE::RIGHT:                                     break;
-    }
-
-    return TEXT_SPIN_STYLE( newSpin );
-}
-
-
-TEXT_SPIN_STYLE TEXT_SPIN_STYLE::MirrorY()
-{
-    SPIN newSpin = m_spin;
-
-    switch( m_spin )
-    {
-    case TEXT_SPIN_STYLE::LEFT:   newSpin = TEXT_SPIN_STYLE::RIGHT; break;
-    case TEXT_SPIN_STYLE::RIGHT:  newSpin = TEXT_SPIN_STYLE::LEFT;  break;
-    case TEXT_SPIN_STYLE::UP:                                       break;
-    case TEXT_SPIN_STYLE::BOTTOM:                                   break;
-    }
-
-    return TEXT_SPIN_STYLE( newSpin );
-}
-
-
 SCH_TEXT::SCH_TEXT( const VECTOR2I& pos, const wxString& text, KICAD_T aType ) :
         SCH_ITEM( nullptr, aType ),
         EDA_TEXT( schIUScale, text )
@@ -119,28 +56,31 @@ SCH_TEXT::SCH_TEXT( const VECTOR2I& pos, const wxString& text, KICAD_T aType ) :
     m_layer = LAYER_NOTES;
 
     SetTextPos( pos );
-    SetTextSpinStyle( TEXT_SPIN_STYLE::LEFT );
     SetMultilineAllowed( true );
+
+    m_excludedFromSim = false;
 }
 
 
 SCH_TEXT::SCH_TEXT( const SCH_TEXT& aText ) :
         SCH_ITEM( aText ),
-        EDA_TEXT( aText ),
-        m_spin_style( aText.m_spin_style )
-{ }
+        EDA_TEXT( aText )
+{
+    m_excludedFromSim = aText.m_excludedFromSim;
+}
 
 
 VECTOR2I SCH_TEXT::GetSchematicTextOffset( const RENDER_SETTINGS* aSettings ) const
 {
-    return VECTOR2I( 0, 0 );
+    // Fudge factor to match KiCad 6
+    return VECTOR2I( 0, -2500 );
 }
 
 
 void SCH_TEXT::MirrorHorizontally( int aCenter )
 {
-    // Text is NOT really mirrored; it is moved to a suitable horizontal position
-    SetTextSpinStyle( GetTextSpinStyle().MirrorY() );
+    if( GetTextAngle() == ANGLE_HORIZONTAL )
+        FlipHJustify();
 
     SetTextX( MIRRORVAL( GetTextPos().x, aCenter ) );
 }
@@ -148,8 +88,8 @@ void SCH_TEXT::MirrorHorizontally( int aCenter )
 
 void SCH_TEXT::MirrorVertically( int aCenter )
 {
-    // Text is NOT really mirrored; it is moved to a suitable vertical position
-    SetTextSpinStyle( GetTextSpinStyle().MirrorX() );
+    if( GetTextAngle() == ANGLE_VERTICAL )
+        FlipHJustify();
 
     SetTextY( MIRRORVAL( GetTextPos().y, aCenter ) );
 }
@@ -169,66 +109,33 @@ void SCH_TEXT::Rotate( const VECTOR2I& aCenter )
 
 void SCH_TEXT::Rotate90( bool aClockwise )
 {
-    if( aClockwise )
-        SetTextSpinStyle( GetTextSpinStyle().RotateCW() );
-    else
-        SetTextSpinStyle( GetTextSpinStyle().RotateCCW() );
+    if( ( GetTextAngle() == ANGLE_HORIZONTAL && aClockwise )
+     || ( GetTextAngle() == ANGLE_VERTICAL && !aClockwise ) )
+    {
+        FlipHJustify();
+    }
+
+    SetTextAngle( GetTextAngle() == ANGLE_VERTICAL ? ANGLE_HORIZONTAL : ANGLE_VERTICAL );
 }
 
 
 void SCH_TEXT::MirrorSpinStyle( bool aLeftRight )
 {
-    if( aLeftRight )
-        SetTextSpinStyle( GetTextSpinStyle().MirrorY() );
-    else
-        SetTextSpinStyle( GetTextSpinStyle().MirrorX() );
-}
-
-
-void SCH_TEXT::SetTextSpinStyle( TEXT_SPIN_STYLE aSpinStyle )
-{
-    m_spin_style = aSpinStyle;
-
-    // Assume "Right" and Left" mean which side of the anchor the text will be on
-    // Thus we want to left justify text up against the anchor if we are on the right
-    switch( aSpinStyle )
+    if( ( GetTextAngle() == ANGLE_HORIZONTAL && aLeftRight )
+     || ( GetTextAngle() == ANGLE_VERTICAL && !aLeftRight ) )
     {
-    default:
-        wxFAIL_MSG( "Bad spin style" );
-        m_spin_style = TEXT_SPIN_STYLE::RIGHT; // Handle the error spin style by resetting
-        KI_FALLTHROUGH;
-
-    case TEXT_SPIN_STYLE::RIGHT:            // Horiz Normal Orientation
-        SetTextAngle( ANGLE_HORIZONTAL );
-        SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
-        break;
-
-    case TEXT_SPIN_STYLE::UP:               // Vert Orientation UP
-        SetTextAngle( ANGLE_VERTICAL );
-        SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
-        break;
-
-    case TEXT_SPIN_STYLE::LEFT:             // Horiz Orientation - Right justified
-        SetTextAngle( ANGLE_HORIZONTAL );
-        SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
-        break;
-
-    case TEXT_SPIN_STYLE::BOTTOM:           //  Vert Orientation BOTTOM
-        SetTextAngle( ANGLE_VERTICAL );
-        SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
-        break;
+        FlipHJustify();
     }
-
-    SetVertJustify( GR_TEXT_V_ALIGN_BOTTOM );
 }
 
 
 void SCH_TEXT::SwapData( SCH_ITEM* aItem )
 {
+    SCH_ITEM::SwapFlags( aItem );
+
     SCH_TEXT* item = static_cast<SCH_TEXT*>( aItem );
 
     std::swap( m_layer, item->m_layer );
-    std::swap( m_spin_style, item->m_spin_style );
 
     SwapText( *item );
     SwapAttributes( *item );
@@ -243,7 +150,7 @@ bool SCH_TEXT::operator<( const SCH_ITEM& aItem ) const
     auto other = static_cast<const SCH_TEXT*>( &aItem );
 
     if( GetLayer() != other->GetLayer() )
-            return GetLayer() < other->GetLayer();
+        return GetLayer() < other->GetLayer();
 
     if( GetPosition().x != other->GetPosition().x )
         return GetPosition().x < other->GetPosition().x;
@@ -251,6 +158,9 @@ bool SCH_TEXT::operator<( const SCH_ITEM& aItem ) const
     if( GetPosition().y != other->GetPosition().y )
         return GetPosition().y < other->GetPosition().y;
 
+    if( GetExcludedFromSim() != other->GetExcludedFromSim() )
+        return GetExcludedFromSim() - other->GetExcludedFromSim();
+
     return GetText() < other->GetText();
 }
 
@@ -339,21 +249,29 @@ const BOX2I SCH_TEXT::GetBoundingBox() const
 }
 
 
-wxString SCH_TEXT::GetShownText( int aDepth, bool aAllowExtraText ) const
+wxString SCH_TEXT::GetShownText( const SCH_SHEET_PATH* aPath, bool aAllowExtraText,
+                                 int aDepth ) const
 {
+    SCH_SHEET* sheet = nullptr;
+
+    if( aPath )
+        sheet = aPath->Last();
+    else if( SCHEMATIC* schematic = Schematic() )
+        sheet = schematic->CurrentSheet().Last();
+
     std::function<bool( wxString* )> textResolver =
             [&]( wxString* token ) -> bool
             {
-                if( SCH_SHEET* sheet = Schematic()->CurrentSheet().Last() )
+                if( sheet )
                 {
-                    if( sheet->ResolveTextVar( token, aDepth + 1 ) )
+                    if( sheet->ResolveTextVar( aPath, token, aDepth + 1 ) )
                         return true;
                 }
 
                 return false;
             };
 
-    wxString text = EDA_TEXT::GetShownText();
+    wxString text = EDA_TEXT::GetShownText( aAllowExtraText, aDepth );
 
     if( text == wxS( "~" ) ) // Legacy placeholder for empty string
     {
@@ -361,7 +279,7 @@ wxString SCH_TEXT::GetShownText( int aDepth, bool aAllowExtraText ) const
     }
     else if( HasTextVars() )
     {
-        if( aDepth < 10 )
+        if( aDepth < ADVANCED_CFG::GetCfg().m_ResolveTextRecursionDepth )
             text = ExpandTextVars( text, &textResolver );
     }
 
@@ -372,7 +290,7 @@ wxString SCH_TEXT::GetShownText( int aDepth, bool aAllowExtraText ) const
 void SCH_TEXT::DoHypertextAction( EDA_DRAW_FRAME* aFrame ) const
 {
     wxCHECK_MSG( IsHypertext(), /* void */,
-                 "Calling a hypertext menu on a SCH_TEXT with no hyperlink?" );
+                 wxT( "Calling a hypertext menu on a SCH_TEXT with no hyperlink?" ) );
 
     SCH_NAVIGATE_TOOL* navTool = aFrame->GetToolManager()->GetTool<SCH_NAVIGATE_TOOL>();
     navTool->HypertextCommand( m_hyperlink );
@@ -381,7 +299,7 @@ void SCH_TEXT::DoHypertextAction( EDA_DRAW_FRAME* aFrame ) const
 
 wxString SCH_TEXT::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 {
-    return wxString::Format( _( "Graphic Text '%s'" ), KIUI::EllipsizeMenuText( GetShownText() ) );
+    return wxString::Format( _( "Graphic Text '%s'" ), KIUI::EllipsizeMenuText( GetText() ) );
 }
 
 
@@ -419,7 +337,8 @@ void SCH_TEXT::ViewGetLayers( int aLayers[], int& aCount ) const
 }
 
 
-void SCH_TEXT::Plot( PLOTTER* aPlotter, bool aBackground ) const
+void SCH_TEXT::Plot( PLOTTER* aPlotter, bool aBackground,
+                     const SCH_PLOT_SETTINGS& aPlotSettings ) const
 {
     if( aBackground )
         return;
@@ -457,7 +376,7 @@ void SCH_TEXT::Plot( PLOTTER* aPlotter, bool aBackground ) const
 
     std::vector<VECTOR2I> positions;
     wxArrayString strings_list;
-    wxStringSplit( GetShownText(), strings_list, '\n' );
+    wxStringSplit( GetShownText( true ), strings_list, '\n' );
     positions.reserve( strings_list.Count() );
 
     GetLinePositions( positions, (int) strings_list.Count() );
@@ -470,7 +389,7 @@ void SCH_TEXT::Plot( PLOTTER* aPlotter, bool aBackground ) const
     {
         VECTOR2I  textpos = positions[ii] + text_offset;
         wxString& txt = strings_list.Item( ii );
-        aPlotter->PlotText( textpos, color, txt, attrs, font );
+        aPlotter->PlotText( textpos, color, txt, attrs, font, GetFontMetrics() );
     }
 
     if( HasHyperlink() )
@@ -485,6 +404,9 @@ void SCH_TEXT::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_IT
     // Don't use GetShownText() here; we want to show the user the variable references
     aList.emplace_back( _( "Graphic Text" ), KIUI::EllipsizeStatusText( aFrame, GetText() ) );
 
+    if( m_excludedFromSim )
+        aList.emplace_back( _( "Exclude from" ), _( "Simulation" ) );
+
     aList.emplace_back( _( "Font" ), GetFont() ? GetFont()->GetName() : _( "Default" ) );
 
     wxString textStyle[] = { _( "Normal" ), _( "Italic" ), _( "Bold" ), _( "Bold Italic" ) };
@@ -493,18 +415,53 @@ void SCH_TEXT::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_IT
 
     aList.emplace_back( _( "Text Size" ), aFrame->MessageTextFromValue( GetTextWidth() ) );
 
-    switch( GetTextSpinStyle() )
+    switch( GetHorizJustify() )
     {
-    case TEXT_SPIN_STYLE::LEFT:   msg = _( "Align right" );   break;
-    case TEXT_SPIN_STYLE::UP:     msg = _( "Align bottom" );  break;
-    case TEXT_SPIN_STYLE::RIGHT:  msg = _( "Align left" );    break;
-    case TEXT_SPIN_STYLE::BOTTOM: msg = _( "Align top" );     break;
-    default:                      msg = wxT( "???" );         break;
+    case GR_TEXT_H_ALIGN_LEFT:   msg = _( "Align left" );   break;
+    case GR_TEXT_H_ALIGN_CENTER: msg = _( "Align center" ); break;
+    case GR_TEXT_H_ALIGN_RIGHT:  msg = _( "Align right" );  break;
     }
 
     aList.emplace_back( _( "Justification" ), msg );
 }
 
+bool SCH_TEXT::operator==( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return false;
+
+    const SCH_TEXT* other = static_cast<const SCH_TEXT*>( &aOther );
+
+    if( GetLayer() != other->GetLayer() )
+        return false;
+
+    if( GetExcludedFromSim() != other->GetExcludedFromSim() )
+        return false;
+
+    return EDA_TEXT::operator==( *other );
+}
+
+
+double SCH_TEXT::Similarity( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return 0.0;
+
+    const SCH_TEXT* other = static_cast<const SCH_TEXT*>( &aOther );
+
+    double retval = 1.0;
+
+    if( GetLayer() != other->GetLayer() )
+        retval *= 0.9;
+
+    if( GetExcludedFromSim() != other->GetExcludedFromSim() )
+        retval *= 0.9;
+
+    retval *= EDA_TEXT::Similarity( *other );
+
+    return retval;
+}
+
 
 #if defined(DEBUG)
 
@@ -523,3 +480,28 @@ void SCH_TEXT::Show( int nestLevel, std::ostream& os ) const
 #endif
 
 
+static struct SCH_TEXT_DESC
+{
+    SCH_TEXT_DESC()
+    {
+        PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
+        REGISTER_TYPE( SCH_TEXT );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_TEXT, SCH_ITEM> );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_TEXT, EDA_TEXT> );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_TEXT ), TYPE_HASH( SCH_ITEM ) );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Mirrored" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Visible" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Width" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Height" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Thickness" ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_TEXT, int>( _HKI( "Text Size" ),
+                &SCH_TEXT::SetSchTextSize, &SCH_TEXT::GetSchTextSize, PROPERTY_DISPLAY::PT_SIZE ),
+                _HKI( "Text Properties" ) );
+
+        // Orientation is exposed differently in schematic; mask the base for now
+        propMgr.Mask( TYPE_HASH( SCH_TEXT ), TYPE_HASH( EDA_TEXT ), _HKI( "Orientation" ) );
+    }
+} _SCH_TEXT_DESC;
