@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <advanced_config.h>
 #include <base_units.h>
 #include <pgm_base.h>
 #include <sch_edit_frame.h>
@@ -44,7 +45,7 @@ using KIGFX::SCH_RENDER_SETTINGS;
 
 
 SCH_TEXTBOX::SCH_TEXTBOX( int aLineWidth, FILL_T aFillType, const wxString& text ) :
-        SCH_SHAPE( SHAPE_T::RECT, aLineWidth, aFillType, SCH_TEXTBOX_T ),
+        SCH_SHAPE( SHAPE_T::RECTANGLE, aLineWidth, aFillType, SCH_TEXTBOX_T ),
         EDA_TEXT( schIUScale, text )
 {
     m_layer = LAYER_NOTES;
@@ -52,13 +53,17 @@ SCH_TEXTBOX::SCH_TEXTBOX( int aLineWidth, FILL_T aFillType, const wxString& text
     SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
     SetVertJustify( GR_TEXT_V_ALIGN_TOP );
     SetMultilineAllowed( true );
+
+    m_excludedFromSim = false;
 }
 
 
 SCH_TEXTBOX::SCH_TEXTBOX( const SCH_TEXTBOX& aText ) :
         SCH_SHAPE( aText ),
         EDA_TEXT( aText )
-{ }
+{
+    m_excludedFromSim = aText.m_excludedFromSim;
+}
 
 
 int SCH_TEXTBOX::GetTextMargin() const
@@ -182,14 +187,14 @@ VECTOR2I SCH_TEXTBOX::GetDrawPos() const
 
 void SCH_TEXTBOX::SwapData( SCH_ITEM* aItem )
 {
+    SCH_SHAPE::SwapData( aItem );
+
     SCH_TEXTBOX* item = static_cast<SCH_TEXTBOX*>( aItem );
 
     std::swap( m_layer, item->m_layer );
 
     SwapText( *item );
     SwapAttributes( *item );
-
-    SCH_SHAPE::SwapData( aItem );
 }
 
 
@@ -209,6 +214,9 @@ bool SCH_TEXTBOX::operator<( const SCH_ITEM& aItem ) const
     if( GetPosition().y != other->GetPosition().y )
         return GetPosition().y < other->GetPosition().y;
 
+    if( GetExcludedFromSim() != other->GetExcludedFromSim() )
+        return GetExcludedFromSim() - other->GetExcludedFromSim();
+
     return GetText() < other->GetText();
 }
 
@@ -226,13 +234,13 @@ KIFONT::FONT* SCH_TEXTBOX::getDrawFont() const
 
 void SCH_TEXTBOX::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset )
 {
-    wxDC*          DC = aSettings->GetPrintDC();
-    int            penWidth = GetPenWidth();
-    bool           blackAndWhiteMode = GetGRForceBlackPenState();
-    VECTOR2I       pt1 = GetStart();
-    VECTOR2I       pt2 = GetEnd();
-    COLOR4D        color = GetStroke().GetColor();
-    PLOT_DASH_TYPE lineStyle = GetStroke().GetPlotStyle();
+    wxDC*      DC = aSettings->GetPrintDC();
+    int        penWidth = GetPenWidth();
+    bool       blackAndWhiteMode = GetGRForceBlackPenState();
+    VECTOR2I   pt1 = GetStart();
+    VECTOR2I   pt2 = GetEnd();
+    COLOR4D    color = GetStroke().GetColor();
+    LINE_STYLE lineStyle = GetStroke().GetLineStyle();
 
     if( GetFillMode() == FILL_T::FILLED_WITH_COLOR && !blackAndWhiteMode )
         GRFilledRect( DC, pt1, pt2, 0, GetFillColor(), GetFillColor() );
@@ -244,10 +252,10 @@ void SCH_TEXTBOX::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffs
         if( blackAndWhiteMode || color == COLOR4D::UNSPECIFIED )
             color = aSettings->GetLayerColor( m_layer );
 
-        if( lineStyle == PLOT_DASH_TYPE::DEFAULT )
-            lineStyle = PLOT_DASH_TYPE::SOLID;
+        if( lineStyle == LINE_STYLE::DEFAULT )
+            lineStyle = LINE_STYLE::SOLID;
 
-        if( lineStyle == PLOT_DASH_TYPE::SOLID )
+        if( lineStyle == LINE_STYLE::SOLID )
         {
             GRRect( DC, pt1, pt2, penWidth, color );
         }
@@ -278,25 +286,31 @@ void SCH_TEXTBOX::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffs
 }
 
 
-wxString SCH_TEXTBOX::GetShownText( int aDepth, bool aAllowExtraText ) const
+wxString SCH_TEXTBOX::GetShownText( const SCH_SHEET_PATH* aPath, bool aAllowExtraText,
+                                    int aDepth ) const
 {
+    SCH_SHEET* sheet = nullptr;
+
+    if( aPath )
+        sheet = aPath->Last();
+
     std::function<bool( wxString* )> textResolver =
             [&]( wxString* token ) -> bool
             {
-                if( SCH_SHEET* sheet = Schematic()->CurrentSheet().Last() )
+                if( sheet )
                 {
-                    if( sheet->ResolveTextVar( token, aDepth + 1 ) )
+                    if( sheet->ResolveTextVar( aPath, token, aDepth + 1 ) )
                         return true;
                 }
 
                 return false;
             };
 
-    wxString text = EDA_TEXT::GetShownText();
+    wxString text = EDA_TEXT::GetShownText( aAllowExtraText, aDepth );
 
     if( HasTextVars() )
     {
-        if( aDepth < 10 )
+        if( aDepth < ADVANCED_CFG::GetCfg().m_ResolveTextRecursionDepth )
             text = ExpandTextVars( text, &textResolver );
     }
 
@@ -305,7 +319,7 @@ wxString SCH_TEXTBOX::GetShownText( int aDepth, bool aAllowExtraText ) const
     if( !font )
         font = KIFONT::FONT::GetFont( GetDefaultFont(), IsBold(), IsItalic() );
 
-    VECTOR2D size = GetEnd() - GetStart();
+    VECTOR2I size = GetEnd() - GetStart();
     int      colWidth = GetTextAngle() == ANGLE_HORIZONTAL ? size.x : size.y;
 
     colWidth = abs( colWidth ) - GetTextMargin() * 2;
@@ -341,7 +355,7 @@ bool SCH_TEXTBOX::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy )
 void SCH_TEXTBOX::DoHypertextAction( EDA_DRAW_FRAME* aFrame ) const
 {
     wxCHECK_MSG( IsHypertext(), /* void */,
-                 "Calling a hypertext menu on a SCH_TEXTBOX with no hyperlink?" );
+                 wxT( "Calling a hypertext menu on a SCH_TEXTBOX with no hyperlink?" ) );
 
     SCH_NAVIGATE_TOOL* navTool = aFrame->GetToolManager()->GetTool<SCH_NAVIGATE_TOOL>();
     navTool->HypertextCommand( m_hyperlink );
@@ -360,18 +374,19 @@ BITMAPS SCH_TEXTBOX::GetMenuImage() const
 }
 
 
-void SCH_TEXTBOX::Plot( PLOTTER* aPlotter, bool aBackground ) const
+void SCH_TEXTBOX::Plot( PLOTTER* aPlotter, bool aBackground,
+                        const SCH_PLOT_SETTINGS& aPlotSettings ) const
 {
     if( aBackground )
     {
-        SCH_SHAPE::Plot( aPlotter, aBackground );
+        SCH_SHAPE::Plot( aPlotter, aBackground, aPlotSettings );
         return;
     }
 
     RENDER_SETTINGS* settings = aPlotter->RenderSettings();
     int              penWidth = GetPenWidth();
     COLOR4D          color = GetStroke().GetColor();
-    PLOT_DASH_TYPE   lineStyle = GetStroke().GetPlotStyle();
+    LINE_STYLE       lineStyle = GetStroke().GetLineStyle();
 
     if( penWidth > 0 )
     {
@@ -380,13 +395,13 @@ void SCH_TEXTBOX::Plot( PLOTTER* aPlotter, bool aBackground ) const
         if( !aPlotter->GetColorMode() || color == COLOR4D::UNSPECIFIED )
             color = settings->GetLayerColor( m_layer );
 
-        if( lineStyle == PLOT_DASH_TYPE::DEFAULT )
-            lineStyle = PLOT_DASH_TYPE::SOLID;
+        if( lineStyle == LINE_STYLE::DEFAULT )
+            lineStyle = LINE_STYLE::SOLID;
 
         aPlotter->SetColor( color );
         aPlotter->SetDash( penWidth, lineStyle );
         aPlotter->Rect( m_start, m_end, FILL_T::NO_FILL, penWidth );
-        aPlotter->SetDash( penWidth, PLOT_DASH_TYPE::SOLID );
+        aPlotter->SetDash( penWidth, LINE_STYLE::SOLID );
     }
 
     KIFONT::FONT* font = GetFont();
@@ -405,7 +420,7 @@ void SCH_TEXTBOX::Plot( PLOTTER* aPlotter, bool aBackground ) const
 
     std::vector<VECTOR2I> positions;
     wxArrayString strings_list;
-    wxStringSplit( GetShownText(), strings_list, '\n' );
+    wxStringSplit( GetShownText( true ), strings_list, '\n' );
     positions.reserve( strings_list.Count() );
 
     GetLinePositions( positions, (int) strings_list.Count() );
@@ -416,7 +431,8 @@ void SCH_TEXTBOX::Plot( PLOTTER* aPlotter, bool aBackground ) const
 
     for( unsigned ii = 0; ii < strings_list.Count(); ii++ )
     {
-        aPlotter->PlotText( positions[ii], color, strings_list.Item( ii ), attrs, font );
+        aPlotter->PlotText( positions[ii], color, strings_list.Item( ii ), attrs, font,
+                            GetFontMetrics() );
     }
 
     if( HasHyperlink() )
@@ -429,6 +445,9 @@ void SCH_TEXTBOX::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL
     // Don't use GetShownText() here; we want to show the user the variable references
     aList.emplace_back( _( "Text Box" ), KIUI::EllipsizeStatusText( aFrame, GetText() ) );
 
+    if( m_excludedFromSim )
+        aList.emplace_back( _( "Exclude from" ), _( "Simulation" ) );
+
     aList.emplace_back( _( "Font" ), GetFont() ? GetFont()->GetName() : _( "Default" ) );
 
     wxString textStyle[] = { _( "Normal" ), _( "Italic" ), _( "Bold" ), _( "Bold Italic" ) };
@@ -445,3 +464,66 @@ void SCH_TEXTBOX::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL
 
     m_stroke.GetMsgPanelInfo( aFrame, aList );
 }
+
+
+bool SCH_TEXTBOX::operator==( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return false;
+
+    const SCH_TEXTBOX& other = static_cast<const SCH_TEXTBOX&>( aOther );
+
+    if( m_excludedFromSim != other.m_excludedFromSim )
+        return false;
+
+    return SCH_SHAPE::operator==( aOther ) && EDA_TEXT::operator==( other );
+}
+
+
+double SCH_TEXTBOX::Similarity( const SCH_ITEM& aOther ) const
+{
+    if( aOther.Type() != Type() )
+        return 0.0;
+
+    auto other = static_cast<const SCH_TEXTBOX&>( aOther );
+
+    double similarity = 1.0;
+
+    if( m_excludedFromSim != other.m_excludedFromSim )
+        similarity *= 0.9;
+
+    similarity *= SCH_SHAPE::Similarity( aOther );
+    similarity *= EDA_TEXT::Similarity( other );
+
+    return similarity;
+}
+
+
+static struct SCH_TEXTBOX_DESC
+{
+    SCH_TEXTBOX_DESC()
+    {
+        PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
+        REGISTER_TYPE( SCH_TEXTBOX );
+
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_TEXTBOX, SCH_SHAPE> );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_TEXTBOX, EDA_SHAPE> );
+        propMgr.AddTypeCast( new TYPE_CAST<SCH_TEXTBOX, EDA_TEXT> );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( SCH_SHAPE ) );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_SHAPE ) );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_TEXT ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_SHAPE ), _HKI( "Shape" ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_TEXT ), _HKI( "Width" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_TEXT ), _HKI( "Height" ) );
+        propMgr.Mask( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_TEXT ), _HKI( "Thickness" ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_TEXTBOX, int>( _HKI( "Text Size" ),
+                    &SCH_TEXTBOX::SetSchTextSize, &SCH_TEXTBOX::GetSchTextSize,
+                    PROPERTY_DISPLAY::PT_SIZE ),
+                _HKI( "Text Properties" ) );
+
+        propMgr.Mask( TYPE_HASH( SCH_TEXTBOX ), TYPE_HASH( EDA_TEXT ), _HKI( "Orientation" ) );
+    }
+} _SCH_TEXTBOX_DESC;
