@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2016 Wayne Stambaugh <stambaughw@gmail.com>
  * Copyright (C) 2022 CERN
- * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +24,10 @@
  */
 
 
+#include <env_vars.h>
 #include <lib_id.h>
 #include <lib_table_lexer.h>
+#include <paths.h>
 #include <pgm_base.h>
 #include <search_stack.h>
 #include <settings/kicad_settings.h>
@@ -33,6 +35,8 @@
 #include <systemdirsappend.h>
 #include <symbol_lib_table.h>
 #include <lib_symbol.h>
+#include <sch_io/database/sch_io_database.h>
+#include <dialogs/dialog_database_lib_settings.h>
 
 #include <wx/dir.h>
 #include "sim/sim_model.h"
@@ -66,8 +70,8 @@ void SYMBOL_LIB_TABLE_ROW::SetType( const wxString& aType )
 {
     type = SCH_IO_MGR::EnumFromStr( aType );
 
-    if( SCH_IO_MGR::SCH_FILE_T( -1 ) == type )
-        type = SCH_IO_MGR::SCH_LEGACY;
+    if( type == SCH_IO_MGR::SCH_FILE_UNKNOWN )
+        type = SCH_IO_MGR::SCH_KICAD;
 
     plugin.release();
 }
@@ -79,7 +83,7 @@ bool SYMBOL_LIB_TABLE_ROW::Refresh()
     {
         wxArrayString dummyList;
 
-        plugin.set( SCH_IO_MGR::FindPlugin( type ) );
+        plugin.reset( SCH_IO_MGR::FindPlugin( type ) );
         SetLoaded( false );
         plugin->SetLibTable( static_cast<SYMBOL_LIB_TABLE*>( GetParent() ) );
         plugin->EnumerateSymbolLib( dummyList, GetFullURI( true ), GetProperties() );
@@ -100,6 +104,27 @@ void SYMBOL_LIB_TABLE_ROW::GetSubLibraryNames( std::vector<wxString>& aNames ) c
 }
 
 
+wxString SYMBOL_LIB_TABLE_ROW::GetSubLibraryDescription( const wxString& aName ) const
+{
+    if( !plugin )
+        return wxEmptyString;
+
+    return plugin->GetSubLibraryDescription( aName );
+}
+
+
+void SYMBOL_LIB_TABLE_ROW::ShowSettingsDialog( wxWindow* aParent ) const
+{
+    wxCHECK( plugin, /* void */ );
+
+    if( type != SCH_IO_MGR::SCH_DATABASE )
+        return;
+
+    DIALOG_DATABASE_LIB_SETTINGS dlg( aParent, static_cast<SCH_IO_DATABASE*>( plugin.get() ) );
+    dlg.ShowModal();
+}
+
+
 SYMBOL_LIB_TABLE::SYMBOL_LIB_TABLE( SYMBOL_LIB_TABLE* aFallBackTable ) :
     LIB_TABLE( aFallBackTable )
 {
@@ -281,10 +306,8 @@ void SYMBOL_LIB_TABLE::Format( OUTPUTFORMATTER* aOutput, int aIndentLevel ) cons
     aOutput->Print( aIndentLevel, "(sym_lib_table\n" );
     aOutput->Print( aIndentLevel + 1, "(version %d)\n", m_version );
 
-    for( LIB_TABLE_ROWS_CITER it = m_rows.begin();  it != m_rows.end();  ++it )
-    {
-        it->Format( aOutput, aIndentLevel+1 );
-    }
+    for( const LIB_TABLE_ROW& row : m_rows )
+        row.Format( aOutput, aIndentLevel + 1 );
 
     aOutput->Print( aIndentLevel, ")\n" );
 }
@@ -301,7 +324,6 @@ int SYMBOL_LIB_TABLE::GetModifyHash()
 
         if( !row || !row->plugin )
         {
-            wxFAIL;
             continue;
         }
 
@@ -346,10 +368,9 @@ SYMBOL_LIB_TABLE_ROW* SYMBOL_LIB_TABLE::FindRow( const wxString& aNickname, bool
     // instantiate a PLUGIN of the proper kind if it is not already in this
     // SYMBOL_LIB_TABLE_ROW.
     if( !row->plugin )
-    {
         row->setPlugin( SCH_IO_MGR::FindPlugin( row->type ) );
-        row->plugin->SetLibTable( this );
-    }
+
+    row->plugin->SetLibTable( this );
 
     return row;
 }
@@ -359,7 +380,9 @@ void SYMBOL_LIB_TABLE::LoadSymbolLib( std::vector<LIB_SYMBOL*>& aSymbolList,
                                       const wxString& aNickname, bool aPowerSymbolsOnly )
 {
     SYMBOL_LIB_TABLE_ROW* row = FindRow( aNickname, true );
-    wxCHECK( row && row->plugin, /* void */  );
+
+    if( !row || !row->plugin )
+        return;
 
     std::lock_guard<std::mutex> lock( row->GetMutex() );
 
@@ -430,7 +453,7 @@ SYMBOL_LIB_TABLE::SAVE_T SYMBOL_LIB_TABLE::SaveSymbol( const wxString& aNickname
     const SYMBOL_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxCHECK( row && row->plugin, SAVE_SKIPPED );
 
-    if( !row->plugin->IsSymbolLibWritable( row->GetFullURI( true ) ) )
+    if( !row->plugin->IsLibraryWritable( row->GetFullURI( true ) ) )
         return SAVE_SKIPPED;
 
     if( !aOverwrite )
@@ -472,7 +495,7 @@ bool SYMBOL_LIB_TABLE::IsSymbolLibWritable( const wxString& aNickname )
 {
     const SYMBOL_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxCHECK( row && row->plugin, false );
-    return row->plugin->IsSymbolLibWritable( row->GetFullURI( true ) );
+    return row->plugin->IsLibraryWritable( row->GetFullURI( true ) );
 }
 
 bool SYMBOL_LIB_TABLE::IsSymbolLibLoaded( const wxString& aNickname )
@@ -487,7 +510,7 @@ void SYMBOL_LIB_TABLE::DeleteSymbolLib( const wxString& aNickname )
 {
     const SYMBOL_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxCHECK( row && row->plugin, /* void */ );
-    row->plugin->DeleteSymbolLib( row->GetFullURI( true ), row->GetProperties() );
+    row->plugin->DeleteLibrary( row->GetFullURI( true ), row->GetProperties() );
 }
 
 
@@ -495,7 +518,7 @@ void SYMBOL_LIB_TABLE::CreateSymbolLib( const wxString& aNickname )
 {
     const SYMBOL_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxCHECK( row && row->plugin, /* void */ );
-    row->plugin->CreateSymbolLib( row->GetFullURI( true ), row->GetProperties() );
+    row->plugin->CreateLibrary( row->GetFullURI( true ), row->GetProperties() );
 }
 
 
@@ -531,7 +554,7 @@ LIB_SYMBOL* SYMBOL_LIB_TABLE::LoadSymbolWithOptionalNickname( const LIB_ID& aLib
 
 const wxString SYMBOL_LIB_TABLE::GlobalPathEnvVariableName()
 {
-    return  "KICAD7_SYMBOL_DIR";
+    return ENV_VAR::GetVersionedEnvVarName( wxS( "SYMBOL_DIR" ) );
 }
 
 
@@ -553,12 +576,15 @@ public:
         wxFileName file = wxFileName::FileName( aFilePath );
 
         // consider a file to be a lib if it's name ends with .kicad_sym and
-        // it is under $KICAD7_3RD_PARTY/symbols/<pkgid>/ i.e. has nested level of at least +2
+        // it is under $KICADn_3RD_PARTY/symbols/<pkgid>/ i.e. has nested level of at least +2
         if( file.GetExt() == wxT( "kicad_sym" ) && file.GetDirCount() >= m_prefix_dir_count + 2 )
         {
+            wxString versionedPath = wxString::Format( wxS( "${%s}" ),
+                                       ENV_VAR::GetVersionedEnvVarName( wxS( "3RD_PARTY" ) ) );
+
             wxArrayString parts = file.GetDirs();
             parts.RemoveAt( 0, m_prefix_dir_count );
-            parts.Insert( "${KICAD7_3RD_PARTY}", 0 );
+            parts.Insert( versionedPath, 0 );
             parts.Add( file.GetFullName() );
 
             wxString libPath = wxJoin( parts, '/' );
@@ -618,11 +644,12 @@ bool SYMBOL_LIB_TABLE::LoadGlobalTable( SYMBOL_LIB_TABLE& aTable )
 
         SystemDirsAppend( &ss );
 
-        wxString templatePath =
-            Pgm().GetLocalEnvVariables().at( wxT( "KICAD7_TEMPLATE_DIR" ) ).GetValue();
+        const ENV_VAR_MAP& envVars = Pgm().GetLocalEnvVariables();
+        std::optional<wxString> v = ENV_VAR::GetVersionedEnvVarValue( envVars,
+                                                                      wxT( "TEMPLATE_DIR" ) );
 
-        if( !templatePath.IsEmpty() )
-            ss.AddPaths( templatePath, 0 );
+        if( v && !v->IsEmpty() )
+            ss.AddPaths( *v, 0 );
 
         wxString fileName = ss.FindValidPath( global_tbl_name );
 
@@ -635,12 +662,19 @@ bool SYMBOL_LIB_TABLE::LoadGlobalTable( SYMBOL_LIB_TABLE& aTable )
         }
     }
 
+    aTable.Clear();
     aTable.Load( fn.GetFullPath() );
 
     SETTINGS_MANAGER& mgr = Pgm().GetSettingsManager();
     KICAD_SETTINGS*   settings = mgr.GetAppSettings<KICAD_SETTINGS>();
 
-    wxString packagesPath = Pgm().GetLocalEnvVariables().at( wxT( "KICAD7_3RD_PARTY" ) ).GetValue();
+    wxCHECK( settings, false );
+
+    wxString packagesPath;
+    const ENV_VAR_MAP& vars = Pgm().GetLocalEnvVariables();
+
+    if( std::optional<wxString> v = ENV_VAR::GetVersionedEnvVarValue( vars, wxT( "3RD_PARTY" ) ) )
+        packagesPath = *v;
 
     if( settings->m_PcmLibAutoAdd )
     {
@@ -672,18 +706,44 @@ bool SYMBOL_LIB_TABLE::LoadGlobalTable( SYMBOL_LIB_TABLE& aTable )
         }
 
         for( const wxString& nickName : to_remove )
-            aTable.RemoveRow( aTable.FindRow( nickName ) );
+        {
+            SYMBOL_LIB_TABLE_ROW* row = aTable.FindRow( nickName );
+
+            wxCHECK2( row, continue );
+
+            aTable.RemoveRow( row );
+        }
     }
 
     return tableExists;
 }
 
 
+bool SYMBOL_LIB_TABLE::operator==( const SYMBOL_LIB_TABLE& aOther ) const
+{
+    if( m_rows.size() != aOther.m_rows.size() )
+        return false;
+
+    unsigned i;
+
+    for( i = 0; i < m_rows.size(); ++i )
+    {
+        const SYMBOL_LIB_TABLE_ROW& curr = static_cast<const SYMBOL_LIB_TABLE_ROW&>( m_rows[i] );
+        const SYMBOL_LIB_TABLE_ROW& curr_other = static_cast<const SYMBOL_LIB_TABLE_ROW&>( aOther.m_rows[i] );
+
+        if( curr != curr_other )
+            return false;
+    }
+
+    return true;
+}
+
+
 wxString SYMBOL_LIB_TABLE::GetGlobalTableFileName()
 {
     wxFileName fn;
 
-    fn.SetPath( SETTINGS_MANAGER::GetUserSettingsPath() );
+    fn.SetPath( PATHS::GetUserSettingsPath() );
     fn.SetName( global_tbl_name );
 
     return fn.GetFullPath();
