@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -80,6 +80,13 @@ HIERARCHY_PANE::HIERARCHY_PANE( SCH_EDIT_FRAME* aParent ) :
     SetSizer( sizer );
     m_tree = new HIERARCHY_TREE( this );
 
+#ifdef __WXMAC__
+    // HiDPI-aware API; will be generally available in wxWidgets 3.4
+    wxVector<wxBitmapBundle> images;
+    images.push_back( KiBitmapBundle( BITMAPS::tree_nosel ) );
+    images.push_back( KiBitmapBundle( BITMAPS::tree_sel ) );
+    m_tree->SetImages( images );
+#else
     // Make an image list containing small icons
     // All icons are expected having the same size.
     wxBitmap tree_nosel_bm( KiBitmap( BITMAPS::tree_nosel ) );
@@ -90,6 +97,7 @@ HIERARCHY_PANE::HIERARCHY_PANE( SCH_EDIT_FRAME* aParent ) :
     imageList->Add( KiBitmap( BITMAPS::tree_sel ) );
 
     m_tree->AssignImageList( imageList );
+#endif
 
     sizer->Add( m_tree, 1, wxEXPAND, wxBORDER_NONE, 0 );
 
@@ -124,7 +132,7 @@ void HIERARCHY_PANE::buildHierarchyTree( SCH_SHEET_PATH* aList, const wxTreeItem
         SCH_SHEET* sheet = static_cast<SCH_SHEET*>( aItem );
         aList->push_back( sheet );
 
-        wxString sheetName = formatPageString( sheet->GetFields()[SHEETNAME].GetShownText(),
+        wxString sheetName = formatPageString( sheet->GetFields()[SHEETNAME].GetShownText( false ),
                                                aList->GetPageNumber() );
         wxTreeItemId child = m_tree->AppendItem( aParent, sheetName, 0, 1 );
         m_tree->SetItemData( child, new TREE_ITEM_DATA( *aList ) );
@@ -151,8 +159,6 @@ void HIERARCHY_PANE::UpdateHierarchySelection()
         m_events_bound = false;
     }
 
-    bool sheetSelected = false;
-
     std::function<void( const wxTreeItemId& )> recursiveDescent =
             [&]( const wxTreeItemId& id )
             {
@@ -162,21 +168,22 @@ void HIERARCHY_PANE::UpdateHierarchySelection()
 
                 if( itemData->m_SheetPath == m_frame->GetCurrentSheet() )
                 {
-                    m_tree->EnsureVisible( id );
+                    wxTreeItemId parent = m_tree->GetItemParent( id );
+
+                    if( parent.IsOk() && !m_tree->IsExpanded( parent ) )
+                        m_tree->Expand( parent );
+
+                    if( !m_tree->IsVisible( id ) )
+                        m_tree->EnsureVisible( id );
+
                     m_tree->SetItemBold( id, true );
+                    m_tree->SetFocusedItem( id );
                 }
                 else
                 {
                     m_tree->SetItemBold( id, false );
                 }
 
-                if( itemData->m_SheetPath.Last()->IsSelected() )
-                {
-                    m_tree->EnsureVisible( id );
-                    m_tree->SelectItem( id );
-                    sheetSelected = true;
-                }
-
                 wxTreeItemIdValue cookie;
                 wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
 
@@ -189,9 +196,6 @@ void HIERARCHY_PANE::UpdateHierarchySelection()
 
     recursiveDescent( m_tree->GetRootItem() );
 
-    if( !sheetSelected && m_tree->GetSelection() )
-        m_tree->SelectItem( m_tree->GetSelection(), false );
-
     if( eventsWereBound )
     {
         // Enable selection events
@@ -220,6 +224,32 @@ void HIERARCHY_PANE::UpdateHierarchyTree()
         m_events_bound = false;
     }
 
+    SCH_SHEET_LIST hierarchy = m_frame->Schematic().GetUnorderedSheets();
+    std::set<SCH_SHEET_PATH> expandedNodes;
+
+    std::function<void( const wxTreeItemId& )> getExpandedNodes =
+            [&]( const wxTreeItemId& id )
+            {
+                wxCHECK_RET( id.IsOk(), wxT( "Invalid tree item" ) );
+
+                TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
+
+                if( m_tree->IsExpanded( id ) && hierarchy.HasPath( itemData->m_SheetPath.Path() ) )
+                    expandedNodes.emplace( itemData->m_SheetPath );
+
+                wxTreeItemIdValue cookie;
+                wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
+
+                while( child.IsOk() )
+                {
+                    getExpandedNodes( child );
+                    child = m_tree->GetNextChild( id, cookie );
+                }
+            };
+
+    if( !m_tree->IsEmpty() )
+        getExpandedNodes( m_tree->GetRootItem() );
+
     m_list.clear();
     m_list.push_back( &m_frame->Schematic().Root() );
 
@@ -231,7 +261,35 @@ void HIERARCHY_PANE::UpdateHierarchyTree()
     buildHierarchyTree( &m_list, root );
     UpdateHierarchySelection();
 
-    m_tree->ExpandAll();
+    if( !expandedNodes.empty() )
+    {
+        std::function<void( const wxTreeItemId& )> expandNodes =
+                [&]( const wxTreeItemId& id )
+                {
+                    wxCHECK_RET( id.IsOk(), wxT( "Invalid tree item" ) );
+
+                    TREE_ITEM_DATA* itemData =
+                            static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
+
+                    if( expandedNodes.find( itemData->m_SheetPath ) != expandedNodes.end() )
+                        m_tree->Expand( id );
+
+                    wxTreeItemIdValue cookie;
+                    wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
+
+                    while( child.IsOk() )
+                    {
+                        expandNodes( child );
+                        child = m_tree->GetNextChild( id, cookie );
+                    }
+                };
+
+        expandNodes( m_tree->GetRootItem() );
+    }
+    else if( m_tree->ItemHasChildren( root ) )
+    {
+        m_tree->Expand( root );
+    }
 
     if( eventsWereBound )
     {
@@ -250,17 +308,66 @@ void HIERARCHY_PANE::UpdateHierarchyTree()
 void HIERARCHY_PANE::onSelectSheetPath( wxTreeEvent& aEvent )
 {
     wxTreeItemId  itemSel = m_tree->GetSelection();
+
+    if( !itemSel.IsOk() )
+        return;
+
     TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( itemSel ) );
 
+    if( !itemData )
+        return;
+
     SetCursor( wxCURSOR_ARROWWAIT );
-    m_frame->GetToolManager()->RunAction( EE_ACTIONS::changeSheet, true, &itemData->m_SheetPath );
+    m_frame->GetToolManager()->RunAction<SCH_SHEET_PATH*>( EE_ACTIONS::changeSheet,
+                                                           &itemData->m_SheetPath );
     SetCursor( wxCURSOR_ARROW );
 }
 
 
+void HIERARCHY_PANE::UpdateLabelsHierarchyTree()
+{
+    // Update the labels of the hierarchical tree of the schematic.
+    // Must be called only for an up to date tree, to update displayed labels after
+    // a sheet name or a sheet number change.
+
+    std::function<void( const wxTreeItemId& )> updateLabel =
+            [&]( const wxTreeItemId& id )
+            {
+                TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
+                SCH_SHEET* sheet = itemData->m_SheetPath.Last();
+                wxString sheetNameLabel =
+                        formatPageString( sheet->GetFields()[SHEETNAME].GetShownText( false ),
+                                          itemData->m_SheetPath.GetPageNumber() );
+
+                if( m_tree->GetItemText( id ) != sheetNameLabel )
+                    m_tree->SetItemText( id, sheetNameLabel );
+            };
+
+    wxTreeItemId rootId  = m_tree->GetRootItem();
+    updateLabel( rootId );
+
+    std::function<void( const wxTreeItemId& )> recursiveDescent =
+            [&]( const wxTreeItemId& id )
+            {
+                wxCHECK_RET( id.IsOk(), wxT( "Invalid tree item" ) );
+                wxTreeItemIdValue cookie;
+                wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
+
+                while( child.IsOk() )
+                {
+                    updateLabel( child );
+                    recursiveDescent( child );
+                    child = m_tree->GetNextChild( id, cookie );
+                }
+            };
+
+    recursiveDescent( rootId );
+}
+
+
 void HIERARCHY_PANE::onRightClick( wxTreeEvent& aEvent )
 {
-    wxTreeItemId  itemSel = m_tree->GetFocusedItem();
+    wxTreeItemId  itemSel = aEvent.GetItem();
 
     if( !itemSel.IsOk() )
         return;
@@ -305,6 +412,8 @@ void HIERARCHY_PANE::onRightClick( wxTreeEvent& aEvent )
             }
 
             m_frame->OnModify();
+
+            UpdateLabelsHierarchyTree();
         }
     }
 }
@@ -316,7 +425,7 @@ wxString HIERARCHY_PANE::getRootString()
     SCH_SHEET_PATH rootPath;
     rootPath.push_back( rootSheet );
 
-    return formatPageString ( _( "Root" ), rootPath.GetPageNumber() );
+    return formatPageString( rootSheet->GetShownName( false ), rootPath.GetPageNumber() );
 }
 
 
