@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2007-2018 Jean-Pierre Charras  jp.charras at wanadoo.fr
- * Copyright (C) 1992-2018 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,17 +22,15 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-/**
- * @file rs274x.cpp
- */
 
 #include <base_units.h>
 #include <math/util.h>      // for KiROUND
 
 #include <gerbview.h>
 #include <gerber_file_image.h>
-#include <ignore.h>
+#include <core/ignore.h>
 #include <macros.h>
+#include <string_utils.h>
 #include <X2_gerber_attributes.h>
 #include <gbr_metadata.h>
 #include <wx/log.h>
@@ -967,7 +965,6 @@ char* GERBER_FILE_IMAGE::GetNextLine( char *aBuff, unsigned int aBuffSize, char*
                 return aText;
         }
     }
-    return aText;
 }
 
 
@@ -1033,7 +1030,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
         else if( !isdigit(*aText)  )     // Ill. symbol
         {
             msg.Printf( wxT( "RS274X: Aperture Macro \"%s\": ill. symbol, line: \"%s\"" ),
-                        am.m_AmName, FROM_UTF8( aBuff ) );
+                        am.m_AmName, From_UTF8( aBuff ) );
             AddMessageToList( msg );
             primitive_type = AMP_COMMENT;
         }
@@ -1049,8 +1046,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
             is_comment = true;
 
             // Skip comment
-            while( *aText && ( *aText != '*' ) )
-                aText++;
+            GetEndOfBlock( m_LineBuffer, GERBER_BUFZ, aText, m_Current_File );
 
             break;
 
@@ -1087,7 +1083,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
 
         default:
             msg.Printf( wxT( "RS274X: Aperture Macro \"%s\": Invalid primitive id code %d, line %d: \"%s\"" ),
-                        am.m_AmName, primitive_type, m_LineNum, FROM_UTF8( aBuff ) );
+                        am.m_AmName, primitive_type, m_LineNum, From_UTF8( aBuff ) );
             AddMessageToList( msg );
             return false;
         }
@@ -1132,7 +1128,7 @@ bool GERBER_FILE_IMAGE::ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
             // in advance, i.e. be immediate.
             wxASSERT( prim.m_Params[1].IsImmediate() );
 
-            paramCount = (int) prim.m_Params[1].GetValue( nullptr ) * 2 + 1;
+            paramCount = (int) prim.m_Params[1].GetValueFromMacro( nullptr ) * 2 + 1;
 
             for( int jj = 0; jj < paramCount && *aText != '*'; ++jj )
             {
