@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 2017-2020 KiCad Developers, see change_log.txt for contributors.
+ * Copyright (C) 2017-2023 KiCad Developers, see change_log.txt for contributors.
+ * Copyright (C) 2023 CERN (www.cern.ch)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -69,18 +70,14 @@ public:
                   const wxString& aTitle, const wxPoint& aPos, const wxSize& aSize,
                   long aStyle, const wxString& aFrameName, const EDA_IU_SCALE& aIuScale );
 
-    /// Don't use this one, only wxformbuilder uses it, and it must be augmented with
-    /// a SetKiway() early in derived constructor.
-    KIWAY_PLAYER( wxWindow* aParent, wxWindowID aId, const wxString& aTitle,
-                  const wxPoint& aPos, const wxSize& aSize, long aStyle,
-                  const wxString& aWdoName = wxFrameNameStr );
-
     ~KIWAY_PLAYER() throw();
 
     // For the aCtl argument of OpenProjectFiles()
-#define KICTL_EAGLE_BRD         (1<<0)   ///< chosen *.brd file is Eagle according to user.
-#define KICTL_CREATE            (1<<1)   ///< caller thinks requested project files may not exist.
-#define KICTL_IMPORT_LIB        (1<<2)   ///< import all footprints into a project library.
+#define KICTL_NONKICAD_ONLY     (1<<0)   ///< chosen file is non-KiCad according to user
+#define KICTL_KICAD_ONLY        (1<<1)   ///< chosen file is from KiCad according to user
+#define KICTL_CREATE            (1<<2)   ///< caller thinks requested project files may not exist.
+#define KICTL_IMPORT_LIB        (1<<3)   ///< import all footprints into a project library.
+#define KICTL_REVERT            (1<<4)   ///< reverting to a previously-saved (KiCad) file.
 
     /**
      * Open a project or set of files given by @a aFileList.
@@ -181,7 +178,6 @@ public:
      */
     virtual void ExecuteRemoteCommand( const char* cmdline ){}
 
-
 protected:
 
     /// event handler, routes to derivative specific virtual KiwayMailIn()
