@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2023, 2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,12 +22,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-/**
- * Plot settings, and plotting engines (PostScript, Gerber, HPGL and DXF)
- *
- * @file plotter.h
- */
-
 #ifndef PLOT_COMMON_H_
 #define PLOT_COMMON_H_
 
@@ -113,7 +107,7 @@ public:
     static const int DO_NOT_SET_LINE_WIDTH = -2;    // Skip selection
     static const int USE_DEFAULT_LINE_WIDTH = -1;   // use the default pen
 
-    PLOTTER();
+    PLOTTER( const PROJECT* aProject = nullptr );
 
     virtual ~PLOTTER();
 
@@ -154,7 +148,7 @@ public:
 
     virtual void SetColor( const COLOR4D& color ) = 0;
 
-    virtual void SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle ) = 0;
+    virtual void SetDash( int aLineWidth, LINE_STYLE aLineStyle ) = 0;
 
     virtual void SetCreator( const wxString& aCreator ) { m_creator = aCreator; }
 
@@ -218,11 +212,12 @@ public:
     virtual void Circle( const VECTOR2I& pos, int diametre, FILL_T fill,
                          int width = USE_DEFAULT_LINE_WIDTH ) = 0;
 
-    /**
-     * Generic fallback: arc rendered as a polyline.
-     */
-    virtual void Arc( const VECTOR2I& aCenter, const VECTOR2I& aStart, const VECTOR2I& aEnd,
-                      FILL_T aFill, int aWidth, int aMaxError );
+    virtual void Arc( const VECTOR2D& aStart, const VECTOR2D& aMid, const VECTOR2D& aEnd,
+                      FILL_T aFill, int aWidth = USE_DEFAULT_LINE_WIDTH );
+
+    virtual void Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                      const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill,
+                      int aWidth = USE_DEFAULT_LINE_WIDTH );
 
     /**
      * Generic fallback: Cubic Bezier curve rendered as a polyline
@@ -304,11 +299,11 @@ public:
     virtual void ThickSegment( const VECTOR2I& start, const VECTOR2I& end, int width,
                                OUTLINE_MODE tracemode, void* aData );
 
-    virtual void ThickArc( const VECTOR2I& aCentre, const VECTOR2I& aStart,
-                           const VECTOR2I& aEnd, int aWidth,
+    virtual void ThickArc( const EDA_SHAPE& aArcShape,
                            OUTLINE_MODE aTraceMode, void* aData );
 
-    virtual void ThickArc( const EDA_SHAPE& aArcShape,
+    virtual void ThickArc( const VECTOR2D& aCentre, const EDA_ANGLE& aStAngle,
+                           const EDA_ANGLE& aAngle, double aRadius, int aWidth,
                            OUTLINE_MODE aTraceMode, void* aData );
 
     virtual void ThickRect( const VECTOR2I& p1, const VECTOR2I& p2, int width, OUTLINE_MODE tracemode,
@@ -409,28 +404,50 @@ public:
     /**
      * Draw text with the plotter.
      *
-     * For convenience it accept the color to use for specific plotters (GERBER) aData is used
-     * to pass extra parameters.
+     * For convenience it accept the color to use for specific plotters
+     * aData is used to pass extra parameters (GERBER).
+     *
+     * @param aPos is the text position (according to aH_justify, aV_justify).
+     * @param aColor is the text color.
+     * @param aText is the text to draw.
+     * @param aOrient is the angle.
+     * @param aSize is the text size (size.x or size.y can be < 0 for mirrored texts).
+     * @param aH_justify is the horizontal justification (Left, center, right).
+     * @param aV_justify is the vertical justification (bottom, center, top).
+     * @param aPenWidth is the line width (if = 0, use plot default line width).
+     * @param aItalic is the true to simulate an italic font.
+     * @param aBold use true to use a bold font Useful only with default width value (aPenWidth = 0).
+     * @param aMultilineAllowed use true to plot text as multiline, otherwise single line.
+     * @param aData is a parameter used by some plotters in SetCurrentLineWidth(),
+     *              not directly used here.
      */
-    virtual void Text( const VECTOR2I&             aPos,
-                       const COLOR4D&              aColor,
-                       const wxString&             aText,
-                       const EDA_ANGLE&            aOrient,
-                       const VECTOR2I&             aSize,
-                       enum GR_TEXT_H_ALIGN_T      aH_justify,
-                       enum GR_TEXT_V_ALIGN_T      aV_justify,
-                       int                         aPenWidth,
-                       bool                        aItalic,
-                       bool                        aBold,
-                       bool                        aMultilineAllowed,
-                       KIFONT::FONT*               aFont,
-                       void*                       aData = nullptr );
+    virtual void Text( const VECTOR2I&        aPos,
+                       const COLOR4D&         aColor,
+                       const wxString&        aText,
+                       const EDA_ANGLE&       aOrient,
+                       const VECTOR2I&        aSize,
+                       enum GR_TEXT_H_ALIGN_T aH_justify,
+                       enum GR_TEXT_V_ALIGN_T aV_justify,
+                       int                    aPenWidth,
+                       bool                   aItalic,
+                       bool                   aBold,
+                       bool                   aMultilineAllowed,
+                       KIFONT::FONT*          aFont,
+                       const KIFONT::METRICS& aFontMetrics,
+                       void*                  aData = nullptr );
 
+    virtual void PlotText( const VECTOR2I&        aPos,
+                           const COLOR4D&         aColor,
+                           const wxString&        aText,
+                           const TEXT_ATTRIBUTES& aAttributes,
+                           KIFONT::FONT*          aFont,
+                           const KIFONT::METRICS& aFontMetrics,
+                           void*                  aData = nullptr );
     /**
      * Create a clickable hyperlink with a rectangular click area
      *
-     * @aBox is the rectangular click target
-     * @aDestinationURL is the target URL
+     * @param aBox is the rectangular click target
+     * @param aDestinationURL is the target URL
      */
     virtual void HyperlinkBox( const BOX2I& aBox, const wxString& aDestinationURL )
     {
@@ -440,8 +457,8 @@ public:
     /**
      * Create a clickable hyperlink menu with a rectangular click area
      *
-     * @aBox is the rectangular click target
-     * @aDestURLs is the list of target URLs for the menu
+     * @param aBox is the rectangular click target
+     * @param aDestURLs is the target URL
      */
     virtual void HyperlinkMenu( const BOX2I& aBox, const std::vector<wxString>& aDestURLs )
     {
@@ -451,8 +468,8 @@ public:
     /**
      * Create a bookmark to a symbol
      *
-     * @aBox is the bounding box of the symbol
-     * @aSymbolReference is the symbol schematic ref
+     * @param aBox is the rectangular click target
+     * @param aSymbolReference is the symbol schematic ref
      */
     virtual void Bookmark( const BOX2I& aBox, const wxString& aName,
                            const wxString& aGroupName = wxEmptyString )
@@ -526,18 +543,27 @@ public:
      */
     virtual void EndBlock( void* aData ) {}
 
+    /**
+     * @return the plot offset in IUs, set by SetViewport() and used
+     * to plot items.
+     */
+    VECTOR2I GetPlotOffsetUserUnits() {return m_plotOffset; }
+
 
 protected:
     /**
      * Generic fallback: arc rendered as a polyline.
+     * Note also aCentre and aRadius are double to avoid creating rounding issues due
+     * to the fact a arc is defined in Kicad by a start point, a end point and third point
+     * not angles and radius.
+     * In some plotters (i.e. dxf) whe need a good precision when calculating an arc
+     * without error introduced by rounding, to avoid moving the end points,
+     * usually important in outlines when plotting an arc given by center, radius and angles.
+     * Winding direction: counter-clockwise in right-down coordinate system.
      */
-    virtual void Arc( const VECTOR2I& aCentre, const EDA_ANGLE& aStartAngle,
-                      const EDA_ANGLE& aEndAngle, int aRadius, FILL_T aFill,
-                      int aWidth = USE_DEFAULT_LINE_WIDTH );
-
-    virtual void ThickArc( const VECTOR2I& aCentre, const EDA_ANGLE& StAngle,
-                           const EDA_ANGLE& EndAngle, int aRadius, int aWidth,
-                           OUTLINE_MODE aTraceMode, void* aData );
+    virtual void polyArc( const VECTOR2D& aCentre, const EDA_ANGLE& aStartAngle,
+                          const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill,
+                          int aWidth = USE_DEFAULT_LINE_WIDTH );
 
     // These are marker subcomponents
     /**
@@ -611,6 +637,7 @@ protected:
 
     double GetDashGapLenIU( int aLineWidth ) const;
 
+
 protected:      // variables used in most of plotters:
     /// Plot scale - chosen by the user (even implicitly with 'fit in a4')
     double           m_plotScale;
@@ -648,6 +675,8 @@ protected:      // variables used in most of plotters:
     wxArrayString    m_headerExtraLines;    // a set of string to print in header file
 
     RENDER_SETTINGS* m_renderSettings;
+
+    const PROJECT*   m_project;
 };
 
 
