@@ -24,6 +24,8 @@
 #include <kicad_curl/kicad_curl.h>
 
 #include "core/wx_stl_compat.h"
+#include <env_vars.h>
+#include <background_jobs_monitor.h>
 #include "build_version.h"
 #include "paths.h"
 #include "pcm.h"
@@ -59,47 +61,10 @@ class THROWING_ERROR_HANDLER : public nlohmann::json_schema::error_handler
 };
 
 
-class STATUS_TEXT_REPORTER : public PROGRESS_REPORTER_BASE
-{
-public:
-    STATUS_TEXT_REPORTER( std::function<void( const wxString )> aStatusCallback ) :
-            PROGRESS_REPORTER_BASE( 1 ), m_statusCallback( aStatusCallback )
-    {
-    }
-
-    void SetTitle( const wxString& aTitle ) override
-    {
-        m_title = aTitle;
-        m_report = wxT( "" );
-    }
-
-    void Report( const wxString& aMessage ) override
-    {
-        m_report = wxString::Format( wxT( ": %s" ), aMessage );
-    }
-
-    void Cancel() { m_cancelled.store( true ); }
-
-private:
-    bool updateUI() override
-    {
-        m_statusCallback( wxString::Format( wxT( "%s%s" ), m_title, m_report ) );
-        return true;
-    }
-
-    const std::function<void( const wxString )> m_statusCallback;
-
-    wxString m_title;
-    wxString m_report;
-};
-
-
 PLUGIN_CONTENT_MANAGER::PLUGIN_CONTENT_MANAGER(
-                                        std::function<void( int )> aAvailableUpdateCallback,
-                                        std::function<void( const wxString )> aStatusCallback ) :
+                                        std::function<void( int )> aAvailableUpdateCallback ) :
         m_dialog( nullptr ),
-        m_availableUpdateCallback( aAvailableUpdateCallback ),
-        m_statusCallback( aStatusCallback )
+        m_availableUpdateCallback( aAvailableUpdateCallback )
 {
     ReadEnvVar();
 
@@ -108,7 +73,7 @@ PLUGIN_CONTENT_MANAGER::PLUGIN_CONTENT_MANAGER(
     schema_file.Normalize( FN_NORMALIZE_FLAGS | wxPATH_NORM_ENV_VARS );
     schema_file.AppendDir( wxS( "schemas" ) );
 
-    std::ifstream  schema_stream( schema_file.GetFullPath().ToUTF8() );
+    std::ifstream  schema_stream( schema_file.GetFullPath().fn_str() );
     nlohmann::json schema;
 
     try
@@ -130,11 +95,11 @@ PLUGIN_CONTENT_MANAGER::PLUGIN_CONTENT_MANAGER(
     }
 
     // Load currently installed packages
-    wxFileName f( SETTINGS_MANAGER::GetUserSettingsPath(), wxT( "installed_packages.json" ) );
+    wxFileName f( PATHS::GetUserSettingsPath(), wxT( "installed_packages.json" ) );
 
     if( f.FileExists() )
     {
-        std::ifstream  installed_stream( f.GetFullPath().ToUTF8() );
+        std::ifstream  installed_stream( f.GetFullPath().fn_str() );
         nlohmann::json installed;
 
         try
@@ -215,7 +180,7 @@ PLUGIN_CONTENT_MANAGER::PLUGIN_CONTENT_MANAGER(
     std::for_each( m_installed.begin(), m_installed.end(),
                    [&]( std::pair<const wxString, PCM_INSTALLATION_ENTRY>& entry )
                    {
-                       preparePackage( entry.second.package );
+                       PreparePackage( entry.second.package );
                    } );
 }
 
@@ -224,10 +189,9 @@ void PLUGIN_CONTENT_MANAGER::ReadEnvVar()
 {
     // Get 3rd party path
     const ENV_VAR_MAP& env = Pgm().GetLocalEnvVariables();
-    auto               it = env.find( wxT( "KICAD7_3RD_PARTY" ) );
 
-    if( it != env.end() && !it->second.GetValue().IsEmpty() )
-        m_3rdparty_path = it->second.GetValue();
+    if( std::optional<wxString> v = ENV_VAR::GetVersionedEnvVarValue( env, wxT( "3RD_PARTY" ) ) )
+        m_3rdparty_path = *v;
     else
         m_3rdparty_path = PATHS::GetDefault3rdPartyPath();
 }
@@ -299,12 +263,7 @@ bool PLUGIN_CONTENT_MANAGER::FetchRepository( const wxString& aUrl, PCM_REPOSITO
     aReporter->SetTitle( _( "Fetching repository" ) );
 
     if( !DownloadToStream( aUrl, &repository_stream, aReporter, 20480 ) )
-    {
-        if( m_dialog )
-            wxLogError( _( "Unable to load repository url" ) );
-
         return false;
-    }
 
     nlohmann::json repository_json;
 
@@ -320,7 +279,7 @@ bool PLUGIN_CONTENT_MANAGER::FetchRepository( const wxString& aUrl, PCM_REPOSITO
     {
         if( m_dialog )
         {
-            wxLogError( wxString::Format( _( "Unable to parse repository: %s" ), e.what() ) );
+            wxLogError( _( "Unable to parse repository: %s" ), e.what() );
             wxLogError( _( "The given repository URL does not look like a valid KiCad package "
                            "repository. Please double check the URL." ) );
         }
@@ -411,7 +370,7 @@ PLUGIN_CONTENT_MANAGER::getCachedRepository( const wxString& aRepositoryId ) con
 }
 
 
-const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryId )
+bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryId )
 {
     if( m_repository_cache.find( aRepositoryId ) != m_repository_cache.end() )
         return true;
@@ -437,7 +396,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
     if( m_dialog )
         reporter = std::make_shared<WX_PROGRESS_REPORTER>( m_dialog, wxEmptyString, 1 );
     else
-        reporter = m_statusReporter;
+        reporter = m_updateBackgroundJob->m_reporter;
 
     if( !FetchRepository( url, current_repo, reporter.get() ) )
         return false;
@@ -452,7 +411,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
     if( repo_cache.FileExists() && packages_cache.FileExists() )
     {
-        std::ifstream  repo_stream( repo_cache.GetFullPath().ToUTF8() );
+        std::ifstream  repo_stream( repo_cache.GetFullPath().fn_str() );
         PCM_REPOSITORY saved_repo;
         try
         {
@@ -469,7 +428,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
         {
             // Cached repo is up to date, use data on disk
             js.clear();
-            std::ifstream packages_cache_stream( packages_cache.GetFullPath().ToUTF8() );
+            std::ifstream packages_cache_stream( packages_cache.GetFullPath().fn_str() );
 
             try
             {
@@ -478,7 +437,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
                 for( size_t i = 0; i < saved_repo.package_list.size(); i++ )
                 {
-                    preparePackage( saved_repo.package_list[i] );
+                    PreparePackage( saved_repo.package_list[i] );
                     saved_repo.package_map[saved_repo.package_list[i].identifier] = i;
                 }
 
@@ -508,16 +467,16 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
         for( size_t i = 0; i < current_repo.package_list.size(); i++ )
         {
-            preparePackage( current_repo.package_list[i] );
+            PreparePackage( current_repo.package_list[i] );
             current_repo.package_map[current_repo.package_list[i].identifier] = i;
         }
 
         repo_cache.Mkdir( wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL );
 
-        std::ofstream repo_cache_stream( repo_cache.GetFullPath().ToUTF8() );
+        std::ofstream repo_cache_stream( repo_cache.GetFullPath().fn_str() );
         repo_cache_stream << std::setw( 4 ) << nlohmann::json( current_repo ) << std::endl;
 
-        std::ofstream packages_cache_stream( packages_cache.GetFullPath().ToUTF8() );
+        std::ofstream packages_cache_stream( packages_cache.GetFullPath().fn_str() );
         js.clear();
         js["packages"] = nlohmann::json( current_repo.package_list );
         packages_cache_stream << std::setw( 4 ) << js << std::endl;
@@ -540,7 +499,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
         if( mtime + 600 < getCurrentTimestamp() && mtime < (time_t) resources.update_timestamp )
         {
-            std::ofstream resources_stream( resource_file.GetFullPath().ToUTF8(),
+            std::ofstream resources_stream( resource_file.GetFullPath().fn_str(),
                                             std::ios_base::binary );
 
             reporter->SetTitle( _( "Downloading resources" ) );
@@ -553,7 +512,7 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
             if( success )
             {
-                std::ifstream read_stream( resource_file.GetFullPath().ToUTF8(),
+                std::ifstream read_stream( resource_file.GetFullPath().fn_str(),
                                            std::ios_base::binary );
 
 
@@ -648,7 +607,7 @@ void PLUGIN_CONTENT_MANAGER::updateInstalledPackagesMetadata( const wxString& aR
 }
 
 
-void PLUGIN_CONTENT_MANAGER::preparePackage( PCM_PACKAGE& aPackage )
+void PLUGIN_CONTENT_MANAGER::PreparePackage( PCM_PACKAGE& aPackage )
 {
     // Parse package version strings
     for( PACKAGE_VERSION& ver : aPackage.versions )
@@ -899,7 +858,7 @@ const wxString PLUGIN_CONTENT_MANAGER::GetPackageUpdateVersion( const PCM_PACKAG
                                            && installed_ver_it->parsed_version < ver.parsed_version;
                                 } );
 
-    return ver_it == aPackage.versions.end() ? wxT( "" ) : ver_it->version;
+    return ver_it == aPackage.versions.end() ? wxString( wxT( "" ) ) : ver_it->version;
 }
 
 time_t PLUGIN_CONTENT_MANAGER::getCurrentTimestamp() const
@@ -921,8 +880,8 @@ void PLUGIN_CONTENT_MANAGER::SaveInstalledPackages()
             js["packages"].emplace_back( pair.second );
         }
 
-        wxFileName    f( SETTINGS_MANAGER::GetUserSettingsPath(), wxT( "installed_packages.json" ) );
-        std::ofstream stream( f.GetFullPath().ToUTF8() );
+        wxFileName    f( PATHS::GetUserSettingsPath(), wxT( "installed_packages.json" ) );
+        std::ofstream stream( f.GetFullPath().fn_str() );
 
         stream << std::setw( 4 ) << js << std::endl;
     }
@@ -1130,20 +1089,43 @@ std::unordered_map<wxString, wxBitmap> PLUGIN_CONTENT_MANAGER::GetInstalledPacka
 }
 
 
+struct UPDATE_CANCELLER
+{
+    UPDATE_CANCELLER( std::shared_ptr<BACKGROUND_JOB>& aJob ) : m_jobToCancel( aJob ) {};
+    ~UPDATE_CANCELLER()
+    {
+        if( m_jobToCancel )
+        {
+            Pgm().GetBackgroundJobMonitor().Remove( m_jobToCancel );
+            m_jobToCancel.reset();
+        }
+    }
+
+    std::shared_ptr<BACKGROUND_JOB>& m_jobToCancel;
+};
+
+
 void PLUGIN_CONTENT_MANAGER::RunBackgroundUpdate()
 {
     // If the thread is already running don't create it again
     if( m_updateThread.joinable() )
         return;
 
-    m_statusReporter = std::make_shared<STATUS_TEXT_REPORTER>( m_statusCallback );
+
+    m_updateBackgroundJob = Pgm().GetBackgroundJobMonitor().Create( _( "PCM Update" ) );
 
     m_updateThread = std::thread(
             [this]()
             {
+                UPDATE_CANCELLER canceller( m_updateBackgroundJob );
+
                 if( m_installed.size() == 0 )
                     return;
 
+                int maxProgress = m_repository_list.size() + m_installed.size();
+                m_updateBackgroundJob->m_reporter->SetNumPhases( maxProgress );
+                m_updateBackgroundJob->m_reporter->Report( _( "Preparing to fetch repositories" ) );
+
                 // Only fetch repositories that have installed not pinned packages
                 std::unordered_set<wxString> repo_ids;
 
@@ -1155,25 +1137,31 @@ void PLUGIN_CONTENT_MANAGER::RunBackgroundUpdate()
 
                 for( const auto& [ repository_id, name, url ] : m_repository_list )
                 {
+                    m_updateBackgroundJob->m_reporter->AdvancePhase();
                     if( repo_ids.count( repository_id ) == 0 )
                         continue;
 
+                    m_updateBackgroundJob->m_reporter->Report(
+                            _( "Fetching repository..." ) );
                     CacheRepository( repository_id );
 
-                    if( m_statusReporter->IsCancelled() )
+                    if( m_updateBackgroundJob->m_reporter->IsCancelled() )
                         break;
                 }
 
-                if( m_statusReporter->IsCancelled() )
+                if( m_updateBackgroundJob->m_reporter->IsCancelled() )
                     return;
 
                 // Count packages with updates
                 int availableUpdateCount = 0;
 
+                m_updateBackgroundJob->m_reporter->Report( _( "Reviewing packages..." ) );
                 for( std::pair<const wxString, PCM_INSTALLATION_ENTRY>& pair : m_installed )
                 {
                     PCM_INSTALLATION_ENTRY& entry = pair.second;
 
+                    m_updateBackgroundJob->m_reporter->AdvancePhase();
+
                     if( m_repository_cache.find( entry.repository_id ) != m_repository_cache.end() )
                     {
                         PCM_PACKAGE_STATE state = GetPackageState( entry.repository_id,
@@ -1183,15 +1171,12 @@ void PLUGIN_CONTENT_MANAGER::RunBackgroundUpdate()
                             availableUpdateCount++;
                     }
 
-                    if( m_statusReporter->IsCancelled() )
+                    if( m_updateBackgroundJob->m_reporter->IsCancelled() )
                         return;
                 }
 
                 // Update the badge on PCM button
                 m_availableUpdateCallback( availableUpdateCount );
-
-                m_statusCallback( availableUpdateCount > 0 ? _( "Package updates are available" )
-                                                           : _( "No package updates available" ) );
             } );
 }
 
@@ -1200,7 +1185,8 @@ void PLUGIN_CONTENT_MANAGER::StopBackgroundUpdate()
 {
     if( m_updateThread.joinable() )
     {
-        m_statusReporter->Cancel();
+        if( m_updateBackgroundJob )
+            m_updateBackgroundJob->m_reporter->Cancel();
         m_updateThread.join();
     }
 }
