@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,6 +19,7 @@
  */
 
 #include <wildcards_and_files_ext.h>
+#include <env_vars.h>
 #include <executable_names.h>
 #include <pgm_base.h>
 #include <policy_keys.h>
@@ -36,6 +37,8 @@
 #include <tools/kicad_manager_actions.h>
 #include <tools/kicad_manager_control.h>
 #include <dialogs/dialog_template_selector.h>
+#include <dialogs/git/dialog_git_repository.h>
+#include <git/git_clone_handler.h>
 #include <gestfich.h>
 #include <paths.h>
 #include <wx/checkbox.h>
@@ -43,15 +46,11 @@
 #include <wx/filedlg.h>
 #include "dialog_pcm.h"
 #include <macros.h>
-#include <sch_io_mgr.h>
-#include <io_mgr.h>
+#include <sch_io/sch_io_mgr.h>
+#include <pcb_io/pcb_io_mgr.h>
 #include <import_proj.h>
 
-#if wxCHECK_VERSION( 3, 1, 7 )
 #include "widgets/filedlg_new_project.h"
-#else
-#include "widgets/legacyfiledlg_new_project.h"
-#endif
 
 KICAD_MANAGER_CONTROL::KICAD_MANAGER_CONTROL() :
         TOOL_INTERACTIVE( "kicad.Control" ),
@@ -66,31 +65,30 @@ void KICAD_MANAGER_CONTROL::Reset( RESET_REASON aReason )
 }
 
 
-int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
+wxFileName KICAD_MANAGER_CONTROL::newProjectDirectory( wxString* aFileName, bool isRepo )
 {
+    wxString default_filename = aFileName ? *aFileName : wxString();
+
     wxString        default_dir = m_frame->GetMruPath();
-    wxFileDialog    dlg( m_frame, _( "Create New Project" ), default_dir, wxEmptyString,
-                         ProjectFileWildcard(), wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
+    wxFileDialog    dlg( m_frame, _( "Create New Project" ), default_dir, default_filename,
+                         ( isRepo ? wxString( "" ) : FILEEXT::ProjectFileWildcard() ),
+                         wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
     // Add a "Create a new directory" checkbox
-#if wxCHECK_VERSION( 3, 1, 7 )
     FILEDLG_NEW_PROJECT newProjectHook;
     dlg.SetCustomizeHook( newProjectHook );
-#else
-    dlg.SetExtraControlCreator( &LEGACYFILEDLG_NEW_PROJECT::Create );
-#endif
 
     if( dlg.ShowModal() == wxID_CANCEL )
-        return -1;
+        return wxFileName();
 
     wxFileName pro( dlg.GetPath() );
 
     // wxFileName automatically extracts an extension.  But if it isn't
     // a .pro extension, we should keep it as part of the filename
-    if( !pro.GetExt().IsEmpty() && pro.GetExt().ToStdString() != ProjectFileExtension )
+    if( !pro.GetExt().IsEmpty() && pro.GetExt().ToStdString() != FILEEXT::ProjectFileExtension )
         pro.SetName( pro.GetName() + wxT( "." ) + pro.GetExt() );
 
-    pro.SetExt( ProjectFileExtension );     // enforce extension
+    pro.SetExt( FILEEXT::ProjectFileExtension ); // enforce extension
 
     if( !pro.IsAbsolute() )
         pro.MakeAbsolute();
@@ -98,11 +96,7 @@ int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
     // Append a new directory with the same name of the project file.
     bool createNewDir = false;
 
-#if wxCHECK_VERSION( 3, 1, 7 )
     createNewDir = newProjectHook.GetCreateNewDir();
-#else
-    createNewDir = static_cast<LEGACYFILEDLG_NEW_PROJECT*>( dlg.GetExtraControl() )->CreateNewDir();
-#endif
 
     if( createNewDir )
         pro.AppendDir( pro.GetName() );
@@ -119,7 +113,7 @@ int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
                            "Make sure you have write permissions and try again." ),
                         pro.GetPath() );
             DisplayErrorMessage( m_frame, msg );
-            return -1;
+            return wxFileName();
         }
     }
     else if( directory.HasFiles() )
@@ -129,9 +123,21 @@ int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
                           "Do you want to continue?" );
 
         if( !IsOK( m_frame, msg ) )
-            return -1;
+            return wxFileName();
     }
 
+    return pro;
+}
+
+
+int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
+{
+
+    wxFileName pro = newProjectDirectory();
+
+    if( !pro.IsOk() )
+        return -1;
+
     m_frame->CreateNewProject( pro );
     m_frame->LoadProject( pro );
 
@@ -139,6 +145,68 @@ int KICAD_MANAGER_CONTROL::NewProject( const TOOL_EVENT& aEvent )
 }
 
 
+int KICAD_MANAGER_CONTROL::NewFromRepository( const TOOL_EVENT& aEvent )
+{
+    DIALOG_GIT_REPOSITORY dlg( m_frame, nullptr );
+
+    dlg.SetTitle( _( "Clone Project from Git Repository" ) );
+
+    int ret = dlg.ShowModal();
+
+    if( ret != wxID_OK )
+        return -1;
+
+    wxString   project_name = dlg.GetRepoName();
+    wxFileName pro = newProjectDirectory( &project_name, true );
+
+    if( !pro.IsOk() )
+        return -1;
+
+    GIT_CLONE_HANDLER cloneHandler;
+
+    cloneHandler.SetURL( dlg.GetRepoURL() );
+    cloneHandler.SetClonePath( pro.GetPath() );
+    cloneHandler.SetConnType( dlg.GetRepoType() );
+    cloneHandler.SetUsername( dlg.GetUsername() );
+    cloneHandler.SetPassword( dlg.GetPassword() );
+    cloneHandler.SetSSHKey( dlg.GetRepoSSHPath() );
+
+    cloneHandler.SetProgressReporter( std::make_unique<WX_PROGRESS_REPORTER>( m_frame, _( "Cloning Repository" ), 1 ) );
+
+    if( !cloneHandler.PerformClone() )
+    {
+        DisplayErrorMessage( m_frame, cloneHandler.GetErrorString() );
+        return -1;
+    }
+
+    std::vector<wxString> projects = cloneHandler.GetProjectDirs();
+
+    if( projects.empty() )
+    {
+        DisplayErrorMessage( m_frame, _( "No project files were found in the repository." ) );
+        return -1;
+    }
+
+    // Currently, we pick the first project file we find in the repository.
+    // TODO: Look into spare checkout to allow the user to pick a partial repository
+    wxString dest = pro.GetPath() + wxFileName::GetPathSeparator() + projects.front();
+    m_frame->LoadProject( dest );
+
+    Prj().GetLocalSettings().m_GitRepoPassword = dlg.GetPassword();
+    Prj().GetLocalSettings().m_GitRepoUsername = dlg.GetUsername();
+    Prj().GetLocalSettings().m_GitSSHKey = dlg.GetRepoSSHPath();
+
+    if( dlg.GetRepoType() == KIGIT_COMMON::GIT_CONN_TYPE::GIT_CONN_SSH )
+        Prj().GetLocalSettings().m_GitRepoType = "ssh";
+    else if( dlg.GetRepoType() == KIGIT_COMMON::GIT_CONN_TYPE::GIT_CONN_HTTPS )
+        Prj().GetLocalSettings().m_GitRepoType = "https";
+    else
+        Prj().GetLocalSettings().m_GitRepoType = "local";
+
+    return 0;
+}
+
+
 int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
 {
     DIALOG_TEMPLATE_SELECTOR* ps = new DIALOG_TEMPLATE_SELECTOR( m_frame );
@@ -146,16 +214,17 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
     wxFileName  templatePath;
 
     // KiCad system template path.
-    ENV_VAR_MAP_CITER it =  Pgm().GetLocalEnvVariables().find( "KICAD7_TEMPLATE_DIR" );
+    std::optional<wxString> v = ENV_VAR::GetVersionedEnvVarValue( Pgm().GetLocalEnvVariables(),
+                                                                  wxT( "TEMPLATE_DIR" ) );
 
-    if( it != Pgm().GetLocalEnvVariables().end() && it->second.GetValue() != wxEmptyString )
+    if( v && !v->IsEmpty() )
     {
-        templatePath.AssignDir( it->second.GetValue() );
+        templatePath.AssignDir( *v );
         ps->AddTemplatesPage( _( "System Templates" ), templatePath );
     }
 
     // User template path.
-    it = Pgm().GetLocalEnvVariables().find( "KICAD_USER_TEMPLATE_DIR" );
+    ENV_VAR_MAP_CITER it = Pgm().GetLocalEnvVariables().find( "KICAD_USER_TEMPLATE_DIR" );
 
     if( it != Pgm().GetLocalEnvVariables().end() && it->second.GetValue() != wxEmptyString )
     {
@@ -178,17 +247,12 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
     // Get project destination folder and project file name.
     wxString        default_dir = wxFileName( Prj().GetProjectFullName() ).GetPathWithSep();
     wxString        title = _( "New Project Folder" );
-    wxFileDialog    dlg( m_frame, title, default_dir, wxEmptyString, ProjectFileWildcard(),
+    wxFileDialog    dlg( m_frame, title, default_dir, wxEmptyString, FILEEXT::ProjectFileWildcard(),
                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
     // Add a "Create a new directory" checkbox
-#if wxCHECK_VERSION( 3, 1, 7 )
     FILEDLG_NEW_PROJECT newProjectHook;
     dlg.SetCustomizeHook( newProjectHook );
-#else
-    dlg.SetExtraControlCreator( &LEGACYFILEDLG_NEW_PROJECT::Create );
-#endif
-
 
     if( dlg.ShowModal() == wxID_CANCEL )
         return -1;
@@ -197,20 +261,16 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
 
     // wxFileName automatically extracts an extension.  But if it isn't a .kicad_pro extension,
     // we should keep it as part of the filename
-    if( !fn.GetExt().IsEmpty() && fn.GetExt().ToStdString() != ProjectFileExtension )
+    if( !fn.GetExt().IsEmpty() && fn.GetExt().ToStdString() != FILEEXT::ProjectFileExtension )
         fn.SetName( fn.GetName() + wxT( "." ) + fn.GetExt() );
 
-    fn.SetExt( ProjectFileExtension );
+    fn.SetExt( FILEEXT::ProjectFileExtension );
 
     if( !fn.IsAbsolute() )
         fn.MakeAbsolute();
 
     bool createNewDir = false;
-#if wxCHECK_VERSION( 3, 1, 7 )
     createNewDir = newProjectHook.GetCreateNewDir();
-#else
-    createNewDir = static_cast<LEGACYFILEDLG_NEW_PROJECT*>( dlg.GetExtraControl() )->CreateNewDir();
-#endif
 
     // Append a new directory with the same name of the project file.
     if( createNewDir )
@@ -300,9 +360,9 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
 
 int KICAD_MANAGER_CONTROL::openProject( const wxString& aDefaultDir )
 {
-    wxString wildcard = AllProjectFilesWildcard()
-                        + "|" + ProjectFileWildcard()
-                        + "|" + LegacyProjectFileWildcard();
+    wxString wildcard = FILEEXT::AllProjectFilesWildcard()
+                        + "|" + FILEEXT::ProjectFileWildcard()
+                        + "|" + FILEEXT::LegacyProjectFileWildcard();
 
     wxFileDialog dlg( m_frame, _( "Open Existing Project" ), aDefaultDir, wxEmptyString, wildcard,
                       wxFD_OPEN | wxFD_FILE_MUST_EXIST );
@@ -343,80 +403,6 @@ int KICAD_MANAGER_CONTROL::CloseProject( const TOOL_EVENT& aEvent )
 }
 
 
-int KICAD_MANAGER_CONTROL::ImportNonKicadProj( const TOOL_EVENT& aEvent )
-{
-    if( !aEvent.Parameter<wxString*>() )
-        return -1;
-
-    wxFileName droppedFileName( *aEvent.Parameter<wxString*>() );
-
-    wxString schFileExtension, pcbFileExtension;
-    int      schFileType, pcbFileType;
-
-    // Define extensions to use according to dropped file.
-    // Eagle project.
-    if( droppedFileName.GetExt() == EagleSchematicFileExtension
-        || droppedFileName.GetExt() == EaglePcbFileExtension )
-    {
-        // Check if droppedFile is an eagle file.
-        // If not, return and do not import files.
-        if( !IsFileFromEDATool( droppedFileName, EDA_TOOLS::EAGLE ) )
-            return -1;
-
-        schFileExtension = EagleSchematicFileExtension;
-        pcbFileExtension = EaglePcbFileExtension;
-        schFileType = SCH_IO_MGR::SCH_EAGLE;
-        pcbFileType = IO_MGR::EAGLE;
-    }
-    // Cadstar project.
-    else if( droppedFileName.GetExt() == CadstarSchematicFileExtension
-             || droppedFileName.GetExt() == CadstarPcbFileExtension )
-    {
-        schFileExtension = CadstarSchematicFileExtension;
-        pcbFileExtension = CadstarPcbFileExtension;
-        schFileType = SCH_IO_MGR::SCH_CADSTAR_ARCHIVE;
-        pcbFileType = IO_MGR::CADSTAR_PCB_ARCHIVE;
-    }
-    else
-    {
-        return -1;
-    }
-
-    IMPORT_PROJ_HELPER importProj( m_frame, droppedFileName.GetFullPath(), schFileExtension,
-                                   pcbFileExtension );
-
-    // Check if the project directory is empty
-    wxDir directory( importProj.GetProjPath() );
-
-    if( directory.HasFiles() )
-    {
-        // Append a new directory with the same name of the project file
-        // Keep iterating until we find an empty directory
-        importProj.CreateEmptyDirForProject();
-
-        if( !wxMkdir( importProj.GetProjPath() ) )
-            return -1;
-    }
-
-    importProj.SetProjAbsolutePath();
-
-    if( !importProj.CopyImportedFiles( false ) )
-    {
-        wxRmdir( importProj.GetProjPath() ); // Remove the previous created directory, before leaving.
-        return -1;
-    }
-
-    m_frame->CloseProject( true );
-
-    m_frame->CreateNewProject( importProj.GetProjFullPath(), false /* Don't create stub files */ );
-    m_frame->LoadProject( importProj.GetProj() );
-
-    importProj.AssociateFilesWithProj( schFileType, pcbFileType );
-    m_frame->ReCreateTreePrj();
-    m_frame->LoadProject( importProj.GetProj() );
-    return 0;
-}
-
 int KICAD_MANAGER_CONTROL::LoadProject( const TOOL_EVENT& aEvent )
 {
     if( aEvent.Parameter<wxString*>() )
@@ -457,9 +443,9 @@ public:
         wxString   ext = destFile.GetExt();
         bool       atRoot = destFile.GetPath() == m_projectDirPath;
 
-        if( ext == LegacyProjectFileExtension
-          || ext == ProjectFileExtension
-          || ext == ProjectLocalSettingsFileExtension )
+        if( ext == FILEEXT::LegacyProjectFileExtension
+          || ext == FILEEXT::ProjectFileExtension
+          || ext == FILEEXT::ProjectLocalSettingsFileExtension )
         {
             wxString destPath = destFile.GetPath();
 
@@ -473,69 +459,74 @@ public:
             {
                 destFile.SetName( m_newProjectName );
 
-                if( atRoot && ext != ProjectLocalSettingsFileExtension )
+                if( atRoot && ext != FILEEXT::ProjectLocalSettingsFileExtension )
                     m_newProjectFile = destFile;
             }
 
-            if( ext == LegacyProjectFileExtension )
+            if( ext == FILEEXT::LegacyProjectFileExtension )
             {
                 // All paths in the settings file are relative so we can just do a straight copy
                 KiCopyFile( aSrcFilePath, destFile.GetFullPath(), m_errors );
             }
-            else if( ext == ProjectFileExtension )
+            else if( ext == FILEEXT::ProjectFileExtension )
             {
                 PROJECT_FILE projectFile( aSrcFilePath );
                 projectFile.LoadFromFile();
                 projectFile.SaveAs( destFile.GetPath(), destFile.GetName() );
             }
-            else if( ext == ProjectLocalSettingsFileExtension )
+            else if( ext == FILEEXT::ProjectLocalSettingsFileExtension )
             {
                 PROJECT_LOCAL_SETTINGS projectLocalSettings( nullptr, aSrcFilePath );
                 projectLocalSettings.LoadFromFile();
                 projectLocalSettings.SaveAs( destFile.GetPath(), destFile.GetName() );
             }
         }
-        else if( ext == KiCadSchematicFileExtension
-               || ext == KiCadSchematicFileExtension + BackupFileSuffix
-               || ext == LegacySchematicFileExtension
-               || ext == LegacySchematicFileExtension + BackupFileSuffix
-               || ext == SchematicSymbolFileExtension
-               || ext == LegacySymbolLibFileExtension
-               || ext == LegacySymbolDocumentFileExtension
-               || ext == KiCadSymbolLibFileExtension
-               || ext == NetlistFileExtension
+        else if( ext == FILEEXT::KiCadSchematicFileExtension
+                 || ext == FILEEXT::KiCadSchematicFileExtension + FILEEXT::BackupFileSuffix
+                 || ext == FILEEXT::LegacySchematicFileExtension
+                 || ext == FILEEXT::LegacySchematicFileExtension + FILEEXT::BackupFileSuffix
+                 || ext == FILEEXT::SchematicSymbolFileExtension
+                 || ext == FILEEXT::LegacySymbolLibFileExtension
+                 || ext == FILEEXT::LegacySymbolDocumentFileExtension
+                 || ext == FILEEXT::KiCadSymbolLibFileExtension
+                 || ext == FILEEXT::NetlistFileExtension
                || destFile.GetName() == "sym-lib-table" )
         {
             KIFACE* eeschema = m_frame->Kiway().KiFACE( KIWAY::FACE_SCH );
             eeschema->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
                                   m_newProjectName, aSrcFilePath, m_errors );
         }
-        else if( ext == KiCadPcbFileExtension
-               || ext == KiCadPcbFileExtension + BackupFileSuffix
-               || ext == LegacyPcbFileExtension
-               || ext == KiCadFootprintFileExtension
-               || ext == LegacyFootprintLibPathExtension
-               || ext == FootprintAssignmentFileExtension
+        else if( ext == FILEEXT::KiCadPcbFileExtension
+                 || ext == FILEEXT::KiCadPcbFileExtension + FILEEXT::BackupFileSuffix
+                 || ext == FILEEXT::LegacyPcbFileExtension
+                 || ext == FILEEXT::KiCadFootprintFileExtension
+                 || ext == FILEEXT::LegacyFootprintLibPathExtension
+                 || ext == FILEEXT::FootprintAssignmentFileExtension
                || destFile.GetName() == "fp-lib-table" )
         {
             KIFACE* pcbnew = m_frame->Kiway().KiFACE( KIWAY::FACE_PCB );
             pcbnew->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
                                 m_newProjectName, aSrcFilePath, m_errors );
         }
-        else if( ext == DrawingSheetFileExtension )
+        else if( ext == FILEEXT::DrawingSheetFileExtension )
         {
             KIFACE* pleditor = m_frame->Kiway().KiFACE( KIWAY::FACE_PL_EDITOR );
             pleditor->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
                                   m_newProjectName, aSrcFilePath, m_errors );
         }
-        else if( ext == GerberJobFileExtension
-               || ext == DrillFileExtension
-               || IsGerberFileExtension(ext) )
+        else if( ext == FILEEXT::GerberJobFileExtension
+               || ext == FILEEXT::DrillFileExtension
+                 || FILEEXT::IsGerberFileExtension( ext ) )
         {
             KIFACE* gerbview = m_frame->Kiway().KiFACE( KIWAY::FACE_GERBVIEW );
             gerbview->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
                                   m_newProjectName, aSrcFilePath, m_errors );
         }
+        else if( destFile.GetName().StartsWith( FILEEXT::LockFilePrefix )
+                 && ext == FILEEXT::LockFileExtension )
+        {
+            // Ignore lock files
+        }
         else
         {
             // Everything we don't recognize just gets a straight copy.
@@ -760,7 +751,7 @@ int KICAD_MANAGER_CONTROL::ShowPlayer( const TOOL_EVENT& aEvent )
 
     if ( !player )
     {
-        wxLogError( _( "Application failed to load." ) );
+        wxLogError( _( "Application cannot start." ) );
         return -1;
     }
 
@@ -791,7 +782,9 @@ int KICAD_MANAGER_CONTROL::ShowPlayer( const TOOL_EVENT& aEvent )
 
         if( !filepath.IsEmpty() )
         {
-            if( !player->OpenProjectFiles( std::vector<wxString>( 1, filepath ) ) )
+            std::vector<wxString> file_list{ filepath };
+
+            if( !player->OpenProjectFiles( file_list ) )
             {
                 player->Destroy();
                 return -1;
@@ -813,6 +806,21 @@ int KICAD_MANAGER_CONTROL::ShowPlayer( const TOOL_EVENT& aEvent )
     if( wxWindow::FindFocus() != player )
         player->SetFocus();
 
+    // Save window state to disk now.  Don't wait around for a crash.
+    if( Pgm().GetCommonSettings()->m_Session.remember_open_files
+            && !player->GetCurrentFileName().IsEmpty()
+            && Prj().GetLocalSettings().ShouldAutoSave() )
+    {
+        wxFileName rfn( player->GetCurrentFileName() );
+        rfn.MakeRelativeTo( Prj().GetProjectPath() );
+
+        WINDOW_SETTINGS windowSettings;
+        player->SaveWindowSettings( &windowSettings );
+
+        Prj().GetLocalSettings().SaveFileState( rfn.GetFullPath(), &windowSettings, true );
+        Prj().GetLocalSettings().SaveToFile( Prj().GetProjectPath() );
+    }
+
     return 0;
 }
 
@@ -867,9 +875,13 @@ int KICAD_MANAGER_CONTROL::Execute( const TOOL_EVENT& aEvent )
     if( pid > 0 )
     {
 #ifdef __WXMAC__
-        // This non-parameterized use of wxExecute is fine because execFile is not derived
+        wxString script = wxString::Format( wxS( "tell application \"System Events\"\n"
+                          "  set frontmost of the first process whose unix id is %l to true\n"
+                          "end tell" ), pid );
+
+        // This non-parameterized use of wxExecute is fine because script is not derived
         // from user input.
-        wxExecute( "osascript -e 'activate application \"" + execFile + "\"'" );
+        wxExecute( wxString::Format( "osascript -e '%s'", script ) );
 #endif
     }
     else
@@ -883,8 +895,8 @@ int KICAD_MANAGER_CONTROL::Execute( const TOOL_EVENT& aEvent )
 
 int KICAD_MANAGER_CONTROL::ShowPluginManager( const TOOL_EVENT& aEvent )
 {
-    if( KIPLATFORM::POLICY::GetPolicyState( POLICY_KEY_PCM )
-        == KIPLATFORM::POLICY::STATE::DISABLED )
+    if( KIPLATFORM::POLICY::GetPolicyBool( POLICY_KEY_PCM )
+        == KIPLATFORM::POLICY::PBOOL::DISABLED )
     {
         // policy disables the plugin manager
         return 0;
@@ -896,6 +908,9 @@ int KICAD_MANAGER_CONTROL::ShowPluginManager( const TOOL_EVENT& aEvent )
     m_frame->SetFocus();
     wxSafeYield();
 
+    if( !m_frame->GetPcm() )
+        m_frame->CreatePCM();
+
     DIALOG_PCM pcm( m_frame, m_frame->GetPcm() );
     pcm.ShowModal();
 
@@ -946,12 +961,12 @@ void KICAD_MANAGER_CONTROL::setTransitions()
 {
     Go( &KICAD_MANAGER_CONTROL::NewProject,         KICAD_MANAGER_ACTIONS::newProject.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::NewFromTemplate,    KICAD_MANAGER_ACTIONS::newFromTemplate.MakeEvent() );
+    Go( &KICAD_MANAGER_CONTROL::NewFromRepository,  KICAD_MANAGER_ACTIONS::newFromRepository.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::OpenDemoProject,    KICAD_MANAGER_ACTIONS::openDemoProject.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::OpenProject,        KICAD_MANAGER_ACTIONS::openProject.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::CloseProject,       KICAD_MANAGER_ACTIONS::closeProject.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::SaveProjectAs,      ACTIONS::saveAs.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::LoadProject,        KICAD_MANAGER_ACTIONS::loadProject.MakeEvent() );
-    Go( &KICAD_MANAGER_CONTROL::ImportNonKicadProj, KICAD_MANAGER_ACTIONS::importNonKicadProj.MakeEvent() );
     Go( &KICAD_MANAGER_CONTROL::ViewDroppedViewers, KICAD_MANAGER_ACTIONS::viewDroppedGerbers.MakeEvent() );
 
     Go( &KICAD_MANAGER_CONTROL::Refresh,         ACTIONS::zoomRedraw.MakeEvent() );
