@@ -32,6 +32,7 @@
 #include <geometry/seg.h>
 #include <geometry/shape.h>
 #include <geometry/shape_arc.h>
+#include <geometry/corner_strategy.h>
 #include <math/vector2d.h>
 
 /**
@@ -80,37 +81,36 @@ struct CLIPPER_Z_VALUE
  */
 class SHAPE_LINE_CHAIN : public SHAPE_LINE_CHAIN_BASE
 {
-private:
+public:
     typedef std::vector<VECTOR2I>::iterator point_iter;
     typedef std::vector<VECTOR2I>::const_iterator point_citer;
 
-public:
     /**
      * Represent an intersection between two line segments
      */
     struct INTERSECTION
     {
-        ///< Point of intersection between our and their.
+        /// Point of intersection between our and their.
         VECTOR2I p;
 
-        ///< Index of the intersecting corner/segment in the 'our' (== this) line.
+        /// Index of the intersecting corner/segment in the 'our' (== this) line.
         int index_our;
 
-        ///< index of the intersecting corner/segment in the 'their' (Intersect() method
-        ///< parameter) line.
+        /// index of the intersecting corner/segment in the 'their' (Intersect() method
+        /// parameter) line.
         int index_their;
 
-        ///< When true, the corner [index_our] of the 'our' line lies exactly on 'their' line.
+        /// When true, the corner [index_our] of the 'our' line lies exactly on 'their' line.
         bool is_corner_our;
 
-        ///< When true, the corner [index_their] of the 'their' line lies exactly on 'our' line.
-        ///< Note that when both is_corner_our and is_corner_their are set, the line chains touch
-        ///< with with corners.
+        /// When true, the corner [index_their] of the 'their' line lies exactly on 'our' line.
+        /// Note that when both is_corner_our and is_corner_their are set, the line chains touch
+        /// with with corners.
         bool is_corner_their;
 
-        ///< Auxiliary flag to avoid copying intersection info to intersection refining code,
-        ///< used by the refining code (e.g. hull handling stuff in the P&S) to reject false
-        ///< intersection points.
+        /// Auxiliary flag to avoid copying intersection info to intersection refining code,
+        /// used by the refining code (e.g. hull handling stuff in the P&S) to reject false
+        /// intersection points.
         bool valid;
 
         INTERSECTION() :
@@ -172,25 +172,9 @@ public:
 
     SHAPE_LINE_CHAIN( const std::vector<int>& aV );
 
-    SHAPE_LINE_CHAIN( const std::vector<VECTOR2I>& aV, bool aClosed = false ) :
-            SHAPE_LINE_CHAIN_BASE( SH_LINE_CHAIN ),
-            m_closed( aClosed ),
-            m_width( 0 )
-    {
-        m_points = aV;
-        m_shapes = std::vector<std::pair<ssize_t, ssize_t>>( aV.size(), SHAPES_ARE_PT );
-    }
+    SHAPE_LINE_CHAIN( const std::vector<VECTOR2I>& aV, bool aClosed = false );
 
-    SHAPE_LINE_CHAIN( const SHAPE_ARC& aArc, bool aClosed = false ) :
-            SHAPE_LINE_CHAIN_BASE( SH_LINE_CHAIN ),
-            m_closed( aClosed ),
-            m_width( 0 )
-    {
-        m_points = aArc.ConvertToPolyline().CPoints();
-        m_arcs.emplace_back( aArc );
-        m_arcs.back().SetWidth( 0 );
-        m_shapes = std::vector<std::pair<ssize_t, ssize_t>>( m_points.size(), { 0, SHAPE_IS_PT } );
-    }
+    SHAPE_LINE_CHAIN( const SHAPE_ARC& aArc, bool aClosed = false );
 
     SHAPE_LINE_CHAIN( const ClipperLib::Path& aPath,
                       const std::vector<CLIPPER_Z_VALUE>& aZValueBuffer,
@@ -233,6 +217,40 @@ public:
     virtual bool Collide( const SEG& aSeg, int aClearance = 0, int* aActual = nullptr,
                           VECTOR2I* aLocation = nullptr ) const override;
 
+    /**
+     * Finds closest points between this and the other line chain. Doesn't test segments or arcs.
+     *
+     * @param aOther the line chain to test against.
+     * @param aPt0 closest point on this line chain (output).
+     * @param aPt1 closest point on the other line chain (output).
+     * @param aDistance distance between points (output).
+     * @return true, if the operation was successful.
+     */
+    bool ClosestPoints( const SHAPE_LINE_CHAIN& aOther, VECTOR2I& aPt0, VECTOR2I& aPt1 ) const;
+
+    static bool ClosestPoints( const point_citer& aMyStart, const point_citer& aMyEnd,
+                               const point_citer& aOtherStart, const point_citer& aOtherEnd,
+                               VECTOR2I& aPt0, VECTOR2I& aPt1, int64_t& aDistSq );
+
+    static bool ClosestSegments( const VECTOR2I& aMyPrevPt, const point_citer& aMyStart,
+                                 const point_citer& aMyEnd, const VECTOR2I& aOtherPrevPt,
+                                 const point_citer& aOtherStart, const point_citer& aOtherEnd,
+                                 VECTOR2I& aPt0, VECTOR2I& aPt1, int64_t& aDistSq );
+
+    /**
+     * Finds closest points between segments of this and the other line chain. Doesn't guarantee
+     * that the points are the absolute closest (use ClosestSegments for that) as there might
+     * be edge cases, but it is much faster.
+     *
+     * @param aOther the line chain to test against.
+     * @param aPt0 closest point on this line chain (output).
+     * @param aPt1 closest point on the other line chain (output).
+     * @param aDistance distance between points (output).
+     * @return true, if the operation was successful.
+     */
+    bool ClosestSegmentsFast( const SHAPE_LINE_CHAIN& aOther, VECTOR2I& aPt0,
+                              VECTOR2I& aPt1 ) const;
+
     SHAPE_LINE_CHAIN& operator=( const SHAPE_LINE_CHAIN& ) = default;
 
     SHAPE* Clone() const override;
@@ -312,6 +330,20 @@ public:
      */
     int ShapeCount() const;
 
+
+    /**
+     * Remove the duplicate points from the line chain.
+    */
+    void RemoveDuplicatePoints();
+
+    /**
+     * Simplify the line chain by removing colinear adjacent segments and duplicate vertices.
+     *
+     * @param aMaxError is the maximum error in internal units.  Setting to 0 means that the
+     * points must be exactly co-linear to be removed.
+     */
+    void Simplify( int aMaxError = 0 );
+
     /**
      * Return the number of points (vertices) in this line chain.
      *
@@ -329,16 +361,7 @@ public:
      *        from the end (i.e. -1 means the last segment in the line chain).
      * @return a segment at the \a aIndex in the line chain.
      */
-    SEG Segment( int aIndex )
-    {
-        if( aIndex < 0 )
-            aIndex += SegmentCount();
-
-        if( aIndex == (int)( m_points.size() - 1 ) && m_closed )
-            return SEG( m_points[aIndex], m_points[0], aIndex );
-        else
-            return SEG( m_points[aIndex], m_points[aIndex + 1], aIndex );
-    }
+    SEG Segment( int aIndex ) const;
 
     /**
      * Return a constant copy of the \a aIndex segment in the line chain.
@@ -347,16 +370,7 @@ public:
      *        from the end (i.e. -1 means the last segment in the line chain).
      * @return a segment at \a aIndex in the line chain.
      */
-    const SEG CSegment( int aIndex ) const
-    {
-        if( aIndex < 0 )
-            aIndex += SegmentCount();
-
-        if( aIndex == (int)( m_points.size() - 1 ) && m_closed )
-            return SEG( m_points[aIndex], m_points[0], aIndex );
-        else
-            return SEG( m_points[aIndex], m_points[aIndex + 1], aIndex );
-    }
+    const SEG CSegment( int aIndex ) const { return Segment( aIndex ); }
 
     /**
      * Return the vertex index of the next shape in the chain, or -1 if \a aPointIndex is the
@@ -367,21 +381,15 @@ public:
      * the arc, in other words, the last point of the arc.
      *
      * @param aPointIndex is a vertex in the chain.
-     * @param aForwards is true if the next shape is desired, false for previous shape.
      * @return the vertex index of the start of the next shape after aPoint's shape or -1 if
      * the end was reached.
      */
-    int NextShape( int aPointIndex, bool aForwards = true ) const;
-
-    int PrevShape( int aPointIndex ) const
-    {
-        return NextShape( aPointIndex, false );
-    }
+    int NextShape( int aPointIndex ) const;
 
     /**
      * Move a point to a specific location.
      *
-     * @param aIndex is the index of the point to move.
+     * @param aIndex is the index of the point to move.  Negative indexes are from the back.
      * @param aPos is the new absolute location of the point.
      */
     void SetPoint( int aIndex, const VECTOR2I& aPos );
@@ -389,7 +397,7 @@ public:
     /**
      * Return a reference to a given point in the line chain.
      *
-     * @param aIndex is the index of the point.
+     * @param aIndex is the index of the point.  Negative indexes are from the back.
      * @return a const reference to the point.
      */
     const VECTOR2I& CPoint( int aIndex ) const
@@ -456,14 +464,6 @@ public:
         return &m_bbox;
     }
 
-    /**
-     * Compute the minimum distance between the line chain and a point \a aP.
-     *
-     * @param aP the point.
-     * @return minimum distance.
-     */
-    int Distance( const VECTOR2I& aP, bool aOutlineOnly = false ) const;
-
     /**
      * Reverse point order in the line chain.
      *
@@ -485,6 +485,14 @@ public:
      */
     long long int Length() const;
 
+    /**
+     * Allocate a number of points all at once (for performance).
+     */
+    void ReservePoints( size_t aSize )
+    {
+        m_points.reserve( aSize );
+    }
+
     /**
      * Append a new point at the end of the line chain.
      *
@@ -586,10 +594,11 @@ public:
      * Insert the point aP belonging to one of the our segments, splitting the adjacent segment
      * in two.
      * @param aP is the point to be inserted.
+     * @param aExact set to skip the split logic when an exact point match exists.
      * @return index of the newly inserted point (or a negative value if aP does not lie on
      *         our line).
      */
-    int Split( const VECTOR2I& aP );
+    int Split( const VECTOR2I& aP, bool aExact = false );
 
     /**
      * Search for point \a aP.
@@ -678,14 +687,6 @@ public:
      */
     const std::optional<INTERSECTION> SelfIntersecting() const;
 
-    /**
-     * Simplify the line chain by removing colinear adjacent segments and duplicate vertices.
-     *
-     * @param aRemoveColinear controls the removal of colinear adjacent segments.
-     * @return reference to this line chain.
-     */
-    SHAPE_LINE_CHAIN& Simplify( bool aRemoveColinear = true );
-
     /**
      * Find the segment nearest the given point.
      *
@@ -785,6 +786,32 @@ public:
      */
     double Area( bool aAbsolute = true ) const;
 
+    /**
+     * Extract parts of this line chain, depending on the starting and ending points.
+     *
+     * @param aStart first split point.
+     * @param aEnd second split point.
+     * @param aPre part before the aStart point.
+     * @param aMid part between aStart and aEnd.
+     * @param aPost part after the aEnd point.
+     */
+    void Split( const VECTOR2I& aStart, const VECTOR2I& aEnd, SHAPE_LINE_CHAIN& aPre,
+                SHAPE_LINE_CHAIN& aMid, SHAPE_LINE_CHAIN& aPost ) const;
+
+    /**
+     * Creates line chains \a aLeft and \a aRight offset to this line chain.
+     *
+     * @param aAmount is the amount to offset.
+     * @param aCornerStrategy is the corner rounding strategy.
+     * @param aMaxError is the max error used for rounding.
+     * @param aLeft left line chain output.
+     * @param aRight right line chain output.
+     * @param aSimplify set to run Simplify on the inflated polygon.
+     */
+    bool OffsetLine( int aAmount, CORNER_STRATEGY aCornerStrategy, int aMaxError,
+                     SHAPE_LINE_CHAIN& aLeft, SHAPE_LINE_CHAIN& aRight,
+                     bool aSimplify = false ) const;
+
     size_t ArcCount() const
     {
         return m_arcs.size();
@@ -811,55 +838,21 @@ public:
      * @param aIndex
      * @return
     */
-    bool IsSharedPt( size_t aIndex ) const
+    bool IsSharedPt( size_t aIndex ) const;
+
+    bool IsPtOnArc( size_t aPtIndex ) const;
+
+    bool IsArcSegment( size_t aSegment ) const;
+
+    bool IsArcStart( size_t aIndex ) const;
+
+    bool IsArcEnd( size_t aIndex ) const;
+
+    using SHAPE::Distance;
+
+    int Distance( const VECTOR2I& aP, bool aOutlineOnly ) const
     {
-        return aIndex < m_shapes.size()
-               && m_shapes[aIndex].first != SHAPE_IS_PT
-               && m_shapes[aIndex].second != SHAPE_IS_PT;
-    }
-
-
-    bool IsPtOnArc( size_t aPtIndex ) const
-    {
-        return aPtIndex < m_shapes.size() && m_shapes[aPtIndex] != SHAPES_ARE_PT;
-    }
-
-
-    bool IsArcSegment( size_t aSegment ) const
-    {
-        /*
-         * A segment is part of an arc except in the special case of two arcs next to each other
-         * but without a shared vertex.  Here there is a segment between the end of the first arc
-         * and the start of the second arc.
-         */
-        size_t nextIdx = aSegment + 1;
-
-        if( nextIdx > m_shapes.size() - 1 )
-        {
-            if( nextIdx == m_shapes.size() && m_closed )
-                nextIdx = 0; // segment between end point and first point
-            else
-                return false;
-        }
-
-        return ( IsPtOnArc( aSegment )
-                 && ( IsSharedPt( aSegment )
-                      || m_shapes[aSegment].first == m_shapes[nextIdx].first ) );
-    }
-
-
-    bool IsArcStart( size_t aIndex ) const
-    {
-        if( aIndex == 0 )
-            return IsPtOnArc( aIndex );
-
-        return ( IsSharedPt( aIndex ) || ( IsPtOnArc( aIndex ) && !IsArcSegment( aIndex - 1 ) ) );
-    }
-
-
-    bool IsArcEnd( size_t aIndex ) const
-    {
-        return ( IsSharedPt( aIndex ) || ( IsPtOnArc( aIndex ) && !IsArcSegment( aIndex ) ) );
+        return sqrt( SquaredDistance( aP, aOutlineOnly ) );
     }
 
     virtual const VECTOR2I GetPoint( int aIndex ) const override { return CPoint(aIndex); }
@@ -867,6 +860,9 @@ public:
     virtual size_t GetPointCount() const override { return PointCount(); }
     virtual size_t GetSegmentCount() const override { return SegmentCount(); }
 
+    void TransformToPolygon( SHAPE_POLY_SET& aBuffer, int aError,
+                             ERROR_LOC aErrorLoc ) const override;
+
 protected:
     friend class SHAPE_POLY_SET;
 
