@@ -25,6 +25,7 @@
 #include <functional>
 using namespace std::placeholders;
 
+#include <wx/log.h>
 #include <fmt/format.h>
 
 #include <tool/tool_manager.h>
@@ -196,7 +197,7 @@ int PL_POINT_EDITOR::Main( const TOOL_EVENT& aEvent )
                 {
                     m_frame->SaveCopyInUndoList();
                 }
-                catch( const fmt::v9::format_error& exc )
+                catch( const fmt::v10::format_error& exc )
                 {
                     wxLogWarning( wxS( "Exception \"%s\" serializing string ocurred." ),
                                   exc.what() );
@@ -337,8 +338,8 @@ void PL_POINT_EDITOR::updateItem() const
         VECTOR2I move_startpoint = m_editPoints->Point( LINE_START ).GetPosition() - line->GetStart();
         VECTOR2I move_endpoint = m_editPoints->Point( LINE_END ).GetPosition() - line->GetEnd();
 
-        dataItem->MoveStartPointToUi( dataItem->GetStartPosUi() + move_startpoint );
-        dataItem->MoveEndPointToUi( dataItem->GetEndPosUi() + move_endpoint );
+        dataItem->MoveStartPointToIU( dataItem->GetStartPosIU() + move_startpoint );
+        dataItem->MoveEndPointToIU( dataItem->GetEndPosIU() + move_endpoint );
 
         for( DS_DRAW_ITEM_BASE* draw_item : dataItem->GetDrawItems() )
         {
@@ -388,8 +389,8 @@ void PL_POINT_EDITOR::updateItem() const
             end_delta.x = botRight.x - rect->GetEnd().x;
         }
 
-        dataItem->MoveStartPointToUi( dataItem->GetStartPosUi() + start_delta );
-        dataItem->MoveEndPointToUi( dataItem->GetEndPosUi() + end_delta );
+        dataItem->MoveStartPointToIU( dataItem->GetStartPosIU() + start_delta );
+        dataItem->MoveEndPointToIU( dataItem->GetEndPosIU() + end_delta );
 
         for( DS_DRAW_ITEM_BASE* draw_item : dataItem->GetDrawItems() )
         {
