@@ -5,7 +5,7 @@
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
  *
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,6 @@
  */
 
 #include <confirm.h>
-#include <pcbnew.h>
 #include <pcb_edit_frame.h>
 #include <widgets/msgpanel.h>
 #include <board.h>
@@ -57,25 +56,6 @@
 #define CELL_IS_ZONE   0x80   /* Area and auto-placement: cell available */
 
 
-/* Penalty (cost) for CntRot90 and CntRot180:
- * CntRot90 and CntRot180 are from 0 (rotation allowed) to 10 (rotation not allowed)
- */
-static const double OrientationPenalty[11] =
-{
-    2.0,        // CntRot = 0 rotation prohibited
-    1.9,        // CntRot = 1
-    1.8,        // CntRot = 2
-    1.7,        // CntRot = 3
-    1.6,        // CntRot = 4
-    1.5,        // CntRot = 5
-    1.4,        // CntRot = 5
-    1.3,        // CntRot = 7
-    1.2,        // CntRot = 8
-    1.1,        // CntRot = 9
-    1.0         // CntRot = 10 rotation authorized, no penalty
-};
-
-
 AR_AUTOPLACER::AR_AUTOPLACER( BOARD* aBoard )
 {
     m_board = aBoard;
@@ -84,7 +64,7 @@ AR_AUTOPLACER::AR_AUTOPLACER( BOARD* aBoard )
     for( FOOTPRINT* footprint : m_board->Footprints() )
         m_connectivity->Add( footprint );
 
-    m_gridSize = Millimeter2iu( STEP_AR_MM );
+    m_gridSize = pcbIUScale.mmToIU( STEP_AR_MM );
     m_progressReporter = nullptr;
     m_refreshCallback = nullptr;
     m_minCost = 0.0;
@@ -92,7 +72,7 @@ AR_AUTOPLACER::AR_AUTOPLACER( BOARD* aBoard )
 
 
 void AR_AUTOPLACER::placeFootprint( FOOTPRINT* aFootprint, bool aDoNotRecreateRatsnest,
-                                    const wxPoint& aPos )
+                                    const VECTOR2I& aPos )
 {
     if( !aFootprint )
         return;
@@ -106,7 +86,7 @@ int AR_AUTOPLACER::genPlacementRoutingMatrix()
 {
     m_matrix.UnInitRoutingMatrix();
 
-    EDA_RECT bbox = m_board->GetBoardEdgesBoundingBox();
+    BOX2I bbox = m_board->GetBoardEdgesBoundingBox();
 
     if( bbox.GetWidth() == 0 || bbox.GetHeight() == 0 )
         return 0;
@@ -161,10 +141,10 @@ bool AR_AUTOPLACER::fillMatrix()
     std::vector <int> x_coordinates;
     bool success = true;
     int step = m_matrix.m_GridRouting;
-    wxPoint coord_orgin = m_matrix.GetBrdCoordOrigin(); // Board coordinate of matruix cell (0,0)
+    VECTOR2I coord_orgin = m_matrix.GetBrdCoordOrigin(); // Board coordinate of matruix cell (0,0)
 
     // Create a single board outline:
-    SHAPE_POLY_SET brd_shape = m_boardShape;
+    SHAPE_POLY_SET brd_shape = m_boardShape.CloneDropTriangulation();
     brd_shape.Fracture( SHAPE_POLY_SET::PM_FAST );
     const SHAPE_LINE_CHAIN& outline = brd_shape.Outline(0);
     const BOX2I& rect = outline.BBox();
@@ -270,22 +250,7 @@ bool AR_AUTOPLACER::fillMatrix()
 }
 
 
-void AR_AUTOPLACER::rotateFootprint( FOOTPRINT* aFootprint, double angle, bool incremental )
-{
-    if( aFootprint == nullptr )
-        return;
-
-    if( incremental )
-        aFootprint->SetOrientation( aFootprint->GetOrientation() + angle );
-    else
-        aFootprint->SetOrientation( angle );
-
-
-    m_board->GetConnectivity()->Update( aFootprint );
-}
-
-
-void AR_AUTOPLACER::addFpBody( const wxPoint& aStart, const wxPoint& aEnd, LSET aLayerMask )
+void AR_AUTOPLACER::addFpBody( const VECTOR2I& aStart, const VECTOR2I& aEnd, LSET aLayerMask )
 {
     // Add a polygonal shape (rectangle) to m_fpAreaFront and/or m_fpAreaBack
     if( aLayerMask[ F_Cu ] )
@@ -311,7 +276,7 @@ void AR_AUTOPLACER::addFpBody( const wxPoint& aStart, const wxPoint& aEnd, LSET
 void AR_AUTOPLACER::addPad( PAD* aPad, int aClearance )
 {
     // Add a polygonal shape (rectangle) to m_fpAreaFront and/or m_fpAreaBack
-    EDA_RECT bbox = aPad->GetBoundingBox();
+    BOX2I bbox = aPad->GetBoundingBox();
     bbox.Inflate( aClearance );
 
     if( aPad->IsOnLayer( F_Cu ) )
@@ -351,7 +316,7 @@ void AR_AUTOPLACER::buildFpAreas( FOOTPRINT* aFootprint, int aFpClearance )
     if( aFootprint->GetLayer() == B_Cu )
         layerMask.set( B_Cu );
 
-    EDA_RECT fpBBox = aFootprint->GetBoundingBox();
+    BOX2I fpBBox = aFootprint->GetBoundingBox( false, false );
 
     fpBBox.Inflate( ( m_matrix.m_GridRouting / 2 ) + aFpClearance );
 
@@ -367,11 +332,11 @@ void AR_AUTOPLACER::buildFpAreas( FOOTPRINT* aFootprint, int aFpClearance )
 }
 
 
-void AR_AUTOPLACER::genModuleOnRoutingMatrix( FOOTPRINT* Module )
+void AR_AUTOPLACER::genModuleOnRoutingMatrix( FOOTPRINT* aFootprint )
 {
-    int         ox, oy, fx, fy;
-    LSET        layerMask;
-    EDA_RECT    fpBBox = Module->GetBoundingBox();
+    int   ox, oy, fx, fy;
+    LSET  layerMask;
+    BOX2I fpBBox = aFootprint->GetBoundingBox( false, false );
 
     fpBBox.Inflate( m_matrix.m_GridRouting / 2 );
     ox  = fpBBox.GetX();
@@ -403,28 +368,28 @@ void AR_AUTOPLACER::genModuleOnRoutingMatrix( FOOTPRINT* Module )
     if( fy > m_matrix.m_BrdBox.GetBottom() )
         fy = m_matrix.m_BrdBox.GetBottom();
 
-    if( Module->GetLayer() == F_Cu )
+    if( aFootprint->GetLayer() == F_Cu )
         layerMask.set( F_Cu );
 
-    if( Module->GetLayer() == B_Cu )
+    if( aFootprint->GetLayer() == B_Cu )
         layerMask.set( B_Cu );
 
     m_matrix.TraceFilledRectangle( ox, oy, fx, fy, layerMask,
                                    CELL_IS_MODULE, AR_MATRIX::WRITE_OR_CELL );
 
     // Trace pads + clearance areas.
-    for( PAD* pad : Module->Pads() )
+    for( PAD* pad : aFootprint->Pads() )
     {
         int margin = (m_matrix.m_GridRouting / 2) + pad->GetOwnClearance( pad->GetLayer() );
         m_matrix.PlacePad( pad, CELL_IS_MODULE, margin, AR_MATRIX::WRITE_OR_CELL );
     }
 
     // Trace clearance.
-    int margin = ( m_matrix.m_GridRouting * Module->GetPadCount() ) / AR_GAIN;
+    int margin = ( m_matrix.m_GridRouting * aFootprint->GetPadCount() ) / AR_GAIN;
     m_matrix.CreateKeepOutRectangle( ox, oy, fx, fy, margin, AR_KEEPOUT_MARGIN , layerMask );
 
     // Build the footprint courtyard
-    buildFpAreas( Module, margin );
+    buildFpAreas( aFootprint, margin );
 
     // Substract the shape to free areas
     m_topFreeArea.BooleanSubtract( m_fpAreaTop, SHAPE_POLY_SET::PM_FAST );
@@ -432,14 +397,14 @@ void AR_AUTOPLACER::genModuleOnRoutingMatrix( FOOTPRINT* Module )
 }
 
 
-int AR_AUTOPLACER::testRectangle( const EDA_RECT& aRect, int side )
+int AR_AUTOPLACER::testRectangle( const BOX2I& aRect, int side )
 {
-    EDA_RECT rect = aRect;
+    BOX2I rect = aRect;
 
     rect.Inflate( m_matrix.m_GridRouting / 2 );
 
-    wxPoint start   = rect.GetOrigin();
-    wxPoint end     = rect.GetEnd();
+    VECTOR2I start = rect.GetOrigin();
+    VECTOR2I end     = rect.GetEnd();
 
     start   -= m_matrix.m_BrdBox.GetOrigin();
     end     -= m_matrix.m_BrdBox.GetOrigin();
@@ -485,10 +450,10 @@ int AR_AUTOPLACER::testRectangle( const EDA_RECT& aRect, int side )
 }
 
 
-unsigned int AR_AUTOPLACER::calculateKeepOutArea( const EDA_RECT& aRect, int side )
+unsigned int AR_AUTOPLACER::calculateKeepOutArea( const BOX2I& aRect, int side )
 {
-    wxPoint start   = aRect.GetOrigin();
-    wxPoint end     = aRect.GetEnd();
+    VECTOR2I start = aRect.GetOrigin();
+    VECTOR2I end = aRect.GetEnd();
 
     start   -= m_matrix.m_BrdBox.GetOrigin();
     end     -= m_matrix.m_BrdBox.GetOrigin();
@@ -535,7 +500,7 @@ unsigned int AR_AUTOPLACER::calculateKeepOutArea( const EDA_RECT& aRect, int sid
 
 
 int AR_AUTOPLACER::testFootprintOnBoard( FOOTPRINT* aFootprint, bool TstOtherSide,
-                                         const wxPoint& aOffset )
+                                         const VECTOR2I& aOffset )
 {
     int side = AR_SIDE_TOP;
     int otherside = AR_SIDE_BOTTOM;
@@ -545,21 +510,17 @@ int AR_AUTOPLACER::testFootprintOnBoard( FOOTPRINT* aFootprint, bool TstOtherSid
         side = AR_SIDE_BOTTOM; otherside = AR_SIDE_TOP;
     }
 
-    EDA_RECT    fpBBox = aFootprint->GetBoundingBox( false, false );
-    fpBBox.Move( -aOffset );
+    BOX2I fpBBox = aFootprint->GetBoundingBox( false, false );
+    fpBBox.Move( -1*aOffset );
 
-    buildFpAreas( aFootprint, 0 );
-
-    int diag = //testModuleByPolygon( aFootprint, side, aOffset );
-        testRectangle( fpBBox, side );
+    int diag = testRectangle( fpBBox, side );
 
     if( diag != AR_FREE_CELL )
         return diag;
 
     if( TstOtherSide )
     {
-        diag = //testModuleByPolygon( aFootprint, otherside, aOffset );
-                testRectangle( fpBBox, otherside );
+        diag = testRectangle( fpBBox, otherside );
 
         if( diag != AR_FREE_CELL )
             return diag;
@@ -575,30 +536,30 @@ int AR_AUTOPLACER::testFootprintOnBoard( FOOTPRINT* aFootprint, bool TstOtherSid
 int AR_AUTOPLACER::getOptimalFPPlacement( FOOTPRINT* aFootprint )
 {
     int     error = 1;
-    wxPoint lastPosOK;
+    VECTOR2I lastPosOK;
     double  min_cost, curr_cost, Score;
     bool    testOtherSide;
 
     lastPosOK = m_matrix.m_BrdBox.GetOrigin();
 
-    wxPoint  fpPos = aFootprint->GetPosition();
-    EDA_RECT fpBBox  = aFootprint->GetBoundingBox( false, false );
+    VECTOR2I fpPos = aFootprint->GetPosition();
+    BOX2I    fpBBox  = aFootprint->GetBoundingBox( false, false );
 
     // Move fpBBox to have the footprint position at (0,0)
     fpBBox.Move( -fpPos );
-    wxPoint fpBBoxOrg = fpBBox.GetOrigin();
+    VECTOR2I fpBBoxOrg = fpBBox.GetOrigin();
 
     // Calculate the limit of the footprint position, relative to the routing matrix area
-    wxPoint xylimit = m_matrix.m_BrdBox.GetEnd() - fpBBox.GetEnd();
+    VECTOR2I xylimit = m_matrix.m_BrdBox.GetEnd() - fpBBox.GetEnd();
 
-    wxPoint initialPos = m_matrix.m_BrdBox.GetOrigin() - fpBBoxOrg;
+    VECTOR2I initialPos = m_matrix.m_BrdBox.GetOrigin() - fpBBoxOrg;
 
     // Stay on grid.
     initialPos.x    -= initialPos.x % m_matrix.m_GridRouting;
     initialPos.y    -= initialPos.y % m_matrix.m_GridRouting;
 
     m_curPosition = initialPos;
-    wxPoint fpOffset = fpPos - m_curPosition;
+    VECTOR2I fpOffset = fpPos - m_curPosition;
 
     // Examine pads, and set testOtherSide to true if a footprint has at least 1 pad through.
     testOtherSide = false;
@@ -645,11 +606,12 @@ int AR_AUTOPLACER::getOptimalFPPlacement( FOOTPRINT* aFootprint )
                 {
                     lastPosOK   = m_curPosition;
                     min_cost    = Score;
+/*
                     wxString msg;
-/*                    msg.Printf( wxT( "Score %g, pos %s, %s" ),
+                    msg.Printf( wxT( "Score %g, pos %s, %s" ),
                                 min_cost,
-                                GetChars( ::CoordinateToString( LastPosOK.x ) ),
-                                GetChars( ::CoordinateToString( LastPosOK.y ) ) );
+                                ::CoordinateToString( LastPosOK.x ),
+                                ::CoordinateToString( LastPosOK.y ) );
                     m_frame->SetStatusText( msg );*/
                 }
             }
@@ -664,7 +626,7 @@ int AR_AUTOPLACER::getOptimalFPPlacement( FOOTPRINT* aFootprint )
 }
 
 
-const PAD* AR_AUTOPLACER::nearestPad( FOOTPRINT *aRefFP, PAD* aRefPad, const wxPoint& aOffset)
+const PAD* AR_AUTOPLACER::nearestPad( FOOTPRINT* aRefFP, PAD* aRefPad, const VECTOR2I& aOffset )
 {
     const PAD* nearest = nullptr;
     int64_t    nearestDist = INT64_MAX;
@@ -697,7 +659,7 @@ const PAD* AR_AUTOPLACER::nearestPad( FOOTPRINT *aRefFP, PAD* aRefPad, const wxP
 }
 
 
-double AR_AUTOPLACER::computePlacementRatsnestCost( FOOTPRINT *aFootprint, const wxPoint& aOffset )
+double AR_AUTOPLACER::computePlacementRatsnestCost( FOOTPRINT* aFootprint, const VECTOR2I& aOffset )
 {
     double  curr_cost;
     VECTOR2I start;      // start point of a ratsnest
@@ -834,7 +796,7 @@ void AR_AUTOPLACER::drawPlacementRoutingMatrix( )
     m_overlay->SetIsFill( true );
     m_overlay->SetIsStroke( false );
 
-    SHAPE_POLY_SET freeArea = m_topFreeArea;
+    SHAPE_POLY_SET freeArea = m_topFreeArea.CloneDropTriangulation();
     freeArea.Fracture( SHAPE_POLY_SET::PM_FAST );
 
     // Draw the free polygon areas, top side:
@@ -862,17 +824,17 @@ AR_RESULT AR_AUTOPLACER::AutoplaceFootprints( std::vector<FOOTPRINT*>& aFootprin
                                               BOARD_COMMIT* aCommit,
                                               bool aPlaceOffboardModules )
 {
-    wxPoint memopos;
-    int     error;
-    bool    cancelled = false;
+    VECTOR2I memopos;
+    int      error;
+    bool     cancelled = false;
 
     memopos = m_curPosition;
 
     m_matrix.m_GridRouting = m_gridSize; //(int) m_frame->GetScreen()->GetGridSize().x;
 
     // Ensure Board.m_GridRouting has a reasonable value:
-    if( m_matrix.m_GridRouting < Millimeter2iu( 0.25 ) )
-        m_matrix.m_GridRouting = Millimeter2iu( 0.25 );
+    if( m_matrix.m_GridRouting < pcbIUScale.mmToIU( 0.25 ) )
+        m_matrix.m_GridRouting = pcbIUScale.mmToIU( 0.25 );
 
     // Compute footprint parameters used in autoplace
     if( genPlacementRoutingMatrix( ) == 0 )
@@ -916,7 +878,6 @@ AR_RESULT AR_AUTOPLACER::AutoplaceFootprints( std::vector<FOOTPRINT*>& aFootprin
 
 
     int         cnt = 0;
-    wxString    msg;
 
     if( m_progressReporter )
     {
@@ -934,101 +895,16 @@ AR_RESULT AR_AUTOPLACER::AutoplaceFootprints( std::vector<FOOTPRINT*>& aFootprin
     while( ( footprint = pickFootprint() ) != nullptr )
     {
         // Display some info about activity, footprint placement can take a while:
-        //m_frame->SetStatusText( msg );
 
         if( m_progressReporter )
             m_progressReporter->SetTitle( wxString::Format( _( "Autoplacing %s" ),
                                                             footprint->GetReference() ) );
 
-        double initialOrient = footprint->GetOrientation();
-
         error = getOptimalFPPlacement( footprint );
-        double bestScore = m_minCost;
-        double bestRotation = 0.0;
-        int rotAllowed;
-
-        if( error == AR_ABORT_PLACEMENT )
-            goto end_of_tst;
-
-        // Try orientations 90, 180, 270 degrees from initial orientation
-        rotAllowed = footprint->GetPlacementCost180();
-
-        if( rotAllowed != 0 )
-        {
-            rotateFootprint( footprint, 1800.0, true );
-            error   = getOptimalFPPlacement( footprint );
-            m_minCost *= OrientationPenalty[rotAllowed];
-
-            if( bestScore > m_minCost )    // This orientation is better.
-            {
-                bestScore   = m_minCost;
-                bestRotation = 1800.0;
-            }
-            else
-            {
-                rotateFootprint( footprint, initialOrient, false );
-            }
-
-            if( error == AR_ABORT_PLACEMENT )
-                goto end_of_tst;
-        }
-
-        // Determine if the best orientation of a footprint is 90.
-        rotAllowed = footprint->GetPlacementCost90();
-
-        if( rotAllowed != 0 )
-        {
-            rotateFootprint( footprint, 900.0, true );
-            error   = getOptimalFPPlacement( footprint );
-            m_minCost *= OrientationPenalty[rotAllowed];
-
-            if( bestScore > m_minCost )    // This orientation is better.
-            {
-                bestScore   = m_minCost;
-                bestRotation = 900.0;
-            }
-            else
-            {
-                rotateFootprint( footprint, initialOrient, false );
-            }
-
-            if( error == AR_ABORT_PLACEMENT )
-                goto end_of_tst;
-        }
-
-        // Determine if the best orientation of a footprint is -90.
-        if( rotAllowed != 0 )
-        {
-            rotateFootprint( footprint, 2700.0, true );
-            error = getOptimalFPPlacement( footprint );
-            m_minCost *= OrientationPenalty[rotAllowed];
-
-            if( bestScore > m_minCost )    // This orientation is better.
-            {
-                bestScore   = m_minCost;
-                bestRotation = 2700.0;
-            }
-            else
-            {
-                rotateFootprint( footprint, initialOrient, false );
-            }
-
-            if( error == AR_ABORT_PLACEMENT )
-                goto end_of_tst;
-        }
-
-end_of_tst:
 
         if( error == AR_ABORT_PLACEMENT )
             break;
 
-        bestRotation += initialOrient;
-
-        if( bestRotation != footprint->GetOrientation() )
-        {
-            rotateFootprint( footprint, bestRotation, false );
-        }
-
         // Place footprint.
         placeFootprint( footprint, true, m_curPosition );
 
