@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016 CERN
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2024 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,9 +23,11 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <macros.h>
 #include <board.h>
 #include <footprint.h>
 #include <pcb_group.h>
+#include <pcb_track.h>
 #include <tool/tool_manager.h>
 #include <tools/pcb_selection_tool.h>
 #include <tools/zone_filler_tool.h>
@@ -34,37 +36,44 @@
 #include <tools/pcb_tool_base.h>
 #include <tools/pcb_actions.h>
 #include <connectivity/connectivity_data.h>
+#include <teardrop/teardrop.h>
 
 #include <functional>
 using namespace std::placeholders;
 
 
-BOARD_COMMIT::BOARD_COMMIT( TOOL_MANAGER* aToolMgr ) :
-        m_toolMgr( aToolMgr ),
-        m_isFootprintEditor( false ),
-        m_isBoardEditor( false )
+BOARD_COMMIT::BOARD_COMMIT( TOOL_BASE* aTool ) :
+        m_toolMgr( aTool->GetManager() ),
+        m_isBoardEditor( false ),
+        m_isFootprintEditor( false )
+{
+    if( PCB_TOOL_BASE* pcb_tool = dynamic_cast<PCB_TOOL_BASE*>( aTool ) )
+    {
+        m_isBoardEditor = pcb_tool->IsBoardEditor();
+        m_isFootprintEditor = pcb_tool->IsFootprintEditor();
+    }
+}
+
+
+BOARD_COMMIT::BOARD_COMMIT( EDA_DRAW_FRAME* aFrame ) :
+        m_toolMgr( aFrame->GetToolManager() ),
+        m_isBoardEditor( aFrame->IsType( FRAME_PCB_EDITOR ) ),
+        m_isFootprintEditor( aFrame->IsType( FRAME_FOOTPRINT_EDITOR ) )
 {
 }
 
 
-BOARD_COMMIT::BOARD_COMMIT( PCB_TOOL_BASE* aTool )
+BOARD_COMMIT::BOARD_COMMIT( TOOL_MANAGER* aMgr ) :
+        m_toolMgr( aMgr ),
+        m_isBoardEditor( false ),
+        m_isFootprintEditor( false )
 {
-    m_toolMgr = aTool->GetManager();
-    m_isFootprintEditor = aTool->IsFootprintEditor();
-    m_isBoardEditor = aTool->IsBoardEditor();
-}
+    EDA_DRAW_FRAME* frame = dynamic_cast<EDA_DRAW_FRAME*>( aMgr->GetToolHolder() );
 
-
-BOARD_COMMIT::BOARD_COMMIT( EDA_DRAW_FRAME* aFrame )
-{
-    m_toolMgr = aFrame->GetToolManager();
-    m_isFootprintEditor = aFrame->IsType( FRAME_FOOTPRINT_EDITOR );
-    m_isBoardEditor = aFrame->IsType( FRAME_PCB_EDITOR );
-}
-
-
-BOARD_COMMIT::~BOARD_COMMIT()
-{
+    if( frame && frame->IsType( FRAME_PCB_EDITOR ) )
+        m_isBoardEditor = true;
+    else if( frame && frame->IsType( FRAME_FOOTPRINT_EDITOR ) )
+        m_isFootprintEditor = true;
 }
 
 
@@ -74,43 +83,39 @@ BOARD* BOARD_COMMIT::GetBoard() const
 }
 
 
-COMMIT& BOARD_COMMIT::Stage( EDA_ITEM* aItem, CHANGE_TYPE aChangeType )
+COMMIT& BOARD_COMMIT::Stage( EDA_ITEM* aItem, CHANGE_TYPE aChangeType, BASE_SCREEN* aScreen )
 {
-    aItem->ClearFlags( IS_MODIFIED_CHILD );
+    wxCHECK( aItem, *this );
 
-    // If aItem belongs a footprint, the full footprint will be saved because undo/redo does
-    // not handle "sub items" modifications.  This has implications for auto-zone-refill, so
-    // we need to store a bit more information.
-    if( aItem && aChangeType == CHT_MODIFY )
+    // Many operations (move, rotate, etc.) are applied directly to a group's children, so they
+    // must be staged as well.
+    if( aChangeType == CHT_MODIFY )
     {
-        if( aItem->Type() == PCB_FOOTPRINT_T )
+        if( PCB_GROUP* group = dynamic_cast<PCB_GROUP*>( aItem ) )
         {
-            static_cast<FOOTPRINT*>( aItem )->RunOnChildren(
+            group->RunOnChildren(
                     [&]( BOARD_ITEM* child )
                     {
-                        child->SetFlags( IS_MODIFIED_CHILD );
+                        Stage( child, aChangeType );
                     } );
         }
-        else if( FOOTPRINT* fp = dynamic_cast<FOOTPRINT*>( aItem->GetParent() ) )
-        {
-            aItem->SetFlags( IS_MODIFIED_CHILD );
-            aItem = fp;
-        }
     }
 
     return COMMIT::Stage( aItem, aChangeType );
 }
 
 
-COMMIT& BOARD_COMMIT::Stage( std::vector<EDA_ITEM*>& container, CHANGE_TYPE aChangeType )
+COMMIT& BOARD_COMMIT::Stage( std::vector<EDA_ITEM*>& container, CHANGE_TYPE aChangeType,
+                             BASE_SCREEN* aScreen )
 {
-    return COMMIT::Stage( container, aChangeType );
+    return COMMIT::Stage( container, aChangeType, aScreen );
 }
 
 
-COMMIT& BOARD_COMMIT::Stage( const PICKED_ITEMS_LIST& aItems, UNDO_REDO aModFlag )
+COMMIT& BOARD_COMMIT::Stage( const PICKED_ITEMS_LIST& aItems, UNDO_REDO aModFlag,
+                             BASE_SCREEN* aScreen )
 {
-    return COMMIT::Stage( aItems, aModFlag );
+    return COMMIT::Stage( aItems, aModFlag, aScreen );
 }
 
 
@@ -120,49 +125,31 @@ void BOARD_COMMIT::dirtyIntersectingZones( BOARD_ITEM* item, int aChangeType )
 
     ZONE_FILLER_TOOL* zoneFillerTool = m_toolMgr->GetTool<ZONE_FILLER_TOOL>();
 
-    if( item->Type() == PCB_ZONE_T || item->Type() == PCB_FP_ZONE_T )
+    if( item->Type() == PCB_ZONE_T )
         zoneFillerTool->DirtyZone( static_cast<ZONE*>( item ) );
 
-    if( item->Type() == PCB_FOOTPRINT_T )
-    {
-        static_cast<FOOTPRINT*>( item )->RunOnChildren(
-                [&]( BOARD_ITEM* child )
-                {
-                    if( aChangeType != CHT_MODIFY || ( child->GetFlags() & IS_MODIFIED_CHILD ) )
-                        dirtyIntersectingZones( child, aChangeType );
-                } );
-    }
-    else if( item->Type() == PCB_GROUP_T )
-    {
-        static_cast<PCB_GROUP*>( item )->RunOnChildren(
-                [&]( BOARD_ITEM* child )
-                {
-                    dirtyIntersectingZones( child, aChangeType );
-                } );
-    }
+    item->RunOnChildren( std::bind( &BOARD_COMMIT::dirtyIntersectingZones, this, _1, aChangeType ) );
+
+    BOARD* board = static_cast<BOARD*>( m_toolMgr->GetModel() );
+    BOX2I  bbox = item->GetBoundingBox();
+    LSET   layers = item->GetLayerSet();
+
+    if( layers.test( Edge_Cuts ) || layers.test( Margin ) )
+        layers = LSET::PhysicalLayersMask();
     else
+        layers &= LSET::AllCuMask();
+
+    if( layers.any() )
     {
-        BOARD* board = static_cast<BOARD*>( m_toolMgr->GetModel() );
-        BOX2I  bbox = item->GetBoundingBox();
-        LSET   layers = item->GetLayerSet();
-
-        if( layers.test( Edge_Cuts ) || layers.test( Margin ) )
-            layers = LSET::PhysicalLayersMask();
-        else
-            layers &= LSET::AllCuMask();
-
-        if( layers.any() )
+        for( ZONE* zone : board->Zones() )
         {
-            for( ZONE* zone : board->Zones() )
-            {
-                if( zone->GetIsRuleArea() )
-                    continue;
+            if( zone->GetIsRuleArea() )
+                continue;
 
-                if( ( zone->GetLayerSet() & layers ).any()
-                        && zone->GetBoundingBox().Intersects( bbox ) )
-                {
-                    zoneFillerTool->DirtyZone( zone );
-                }
+            if( ( zone->GetLayerSet() & layers ).any()
+                    && zone->GetBoundingBox().Intersects( bbox ) )
+            {
+                zoneFillerTool->DirtyZone( zone );
             }
         }
     }
@@ -171,27 +158,32 @@ void BOARD_COMMIT::dirtyIntersectingZones( BOARD_ITEM* item, int aChangeType )
 
 void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
 {
-    // Objects potentially interested in changes:
-    PICKED_ITEMS_LIST   undoList;
     KIGFX::VIEW*        view = m_toolMgr->GetView();
     BOARD*              board = static_cast<BOARD*>( m_toolMgr->GetModel() );
     PCB_BASE_FRAME*     frame = dynamic_cast<PCB_BASE_FRAME*>( m_toolMgr->GetToolHolder() );
-    std::set<EDA_ITEM*> savedModules;
     PCB_SELECTION_TOOL* selTool = m_toolMgr->GetTool<PCB_SELECTION_TOOL>();
+
+    // Notification info
+    PICKED_ITEMS_LIST   undoList;
     bool                itemsDeselected = false;
-    bool                solderMaskDirty = false;
-    bool                autofillZones = false;
     bool                selectedModified = false;
 
+    // Dirty flags and lists
+    bool                     solderMaskDirty = false;
+    bool                     autofillZones = false;
+    std::vector<BOARD_ITEM*> staleTeardropPadsAndVias;
+    std::set<PCB_TRACK*>     staleTeardropTracks;
+    PCB_GROUP*               addedGroup = nullptr;
+
     if( Empty() )
         return;
 
+    undoList.SetDescription( aMessage );
+
+    TEARDROP_MANAGER                   teardropMgr( board, m_toolMgr );
     std::shared_ptr<CONNECTIVITY_DATA> connectivity = board->GetConnectivity();
 
-    // Note:
-    // frame == nullptr happens in QA tests
-    // in this case m_isBoardEditor and m_isFootprintEditor are set to false
-    // But we also test frame == nullptr mainly to make Coverity happy
+    // Note: frame == nullptr happens in QA tests
 
     std::vector<BOARD_ITEM*> bulkAddedItems;
     std::vector<BOARD_ITEM*> bulkRemovedItems;
@@ -209,101 +201,102 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
 
     for( COMMIT_LINE& ent : m_changes )
     {
-        int changeType = ent.m_type & CHT_TYPE;
-        int changeFlags = ent.m_type & CHT_FLAGS;
+        if( !ent.m_item || !ent.m_item->IsBOARD_ITEM() )
+            continue;
+
         BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
 
-        wxASSERT( ent.m_item );
-
-        // Module items need to be saved in the undo buffer before modification
-        if( m_isFootprintEditor )
+        if( m_isBoardEditor )
         {
-            // Be sure that we are storing a footprint
-            if( ent.m_item->Type() != PCB_FOOTPRINT_T )
+            if( boardItem->Type() == PCB_VIA_T || boardItem->Type() == PCB_FOOTPRINT_T
+                    || boardItem->IsOnLayer( F_Mask ) || boardItem->IsOnLayer( B_Mask ) )
             {
-                ent.m_item = ent.m_item->GetParent();
-                wxASSERT( ent.m_item );
+                solderMaskDirty = true;
             }
 
-            // We have not saved the footprint yet, so let's create an entry
-            if( savedModules.count( ent.m_item ) == 0 )
+            if( !( aCommitFlags & SKIP_TEARDROPS ) )
             {
-                if( !ent.m_copy )
+                if( boardItem->Type() == PCB_FOOTPRINT_T )
                 {
-                    wxASSERT( changeType != CHT_MODIFY );     // too late to make a copy..
-                    ent.m_copy = makeImage( ent.m_item );
+                    for( PAD* pad : static_cast<FOOTPRINT*>( boardItem )->Pads() )
+                        staleTeardropPadsAndVias.push_back( pad );
                 }
-
-                wxASSERT( ent.m_item->Type() == PCB_FOOTPRINT_T );
-                wxASSERT( ent.m_copy->Type() == PCB_FOOTPRINT_T );
-
-                if( !( aCommitFlags & SKIP_UNDO ) && frame )
+                else if( boardItem->Type() == PCB_PAD_T || boardItem->Type() == PCB_VIA_T )
                 {
-                    ITEM_PICKER itemWrapper( nullptr, ent.m_item, UNDO_REDO::CHANGED );
-                    itemWrapper.SetLink( ent.m_copy );
-                    undoList.PushItem( itemWrapper );
-                    frame->SaveCopyInUndoList( undoList, UNDO_REDO::CHANGED );
+                    staleTeardropPadsAndVias.push_back( boardItem );
                 }
+                else if( boardItem->Type() == PCB_TRACE_T || boardItem->Type() == PCB_ARC_T )
+                {
+                    PCB_TRACK* track = static_cast<PCB_TRACK*>( boardItem );
 
-                savedModules.insert( ent.m_item );
+                    staleTeardropTracks.insert( track );
+
+                    std::vector<PAD*>     connectedPads;
+                    std::vector<PCB_VIA*> connectedVias;
+
+                    connectivity->GetConnectedPadsAndVias( track, &connectedPads, &connectedVias );
+
+                    for( PAD* pad : connectedPads )
+                        staleTeardropPadsAndVias.push_back( pad );
+
+                    for( PCB_VIA* via : connectedVias )
+                        staleTeardropPadsAndVias.push_back( via );
+                }
             }
         }
 
-        if( boardItem->Type() == PCB_VIA_T || boardItem->Type() == PCB_FOOTPRINT_T
-                || boardItem->IsOnLayer( F_Mask ) || boardItem->IsOnLayer( B_Mask ) )
-        {
-            solderMaskDirty = true;
-        }
-
         if( boardItem->IsSelected() )
             selectedModified = true;
+    }
+
+    // Old teardrops must be removed before connectivity is rebuilt
+    if( !staleTeardropPadsAndVias.empty() || !staleTeardropTracks.empty() )
+        teardropMgr.RemoveTeardrops( *this, &staleTeardropPadsAndVias, &staleTeardropTracks );
+
+    for( COMMIT_LINE& ent : m_changes )
+    {
+        if( !ent.m_item || !ent.m_item->IsBOARD_ITEM() )
+            continue;
+
+        BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
+        int         changeType = ent.m_type & CHT_TYPE;
+        int         changeFlags = ent.m_type & CHT_FLAGS;
 
         switch( changeType )
         {
         case CHT_ADD:
-        {
             if( selTool && selTool->GetEnteredGroup() && !boardItem->GetParentGroup()
                     && PCB_GROUP::IsGroupableType( boardItem->Type() ) )
             {
                 selTool->GetEnteredGroup()->AddItem( boardItem );
             }
 
-            if( m_isFootprintEditor )
-            {
-                // footprints inside footprints are not supported yet
-                wxASSERT( boardItem->Type() != PCB_FOOTPRINT_T );
+            if( !( aCommitFlags & SKIP_UNDO ) )
+                undoList.PushItem( ITEM_PICKER( nullptr, boardItem, UNDO_REDO::NEWITEM ) );
 
-                boardItem->SetParent( board->Footprints().front() );
-
-                if( !( changeFlags & CHT_DONE ) )
-                    board->Footprints().front()->Add( boardItem );
-            }
-            else if( boardItem->Type() == PCB_PAD_T
-                   || boardItem->Type() == PCB_FP_TEXT_T
-                   || boardItem->Type() == PCB_FP_TEXTBOX_T
-                   || boardItem->Type() == PCB_FP_SHAPE_T
-                   || boardItem->Type() == PCB_FP_DIM_ALIGNED_T
-                   || boardItem->Type() == PCB_FP_DIM_LEADER_T
-                   || boardItem->Type() == PCB_FP_DIM_CENTER_T
-                   || boardItem->Type() == PCB_FP_DIM_RADIAL_T
-                   || boardItem->Type() == PCB_FP_DIM_ORTHOGONAL_T
-                   || boardItem->Type() == PCB_FP_ZONE_T )
+            if( !( changeFlags & CHT_DONE ) )
             {
-                wxASSERT( boardItem->GetParent() &&
-                          boardItem->GetParent()->Type() == PCB_FOOTPRINT_T );
-            }
-            else
-            {
-                if( !( aCommitFlags & SKIP_UNDO ) )
-                    undoList.PushItem( ITEM_PICKER( nullptr, boardItem, UNDO_REDO::NEWITEM ) );
-
-                if( !( changeFlags & CHT_DONE ) )
+                if( m_isFootprintEditor )
+                {
+                    FOOTPRINT* parentFP = board->GetFirstFootprint();
+                    wxCHECK2_MSG( parentFP, continue, "Commit thinks this is footprint editor, but "
+                                                      "there is no first footprint!" );
+                    parentFP->Add( boardItem );
+                }
+                else if( FOOTPRINT* parentFP = boardItem->GetParentFootprint() )
+                {
+                    parentFP->Add( boardItem );
+                }
+                else
                 {
                     board->Add( boardItem, ADD_MODE::BULK_INSERT ); // handles connectivity
                     bulkAddedItems.push_back( boardItem );
                 }
             }
 
+            if( boardItem->Type() == PCB_GROUP_T || boardItem->Type() == PCB_GENERATOR_T )
+                addedGroup = static_cast<PCB_GROUP*>( boardItem );
+
             if( autofillZones && boardItem->Type() != PCB_MARKER_T )
                 dirtyIntersectingZones( boardItem, changeType );
 
@@ -311,13 +304,13 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
                 view->Add( boardItem );
 
             break;
-        }
 
         case CHT_REMOVE:
         {
+            FOOTPRINT* parentFP = boardItem->GetParentFootprint();
             PCB_GROUP* parentGroup = boardItem->GetParentGroup();
 
-            if( !m_isFootprintEditor && !( aCommitFlags & SKIP_UNDO ) )
+            if( !( aCommitFlags & SKIP_UNDO ) )
                 undoList.PushItem( ITEM_PICKER( nullptr, boardItem, UNDO_REDO::DELETED ) );
 
             if( boardItem->IsSelected() )
@@ -331,52 +324,23 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             if( parentGroup && !( parentGroup->GetFlags() & STRUCT_DELETED ) )
                 parentGroup->RemoveItem( boardItem );
 
+            if( parentFP && !( parentFP->GetFlags() & STRUCT_DELETED ) )
+                ent.m_parent = parentFP->m_Uuid;
+
             if( autofillZones )
                 dirtyIntersectingZones( boardItem, changeType );
 
             switch( boardItem->Type() )
             {
-                // Footprint items
-            case PCB_PAD_T:
-            case PCB_FP_SHAPE_T:
-            case PCB_FP_TEXT_T:
-            case PCB_FP_TEXTBOX_T:
-            case PCB_FP_DIM_ALIGNED_T:
-            case PCB_FP_DIM_LEADER_T:
-            case PCB_FP_DIM_CENTER_T:
-            case PCB_FP_DIM_RADIAL_T:
-            case PCB_FP_DIM_ORTHOGONAL_T:
-            case PCB_FP_ZONE_T:
-                // This level can only handle footprint children in the footprint editor as
-                // only in that case has the entire footprint (and all its children) already
-                // been saved for undo.
-                wxASSERT( m_isFootprintEditor );
-
-                if( boardItem->Type() == PCB_FP_TEXT_T )
-                {
-                    FP_TEXT* text = static_cast<FP_TEXT*>( boardItem );
-
-                    // don't allow deletion of Reference or Value
-                    if( text->GetType() != FP_TEXT::TEXT_is_DIVERS )
-                        break;
-                }
-
-                if( view )
-                    view->Remove( boardItem );
-
-                if( !( changeFlags & CHT_DONE ) )
-                {
-                    FOOTPRINT* footprint = static_cast<FOOTPRINT*>( boardItem->GetParent() );
-                    wxASSERT( footprint && footprint->Type() == PCB_FOOTPRINT_T );
-                    footprint->Delete( boardItem );
-                }
-
+            case PCB_FIELD_T:
+                static_cast<PCB_FIELD*>( boardItem )->SetVisible( false );
                 break;
 
-            // Board items
+            case PCB_TEXT_T:
+            case PCB_PAD_T:
             case PCB_SHAPE_T:            // a shape (normally not on copper layers)
-            case PCB_BITMAP_T:           // a bitmap on a user layer
-            case PCB_TEXT_T:             // a text on a layer
+            case PCB_REFERENCE_IMAGE_T:  // a bitmap on an associated layer
+            case PCB_GENERATOR_T:        // a generator on a layer
             case PCB_TEXTBOX_T:          // a wrapped text on a layer
             case PCB_TRACE_T:            // a track segment (segment on a copper layer)
             case PCB_ARC_T:              // an arced track segment (segment on a copper layer)
@@ -389,46 +353,17 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             case PCB_TARGET_T:           // a target (graphic item)
             case PCB_MARKER_T:           // a marker used to show something
             case PCB_ZONE_T:
-                if( view )
-                    view->Remove( boardItem );
-
-                if( !( changeFlags & CHT_DONE ) )
-                {
-                    board->Remove( boardItem, REMOVE_MODE::BULK );
-                    bulkRemovedItems.push_back( boardItem );
-                }
-
-                break;
-
             case PCB_FOOTPRINT_T:
-            {
-                // No support for nested footprints (yet)
-                wxASSERT( !m_isFootprintEditor );
-
-                FOOTPRINT* footprint = static_cast<FOOTPRINT*>( boardItem );
-
-                if( view )
-                    view->Remove( footprint );
-
-                footprint->ClearFlags();
-
-                if( !( changeFlags & CHT_DONE ) )
-                {
-                    board->Remove( footprint, REMOVE_MODE::BULK ); // handles connectivity
-                    bulkRemovedItems.push_back( footprint );
-                }
-            }
-
-            break;
-
             case PCB_GROUP_T:
                 if( view )
                     view->Remove( boardItem );
 
                 if( !( changeFlags & CHT_DONE ) )
                 {
-                    if( m_isFootprintEditor )
-                        board->GetFirstFootprint()->Remove( boardItem );
+                    if( parentFP )
+                    {
+                        parentFP->Remove( boardItem );
+                    }
                     else
                     {
                         board->Remove( boardItem, REMOVE_MODE::BULK );
@@ -449,47 +384,70 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
                 break;
             }
 
+            // The item has been removed from the board; it is now owned by undo/redo.
+            boardItem->SetFlags( UR_TRANSIENT );
+
             break;
         }
 
+        case CHT_UNGROUP:
+            if( PCB_GROUP* group = boardItem->GetParentGroup() )
+            {
+                if( !( aCommitFlags & SKIP_UNDO ) )
+                {
+                    ITEM_PICKER itemWrapper( nullptr, boardItem, UNDO_REDO::UNGROUP );
+                    itemWrapper.SetGroupId( group->m_Uuid );
+                    undoList.PushItem( itemWrapper );
+                }
+
+                group->RemoveItem( boardItem );
+            }
+
+            break;
+
+        case CHT_GROUP:
+            if( addedGroup )
+            {
+                addedGroup->AddItem( boardItem );
+
+                if( !( aCommitFlags & SKIP_UNDO ) )
+                    undoList.PushItem( ITEM_PICKER( nullptr, boardItem, UNDO_REDO::REGROUP ) );
+            }
+
+            break;
+
         case CHT_MODIFY:
         {
-            if( !m_isFootprintEditor && !( aCommitFlags & SKIP_UNDO ) )
+            BOARD_ITEM* boardItemCopy = nullptr;
+
+            if( ent.m_copy && ent.m_copy->IsBOARD_ITEM() )
+                boardItemCopy = static_cast<BOARD_ITEM*>( ent.m_copy );
+
+            if( !( aCommitFlags & SKIP_UNDO ) )
             {
                 ITEM_PICKER itemWrapper( nullptr, boardItem, UNDO_REDO::CHANGED );
-                wxASSERT( ent.m_copy );
-                itemWrapper.SetLink( ent.m_copy );
+                wxASSERT( boardItemCopy );
+                itemWrapper.SetLink( boardItemCopy );
                 undoList.PushItem( itemWrapper );
             }
 
             if( !( aCommitFlags & SKIP_CONNECTIVITY ) )
             {
-                if( ent.m_copy )
-                    connectivity->MarkItemNetAsDirty( static_cast<BOARD_ITEM*>( ent.m_copy ) );
+                if( boardItemCopy )
+                    connectivity->MarkItemNetAsDirty( boardItemCopy );
 
                 connectivity->Update( boardItem );
             }
 
-            if( autofillZones )
+            if( m_isBoardEditor && autofillZones )
             {
-                dirtyIntersectingZones( static_cast<BOARD_ITEM*>( ent.m_copy ), changeType );   // before
-                dirtyIntersectingZones( boardItem, changeType );                                // after
+                dirtyIntersectingZones( boardItemCopy, changeType );   // before
+                dirtyIntersectingZones( boardItem, changeType );       // after
             }
 
             if( view )
-            {
                 view->Update( boardItem );
 
-                if( m_isFootprintEditor )
-                {
-                    static_cast<FOOTPRINT*>( boardItem )->RunOnChildren(
-                            [&]( BOARD_ITEM* aChild )
-                            {
-                                view->Update( aChild );
-                            });
-                }
-            }
-
             itemsChanged.push_back( boardItem );
 
             // if no undo entry is needed, the copy would create a memory leak
@@ -500,20 +458,18 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
         }
 
         default:
-            wxASSERT( false );
+            UNIMPLEMENTED_FOR( boardItem->GetClass() );
             break;
         }
+
+        boardItem->ClearEditFlags();
+        boardItem->RunOnDescendants(
+                [&]( BOARD_ITEM* item )
+                {
+                    item->ClearEditFlags();
+                } );
     }
 
-    if( bulkAddedItems.size() > 0 )
-        board->FinalizeBulkAdd( bulkAddedItems );
-
-    if( bulkRemovedItems.size() > 0 )
-        board->FinalizeBulkRemove( bulkRemovedItems );
-
-    if( itemsChanged.size() > 0 )
-        board->OnItemsChanged( itemsChanged );
-
     if( m_isBoardEditor )
     {
         size_t num_changes = m_changes.size();
@@ -531,6 +487,8 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
 
             if( frame )
                 frame->GetCanvas()->RedrawRatsnest();
+
+            board->OnRatsnestChanged();
         }
 
         if( solderMaskDirty )
@@ -539,20 +497,33 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
                 frame->HideSolderMask();
         }
 
-        // Log undo items for any connectivity changes
+        if( !staleTeardropPadsAndVias.empty() || !staleTeardropTracks.empty() )
+        {
+            teardropMgr.UpdateTeardrops( *this, &staleTeardropPadsAndVias, &staleTeardropTracks );
+
+            // UpdateTeardrops() can modify the ratsnest data. So rebuild this ratsnest data
+            connectivity->RecalculateRatsnest( this );
+        }
+
+        // Log undo items for any connectivity or teardrop changes
         for( size_t i = num_changes; i < m_changes.size(); ++i )
         {
             COMMIT_LINE& ent = m_changes[i];
+            BOARD_ITEM*  boardItem = nullptr;
+            BOARD_ITEM*  boardItemCopy = nullptr;
 
-            wxASSERT( ( ent.m_type & CHT_TYPE ) == CHT_MODIFY );
+            if( ent.m_item && ent.m_item->IsBOARD_ITEM() )
+                boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
 
-            BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
+            if( ent.m_copy && ent.m_copy->IsBOARD_ITEM() )
+                boardItemCopy = static_cast<BOARD_ITEM*>( ent.m_copy );
+
+            wxCHECK2( boardItem, continue );
 
             if( !( aCommitFlags & SKIP_UNDO ) )
             {
-                ITEM_PICKER itemWrapper( nullptr, boardItem, UNDO_REDO::CHANGED );
-                wxASSERT( ent.m_copy );
-                itemWrapper.SetLink( ent.m_copy );
+                ITEM_PICKER itemWrapper( nullptr, boardItem, convert( ent.m_type & CHT_TYPE ) );
+                itemWrapper.SetLink( boardItemCopy );
                 undoList.PushItem( itemWrapper );
             }
             else
@@ -561,16 +532,35 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             }
 
             if( view )
-                view->Update( boardItem );
+            {
+                if( ( ent.m_type & CHT_TYPE ) == CHT_ADD )
+                    view->Add( boardItem );
+                else if( ( ent.m_type & CHT_TYPE ) == CHT_REMOVE )
+                    view->Remove( boardItem );
+                else
+                    view->Update( boardItem );
+            }
         }
     }
 
-    if( m_isBoardEditor && !( aCommitFlags & SKIP_UNDO ) && frame )
+    if( bulkAddedItems.size() > 0 )
+        board->FinalizeBulkAdd( bulkAddedItems );
+
+    if( bulkRemovedItems.size() > 0 )
+        board->FinalizeBulkRemove( bulkRemovedItems );
+
+    if( itemsChanged.size() > 0 )
+        board->OnItemsChanged( itemsChanged );
+
+    if( frame )
     {
-        if( aCommitFlags & APPEND_UNDO )
-            frame->AppendCopyToUndoList( undoList, UNDO_REDO::UNSPECIFIED );
-        else
-            frame->SaveCopyInUndoList( undoList, UNDO_REDO::UNSPECIFIED );
+        if( !( aCommitFlags & SKIP_UNDO ) )
+        {
+            if( aCommitFlags & APPEND_UNDO )
+                frame->AppendCopyToUndoList( undoList, UNDO_REDO::UNSPECIFIED );
+            else
+                frame->SaveCopyInUndoList( undoList, UNDO_REDO::UNSPECIFIED );
+        }
     }
 
     m_toolMgr->PostEvent( { TC_MESSAGE, TA_MODEL_CHANGE, AS_GLOBAL } );
@@ -579,7 +569,7 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
         m_toolMgr->PostEvent( EVENTS::UnselectedEvent );
 
     if( autofillZones )
-        m_toolMgr->RunAction( PCB_ACTIONS::zoneFillDirty );
+        m_toolMgr->PostAction( PCB_ACTIONS::zoneFillDirty );
 
     if( selectedModified )
         m_toolMgr->ProcessEvent( EVENTS::SelectedItemsModified );
@@ -590,6 +580,15 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             frame->OnModify();
         else
             frame->Update3DView( true, frame->GetPcbNewSettings()->m_Display.m_Live3DRefresh );
+
+        // Ensure the message panel is updated after committing changes.
+        // By default (i.e. if no event posted), display the updated board info
+        if( !itemsDeselected && !autofillZones && !selectedModified )
+        {
+            std::vector<MSG_PANEL_ITEM> msg_list;
+            board->GetMsgPanelInfo( frame, msg_list );
+            frame->SetMsgPanel( msg_list );
+        }
     }
 
     clear();
@@ -598,37 +597,25 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
 
 EDA_ITEM* BOARD_COMMIT::parentObject( EDA_ITEM* aItem ) const
 {
-    switch( aItem->Type() )
-    {
-    case PCB_PAD_T:
-    case PCB_FP_SHAPE_T:
-    case PCB_FP_TEXT_T:
-    case PCB_FP_TEXTBOX_T:
-    case PCB_FP_DIM_ALIGNED_T:
-    case PCB_FP_DIM_LEADER_T:
-    case PCB_FP_DIM_CENTER_T:
-    case PCB_FP_DIM_RADIAL_T:
-    case PCB_FP_DIM_ORTHOGONAL_T:
-    case PCB_FP_ZONE_T:
-        return aItem->GetParent();
-
-    case PCB_ZONE_T:
-        wxASSERT( !dynamic_cast<FOOTPRINT*>( aItem->GetParent() ) );
-        return aItem;
-
-    default:
-        break;
-    }
-
     return aItem;
 }
 
 
 EDA_ITEM* BOARD_COMMIT::makeImage( EDA_ITEM* aItem ) const
 {
-    BOARD_ITEM* clone = static_cast<BOARD_ITEM*>( aItem->Clone() );
+    return MakeImage( aItem );
+}
+
+
+EDA_ITEM* BOARD_COMMIT::MakeImage( EDA_ITEM* aItem )
+{
+    EDA_ITEM* clone = aItem->Clone();
+
+    if( clone->IsBOARD_ITEM() )
+        static_cast<BOARD_ITEM*>( clone )->SetParentGroup( nullptr );
+
+    clone->SetFlags( UR_TRANSIENT );
 
-    clone->SetParentGroup( nullptr );
     return clone;
 }
 
@@ -649,63 +636,98 @@ void BOARD_COMMIT::Revert()
     for( auto it = m_changes.rbegin(); it != m_changes.rend(); ++it )
     {
         COMMIT_LINE& ent = *it;
-        BOARD_ITEM* item = static_cast<BOARD_ITEM*>( ent.m_item );
-        BOARD_ITEM* copy = static_cast<BOARD_ITEM*>( ent.m_copy );
-        int changeType = ent.m_type & CHT_TYPE;
-        int changeFlags = ent.m_type & CHT_FLAGS;
+
+        if( !ent.m_item || !ent.m_item->IsBOARD_ITEM() )
+            continue;
+
+        BOARD_ITEM*  boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
+        int          changeType = ent.m_type & CHT_TYPE;
+        int          changeFlags = ent.m_type & CHT_FLAGS;
 
         switch( changeType )
         {
         case CHT_ADD:
+            // Items are auto-added to the parent group by BOARD_ITEM::Duplicate(), not when
+            // the commit is pushed.
+            if( PCB_GROUP* parentGroup = boardItem->GetParentGroup() )
+            {
+                if( GetStatus( parentGroup ) == 0 )
+                    parentGroup->RemoveItem( boardItem );
+            }
+
             if( !( changeFlags & CHT_DONE ) )
                 break;
 
-            view->Remove( item );
-            connectivity->Remove( item );
-            board->Remove( item, REMOVE_MODE::BULK );
-            bulkRemovedItems.push_back( item );
+            view->Remove( boardItem );
+            connectivity->Remove( boardItem );
+
+            if( FOOTPRINT* parentFP = boardItem->GetParentFootprint() )
+            {
+                parentFP->Remove( boardItem );
+            }
+            else
+            {
+                board->Remove( boardItem, REMOVE_MODE::BULK );
+                bulkRemovedItems.push_back( boardItem );
+            }
+
             break;
 
         case CHT_REMOVE:
+        {
             if( !( changeFlags & CHT_DONE ) )
                 break;
 
-            view->Add( item );
-            connectivity->Add( item );
-            board->Add( item, ADD_MODE::INSERT );
-            bulkAddedItems.push_back( item );
+            view->Add( boardItem );
+            connectivity->Add( boardItem );
+
+            BOARD_ITEM* parent = board->GetItem( ent.m_parent );
+
+            if( parent && parent->Type() == PCB_FOOTPRINT_T )
+            {
+                static_cast<FOOTPRINT*>( parent )->Add( boardItem, ADD_MODE::INSERT );
+            }
+            else
+            {
+                board->Add( boardItem, ADD_MODE::INSERT );
+                bulkAddedItems.push_back( boardItem );
+            }
+
             break;
+        }
 
         case CHT_MODIFY:
         {
-            view->Remove( item );
-            connectivity->Remove( item );
+            view->Remove( boardItem );
+            connectivity->Remove( boardItem );
 
-            item->SwapItemData( copy );
+            wxASSERT( ent.m_copy && ent.m_copy->IsBOARD_ITEM() );
+            BOARD_ITEM* boardItemCopy = static_cast<BOARD_ITEM*>( ent.m_copy );
+            boardItem->SwapItemData( boardItemCopy );
 
-            if( item->Type() == PCB_GROUP_T )
+            if( PCB_GROUP* group = dynamic_cast<PCB_GROUP*>( boardItem ) )
             {
-                PCB_GROUP* group = static_cast<PCB_GROUP*>( item );
-
-                group->RunOnChildren( [&]( BOARD_ITEM* child )
-                                      {
-                                          child->SetParentGroup( group );
-                                      } );
+                group->RunOnChildren(
+                        [&]( BOARD_ITEM* child )
+                        {
+                            child->SetParentGroup( group );
+                        } );
             }
 
-            view->Add( item );
-            connectivity->Add( item );
-            board->OnItemChanged( item );
-            itemsChanged.push_back( item );
+            view->Add( boardItem );
+            connectivity->Add( boardItem );
+            itemsChanged.push_back( boardItem );
 
-            delete copy;
+            delete ent.m_copy;
             break;
         }
 
         default:
-            wxASSERT( false );
+            UNIMPLEMENTED_FOR( boardItem->GetClass() );
             break;
         }
+
+        boardItem->ClearEditFlags();
     }
 
     if( bulkAddedItems.size() > 0 )
@@ -717,15 +739,19 @@ void BOARD_COMMIT::Revert()
     if( itemsChanged.size() > 0 )
         board->OnItemsChanged( itemsChanged );
 
-    if ( !m_isFootprintEditor )
+    if( m_isBoardEditor )
     {
         connectivity->RecalculateRatsnest();
         board->UpdateRatsnestExclusions();
+        board->OnRatsnestChanged();
     }
 
     PCB_SELECTION_TOOL* selTool = m_toolMgr->GetTool<PCB_SELECTION_TOOL>();
     selTool->RebuildSelection();
 
+    // Property panel needs to know about the reselect
+    m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
+
     clear();
 }
 
