@@ -42,7 +42,6 @@
 #include <functional>
 #include <vector>
 #include <deque>
-#include <intrusive_list.h>
 
 #include <connectivity/connectivity_rtree.h>
 #include <connectivity/connectivity_data.h>
@@ -93,6 +92,15 @@ public:
     void SetSourceNode( const std::shared_ptr<const CN_ANCHOR>& aNode ) { m_source = aNode; }
     void SetTargetNode( const std::shared_ptr<const CN_ANCHOR>& aNode ) { m_target = aNode; }
 
+    void RemoveInvalidRefs()
+    {
+        if( m_source && !m_source->Valid() )
+            m_source.reset();
+
+        if( m_target && !m_target->Valid() )
+            m_target.reset();
+    }
+
     void SetWeight( unsigned weight ) { m_weight = weight; }
     unsigned GetWeight() const { return m_weight; }
 
@@ -101,7 +109,7 @@ public:
 
     const VECTOR2I GetSourcePos() const { return m_source->Pos(); }
     const VECTOR2I GetTargetPos() const { return m_target->Pos(); }
-    const unsigned GetLength() const
+    unsigned GetLength() const
     {
         return ( m_target->Pos() - m_source->Pos() ).EuclideanNorm();
     }
@@ -126,6 +134,11 @@ public:
 
     using CLUSTERS = std::vector<std::shared_ptr<CN_CLUSTER>>;
 
+    /*
+     * Holds a list of CN_ITEMs for a given BOARD_CONNECTED_ITEM.  For most items (pads, tracks,
+     * etc.) the list will have a single CN_ITEM, but for ZONEs it will have one item for each
+     * distinct outline on each layer.
+     */
     class ITEM_MAP_ENTRY
     {
     public:
@@ -154,8 +167,15 @@ public:
         std::list<CN_ITEM*> m_items;
     };
 
-    CN_CONNECTIVITY_ALGO() {}
-    ~CN_CONNECTIVITY_ALGO() { Clear(); }
+    CN_CONNECTIVITY_ALGO( CONNECTIVITY_DATA* aParentConnectivityData ) :
+            m_parentConnectivityData( aParentConnectivityData ),
+            m_isLocal( false )
+    {}
+
+    ~CN_CONNECTIVITY_ALGO()
+    {
+        Clear();
+    }
 
     bool ItemExists( const BOARD_CONNECTED_ITEM* aItem ) const
     {
@@ -197,8 +217,9 @@ public:
         return m_dirtyNets.size();
     }
 
-    void Build( BOARD* aZoneLayer, PROGRESS_REPORTER* aReporter = nullptr );
-    void LocalBuild( const std::vector<BOARD_ITEM*>& aItems );
+    void Build( BOARD* aBoard, PROGRESS_REPORTER* aReporter = nullptr );
+    void LocalBuild( std::shared_ptr<CONNECTIVITY_DATA> aGlobalConnectivity,
+                     const std::vector<BOARD_ITEM*>& aLocalItems );
 
     void Clear();
 
@@ -216,18 +237,11 @@ public:
      */
     void PropagateNets( BOARD_COMMIT* aCommit = nullptr );
 
-    void FindIsolatedCopperIslands( ZONE* aZone, PCB_LAYER_ID aLayer, std::vector<int>& aIslands );
-
     /**
-     * Find the copper islands that are not connected to a net.
-     *
-     * These are added to the m_islands vector.
-     * N.B. This must be called after aZones has been refreshed.
-     *
-     * @param: aZones is the set of zones to search for islands.
+     * Fill in the isolated islands map with copper islands that are not connected to a net.
      */
-    void FindIsolatedCopperIslands( std::vector<CN_ZONE_ISOLATED_ISLAND_LIST>& aZones,
-                                    bool aConnectivityAlreadyRebuilt );
+    void FillIsolatedIslandsMap( std::map<ZONE*, std::map<PCB_LAYER_ID, ISOLATED_ISLANDS>>& aMap,
+                                 bool aConnectivityAlreadyRebuilt );
 
     const CLUSTERS& GetClusters();
 
@@ -254,6 +268,8 @@ public:
     }
 
     void MarkNetAsDirty( int aNet );
+    void RemoveInvalidRefs();
+
     void SetProgressReporter( PROGRESS_REPORTER* aReporter );
 
 private:
@@ -272,6 +288,7 @@ private:
     void markItemNetAsDirty( const BOARD_ITEM* aItem );
 
 private:
+    CONNECTIVITY_DATA*                                    m_parentConnectivityData;
     CN_LIST                                               m_itemList;
     std::unordered_map<const BOARD_ITEM*, ITEM_MAP_ENTRY> m_itemMap;
 
@@ -279,6 +296,9 @@ private:
     std::vector<std::shared_ptr<CN_CLUSTER>>              m_ratsnestClusters;
     std::vector<bool>                                     m_dirtyNets;
 
+    bool                                                  m_isLocal;
+    std::shared_ptr<CONNECTIVITY_DATA>                    m_globalConnectivityData;
+
     PROGRESS_REPORTER* m_progressReporter = nullptr;
 };
 
