@@ -23,6 +23,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <unordered_set>
+
 #include <trigo.h>
 #include <macros.h>
 
@@ -35,6 +37,8 @@
 #include <geometry/shape_poly_set.h>
 #include <geometry/geometry_utils.h>
 #include <convert_shape_list_to_polygon.h>
+#include <board.h>
+#include <collectors.h>
 
 #include <wx/log.h>
 
@@ -48,43 +52,58 @@
  */
 const wxChar* traceBoardOutline = wxT( "KICAD_BOARD_OUTLINE" );
 
+
+class SCOPED_FLAGS_CLEANER : public std::unordered_set<EDA_ITEM*>
+{
+    EDA_ITEM_FLAGS m_flagsToClear;
+
+public:
+    SCOPED_FLAGS_CLEANER( const EDA_ITEM_FLAGS& aFlagsToClear ) : m_flagsToClear( aFlagsToClear ) {}
+
+    ~SCOPED_FLAGS_CLEANER()
+    {
+        for( EDA_ITEM* item : *this )
+            item->ClearFlags( m_flagsToClear );
+    }
+};
+
+
 /**
- * Function close_enough
- * is a local and tunable method of qualifying the proximity of two points.
+ * Local and tunable method of qualifying the proximity of two points.
  *
- * @param aLeft is the first point
- * @param aRight is the second point
+ * @param aLeft is the first point.
+ * @param aRight is the second point.
  * @param aLimit is a measure of proximity that the caller knows about.
- * @return bool - true if the two points are close enough, else false.
+ * @return true if the two points are close enough, else false.
  */
-bool close_enough( VECTOR2I aLeft, VECTOR2I aRight, unsigned aLimit )
+static bool close_enough( VECTOR2I aLeft, VECTOR2I aRight, unsigned aLimit )
 {
     return ( aLeft - aRight ).SquaredEuclideanNorm() <= SEG::Square( aLimit );
 }
 
+
 /**
- * Function closer_to_first
  * Local method which qualifies whether the start or end point of a segment is closest to a point.
  *
  * @param aRef is the reference point
  * @param aFirst is the first point
  * @param aSecond is the second point
- * @return bool - true if the first point is closest to the reference, otherwise false.
+ * @return true if the first point is closest to the reference, otherwise false.
  */
-bool closer_to_first( VECTOR2I aRef, VECTOR2I aFirst, VECTOR2I aSecond )
+static bool closer_to_first( VECTOR2I aRef, VECTOR2I aFirst, VECTOR2I aSecond )
 {
     return ( aRef - aFirst ).SquaredEuclideanNorm() < ( aRef - aSecond ).SquaredEuclideanNorm();
 }
 
 
 /**
- * Searches for a PCB_SHAPE matching a given end point or start point in a list.
+ * Search for a #PCB_SHAPE matching a given end point or start point in a list.
+ *
  * @param aShape The starting shape.
  * @param aPoint The starting or ending point to search for.
  * @param aList The list to remove from.
  * @param aLimit is the distance from \a aPoint that still constitutes a valid find.
- * @return PCB_SHAPE* - The first PCB_SHAPE that has a start or end point matching
- *   aPoint, otherwise NULL if none.
+ * @return The first #PCB_SHAPE that has a start or end point matching aPoint, otherwise nullptr.
  */
 static PCB_SHAPE* findNext( PCB_SHAPE* aShape, const VECTOR2I& aPoint,
                             const std::vector<PCB_SHAPE*>& aList, unsigned aLimit )
@@ -132,7 +151,7 @@ static PCB_SHAPE* findNext( PCB_SHAPE* aShape, const VECTOR2I& aPoint,
 }
 
 
-bool isCopperOutside( const FOOTPRINT* aFootprint, SHAPE_POLY_SET& aShape )
+static bool isCopperOutside( const FOOTPRINT* aFootprint, SHAPE_POLY_SET& aShape )
 {
     bool padOutside = false;
 
@@ -140,7 +159,10 @@ bool isCopperOutside( const FOOTPRINT* aFootprint, SHAPE_POLY_SET& aShape )
     {
         SHAPE_POLY_SET poly = aShape.CloneDropTriangulation();
 
-        poly.BooleanIntersection( *pad->GetEffectivePolygon(), SHAPE_POLY_SET::PM_FAST );
+        poly.ClearArcs();
+
+        poly.BooleanIntersection( *pad->GetEffectivePolygon( ERROR_INSIDE ),
+                                  SHAPE_POLY_SET::PM_FAST );
 
         if( poly.OutlineCount() == 0 )
         {
@@ -160,21 +182,10 @@ bool isCopperOutside( const FOOTPRINT* aFootprint, SHAPE_POLY_SET& aShape )
 }
 
 
-/**
- * Function ConvertOutlineToPolygon
- * Build a polygon (with holes) from a PCB_SHAPE list, which is expected to be a closed main
- * outline with perhaps closed inner outlines.  These closed inner outlines are considered as
- * holes in the main outline.
- * @param aShapeList the initial list of SHAPEs (only lines, circles and arcs).
- * @param aPolygons will contain the complex polygon.
- * @param aErrorMax is the max error distance when polygonizing a curve (internal units)
- * @param aChainingEpsilon is the max error distance when polygonizing a curve (internal units)
- * @param aAllowDisjoint indicates multiple top-level outlines are allowed
- * @param aErrorHandler = an optional error handler
- */
-bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SET& aPolygons,
-                              int aErrorMax, int aChainingEpsilon, bool aAllowDisjoint,
-                              OUTLINE_ERROR_HANDLER* aErrorHandler )
+bool doConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SET& aPolygons,
+                                int aErrorMax, int aChainingEpsilon, bool aAllowDisjoint,
+                                OUTLINE_ERROR_HANDLER* aErrorHandler, bool aAllowUseArcsInPolygons,
+                                SCOPED_FLAGS_CLEANER& aCleaner )
 {
     if( aShapeList.size() == 0 )
         return true;
@@ -209,31 +220,22 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
     {
         graphic = (PCB_SHAPE*) *startCandidates.begin();
         graphic->SetFlags( SKIP_STRUCT );
+        aCleaner.insert( graphic );
         startCandidates.erase( startCandidates.begin() );
 
         contours.emplace_back();
 
         SHAPE_LINE_CHAIN& currContour = contours.back();
+        currContour.SetWidth( graphic->GetWidth() );
         bool firstPt = true;
 
         // Circles, rects and polygons are closed shapes unto themselves (and do not combine
         // with other shapes), so process them separately.
         if( graphic->GetShape() == SHAPE_T::POLY )
         {
-            EDA_ANGLE orientation = ANGLE_0;
-            VECTOR2I  offset = VECTOR2I( 0, 0 );
-
-            if( graphic->GetParentFootprint() )
-            {
-                orientation = graphic->GetParentFootprint()->GetOrientation();
-                offset = graphic->GetParentFootprint()->GetPosition();
-            }
-
             for( auto it = graphic->GetPolyShape().CIterate(); it; it++ )
             {
                 VECTOR2I pt = *it;
-                RotatePoint( pt, orientation );
-                pt += offset;
 
                 currContour.Append( pt );
 
@@ -249,32 +251,27 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
         }
         else if( graphic->GetShape() == SHAPE_T::CIRCLE )
         {
-            // make a circle by segments;
             VECTOR2I center = graphic->GetCenter();
-            VECTOR2I start = center;
             int      radius  = graphic->GetRadius();
-            int      steps   = GetArcToSegmentCount( radius, aErrorMax, FULL_CIRCLE );
-            VECTOR2I nextPt;
-
+            VECTOR2I start = center;
             start.x += radius;
 
-            for( int step = 0; step < steps; ++step )
+            // Add 360 deg Arc in currContour
+            SHAPE_ARC arc360( center, start, ANGLE_360, 0 );
+            currContour.Append( arc360, aErrorMax );
+            currContour.SetClosed( true );
+
+            // set shapeOwners for currContour points created by appending the arc360:
+            for( int ii = 1; ii < currContour.PointCount(); ++ii )
             {
-                nextPt = start;
-                RotatePoint( nextPt, center, ANGLE_360 * step / steps );
-                currContour.Append( nextPt );
-
-                if( firstPt )
-                    firstPt = false;
-                else
-                    shapeOwners[ std::make_pair( prevPt, nextPt ) ] = graphic;
-
-                prevPt = nextPt;
+                shapeOwners[ std::make_pair( currContour.CPoint( ii-1 ),
+                                             currContour.CPoint( ii ) ) ] = graphic;
             }
 
-            currContour.SetClosed( true );
+            if( !aAllowUseArcsInPolygons )
+                currContour.ClearArcs();
         }
-        else if( graphic->GetShape() == SHAPE_T::RECT )
+        else if( graphic->GetShape() == SHAPE_T::RECTANGLE )
         {
             std::vector<VECTOR2I> pts = graphic->GetRectCorners();
 
@@ -296,7 +293,6 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
         {
             // Polygon start point. Arbitrarily chosen end of the segment and build the poly
             // from here.
-
             VECTOR2I startPt = graphic->GetEnd();
             prevPt = startPt;
             currContour.Append( prevPt );
@@ -306,15 +302,15 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
             {
                 switch( graphic->GetShape() )
                 {
-                case SHAPE_T::RECT:
+                case SHAPE_T::RECTANGLE:
                 case SHAPE_T::CIRCLE:
                 {
                     // As a non-first item, closed shapes can't be anything but self-intersecting
-
                     if( aErrorHandler )
                     {
                         wxASSERT( prevGraphic );
-                        (*aErrorHandler)( _( "(self-intersecting)" ), prevGraphic, graphic, prevPt );
+                        (*aErrorHandler)( _( "(self-intersecting)" ), prevGraphic, graphic,
+                                          prevPt );
                     }
 
                     selfIntersecting = true;
@@ -324,118 +320,115 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                 }
 
                 case SHAPE_T::SEGMENT:
-                    {
-                        VECTOR2I nextPt;
+                {
+                    VECTOR2I nextPt;
 
-                        // Use the line segment end point furthest away from prevPt as we assume
-                        // the other end to be ON prevPt or very close to it.
+                    // Use the line segment end point furthest away from prevPt as we assume
+                    // the other end to be ON prevPt or very close to it.
+                    if( closer_to_first( prevPt, graphic->GetStart(), graphic->GetEnd()) )
+                        nextPt = graphic->GetEnd();
+                    else
+                        nextPt = graphic->GetStart();
 
-                        if( closer_to_first( prevPt, graphic->GetStart(), graphic->GetEnd()) )
-                            nextPt = graphic->GetEnd();
-                        else
-                            nextPt = graphic->GetStart();
-
-                        currContour.Append( nextPt );
-                        shapeOwners[ std::make_pair( prevPt, nextPt ) ] = graphic;
-                        prevPt = nextPt;
-                    }
-                    break;
+                    currContour.Append( nextPt );
+                    shapeOwners[ std::make_pair( prevPt, nextPt ) ] = graphic;
+                    prevPt = nextPt;
+                }
+                break;
 
                 case SHAPE_T::ARC:
-                    // We do not support arcs in polygons, so approximate an arc with a series of
-                    // short lines and put those line segments into the !same! PATH.
+                {
+                    VECTOR2I  pstart = graphic->GetStart();
+                    VECTOR2I  pmid = graphic->GetArcMid();
+                    VECTOR2I  pend = graphic->GetEnd();
+
+                    if( !close_enough( prevPt, pstart, aChainingEpsilon ) )
                     {
-                        VECTOR2I  pstart  = graphic->GetStart();
-                        VECTOR2I  pend    = graphic->GetEnd();
-                        VECTOR2I  pcenter = graphic->GetCenter();
-                        EDA_ANGLE angle   = -graphic->GetArcAngle();
-                        int       radius  = graphic->GetRadius();
-                        int       steps   = GetArcToSegmentCount( radius, aErrorMax, angle );
+                        wxASSERT( close_enough( prevPt, graphic->GetEnd(), aChainingEpsilon ) );
 
-                        if( !close_enough( prevPt, pstart, aChainingEpsilon ) )
+                        std::swap( pstart, pend );
+                    }
+
+                    // Snap the arc start point to avoid potential self-intersections
+                    pstart = prevPt;
+
+                    SHAPE_ARC sarc( pstart, pmid, pend, 0 );
+
+                    SHAPE_LINE_CHAIN arcChain;
+                    arcChain.Append( sarc, aErrorMax );
+
+                    if( !aAllowUseArcsInPolygons )
+                        arcChain.ClearArcs();
+
+                    // set shapeOwners for arcChain points created by appending the sarc:
+                    for( int ii = 1; ii < arcChain.PointCount(); ++ii )
+                    {
+                        shapeOwners[std::make_pair( arcChain.CPoint( ii - 1 ),
+                                                    arcChain.CPoint( ii ) )] = graphic;
+                    }
+
+                    currContour.Append( arcChain );
+
+                    prevPt = pend;
+                }
+                break;
+
+                case SHAPE_T::BEZIER:
+                {
+                    // We do not support Bezier curves in polygons, so approximate with a series
+                    // of short lines and put those line segments into the !same! PATH.
+                    VECTOR2I nextPt;
+                    bool    reverse = false;
+
+                    // Use the end point furthest away from  prevPt as we assume the other
+                    // end to be ON prevPt or very close to it.
+                    if( closer_to_first( prevPt, graphic->GetStart(), graphic->GetEnd()) )
+                    {
+                        nextPt = graphic->GetEnd();
+                    }
+                    else
+                    {
+                        nextPt = graphic->GetStart();
+                        reverse = true;
+                    }
+
+                    // Ensure the approximated Bezier shape is built
+                    // a good value is between (Bezier curve width / 2) and (Bezier curve width)
+                    // ( and at least 0.05 mm to avoid very small segments)
+                    int min_segm_length = std::max( pcbIUScale.mmToIU( 0.05 ),
+                                                    graphic->GetWidth() );
+                    graphic->RebuildBezierToSegmentsPointsList( min_segm_length );
+
+                    if( reverse )
+                    {
+                        for( int jj = graphic->GetBezierPoints().size()-1; jj >= 0; jj-- )
                         {
-                            wxASSERT( close_enough( prevPt, graphic->GetEnd(), aChainingEpsilon ) );
+                            const VECTOR2I& pt = graphic->GetBezierPoints()[jj];
 
-                            angle = -angle;
-                            std::swap( pstart, pend );
-                        }
-
-                        // Create intermediate points between start and end:
-                        for( int step = 1; step < steps; ++step )
-                        {
-                            EDA_ANGLE rotation = ( angle * step ) / steps;
-                            VECTOR2I  pt = pstart;
-
-                            RotatePoint( pt, pcenter, rotation );
+                            if( prevPt == pt )
+                                continue;
 
                             currContour.Append( pt );
                             shapeOwners[ std::make_pair( prevPt, pt ) ] = graphic;
                             prevPt = pt;
                         }
-
-                        // Append the last arc end point
-                        currContour.Append( pend );
-                        shapeOwners[ std::make_pair( prevPt, pend ) ] = graphic;
-                        prevPt = pend;
                     }
-                    break;
-
-                case SHAPE_T::BEZIER:
-                    // We do not support Bezier curves in polygons, so approximate with a series
-                    // of short lines and put those line segments into the !same! PATH.
+                    else
                     {
-                        VECTOR2I nextPt;
-                        bool    reverse = false;
-
-                        // Use the end point furthest away from  prevPt as we assume the other
-                        // end to be ON prevPt or very close to it.
-
-                        if( closer_to_first( prevPt, graphic->GetStart(), graphic->GetEnd()) )
+                        for( const VECTOR2I& pt : graphic->GetBezierPoints() )
                         {
-                            nextPt = graphic->GetEnd();
+                            if( prevPt == pt )
+                                continue;
+
+                            currContour.Append( pt );
+                            shapeOwners[ std::make_pair( prevPt, pt ) ] = graphic;
+                            prevPt = pt;
                         }
-                        else
-                        {
-                            nextPt = graphic->GetStart();
-                            reverse = true;
-                        }
-
-                        // Ensure the approximated Bezier shape is built
-                        // a good value is between (Bezier curve width / 2) and (Bezier curve width)
-                        // ( and at least 0.05 mm to avoid very small segments)
-                        int min_segm_lenght = std::max( pcbIUScale.mmToIU( 0.05 ), graphic->GetWidth() );
-                        graphic->RebuildBezierToSegmentsPointsList( min_segm_lenght );
-
-                        if( reverse )
-                        {
-                            for( int jj = graphic->GetBezierPoints().size()-1; jj >= 0; jj-- )
-                            {
-                                const VECTOR2I& pt = graphic->GetBezierPoints()[jj];
-
-                                if( prevPt == pt )
-                                    continue;
-
-                                currContour.Append( pt );
-                                shapeOwners[ std::make_pair( prevPt, pt ) ] = graphic;
-                                prevPt = pt;
-                            }
-                        }
-                        else
-                        {
-                            for( const VECTOR2I& pt : graphic->GetBezierPoints() )
-                            {
-                                if( prevPt == pt )
-                                    continue;
-
-                                currContour.Append( pt );
-                                shapeOwners[ std::make_pair( prevPt, pt ) ] = graphic;
-                                prevPt = pt;
-                            }
-                        }
-
-                        prevPt = nextPt;
                     }
-                    break;
+
+                    prevPt = nextPt;
+                }
+                break;
 
                 default:
                     UNIMPLEMENTED_FOR( graphic->SHAPE_T_asString() );
@@ -443,7 +436,6 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                 }
 
                 // Get next closest segment.
-
                 PCB_SHAPE* nextGraphic = findNext( graphic, prevPt, aShapeList, aChainingEpsilon );
 
                 if( nextGraphic && !( nextGraphic->GetFlags() & SKIP_STRUCT ) )
@@ -451,21 +443,63 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                     prevGraphic = graphic;
                     graphic = nextGraphic;
                     graphic->SetFlags( SKIP_STRUCT );
+                    aCleaner.insert( graphic );
                     startCandidates.erase( graphic );
                     continue;
                 }
 
                 // Finished, or ran into trouble...
-
                 if( close_enough( startPt, prevPt, aChainingEpsilon ) )
                 {
+                    if( startPt != prevPt && currContour.PointCount() > 2 )
+                    {
+                        // Snap the last shape's endpoint to the outline startpoint
+                        PCB_SHAPE* owner = fetchOwner( currContour.CSegment( -1 ) );
+
+                        if( currContour.IsArcEnd( currContour.PointCount() - 1 ) )
+                        {
+                            SHAPE_ARC arc = currContour.Arc(
+                                    currContour.ArcIndex( currContour.PointCount() - 1 ) );
+
+                            // Snap the arc endpoint
+                            SHAPE_ARC sarc( arc.GetP0(), arc.GetArcMid(), startPt, 0 );
+
+                            SHAPE_LINE_CHAIN arcChain;
+                            arcChain.Append( sarc, aErrorMax );
+
+                            if( !aAllowUseArcsInPolygons )
+                                arcChain.ClearArcs();
+
+                            // Set shapeOwners for arcChain points created by appending the sarc:
+                            for( int ii = 1; ii < arcChain.PointCount(); ++ii )
+                            {
+                                shapeOwners[std::make_pair( arcChain.CPoint( ii - 1 ),
+                                                            arcChain.CPoint( ii ) )] = owner;
+                            }
+
+                            currContour.RemoveShape( currContour.PointCount() - 1 );
+                            currContour.Append( arcChain );
+                        }
+                        else
+                        {
+                            // Snap the segment endpoint
+                            currContour.SetPoint( -1, startPt );
+
+                            shapeOwners[std::make_pair( currContour.CPoint( -2 ),
+                                                        currContour.CPoint( -1 ) )] = owner;
+                        }
+
+                        prevPt = startPt;
+                    }
+
                     currContour.SetClosed( true );
                     break;
                 }
                 else if( nextGraphic )  // encountered already-used segment, but not at the start
                 {
                     if( aErrorHandler )
-                        (*aErrorHandler)( _( "(self-intersecting)" ), graphic, nextGraphic, prevPt );
+                        (*aErrorHandler)( _( "(self-intersecting)" ), graphic, nextGraphic,
+                                          prevPt );
 
                     break;
                 }
@@ -487,7 +521,6 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
     }
 
     // First, collect the parents of each contour
-    //
     std::map<int, std::vector<int>> contourToParentIndexesMap;
 
     for( size_t ii = 0; ii < contours.size(); ++ii )
@@ -506,11 +539,10 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                 parents.push_back( jj );
         }
 
-        contourToParentIndexesMap[ii] = parents;
+        contourToParentIndexesMap[ii] = std::move( parents );
     }
 
     // Next add those that are top-level outlines to the SHAPE_POLY_SET
-    //
     std::map<int, int> contourToOutlineIdxMap;
 
     for( const auto& [ contourIndex, parentIndexes ] : contourToParentIndexesMap )
@@ -529,10 +561,10 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                     {
                         (*aErrorHandler)( _( "(multiple board outlines not supported)" ), a, b,
                                           contours[ contourIndex ].GetPoint( 0 ) );
+
+                        return false;
                     }
                 }
-
-                return false;
             }
 
             aPolygons.AddOutline( contours[ contourIndex ] );
@@ -541,7 +573,6 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
     }
 
     // And finally add the holes
-    //
     for( const auto& [ contourIndex, parentIndexes ] : contourToParentIndexesMap )
     {
         if( parentIndexes.size() %2 == 1 )
@@ -565,7 +596,6 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
     // All of the silliness that follows is to work around the segment iterator while checking
     // for collisions.
     // TODO: Implement proper segment and point iterators that follow std
-
     for( auto seg1 = aPolygons.IterateSegmentsWithHoles(); seg1; seg1++ )
     {
         auto seg2 = seg1;
@@ -579,9 +609,7 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                 {
                     BOARD_ITEM* a = fetchOwner( *seg1 );
                     BOARD_ITEM* b = fetchOwner( *seg2 );
-
-                    if( a && b )
-                        (*aErrorHandler)( _( "(self-intersecting)" ), a, b, ( *seg1 ).A );
+                    (*aErrorHandler)( _( "(self-intersecting)" ), a, b, ( *seg1 ).A );
                 }
 
                 selfIntersecting = true;
@@ -593,9 +621,7 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
                 {
                     BOARD_ITEM* a = fetchOwner( *seg1 );
                     BOARD_ITEM* b = fetchOwner( *seg2 );
-
-                    if( a && b )
-                        (*aErrorHandler)( _( "(self-intersecting)" ), a, b, *pt );
+                    (*aErrorHandler)( _( "(self-intersecting)" ), a, b, *pt );
                 }
 
                 selfIntersecting = true;
@@ -607,23 +633,164 @@ bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SE
 }
 
 
-#include <board.h>
-#include <collectors.h>
-
-/* This function is used to extract a board outlines (3D view, automatic zones build ...)
- * Any closed outline inside the main outline is a hole
- * All contours should be closed, i.e. valid closed polygon vertices
- */
-bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aErrorMax,
-                                int aChainingEpsilon, OUTLINE_ERROR_HANDLER* aErrorHandler )
+bool ConvertOutlineToPolygon( std::vector<PCB_SHAPE*>& aShapeList, SHAPE_POLY_SET& aPolygons,
+                              int aErrorMax, int aChainingEpsilon, bool aAllowDisjoint,
+                              OUTLINE_ERROR_HANDLER* aErrorHandler, bool aAllowUseArcsInPolygons )
 {
+    SCOPED_FLAGS_CLEANER cleaner( SKIP_STRUCT );
+
+    return doConvertOutlineToPolygon( aShapeList, aPolygons, aErrorMax, aChainingEpsilon,
+                                      aAllowDisjoint, aErrorHandler, aAllowUseArcsInPolygons,
+                                      cleaner );
+}
+
+
+bool TestBoardOutlinesGraphicItems( BOARD* aBoard, int aMinDist,
+                                    OUTLINE_ERROR_HANDLER* aErrorHandler )
+{
+    bool                success = true;
     PCB_TYPE_COLLECTOR  items;
-    bool                success = false;
+    int                 min_dist = std::max( 0, aMinDist );
 
-    SHAPE_POLY_SET      fpHoles;
+    // Get all the shapes into 'items', then keep only those on layer == Edge_Cuts.
+    items.Collect( aBoard, { PCB_SHAPE_T } );
 
-    // Get all the PCB and FP shapes into 'items', then keep only those on layer == Edge_Cuts.
-    items.Collect( aBoard, { PCB_SHAPE_T, PCB_FP_SHAPE_T } );
+    std::vector<PCB_SHAPE*> segList;
+
+    for( int ii = 0; ii < items.GetCount(); ii++ )
+    {
+        PCB_SHAPE* seg = static_cast<PCB_SHAPE*>( items[ii] );
+
+        if( seg->GetLayer() == Edge_Cuts )
+            segList.push_back( seg );
+    }
+
+    for( FOOTPRINT* fp : aBoard->Footprints() )
+    {
+        PCB_TYPE_COLLECTOR fpItems;
+        fpItems.Collect( fp, { PCB_SHAPE_T } );
+
+        for( int ii = 0; ii < fpItems.GetCount(); ii++ )
+        {
+            PCB_SHAPE* fpSeg = static_cast<PCB_SHAPE*>( fpItems[ii] );
+
+            if( fpSeg->GetLayer() == Edge_Cuts )
+                segList.push_back( fpSeg );
+        }
+    }
+
+    // Now Test validity of collected items
+    for( PCB_SHAPE* graphic : segList )
+    {
+        switch( graphic->GetShape() )
+        {
+        case SHAPE_T::RECTANGLE:
+        {
+            VECTOR2I seg = graphic->GetEnd() - graphic->GetStart();
+            int dim = seg.EuclideanNorm();
+
+            if( dim <= min_dist )
+            {
+                success = false;
+
+                if( aErrorHandler )
+                {
+                    (*aErrorHandler)( wxString::Format( _( "(rectangle has null or very small "
+                                                           "size: %d nm)" ), dim ),
+                                      graphic, nullptr, graphic->GetStart() );
+                }
+            }
+            break;
+        }
+
+        case SHAPE_T::CIRCLE:
+        {
+            int r = graphic->GetRadius();
+
+            if( r <= min_dist )
+            {
+                success = false;
+
+                if( aErrorHandler )
+                {
+                    (*aErrorHandler)( wxString::Format( _( "(circle has null or very small "
+                                                           "radius: %d nm)" ), r ),
+                                      graphic, nullptr, graphic->GetStart() );
+                }
+            }
+            break;
+        }
+
+        case SHAPE_T::SEGMENT:
+        {
+            VECTOR2I seg = graphic->GetEnd() - graphic->GetStart();
+            int dim = seg.EuclideanNorm();
+
+            if( dim <= min_dist )
+            {
+                success = false;
+
+                if( aErrorHandler )
+                {
+                    (*aErrorHandler)( wxString::Format( _( "(segment has null or very small "
+                                                           "length: %d nm)" ), dim ),
+                                      graphic, nullptr, graphic->GetStart() );
+                }
+            }
+            break;
+            }
+
+        case SHAPE_T::ARC:
+        {
+            // Arc size can be evaluated from the distance between arc middle point and arc ends
+            // We do not need a precise value, just an idea of its size
+            VECTOR2I arcMiddle = graphic->GetArcMid();
+            VECTOR2I seg1 = arcMiddle - graphic->GetStart();
+            VECTOR2I seg2 = graphic->GetEnd() - arcMiddle;
+            int dim = seg1.EuclideanNorm() + seg2.EuclideanNorm();
+
+            if( dim <= min_dist )
+            {
+                success = false;
+
+                if( aErrorHandler )
+                {
+                    (*aErrorHandler)( wxString::Format( _( "(arc has null or very small size: "
+                                                           "%d nm)" ), dim ),
+                                      graphic, nullptr, graphic->GetStart() );
+                }
+            }
+            break;
+            }
+
+        case SHAPE_T::POLY:
+            break;
+
+        case SHAPE_T::BEZIER:
+            break;
+
+        default:
+            UNIMPLEMENTED_FOR( graphic->SHAPE_T_asString() );
+            return false;
+        }
+    }
+
+    return success;
+}
+
+
+bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aErrorMax,
+                                int aChainingEpsilon, OUTLINE_ERROR_HANDLER* aErrorHandler,
+                                bool aAllowUseArcsInPolygons )
+{
+    PCB_TYPE_COLLECTOR items;
+    SHAPE_POLY_SET     fpHoles;
+    bool               success = false;
+
+    SCOPED_FLAGS_CLEANER cleaner( SKIP_STRUCT );
+
+    // Get all the shapes into 'items', then keep only those on layer == Edge_Cuts.
+    items.Collect( aBoard, { PCB_SHAPE_T } );
 
     for( int ii = 0; ii < items.GetCount(); ++ii )
         items[ii]->ClearFlags( SKIP_STRUCT );
@@ -631,7 +798,7 @@ bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aE
     for( FOOTPRINT* fp : aBoard->Footprints() )
     {
         PCB_TYPE_COLLECTOR fpItems;
-        fpItems.Collect( fp, { PCB_SHAPE_T, PCB_FP_SHAPE_T } );
+        fpItems.Collect( fp, { PCB_SHAPE_T } );
 
         std::vector<PCB_SHAPE*> fpSegList;
 
@@ -646,13 +813,18 @@ bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aE
         if( !fpSegList.empty() )
         {
             SHAPE_POLY_SET fpOutlines;
-            success = ConvertOutlineToPolygon( fpSegList, fpOutlines, aErrorMax, aChainingEpsilon,
-                                               false,
-                                               // don't report errors here; the second pass also
-                                               // gets an opportunity to use these segments
-                                               nullptr );
+            success = doConvertOutlineToPolygon( fpSegList, fpOutlines, aErrorMax, aChainingEpsilon,
+                                                 false,
+                                                 // don't report errors here; the second pass also
+                                                 // gets an opportunity to use these segments
+                                                 nullptr, aAllowUseArcsInPolygons, cleaner );
 
-            if( success && isCopperOutside( fp, fpOutlines ) )
+            // Test to see if we should make holes or outlines.  Holes are made if the footprint
+            // has copper outside of a single, closed outline.  If there are multiple outlines,
+            // we assume that the footprint edges represent holes as we do not support multiple
+            // boards.  Similarly, if any of the footprint pads are located outside of the edges,
+            // then the edges are holes
+            if( success && ( isCopperOutside( fp, fpOutlines ) || fpOutlines.OutlineCount() > 1 ) )
             {
                 fpHoles.Append( fpOutlines );
             }
@@ -683,8 +855,8 @@ bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aE
 
     if( segList.size() )
     {
-        success = ConvertOutlineToPolygon( segList, aOutlines, aErrorMax, aChainingEpsilon,
-                                           true, aErrorHandler );
+        success = doConvertOutlineToPolygon( segList, aOutlines, aErrorMax, aChainingEpsilon, true,
+                                             aErrorHandler, aAllowUseArcsInPolygons, cleaner );
     }
 
     if( !success || !aOutlines.OutlineCount() )
@@ -692,7 +864,6 @@ bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aE
         // Couldn't create a valid polygon outline.  Use the board edge cuts bounding box to
         // create a rectangular outline, or, failing that, the bounding box of the items on
         // the board.
-
         BOX2I bbbox = aBoard->GetBoardEdgesBoundingBox();
 
         // If null area, uses the global bounding box.
@@ -706,7 +877,7 @@ bool BuildBoardPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aE
         aOutlines.RemoveAllContours();
         aOutlines.NewOutline();
 
-        wxPoint corner;
+        VECTOR2I corner;
         aOutlines.Append( bbbox.GetOrigin() );
 
         corner.x = bbbox.GetOrigin().x;
@@ -852,17 +1023,6 @@ int findEndSegments( SHAPE_LINE_CHAIN& aChain, SEG& aStartSeg, SEG& aEndSeg )
 }
 
 
-/**
- * This function is used to extract a board outline for a footprint view.
- *
- * Notes:
- * * Incomplete outlines will be closed by joining the end of the outline onto the bounding box
- *   (by simply projecting the end points) and then take the area that contains the copper.
- * * If all copper lies inside a closed outline, than that outline will be treated as an external
- *   board outline.
- * * If copper is located outside a closed outline, then that outline will be treated as a hole,
- *   and the outer edge will be formed using the bounding box.
- */
 bool BuildFootprintPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, int aErrorMax,
                                     int aChainingEpsilon, OUTLINE_ERROR_HANDLER* aErrorHandler )
 
@@ -876,12 +1036,14 @@ bool BuildFootprintPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, in
         return false;
     }
 
-    PCB_TYPE_COLLECTOR  items;
-    SHAPE_POLY_SET      outlines;
-    bool                success = false;
+    PCB_TYPE_COLLECTOR items;
+    SHAPE_POLY_SET     outlines;
+    bool               success = false;
+
+    SCOPED_FLAGS_CLEANER cleaner( SKIP_STRUCT );
 
     // Get all the SHAPEs into 'items', then keep only those on layer == Edge_Cuts.
-    items.Collect( aBoard, { PCB_SHAPE_T, PCB_FP_SHAPE_T } );
+    items.Collect( aBoard, { PCB_SHAPE_T } );
 
     // Make a working copy of aSegList, because the list is modified during calculations
     std::vector<PCB_SHAPE*> segList;
@@ -894,8 +1056,8 @@ bool BuildFootprintPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, in
 
     if( !segList.empty() )
     {
-        success = ConvertOutlineToPolygon( segList, outlines, aErrorMax, aChainingEpsilon,
-                                           true, aErrorHandler );
+        success = doConvertOutlineToPolygon( segList, outlines, aErrorMax, aChainingEpsilon, true,
+                                             aErrorHandler, false, cleaner );
     }
 
     // A closed outline was found on Edge_Cuts
@@ -904,7 +1066,8 @@ bool BuildFootprintPolygonOutlines( BOARD* aBoard, SHAPE_POLY_SET& aOutlines, in
         wxLogTrace( traceBoardOutline, wxT( "Closed outline found" ) );
 
         // If copper is outside a closed polygon, treat it as a hole
-        if( isCopperOutside( footprint, outlines ) )
+        // If there are multiple outlines in the footprint, they are also holes
+        if( isCopperOutside( footprint, outlines ) || outlines.OutlineCount() > 1 )
         {
             wxLogTrace( traceBoardOutline, wxT( "Treating outline as a hole" ) );
 
