@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Brian Piccioni brian@documenteddesigns.com
- * Copyright (C) 2020-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Brian Piccioni <brian@documenteddesigns.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@
 #include <confirm.h>
 #include <ctype.h>
 #include <dialog_board_reannotate.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <string_utils.h>  // StrNumCmp
 #include <kiface_base.h>
 #include <pcbnew_settings.h>
@@ -82,18 +83,18 @@ int BackDirectionsArray[] = {
 
 
 wxString AnnotateString[] = {
-    _( "All" ),          // AnnotateAll
+    _( "All" ),          // ANNOTATE_ALL
     _( "Only front" ),   // AnnotateFront
     _( "Only back" ),    // AnnotateBack
-    _( "Only selected" ) // AnnotateSelected
+    _( "Only selected" ) // ANNOTATE_SELECTED
 };
 
 
 wxString ActionMessage[] = {
-    "",             // UpdateRefDes
-    _( "Empty" ),   // EmptyRefDes
-    _( "Invalid" ), // InvalidRefDes
-    _( "Excluded" ) // Exclude
+    "",             // UPDATE_REFDES
+    _( "Empty" ),   // EMPTY_REFDES
+    _( "Invalid" ), // INVALID_REFDES
+    _( "Excluded" ) // EXCLUDE_REFDES
 };
 
 
@@ -105,6 +106,16 @@ DIALOG_BOARD_REANNOTATE::DIALOG_BOARD_REANNOTATE( PCB_EDIT_FRAME* aParentFrame )
     m_Config = Kiface().KifaceSettings();
     InitValues();
 
+    // Init bitmaps associated to some wxRadioButton
+    reannotate_down_right_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_right_down ) );
+    reannotate_right_down_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_left_down ) );
+    reannotate_down_left_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_right_up ) );
+    reannotate_left_down_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_left_up ) );
+    reannotate_up_right_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_down_left ) );
+    reannotate_right_up_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_up_left ) );
+    reannotate_up_left_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_down_right ) );
+    reannotate_left_up_bitmap->SetBitmap( KiBitmapBundle( BITMAPS::reannotate_up_right ) );
+
     m_FrontRefDesStart->SetValidator( wxTextValidator( wxFILTER_DIGITS ) );
     m_BackRefDesStart->SetValidator( wxTextValidator( wxFILTER_DIGITS ) );
 
@@ -125,29 +136,27 @@ DIALOG_BOARD_REANNOTATE::DIALOG_BOARD_REANNOTATE( PCB_EDIT_FRAME* aParentFrame )
     m_GridChoice->Set( gridslist );
     m_GridChoice->SetSelection( m_gridIndex );
 
+    // Ensure m_sortCode is a valid value (0 .. m_sortButtons.size()-1)
+    m_sortCode = std::max( 0, m_sortCode );
+    m_sortCode = std::min( m_sortCode, (int)m_sortButtons.size()-1 );
+
     for( wxRadioButton* button : m_sortButtons )
         button->SetValue( false );
 
-    m_sortButtons[m_sortCode]->SetValue( true );
-
     m_selection = m_frame->GetToolManager()->GetTool<PCB_SELECTION_TOOL>()->GetSelection();
 
     if( !m_selection.Empty() )
-        m_annotationScope = AnnotateSelected;
+        m_annotationScope = ANNOTATE_SELECTED;
+
+    // Ensure m_annotationScope is a valid value (0 .. m_scopeRadioButtons.size()-1)
+    m_annotationScope = std::max( 0, m_annotationScope );
+    m_annotationScope = std::min( m_annotationScope, (int)m_scopeRadioButtons.size()-1 );
 
     for( wxRadioButton* button : m_scopeRadioButtons )
         button->SetValue( false );
 
-    m_scopeRadioButtons[m_annotationScope]->SetValue( true );
-
-    reannotate_down_right_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_right_down ) );
-    reannotate_right_down_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_left_down ) );
-    reannotate_down_left_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_right_up ) );
-    reannotate_left_down_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_left_up ) );
-    reannotate_up_right_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_down_left ) );
-    reannotate_right_up_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_up_left ) );
-    reannotate_up_left_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_down_right ) );
-    reannotate_left_up_bitmap->SetBitmap( KiBitmap( BITMAPS::reannotate_up_right ) );
+    m_scopeRadioButtons.at( m_annotationScope )->SetValue( true );
+    m_sortButtons.at( m_sortCode )->SetValue( true );
 
     m_ExcludeList->SetToolTip( m_ExcludeListText->GetToolTipText() );
     m_GridChoice->SetToolTip( m_SortGridText->GetToolTipText() );
@@ -161,23 +170,37 @@ DIALOG_BOARD_REANNOTATE::DIALOG_BOARD_REANNOTATE( PCB_EDIT_FRAME* aParentFrame )
 DIALOG_BOARD_REANNOTATE::~DIALOG_BOARD_REANNOTATE()
 {
     GetParameters(); // Get the current menu settings
-    PCBNEW_SETTINGS* cfg = m_frame->GetPcbNewSettings();
-    cfg->m_Reannotate.sort_on_fp_location = m_locationChoice->GetSelection() == 0;
-    cfg->m_Reannotate.remove_front_prefix     = m_RemoveFrontPrefix->GetValue();
-    cfg->m_Reannotate.remove_back_prefix      = m_RemoveBackPrefix->GetValue();
-    cfg->m_Reannotate.exclude_locked          = m_ExcludeLocked->GetValue();
 
-    cfg->m_Reannotate.grid_index              = m_gridIndex;
-    cfg->m_Reannotate.sort_code               = m_sortCode;
-    cfg->m_Reannotate.annotation_choice       = m_annotationScope;
-    cfg->m_Reannotate.report_severity         = m_severity;
+    PCBNEW_SETTINGS* cfg = nullptr;
 
-    cfg->m_Reannotate.front_refdes_start      = m_FrontRefDesStart->GetValue();
-    cfg->m_Reannotate.back_refdes_start       = m_BackRefDesStart->GetValue();
-    cfg->m_Reannotate.front_prefix            = m_FrontPrefix->GetValue();
-    cfg->m_Reannotate.back_prefix             = m_BackPrefix->GetValue();
-    cfg->m_Reannotate.exclude_list            = m_ExcludeList->GetValue();
-    cfg->m_Reannotate.report_file_name        = m_MessageWindow->GetFileName();
+    try
+    {
+        cfg = m_frame->GetPcbNewSettings();
+    }
+    catch( const std::runtime_error& e )
+    {
+        wxFAIL_MSG( e.what() );
+    }
+
+    if( cfg )
+    {
+        cfg->m_Reannotate.sort_on_fp_location = m_locationChoice->GetSelection() == 0;
+        cfg->m_Reannotate.remove_front_prefix     = m_RemoveFrontPrefix->GetValue();
+        cfg->m_Reannotate.remove_back_prefix      = m_RemoveBackPrefix->GetValue();
+        cfg->m_Reannotate.exclude_locked          = m_ExcludeLocked->GetValue();
+
+        cfg->m_Reannotate.grid_index              = m_gridIndex;
+        cfg->m_Reannotate.sort_code               = m_sortCode;
+        cfg->m_Reannotate.annotation_choice       = m_annotationScope;
+        cfg->m_Reannotate.report_severity         = m_severity;
+
+        cfg->m_Reannotate.front_refdes_start      = m_FrontRefDesStart->GetValue();
+        cfg->m_Reannotate.back_refdes_start       = m_BackRefDesStart->GetValue();
+        cfg->m_Reannotate.front_prefix            = m_FrontPrefix->GetValue();
+        cfg->m_Reannotate.back_prefix             = m_BackPrefix->GetValue();
+        cfg->m_Reannotate.exclude_list            = m_ExcludeList->GetValue();
+        cfg->m_Reannotate.report_file_name        = m_MessageWindow->GetFileName();
+    }
 }
 
 
@@ -221,7 +244,7 @@ void DIALOG_BOARD_REANNOTATE::FilterPrefix( wxTextCtrl* aPrefix )
     if( isalnum( (int) lastc ) )
         return;
 
-    if( std::string::npos != tmps.find( lastc ) )
+    if( tmps.find( lastc ) != std::string::npos )
         return;
 
     tmps = aPrefix->GetValue();
@@ -231,7 +254,7 @@ void DIALOG_BOARD_REANNOTATE::FilterPrefix( wxTextCtrl* aPrefix )
 }
 
 
-RefDesTypeStr* DIALOG_BOARD_REANNOTATE::GetOrBuildRefDesInfo( const wxString& aRefDesPrefix,
+REFDES_TYPE_STR* DIALOG_BOARD_REANNOTATE::GetOrBuildRefDesInfo( const wxString& aRefDesPrefix,
                                                               unsigned int    aStartRefDes )
 {
     unsigned int requiredLastRef = ( aStartRefDes == 0 ? 1 : aStartRefDes ) - 1;
@@ -248,7 +271,7 @@ RefDesTypeStr* DIALOG_BOARD_REANNOTATE::GetOrBuildRefDesInfo( const wxString& aR
     }
 
     // Wasn't in the types array so add it
-    RefDesTypeStr newtype;
+    REFDES_TYPE_STR newtype;
     newtype.RefDesType = aRefDesPrefix;
     newtype.LastUsedRefDes = requiredLastRef;
     m_refDesTypes.push_back( newtype );
@@ -395,21 +418,12 @@ void DIALOG_BOARD_REANNOTATE::GetParameters()
     // Get the chosen sort grid for rounding
     m_gridIndex = m_GridChoice->GetSelection();
 
-    if( m_gridIndex >= ( int ) m_settings->m_Window.grid.sizes.size() )
-    {
-        m_sortGridx = EDA_UNIT_UTILS::UI::DoubleValueFromString(
-                pcbIUScale, EDA_UNITS::MILS, m_settings->m_Window.grid.user_grid_x );
-        m_sortGridy = EDA_UNIT_UTILS::UI::DoubleValueFromString(
-                pcbIUScale, EDA_UNITS::MILS, m_settings->m_Window.grid.user_grid_y );
-    }
-    else
-    {
-        m_sortGridx = EDA_UNIT_UTILS::UI::DoubleValueFromString(
-                pcbIUScale, EDA_UNITS::MILS, m_settings->m_Window.grid.sizes[m_gridIndex] );
-        m_sortGridy = m_sortGridx;
-    }
+    m_sortGridx = EDA_UNIT_UTILS::UI::DoubleValueFromString(
+            pcbIUScale, EDA_UNITS::MILS, m_settings->m_Window.grid.grids[m_gridIndex].x );
+    m_sortGridy = EDA_UNIT_UTILS::UI::DoubleValueFromString(
+            pcbIUScale, EDA_UNITS::MILS, m_settings->m_Window.grid.grids[m_gridIndex].y );
 
-    m_annotationScope = AnnotateAll;
+    m_annotationScope = ANNOTATE_ALL;
 
     for( wxRadioButton* button : m_scopeRadioButtons )
     {
@@ -419,6 +433,10 @@ void DIALOG_BOARD_REANNOTATE::GetParameters()
             m_annotationScope++;
     }
 
+    // Ensure m_annotationScope value is valid
+    if( m_annotationScope >= (int)m_scopeRadioButtons.size() )
+        m_annotationScope = ANNOTATE_ALL;
+
     m_MessageWindow->SetLazyUpdate( true );
 }
 
@@ -441,7 +459,7 @@ int DIALOG_BOARD_REANNOTATE::RoundToGrid( int aCoord, int aGrid )
 
 /// Compare function used to compare ChangeArray element for sort
 /// @return true is A < B
-static bool ChangeArrayCompare( const RefDesChange& aA, const RefDesChange& aB )
+static bool ChangeArrayCompare( const REFDES_CHANGE& aA, const REFDES_CHANGE& aB )
 {
     return ( StrNumCmp( aA.OldRefDesString, aB.OldRefDesString ) < 0 );
 }
@@ -449,7 +467,7 @@ static bool ChangeArrayCompare( const RefDesChange& aA, const RefDesChange& aB )
 
 /// Compare function to sort footprints.
 /// @return true if the first coordinate should be before the second coordinate
-static bool ModuleCompare( const RefDesInfo& aA, const RefDesInfo& aB )
+static bool ModuleCompare( const REFDES_INFO& aA, const REFDES_INFO& aB )
 {
     int X0 = aA.roundedx, X1 = aB.roundedx, Y0 = aA.roundedy, Y1 = aB.roundedy;
 
@@ -489,15 +507,12 @@ wxString DIALOG_BOARD_REANNOTATE::CoordTowxString( int aX, int aY )
 
 void DIALOG_BOARD_REANNOTATE::ShowReport( const wxString& aMessage, SEVERITY aSeverity )
 {
-    size_t pos = 0, prev = 0;
+    wxStringTokenizer msgs( aMessage, wxT( "\n" ) );
 
-    do
+    while( msgs.HasMoreTokens() )
     {
-        pos = aMessage.ToStdString().find( '\n', prev );
-        m_MessageWindow->Report( aMessage.ToStdString().substr( prev, pos - prev ), aSeverity );
-        prev = pos + 1;
-    } while( std::string::npos != pos );
-
+        m_MessageWindow->Report( msgs.GetNextToken(), aSeverity );
+    }
 }
 
 
@@ -510,7 +525,7 @@ void DIALOG_BOARD_REANNOTATE::LogChangePlan()
                        "**********************************************************\n" ),
                     (int) m_refDesTypes.size() );
 
-    for( RefDesTypeStr Type : m_refDesTypes ) // Show all the types of refdes
+    for( REFDES_TYPE_STR Type : m_refDesTypes ) // Show all the types of refdes
         message += Type.RefDesType + ( 0 == ( i++ % 16 ) ? wxT( "\n" ) : wxS( " " ) );
 
     if( !m_excludeArray.empty() )
@@ -525,14 +540,14 @@ void DIALOG_BOARD_REANNOTATE::LogChangePlan()
 
     message += _( "\n    Change Array\n***********************\n" );
 
-    for( const RefDesChange& change : m_changeArray )
+    for( const REFDES_CHANGE& change : m_changeArray )
     {
         message += wxString::Format( wxT( "%s -> %s  %s %s\n" ),
                                      change.OldRefDesString,
                                      change.NewRefDes,
                                      ActionMessage[change.Action],
-                                     UpdateRefDes != change.Action ? _( " will be ignored" )
-                                                                   : wxT( "" ) );
+                                     UPDATE_REFDES != change.Action ? _( " will be ignored" )
+                                                                   : wxString( wxT( "" ) ) );
     }
 
     ShowReport( message, RPT_SEVERITY_INFO );
@@ -540,7 +555,7 @@ void DIALOG_BOARD_REANNOTATE::LogChangePlan()
 
 
 void DIALOG_BOARD_REANNOTATE::LogFootprints( const wxString& aMessage,
-                                             const std::vector<RefDesInfo>& aFootprints )
+                                             const std::vector<REFDES_INFO>& aFootprints )
 {
     wxString message = aMessage;
 
@@ -557,7 +572,7 @@ void DIALOG_BOARD_REANNOTATE::LogFootprints( const wxString& aMessage,
 
         message += wxString::Format( _( "\nSort Code %d" ), m_sortCode );
 
-        for( const RefDesInfo& mod : aFootprints )
+        for( const REFDES_INFO& mod : aFootprints )
         {
             message += wxString::Format( _( "\n%d %s UUID: [%s], X, Y: %s, Rounded X, Y, %s" ),
                                          i++,
@@ -574,10 +589,10 @@ void DIALOG_BOARD_REANNOTATE::LogFootprints( const wxString& aMessage,
 
 bool DIALOG_BOARD_REANNOTATE::ReannotateBoard()
 {
-    std::vector<RefDesInfo> BadRefDes;
+    std::vector<REFDES_INFO> BadRefDes;
     wxString                message, badrefdes;
     STRING_FORMATTER        stringformatter;
-    RefDesChange*           newref;
+    REFDES_CHANGE*           newref;
     NETLIST                 netlist;
 
     if( !BuildFootprintList( BadRefDes ) )
@@ -594,7 +609,7 @@ bool DIALOG_BOARD_REANNOTATE::ReannotateBoard()
                            "schematic' checked.\n" ),
                         (int) BadRefDes.size() );
 
-        for( const RefDesInfo& mod : BadRefDes )
+        for( const REFDES_INFO& mod : BadRefDes )
         {
             badrefdes += wxString::Format( _( "\nRefDes: %s Footprint: %s:%s at %s on PCB." ),
                                            mod.RefDesString,
@@ -629,7 +644,7 @@ bool DIALOG_BOARD_REANNOTATE::ReannotateBoard()
 }
 
 
-bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadRefDes )
+bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<REFDES_INFO>& aBadRefDes )
 {
     bool annotateSelected = m_AnnotateSelection->GetValue();
     bool annotateFront = m_AnnotateFront->GetValue(); // Unless only doing back
@@ -668,12 +683,12 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
         }
         else
             exclude += thischar;
-
-        if( !exclude.empty() )
-            m_excludeArray.push_back( exclude );
     }
 
-    RefDesInfo fpData;
+    if( !exclude.empty() )      // last item to exclude
+        m_excludeArray.push_back( exclude );
+
+    REFDES_INFO fpData;
     bool       useModuleLocation = m_locationChoice->GetSelection() == 0;
 
     for( FOOTPRINT* footprint : m_footprints )
@@ -688,18 +703,18 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
         fpData.roundedx     = RoundToGrid( fpData.x, m_sortGridx ); // Round to sort
         fpData.roundedy     = RoundToGrid( fpData.y, m_sortGridy );
         fpData.Front        = footprint->GetLayer() == F_Cu;
-        fpData.Action       = UpdateRefDes; // Usually good
+        fpData.Action       = UPDATE_REFDES; // Usually good
 
         if( fpData.RefDesString.IsEmpty() )
         {
-            fpData.Action = EmptyRefDes;
+            fpData.Action = EMPTY_REFDES;
         }
         else
         {
             firstnum = fpData.RefDesString.find_first_of( wxT( "0123456789" ) );
 
             if( std::string::npos == firstnum )
-                fpData.Action = InvalidRefDes; // do not change ref des such as 12 or +1, or L
+                fpData.Action = INVALID_REFDES; // do not change ref des such as 12 or +1, or L
         }
 
         // Get the type (R, C, etc)
@@ -709,7 +724,7 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
         {
             if( excluded == fpData.RefDesType ) // Am I supposed to exclude this type?
             {
-                fpData.Action = Exclude; // Yes
+                fpData.Action = EXCLUDE_REFDES; // Yes
                 break;
             }
         }
@@ -718,18 +733,18 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
                 ( !fpData.Front && annotateFront ) ||     // If a back fp and doing front only
                 ( footprint->IsLocked() && skipLocked ) ) // If excluding locked and it is locked
         {
-            fpData.Action = Exclude;
+            fpData.Action = EXCLUDE_REFDES;
         }
 
         if( annotateSelected )
         {                                // If only annotating selected c
-            fpData.Action = Exclude;     // Assume it isn't selected
+            fpData.Action = EXCLUDE_REFDES;     // Assume it isn't selected
 
             for( KIID sel : selected )
             {
                 if( fpData.Uuid == sel )
                 {                                  // Found in selected footprints
-                    fpData.Action = UpdateRefDes;  // Update it
+                    fpData.Action = UPDATE_REFDES;  // Update it
                     break;
                 }
             }
@@ -779,8 +794,8 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
 
     for( size_t i = 0; i < changearraysize; i++ ) // Scan through for duplicates if update or skip
     {
-        if( ( m_changeArray[i].Action != EmptyRefDes )
-                && ( m_changeArray[i].Action != InvalidRefDes ) )
+        if( ( m_changeArray[i].Action != EMPTY_REFDES )
+                && ( m_changeArray[i].Action != INVALID_REFDES ) )
         {
             for( size_t j = i + 1; j < changearraysize; j++ )
             {
@@ -808,35 +823,35 @@ bool DIALOG_BOARD_REANNOTATE::BuildFootprintList( std::vector<RefDesInfo>& aBadR
 
 void DIALOG_BOARD_REANNOTATE::BuildUnavailableRefsList()
 {
-    std::vector<RefDesInfo> excludedFootprints;
+    std::vector<REFDES_INFO> excludedFootprints;
 
-    for( RefDesInfo fpData : m_frontFootprints )
+    for( REFDES_INFO fpData : m_frontFootprints )
     {
-        if( fpData.Action == Exclude )
+        if( fpData.Action == EXCLUDE_REFDES )
             excludedFootprints.push_back( fpData );
     }
 
-    for( RefDesInfo fpData : m_backFootprints )
+    for( REFDES_INFO fpData : m_backFootprints )
     {
-        if( fpData.Action == Exclude )
+        if( fpData.Action == EXCLUDE_REFDES )
             excludedFootprints.push_back( fpData );
     }
 
-    for( RefDesInfo fpData : excludedFootprints )
+    for( REFDES_INFO fpData : excludedFootprints )
     {
-        if( fpData.Action == Exclude )
+        if( fpData.Action == EXCLUDE_REFDES )
         {
-            RefDesTypeStr* refDesInfo = GetOrBuildRefDesInfo( fpData.RefDesType );
+            REFDES_TYPE_STR* refDesInfo = GetOrBuildRefDesInfo( fpData.RefDesType );
             refDesInfo->UnavailableRefs.insert( UTIL::GetRefDesNumber( fpData.RefDesString ) );
         }
     }
 }
 
 
-void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<RefDesInfo>& aFootprints,
+void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<REFDES_INFO>& aFootprints,
                                                 unsigned int aStartRefDes, const wxString& aPrefix,
                                                 bool aRemovePrefix,
-                                                std::vector<RefDesInfo>& aBadRefDes )
+                                                std::vector<REFDES_INFO>& aBadRefDes )
 {
     size_t   prefixsize = aPrefix.size();
 
@@ -857,9 +872,9 @@ void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<RefDesInfo>& aFootpr
     }
 
 
-    for( RefDesInfo fpData : aFootprints )
+    for( REFDES_INFO fpData : aFootprints )
     {
-        RefDesChange change;
+        REFDES_CHANGE change;
 
         change.Uuid            = fpData.Uuid;
         change.Action          = fpData.Action;
@@ -868,18 +883,18 @@ void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<RefDesInfo>& aFootpr
         change.Front           = fpData.Front;
 
         if( fpData.RefDesString.IsEmpty() )
-            fpData.Action = EmptyRefDes;
+            fpData.Action = EMPTY_REFDES;
 
-        if( ( change.Action == EmptyRefDes ) || ( change.Action == InvalidRefDes ) )
+        if( ( change.Action == EMPTY_REFDES ) || ( change.Action == INVALID_REFDES ) )
         {
             m_changeArray.push_back( change );
             aBadRefDes.push_back( fpData );
             continue;
         }
 
-        if( change.Action == UpdateRefDes )
+        if( change.Action == UPDATE_REFDES )
         {
-            prefixpresent = ( 0 == fpData.RefDesType.find( aPrefix ) );
+            prefixpresent = ( fpData.RefDesType.find( aPrefix ) == 0 );
 
             if( addprefix && !prefixpresent )
                 fpData.RefDesType.insert( 0, aPrefix ); // Add prefix once only
@@ -887,7 +902,7 @@ void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<RefDesInfo>& aFootpr
             if( aRemovePrefix && prefixpresent ) // If there is a prefix remove it
                 fpData.RefDesType.erase( 0, prefixsize );
 
-            RefDesTypeStr* refDesInfo = GetOrBuildRefDesInfo( fpData.RefDesType, aStartRefDes );
+            REFDES_TYPE_STR* refDesInfo = GetOrBuildRefDesInfo( fpData.RefDesType, aStartRefDes );
             unsigned int  newRefDesNumber = refDesInfo->LastUsedRefDes + 1;
 
             while( refDesInfo->UnavailableRefs.count( newRefDesNumber ) )
@@ -902,7 +917,7 @@ void DIALOG_BOARD_REANNOTATE::BuildChangeArray( std::vector<RefDesInfo>& aFootpr
 }
 
 
-RefDesChange* DIALOG_BOARD_REANNOTATE::GetNewRefDes( FOOTPRINT* aFootprint )
+REFDES_CHANGE* DIALOG_BOARD_REANNOTATE::GetNewRefDes( FOOTPRINT* aFootprint )
 {
     size_t i;
 
