@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2009-2016 Dick Hollenbeck, dick@softplc.com
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2023, 2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <macros.h>
 #include <pad.h>
 #include <drc/drc_item.h>
+#include <drc/drc_report.h>
 #include <connectivity/connectivity_data.h>
 #include <connectivity/connectivity_algo.h>
 #include <drawing_sheet/ds_proxy_view_item.h>
@@ -39,6 +40,7 @@
 #include <tools/pcb_actions.h>
 #include <wildcards_and_files_ext.h>
 #include <pcb_marker.h>
+#include <pgm_base.h>
 #include <wx/filedlg.h>
 #include <wx/wupdlock.h>
 #include <widgets/appearance_controls.h>
@@ -71,10 +73,11 @@ DIALOG_DRC::DIALOG_DRC( PCB_EDIT_FRAME* aEditorFrame, wxWindow* aParent ) :
         m_markersTreeModel( nullptr ),
         m_unconnectedTreeModel( nullptr ),
         m_fpWarningsTreeModel( nullptr ),
-        m_centerMarkerOnIdle( nullptr ),
-        m_severities( RPT_SEVERITY_ERROR | RPT_SEVERITY_WARNING )
+        m_severities( RPT_SEVERITY_ERROR | RPT_SEVERITY_WARNING ),
+        m_lastUpdateUi( std::chrono::steady_clock::now() )
 {
     SetName( DIALOG_DRC_WINDOW_NAME ); // Set a window name to be able to find it
+    KIPLATFORM::UI::SetFloatLevel( this );
 
     m_frame = aEditorFrame;
     m_currentBoard = m_frame->GetBoard();
@@ -159,14 +162,27 @@ DIALOG_DRC::~DIALOG_DRC()
     for( int ii = 0; ii < m_ignoredList->GetItemCount(); ++ii )
         g_lastIgnored.push_back( m_ignoredList->GetItemText( ii ) );
 
-    PCBNEW_SETTINGS* settings = m_frame->GetPcbNewSettings();
-    settings->m_DrcDialog.refill_zones          = m_cbRefillZones->GetValue();
-    settings->m_DrcDialog.test_all_track_errors = m_cbReportAllTrackErrors->GetValue();
+    PCBNEW_SETTINGS* cfg = nullptr;
 
-    if( !Kiface().IsSingle() )
-        settings->m_DrcDialog.test_footprints   = m_cbTestFootprints->GetValue();
+    try
+    {
+        cfg = m_frame->GetPcbNewSettings();
+    }
+    catch( const std::runtime_error& e )
+    {
+        wxFAIL_MSG( e.what() );
+    }
 
-    settings->m_DrcDialog.severities            = m_severities;
+    if( cfg )
+    {
+        cfg->m_DrcDialog.refill_zones          = m_cbRefillZones->GetValue();
+        cfg->m_DrcDialog.test_all_track_errors = m_cbReportAllTrackErrors->GetValue();
+
+        if( !Kiface().IsSingle() )
+            cfg->m_DrcDialog.test_footprints   = m_cbTestFootprints->GetValue();
+
+        cfg->m_DrcDialog.severities            = m_severities;
+    }
 
     m_markersTreeModel->DecRef();
     m_unconnectedTreeModel->DecRef();
@@ -193,11 +209,22 @@ void DIALOG_DRC::OnActivateDlg( wxActivateEvent& aEvent )
 
 bool DIALOG_DRC::updateUI()
 {
-    double cur = (double) m_progress.load() / m_maxProgress;
-    cur = std::max( 0.0, std::min( cur, 1.0 ) );
+    double cur = alg::clamp( 0.0, (double) m_progress.load() / m_maxProgress, 1.0 );
 
-    m_gauge->SetValue( KiROUND( cur * 1000.0 ) );
-    wxSafeYield( this );
+    int newValue = KiROUND( cur * 1000.0 );
+    m_gauge->SetValue( newValue );
+
+    // There is significant overhead on at least Windows when updateUi is called constantly thousands of times
+    // in the drc process and safeyieldfor is called each time.
+    // Gate the yield to a limited rate which still allows the UI to function without slowing down the main thread
+    // which is also running DRC
+    std::chrono::steady_clock::time_point now = std::chrono::steady_clock::now();
+    if( std::chrono::duration_cast<std::chrono::milliseconds>( now - m_lastUpdateUi ).count()
+        > 100 )
+    {
+        Pgm().App().SafeYieldFor( this, wxEVT_CATEGORY_NATIVE_EVENTS );
+        m_lastUpdateUi = now;
+    }
 
     return !m_cancelled;
 }
@@ -260,20 +287,22 @@ void DIALOG_DRC::OnRunDRCClick( wxCommandEvent& aEvent )
         m_saveReport->Enable( false );
 
         m_messages->Clear();
-        m_messages->Report( _( "DRC incomplete: could not compile custom design rules.  " )
-                            + wxT( "<a href='boardsetup'>" )
-                            + _( "Show design rules." )
-                            + wxT( "</a>" ) );
+        m_messages->Report( _( "DRC incomplete: could not compile custom design rules." )
+                            + wxS( "&nbsp;&nbsp;" )
+                            + wxS( "<a href='$CUSTOM_RULES'>" ) + _( "Show design rules." ) + wxT( "</a>" ) );
         m_messages->Flush();
 
         Raise();
+
+        // set float level again, it can be lost due to window events during test run
+        KIPLATFORM::UI::SetFloatLevel( this );
         return;
     }
 
     m_footprintTestsRun = false;
     m_cancelled = false;
 
-    m_frame->RecordDRCExclusions();
+    m_frame->GetBoard()->RecordDRCExclusions();
     deleteAllMarkers( true );
 
     std::vector<std::reference_wrapper<RC_ITEM>> violations = DRC_ITEM::GetItemsWithSeverities();
@@ -288,6 +317,8 @@ void DIALOG_DRC::OnRunDRCClick( wxCommandEvent& aEvent )
         }
     }
 
+    m_ignoredList->SetColumnWidth( 0, m_ignoredList->GetParent()->GetClientSize().x - 20 );
+
     Raise();
 
     m_runningResultsBook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
@@ -323,13 +354,20 @@ void DIALOG_DRC::OnRunDRCClick( wxCommandEvent& aEvent )
 
     if( !m_cancelled )
     {
+        m_sdbSizerCancel->SetDefault();
+        // wxWidgets has a tendency to keep both buttons highlighted without the following:
+        m_sdbSizerOK->Enable( false );
+
         wxMilliSleep( 500 );
         m_runningResultsBook->ChangeSelection( 1 );
-#ifndef __WXGTK__
-        KIPLATFORM::UI::ForceFocus( m_markerDataView );
-#endif
+        KIPLATFORM::UI::ForceFocus( m_Notebook );
+
+        // now re-enable m_sdbSizerOK button
+        m_sdbSizerOK->Enable( true );
     }
 
+    // set float level again, it can be lost due to window events during test run
+    KIPLATFORM::UI::SetFloatLevel( this );
     refreshEditor();
 }
 
@@ -379,15 +417,6 @@ void DIALOG_DRC::OnDRCItemSelected( wxDataViewEvent& aEvent )
         return;
     }
 
-    if( m_centerMarkerOnIdle )
-    {
-        // we already came from a cross-probe of the marker in the document; don't go
-        // around in circles
-
-        aEvent.Skip();
-        return;
-    }
-
     std::shared_ptr<RC_ITEM> rc_item = node->m_RcItem;
 
     if( rc_item->GetErrorCode() == DRCE_UNRESOLVED_VARIABLE
@@ -472,10 +501,12 @@ void DIALOG_DRC::OnDRCItemSelected( wxDataViewEvent& aEvent )
     if( rc_item->GetErrorCode() == DRCE_UNCONNECTED_ITEMS )
     {
         if( !m_frame->GetPcbNewSettings()->m_Display.m_ShowGlobalRatsnest )
-            m_frame->GetToolManager()->RunAction( PCB_ACTIONS::showRatsnest, true );
+            m_frame->GetToolManager()->RunAction( PCB_ACTIONS::showRatsnest );
 
         if( item->Type() == PCB_ZONE_T )
         {
+            m_frame->FocusOnItem( item, principalLayer );
+
             m_frame->GetBoard()->GetConnectivity()->RunOnUnconnectedEdges(
                     [&]( CN_EDGE& edge )
                     {
@@ -488,12 +519,25 @@ void DIALOG_DRC::OnDRCItemSelected( wxDataViewEvent& aEvent )
                             return true;
 
                         if( edge.GetSourceNode()->Parent() == a
-                                && edge.GetTargetNode()->Parent() == b )
+                            && edge.GetTargetNode()->Parent() == b )
                         {
-                            if( item == a )
-                                m_frame->FocusOnLocation( edge.GetSourcePos() );
+                            VECTOR2I focusPos;
+
+                            if( item == a && item == b )
+                            {
+                                focusPos = ( node->m_Type == RC_TREE_NODE::MAIN_ITEM )
+                                                   ? edge.GetSourcePos()
+                                                   : edge.GetTargetPos();
+                            }
                             else
-                                m_frame->FocusOnLocation( edge.GetTargetPos() );
+                            {
+                                focusPos = ( item == edge.GetSourceNode()->Parent() )
+                                                   ? edge.GetSourcePos()
+                                                   : edge.GetTargetPos();
+                            }
+
+                            m_frame->FocusOnLocation( focusPos );
+                            m_frame->RefreshCanvas();
 
                             return false;
                         }
@@ -607,7 +651,7 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
             || rcItem->GetErrorCode() == DRCE_HOLE_CLEARANCE
             || rcItem->GetErrorCode() == DRCE_DRILLED_HOLES_TOO_CLOSE )
     {
-        menu.Append( 3, _( "Run clearance resolution tool..." ) );
+        menu.Append( 3, _( "Run Inspect > Clearance Resolution" ) );
     }
     else if( rcItem->GetErrorCode() == DRCE_TEXT_HEIGHT
             || rcItem->GetErrorCode() == DRCE_TEXT_THICKNESS
@@ -617,9 +661,14 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
             || rcItem->GetErrorCode() == DRCE_ANNULAR_WIDTH
             || rcItem->GetErrorCode() == DRCE_DRILL_OUT_OF_RANGE
             || rcItem->GetErrorCode() == DRCE_MICROVIA_DRILL_OUT_OF_RANGE
-            || rcItem->GetErrorCode() == DRCE_CONNECTION_WIDTH )
+            || rcItem->GetErrorCode() == DRCE_CONNECTION_WIDTH
+            || rcItem->GetErrorCode() == DRCE_ASSERTION_FAILURE )
     {
-        menu.Append( 3, _( "Run constraints resolution tool..." ) );
+        menu.Append( 3, _( "Run Inspect > Constraints Resolution" ) );
+    }
+    else if( rcItem->GetErrorCode() == DRCE_LIB_FOOTPRINT_MISMATCH )
+    {
+        menu.Append( 3, _( "Run Inspect > Compare Footprint with Library" ) );
     }
 
     menu.AppendSeparator();
@@ -659,6 +708,7 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
         if( marker )
         {
             marker->SetExcluded( false );
+            m_frame->GetDesignSettings().m_DrcExclusions.erase( marker->Serialize() );
 
             if( rcItem->GetErrorCode() == DRCE_UNCONNECTED_ITEMS )
             {
@@ -685,6 +735,7 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
         if( marker )
         {
             marker->SetExcluded( true );
+            m_frame->GetDesignSettings().m_DrcExclusions.insert( marker->Serialize() );
 
             if( rcItem->GetErrorCode() == DRCE_UNCONNECTED_ITEMS )
             {
@@ -715,7 +766,10 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
             DRC_ITEM* candidateDrcItem = static_cast<DRC_ITEM*>( marker->GetRCItem().get() );
 
             if( candidateDrcItem->GetViolatingRule() == drcItem->GetViolatingRule() )
+            {
                 marker->SetExcluded( false );
+                m_frame->GetDesignSettings().m_DrcExclusions.erase( marker->Serialize() );
+            }
         }
 
         // Rebuild model and view
@@ -731,7 +785,10 @@ void DIALOG_DRC::OnDRCItemRClick( wxDataViewEvent& aEvent )
             DRC_ITEM* candidateDrcItem = static_cast<DRC_ITEM*>( marker->GetRCItem().get() );
 
             if( candidateDrcItem->GetViolatingRule() == drcItem->GetViolatingRule() )
+            {
                 marker->SetExcluded( true );
+                m_frame->GetDesignSettings().m_DrcExclusions.insert( marker->Serialize() );
+            }
         }
 
         // Rebuild model and view
@@ -855,10 +912,11 @@ void DIALOG_DRC::OnSeverity( wxCommandEvent& aEvent )
 
 void DIALOG_DRC::OnSaveReport( wxCommandEvent& aEvent )
 {
-    wxFileName fn( "DRC." + ReportFileExtension );
+    wxFileName fn( "DRC." + FILEEXT::ReportFileExtension );
 
-    wxFileDialog dlg( this, _( "Save Report to File" ), Prj().GetProjectPath(), fn.GetFullName(),
-                      ReportFileWildcard(), wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
+    wxFileDialog dlg( this, _( "Save Report File" ), Prj().GetProjectPath(), fn.GetFullName(),
+                      FILEEXT::ReportFileWildcard() + wxS( "|" ) + FILEEXT::JsonFileWildcard(),
+                      wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
     if( dlg.ShowModal() != wxID_OK )
         return;
@@ -866,7 +924,7 @@ void DIALOG_DRC::OnSaveReport( wxCommandEvent& aEvent )
     fn = dlg.GetPath();
 
     if( fn.GetExt().IsEmpty() )
-        fn.SetExt( ReportFileExtension );
+        fn.SetExt( FILEEXT::ReportFileExtension );
 
     if( !fn.IsAbsolute() )
     {
@@ -874,7 +932,16 @@ void DIALOG_DRC::OnSaveReport( wxCommandEvent& aEvent )
         fn.MakeAbsolute( prj_path );
     }
 
-    if( writeReport( fn.GetFullPath() ) )
+    DRC_REPORT reportWriter( m_frame->GetBoard(), GetUserUnits(), m_markersProvider,
+                             m_ratsnestProvider, m_fpWarningsProvider );
+
+    bool success = false;
+    if( fn.GetExt() == FILEEXT::JsonFileExtension )
+        success = reportWriter.WriteJsonReport( fn.GetFullPath() );
+    else
+        success = reportWriter.WriteTextReport( fn.GetFullPath() );
+
+    if( success )
     {
         m_messages->Report( wxString::Format( _( "Report file '%s' created<br>" ),
                                               fn.GetFullPath() ) );
@@ -919,13 +986,11 @@ void DIALOG_DRC::OnCancelClick( wxCommandEvent& aEvent )
 
 void DIALOG_DRC::OnChangingNotebookPage( wxNotebookEvent& aEvent )
 {
-    // Shouldn't be necessary, but is on at least OSX
-    if( aEvent.GetSelection() >= 0 )
-        m_Notebook->ChangeSelection( (unsigned) aEvent.GetSelection() );
-
     m_markerDataView->UnselectAll();
     m_unconnectedDataView->UnselectAll();
     m_footprintsDataView->UnselectAll();
+
+    aEvent.Skip();
 }
 
 
@@ -971,25 +1036,24 @@ void DIALOG_DRC::SelectMarker( const PCB_MARKER* aMarker )
 {
     if( m_Notebook->IsShown() )
     {
-        m_Notebook->SetSelection( 0 );
+        enum MARKER_BASE::MARKER_T markerType = aMarker->GetMarkerType();
+
+        if( markerType == MARKER_BASE::MARKER_DRC )
+            m_Notebook->SetSelection( 0 );
+        else if( markerType == MARKER_BASE::MARKER_PARITY )
+            m_Notebook->SetSelection( 2 );
+
         m_markersTreeModel->SelectMarker( aMarker );
 
-        // wxWidgets on some platforms fails to correctly ensure that a selected item is
-        // visible, so we have to do it in a separate idle event.
-        m_centerMarkerOnIdle = aMarker;
-        Bind( wxEVT_IDLE, &DIALOG_DRC::centerMarkerIdleHandler, this );
+        CallAfter(
+                [=]
+                {
+                    m_markersTreeModel->CenterMarker( aMarker );
+                } );
     }
 }
 
 
-void DIALOG_DRC::centerMarkerIdleHandler( wxIdleEvent& aEvent )
-{
-    m_markersTreeModel->CenterMarker( m_centerMarkerOnIdle );
-    m_centerMarkerOnIdle = nullptr;
-    Unbind( wxEVT_IDLE, &DIALOG_DRC::centerMarkerIdleHandler, this );
-}
-
-
 void DIALOG_DRC::ExcludeMarker()
 {
     if( !m_Notebook->IsShown() || m_Notebook->GetSelection() != 0 )
@@ -1001,6 +1065,7 @@ void DIALOG_DRC::ExcludeMarker()
     if( marker && marker->GetSeverity() != RPT_SEVERITY_EXCLUSION )
     {
         marker->SetExcluded( true );
+        m_frame->GetDesignSettings().m_DrcExclusions.insert( marker->Serialize() );
         m_frame->GetCanvas()->GetView()->Update( marker );
 
         // Update view
@@ -1019,81 +1084,15 @@ void DIALOG_DRC::ExcludeMarker()
 void DIALOG_DRC::deleteAllMarkers( bool aIncludeExclusions )
 {
     // Clear current selection list to avoid selection of deleted items
-    m_frame->GetToolManager()->RunAction( PCB_ACTIONS::selectionClear, true );
+    Freeze();
+    m_frame->GetToolManager()->RunAction( PCB_ACTIONS::selectionClear );
 
     m_markersTreeModel->DeleteItems( false, aIncludeExclusions, false );
     m_unconnectedTreeModel->DeleteItems( false, aIncludeExclusions, false );
     m_fpWarningsTreeModel->DeleteItems( false, aIncludeExclusions, false );
 
     m_frame->GetBoard()->DeleteMARKERs( true, aIncludeExclusions );
-}
-
-
-bool DIALOG_DRC::writeReport( const wxString& aFullFileName )
-{
-    FILE* fp = wxFopen( aFullFileName, wxT( "w" ) );
-
-    if( fp == nullptr )
-        return false;
-
-    std::map<KIID, EDA_ITEM*> itemMap;
-    m_frame->GetBoard()->FillItemMap( itemMap );
-
-    BOARD_DESIGN_SETTINGS& bds = m_frame->GetBoard()->GetDesignSettings();
-    UNITS_PROVIDER         unitsProvider( pcbIUScale, GetUserUnits() );
-    int                    count;
-
-    fprintf( fp, "** Drc report for %s **\n", TO_UTF8( m_frame->GetBoard()->GetFileName() ) );
-
-    wxDateTime now = wxDateTime::Now();
-
-    fprintf( fp, "** Created on %s **\n", TO_UTF8( now.Format( wxT( "%F %T" ) ) ) );
-
-    count = m_markersProvider->GetCount();
-
-    fprintf( fp, "\n** Found %d DRC violations **\n", count );
-
-    for( int i = 0; i < count; ++i )
-    {
-        const std::shared_ptr<RC_ITEM>& item = m_markersProvider->GetItem( i );
-        SEVERITY severity = item->GetParent()->GetSeverity();
-
-        if( severity == RPT_SEVERITY_EXCLUSION )
-            severity = bds.GetSeverity( item->GetErrorCode() );
-
-        fprintf( fp, "%s", TO_UTF8( item->ShowReport( &unitsProvider, severity, itemMap ) ) );
-    }
-
-    count = m_ratsnestProvider->GetCount();
-
-    fprintf( fp, "\n** Found %d unconnected pads **\n", count );
-
-    for( int i = 0; i < count; ++i )
-    {
-        const std::shared_ptr<RC_ITEM>& item = m_ratsnestProvider->GetItem( i );
-        SEVERITY severity = bds.GetSeverity( item->GetErrorCode() );
-
-        fprintf( fp, "%s", TO_UTF8( item->ShowReport( &unitsProvider, severity, itemMap ) ) );
-    }
-
-    count = m_fpWarningsProvider->GetCount();
-
-    fprintf( fp, "\n** Found %d Footprint errors **\n", count );
-
-    for( int i = 0; i < count; ++i )
-    {
-        const std::shared_ptr<RC_ITEM>& item = m_fpWarningsProvider->GetItem( i );
-        SEVERITY severity = bds.GetSeverity( item->GetErrorCode() );
-
-        fprintf( fp, "%s", TO_UTF8( item->ShowReport( &unitsProvider, severity, itemMap ) ) );
-    }
-
-
-    fprintf( fp, "\n** End of Report **\n" );
-
-    fclose( fp );
-
-    return true;
+    Thaw();
 }
 
 
@@ -1102,7 +1101,7 @@ void DIALOG_DRC::OnDeleteOneClick( wxCommandEvent& aEvent )
     if( m_Notebook->GetSelection() == 0 )
     {
         // Clear the selection.  It may be the selected DRC marker.
-        m_frame->GetToolManager()->RunAction( PCB_ACTIONS::selectionClear, true );
+        m_frame->GetToolManager()->RunAction( PCB_ACTIONS::selectionClear );
 
         m_markersTreeModel->DeleteCurrentItem( true );
 
@@ -1139,17 +1138,17 @@ void DIALOG_DRC::OnDeleteAllClick( wxCommandEvent& aEvent )
 
     if( numExcluded > 0 )
     {
-        wxRichMessageDialog dlg( this, _( "Do you wish to delete excluded markers as well?" ),
-                                 _( "Delete All Markers" ),
-                                 wxOK | wxCANCEL | wxCENTER | wxICON_QUESTION );
-        dlg.ShowCheckBox( _( "Delete exclusions" ), s_includeExclusions );
+        wxMessageDialog dlg( this, _( "Delete exclusions too?" ), _( "Delete All Markers" ),
+                             wxYES_NO | wxCANCEL | wxCENTER | wxICON_QUESTION );
+        dlg.SetYesNoLabels( _( "Errors and Warnings Only" ),
+                            _( "Errors, Warnings and Exclusions" ) );
 
         int ret = dlg.ShowModal();
 
         if( ret == wxID_CANCEL )
             return;
-        else
-            s_includeExclusions = dlg.IsCheckBoxChecked();
+        else if( ret == wxID_NO )
+            s_includeExclusions = true;
     }
 
     deleteAllMarkers( s_includeExclusions );
@@ -1226,7 +1225,8 @@ void DIALOG_DRC::updateDisplayedCounts()
             else if(    ii == DRCE_MISSING_FOOTPRINT
                      || ii == DRCE_DUPLICATE_FOOTPRINT
                      || ii == DRCE_EXTRA_FOOTPRINT
-                     || ii == DRCE_NET_CONFLICT )
+                     || ii == DRCE_NET_CONFLICT
+                     || ii == DRCE_SCHEMATIC_PARITY_ISSUES )
             {
                 if( showWarnings && bds.GetSeverity( ii ) == RPT_SEVERITY_WARNING )
                     footprintsOverflowed = true;
