@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2012 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,13 +27,12 @@
 #include <pcb_edit_frame.h>
 #include <footprint_edit_frame.h>
 #include <pcb_layer_box_selector.h>
-#include <pcbnew.h>
 #include <board.h>
 #include <board_design_settings.h>
 #include <footprint.h>
 #include <pcb_group.h>
 #include <pcb_dimension.h>
-#include <fp_shape.h>
+#include <pcb_shape.h>
 #include <pcb_text.h>
 #include <widgets/unit_binder.h>
 #include <widgets/font_choice.h>
@@ -41,7 +40,6 @@
 #include <tools/global_edit_tool.h>
 #include <tools/footprint_editor_control.h>
 #include <dialog_global_edit_text_and_graphics_base.h>
-#include "font/kicad_font_name.h"
 
 // Columns of layer classes grid
 enum
@@ -101,6 +99,7 @@ public:
 protected:
     void onActionButtonChange( wxCommandEvent& event ) override;
     void onSpecifiedValueUpdateUI( wxUpdateUIEvent& event ) override;
+    void onDimensionItemCheckbox( wxCommandEvent& aEvent ) override;
 
     void OnLayerFilterSelect( wxCommandEvent& event ) override
     {
@@ -138,6 +137,7 @@ DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS( PCB_
     {
         m_otherFields->SetLabel( _( "Other text items" ) );
         m_footprintGraphics->SetLabel( _( "Graphic items" ) );
+        m_footprintDimensions->SetLabel( _( "Dimension items" ) );
 
         m_boardText->Show( false );
         m_boardGraphics->Show( false );
@@ -238,6 +238,9 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataToWindow()
     m_visible->Set3StateValue( wxCHK_UNDETERMINED );
     m_LayerCtrl->SetLayerSelection( UNDEFINED_LAYER );
 
+    wxCommandEvent dummy;
+    onActionButtonChange( dummy );
+
 #define SET_INT_VALUE( aRow, aCol, aValue ) \
         m_grid->SetCellValue( aRow, aCol, m_parent->StringFromValue( aValue, true ) )
 
@@ -309,10 +312,21 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::onActionButtonChange( wxCommandEvent&
     // Update the UNIT_BINDER controls if the action to take is changed
     bool enable = m_setToSpecifiedValues->GetValue();
 
+    m_LayerLabel->Enable( enable );
+    m_LayerCtrl->Enable( enable );
+
     m_lineWidth.Enable( enable );
     m_textWidth.Enable( enable );
     m_textHeight.Enable( enable );
     m_thickness.Enable( enable );
+
+    m_fontLabel->Enable( enable );
+    m_fontCtrl->Enable( enable );
+
+    m_centerOnFP->Enable( enable );
+
+    enable = !enable;
+    m_grid->Enable( enable );
 }
 
 
@@ -323,67 +337,75 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::onSpecifiedValueUpdateUI( wxUpdateUIE
 }
 
 
+void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::onDimensionItemCheckbox( wxCommandEvent& aEvent )
+{
+    if( m_footprintDimensions->GetValue() || m_boardDimensions->GetValue() )
+        m_setToLayerDefaults->SetLabel( _( "Set to layer and dimension default values:" ) );
+    else
+        m_setToLayerDefaults->SetLabel( _( "Set to layer default values:" ) );
+}
+
+
 void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, BOARD_ITEM* aItem )
 {
     aCommit.Modify( aItem );
 
-    EDA_TEXT*           edaText = dynamic_cast<EDA_TEXT*>( aItem );
-    FP_TEXT*            fpText = dynamic_cast<FP_TEXT*>( aItem );
+    PCB_TEXT*           text = dynamic_cast<PCB_TEXT*>( aItem );
     PCB_SHAPE*          shape = dynamic_cast<PCB_SHAPE*>( aItem );
     PCB_DIMENSION_BASE* dimension = dynamic_cast<PCB_DIMENSION_BASE*>( aItem );
-
-    if( dimension )
-        edaText = &dimension->Text();
+    FOOTPRINT*          parentFP = aItem->GetParentFootprint();
 
     if( m_setToSpecifiedValues->GetValue() )
     {
         if( m_LayerCtrl->GetLayerSelection() != UNDEFINED_LAYER )
             aItem->SetLayer( ToLAYER_ID( m_LayerCtrl->GetLayerSelection() ) );
 
-        if( edaText )
+        if( text )
         {
             if( !m_textWidth.IsIndeterminate() )
-                edaText->SetTextSize( wxSize( m_textWidth.GetValue(), edaText->GetTextSize().y ) );
+                text->SetTextSize( VECTOR2I( m_textWidth.GetValue(), text->GetTextSize().y ) );
 
             if( !m_textHeight.IsIndeterminate() )
-                edaText->SetTextSize( wxSize( edaText->GetTextSize().x, m_textHeight.GetValue() ) );
+                text->SetTextSize( VECTOR2I( text->GetTextSize().x, m_textHeight.GetValue() ) );
 
             if( !m_thickness.IsIndeterminate() )
-                edaText->SetTextThickness( m_thickness.GetValue() );
+                text->SetTextThickness( m_thickness.GetValue() );
 
+            // Must be after SetTextSize()
             if( m_bold->Get3StateValue() != wxCHK_UNDETERMINED )
-                edaText->SetBold( m_bold->GetValue() );
+                text->SetBold( m_bold->GetValue() );
 
             if( m_italic->Get3StateValue() != wxCHK_UNDETERMINED )
-                edaText->SetItalic( m_italic->GetValue() );
+                text->SetItalic( m_italic->GetValue() );
 
             // Must come after setting bold & italic
             if( m_fontCtrl->GetStringSelection() != INDETERMINATE_ACTION )
             {
-                edaText->SetFont( m_fontCtrl->GetFontSelection( edaText->IsBold(),
-                                                                edaText->IsItalic() ) );
+                text->SetFont( m_fontCtrl->GetFontSelection( text->IsBold(), text->IsItalic() ) );
             }
             else if(( m_italic->Get3StateValue() != wxCHK_UNDETERMINED
                     || m_bold->Get3StateValue() != wxCHK_UNDETERMINED ) )
             {
-                wxString fontName = edaText->GetFontName();
+                wxString fontName = text->GetFontName();
 
-                if( !edaText->GetFontName().IsEmpty() )
+                if( !text->GetFontName().IsEmpty() )
                 {
-                    edaText->SetFont( KIFONT::FONT::GetFont( edaText->GetFontName(),
-                                                             edaText->IsBold(),
-                                                             edaText->IsItalic() ) );
+                    text->SetFont( KIFONT::FONT::GetFont( text->GetFontName(), text->IsBold(),
+                                                          text->IsItalic() ) );
                 }
             }
 
-            if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
-                edaText->SetVisible( m_visible->GetValue() );
-        }
+            if( parentFP )
+            {
+                if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
+                    text->SetVisible( m_visible->GetValue() );
 
-        if( fpText )
-        {
-            if( m_keepUpright->Get3StateValue() != wxCHK_UNDETERMINED )
-                fpText->SetKeepUpright( m_keepUpright->GetValue() );
+                if( m_keepUpright->Get3StateValue() != wxCHK_UNDETERMINED )
+                    text->SetKeepUpright( m_keepUpright->GetValue() );
+
+                if( m_centerOnFP->GetValue() == wxCHK_CHECKED )
+                    text->SetTextPos( text->GetParent()->GetCenter() );
+            }
         }
 
         if( !m_lineWidth.IsIndeterminate() )
@@ -403,15 +425,15 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, B
     {
         PCB_LAYER_ID layer = aItem->GetLayer();
 
-        if( edaText )
+        if( text )
         {
-            edaText->SetTextSize( m_brdSettings->GetTextSize( layer ) );
-            edaText->SetTextThickness( m_brdSettings->GetTextThickness( layer ) );
-            edaText->SetItalic( m_brdSettings->GetTextItalic( layer ) );
-        }
+            text->SetTextSize( m_brdSettings->GetTextSize( layer ) );
+            text->SetTextThickness( m_brdSettings->GetTextThickness( layer ) );
+            text->SetItalic( m_brdSettings->GetTextItalic( layer ) );
 
-        if( fpText )
-            fpText->SetKeepUpright( m_brdSettings->GetTextUpright( layer ) );
+            if( parentFP )
+                text->SetKeepUpright( m_brdSettings->GetTextUpright( layer ) );
+        }
 
         if( shape )
         {
@@ -421,7 +443,16 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( BOARD_COMMIT& aCommit, B
         }
 
         if( dimension )
+        {
             dimension->SetLineThickness( m_brdSettings->GetLineThickness( layer ) );
+            dimension->SetUnitsMode( m_brdSettings->m_DimensionUnitsMode );
+            dimension->SetUnitsFormat( m_brdSettings->m_DimensionUnitsFormat );
+            dimension->SetPrecision( m_brdSettings->m_DimensionPrecision );
+            dimension->SetSuppressZeroes( m_brdSettings->m_DimensionSuppressZeroes );
+            dimension->SetTextPositionMode( m_brdSettings->m_DimensionTextPosition );
+            dimension->SetKeepTextAligned( m_brdSettings->m_DimensionKeepTextAligned );
+            dimension->Update();    // refresh text & geometry
+        }
     }
 }
 
@@ -460,9 +491,7 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( BOARD_COMMIT& aCommit, BOA
     {
         if( m_referenceFilterOpt->GetValue() && !m_referenceFilter->GetValue().IsEmpty() )
         {
-            FOOTPRINT* fp = dynamic_cast<FOOTPRINT*>( aItem->GetParent() );
-
-            if( fp )
+            if( FOOTPRINT* fp = aItem->GetParentFootprint() )
             {
                 if( !WildCompareString( m_referenceFilter->GetValue(), fp->GetReference(), false ) )
                     return;
@@ -471,14 +500,12 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( BOARD_COMMIT& aCommit, BOA
 
         if( m_footprintFilterOpt->GetValue() && !m_footprintFilter->GetValue().IsEmpty() )
         {
-            FOOTPRINT* fp = dynamic_cast<FOOTPRINT*>( aItem->GetParent() );
-
-            if( fp )
+            if( FOOTPRINT* fp = aItem->GetParentFootprint() )
             {
                 if( !WildCompareString( m_footprintFilter->GetValue(), fp->GetFPID().Format(), false ) )
                     return;
             }
-    }
+        }
     }
 
     processItem( aCommit, aItem );
@@ -487,8 +514,11 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( BOARD_COMMIT& aCommit, BOA
 
 bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
 {
-    if( !m_textWidth.Validate( TEXTS_MIN_SIZE, TEXTS_MAX_SIZE )
-        || !m_textHeight.Validate( TEXTS_MIN_SIZE, TEXTS_MAX_SIZE ) )
+    int minTextSize = pcbIUScale.mmToIU( TEXT_MIN_SIZE_MM );
+    int maxTextSize = pcbIUScale.mmToIU( TEXT_MAX_SIZE_MM );
+
+    if( !m_textWidth.Validate( minTextSize, maxTextSize )
+        || !m_textHeight.Validate( minTextSize, maxTextSize ) )
     {
         return false;
     }
@@ -509,12 +539,17 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
         {
             KICAD_T itemType = boardItem->Type();
 
-            if( itemType == PCB_FP_TEXT_T || itemType == PCB_FP_TEXTBOX_T )
+            if( itemType == PCB_FIELD_T || itemType == PCB_TEXT_T || itemType == PCB_TEXTBOX_T )
             {
                 if( m_otherFields->GetValue() )
                     visitItem( commit, boardItem );
             }
-            else if( itemType == PCB_FP_SHAPE_T || BaseType( itemType ) == PCB_DIMENSION_T )
+            else if( BaseType( itemType ) == PCB_DIMENSION_T )
+            {
+                if( m_footprintDimensions->GetValue() )
+                    visitItem( commit, boardItem );
+            }
+            else if( itemType == PCB_SHAPE_T )
             {
                 if( m_footprintGraphics->GetValue() )
                     visitItem( commit, boardItem );
@@ -534,6 +569,11 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
                 if( m_boardText->GetValue() )
                     visitItem( commit, boardItem );
             }
+            else if( BaseType( itemType ) == PCB_DIMENSION_T )
+            {
+                if( m_boardDimensions->GetValue() )
+                    visitItem( commit, boardItem );
+            }
             else if( itemType == PCB_SHAPE_T || BaseType( itemType ) == PCB_DIMENSION_T )
             {
                 if( m_boardGraphics->GetValue() )
