@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004-2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2014 Dick Hollenbeck, dick@softplc.com
- * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2022, 2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
  */
 
 #include <atomic>
+#include <wx/log.h>
 #include <reporter.h>
 #include <progress_reporter.h>
 #include <string_utils.h>
@@ -39,7 +40,8 @@
 #include <footprint.h>
 #include <pad.h>
 #include <pcb_track.h>
-#include <thread_pool.h>
+#include <core/profile.h>
+#include <core/thread_pool.h>
 #include <zone.h>
 
 
@@ -49,6 +51,16 @@
 #define EXTENDED_ERROR_LIMIT 499
 
 
+/**
+ * Flag to enable DRC profile timing logging.
+ *
+ * Use "KICAD_DRC_PROFILE" to enable.
+ *
+ * @ingroup trace_env_vars
+ */
+static const wxChar* traceDrcProfile = wxT( "KICAD_DRC_PROFILE" );
+
+
 void drcPrintDebugMessage( int level, const wxString& msg, const char *function, int line )
 {
     wxString valueStr;
@@ -98,10 +110,7 @@ DRC_ENGINE::~DRC_ENGINE()
 
 static bool isKeepoutZone( const BOARD_ITEM* aItem, bool aCheckFlags )
 {
-    if( !aItem )
-        return false;
-
-    if( aItem->Type() != PCB_ZONE_T && aItem->Type() != PCB_FP_ZONE_T )
+    if( !aItem || aItem->Type() != PCB_ZONE_T )
         return false;
 
     const ZONE* zone = static_cast<const ZONE*>( aItem );
@@ -232,13 +241,21 @@ void DRC_ENGINE::loadImplicitRules()
     auto makeNetclassRules =
             [&]( const std::shared_ptr<NETCLASS>& nc, bool isDefault )
             {
-                wxString ncName = nc->GetName();
-                wxString expr;
+                wxString  ncName = nc->GetName();
+                wxString  friendlyName;
+                wxString* shownName = &ncName;
+                wxString  expr;
+
+                if( ncName.Replace( "'", "\\'" ) )
+                {
+                    friendlyName = nc->GetName();
+                    shownName = &friendlyName;
+                }
 
                 if( nc->GetClearance() || nc->GetTrackWidth() )
                 {
                     std::shared_ptr<DRC_RULE> netclassRule = std::make_shared<DRC_RULE>();
-                    netclassRule->m_Name = wxString::Format( _( "netclass '%s'" ), ncName );
+                    netclassRule->m_Name = wxString::Format( _( "netclass '%s'" ), *shownName );
                     netclassRule->m_Implicit = true;
 
                     expr = wxString::Format( wxT( "A.NetClass == '%s'" ), ncName );
@@ -265,7 +282,7 @@ void DRC_ENGINE::loadImplicitRules()
                 {
                     std::shared_ptr<DRC_RULE> netclassRule = std::make_shared<DRC_RULE>();
                     netclassRule->m_Name = wxString::Format( _( "netclass '%s' (diff pair)" ),
-                                                             ncName );
+                                                             *shownName );
                     netclassRule->m_Implicit = true;
 
                     expr = wxString::Format( wxT( "A.NetClass == '%s' && A.inDiffPair('*')" ),
@@ -283,7 +300,7 @@ void DRC_ENGINE::loadImplicitRules()
                 {
                     std::shared_ptr<DRC_RULE> netclassRule = std::make_shared<DRC_RULE>();
                     netclassRule->m_Name = wxString::Format( _( "netclass '%s' (diff pair)" ),
-                                                             ncName );
+                                                             *shownName );
                     netclassRule->m_Implicit = true;
 
                     expr = wxString::Format( wxT( "A.NetClass == '%s'" ), ncName );
@@ -300,7 +317,7 @@ void DRC_ENGINE::loadImplicitRules()
                     {
                         netclassRule = std::make_shared<DRC_RULE>();
                         netclassRule->m_Name = wxString::Format( _( "netclass '%s' (diff pair)" ),
-                                                                 ncName );
+                                                                 *shownName );
                         netclassRule->m_Implicit = true;
 
                         expr = wxString::Format( wxT( "A.NetClass == '%s' && AB.isCoupledDiffPair()" ),
@@ -317,7 +334,7 @@ void DRC_ENGINE::loadImplicitRules()
                 if( nc->GetViaDiameter() || nc->GetViaDrill() )
                 {
                     std::shared_ptr<DRC_RULE> netclassRule = std::make_shared<DRC_RULE>();
-                    netclassRule->m_Name = wxString::Format( _( "netclass '%s'" ), ncName );
+                    netclassRule->m_Name = wxString::Format( _( "netclass '%s'" ), *shownName );
                     netclassRule->m_Implicit = true;
 
                     expr = wxString::Format( wxT( "A.NetClass == '%s' && A.Via_Type != 'Micro'" ),
@@ -345,7 +362,8 @@ void DRC_ENGINE::loadImplicitRules()
                 if( nc->GetuViaDiameter() || nc->GetuViaDrill() )
                 {
                     std::shared_ptr<DRC_RULE> netclassRule = std::make_shared<DRC_RULE>();
-                    netclassRule->m_Name = wxString::Format( _( "netclass '%s'" ), ncName );
+                    netclassRule->m_Name = wxString::Format( _( "netclass '%s' (uvia)" ),
+                                                             *shownName );
                     netclassRule->m_Implicit = true;
 
                     expr = wxString::Format( wxT( "A.NetClass == '%s' && A.Via_Type == 'Micro'" ),
@@ -566,6 +584,8 @@ void DRC_ENGINE::InitEngine( const wxFileName& aRulePath )
 
 void DRC_ENGINE::RunTests( EDA_UNITS aUnits, bool aReportAllTrackErrors, bool aTestFootprints )
 {
+    PROF_TIMER timer;
+
     SetUserUnits( aUnits );
 
     m_reportAllTrackErrors = aReportAllTrackErrors;
@@ -601,6 +621,9 @@ void DRC_ENGINE::RunTests( EDA_UNITS aUnits, bool aReportAllTrackErrors, bool aT
             break;
     }
 
+    timer.Stop();
+    wxLogTrace( traceDrcProfile, "DRC took %0.3f ms", timer.msecs() );
+
     // DRC tests are multi-threaded; anything that causes us to attempt to re-generate the
     // caches while DRC is running is problematic.
     wxASSERT( timestamp == m_board->GetTimeStamp() );
@@ -651,11 +674,7 @@ bool hasDrilledHole( const BOARD_ITEM* aItem )
     switch( aItem->Type() )
     {
     case PCB_VIA_T:
-    {
-        const PCB_VIA* via = static_cast<const PCB_VIA*>( aItem );
-
-        return via->GetViaType() == VIATYPE::THROUGH;
-    }
+        return true;
 
     case PCB_PAD_T:
     {
@@ -697,21 +716,89 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
     {
         if( a && a->Type() == PCB_PAD_T )
             pad = static_cast<const PAD*>( a );
-        else if( a && ( a->Type() == PCB_ZONE_T || a->Type() == PCB_FP_ZONE_T ) )
+        else if( a && a->Type() == PCB_ZONE_T )
             zone = static_cast<const ZONE*>( a );
 
         if( b && b->Type() == PCB_PAD_T )
             pad = static_cast<const PAD*>( b );
-        else if( b && ( b->Type() == PCB_ZONE_T || b->Type() == PCB_FP_ZONE_T ) )
+        else if( b && b->Type() == PCB_ZONE_T )
             zone = static_cast<const ZONE*>( b );
 
         if( pad )
-            parentFootprint = static_cast<FOOTPRINT*>( pad->GetParentFootprint() );
+            parentFootprint = pad->GetParentFootprint();
     }
 
     DRC_CONSTRAINT constraint;
     constraint.m_Type = aConstraintType;
 
+    auto applyConstraint =
+            [&]( const DRC_ENGINE_CONSTRAINT* c )
+            {
+                if( c->constraint.m_Value.HasMin() )
+                {
+                    if( c->parentRule && c->parentRule->m_Implicit )
+                        constraint.m_ImplicitMin = true;
+
+                    constraint.m_Value.SetMin( c->constraint.m_Value.Min() );
+                }
+
+                if( c->constraint.m_Value.HasOpt() )
+                    constraint.m_Value.SetOpt( c->constraint.m_Value.Opt() );
+
+                if( c->constraint.m_Value.HasMax() )
+                    constraint .m_Value.SetMax( c->constraint.m_Value.Max() );
+
+                // While the expectation would be to OR the disallow flags, we've already
+                // masked them down to aItem's type -- so we're really only looking for a
+                // boolean here.
+                constraint.m_DisallowFlags = c->constraint.m_DisallowFlags;
+
+                constraint.m_ZoneConnection = c->constraint.m_ZoneConnection;
+
+                constraint.SetParentRule( c->constraint.GetParentRule() );
+            };
+
+    const FOOTPRINT* footprints[2] = {a ? a->GetParentFootprint() : nullptr,
+                                      b ? b->GetParentFootprint() : nullptr};
+
+    // Handle Footprint net ties, which will zero out the clearance for footprint objects
+    if( aConstraintType == CLEARANCE_CONSTRAINT // Only zero clearance, other constraints still apply
+        && ( ( ( !ac ) ^ ( !bc ) )// Only apply to cases where we are comparing a connected item to a non-connected item
+                                  // and not both connected.  Connected items of different nets still need to be checked
+                                  // for their standard clearance value
+            || ( ( footprints[0] == footprints[1] )  // Unless both items are in the same footprint
+                   && footprints[0] ) )              // And that footprint exists
+        && !a_is_non_copper       // Also, both elements need to be on copper layers
+        && !b_is_non_copper )
+    {
+        const BOARD_ITEM* child_items[2] = {a, b};
+
+        // These are the items being compared against, so the order is reversed
+        const BOARD_CONNECTED_ITEM* alt_items[2] = {bc, ac};
+
+        for( int ii = 0; ii < 2; ++ii )
+        {
+            // We need both a footprint item and a connected item to check for a net tie
+            if( !footprints[ii] || !alt_items[ii] )
+                continue;
+
+            const std::set<int>& netcodes = footprints[ii]->GetNetTieCache( child_items[ii] );
+
+            auto it = netcodes.find( alt_items[ii]->GetNetCode() );
+
+            if( it != netcodes.end() )
+            {
+                REPORT( "" )
+                REPORT( wxString::Format( _( "Net tie on %s; clearance: 0." ),
+                                          EscapeHTML( footprints[ii]->GetItemDescription( this ) ) ) )
+
+                constraint.SetName( _( "net tie" ) );
+                constraint.m_Value.SetMin( 0 );
+                return constraint;
+            }
+        }
+    }
+
     // Local overrides take precedence over everything *except* board min clearance
     if( aConstraintType == CLEARANCE_CONSTRAINT || aConstraintType == HOLE_CLEARANCE_CONSTRAINT )
     {
@@ -863,7 +950,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
             };
 
     auto processConstraint =
-            [&]( const DRC_ENGINE_CONSTRAINT* c ) -> bool
+            [&]( const DRC_ENGINE_CONSTRAINT* c )
             {
                 bool implicit = c->parentRule && c->parentRule->m_Implicit;
 
@@ -883,7 +970,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                                               MessageTextFromValue( c->constraint.m_Value.Min() ) ) )
                     break;
 
-                case DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT:
+                case MAX_UNCOUPLED_CONSTRAINT:
                     REPORT( wxString::Format( _( "Checking %s max uncoupled length: %s." ),
                                               EscapeHTML( c->constraint.GetName() ),
                                               MessageTextFromValue( c->constraint.m_Value.Max() ) ) )
@@ -953,8 +1040,8 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                                 }
                                 else if( c->constraint.m_Value.HasMin() )
                                 {
-                                    REPORT( wxString::Format( _( "Checking board setup constraints "
-                                                                 "track width: min %s." ),
+                                    REPORT( wxString::Format( _( "Checking %s track width: min %s." ),
+                                                              EscapeHTML( c->constraint.GetName() ),
                                                               min ) )
                                 }
 
@@ -975,8 +1062,8 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                                 }
                                 else if( c->constraint.m_Value.HasMin() )
                                 {
-                                    REPORT( wxString::Format( _( "Checking board setup constraints "
-                                                                 "via diameter: min %s." ),
+                                    REPORT( wxString::Format( _( "Checking %s via diameter: min %s." ),
+                                                              EscapeHTML( c->constraint.GetName() ),
                                                               min ) )
                                 }
                                 break;
@@ -990,8 +1077,8 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                                 }
                                 else if( c->constraint.m_Value.HasMin() )
                                 {
-                                    REPORT( wxString::Format( _( "Checking board setup constraints "
-                                                                 "hole size: min %s." ),
+                                    REPORT( wxString::Format( _( "Checking %s hole size: min %s." ),
+                                                              EscapeHTML( c->constraint.GetName() ),
                                                               min ) )
                                 }
 
@@ -1008,23 +1095,22 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                             case DIFF_PAIR_GAP_CONSTRAINT:
                                 if( c->constraint.m_Value.HasOpt() )
                                 {
-                                    REPORT( wxString::Format( _( "Checking %s diff pair gap: "
-                                                                 "opt %s." ),
+                                    REPORT( wxString::Format( _( "Checking %s diff pair gap: opt %s." ),
                                                               EscapeHTML( c->constraint.GetName() ),
                                                               opt ) )
                                 }
                                 else if( c->constraint.m_Value.HasMin() )
                                 {
-                                    REPORT( wxString::Format( _( "Checking board setup constraints "
-                                                                 "clearance: min %s." ),
+                                    REPORT( wxString::Format( _( "Checking %s clearance: min %s." ),
+                                                              EscapeHTML( c->constraint.GetName() ),
                                                               min ) )
                                 }
 
                                 break;
 
                             case HOLE_TO_HOLE_CONSTRAINT:
-                                REPORT( wxString::Format( _( "Checking board setup constraints "
-                                                             "hole to hole: min %s." ),
+                                REPORT( wxString::Format( _( "Checking %s hole to hole: min %s." ),
+                                                          EscapeHTML( c->constraint.GetName() ),
                                                           min ) )
                                 break;
 
@@ -1061,11 +1147,24 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
 
                 if( c->constraint.m_Type == CLEARANCE_CONSTRAINT )
                 {
-                    if( implicit && ( a_is_non_copper || b_is_non_copper ) )
+                    if( a_is_non_copper || b_is_non_copper )
                     {
-                        REPORT( _( "Board and netclass clearances apply only between copper "
-                                   "items." ) );
-                        return true;
+                        if( implicit )
+                        {
+                            REPORT( _( "Netclass clearances apply only between copper items." ) )
+                        }
+                        else if( a_is_non_copper )
+                        {
+                            REPORT( wxString::Format( _( "%s contains no copper.  Rule ignored." ),
+                                                      EscapeHTML( a->GetItemDescription( this ) ) ) )
+                        }
+                        else if( b_is_non_copper )
+                        {
+                            REPORT( wxString::Format( _( "%s contains no copper.  Rule ignored." ),
+                                                      EscapeHTML( b->GetItemDescription( this ) ) ) )
+                        }
+
+                        return;
                     }
                 }
                 else if( c->constraint.m_Type == DISALLOW_CONSTRAINT )
@@ -1096,24 +1195,18 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                         case PCB_PAD_T:          mask = DRC_DISALLOW_PADS;       break;
                         case PCB_FOOTPRINT_T:    mask = DRC_DISALLOW_FOOTPRINTS; break;
                         case PCB_SHAPE_T:        mask = DRC_DISALLOW_GRAPHICS;   break;
-                        case PCB_FP_SHAPE_T:     mask = DRC_DISALLOW_GRAPHICS;   break;
+                        case PCB_FIELD_T:        mask = DRC_DISALLOW_TEXTS;      break;
                         case PCB_TEXT_T:         mask = DRC_DISALLOW_TEXTS;      break;
                         case PCB_TEXTBOX_T:      mask = DRC_DISALLOW_TEXTS;      break;
-                        case PCB_FP_TEXT_T:      mask = DRC_DISALLOW_TEXTS;      break;
-                        case PCB_FP_TEXTBOX_T:   mask = DRC_DISALLOW_TEXTS;      break;
-                        case PCB_ZONE_T:
-                        case PCB_FP_ZONE_T:
-                        {
-                            const ZONE* test_zone = static_cast<const ZONE*>( a );
 
+                        case PCB_ZONE_T:
                             // Treat teardrop areas as tracks for DRC purposes
-                            if( test_zone->IsTeardropArea() )
+                            if( static_cast<const ZONE*>( a )->IsTeardropArea() )
                                 mask = DRC_DISALLOW_TRACKS;
                             else
                                 mask = DRC_DISALLOW_ZONES;
 
                             break;
-                        }
 
                         case PCB_LOCATE_HOLE_T:  mask = DRC_DISALLOW_HOLES;      break;
                         default:                 mask = 0;                       break;
@@ -1127,7 +1220,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                         else
                             REPORT( _( "Disallow constraint not met." ) )
 
-                        return false;
+                        return;
                     }
 
                     LSET itemLayers = a->GetLayerSet();
@@ -1159,7 +1252,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                             REPORT( _( "Rule layer not matched; rule ignored." ) )
                         }
 
-                        return false;
+                        return;
                     }
                 }
 
@@ -1168,8 +1261,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                 {
                     if( implicit )
                     {
-                        // JEY TODO: translate for 8.0....
-                        REPORT( wxS( "Constraint layer not matched." ) )
+                        REPORT( _( "Constraint layer not matched." ) )
                     }
                     else if( c->parentRule )
                     {
@@ -1180,11 +1272,8 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                     {
                         REPORT( _( "Rule layer not matched; rule ignored." ) )
                     }
-
-                    return false;
                 }
-
-                if( c->constraint.m_Type == HOLE_TO_HOLE_CONSTRAINT
+                else if( c->constraint.m_Type == HOLE_TO_HOLE_CONSTRAINT
                         && ( !hasDrilledHole( a ) || !hasDrilledHole( b ) ) )
                 {
                     // Report non-drilled-holes as an implicit condition
@@ -1195,8 +1284,6 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                         REPORT( wxString::Format( _( "%s is not a drilled hole; rule ignored." ),
                                                   x->GetItemDescription( this ) ) )
                     }
-
-                    return false;
                 }
                 else if( !c->condition || c->condition->GetExpression().IsEmpty() )
                 {
@@ -1217,25 +1304,7 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                         }
                     }
 
-                    if( c->constraint.m_Value.HasMin() )
-                        constraint.m_Value.SetMin( c->constraint.m_Value.Min() );
-
-                    if( c->constraint.m_Value.HasOpt() )
-                        constraint.m_Value.SetOpt( c->constraint.m_Value.Opt() );
-
-                    if( c->constraint.m_Value.HasMax() )
-                        constraint .m_Value.SetMax( c->constraint.m_Value.Max() );
-
-                    // While the expectation would be to OR the disallow flags, we've already
-                    // masked them down to aItem's type -- so we're really only looking for a
-                    // boolean here.
-                    constraint.m_DisallowFlags = c->constraint.m_DisallowFlags;
-
-                    constraint.m_ZoneConnection = c->constraint.m_ZoneConnection;
-
-                    constraint.SetParentRule( c->constraint.GetParentRule() );
-
-                    return true;
+                    applyConstraint( c );
                 }
                 else
                 {
@@ -1268,32 +1337,12 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                             }
                         }
 
-                        if( c->constraint.m_Value.HasMin() )
-                            constraint.m_Value.SetMin( c->constraint.m_Value.Min() );
-
-                        if( c->constraint.m_Value.HasOpt() )
-                            constraint.m_Value.SetOpt( c->constraint.m_Value.Opt() );
-
-                        if( c->constraint.m_Value.HasMax() )
-                            constraint .m_Value.SetMax( c->constraint.m_Value.Max() );
-
-                        // While the expectation would be to OR the disallow flags, we've already
-                        // masked them down to aItem's type -- so we're really only looking for a
-                        // boolean here.
-                        constraint.m_DisallowFlags = c->constraint.m_DisallowFlags;
-
-                        constraint.m_ZoneConnection = c->constraint.m_ZoneConnection;
-
-                        constraint.SetParentRule( c->constraint.GetParentRule() );
-
-                        return true;
+                        applyConstraint( c );
                     }
                     else
                     {
                         REPORT( implicit ? _( "Membership not satisfied; constraint ignored." )
                                          : _( "Condition not satisfied; rule ignored." ) )
-
-                        return false;
                     }
                 }
             };
@@ -1339,15 +1388,21 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
     // rule selection paradigm is "winner takes all".
     if( aConstraintType == CLEARANCE_CONSTRAINT )
     {
-        int global = constraint.m_Value.Min();
-        int localA = ac ? ac->GetLocalClearance( nullptr ) : 0;
-        int localB = bc ? bc->GetLocalClearance( nullptr ) : 0;
-        int clearance = global;
+        int  global = constraint.m_Value.Min();
+        int  localA = ac ? ac->GetLocalClearance( nullptr ) : 0;
+        int  localB = bc ? bc->GetLocalClearance( nullptr ) : 0;
+        int  clearance = global;
+        bool needBlankLine = true;
 
         if( localA > 0 )
         {
-            REPORT( "" )
-            REPORT( wxString::Format( _( "Local clearance on %s; clearance: %s." ),
+            if( needBlankLine )
+            {
+                REPORT( "" )
+                needBlankLine = false;
+            }
+
+            REPORT( wxString::Format( _( "Local clearance on %s: %s." ),
                                       EscapeHTML( a->GetItemDescription( this ) ),
                                       MessageTextFromValue( localA ) ) )
 
@@ -1363,8 +1418,13 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
 
         if( localB > 0 )
         {
-            REPORT( "" )
-            REPORT( wxString::Format( _( "Local clearance on %s; clearance: %s." ),
+            if( needBlankLine )
+            {
+                REPORT( "" )
+                needBlankLine = false;
+            }
+
+            REPORT( wxString::Format( _( "Local clearance on %s: %s." ),
                                       EscapeHTML( b->GetItemDescription( this ) ),
                                       MessageTextFromValue( localB ) ) )
 
@@ -1378,22 +1438,23 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
             }
         }
 
-        REPORT( "" )
-        REPORT( wxString::Format( _( "Board minimum clearance: %s." ),
-                                  MessageTextFromValue( m_designSettings->m_MinClearance ) ) )
-
-        if( clearance < m_designSettings->m_MinClearance )
+        if( !a_is_non_copper && !b_is_non_copper )
         {
-            constraint.SetParentRule( nullptr );
-            constraint.SetName( _( "board minimum" ) );
-            constraint.m_Value.SetMin( m_designSettings->m_MinClearance );
-        }
+            if( needBlankLine )
+            {
+                REPORT( "" )
+                needBlankLine = false;
+            }
 
-        if( a_is_non_copper || b_is_non_copper )
-        {
-            REPORT( _( "Board and netclass clearances apply only between copper items." ) );
-            constraint.m_Type = NULL_CONSTRAINT;
-            constraint.m_DisallowFlags = 0;
+            REPORT( wxString::Format( _( "Board minimum clearance: %s." ),
+                                      MessageTextFromValue( m_designSettings->m_MinClearance ) ) )
+
+            if( clearance < m_designSettings->m_MinClearance )
+            {
+                constraint.SetParentRule( nullptr );
+                constraint.SetName( _( "board minimum" ) );
+                constraint.m_Value.SetMin( m_designSettings->m_MinClearance );
+            }
         }
 
         return constraint;
@@ -1803,17 +1864,18 @@ bool DRC_ENGINE::IsNetADiffPair( BOARD* aBoard, NETINFO_ITEM* aNet, int& aNetP,
 bool DRC_ENGINE::IsNetTieExclusion( int aTrackNetCode, PCB_LAYER_ID aTrackLayer,
                                     const VECTOR2I& aCollisionPos, BOARD_ITEM* aCollidingItem )
 {
-    FOOTPRINT* parentFootprint = static_cast<FOOTPRINT*>( aCollidingItem->GetParentFootprint() );
+    FOOTPRINT* parentFootprint = aCollidingItem->GetParentFootprint();
 
     if( parentFootprint && parentFootprint->IsNetTie() )
     {
+        int                     epsilon = GetDesignSettings()->GetDRCEpsilon();
         std::map<wxString, int> padToNetTieGroupMap = parentFootprint->MapPadNumbersToNetTieGroups();
 
         for( PAD* pad : parentFootprint->Pads() )
         {
             if( padToNetTieGroupMap[ pad->GetNumber() ] >= 0 && aTrackNetCode == pad->GetNetCode() )
             {
-                if( pad->GetEffectiveShape( aTrackLayer )->Collide( aCollisionPos, 0 ) )
+                if( pad->GetEffectiveShape( aTrackLayer )->Collide( aCollisionPos, epsilon ) )
                     return true;
             }
         }
