@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2022 KiCad Developers.
+ * Copyright (C) 2022-2024 KiCad Developers.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,7 +44,7 @@
 #include <math/vector2d.h>
 #include <pcb_shape.h>
 #include <progress_reporter.h>
-#include <thread_pool.h>
+#include <core/thread_pool.h>
 #include <pcb_track.h>
 #include <pad.h>
 #include <zone.h>
@@ -114,8 +114,8 @@ private:
 class POLYGON_TEST
 {
 public:
-    POLYGON_TEST( int aLimit, int aErrorLimit ) :
-        m_limit( aLimit ), m_max_error( aErrorLimit )
+    POLYGON_TEST( int aLimit ) :
+        m_limit( aLimit )
     {
     };
 
@@ -137,7 +137,7 @@ public:
             Vertex* match = nullptr;
 
             // Only run the expensive search if we don't already have a match for the point
-            if( ( all_hits.empty() || all_hits.count( p ) == 0 ) && ( match = getKink( p ) ) )
+            if( ( all_hits.empty() || all_hits.count( p ) == 0 ) && ( match = getKink( p ) ) != nullptr )
             {
                 if( !all_hits.count( match ) && m_hits.emplace( p->i, match->i ).second )
                 {
@@ -335,13 +335,13 @@ private:
         // a new fracture point, then we know that we are proceeding
         // in the wrong direction from the fracture and should
         // fall through to the next point
-        if( same_point( aPt, nz )
+        if( same_point( aPt, nz ) && same_point( aPt->next, nz->prev )
                 && aPt->y == aPt->next->y )
         {
             return nz->next;
         }
 
-        if( same_point( aPt, pz )
+        if( same_point( aPt, pz ) && same_point( aPt->next, pz->prev )
                 && aPt->y == aPt->next->y )
         {
             return pz->next;
@@ -388,14 +388,8 @@ private:
      */
     bool isSubstantial( const Vertex* aA, const Vertex* aB ) const
     {
-        // `directions` is a bitfield where
-        // bit 0 = pos y
-        // bit 1 = neg y
-        // bit 2 = pos x
-        // bit 3 = neg x
-        // So, once directions = 15, we have all directions
-        int directions = 0;
-        constexpr int all_dirs = 0b1111;
+        bool x_change = false;
+        bool y_change = false;
 
         // This is a failsafe in case of invalid lists.  Never check
         // more than the total number of points in m_vertices
@@ -405,25 +399,21 @@ private:
         const Vertex* p0 = aA;
         const Vertex* p = getNextOutlineVertex( p0 );
 
-
-        while( !same_point( p0, aB ) && checked < total_pts && directions != all_dirs )
+        while( !same_point( p, aB )             // We've reached the other inflection point
+                && !same_point( p, aA )         // We've gone around in a circle
+                && checked < total_pts          // Fail-safe for invalid lists
+                && !( x_change && y_change ) )  // We've found a substantial change in both directions
         {
             double diff_x = std::abs( p->x - p0->x );
             double diff_y = std::abs( p->y - p0->y );
 
-            // Floating point zeros can have a negative sign, so we need to
-            // ensure that only substantive diversions count for a direction
-            // change
-            if( diff_x > m_max_error )
-                directions |= ( 1 << ( 2 + std::signbit( p->x - p0->x ) ) );
+            // Check for a substantial change in the x or y direction
+            // This is measured by the set value of the minimum connection width
+            if( diff_x > m_limit )
+                x_change = true;
 
-            if( diff_y > m_max_error )
-                directions |= ( 1 << std::signbit( p->y - p0->y ) );
-
-            // In the case of a circle, we need to eventually get the direction
-            // so keep the p0 at the same point
-            if( diff_x > m_max_error || diff_y > m_max_error || p == aB )
-                p0 = p;
+            if( diff_y > m_limit )
+                y_change = true;
 
             p = getNextOutlineVertex( p );
 
@@ -432,16 +422,19 @@ private:
 
         wxCHECK_MSG( checked < total_pts, false, wxT( "Invalid polygon detected.  Missing points to check" ) );
 
-        if( directions != all_dirs )
+        if( !same_point( p, aA ) && ( !x_change || !y_change ) )
             return false;
 
-        p0 = aA;
         p = getPrevOutlineVertex( p0 );
 
-        directions = 0;
+        x_change = false;
+        y_change = false;
         checked = 0;
 
-        while( !same_point( p0, aB ) && checked < total_pts && directions != all_dirs )
+        while( !same_point( p, aB )             // We've reached the other inflection point
+                && !same_point( p, aA )         // We've gone around in a circle
+                && checked < total_pts          // Fail-safe for invalid lists
+                && !( x_change && y_change ) )  // We've found a substantial change in both directions
         {
             double diff_x = std::abs( p->x - p0->x );
             double diff_y = std::abs( p->y - p0->y );
@@ -449,16 +442,11 @@ private:
             // Floating point zeros can have a negative sign, so we need to
             // ensure that only substantive diversions count for a direction
             // change
-            if( diff_x > m_max_error )
-                directions |= ( 1 << ( 2 + std::signbit( p->x - p0->x ) ) );
+            if( diff_x > m_limit )
+                x_change = true;
 
-            if( diff_y > m_max_error )
-                directions |= ( 1 << std::signbit( p->y - p0->y ) );
-
-            // In the case of a circle, we need to eventually get the direction
-            // so keep the p0 at the same point
-            if( diff_x > m_max_error || diff_y > m_max_error || p == aB )
-                p0 = p;
+            if( diff_y > m_limit )
+                y_change = true;
 
             p = getPrevOutlineVertex( p );
 
@@ -467,7 +455,7 @@ private:
 
         wxCHECK_MSG( checked < total_pts, false, wxT( "Invalid polygon detected.  Missing points to check" ) );
 
-        return ( directions == all_dirs );
+        return ( same_point( p, aA ) || ( x_change && y_change ) );
     }
 
     /**
@@ -515,12 +503,7 @@ private:
         // z-order range for the current point ± limit bounding box
         const int32_t     maxZ = zOrder( aPt->x + m_limit, aPt->y + m_limit );
         const int32_t     minZ = zOrder( aPt->x - m_limit, aPt->y - m_limit );
-
-        // Subtract 1 to account for rounding inaccuracies in SquaredEuclideanNorm()
-        // below.  We would usually test for rounding in the final value but since we
-        // are working in squared integers here, we allow the 1nm slop rather than
-        // force a separate calculation
-        const SEG::ecoord limit2 = SEG::Square( m_limit - 1 );
+        const SEG::ecoord limit2 = SEG::Square( m_limit );
 
         // first look for points in increasing z-order
         Vertex* p = aPt->nextZ;
@@ -533,7 +516,7 @@ private:
             VECTOR2D diff( p->x - aPt->x, p->y - aPt->y );
             SEG::ecoord dist2 = diff.SquaredEuclideanNorm();
 
-            if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0.0
+            if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0
                     && locallyInside( p, aPt ) && isSubstantial( p, aPt ) && isSubstantial( aPt, p ) )
             {
                 min_dist = dist2;
@@ -551,7 +534,7 @@ private:
             VECTOR2D diff( p->x - aPt->x, p->y - aPt->y );
             SEG::ecoord dist2 = diff.SquaredEuclideanNorm();
 
-            if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0.0
+            if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0
                     && locallyInside( p, aPt ) && isSubstantial( p, aPt ) && isSubstantial( aPt, p ) )
             {
                 min_dist = dist2;
@@ -622,7 +605,6 @@ private:
 
 private:
     int                             m_limit;
-    double                          m_max_error;
     BOX2I                           m_bbox;
     std::deque<Vertex>              m_vertices;
     std::set<std::pair<int, int>>   m_hits;
@@ -643,9 +625,13 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
     if( !reportPhase( _( "Checking nets for minimum connection width..." ) ) )
         return false;   // DRC cancelled
 
-    LSET          copperLayerSet = m_drcEngine->GetBoard()->GetEnabledLayers() & LSET::AllCuMask();
-    LSEQ          copperLayers = copperLayerSet.Seq();
-    BOARD*        board = m_drcEngine->GetBoard();
+    LSET   copperLayerSet = m_drcEngine->GetBoard()->GetEnabledLayers() & LSET::AllCuMask();
+    LSEQ   copperLayers = copperLayerSet.Seq();
+    BOARD* board = m_drcEngine->GetBoard();
+    int    epsilon = board->GetDesignSettings().GetDRCEpsilon();
+
+    // A neck in a zone fill will be DRCEpsilon smaller on *each* side
+    epsilon *= 2;
 
     /*
      * Build a set of distinct minWidths specified by various DRC rules.  We'll run a test for
@@ -722,7 +708,9 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
                 if( m_drcEngine->IsCancelled() )
                     return 0;
 
-                POLYGON_TEST test( aMinWidth, m_drcEngine->GetDesignSettings()->m_MaxError );
+                int testWidth = aMinWidth - epsilon;
+
+                POLYGON_TEST test( testWidth );
 
                 for( int ii = 0; ii < aItemsPoly.Poly.OutlineCount(); ++ii )
                 {
@@ -759,8 +747,12 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
 
                         for( auto& [ zone, rtree ] : board->m_CopperZoneRTreeCache )
                         {
-                            if( !rtree->GetObjectsAt( location, aLayer, aMinWidth ).empty()
-                                    && zone->HitTestFilledArea( aLayer, location, aMinWidth ) )
+                            if( !rtree.get() )
+                                continue;
+
+                            auto obj_list = rtree->GetObjectsAt( location, aLayer, aMinWidth );
+
+                            if( !obj_list.empty() && zone->HitTestFilledArea( aLayer, location, aMinWidth ) )
                             {
                                 contributingItems.push_back( zone );
                             }
@@ -781,7 +773,7 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
 
                                 msg = formatMsg( _( "(%s minimum connection width %s; actual %s)" ),
                                                  c.GetName(),
-                                                 aMinWidth,
+                                                 c.Value().Min(),
                                                  dist );
 
                                 msg += wxS( " " ) + layerDesc( aLayer );
@@ -858,7 +850,7 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
 
         while( status != std::future_status::ready )
         {
-            m_drcEngine->ReportProgress( static_cast<double>( done ) / total_effort );
+            reportProgress( done, total_effort );
             status = ret.wait_for( std::chrono::milliseconds( 250 ) );
         }
     }
@@ -869,7 +861,12 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
     for( const auto& [ netLayer, itemsPoly ] : dataset )
     {
         for( int minWidth : distinctMinWidths )
+        {
+            if( minWidth - epsilon <= 0 )
+                continue;
+
             returns.emplace_back( tp.submit( min_checker, itemsPoly, netLayer.Layer, minWidth ) );
+        }
     }
 
     for( std::future<size_t>& ret : returns )
@@ -878,7 +875,7 @@ bool DRC_TEST_PROVIDER_CONNECTION_WIDTH::Run()
 
         while( status != std::future_status::ready )
         {
-            m_drcEngine->ReportProgress( static_cast<double>( done ) / total_effort );
+            reportProgress( done, total_effort );
             status = ret.wait_for( std::chrono::milliseconds( 250 ) );
         }
     }
