@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2022 KiCad Developers.
+ * Copyright (C) 2004-2023 KiCad Developers.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,6 +18,7 @@
  */
 
 
+#include <advanced_config.h>
 #include <board.h>
 #include <board_design_settings.h>
 #include <pcb_track.h>
@@ -122,6 +123,84 @@ static bool commonParallelProjection( SEG p, SEG n, SEG &pClip, SEG& nClip )
 }
 
 
+static bool commonParallelProjection( const PCB_ARC& p, const PCB_ARC& n, SHAPE_ARC &pClip, SHAPE_ARC& nClip )
+{
+    VECTOR2I p_center = p.GetCenter();
+    VECTOR2I n_center = n.GetCenter();
+    double p_radius = p.GetRadius();
+    double n_radius = n.GetRadius();
+
+    VECTOR2I p_start( p.GetStart() );
+    VECTOR2I p_end( p.GetEnd() );
+
+    if( !p.IsCCW() )
+        std::swap( p_start, p_end );
+
+    VECTOR2I n_start( n.GetStart() );
+    VECTOR2I n_end( n.GetEnd() );
+
+    if( !n.IsCCW() )
+        std::swap( n_start, n_end );
+
+    SHAPE_ARC p_arc( p_start, p.GetMid(), p_end, 0 );
+    SHAPE_ARC n_arc( n_start, n.GetMid(), n_end, 0 );
+
+    EDA_ANGLE p_start_angle = p_arc.GetStartAngle();
+
+    // Rotate the arcs to a common 0 starting angle
+    p_arc.Rotate( -p_start_angle, p_center );
+    n_arc.Rotate( -p_start_angle, n_center );
+
+    EDA_ANGLE p_end_angle = p_arc.GetEndAngle();
+    EDA_ANGLE n_start_angle = n_arc.GetStartAngle();
+    EDA_ANGLE n_end_angle = n_arc.GetEndAngle();
+
+
+    EDA_ANGLE clip_total_angle;
+    EDA_ANGLE clip_start_angle;
+
+    if( n_start_angle > p_end_angle )
+    {
+        // n is fully outside of p
+        if( n_end_angle > p_end_angle )
+            return false;
+
+        // n starts before angle 0 and ends in the middle of p
+        clip_total_angle = n_end_angle + p_start_angle;
+        clip_start_angle = p_start_angle;
+    }
+    else
+    {
+        clip_start_angle = n_start_angle + p_start_angle;
+
+        // n is fully inside of p
+        if( n_end_angle < p_end_angle )
+            clip_total_angle = n_end_angle - n_start_angle;
+        else // n starts after 0 and ends after p
+            clip_total_angle = p_end_angle - n_start_angle;
+    }
+
+    // One arc starts approximately where the other ends
+    if( clip_total_angle <= EDA_ANGLE( ADVANCED_CFG::GetCfg().m_MinParallelAngle ) )
+        return false;
+
+    VECTOR2I n_start_pt = n_center + VECTOR2I( KiROUND( n_radius ), 0 );
+    VECTOR2I p_start_pt = p_center + VECTOR2I( KiROUND( p_radius ), 0 );
+
+    RotatePoint( n_start_pt, n_center, clip_start_angle );
+    RotatePoint( p_start_pt, p_center, clip_start_angle );
+
+    pClip = SHAPE_ARC( p_center, p_start_pt, clip_total_angle );
+    nClip = SHAPE_ARC( n_center, n_start_pt, clip_total_angle );
+
+    if( std::abs( pClip.GetP0().x - pClip.GetArcMid().x ) < 10
+     || std::abs( nClip.GetP0().x - nClip.GetArcMid().x ) < 10 )
+        return false;
+
+    return true;
+}
+
+
 struct DIFF_PAIR_KEY
 {
     bool operator<( const DIFF_PAIR_KEY& b ) const
@@ -160,14 +239,20 @@ struct DIFF_PAIR_COUPLED_SEGMENTS
 {
     SEG          coupledN;
     SEG          coupledP;
+    bool         isArc;
+    SHAPE_ARC    coupledArcN;
+    SHAPE_ARC    coupledArcP;
     PCB_TRACK*   parentN;
     PCB_TRACK*   parentP;
-    int          computedGap;
+    int64_t      computedGap;
+    VECTOR2I     nearestN;
+    VECTOR2I     nearestP;
     PCB_LAYER_ID layer;
     bool         couplingFailMin;
     bool         couplingFailMax;
 
     DIFF_PAIR_COUPLED_SEGMENTS() :
+        isArc( false ),
         parentN( nullptr ),
         parentP( nullptr ),
         computedGap( 0 ),
@@ -193,8 +278,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
         PCB_TRACK* sp = dyn_cast<PCB_TRACK*>( itemP );
-        std::optional<DIFF_PAIR_COUPLED_SEGMENTS> bestCoupled;
-        int bestGap = std::numeric_limits<int>::max();
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
 
         if( !sp )
             continue;
@@ -214,7 +298,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 
             // Segments that are ~ 1 IU in length per side are approximately parallel (tolerance is 1 IU)
             // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
-            if( ssp.SquaredLength() > 2 && ssn.SquaredLength() > 2 && ssp.ApproxParallel(ssn) )
+            if( ssp.SquaredLength() > 2 && ssn.SquaredLength() > 2 && !ssp.Intersect( ssn, false, true ) )
             {
                 DIFF_PAIR_COUPLED_SEGMENTS cpair;
                 bool coupled = commonParallelProjection( ssp, ssn, cpair.coupledP, cpair.coupledN );
@@ -224,51 +308,127 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
                     cpair.parentP = sp;
                     cpair.parentN = sn;
                     cpair.layer = sp->GetLayer();
-
-                    int gap = (cpair.coupledP.A - cpair.coupledN.A).EuclideanNorm();
-                    if( gap < bestGap )
-                    {
-                        bestGap = gap;
-                        bestCoupled = cpair;
-                    }
+                    cpair.coupledP.NearestPoints( cpair.coupledN, cpair.nearestP, cpair.nearestN,
+                                                  cpair.computedGap );
+                    cpair.computedGap = std::sqrt( cpair.computedGap );  // NearestPoints returns squared distance
+                    cpair.computedGap -= ( sp->GetWidth() + sn->GetWidth() ) / 2;
+                    coupled_vec.push_back( cpair );
                 }
 
             }
         }
 
-        if( bestCoupled )
+        for( auto coupled : coupled_vec )
+        {
+            auto excludeSelf = [&]( BOARD_ITEM* aItem )
+            {
+                if( aItem == coupled->parentN || aItem == coupled->parentP )
+                    return false;
+
+                if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T
+                    || aItem->Type() == PCB_ARC_T )
+                {
+                    PCB_TRACK* bci = static_cast<PCB_TRACK*>( aItem );
+
+                    // Directly connected items don't count
+                    if( bci->HitTest( coupled->coupledN.A, 0 )
+                        || bci->HitTest( coupled->coupledN.B, 0 )
+                        || bci->HitTest( coupled->coupledP.A, 0 )
+                        || bci->HitTest( coupled->coupledP.B, 0 ) )
+                    {
+                        return false;
+                    }
+                }
+
+                return true;
+            };
+
+            SHAPE_SEGMENT checkSeg( coupled->nearestN, coupled->nearestP );
+            DRC_RTREE*    tree = coupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
+
+            // check if there's anything in between the segments suspected to be coupled. If
+            // there's nothing, assume they are really coupled.
+
+            if( !tree->CheckColliding( &checkSeg, sp->GetLayer(), 0, excludeSelf ) )
+            {
+                aDp.coupled.push_back( *coupled );
+            }
+        }
+    }
+
+    for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
+    {
+        PCB_ARC* sp = dyn_cast<PCB_ARC*>( itemP );
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
+
+        if( !sp )
+            continue;
+
+        for ( BOARD_CONNECTED_ITEM* itemN : aDp.itemsN )
+        {
+            PCB_ARC* sn = dyn_cast<PCB_ARC*> ( itemN );
+
+            if( !sn )
+                continue;
+
+            if( ( sn->GetLayerSet() & sp->GetLayerSet() ).none() )
+                continue;
+
+            // Segments that are ~ 1 IU in length per side are approximately parallel (tolerance is 1 IU)
+            // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
+            int64_t sqWidth = static_cast<int64_t>( sp->GetWidth() ) * sp->GetWidth();
+
+            if( sp->GetLength() > 2 && sn->GetLength() > 2 && sp->GetCenter().SquaredDistance( sn->GetCenter() ) < sqWidth )
+            {
+                DIFF_PAIR_COUPLED_SEGMENTS cpair;
+                cpair.isArc = true;
+                bool coupled = commonParallelProjection( *sp, *sn, cpair.coupledArcP, cpair.coupledArcN );
+
+                if( coupled )
+                {
+                    cpair.parentP = sp;
+                    cpair.parentN = sn;
+                    cpair.layer = sp->GetLayer();
+                    cpair.coupledArcP.NearestPoints( cpair.coupledArcN, cpair.nearestP, cpair.nearestN,
+                                                  cpair.computedGap );
+                    cpair.computedGap = std::sqrt( cpair.computedGap );  // NearestPoints returns squared distance
+                    cpair.computedGap -= ( sp->GetWidth() + sn->GetWidth() ) / 2;
+                    coupled_vec.push_back( cpair );
+                }
+            }
+        }
+
+        for( auto coupled : coupled_vec )
         {
             auto excludeSelf =
                     [&] ( BOARD_ITEM *aItem )
                     {
-                        if( aItem == bestCoupled->parentN || aItem == bestCoupled->parentP )
-                        {
+                        if( aItem == coupled->parentN || aItem == coupled->parentP )
                             return false;
-                        }
 
-                        if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T )
+                        if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T || aItem->Type() == PCB_ARC_T )
                         {
                             auto bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
 
-                            if( bci->GetNetCode() == bestCoupled->parentN->GetNetCode()
-                            ||  bci->GetNetCode() == bestCoupled->parentP->GetNetCode() )
+                            if( bci->GetNetCode() == coupled->parentN->GetNetCode()
+                            ||  bci->GetNetCode() == coupled->parentP->GetNetCode() )
+                            {
                                 return false;
+                            }
                         }
 
                         return true;
                     };
 
-            SHAPE_SEGMENT checkSegStart( bestCoupled->coupledP.A, bestCoupled->coupledN.A );
-            SHAPE_SEGMENT checkSegEnd( bestCoupled->coupledP.B, bestCoupled->coupledN.B );
-            DRC_RTREE*    tree = bestCoupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
+            SHAPE_SEGMENT checkArcMid( coupled->coupledArcN.GetArcMid(), coupled->coupledArcP.GetArcMid() );
+            DRC_RTREE*    tree = coupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
 
             // check if there's anything in between the segments suspected to be coupled. If
             // there's nothing, assume they are really coupled.
 
-            if( !tree->CheckColliding( &checkSegStart, sp->GetLayer(), 0, excludeSelf )
-                  && !tree->CheckColliding( &checkSegEnd, sp->GetLayer(), 0, excludeSelf ) )
+            if( !tree->CheckColliding( &checkArcMid, sp->GetLayer(), 0, excludeSelf ) )
             {
-                aDp.coupled.push_back( *bestCoupled );
+                aDp.coupled.push_back( *coupled );
             }
         }
     }
@@ -297,7 +457,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
                     const DRC_CONSTRAINT_T constraintsToCheck[] = {
                             DIFF_PAIR_GAP_CONSTRAINT,
-                            DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT
+                            MAX_UNCOUPLED_CONSTRAINT
                     };
 
                     for( int i = 0; i < 2; i++ )
@@ -322,7 +482,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                             key.gapRuleName = ruleName;
                             break;
 
-                        case DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT:
+                        case MAX_UNCOUPLED_CONSTRAINT:
                             key.uncoupledConstraint = constraint.GetValue();
                             key.uncoupledRule = parentRule;
                             key.uncoupledRuleName = ruleName;
@@ -373,32 +533,31 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
         drc_dbg(10, wxT( "       coupled prims : %d\n" ), (int) itemSet.coupled.size() );
 
+        std::set<BOARD_CONNECTED_ITEM*> allItems;
+
         for( BOARD_CONNECTED_ITEM* item : itemSet.itemsN )
         {
-            // fixme: include vias
-            if( PCB_TRACK* track = dyn_cast<PCB_TRACK*>( item ) )
-                itemSet.totalLengthN += track->GetLength();
+            if( PCB_TRACK* track = dynamic_cast<PCB_TRACK*>( item ) )
+            {
+                if( allItems.insert( item ).second)
+                    itemSet.totalLengthN += track->GetLength();
+            }
         }
 
         for( BOARD_CONNECTED_ITEM* item : itemSet.itemsP )
         {
-            // fixme: include vias
-            if( PCB_TRACK* track = dyn_cast<PCB_TRACK*>( item ) )
-                itemSet.totalLengthP += track->GetLength();
+            if( PCB_TRACK* track = dynamic_cast<PCB_TRACK*>( item ) )
+            {
+                if( allItems.insert( item ).second)
+                    itemSet.totalLengthP += track->GetLength();
+            }
         }
 
         for( DIFF_PAIR_COUPLED_SEGMENTS& dp : itemSet.coupled )
         {
-            int length = dp.coupledN.Length();
-            int gap = dp.coupledN.Distance( dp.coupledP );
-
+            int length = dp.isArc ? dp.coupledArcN.GetLength() : dp.coupledN.Length();
             wxCHECK2( dp.parentN && dp.parentP, continue );
 
-            gap -= dp.parentN->GetWidth() / 2;
-            gap -= dp.parentP->GetWidth() / 2;
-
-            dp.computedGap = gap;
-
             std::shared_ptr<KIGFX::VIEW_OVERLAY> overlay = m_drcEngine->GetDebugOverlay();
 
             if( overlay )
@@ -412,23 +571,23 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                 overlay->Line( dp.coupledN );
             }
 
-            drc_dbg( 10, wxT( "               len %d gap %d l %d\n" ),
+            drc_dbg( 10, wxT( "               len %d gap %ld l %d\n" ),
                      length,
-                     gap,
+                     dp.computedGap,
                      dp.parentP->GetLayer() );
 
             if( key.gapConstraint )
             {
                 if( key.gapConstraint->HasMin()
                     && key.gapConstraint->Min() >= 0
-                    && ( gap < key.gapConstraint->Min() - epsilon ) )
+                    && ( dp.computedGap < key.gapConstraint->Min() - epsilon ) )
                 {
                     dp.couplingFailMin = true;
                 }
 
                 if( key.gapConstraint->HasMax()
                     && key.gapConstraint->Max() >= 0
-                    && ( gap > key.gapConstraint->Max() + epsilon ) )
+                    && ( dp.computedGap > key.gapConstraint->Max() + epsilon ) )
                 {
                     dp.couplingFailMax = true;
                 }
@@ -443,8 +602,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                                      MessageTextFromValue( itemSet.totalCoupled ),
                                      MessageTextFromValue( totalLen ) ) );
 
-        int totalUncoupled = totalLen - itemSet.totalCoupled;
-
+        int  totalUncoupled = totalLen - itemSet.totalCoupled;
         bool uncoupledViolation = false;
 
         if( key.uncoupledConstraint && ( !itemSet.itemsP.empty() || !itemSet.itemsN.empty() ) )
@@ -497,7 +655,9 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
         {
             for( DIFF_PAIR_COUPLED_SEGMENTS& dp : itemSet.coupled )
             {
-                if( ( dp.couplingFailMin || dp.couplingFailMax ) && ( dp.parentP || dp.parentN ) )
+                wxCHECK2( dp.parentP && dp.parentN, continue );
+
+                if( ( dp.couplingFailMin || dp.couplingFailMax ) )
                 {
                     // We have a candidate violation, now we need to re-query for a constraint
                     // given the actual items, because there may be a location-based rule in play.
