@@ -21,20 +21,22 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <geometry/shape_segment.h>
 #include <footprint.h>
 #include <pad.h>
 #include <pcb_track.h>
+#include <board_design_settings.h>
 #include <drc/drc_engine.h>
 #include <drc/drc_item.h>
 #include <drc/drc_rule.h>
 #include <drc/drc_test_provider_clearance_base.h>
 
-
 /*
     Drilled hole size test. scans vias/through-hole pads and checks for min drill sizes
     Errors generated:
     - DRCE_DRILL_OUT_OF_RANGE
     - DRCE_MICROVIA_DRILL_OUT_OF_RANGE
+    - DRCE_PADSTACK
 */
 
 class DRC_TEST_PROVIDER_HOLE_SIZE : public DRC_TEST_PROVIDER
@@ -60,13 +62,9 @@ public:
         return wxT( "Tests sizes of drilled holes (via/pad drills)" );
     }
 
-    virtual std::set<DRC_CONSTRAINT_T> GetConstraintTypes() const override;
-
-    int GetNumPhases() const override;
-
 private:
-    void checkVia( PCB_VIA* via, bool aExceedMicro, bool aExceedStd );
-    void checkPad( PAD* aPad );
+    void checkViaHole( PCB_VIA* via, bool aExceedMicro, bool aExceedStd );
+    void checkPadHole( PAD* aPad );
 };
 
 
@@ -79,15 +77,10 @@ bool DRC_TEST_PROVIDER_HOLE_SIZE::Run()
 
         for( FOOTPRINT* footprint : m_drcEngine->GetBoard()->Footprints() )
         {
-            if( m_drcEngine->IsErrorLimitExceeded( DRCE_DRILL_OUT_OF_RANGE ) )
-                break;
-
             for( PAD* pad : footprint->Pads() )
             {
-                if( m_drcEngine->IsErrorLimitExceeded( DRCE_DRILL_OUT_OF_RANGE ) )
-                    break;
-
-                checkPad( pad );
+                if( !m_drcEngine->IsErrorLimitExceeded( DRCE_DRILL_OUT_OF_RANGE ) )
+                    checkPadHole( pad );
             }
         }
     }
@@ -116,18 +109,17 @@ bool DRC_TEST_PROVIDER_HOLE_SIZE::Run()
                 if( exceedMicro && exceedStd )
                     break;
 
-                checkVia( static_cast<PCB_VIA*>( track ), exceedMicro, exceedStd );
+                checkViaHole( static_cast<PCB_VIA*>( track ), exceedMicro, exceedStd );
             }
         }
     }
 
     reportRuleStatistics();
 
-    return true;
+    return !m_drcEngine->IsCancelled();
 }
 
-
-void DRC_TEST_PROVIDER_HOLE_SIZE::checkPad( PAD* aPad )
+void DRC_TEST_PROVIDER_HOLE_SIZE::checkPadHole( PAD* aPad )
 {
     int holeMinor = std::min( aPad->GetDrillSize().x, aPad->GetDrillSize().y );
     int holeMajor = std::max( aPad->GetDrillSize().x, aPad->GetDrillSize().y );
@@ -139,7 +131,10 @@ void DRC_TEST_PROVIDER_HOLE_SIZE::checkPad( PAD* aPad )
                                               UNDEFINED_LAYER /* holes are not layer-specific */ );
     bool fail_min = false;
     bool fail_max = false;
-    int  constraintValue;
+    int  constraintValue = 0;
+
+    if( constraint.GetSeverity() == RPT_SEVERITY_IGNORE )
+        return;
 
     if( constraint.Value().HasMin() && holeMinor < constraint.Value().Min() )
     {
@@ -156,33 +151,37 @@ void DRC_TEST_PROVIDER_HOLE_SIZE::checkPad( PAD* aPad )
     if( fail_min || fail_max )
     {
         std::shared_ptr<DRC_ITEM> drcItem = DRC_ITEM::Create( DRCE_DRILL_OUT_OF_RANGE );
+        wxString constraintName = constraint.GetName();
         wxString msg;
 
         if( fail_min )
         {
-            msg.Printf( _( "(%s min width %s; actual %s)" ),
-                          constraint.GetName(),
-                          MessageTextFromValue( userUnits(), constraintValue ),
-                          MessageTextFromValue( userUnits(), holeMinor ) );
+            if( constraint.GetParentRule() && constraint.GetParentRule()->m_Implicit )
+                constraintName = _( "board setup constraints" );
+
+            msg = formatMsg( _( "(%s min hole %s; actual %s)" ),
+                             constraintName,
+                             constraintValue,
+                             holeMinor );
         }
         else
         {
-            msg.Printf( _( "(%s max width %s; actual %s)" ),
-                          constraint.GetName(),
-                          MessageTextFromValue( userUnits(), constraintValue ),
-                          MessageTextFromValue( userUnits(), holeMajor ) );
+            msg = formatMsg( _( "(%s max hole %s; actual %s)" ),
+                             constraintName,
+                             constraintValue,
+                             holeMajor );
         }
 
         drcItem->SetErrorMessage( drcItem->GetErrorText() + wxS( " " ) + msg );
         drcItem->SetItems( aPad );
         drcItem->SetViolatingRule( constraint.GetParentRule() );
 
-        reportViolation( drcItem, aPad->GetPosition() );
+        reportViolation( drcItem, aPad->GetPosition(), UNDEFINED_LAYER );
     }
 }
 
 
-void DRC_TEST_PROVIDER_HOLE_SIZE::checkVia( PCB_VIA* via, bool aExceedMicro, bool aExceedStd )
+void DRC_TEST_PROVIDER_HOLE_SIZE::checkViaHole( PCB_VIA* via, bool aExceedMicro, bool aExceedStd )
 {
     int errorCode;
 
@@ -205,7 +204,10 @@ void DRC_TEST_PROVIDER_HOLE_SIZE::checkVia( PCB_VIA* via, bool aExceedMicro, boo
                                               UNDEFINED_LAYER /* holes are not layer-specific */ );
     bool fail_min = false;
     bool fail_max = false;
-    int  constraintValue;
+    int  constraintValue = 0;
+
+    if( constraint.GetSeverity() == RPT_SEVERITY_IGNORE )
+        return;
 
     if( constraint.Value().HasMin() && via->GetDrillValue() < constraint.Value().Min() )
     {
@@ -222,44 +224,36 @@ void DRC_TEST_PROVIDER_HOLE_SIZE::checkVia( PCB_VIA* via, bool aExceedMicro, boo
     if( fail_min || fail_max )
     {
         std::shared_ptr<DRC_ITEM> drcItem = DRC_ITEM::Create( errorCode );
+        wxString constraintName = constraint.GetName();
         wxString msg;
 
         if( fail_min )
         {
-            msg.Printf( _( "(%s min width %s; actual %s)" ),
-                          constraint.GetName(),
-                          MessageTextFromValue( userUnits(), constraintValue ),
-                          MessageTextFromValue( userUnits(), via->GetDrillValue() ) );
+            if( constraint.m_ImplicitMin )
+                constraintName = _( "board setup constraints" );
+
+            msg = formatMsg( _( "(%s min hole %s; actual %s)" ),
+                             constraintName,
+                             constraintValue,
+                             via->GetDrillValue() );
         }
         else
         {
-            msg.Printf( _( "(%s max width %s; actual %s)" ),
-                          constraint.GetName(),
-                          MessageTextFromValue( userUnits(), constraintValue ),
-                          MessageTextFromValue( userUnits(), via->GetDrillValue() ) );
+            msg = formatMsg( _( "(%s max hole %s; actual %s)" ),
+                             constraintName,
+                             constraintValue,
+                             via->GetDrillValue() );
         }
 
         drcItem->SetErrorMessage( drcItem->GetErrorText() + wxS( " " ) + msg );
         drcItem->SetItems( via );
         drcItem->SetViolatingRule( constraint.GetParentRule() );
 
-        reportViolation( drcItem, via->GetPosition() );
+        reportViolation( drcItem, via->GetPosition(), UNDEFINED_LAYER );
     }
 }
 
 
-int DRC_TEST_PROVIDER_HOLE_SIZE::GetNumPhases() const
-{
-    return 2;
-}
-
-
-std::set<DRC_CONSTRAINT_T> DRC_TEST_PROVIDER_HOLE_SIZE::GetConstraintTypes() const
-{
-    return { HOLE_SIZE_CONSTRAINT };
-}
-
-
 namespace detail
 {
 static DRC_REGISTER_TEST_PROVIDER<DRC_TEST_PROVIDER_HOLE_SIZE> dummy;
