@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013  Cirilo Bernardo
- * Copyright (C) 2018-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2018-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,10 +31,11 @@
 #include <board_design_settings.h>
 #include <footprint.h>
 #include <fp_lib_table.h>
-#include <fp_shape.h>
 #include <idf_parser.h>
 #include <pad.h>
+#include <pcb_shape.h>
 #include <build_version.h>
+#include <project_pcb.h>
 #include <wx/msgdlg.h>
 #include "project.h"
 #include "kiway.h"
@@ -59,12 +60,9 @@ static FILENAME_RESOLVER* resolver;
 static void idf_export_outline( BOARD* aPcb, IDF3_BOARD& aIDFBoard )
 {
     double scale = aIDFBoard.GetUserScale();
-
-    PCB_SHAPE* graphic;                 // KiCad graphical item
     IDF_POINT sp, ep;                   // start and end points from KiCad item
-
     std::list< IDF_SEGMENT* > lines;    // IDF intermediate form of KiCad graphical item
-    IDF_OUTLINE* outline = nullptr;        // graphical items forming an outline or cutout
+    IDF_OUTLINE* outline = nullptr;     // graphical items forming an outline or cutout
 
     // NOTE: IMPLEMENTATION
     // If/when component cutouts are allowed, we must implement them separately. Cutouts
@@ -80,7 +78,7 @@ static void idf_export_outline( BOARD* aPcb, IDF3_BOARD& aIDFBoard )
         if( item->Type() != PCB_SHAPE_T || item->GetLayer() != Edge_Cuts )
             continue;
 
-        graphic = (PCB_SHAPE*) item;
+        PCB_SHAPE* graphic = static_cast<PCB_SHAPE*>( item );
 
         switch( graphic->GetShape() )
         {
@@ -101,7 +99,7 @@ static void idf_export_outline( BOARD* aPcb, IDF3_BOARD& aIDFBoard )
             break;
         }
 
-        case SHAPE_T::RECT:
+        case SHAPE_T::RECTANGLE:
         {
             if( graphic->GetStart() == graphic->GetEnd() )
                 break;
@@ -274,7 +272,7 @@ UseBoundingBox:
 static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD& aIDFBoard )
 {
     // Reference Designator
-    std::string crefdes = TO_UTF8( aFootprint->Reference().GetShownText() );
+    std::string crefdes = TO_UTF8( aFootprint->Reference().GetShownText( false ) );
 
     wxString libraryName = aFootprint->GetFPID().GetLibNickname();
     wxString footprintBasePath = wxEmptyString;
@@ -285,7 +283,7 @@ static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD
 
         try
         {
-            fpRow = aPcb->GetProject()->PcbFootprintLibs()->FindRow( libraryName, false );
+            fpRow = PROJECT_PCB::PcbFootprintLibs( aPcb->GetProject() )->FindRow( libraryName, false );
         }
         catch( ... )
         {
@@ -298,7 +296,7 @@ static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD
 
     if( crefdes.empty() || !crefdes.compare( "~" ) )
     {
-        std::string cvalue = TO_UTF8( aFootprint->Value().GetShownText() );
+        std::string cvalue = TO_UTF8( aFootprint->Value().GetShownText( false ) );
 
         // if both the RefDes and Value are empty or set to '~' the board owns the part,
         // otherwise associated parts of the footprint must be marked NOREFDES.
@@ -311,7 +309,7 @@ static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD
     // TODO: If footprint cutouts are supported we must add code here
     // for( EDA_ITEM* item = aFootprint->GraphicalItems();  item != NULL;  item = item->Next() )
     // {
-    //     if( item->Type() != PCB_FP_SHAPE_T || item->GetLayer() != Edge_Cuts )
+    //     if( item->Type() != PCB_SHAPE_T || item->GetLayer() != Edge_Cuts )
     //         continue;
     //     code to export cutouts
     // }
@@ -443,7 +441,7 @@ static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD
 
         if( refdes.empty() )
         {
-            refdes = TO_UTF8( aFootprint->Reference().GetShownText() );
+            refdes = TO_UTF8( aFootprint->Reference().GetShownText( false ) );
 
             // NOREFDES cannot be used or else the software gets confused
             // when writing out the placement data due to conflicting
@@ -461,9 +459,9 @@ static void idf_export_footprint( BOARD* aPcb, FOOTPRINT* aFootprint, IDF3_BOARD
             throw( std::runtime_error( aIDFBoard.GetError() ) );
 
         double rotz = aFootprint->GetOrientation().AsDegrees();
-        double locx = sM->m_Offset.x * 25.4;  // part offsets are in inches
-        double locy = sM->m_Offset.y * 25.4;
-        double locz = sM->m_Offset.z * 25.4;
+        double locx = sM->m_Offset.x;  // part offsets are in mm
+        double locy = sM->m_Offset.y;
+        double locz = sM->m_Offset.z;
         double lrot = sM->m_Rotation.z;
 
         bool top = ( aFootprint->GetLayer() == B_Cu ) ? false : true;
@@ -608,7 +606,7 @@ bool PCB_EDIT_FRAME::Export_IDF3( BOARD* aPcb, const wxString& aFullFileName,
     // Switch the locale to standard C (needed to print floating point numbers)
     LOCALE_IO toggle;
 
-    resolver = Prj().Get3DCacheManager()->GetResolver();
+    resolver = PROJECT_PCB::Get3DCacheManager( &Prj() )->GetResolver();
 
     bool ok = true;
     double scale = pcbIUScale.MM_PER_IU;   // we must scale internal units to mm for IDF
@@ -652,7 +650,7 @@ bool PCB_EDIT_FRAME::Export_IDF3( BOARD* aPcb, const wxString& aFullFileName,
         if( !idfBoard.WriteFile( aFullFileName, idfUnit, false ) )
         {
             wxString msg;
-            msg << _( "IDF Export Failed:\n" ) << FROM_UTF8( idfBoard.GetError().c_str() );
+            msg << _( "IDF Export Failed:\n" ) << From_UTF8( idfBoard.GetError().c_str() );
             wxMessageBox( msg );
 
             ok = false;
@@ -669,7 +667,7 @@ bool PCB_EDIT_FRAME::Export_IDF3( BOARD* aPcb, const wxString& aFullFileName,
     catch( const std::exception& e )
     {
         wxString msg;
-        msg << _( "IDF Export Failed:\n" ) << FROM_UTF8( e.what() );
+        msg << _( "IDF Export Failed:\n" ) << From_UTF8( e.what() );
         wxMessageBox( msg );
         ok = false;
     }
