@@ -31,9 +31,11 @@
 #include <lib_id.h>
 #include <progress_reporter.h>
 #include <string_utils.h>
-#include <thread_pool.h>
+#include <core/thread_pool.h>
 #include <wildcards_and_files_ext.h>
 
+#include <kiplatform/io.h>
+
 #include <wx/textfile.h>
 #include <wx/txtstrm.h>
 #include <wx/wfstream.h>
@@ -57,13 +59,20 @@ void FOOTPRINT_INFO_IMPL::load()
         m_pad_count = footprint->GetPadCount( DO_NOT_INCLUDE_NPTH );
         m_unique_pad_count = footprint->GetUniquePadCount( DO_NOT_INCLUDE_NPTH );
         m_keywords = footprint->GetKeywords();
-        m_doc = footprint->GetDescription();
+        m_doc = footprint->GetLibDescription();
     }
 
     m_loaded = true;
 }
 
 
+void FOOTPRINT_LIST_IMPL::Clear()
+{
+    m_list.clear();
+    m_list_timestamp = 0;
+}
+
+
 bool FOOTPRINT_LIST_IMPL::CatchErrors( const std::function<void()>& aFunc )
 {
     try
@@ -331,6 +340,9 @@ void FOOTPRINT_LIST_IMPL::WriteCacheToFile( const wxString& aFilePath )
     txtStream.Flush();
     outStream.Close();
 
+    // Preserve the permissions of the current file
+    KIPLATFORM::IO::DuplicatePermissions( aFilePath, tmpFileName.GetFullPath() );
+
     if( !wxRenameFile( tmpFileName.GetFullPath(), aFilePath, true ) )
     {
         // cleanup in case rename failed
