@@ -38,11 +38,14 @@
 #include <netlist_reader/pcb_netlist.h>
 #include <widgets/msgpanel.h>
 #include <widgets/wx_listbox.h>
+#include <widgets/wx_aui_utils.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <pcb_draw_panel_gal.h>
 #include <pcb_painter.h>
 #include <pcbnew_id.h>
 #include <footprint_editor_settings.h>
 #include <pgm_base.h>
+#include <project_pcb.h>
 #include <project/project_file.h>
 #include <settings/settings_manager.h>
 #include <tool/action_toolbar.h>
@@ -64,13 +67,17 @@
 #include <wx/srchctrl.h>
 #include <wx/tokenzr.h>
 #include <wx/choice.h>
+#include <wx/hyperlink.h>
+
+#include "invoke_pcb_dialog.h"
 
 using namespace std::placeholders;
 
 
-#define NEXT_PART       1
 #define NEW_PART        0
-#define PREVIOUS_PART   -1
+#define NEXT_PART       1
+#define PREVIOUS_PART   2
+#define RELOAD_PART     3
 
 
 BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, PCB_BASE_FRAME )
@@ -87,7 +94,6 @@ BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, PCB_BASE_FRAME )
     EVT_TOOL( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB )
     EVT_CHOICE( ID_ON_ZOOM_SELECT, FOOTPRINT_VIEWER_FRAME::OnSelectZoom )
     EVT_CHOICE( ID_ON_GRID_SELECT, FOOTPRINT_VIEWER_FRAME::OnSelectGrid )
-    EVT_MENU( ID_GRID_SETTINGS, FOOTPRINT_VIEWER_FRAME::OnGridSettings )
 
     EVT_UPDATE_UI( ID_ADD_FOOTPRINT_TO_BOARD, FOOTPRINT_VIEWER_FRAME::OnUpdateFootprintButton )
 
@@ -101,30 +107,13 @@ BEGIN_EVENT_TABLE( FOOTPRINT_VIEWER_FRAME, PCB_BASE_FRAME )
 END_EVENT_TABLE()
 
 
-/*
- * Note: FOOTPRINT_VIEWER_FRAME can be created in "modal mode", or as a usual frame.
- */
-#define PARENT_STYLE   ( KICAD_DEFAULT_DRAWFRAME_STYLE | wxFRAME_FLOAT_ON_PARENT )
-#define MODAL_STYLE    ( KICAD_DEFAULT_DRAWFRAME_STYLE | wxSTAY_ON_TOP )
-#define NONMODAL_STYLE ( KICAD_DEFAULT_DRAWFRAME_STYLE )
-
-
-FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent,
-                                                FRAME_T aFrameType ) :
-    PCB_BASE_FRAME( aKiway, aParent, aFrameType, _( "Footprint Library Browser" ),
-                    wxDefaultPosition, wxDefaultSize,
-                    aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? ( aParent ? PARENT_STYLE : MODAL_STYLE )
-                                                               : NONMODAL_STYLE,
-                    aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL ? FOOTPRINT_VIEWER_FRAME_NAME_MODAL
-                                                               : FOOTPRINT_VIEWER_FRAME_NAME ),
+FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
+    PCB_BASE_FRAME( aKiway, aParent, FRAME_FOOTPRINT_VIEWER, _( "Footprint Library Browser" ),
+                    wxDefaultPosition, wxDefaultSize, KICAD_DEFAULT_DRAWFRAME_STYLE,
+                    FOOTPRINT_VIEWER_FRAME_NAME ),
    m_comp( LIB_ID(), wxEmptyString, wxEmptyString, KIID_PATH(), {} )
 {
-    wxASSERT( aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL || aFrameType == FRAME_FOOTPRINT_VIEWER );
-
-    if( aFrameType == FRAME_FOOTPRINT_VIEWER_MODAL )
-        SetModal( true );
-
-    m_aboutTitle = _HKI( "KiCad Footprint Library Viewer" );
+    m_aboutTitle = _HKI( "KiCad Footprint Library Browser" );
 
     // Force the items to always snap
     m_magneticItems.pads     = MAGNETIC_OPTIONS::CAPTURE_ALWAYS;
@@ -181,7 +170,9 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     m_fpList = new WX_LISTBOX( fpPanel, ID_MODVIEW_FOOTPRINT_LIST, wxDefaultPosition, wxDefaultSize,
                                0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
 
-    m_fpList->Connect( wxEVT_LEFT_DCLICK, wxMouseEventHandler( FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList ), nullptr, this );
+    m_fpList->Connect( wxEVT_LEFT_DCLICK,
+                       wxMouseEventHandler( FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList ),
+                       nullptr, this );
     fpSizer->Add( m_fpList, 1, wxEXPAND, 5 );
 
     fpPanel->SetSizer( fpSizer );
@@ -260,7 +251,10 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
         FOOTPRINT* footprint = loadFootprint( id );
 
         if( footprint )
+        {
             GetBoard()->Add( footprint );
+            setFPWatcher( footprint );
+        }
     }
 
     drawPanel->DisplayBoard( m_pcb );
@@ -284,42 +278,10 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     m_auimgr.Update();
 
     if( m_libListWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Libraries" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_libListWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 100, -1 );
-    }
+        SetAuiPaneSize( m_auimgr, m_auimgr.GetPane( "Libraries" ), m_libListWidth, -1 );
 
     if( m_fpListWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Footprints" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_fpListWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 100, -1 );
-    }
+        SetAuiPaneSize( m_auimgr, m_auimgr.GetPane( "Footprints" ), m_fpListWidth, -1 );
 
     // The canvas should not steal the focus from the list boxes
     GetCanvas()->SetCanFocus( false );
@@ -341,12 +303,9 @@ FOOTPRINT_VIEWER_FRAME::FOOTPRINT_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent
     updateView();
     setupUnits( config() );
 
-    if( !IsModal() )        // For modal mode, calling ShowModal() will show this frame
-    {
-        ReCreateFootprintList();
-        Raise();            // On some window managers, this is needed
-        Show( true );
-    }
+    ReCreateFootprintList();
+    Raise();            // On some window managers, this is needed
+    Show( true );
 }
 
 
@@ -360,7 +319,10 @@ FOOTPRINT_VIEWER_FRAME::~FOOTPRINT_VIEWER_FRAME()
     GetCanvas()->GetView()->Clear();
     // Be sure any event cannot be fired after frame deletion:
     GetCanvas()->SetEvtHandlerEnabled( false );
-    m_fpList->Disconnect( wxEVT_LEFT_DCLICK, wxMouseEventHandler( FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList ), nullptr, this );
+    m_fpList->Disconnect( wxEVT_LEFT_DCLICK,
+                          wxMouseEventHandler( FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList ),
+                          nullptr, this );
+    setFPWatcher( nullptr );
 }
 
 
@@ -374,9 +336,7 @@ void FOOTPRINT_VIEWER_FRAME::UpdateMsgPanel()
 {
     EDA_DRAW_FRAME::UpdateMsgPanel();
 
-    FOOTPRINT* fp = static_cast<FOOTPRINT*>( GetModel() );
-
-    if( fp )
+    if( FOOTPRINT* fp = static_cast<FOOTPRINT*>( GetModel() ) )
     {
         std::vector<MSG_PANEL_ITEM> msgItems;
         fp->GetMsgPanelInfo( this, msgItems );
@@ -431,19 +391,7 @@ void FOOTPRINT_VIEWER_FRAME::doCloseWindow()
 
     GetCanvas()->StopDrawing();
 
-    if( IsModal() )
-    {
-        // Only dismiss a modal frame once, so that the return values set by
-        // the prior DismissModal() are not bashed for ShowModal().
-        if( !IsDismissed() )
-            DismissModal( false );
-
-        // window to be destroyed by the caller of KIWAY_PLAYER::ShowModal()
-    }
-    else
-    {
-        Destroy();
-    }
+    Destroy();
 }
 
 
@@ -462,7 +410,7 @@ void FOOTPRINT_VIEWER_FRAME::ReCreateLibraryList()
 
     COMMON_SETTINGS*      cfg = Pgm().GetCommonSettings();
     PROJECT_FILE&         project = Kiway().Prj().GetProjectFile();
-    std::vector<wxString> nicknames = Prj().PcbFootprintLibs()->GetLogicalLibs();
+    std::vector<wxString> nicknames = PROJECT_PCB::PcbFootprintLibs( &Prj() )->GetLogicalLibs();
     std::vector<wxString> pinnedMatches;
     std::vector<wxString> otherMatches;
 
@@ -547,7 +495,7 @@ void FOOTPRINT_VIEWER_FRAME::ReCreateFootprintList()
 
     wxString nickname = getCurNickname();
 
-    fp_info_list->ReadFootprintFiles( Prj().PcbFootprintLibs(), !nickname ? nullptr : &nickname );
+    fp_info_list->ReadFootprintFiles( PROJECT_PCB::PcbFootprintLibs( &Prj() ), !nickname ? nullptr : &nickname );
 
     if( fp_info_list->GetErrorCount() )
     {
@@ -567,16 +515,16 @@ void FOOTPRINT_VIEWER_FRAME::ReCreateFootprintList()
 
         while( tokenizer.HasMoreTokens() )
         {
-            const wxString       term = tokenizer.GetNextToken().Lower();
-            EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
+            const wxString       filterTerm = tokenizer.GetNextToken().Lower();
+            EDA_COMBINED_MATCHER matcher( filterTerm, CTX_LIBITEM );
 
             for( const std::unique_ptr<FOOTPRINT_INFO>& footprint : fp_info_list->GetList() )
             {
-                wxString search = footprint->GetFootprintName() + wxS( " " ) + footprint->GetSearchText();
-                bool     matched = matcher.Find( search.Lower() );
+                std::vector<SEARCH_TERM> searchTerms = footprint->GetSearchTerms();
+                int                      matched = matcher.ScoreTerms( searchTerms );
 
-                if( !matched && term.IsNumber() )
-                    matched = ( wxAtoi( term ) == (int)footprint->GetPadCount() );
+                if( filterTerm.IsNumber() && wxAtoi( filterTerm ) == (int)footprint->GetPadCount() )
+                    matched++;
 
                 if( !matched )
                     excludes.insert( footprint->GetFootprintName() );
@@ -730,7 +678,7 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnLibList( wxCommandEvent& aEvent )
 
     setCurNickname( name );
 
-    // Ensure the displayed footprint is loaded/reloaded from the new library
+    // Ensure the displayed footprint is loade/reloaded from the new library
     setCurFootprintName( wxEmptyString );
 
     ReCreateFootprintList();
@@ -753,42 +701,7 @@ void FOOTPRINT_VIEWER_FRAME::ClickOnFootprintList( wxCommandEvent& aEvent )
     if( getCurFootprintName().CmpNoCase( name ) != 0 )
     {
         setCurFootprintName( name );
-
-        // Delete the current footprint (MUST reset tools first)
-        GetToolManager()->ResetTools( TOOL_BASE::MODEL_RELOAD );
-
-        GetBoard()->DeleteAllFootprints();
-        GetBoard()->GetNetInfo().RemoveUnusedNets();
-
-        LIB_ID id;
-        id.SetLibNickname( getCurNickname() );
-        id.SetLibItemName( getCurFootprintName() );
-
-        FOOTPRINT* footprint = nullptr;
-
-        try
-        {
-            footprint = loadFootprint( id );
-        }
-        catch( const IO_ERROR& ioe )
-        {
-            wxString msg = wxString::Format( _( "Could not load footprint '%s' from library '%s'."
-                                             "\n\n%s" ),
-                                             getCurFootprintName(),
-                                             getCurNickname(),
-                                             ioe.Problem() );
-            DisplayError( this, msg );
-        }
-
-        if( footprint )
-            displayFootprint( footprint );
-
-        UpdateTitle();
-
-        updateView();
-
-        GetCanvas()->Refresh();
-        Update3DView( true, true );
+        SelectAndViewFootprint( NEW_PART );
     }
 }
 
@@ -821,19 +734,7 @@ void FOOTPRINT_VIEWER_FRAME::DClickOnFootprintList( wxMouseEvent& aEvent )
 
 void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
 {
-    if( IsModal() )
-    {
-        if( m_fpList->GetSelection() >= 0 )
-        {
-            LIB_ID fpid( getCurNickname(), m_fpList->GetStringSelection() );
-            DismissModal( true, fpid.Format() );
-        }
-        else
-        {
-            DismissModal( false );
-        }
-    }
-    else if( GetBoard()->GetFirstFootprint() )
+    if( GetBoard()->GetFirstFootprint() )
     {
         PCB_EDIT_FRAME*  pcbframe = (PCB_EDIT_FRAME*) Kiway().Player( FRAME_PCB_EDITOR, false );
         PCBNEW_SETTINGS* cfg = pcbframe->GetPcbNewSettings();
@@ -857,7 +758,7 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
         if( blocking_dialog )
             blocking_dialog->Close( true );
 
-        toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+        toolMgr->RunAction( PCB_ACTIONS::selectionClear );
         BOARD_COMMIT commit( pcbframe );
 
         // Create the "new" footprint
@@ -887,12 +788,12 @@ void FOOTPRINT_VIEWER_FRAME::AddFootprintToPCB( wxCommandEvent& aEvent )
         viewControls->SetCrossHairCursorPosition( VECTOR2D( 0, 0 ), false );
         pcbframe->PlaceFootprint( newFootprint );
 
-        newFootprint->SetPosition( wxPoint( 0, 0 ) );
+        newFootprint->SetPosition( VECTOR2I( 0, 0 ) );
         viewControls->SetCrossHairCursorPosition( cursorPos, false );
         commit.Push( wxT( "Insert footprint" ) );
 
         pcbframe->Raise();
-        toolMgr->RunAction( PCB_ACTIONS::placeFootprint, false, newFootprint );
+        toolMgr->PostAction( PCB_ACTIONS::placeFootprint, newFootprint );
 
         newFootprint->ClearFlags();
     }
@@ -910,7 +811,9 @@ void FOOTPRINT_VIEWER_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
     PCB_BASE_FRAME::LoadSettings( aCfg );
 
     // Fetch display and grid settings from Footprint Editor
-    auto fpedit = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    SETTINGS_MANAGER&          mgr = Pgm().GetSettingsManager();
+    FOOTPRINT_EDITOR_SETTINGS* fpedit = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
+
     m_displayOptions = fpedit->m_Display;
     GetGalDisplayOptions().ReadWindowSettings( fpedit->m_Window );
 
@@ -960,12 +863,13 @@ WINDOW_SETTINGS* FOOTPRINT_VIEWER_FRAME::GetWindowSettings( APP_SETTINGS_BASE* a
 
 COLOR_SETTINGS* FOOTPRINT_VIEWER_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
-    auto* settings = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    SETTINGS_MANAGER&          mgr = Pgm().GetSettingsManager();
+    FOOTPRINT_EDITOR_SETTINGS* cfg = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
 
-    if( settings )
-        return Pgm().GetSettingsManager().GetColorSettings( settings->m_ColorTheme );
+    if( cfg )
+        return mgr.GetColorSettings( cfg->m_ColorTheme );
     else
-        return Pgm().GetSettingsManager().GetColorSettings();
+        return mgr.GetColorSettings();
 }
 
 
@@ -1009,7 +913,7 @@ void FOOTPRINT_VIEWER_FRAME::OnActivate( wxActivateEvent& event )
     if( event.GetActive() )
     {
         // Ensure we have the right library list:
-        std::vector< wxString > libNicknames = Prj().PcbFootprintLibs()->GetLogicalLibs();
+        std::vector< wxString > libNicknames = PROJECT_PCB::PcbFootprintLibs( &Prj() )->GetLogicalLibs();
         bool                    stale = false;
 
         if( libNicknames.size() != m_libList->GetCount() )
@@ -1043,104 +947,32 @@ void FOOTPRINT_VIEWER_FRAME::OnUpdateFootprintButton( wxUpdateUIEvent& aEvent )
 }
 
 
-void FOOTPRINT_VIEWER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
+void FOOTPRINT_VIEWER_FRAME::ReloadFootprint( FOOTPRINT* aFootprint )
 {
-    const std::string& payload = mail.GetPayload();
-
-    switch( mail.Command() )
-    {
-    case MAIL_SYMBOL_NETLIST:
-    {
-        /*
-         * Symbol netlist format:
-         *   library:footprint
-         *   reference
-         *   value
-         *   pinName,netName,pinFunction,pinType
-         *   pinName,netName,pinFunction,pinType
-         *   ...
-         */
-        std::vector<std::string> strings = split( payload, "\r" );
-        LIB_ID libid;
-
-        if( strings.size() >= 3 )
-        {
-            libid.Parse( strings[0] );
-
-            m_comp.SetFPID( libid );
-            m_comp.SetReference( strings[1] );
-            m_comp.SetValue( strings[2] );
-
-            m_comp.ClearNets();
-
-            for( size_t ii = 3; ii < strings.size(); ++ii )
-            {
-                std::vector<std::string> pinData = split( strings[ii], "," );
-                m_comp.AddNet( pinData[0], pinData[1], pinData[2], pinData[3] );
-            }
-        }
-
-        break;
-    }
-    case MAIL_RELOAD_LIB:
-    {
-        ReCreateLibraryList();
-        break;
-    }
-
-    default:
-        ;
-    }
+    setCurNickname( aFootprint->GetFPID().GetLibNickname() );
+    setCurFootprintName( aFootprint->GetFPID().GetLibItemName() );
+    SelectAndViewFootprint( RELOAD_PART );
 }
 
 
-bool FOOTPRINT_VIEWER_FRAME::ShowModal( wxString* aFootprint, wxWindow* aParent )
+void FOOTPRINT_VIEWER_FRAME::HardRedraw()
 {
-    if( aFootprint && !aFootprint->IsEmpty() )
-    {
-        wxString msg;
-        LIB_TABLE* fpTable = Prj().PcbFootprintLibs();
-        LIB_ID fpid;
-
-        fpid.Parse( *aFootprint, true );
-
-        if( fpid.IsValid() )
-        {
-            wxString libraryName = fpid.GetLibNickname();
-
-            if( !fpTable->HasLibrary( fpid.GetLibNickname(), false ) )
-            {
-                msg.sprintf( _( "The current configuration does not include library '%s'.  Use "
-                                "Manage Footprint Libraries to edit the configuration." ),
-                             libraryName );
-                DisplayErrorMessage( aParent, _( "Footprint library not found." ), msg );
-            }
-            else if ( !fpTable->HasLibrary( fpid.GetLibNickname(), true ) )
-            {
-                msg.sprintf( _( "Library '%s' is not enabled in the current configuration.  Use "
-                                "Manage Footprint Libraries to edit the configuration." ),
-                             libraryName );
-                DisplayErrorMessage( aParent, _( "Footprint library not enabled." ), msg );
-            }
-            else
-            {
-                // Update last selection:
-                setCurNickname( libraryName );
-                setCurFootprintName( fpid.GetLibItemName() );
-                m_libList->SetStringSelection( libraryName );
-            }
-        }
-    }
-
-    // Rebuild the fp list from the last selected library,
-    // and show the last selected footprint
+    ReCreateLibraryList();
     ReCreateFootprintList();
-    SelectAndViewFootprint( NEW_PART );
+    ReloadFootprint( GetBoard()->GetFirstFootprint() );
+}
 
-    bool retval = KIWAY_PLAYER::ShowModal( aFootprint, aParent );
+void FOOTPRINT_VIEWER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
+{
+    switch( mail.Command() )
+    {
+    case MAIL_RELOAD_LIB:
+        ReCreateLibraryList();
+        break;
 
-    m_libFilter->SetFocus();
-    return retval;
+    default:
+        break;
+    }
 }
 
 
@@ -1184,7 +1016,7 @@ void FOOTPRINT_VIEWER_FRAME::UpdateTitle()
     {
         try
         {
-            FP_LIB_TABLE* libtable = Prj().PcbFootprintLibs();
+            FP_LIB_TABLE* libtable = PROJECT_PCB::PcbFootprintLibs( &Prj() );
             const LIB_TABLE_ROW* row = libtable->FindRow( getCurNickname() );
 
             title = getCurNickname() + wxT( " \u2014 " ) + row->GetFullURI( true );
@@ -1236,14 +1068,17 @@ void FOOTPRINT_VIEWER_FRAME::SelectAndViewFootprint( int aMode )
             selectionTool->ClearSelection( true /* quiet mode */ );
 
         GetBoard()->DeleteAllFootprints();
-        GetBoard()->GetNetInfo().RemoveUnusedNets();
+        GetBoard()->RemoveUnusedNets( nullptr );
 
-        FOOTPRINT* footprint = Prj().PcbFootprintLibs()->FootprintLoad( getCurNickname(),
+        FOOTPRINT* footprint = PROJECT_PCB::PcbFootprintLibs( &Prj() )->FootprintLoad( getCurNickname(),
                                                                         getCurFootprintName() );
 
         if( footprint )
             displayFootprint( footprint );
 
+        if( aMode != RELOAD_PART )
+            setFPWatcher( footprint );
+
         Update3DView( true, true );
         updateView();
     }
@@ -1264,9 +1099,9 @@ void FOOTPRINT_VIEWER_FRAME::updateView()
     wxAuiToolBarItem* toolOpt = m_mainToolBar->FindTool( ID_FPVIEWER_AUTOZOOM_TOOL );
 
     if( toolOpt->GetState() & wxAUI_BUTTON_STATE_CHECKED )
-        m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+        m_toolManager->RunAction( ACTIONS::zoomFitScreen );
     else
-        m_toolManager->RunAction( ACTIONS::centerContents, true );
+        m_toolManager->RunAction( ACTIONS::centerContents );
 
     UpdateMsgPanel();
 }
