@@ -6,7 +6,7 @@
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
  *
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@
 
 #include <base_units.h>
 #include <board.h>
+#include <board_design_settings.h>
 #include <netinfo.h>
 #include <footprint.h>
 #include <pad.h>
@@ -120,10 +121,10 @@ VECTOR2I BOARD_NETLIST_UPDATER::estimateFootprintInsertionPosition()
     else
     {
         // Position new components in the center of the page when the board is empty.
-        wxSize pageSize = m_board->GetPageSettings().GetSizeIU( pcbIUScale.IU_PER_MILS );
+        VECTOR2I pageSize = m_board->GetPageSettings().GetSizeIU( pcbIUScale.IU_PER_MILS );
 
-        bestPosition.x = pageSize.GetWidth() / 2;
-        bestPosition.y = pageSize.GetHeight() / 2;
+        bestPosition.x = pageSize.x / 2;
+        bestPosition.y = pageSize.y / 2;
     }
 
     return bestPosition;
@@ -314,6 +315,23 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
         m_reporter->Report( msg, RPT_SEVERITY_ACTION );
     }
 
+    // Test for footprint change. This is controlled by a separate flag, that will output
+    // its own message if the footprint is changed, so we just set the field here.
+    if( ( m_replaceFootprints || ( aPcbFootprint->GetAttributes() & FP_JUST_ADDED ) )
+        && !m_isDryRun )
+    {
+        // Update FOOTPRINT_FIELD (if exists in the netlist)
+        try
+        {
+            aPcbFootprint->Footprint().SetText(
+                aNetlistComponent->GetFields()[GetCanonicalFieldName( FOOTPRINT_FIELD )] );
+        }
+        catch( ... )
+        {
+            // If not exist (old netlist), just skip it: What else?
+        }
+    }
+
     // Test for time stamp change.
     KIID_PATH new_path = aNetlistComponent->GetPath();
 
@@ -343,20 +361,185 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
         m_reporter->Report( msg, RPT_SEVERITY_ACTION );
     }
 
-    if( aPcbFootprint->GetProperties() != aNetlistComponent->GetProperties() )
+    nlohmann::ordered_map<wxString, wxString> fpFieldsAsMap;
+
+    for( PCB_FIELD* field : aPcbFootprint->GetFields() )
+    {
+        // These fields are individually checked above
+        if( field->IsReference() || field->IsValue() || field->IsFootprint() )
+            continue;
+
+        fpFieldsAsMap[field->GetName()] = field->GetText();
+    }
+
+    // Remove the ref/value/footprint fields that are individually handled
+    nlohmann::ordered_map<wxString, wxString> compFields = aNetlistComponent->GetFields();
+    compFields.erase( GetCanonicalFieldName( REFERENCE_FIELD ) );
+    compFields.erase( GetCanonicalFieldName( VALUE_FIELD ) );
+    compFields.erase( GetCanonicalFieldName( FOOTPRINT_FIELD ) );
+
+    // Fields are stored as an ordered map, but we don't (yet) support reordering
+    // the footprint fields to match the symbol, so we manually check the fields
+    // in the order they are stored in the symbol.
+    bool same = true;
+
+    for( std::pair<wxString, wxString> field : compFields )
+    {
+        if( fpFieldsAsMap.count( field.first ) == 0 || fpFieldsAsMap[field.first] != field.second )
+        {
+            same = false;
+            break;
+        }
+    }
+
+    if( !same )
     {
         if( m_isDryRun )
         {
-            msg.Printf( _( "Update %s properties." ),
-                        aPcbFootprint->GetReference() );
+            msg.Printf( _( "Update %s fields." ), aPcbFootprint->GetReference() );
+            m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+
+            // Remove fields that aren't present in the symbol
+            for( PCB_FIELD* field : aPcbFootprint->GetFields() )
+            {
+                if( field->IsMandatoryField() )
+                    continue;
+
+                if( compFields.count( field->GetName() ) == 0 )
+                {
+                    msg.Printf( _( "Remove %s footprint fields not in symbol." ),
+                                aPcbFootprint->GetReference() );
+                    m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                    break;
+                }
+            }
         }
         else
         {
-            msg.Printf( _( "Updated %s properties." ),
-                        aPcbFootprint->GetReference() );
+            msg.Printf( _( "Updated %s fields." ), aPcbFootprint->GetReference() );
+            m_reporter->Report( msg, RPT_SEVERITY_ACTION );
 
             changed = true;
-            aPcbFootprint->SetProperties( aNetlistComponent->GetProperties() );
+
+            // Add or change field value
+            for( auto& [name, value] : compFields )
+            {
+                if( aPcbFootprint->HasFieldByName( name ) )
+                {
+                    aPcbFootprint->GetFieldByName( name )->SetText( value );
+                }
+                else
+                {
+                    int        idx = aPcbFootprint->GetFieldCount();
+                    PCB_FIELD* newField = aPcbFootprint->AddField( PCB_FIELD( aPcbFootprint, idx ) );
+
+                    newField->SetName( name );
+                    newField->SetText( value );
+                    newField->SetVisible( false );
+                    newField->SetLayer( aPcbFootprint->GetLayer() == F_Cu ? F_Fab : B_Fab );
+
+                    // Give the relative position (0,0) in footprint
+                    newField->SetPosition( aPcbFootprint->GetPosition() );
+                    // Give the footprint orientation
+                    newField->Rotate( aPcbFootprint->GetPosition(), aPcbFootprint->GetOrientation() );
+
+                    if( m_frame )
+                        newField->StyleFromSettings( m_frame->GetDesignSettings() );
+                }
+            }
+
+            // Remove fields that aren't present in the symbol
+            bool warned = false;
+
+            for( PCB_FIELD* field : aPcbFootprint->GetFields() )
+            {
+                if( field->IsMandatoryField() )
+                    continue;
+
+                if( compFields.count( field->GetName() ) == 0 )
+                {
+                    if( !warned )
+                    {
+                        warned = true;
+                        msg.Printf( _( "Removed %s footprint fields not in symbol." ),
+                                    aPcbFootprint->GetReference() );
+                        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                    }
+
+                    aPcbFootprint->RemoveField( field->GetCanonicalName() );
+
+                    if( m_frame )
+                        m_frame->GetCanvas()->GetView()->Remove( field );
+                }
+            }
+        }
+    }
+
+    wxString sheetname;
+    wxString sheetfile;
+    wxString fpFilters;
+
+    if( aNetlistComponent->GetProperties().count( wxT( "Sheetname" ) ) > 0 )
+        sheetname = aNetlistComponent->GetProperties().at( wxT( "Sheetname" ) );
+
+    if( aNetlistComponent->GetProperties().count( wxT( "Sheetfile" ) ) > 0 )
+        sheetfile = aNetlistComponent->GetProperties().at( wxT( "Sheetfile" ) );
+
+    if( aNetlistComponent->GetProperties().count( wxT( "ki_fp_filters" ) ) > 0 )
+        fpFilters = aNetlistComponent->GetProperties().at( wxT( "ki_fp_filters" ) );
+
+    if( sheetname != aPcbFootprint->GetSheetname() )
+    {
+        if( m_isDryRun )
+        {
+            msg.Printf( _( "Update %s sheetname to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        sheetname );
+        }
+        else
+        {
+            aPcbFootprint->SetSheetname( sheetname );
+            msg.Printf( _( "Updated %s sheetname to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        sheetname );
+        }
+
+        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+    }
+
+    if( sheetfile != aPcbFootprint->GetSheetfile() )
+    {
+        if( m_isDryRun )
+        {
+            msg.Printf( _( "Update %s sheetfile to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        sheetfile );
+        }
+        else
+        {
+            aPcbFootprint->SetSheetfile( sheetfile );
+            msg.Printf( _( "Updated %s sheetfile to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        sheetfile );
+        }
+
+        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+    }
+
+    if( fpFilters != aPcbFootprint->GetFilters() )
+    {
+        if( m_isDryRun )
+        {
+            msg.Printf( _( "Update %s footprint filters to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        fpFilters );
+        }
+        else
+        {
+            aPcbFootprint->SetFilters( fpFilters );
+            msg.Printf( _( "Updated %s footprint filters to '%s'." ),
+                        aPcbFootprint->GetReference(),
+                        fpFilters );
         }
 
         m_reporter->Report( msg, RPT_SEVERITY_ACTION );
@@ -402,6 +585,46 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
         m_reporter->Report( msg, RPT_SEVERITY_ACTION );
     }
 
+    if( ( aNetlistComponent->GetProperties().count( wxT( "dnp" ) ) > 0 )
+            != ( ( aPcbFootprint->GetAttributes() & FP_DNP ) > 0 ) )
+    {
+        if( m_isDryRun )
+        {
+            if( aNetlistComponent->GetProperties().count( wxT( "dnp" ) ) )
+            {
+                msg.Printf( _( "Add %s 'Do not place' fabrication attribute." ),
+                            aPcbFootprint->GetReference() );
+            }
+            else
+            {
+                msg.Printf( _( "Remove %s 'Do not place' fabrication attribute." ),
+                            aPcbFootprint->GetReference() );
+            }
+        }
+        else
+        {
+            int attributes = aPcbFootprint->GetAttributes();
+
+            if( aNetlistComponent->GetProperties().count( wxT( "dnp" ) ) )
+            {
+                attributes |= FP_DNP;
+                msg.Printf( _( "Added %s 'Do not place' fabrication attribute." ),
+                            aPcbFootprint->GetReference() );
+            }
+            else
+            {
+                attributes &= ~FP_DNP;
+                msg.Printf( _( "Removed %s 'Do not place' fabrication attribute." ),
+                            aPcbFootprint->GetReference() );
+            }
+
+            changed = true;
+            aPcbFootprint->SetAttributes( attributes );
+        }
+
+        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+    }
+
     if( changed && copy )
         m_commit.Modified( aPcbFootprint, copy );
     else if( copy )
@@ -419,7 +642,7 @@ bool BOARD_NETLIST_UPDATER::updateComponentPadConnections( FOOTPRINT* aFootprint
     // Create a copy only if the footprint has not been added during this update
     FOOTPRINT* copy = nullptr;
 
-    if( !m_commit.GetStatus( aFootprint ) )
+    if( !m_isDryRun && !m_commit.GetStatus( aFootprint ) )
     {
         copy = static_cast<FOOTPRINT*>( aFootprint->Clone() );
         copy->SetParentGroup( nullptr );
@@ -428,7 +651,16 @@ bool BOARD_NETLIST_UPDATER::updateComponentPadConnections( FOOTPRINT* aFootprint
     bool changed = false;
 
     // At this point, the component footprint is updated.  Now update the nets.
-    for( PAD* pad : aFootprint->Pads() )
+    PADS pads = aFootprint->Pads();
+    std::set<wxString> padNetnames;
+
+    std::sort( pads.begin(), pads.end(),
+               []( PAD* a, PAD* b )
+               {
+                   return a->m_Uuid < b->m_Uuid;
+               } );
+
+    for( PAD* pad : pads )
     {
         const COMPONENT_NET& net = aNewComponent->GetNet( pad->GetNumber() );
 
@@ -483,8 +715,9 @@ bool BOARD_NETLIST_UPDATER::updateComponentPadConnections( FOOTPRINT* aFootprint
             else if( pad->IsOnCopperLayer() && !pad->GetNumber().IsEmpty() )
             {
                 // pad is connectable but has no net found in netlist
-                msg.Printf( _( "No net found for symbol %s pin %s." ),
+                msg.Printf( _( "No net found for component %s pad %s (no pin %s in symbol)." ),
                             aFootprint->GetReference(),
+                            pad->GetNumber(),
                             pad->GetNumber() );
                 m_reporter->Report( msg, RPT_SEVERITY_WARNING);
                 ++m_warningCount;
@@ -508,7 +741,16 @@ bool BOARD_NETLIST_UPDATER::updateComponentPadConnections( FOOTPRINT* aFootprint
         }
         else                                 // New footprint pad has a net.
         {
-            const wxString& netName = net.GetNetName();
+            wxString netName = net.GetNetName();
+
+            if( pad->IsNoConnectPad() )
+            {
+                netName = wxString::Format( wxS( "%s" ), net.GetNetName() );
+
+                for( int jj = 1; !padNetnames.insert( netName ).second; jj++ )
+                    netName = wxString::Format( wxS( "%s_%d" ), net.GetNetName(), jj );
+            }
+
             NETINFO_ITEM* netinfo = m_board->FindNet( netName );
 
             if( netinfo && !m_isDryRun )
@@ -786,6 +1028,15 @@ bool BOARD_NETLIST_UPDATER::updateCopperZoneNets( NETLIST& aNetlist )
                     PCB_LAYER_ID layer = zone->GetLayer();
                     VECTOR2I     pos = zone->GetPosition();
 
+                    if( m_frame && m_frame->GetPcbNewSettings() )
+                    {
+                        if( m_frame->GetPcbNewSettings()->m_Display.m_DisplayInvertXAxis )
+                            pos.x *= -1;
+
+                        if( m_frame->GetPcbNewSettings()->m_Display.m_DisplayInvertYAxis )
+                            pos.y *= -1;
+                    }
+
                     msg.Printf( _( "Copper zone on layer %s at (%s, %s) has no pads connected." ),
                                 m_board->GetLayerName( layer ),
                                 m_frame->MessageTextFromValue( pos.x ),
@@ -873,8 +1124,6 @@ bool BOARD_NETLIST_UPDATER::UpdateNetlist( NETLIST& aNetlist )
     //
     if( !m_isDryRun )
     {
-        m_board->SetStatus( 0 );
-
         for( NETINFO_ITEM* net : m_board->GetNetInfo() )
             net->SetIsCurrent( net->GetNetCode() == 0 );
     }
@@ -1011,6 +1260,9 @@ bool BOARD_NETLIST_UPDATER::UpdateNetlist( NETLIST& aNetlist )
             }
             else
             {
+                if( footprint->GetParentGroup() )
+                    m_commit.Stage( footprint, CHT_UNGROUP );
+
                 m_commit.Remove( footprint );
                 msg.Printf( _( "Removed unused footprint %s." ), footprint->GetReference() );
             }
@@ -1041,11 +1293,10 @@ bool BOARD_NETLIST_UPDATER::UpdateNetlist( NETLIST& aNetlist )
             {
                 msg.Printf( _( "Removed unused net %s." ), net->GetNetname() );
                 m_reporter->Report( msg, RPT_SEVERITY_ACTION );
-                m_commit.Removed( net );
             }
         }
 
-        m_board->GetNetInfo().RemoveUnusedNets();
+        m_board->RemoveUnusedNets( &m_commit );
 
         // When new footprints are added, the automatic zone refill is disabled because:
         // * it creates crashes when calculating dynamic ratsnests if auto refill is enabled.
