@@ -322,7 +322,7 @@ void PCB_DRAW_PANEL_GAL::UpdateColors()
     }
     else
     {
-        PCBNEW_SETTINGS* app = Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>();
+        auto* app = Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>( "pcbnew" );
 
         if( app )
             cs = Pgm().GetSettingsManager().GetColorSettings( app->m_ColorTheme );
@@ -511,6 +511,7 @@ void PCB_DRAW_PANEL_GAL::SyncLayersVisibility( const BOARD* aBoard )
 
     // Enable some layers that are GAL specific
     m_view->SetLayerVisible( LAYER_PAD_PLATEDHOLES, true );
+    m_view->SetLayerVisible( LAYER_NON_PLATEDHOLES, true );
     m_view->SetLayerVisible( LAYER_PAD_HOLEWALLS, true );
     m_view->SetLayerVisible( LAYER_VIA_HOLES, true );
     m_view->SetLayerVisible( LAYER_VIA_HOLEWALLS, true );
@@ -722,11 +723,11 @@ void PCB_DRAW_PANEL_GAL::setDefaultLayerDeps()
     m_view->SetLayerDisplayOnly( LAYER_RATSNEST );
 
     m_view->SetLayerTarget( LAYER_DRC_ERROR, KIGFX::TARGET_OVERLAY );
-    m_view->SetLayerDisplayOnly( LAYER_DRC_ERROR );
+    //m_view->SetLayerDisplayOnly( LAYER_DRC_ERROR );
     m_view->SetLayerTarget( LAYER_DRC_WARNING, KIGFX::TARGET_OVERLAY );
-    m_view->SetLayerDisplayOnly( LAYER_DRC_WARNING );
+    //m_view->SetLayerDisplayOnly( LAYER_DRC_WARNING );
     m_view->SetLayerTarget( LAYER_DRC_EXCLUSION, KIGFX::TARGET_OVERLAY );
-    m_view->SetLayerDisplayOnly( LAYER_DRC_EXCLUSION );
+    //m_view->SetLayerDisplayOnly( LAYER_DRC_EXCLUSION );
     m_view->SetLayerTarget( LAYER_MARKER_SHADOWS, KIGFX::TARGET_OVERLAY );
     m_view->SetLayerDisplayOnly( LAYER_MARKER_SHADOWS );
 
