@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2016 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,9 +25,10 @@
 
 #include <pcbnew_scripting_helpers.h>
 #include <pgm_base.h>
+#include <cli_progress_reporter.h>
+#include <confirm.h>
 #include <kiface_base.h>
 #include <kiface_ids.h>
-#include <confirm.h>
 #include <pcb_edit_frame.h>
 #include <eda_dde.h>
 #include <macros.h>
@@ -37,15 +38,18 @@
 #include <pcbnew_settings.h>
 #include <footprint_editor_settings.h>
 #include <settings/settings_manager.h>
+#include <settings/cvpcb_settings.h>
 #include <fp_lib_table.h>
 #include <footprint_edit_frame.h>
 #include <footprint_viewer_frame.h>
+#include <footprint_chooser_frame.h>
 #include <footprint_wizard_frame.h>
 #include <footprint_preview_panel.h>
 #include <footprint_info_impl.h>
 #include <dialogs/dialog_configure_paths.h>
+#include <dialogs/panel_grid_settings.h>
 #include <dialog_global_fp_lib_table_config.h>
-#include <panel_display_options.h>
+#include <panel_pcb_display_options.h>
 #include <panel_edit_options.h>
 #include <panel_fp_editor_defaults.h>
 #include <panel_fp_editor_color_settings.h>
@@ -55,7 +59,6 @@
 #include <panel_3D_display_options.h>
 #include <panel_3D_opengl_options.h>
 #include <panel_3D_raytracing_options.h>
-#include <panel_3D_colors.h>
 #include <python_scripting.h>
 
 #include "invoke_pcb_dialog.h"
@@ -79,7 +82,7 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             UNITS_PROVIDER( pcbIUScale, EDA_UNITS::MILLIMETRES )
     {}
 
-    bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits ) override;
+    bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits, KIWAY* aKiway ) override;
 
     void Reset() override;
 
@@ -110,14 +113,16 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             return new FOOTPRINT_EDIT_FRAME( aKiway, aParent );
 
         case FRAME_FOOTPRINT_VIEWER:
-        case FRAME_FOOTPRINT_VIEWER_MODAL:
-            return new FOOTPRINT_VIEWER_FRAME( aKiway, aParent, FRAME_T( aClassId ) );
+            return new FOOTPRINT_VIEWER_FRAME( aKiway, aParent );
+
+        case FRAME_FOOTPRINT_CHOOSER:
+            return new FOOTPRINT_CHOOSER_FRAME( aKiway, aParent );
 
         case FRAME_FOOTPRINT_WIZARD:
             return new FOOTPRINT_WIZARD_FRAME( aKiway, aParent, FRAME_T( aClassId ) );
 
         case FRAME_FOOTPRINT_PREVIEW:
-            return dynamic_cast< wxWindow* >( FOOTPRINT_PREVIEW_PANEL::New( aKiway, aParent ) );
+            return FOOTPRINT_PREVIEW_PANEL::New( aKiway, aParent, this );
 
         case DIALOG_CONFIGUREPATHS:
         {
@@ -146,7 +151,33 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
             APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
 
-            return new PANEL_DISPLAY_OPTIONS( aParent, cfg );
+            return new PANEL_PCB_DISPLAY_OPTIONS( aParent, cfg );
+        }
+
+        case PANEL_FP_GRIDS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+            EDA_BASE_FRAME*    frame = aKiway->Player( FRAME_FOOTPRINT_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_FOOTPRINT_VIEWER, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_PCB_EDITOR, false );
+
+            if( frame )
+                SetUserUnits( frame->GetUserUnits() );
+
+            return new PANEL_GRID_SETTINGS( aParent, this, frame, cfg, FRAME_FOOTPRINT_EDITOR );
+        }
+
+        case PANEL_FP_ORIGINS_AXES:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+
+            return new PANEL_PCBNEW_DISPLAY_ORIGIN( aParent, cfg, FRAME_FOOTPRINT_EDITOR );
         }
 
         case PANEL_FP_EDIT_OPTIONS:
@@ -184,12 +215,38 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
         case PANEL_FP_COLORS:
             return new PANEL_FP_EDITOR_COLOR_SETTINGS( aParent );
 
-        case PANEL_PCB_DISPLAY_OPTIONS:
+        case PANEL_PCB_DISPLAY_OPTS:
         {
             SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
             APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<PCBNEW_SETTINGS>();
 
-            return new PANEL_DISPLAY_OPTIONS( aParent, cfg );
+            return new PANEL_PCB_DISPLAY_OPTIONS( aParent, cfg );
+        }
+
+        case PANEL_PCB_GRIDS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<PCBNEW_SETTINGS>();
+            EDA_BASE_FRAME*    frame = aKiway->Player( FRAME_PCB_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_FOOTPRINT_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_FOOTPRINT_VIEWER, false );
+
+            if( frame )
+                SetUserUnits( frame->GetUserUnits() );
+
+            return new PANEL_GRID_SETTINGS( aParent, this, frame, cfg, FRAME_PCB_EDITOR );
+        }
+
+        case PANEL_PCB_ORIGINS_AXES:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<PCBNEW_SETTINGS>();
+
+            return new PANEL_PCBNEW_DISPLAY_ORIGIN( aParent, cfg, FRAME_PCB_EDITOR );
         }
 
         case PANEL_PCB_EDIT_OPTIONS:
@@ -222,9 +279,6 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
         case PANEL_PCB_ACTION_PLUGINS:
             return new PANEL_PCBNEW_ACTION_PLUGINS( aParent );
 
-        case PANEL_PCB_ORIGINS_AXES:
-            return new PANEL_PCBNEW_DISPLAY_ORIGIN( aParent );
-
         case PANEL_3DV_DISPLAY_OPTIONS:
             return new PANEL_3D_DISPLAY_OPTIONS( aParent );
 
@@ -234,9 +288,6 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
         case PANEL_3DV_RAYTRACING:
             return new PANEL_3D_RAYTRACING_OPTIONS( aParent );
 
-        case PANEL_3DV_COLORS:
-            return new PANEL_3D_COLORS( aParent );
-
         default:
             return nullptr;
         }
@@ -308,7 +359,7 @@ KIFACE_BASE& Kiface() { return kiface; }
 
 // KIFACE_GETTER's actual spelling is a substitution macro found in kiway.h.
 // KIFACE_GETTER will not have name mangling due to declaration in kiway.h.
-MY_API( KIFACE* ) KIFACE_GETTER( int* aKIFACEversion, int aKiwayVersion, PGM_BASE* aProgram )
+KIFACE_API KIFACE* KIFACE_GETTER( int* aKIFACEversion, int aKiwayVersion, PGM_BASE* aProgram )
 {
     process = aProgram;
     return &kiface;
@@ -342,19 +393,50 @@ FP_LIB_TABLE          GFootprintTable;
 FOOTPRINT_LIST_IMPL   GFootprintList;
 
 
-bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits )
+bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits, KIWAY* aKiway )
 {
     // This is process-level-initialization, not project-level-initialization of the DSO.
     // Do nothing in here pertinent to a project!
     InitSettings( new PCBNEW_SETTINGS );
+    aProgram->GetSettingsManager().RegisterSettings( new FOOTPRINT_EDITOR_SETTINGS );
+    aProgram->GetSettingsManager().RegisterSettings( new EDA_3D_VIEWER_SETTINGS );
+
+    // We intentionally register KifaceSettings after FOOTPRINT_EDITOR_SETTINGS and EDA_3D_VIEWER_SETTINGS
+    // In legacy configs, many settings were in a single editor config and the migration routine
+    // for the main editor file will try and call into the now separate settings stores
+    // to move the settings into them
     aProgram->GetSettingsManager().RegisterSettings( KifaceSettings() );
 
+    // Register the footprint editor settings as well because they share a KiFACE and need to be
+    // loaded prior to use to avoid threading deadlocks
+    aProgram->GetSettingsManager().RegisterSettings( new CVPCB_SETTINGS );
+
     start_common( aCtlBits );
 
     if( !loadGlobalLibTable() )
-        return false;
+    {
+        // we didnt get anywhere deregister the
+        aProgram->GetSettingsManager().FlushAndRelease(
+                aProgram->GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>(), false );
 
-    m_jobHandler = std::make_unique<PCBNEW_JOBS_HANDLER>();
+        aProgram->GetSettingsManager().FlushAndRelease( KifaceSettings(), false );
+
+        aProgram->GetSettingsManager().FlushAndRelease(
+                aProgram->GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>(), false );
+
+        aProgram->GetSettingsManager().FlushAndRelease(
+                aProgram->GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>(), false );
+
+        return false;
+    }
+
+    m_jobHandler = std::make_unique<PCBNEW_JOBS_HANDLER>( aKiway );
+
+    if( m_start_flags & KFCTL_CLI )
+    {
+        m_jobHandler->SetReporter( &CLI_REPORTER::GetInstance() );
+        m_jobHandler->SetProgressReporter( &CLI_PROGRESS_REPORTER::GetInstance() );
+    }
 
     return true;
 }
@@ -374,9 +456,13 @@ bool IFACE::loadGlobalLibTable()
     {
         if( !( m_start_flags & KFCTL_CLI ) )
         {
+            // Ensure the splash screen does not hide the dialog:
+            Pgm().HideSplash();
+
             DIALOG_GLOBAL_FP_LIB_TABLE_CONFIG fpDialog( nullptr );
 
-            fpDialog.ShowModal();
+            if( fpDialog.ShowModal() != wxID_OK )
+                return false;
         }
     }
     else
@@ -432,26 +518,28 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aSrcPr
 
     destFile.SetPath( destPath );
 
-    if( ext == KiCadPcbFileExtension || ext == KiCadPcbFileExtension + BackupFileSuffix )
+    if( ext == FILEEXT::KiCadPcbFileExtension
+        || ext == FILEEXT::KiCadPcbFileExtension + FILEEXT::BackupFileSuffix )
     {
         if( destFile.GetName() == aSrcProjectName )
             destFile.SetName( aNewProjectName );
 
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == LegacyPcbFileExtension )
+    else if( ext == FILEEXT::LegacyPcbFileExtension )
     {
         if( destFile.GetName() == aSrcProjectName )
             destFile.SetName( aNewProjectName );
 
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == LegacyFootprintLibPathExtension || ext == KiCadFootprintFileExtension )
+    else if( ext == FILEEXT::LegacyFootprintLibPathExtension
+             || ext == FILEEXT::KiCadFootprintFileExtension )
     {
         // Footprints are not project-specific.  Keep their source names.
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == FootprintAssignmentFileExtension )
+    else if( ext == FILEEXT::FootprintAssignmentFileExtension )
     {
         // TODO
     }
@@ -504,4 +592,4 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aSrcPr
 int IFACE::HandleJob( JOB* aJob )
 {
     return m_jobHandler->RunJob( aJob );
-}
\ No newline at end of file
+}
