@@ -1,14 +1,7 @@
-/**
- * @file plot_board_layers.cpp
- * @brief Functions to plot one board layer (silkscreen layers or other layers).
- * Silkscreen layers have specific requirement for pads (not filled) and texts
- * (with option to remove them from some copper areas (pads...)
- */
-
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,26 +21,21 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-
+#include <wx/log.h>
 #include <eda_item.h>
 #include <layer_ids.h>
 #include <geometry/geometry_utils.h>
 #include <geometry/shape_segment.h>
 #include <pcb_base_frame.h>
 #include <math/util.h>      // for KiROUND
-
 #include <board.h>
-#include <board_design_settings.h>
 #include <footprint.h>
 #include <pcb_track.h>
-#include <pcb_text.h>
-#include <fp_shape.h>
 #include <pad.h>
 #include <zone.h>
 #include <pcb_shape.h>
 #include <pcb_target.h>
 #include <pcb_dimension.h>
-
 #include <pcbplot.h>
 #include <plotters/plotter_dxf.h>
 #include <plotters/plotter_hpgl.h>
@@ -70,15 +58,43 @@ void PlotBoardLayers( BOARD* aBoard, PLOTTER* aPlotter, const LSEQ& aLayers,
 {
     wxCHECK( aBoard && aPlotter && aLayers.size(), /* void */ );
 
+    // if a drill mark must be plotted, the copper layer needs to be plotted
+    // after other layers because the drill mark must be plotted as a filled
+    // white shape *after* all other shapes are plotted
+    bool plot_mark = aPlotOptions.GetDrillMarksType() != DRILL_MARKS::NO_DRILL_SHAPE;
+
     for( LSEQ seq = aLayers; seq; ++seq )
+    {
+        // copper layers with drill marks will be plotted after all other layers
+        if( *seq <= B_Cu && plot_mark )
+            continue;
+
         PlotOneBoardLayer( aBoard, aPlotter, *seq, aPlotOptions );
+    }
+
+    if( !plot_mark )
+        return;
+
+    for( LSEQ seq = aLayers; seq; ++seq )
+    {
+        if( *seq > B_Cu )   // already plotted
+            continue;
+
+        PlotOneBoardLayer( aBoard, aPlotter, *seq, aPlotOptions );
+    }
 }
 
 
-void PlotInteractiveLayer( BOARD* aBoard, PLOTTER* aPlotter )
+void PlotInteractiveLayer( BOARD* aBoard, PLOTTER* aPlotter, const PCB_PLOT_PARAMS& aPlotOpt )
 {
     for( const FOOTPRINT* fp : aBoard->Footprints() )
     {
+        if( fp->GetLayer() == F_Cu && !aPlotOpt.m_PDFFrontFPPropertyPopups )
+            continue;
+
+        if( fp->GetLayer() == B_Cu && !aPlotOpt.m_PDFBackFPPropertyPopups )
+            continue;
+
         std::vector<wxString> properties;
 
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
@@ -89,22 +105,50 @@ void PlotInteractiveLayer( BOARD* aBoard, PLOTTER* aPlotter )
                                                    _( "Value" ),
                                                    fp->Value().GetShownText( false ) ) );
 
-        for( const auto& [ name, value ] : fp->GetProperties() )
-            properties.emplace_back( wxString::Format( wxT( "!%s = %s" ), name, value ) );
-
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
                                                    _( "Footprint" ),
-                                                   fp->GetFPIDAsString() ) );
+                                                   fp->GetFPID().GetUniStringLibItemName() ) );
 
-        properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
-                                                   _( "Description" ),
-                                                   fp->GetDescription() ) );
+        for( int i = 0; i < fp->GetFieldCount(); i++ )
+        {
+            PCB_FIELD* field = fp->GetFields().at( i );
+
+            if( field->IsReference() || field->IsValue() || field->IsFootprint() )
+                continue;
+
+            if( field->GetText().IsEmpty() )
+                continue;
+
+            properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
+                                                       field->GetName(),
+                                                       field->GetText() ) );
+        }
+
+        // These 2 properties are not very useful in a plot file (like a PDF)
+#if 0
+        properties.emplace_back( wxString::Format( wxT( "!%s = %s" ), _( "Library Description" ),
+                                                   fp->GetLibDescription() ) );
 
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
                                                    _( "Keywords" ),
                                                    fp->GetKeywords() ) );
+#endif
+        // Draw items are plotted with a position offset. So we need to move
+        // our boxes (which are not plotted) by the same offset.
+        VECTOR2I offset = -aPlotter->GetPlotOffsetUserUnits();
 
-        aPlotter->HyperlinkMenu( fp->GetBoundingBox(), properties );
+        // Use a footprint bbox without texts to create the hyperlink area
+        BOX2I bbox = fp->GetBoundingBox( false, false );
+        bbox.Move( offset );
+        aPlotter->HyperlinkMenu( bbox, properties );
+
+        // Use a footprint bbox with visible texts only to create the bookmark area
+        // which is the area to zoom on ft selection
+        // However the bbox need to be inflated for a better look.
+        bbox = fp->GetBoundingBox( true, false );
+        bbox.Move( offset );
+        bbox.Inflate( bbox.GetWidth() /2, bbox.GetHeight() /2 );
+        aPlotter->Bookmark( bbox, fp->GetReference(), _( "Footprints" ) );
     }
 }
 
@@ -112,6 +156,16 @@ void PlotInteractiveLayer( BOARD* aBoard, PLOTTER* aPlotter )
 void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
                         const PCB_PLOT_PARAMS& aPlotOpt )
 {
+    auto plotLayer =
+            [&]( LSET layerMask, PCB_PLOT_PARAMS& plotOpts )
+            {
+                // PlotLayerOutlines() is designed only for DXF plotters.
+                if( plotOpts.GetFormat() == PLOT_FORMAT::DXF && plotOpts.GetDXFPlotPolygonMode() )
+                    PlotLayerOutlines( aBoard, aPlotter, layerMask, plotOpts );
+                else
+                    PlotStandardLayer( aBoard, aPlotter, layerMask, plotOpts );
+            };
+
     PCB_PLOT_PARAMS plotOpt = aPlotOpt;
     int soldermask_min_thickness = aBoard->GetDesignSettings().m_SolderMaskMinWidth;
 
@@ -126,17 +180,13 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
     if( IsCopperLayer( aLayer ) )
     {
         // Skip NPTH pads on copper layers ( only if hole size == pad size ):
-        // Drill mark will be plotted if drill mark is SMALL_DRILL_SHAPE  or FULL_DRILL_SHAPE
+        // Drill mark will be plotted if drill mark is SMALL_DRILL_SHAPE or FULL_DRILL_SHAPE
         if( plotOpt.GetFormat() == PLOT_FORMAT::DXF )
-        {
-            plotOpt.SetSkipPlotNPTH_Pads( false );
-            PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-        }
+            plotOpt.SetDXFPlotPolygonMode( true );
         else
-        {
             plotOpt.SetSkipPlotNPTH_Pads( true );
-            PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
-        }
+
+        plotLayer( layer_mask, plotOpt );
     }
     else
     {
@@ -144,18 +194,16 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
         {
         case B_Mask:
         case F_Mask:
-            plotOpt.SetSkipPlotNPTH_Pads( false );
-
             // Disable plot pad holes
             plotOpt.SetDrillMarksType( DRILL_MARKS::NO_DRILL_SHAPE );
 
+            // Use outline mode for DXF
+            plotOpt.SetDXFPlotPolygonMode( true );
+
             // Plot solder mask:
             if( soldermask_min_thickness == 0 )
             {
-                if( plotOpt.GetFormat() == PLOT_FORMAT::DXF )
-                    PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-                else
-                    PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
+                plotLayer( layer_mask, plotOpt );
             }
             else
             {
@@ -169,26 +217,19 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
         case F_Adhes:
         case B_Paste:
         case F_Paste:
-            plotOpt.SetSkipPlotNPTH_Pads( false );
-
             // Disable plot pad holes
             plotOpt.SetDrillMarksType( DRILL_MARKS::NO_DRILL_SHAPE );
 
-            if( plotOpt.GetFormat() == PLOT_FORMAT::DXF )
-                PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-            else
-                PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
+            // Use outline mode for DXF
+            plotOpt.SetDXFPlotPolygonMode( true );
+
+            plotLayer( layer_mask, plotOpt );
 
             break;
 
         case F_SilkS:
         case B_SilkS:
-            if( plotOpt.GetFormat() == PLOT_FORMAT::DXF && plotOpt.GetDXFPlotPolygonMode() )
-                // PlotLayerOutlines() is designed only for DXF plotters.
-                // and must not be used for other plot formats
-                PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-            else
-                PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
+            plotLayer( layer_mask, plotOpt );
 
             // Gerber: Subtract soldermask from silkscreen if enabled
             if( aPlotter->GetPlotterType() == PLOT_FORMAT::GERBER
@@ -214,9 +255,6 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
 
             break;
 
-        // These layers are plotted like silk screen layers.
-        // Mainly, pads on these layers are not filled.
-        // This is not necessary the best choice.
         case Dwgs_User:
         case Cmts_User:
         case Eco1_User:
@@ -227,29 +265,8 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
         case B_CrtYd:
         case F_Fab:
         case B_Fab:
-            plotOpt.SetSkipPlotNPTH_Pads( false );
-            plotOpt.SetDrillMarksType( DRILL_MARKS::NO_DRILL_SHAPE );
-
-            if( plotOpt.GetFormat() == PLOT_FORMAT::DXF && plotOpt.GetDXFPlotPolygonMode() )
-                // PlotLayerOutlines() is designed only for DXF plotters.
-                // and must not be used for other plot formats
-                PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-            else
-                PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
-
-            break;
-
         default:
-            plotOpt.SetSkipPlotNPTH_Pads( false );
-            plotOpt.SetDrillMarksType( DRILL_MARKS::NO_DRILL_SHAPE );
-
-            if( plotOpt.GetFormat() == PLOT_FORMAT::DXF && plotOpt.GetDXFPlotPolygonMode() )
-                // PlotLayerOutlines() is designed only for DXF plotters.
-                // and must not be used for other plot formats
-                PlotLayerOutlines( aBoard, aPlotter, layer_mask, plotOpt );
-            else
-                PlotStandardLayer( aBoard, aPlotter, layer_mask, plotOpt );
-
+            plotLayer( layer_mask, plotOpt );
             break;
         }
     }
@@ -257,14 +274,13 @@ void PlotOneBoardLayer( BOARD *aBoard, PLOTTER* aPlotter, PCB_LAYER_ID aLayer,
 
 
 /**
- * Plot a copper layer or mask.
- *
- * Silk screen layers are not plotted here.
+ * Plot any layer EXCEPT a solder-mask with an enforced minimum width.
  */
 void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                         const PCB_PLOT_PARAMS& aPlotOpt )
 {
     BRDITEMS_PLOTTER itemplotter( aPlotter, aBoard, aPlotOpt );
+    int              maxError = aBoard->GetDesignSettings().m_MaxError;
 
     itemplotter.SetLayerSet( aLayerMask );
 
@@ -272,12 +288,13 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
     bool onCopperLayer = ( LSET::AllCuMask() & aLayerMask ).any();
     bool onSolderMaskLayer = ( LSET( 2, F_Mask, B_Mask ) & aLayerMask ).any();
     bool onSolderPasteLayer = ( LSET( 2, F_Paste, B_Paste ) & aLayerMask ).any();
-    bool onFrontFab = ( LSET(  F_Fab ) & aLayerMask ).any();
+    bool onFrontFab = ( LSET( F_Fab ) & aLayerMask ).any();
     bool onBackFab  = ( LSET( B_Fab ) & aLayerMask ).any();
     bool sketchPads = ( onFrontFab || onBackFab ) && aPlotOpt.GetSketchPadsOnFabLayers();
 
      // Plot edge layer and graphic items
-    itemplotter.PlotBoardGraphicItems();
+    for( const BOARD_ITEM* item : aBoard->Drawings() )
+        itemplotter.PlotBoardGraphicItem( item );
 
     // Draw footprint texts:
     for( const FOOTPRINT* footprint : aBoard->Footprints() )
@@ -363,13 +380,17 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
             PAD_SHAPE padShape = pad->GetShape();
             VECTOR2I  padSize = pad->GetSize();
             VECTOR2I  padDelta = pad->GetDelta(); // has meaning only for trapezoidal pads
-            double    padCornerRadius = pad->GetRoundRectCornerRadius();
+            // CornerRadius and CornerRadiusRatio can be modified
+            // the radius is built from the ratio, so saving/restoring the ratio is enough
+            double    padCornerRadiusRatio = pad->GetRoundRectRadiusRatio();
 
             // Don't draw a 0 sized pad.
             // Note: a custom pad can have its pad anchor with size = 0
             if( pad->GetShape() != PAD_SHAPE::CUSTOM
                 && ( padPlotsSize.x <= 0 || padPlotsSize.y <= 0 ) )
+            {
                 continue;
+            }
 
             switch( pad->GetShape() )
             {
@@ -388,7 +409,7 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                 itemplotter.PlotPad( pad, color, padPlotMode );
                 break;
 
-            case PAD_SHAPE::RECT:
+            case PAD_SHAPE::RECTANGLE:
                 pad->SetSize( padPlotsSize );
 
                 if( mask_clearance > 0 )
@@ -429,9 +450,8 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
                     // Shape polygon can have holes so use InflateWithLinkedHoles(), not Inflate()
                     // which can create bad shapes if margin.x is < 0
-                    int maxError = aBoard->GetDesignSettings().m_MaxError;
-                    int numSegs = GetArcToSegmentCount( mask_clearance, maxError, FULL_CIRCLE );
-                    outline.InflateWithLinkedHoles( mask_clearance, numSegs,
+                    outline.InflateWithLinkedHoles( mask_clearance,
+                                                    CORNER_STRATEGY::ROUND_ALL_CORNERS, maxError,
                                                     SHAPE_POLY_SET::PM_FAST );
                     dummy.DeletePrimitivesList();
                     dummy.AddPrimitivePoly( outline, 0, true );
@@ -439,7 +459,7 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                     // Be sure the anchor pad is not bigger than the deflated shape because this
                     // anchor will be added to the pad shape when plotting the pad. So now the
                     // polygonal shape is built, we can clamp the anchor size
-                    dummy.SetSize( wxSize( 0,0 ) );
+                    dummy.SetSize( VECTOR2I( 0, 0 ) );
 
                     itemplotter.PlotPad( &dummy, color, padPlotMode );
                 }
@@ -448,11 +468,12 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
             case PAD_SHAPE::ROUNDRECT:
             {
-                // rounding is stored as a percent, but we have to change the new radius
-                // to initial_radius + clearance to have a inflated/deflated similar shape
-                int initial_radius = pad->GetRoundRectCornerRadius();
+                // rounding is stored as a percent, but we have to update this ratio
+                // to force recalculation of other values after size changing (we do not
+                // really change the rounding percent value)
+                double radius_ratio = pad->GetRoundRectRadiusRatio();
                 pad->SetSize( padPlotsSize );
-                pad->SetRoundRectCornerRadius( std::max( initial_radius + mask_clearance, 0 ) );
+                pad->SetRoundRectRadiusRatio( radius_ratio );
 
                 itemplotter.PlotPad( pad, color, padPlotMode );
                 break;
@@ -472,16 +493,16 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                     // and use a dummy  CUSTOM pad to plot the final shape.
                     PAD dummy( *pad );
                     // Build the dummy pad outline with coordinates relative to the pad position
-                    // and orientation 0. The actual pos and rotation will be taken in account
-                    // later by the plot function
+                    // pad offset and orientation 0. The actual pos, offset and rotation will be
+                    // taken in account later by the plot function
                     dummy.SetPosition( VECTOR2I( 0, 0 ) );
+                    dummy.SetOffset( VECTOR2I( 0, 0 ) );
                     dummy.SetOrientation( ANGLE_0 );
                     SHAPE_POLY_SET outline;
-                    int maxError = aBoard->GetDesignSettings().m_MaxError;
-                    int numSegs = GetArcToSegmentCount( mask_clearance, maxError, FULL_CIRCLE );
                     dummy.TransformShapeToPolygon( outline, UNDEFINED_LAYER, 0, maxError,
                                                    ERROR_INSIDE );
-                    outline.InflateWithLinkedHoles( mask_clearance, numSegs,
+                    outline.InflateWithLinkedHoles( mask_clearance,
+                                                    CORNER_STRATEGY::ROUND_ALL_CORNERS, maxError,
                                                     SHAPE_POLY_SET::PM_FAST );
 
                     // Initialize the dummy pad shape:
@@ -493,8 +514,10 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                     // Be sure the anchor pad is not bigger than the deflated shape because this
                     // anchor will be added to the pad shape when plotting the pad.
                     // So we set the anchor size to 0
-                    dummy.SetSize( wxSize( 0,0 ) );
+                    dummy.SetSize( VECTOR2I( 0, 0 ) );
+                    // Restore pad position and offset
                     dummy.SetPosition( pad->GetPosition() );
+                    dummy.SetOffset( pad->GetOffset() );
                     dummy.SetOrientation( pad->GetOrientation() );
 
                     itemplotter.PlotPad( &dummy, color, padPlotMode );
@@ -507,14 +530,16 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                 // inflate/deflate a custom shape is a bit complex.
                 // so build a similar pad shape, and inflate/deflate the polygonal shape
                 PAD dummy( *pad );
+                dummy.SetParentGroup( nullptr );
+
                 SHAPE_POLY_SET shape;
                 pad->MergePrimitivesAsPolygon( &shape );
 
                 // Shape polygon can have holes so use InflateWithLinkedHoles(), not Inflate()
                 // which can create bad shapes if margin.x is < 0
-                int maxError = aBoard->GetDesignSettings().m_MaxError;
-                int numSegs = GetArcToSegmentCount( mask_clearance, maxError, FULL_CIRCLE );
-                shape.InflateWithLinkedHoles( mask_clearance, numSegs, SHAPE_POLY_SET::PM_FAST );
+                shape.InflateWithLinkedHoles( mask_clearance,
+                                              CORNER_STRATEGY::ROUND_ALL_CORNERS, maxError,
+                                              SHAPE_POLY_SET::PM_FAST );
                 dummy.DeletePrimitivesList();
                 dummy.AddPrimitivePoly( shape, 0, true );
 
@@ -522,7 +547,8 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                 // anchor will be added to the pad shape when plotting the pad. So now the
                 // polygonal shape is built, we can clamp the anchor size
                 if( mask_clearance < 0 )  // we expect margin.x = margin.y for custom pads
-                    dummy.SetSize( padPlotsSize );
+                    dummy.SetSize( VECTOR2I( std::max( 0, padPlotsSize.x ),
+                                             std::max( 0, padPlotsSize.y ) ) );
 
                 itemplotter.PlotPad( &dummy, color, padPlotMode );
                 break;
@@ -533,11 +559,10 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
             pad->SetSize( padSize );
             pad->SetDelta( padDelta );
             pad->SetShape( padShape );
-            pad->SetRoundRectCornerRadius( padCornerRadius );
+            pad->SetRoundRectRadiusRatio( padCornerRadiusRatio );
         }
 
         aPlotter->EndBlock( nullptr );
-        aPlotter->Bookmark( footprint->GetBoundingBox(), footprint->GetReference(), _( "Footprints" ) );
     }
 
     // Plot vias on copper layers, and if aPlotOpt.GetPlotViaOnMaskLayer() is true,
@@ -554,30 +579,31 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
     }
 
     aPlotter->StartBlock( nullptr );
+    bool plotViasOnMask = aPlotOpt.GetPlotViaOnMaskLayer();
+    LSET maskLayers( F_Mask );
+    maskLayers.set( B_Mask );
 
     for( const PCB_TRACK* track : aBoard->Tracks() )
     {
-        const PCB_VIA* via = dyn_cast<const PCB_VIA*>( track );
-
-        if( !via )
+        if( track->Type() != PCB_VIA_T )
             continue;
 
-        // vias are not plotted if not on selected layer, but if layer is SOLDERMASK_LAYER_BACK
-        // or SOLDERMASK_LAYER_FRONT, vias are drawn only if they are on the corresponding
-        // external copper layer
+        const PCB_VIA* via = static_cast<const PCB_VIA*>( track );
+
+        // vias are not plotted if not on selected layer
         LSET via_mask_layer = via->GetLayerSet();
 
-        if( aPlotOpt.GetPlotViaOnMaskLayer() )
+        // The board may have a different setting for via tenting than the plot options.  Plot
+        // options should win, so we want to handle mask layers differently from other layers here.
+        if( ( aLayerMask & maskLayers ).any() )
         {
-            if( via_mask_layer[B_Cu] )
-                via_mask_layer.set( B_Mask );
-
-            if( via_mask_layer[F_Cu] )
-                via_mask_layer.set( F_Mask );
+            if( !plotViasOnMask )
+                continue;
         }
-
-        if( !( via_mask_layer & aLayerMask ).any() )
+        else if( !( via_mask_layer & aLayerMask ).any() )
+        {
             continue;
+        }
 
         int via_margin = 0;
         double width_adj = 0;
@@ -636,31 +662,21 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
         if( track->Type() == PCB_ARC_T )
         {
-            const     PCB_ARC* arc = static_cast<const PCB_ARC*>( track );
+            const PCB_ARC* arc = static_cast<const PCB_ARC*>( track );
 
             // Too small arcs cannot be really handled: arc center (and arc radius)
             // cannot be safely computed
-            if( arc->IsDegenerated( 10 /* in IU */ ) )
+            if( !arc->IsDegenerated( 10 /* in IU */ ) )
+            {
+                aPlotter->ThickArc( arc->GetCenter(), arc->GetArcAngleStart(), arc->GetAngle(),
+                                    arc->GetRadius(), width, plotMode, &gbr_metadata );
+            }
+            else
             {
                 // Approximate this very small arc by a segment.
                 aPlotter->ThickSegment( track->GetStart(), track->GetEnd(), width, plotMode,
                                         &gbr_metadata );
             }
-            else
-            {
-                // ThickArc expects only positive angle arcs, so flip start/end if
-                // we are negative
-                if( arc->GetAngle() < ANGLE_0 )
-                {
-                    aPlotter->ThickArc( arc->GetCenter(), arc->GetEnd(), arc->GetStart(),
-                                        width, plotMode, &gbr_metadata );
-                }
-                else
-                {
-                    aPlotter->ThickArc( arc->GetCenter(), arc->GetStart(), arc->GetEnd(),
-                                        width, plotMode, &gbr_metadata );
-                }
-            }
         }
         else
         {
@@ -678,6 +694,9 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
     for( const ZONE* zone : aBoard->Zones() )
     {
+        if( zone->GetIsRuleArea() )
+            continue;
+
         for( PCB_LAYER_ID layer : zone->GetLayerSet().Seq() )
         {
             if( !aLayerMask[layer] )
@@ -695,13 +714,13 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                 }
             }
 
-            itemplotter.PlotFilledAreas( zone, layer, mainArea );
+            itemplotter.PlotZone( zone, layer, mainArea );
 
             if( !islands.IsEmpty() )
             {
                 ZONE dummy( *zone );
                 dummy.SetNet( &nonet );
-                itemplotter.PlotFilledAreas( &dummy, layer, islands );
+                itemplotter.PlotZone( &dummy, layer, islands );
             }
         }
     }
@@ -715,7 +734,7 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
 
 /**
- * Plot outlines of copper layer.
+ * Plot outlines.
  */
 void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                         const PCB_PLOT_PARAMS& aPlotOpt )
@@ -725,7 +744,7 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
     SHAPE_POLY_SET outlines;
 
-    for( LSEQ seq = aLayerMask.Seq( aLayerMask.SeqStackupBottom2Top() );  seq;  ++seq )
+    for( LSEQ seq = aLayerMask.Seq( aLayerMask.SeqStackupForPlotting() );  seq;  ++seq )
     {
         PCB_LAYER_ID layer = *seq;
 
@@ -743,8 +762,8 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
             for( int kk = 0; kk <= outlines.HoleCount(ii); kk++ )
             {
                 cornerList.clear();
-                const SHAPE_LINE_CHAIN& path =
-                        ( kk == 0 ) ? outlines.COutline( ii ) : outlines.CHole( ii, kk - 1 );
+                const SHAPE_LINE_CHAIN& path = ( kk == 0 ) ? outlines.COutline( ii )
+                                                           : outlines.CHole( ii, kk - 1 );
 
                 aPlotter->PlotPoly( path, FILL_T::NO_FILL );
             }
@@ -754,8 +773,8 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
         if( aPlotOpt.GetDrillMarksType() != DRILL_MARKS::NO_DRILL_SHAPE )
         {
             int smallDrill = ( aPlotOpt.GetDrillMarksType() == DRILL_MARKS::SMALL_DRILL_SHAPE )
-                             ? pcbIUScale.mmToIU( ADVANCED_CFG::GetCfg().m_SmallDrillMarkSize ) :
-                             INT_MAX;
+                                  ? pcbIUScale.mmToIU( ADVANCED_CFG::GetCfg().m_SmallDrillMarkSize )
+                                  : INT_MAX;
 
             for( FOOTPRINT* footprint : aBoard->Footprints() )
             {
@@ -773,8 +792,7 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                         else
                         {
                             // Note: small drill marks have no significance when applied to slots
-                            aPlotter->ThickSegment( slot->GetSeg().A,
-                                                    slot->GetSeg().B,
+                            aPlotter->ThickSegment( slot->GetSeg().A, slot->GetSeg().B,
                                                     slot->GetWidth(), SKETCH, nullptr );
                         }
                     }
@@ -785,9 +803,12 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
         // Plot vias holes
         for( PCB_TRACK* track : aBoard->Tracks() )
         {
-            const PCB_VIA* via = dyn_cast<const PCB_VIA*>( track );
+            if( track->Type() != PCB_VIA_T )
+                continue;
 
-            if( via && via->IsOnLayer( layer ) )    // via holes can be not through holes
+            const PCB_VIA* via = static_cast<const PCB_VIA*>( track );
+
+            if( via->GetLayerSet().Contains( layer ) )   // via holes can be not through holes
                 aPlotter->Circle( via->GetPosition(), via->GetDrillValue(), FILL_T::NO_FILL );
         }
     }
@@ -799,26 +820,16 @@ void PlotLayerOutlines( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
  *
  * Solder mask layers have a minimum thickness value and cannot be drawn like standard layers,
  * unless the minimum thickness is 0.
- * Currently the algo is:
- * 1 - build all pad shapes as polygons with a size inflated by
- *      mask clearance + (min width solder mask /2)
- * 2 - Merge shapes
- * 3 - deflate result by (min width solder mask /2)
- * 4 - ORing result by all pad shapes as polygons with a size inflated by
- *      mask clearance only (because deflate sometimes creates shape artifacts)
- * 5 - draw result as polygons
  *
  * The algorithm is somewhat complicated to allow for min web thickness while also preserving
  * pad attributes in Gerber.
  *
- * create initial polygons for every shape (pad or polygon),
- * inflate and deflate polygons
- * with Min Thickness/2, and merges the result (like initial algo)
- * remove all initial polygons.
- * The remaining polygons are areas with thickness < min thickness
- * plot all initial shapes by flashing (or using regions) for pad and polygons
- * (shapes will be better) and remaining polygons to
- * remove areas with thickness < min thickness from final mask
+ * 1 - create initial polygons for every shape
+ * 2 - inflate and deflate polygons with Min Thickness/2, and merges the result
+ * 3 - substract all initial polygons from (2), leaving the areas where the thickness was less
+ *      than min thickness
+ * 4 - plot all initial shapes by flashing (or using regions), including Gerber attribute data
+ * 5 - plot remaining polygons from (2) (witout any Gerber attributes)
  */
 
 void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
@@ -844,17 +855,19 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
     // Extra margin is half the min width for solder mask, which is used to merge too-close shapes
     // (distance < aMinThickness), and will be removed when creating the actual shapes.
 
-    // Will contain shapes inflated by inflate value that will be merged and deflated by
-    // inflate value to build final polygons
-    // After calculations the remaining polygons are polygons to plot
+    // Will contain shapes inflated by inflate value that will be merged and deflated by inflate
+    // value to build final polygons
     SHAPE_POLY_SET areas;
 
     // Will contain exact shapes of all items on solder mask
     SHAPE_POLY_SET initialPolys;
 
     auto plotFPTextItem =
-            [&]( const FP_TEXT& aText )
+            [&]( const PCB_TEXT& aText )
             {
+                if( !itemplotter.GetPlotFPText() )
+                    return;
+
                 if( !aText.IsVisible() && !itemplotter.GetPlotInvisibleText()  )
                     return;
 
@@ -883,19 +896,25 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
             // add shapes inflated by aMinThickness/2 in areas
             footprint->TransformPadsToPolySet( areas, layer, inflate, maxError, ERROR_OUTSIDE );
 
-            if( itemplotter.GetPlotReference() && footprint->Reference().IsOnLayer( layer ) )
-                plotFPTextItem( footprint->Reference() );
+            for( const PCB_FIELD* field : footprint->Fields() )
+            {
+                if( field->IsReference() && !itemplotter.GetPlotReference() )
+                    continue;
 
-            if( itemplotter.GetPlotValue() && footprint->Value().IsOnLayer( layer ) )
-                plotFPTextItem( footprint->Value() );
+                if( field->IsValue() && !itemplotter.GetPlotValue() )
+                    continue;
+
+                if( field->IsOnLayer( layer ) )
+                    plotFPTextItem( static_cast<const PCB_TEXT&>( *field ) );
+            }
 
             for( const BOARD_ITEM* item : footprint->GraphicalItems() )
             {
                 if( item->IsOnLayer( layer ) )
                 {
-                    if( item->Type() == PCB_FP_TEXT_T )
+                    if( item->Type() == PCB_TEXT_T )
                     {
-                        plotFPTextItem( static_cast<const FP_TEXT&>( *item ) );
+                        plotFPTextItem( static_cast<const PCB_TEXT&>( *item ) );
                     }
                     else
                     {
@@ -914,10 +933,13 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
         // Plot (untented) vias
         for( const PCB_TRACK* track : aBoard->Tracks() )
         {
-            const PCB_VIA* via = dyn_cast<const PCB_VIA*>( track );
+            if( track->Type() != PCB_VIA_T )
+                continue;
+
+            const PCB_VIA* via = static_cast<const PCB_VIA*>( track );
 
             // Note: IsOnLayer() checks relevant mask layers of untented vias
-            if( !via || !via->IsOnLayer( layer ) )
+            if( !via->IsOnLayer( layer ) )
                 continue;
 
             int clearance = via->GetSolderMaskExpansion();
@@ -966,6 +988,9 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
 
         for( ZONE* zone : aBoard->Zones() )
         {
+            if( zone->GetIsRuleArea() )
+                continue;
+
             if( !zone->IsOnLayer( layer ) )
                 continue;
 
@@ -979,12 +1004,10 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
         }
     }
 
-    int numSegs = GetArcToSegmentCount( inflate, maxError, FULL_CIRCLE );
-
-    // Merge all polygons: After deflating, not merged (not overlapping) polygons
-    // will have the initial shape (with perhaps small changes due to deflating transform)
+    // Merge all polygons: After deflating, not merged (not overlapping) polygons will have the
+    // initial shape (with perhaps small changes due to deflating transform)
     areas.Simplify( SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
-    areas.Deflate( inflate, numSegs );
+    areas.Deflate( inflate, CORNER_STRATEGY::CHAMFER_ALL_CORNERS, maxError );
 
     // To avoid a lot of code, use a ZONE to handle and plot polygons, because our polygons look
     // exactly like filled areas in zones.
@@ -1000,7 +1023,7 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
     areas.BooleanAdd( initialPolys, SHAPE_POLY_SET::PM_FAST );
     areas.Fracture( SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
 
-    itemplotter.PlotFilledAreas( &zone, layer, areas );
+    itemplotter.PlotZone( &zone, layer, areas );
 }
 
 
@@ -1017,8 +1040,8 @@ static void initializePlotter( PLOTTER* aPlotter, const BOARD* aBoard,
     const PAGE_INFO& pageInfo = aBoard->GetPageSettings();
     const PAGE_INFO* sheet_info;
     double paperscale; // Page-to-paper ratio
-    wxSize paperSizeIU;
-    wxSize           pageSizeIU( pageInfo.GetSizeIU( pcbIUScale.IU_PER_MILS ) );
+    VECTOR2I paperSizeIU;
+    VECTOR2I pageSizeIU( pageInfo.GetSizeIU( pcbIUScale.IU_PER_MILS ) );
     bool autocenter = false;
 
     // Special options: to fit the sheet to an A4 sheet replace the paper size. However there
@@ -1133,9 +1156,11 @@ static void ConfigureHPGLPenSizes( HPGL_PLOTTER *aPlotter, const PCB_PLOT_PARAMS
  * @return the plotter object if OK, NULL if the file is not created (or has a problem).
  */
 PLOTTER* StartPlotBoard( BOARD *aBoard, const PCB_PLOT_PARAMS *aPlotOpts, int aLayer,
-                         const wxString& aFullFileName, const wxString& aSheetName,
-                         const wxString& aSheetPath )
+                         const wxString& aLayerName, const wxString& aFullFileName,
+                         const wxString& aSheetName, const wxString& aSheetPath )
 {
+    wxCHECK( aBoard && aPlotOpts, nullptr );
+
     // Create the plotter driver and set the few plotter specific options
     PLOTTER*    plotter = nullptr;
 
@@ -1158,7 +1183,7 @@ PLOTTER* StartPlotBoard( BOARD *aBoard, const PCB_PLOT_PARAMS *aPlotOpts, int aL
         break;
 
     case PLOT_FORMAT::PDF:
-        plotter = new PDF_PLOTTER();
+        plotter = new PDF_PLOTTER( aBoard->GetProject() );
         break;
 
     case PLOT_FORMAT::HPGL:
@@ -1195,9 +1220,7 @@ PLOTTER* StartPlotBoard( BOARD *aBoard, const PCB_PLOT_PARAMS *aPlotOpts, int aL
     KIGFX::PCB_RENDER_SETTINGS* renderSettings = new KIGFX::PCB_RENDER_SETTINGS();
     renderSettings->LoadColors( aPlotOpts->ColorSettings() );
     renderSettings->SetDefaultPenWidth( pcbIUScale.mmToIU( 0.0212 ) );  // Hairline at 1200dpi
-
-    if( aLayer >= 0 && aLayer < GAL_LAYER_ID_END )
-        renderSettings->SetLayerName( aBoard->GetLayerName( ToLAYER_ID( aLayer ) ) );
+    renderSettings->SetLayerName( aLayerName );
 
     plotter->SetRenderSettings( renderSettings );
 
@@ -1235,8 +1258,9 @@ PLOTTER* StartPlotBoard( BOARD *aBoard, const PCB_PLOT_PARAMS *aPlotOpts, int aL
             if( aPlotOpts->GetPlotFrameRef() )
             {
                 PlotDrawingSheet( plotter, aBoard->GetProject(), aBoard->GetTitleBlock(),
-                                aBoard->GetPageSettings(), &aBoard->GetProperties(), wxT( "1" ), 1,
-                                aSheetName, aSheetPath, aBoard->GetFileName() );
+                                  aBoard->GetPageSettings(), &aBoard->GetProperties(), wxT( "1" ),
+                                  1, aSheetName, aSheetPath, aBoard->GetFileName(),
+                                  renderSettings->GetLayerColor( LAYER_DRAWINGSHEET ) );
 
                 if( aPlotOpts->GetMirror() )
                     initializePlotter( plotter, aBoard, aPlotOpts );
