@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
@@ -31,7 +31,7 @@
  */
 
 #ifdef PROFILE
-#include <profile.h>
+#include <core/profile.h>
 #endif
 
 #include <ratsnest/ratsnest_data.h>
@@ -123,6 +123,8 @@ void RN_NET::kruskalMST( const std::vector<CN_EDGE> &aEdges )
         const std::shared_ptr<const CN_ANCHOR>& source = tmp.GetSourceNode();
         const std::shared_ptr<const CN_ANCHOR>& target = tmp.GetTargetNode();
 
+        wxCHECK2( source && !source->Dirty() && target && !target->Dirty(), continue );
+
         if( dset.unite( source->GetTag(), target->GetTag() ) )
         {
             if( tmp.GetWeight() > 0 )
@@ -166,7 +168,7 @@ public:
         m_allNodes.clear();
     }
 
-    void AddNode( std::shared_ptr<CN_ANCHOR> aNode )
+    void AddNode( const std::shared_ptr<CN_ANCHOR>& aNode )
     {
         m_allNodes.insert( aNode );
     }
@@ -201,10 +203,28 @@ public:
             anchorChains[anchors.size() - 1].push_back( n );
         }
 
-        if( anchors.size() < 2 )
+        if( anchors.empty() )
         {
             return;
         }
+        else if( anchors.size() == 1 )
+        {
+            // The anchors all have the same position, but may not have overlapping layers.
+            prev = nullptr;
+
+            for( const std::shared_ptr<CN_ANCHOR>& n : m_allNodes )
+            {
+                if( prev && !( prev->Parent()->GetLayerSet() & n->Parent()->GetLayerSet() ).any() )
+                {
+                    // Use a minimal but non-zero distance or the edge will be ignored
+                    mstEdges.emplace_back( prev, n, 1 );
+                }
+
+                prev = n;
+            }
+
+            return;
+        }
         else if( areNodesColinear( anchors ) )
         {
             // special case: all nodes are on the same line - there's no
@@ -328,12 +348,12 @@ void RN_NET::compute()
 }
 
 
-void RN_NET::optimizeRNEdges()
+void RN_NET::OptimizeRNEdges()
 {
     auto optimizeZoneAnchor =
             [&]( const VECTOR2I& aPos, const LSET& aLayerSet,
                  const std::shared_ptr<const CN_ANCHOR>& aAnchor,
-                 std::function<void( std::shared_ptr<const CN_ANCHOR> )> setOptimizedTo )
+                 const std::function<void( std::shared_ptr<const CN_ANCHOR> )>& setOptimizedTo )
             {
                 SEG::ecoord closest_dist_sq = ( aAnchor->Pos() - aPos ).SquaredEuclideanNorm();
                 VECTOR2I    closest_pt;
@@ -341,13 +361,17 @@ void RN_NET::optimizeRNEdges()
 
                 for( CN_ITEM* item : aAnchor->Item()->ConnectedItems() )
                 {
+                    // Don't consider shorted items
+                    if( aAnchor->Item()->Net() != item->Net() )
+                        continue;
+
                     CN_ZONE_LAYER* zoneLayer = dynamic_cast<CN_ZONE_LAYER*>( item );
 
                     if( zoneLayer && aLayerSet.test( zoneLayer->Layer() ) )
                     {
                         const std::vector<VECTOR2I>& pts = zoneLayer->GetOutline().CPoints();
 
-                        for( VECTOR2I pt : pts )
+                        for( const VECTOR2I& pt : pts )
                         {
                             SEG::ecoord dist_sq = ( pt - aPos ).SquaredEuclideanNorm();
 
@@ -368,54 +392,123 @@ void RN_NET::optimizeRNEdges()
     auto optimizeZoneToZoneAnchors =
             [&]( const std::shared_ptr<const CN_ANCHOR>& a,
                  const std::shared_ptr<const CN_ANCHOR>& b,
-                 std::function<void(const std::shared_ptr<const CN_ANCHOR>&)> setOptimizedATo,
-                 std::function<void(const std::shared_ptr<const CN_ANCHOR>&)> setOptimizedBTo )
+                 const std::function<void( const std::shared_ptr<const CN_ANCHOR>& )>&
+                         setOptimizedATo,
+                 const std::function<void( const std::shared_ptr<const CN_ANCHOR>& )>&
+                         setOptimizedBTo )
+    {
+        struct CENTER
+        {
+            VECTOR2I pt;
+            bool     valid = false;
+        };
+
+        struct DIST_PAIR
+        {
+            DIST_PAIR( int64_t aDistSq, size_t aIdA, size_t aIdB )
+                    : dist_sq( aDistSq ), idA( aIdA ), idB( aIdB )
+            {}
+
+            int64_t dist_sq;
+            size_t  idA;
+            size_t  idB;
+        };
+
+        const std::vector<CN_ITEM*>& connectedItemsA = a->Item()->ConnectedItems();
+        const std::vector<CN_ITEM*>& connectedItemsB = b->Item()->ConnectedItems();
+
+        std::vector<CENTER> centersA( connectedItemsA.size() );
+        std::vector<CENTER> centersB( connectedItemsB.size() );
+
+        for( size_t i = 0; i < connectedItemsA.size(); i++ )
+        {
+            CN_ITEM*       itemA = connectedItemsA[i];
+            CN_ZONE_LAYER* zoneLayerA = dynamic_cast<CN_ZONE_LAYER*>( itemA );
+
+            if( !zoneLayerA )
+                continue;
+
+            const SHAPE_LINE_CHAIN& shapeA = zoneLayerA->GetOutline();
+            centersA[i].pt = shapeA.BBox().GetCenter();
+            centersA[i].valid = true;
+        }
+
+        for( size_t i = 0; i < connectedItemsB.size(); i++ )
+        {
+            CN_ITEM*       itemB = connectedItemsB[i];
+            CN_ZONE_LAYER* zoneLayerB = dynamic_cast<CN_ZONE_LAYER*>( itemB );
+
+            if( !zoneLayerB )
+                continue;
+
+            const SHAPE_LINE_CHAIN& shapeB = zoneLayerB->GetOutline();
+            centersB[i].pt = shapeB.BBox().GetCenter();
+            centersB[i].valid = true;
+        }
+
+        std::vector<DIST_PAIR> pairsToTest;
+
+        for( size_t ia = 0; ia < centersA.size(); ia++ )
+        {
+            for( size_t ib = 0; ib < centersB.size(); ib++ )
             {
-                for( CN_ITEM* itemA : a->Item()->ConnectedItems() )
-                {
-                    CN_ZONE_LAYER* zoneLayerA = dynamic_cast<CN_ZONE_LAYER*>( itemA );
+                const CENTER& ca = centersA[ia];
+                const CENTER& cb = centersB[ib];
 
-                    if( !zoneLayerA )
-                        continue;
+                if( !ca.valid || !cb.valid )
+                    continue;
 
-                    for( CN_ITEM* itemB : b->Item()->ConnectedItems() )
-                    {
-                        CN_ZONE_LAYER* zoneLayerB = dynamic_cast<CN_ZONE_LAYER*>( itemB );
+                VECTOR2L pA( ca.pt );
+                VECTOR2L pB( cb.pt );
 
-                        if( zoneLayerB && zoneLayerB->Layer() == zoneLayerA->Layer() )
-                        {
-                            // Process the first matching layer.  We don't really care if it's
-                            // the "best" layer or not, as anything will be better than the
-                            // original anchors (which are connected to the zone and so certainly
-                            // don't look like they should have ratsnest lines coming off them).
+                int64_t dist_sq = ( pB - pA ).SquaredEuclideanNorm();
+                pairsToTest.emplace_back( dist_sq, ia, ib );
+            }
+        }
 
-                            VECTOR2I     startA = zoneLayerA->GetOutline().GetPoint( 0 );
-                            VECTOR2I     startB = zoneLayerB->GetOutline().GetPoint( 0 );
-                            const SHAPE* shapeA = &zoneLayerA->GetOutline();
-                            const SHAPE* shapeB = &zoneLayerB->GetOutline();
-                            int          startDist = ( startA - startB ).EuclideanNorm();
+        std::sort( pairsToTest.begin(), pairsToTest.end(),
+                   []( const DIST_PAIR& _a, const DIST_PAIR& _b )
+                   {
+                       return _a.dist_sq < _b.dist_sq;
+                   } );
 
-                            VECTOR2I ptA;
-                            shapeA->Collide( shapeB, startDist + 10, nullptr, &ptA );
-                            setOptimizedATo( std::make_shared<CN_ANCHOR>( ptA, zoneLayerA ) );
+        const int c_polyPairsLimit = 3;
 
-                            VECTOR2I ptB;
-                            shapeB->Collide( shapeA, startDist + 10, nullptr, &ptB );
-                            setOptimizedBTo( std::make_shared<CN_ANCHOR>( ptB, zoneLayerB ) );
-                        }
-                    }
-                }
-            };
+        for( size_t i = 0; i < pairsToTest.size() && i < c_polyPairsLimit; i++ )
+        {
+            const DIST_PAIR& pair = pairsToTest[i];
+
+            CN_ZONE_LAYER* zoneLayerA = static_cast<CN_ZONE_LAYER*>( connectedItemsA[pair.idA] );
+            CN_ZONE_LAYER* zoneLayerB = static_cast<CN_ZONE_LAYER*>( connectedItemsB[pair.idB] );
+
+            if( zoneLayerA == zoneLayerB )
+                continue;
+
+            const SHAPE_LINE_CHAIN& shapeA = zoneLayerA->GetOutline();
+            const SHAPE_LINE_CHAIN& shapeB = zoneLayerB->GetOutline();
+
+            VECTOR2I ptA;
+            VECTOR2I ptB;
+
+            if( shapeA.ClosestSegmentsFast( shapeB, ptA, ptB ) )
+            {
+                setOptimizedATo( std::make_shared<CN_ANCHOR>( ptA, zoneLayerA ) );
+                setOptimizedBTo( std::make_shared<CN_ANCHOR>( ptB, zoneLayerB ) );
+            }
+        }
+    };
 
     for( CN_EDGE& edge : m_rnEdges )
     {
         const std::shared_ptr<const CN_ANCHOR>& source = edge.GetSourceNode();
         const std::shared_ptr<const CN_ANCHOR>& target = edge.GetTargetNode();
 
+        wxCHECK2( source && !source->Dirty() && target && !target->Dirty(), continue );
+
         if( source->ConnectedItemsCount() == 0 )
         {
             optimizeZoneAnchor( source->Pos(), source->Parent()->GetLayerSet(), target,
-                                [&]( std::shared_ptr<const CN_ANCHOR> optimized )
+                                [&]( const std::shared_ptr<const CN_ANCHOR>& optimized )
                                 {
                                     edge.SetTargetNode( optimized );
                                 } );
@@ -423,7 +516,7 @@ void RN_NET::optimizeRNEdges()
         else if( target->ConnectedItemsCount() == 0 )
         {
             optimizeZoneAnchor( target->Pos(), target->Parent()->GetLayerSet(), source,
-                                [&]( std::shared_ptr<const CN_ANCHOR> optimized )
+                                [&]( const std::shared_ptr<const CN_ANCHOR>& optimized )
                                 {
                                     edge.SetSourceNode( optimized );
                                 } );
@@ -431,11 +524,11 @@ void RN_NET::optimizeRNEdges()
         else
         {
             optimizeZoneToZoneAnchors( source, target,
-                                       [&]( std::shared_ptr<const CN_ANCHOR> optimized )
+                                       [&]( const std::shared_ptr<const CN_ANCHOR>& optimized )
                                        {
                                            edge.SetSourceNode( optimized );
                                        },
-                                       [&]( std::shared_ptr<const CN_ANCHOR> optimized )
+                                       [&]( const std::shared_ptr<const CN_ANCHOR>& optimized )
                                        {
                                            edge.SetTargetNode( optimized );
                                        } );
@@ -448,18 +541,20 @@ void RN_NET::UpdateNet()
 {
     compute();
 
-#ifdef PROFILE
-    PROF_TIMER cnt( "optimize" );
-#endif
-    optimizeRNEdges();
-#ifdef PROFILE
-    cnt.Show();
-#endif
-
     m_dirty = false;
 }
 
 
+void RN_NET::RemoveInvalidRefs()
+{
+    for( CN_EDGE& edge : m_rnEdges )
+        edge.RemoveInvalidRefs();
+
+    for( CN_EDGE& edge : m_boardEdges )
+        edge.RemoveInvalidRefs();
+}
+
+
 void RN_NET::Clear()
 {
     m_rnEdges.clear();
