@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2013 CERN
- * Copyright (C) 2018-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2018-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -31,6 +31,7 @@
 #include <ratsnest/ratsnest_view_item.h>
 
 #include <kiface_base.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <pcbnew_settings.h>
 #include <pcb_painter.h>
 #include <ratsnest/ratsnest_data.h>
@@ -43,7 +44,8 @@
 
 
 RATSNEST_VIEW_ITEM::RATSNEST_VIEW_ITEM( std::shared_ptr<CONNECTIVITY_DATA> aData ) :
-        EDA_ITEM( NOT_USED ), m_data( std::move(aData) )
+        EDA_ITEM( NOT_USED ),
+        m_data( std::move( aData ) )
 {
 }
 
@@ -67,16 +69,16 @@ void RATSNEST_VIEW_ITEM::ViewDraw( int aLayer, KIGFX::VIEW* aView ) const
 
     constexpr int CROSS_SIZE = 200000;
 
-    auto gal = aView->GetGAL();
-	gal->SetIsStroke( true );
-    gal->SetIsFill( false );
-    gal->SetLineWidth( 1.0 );
-    auto cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() );
+    PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() );
 
     if( !cfg )
         return;
 
     auto rs = static_cast<KIGFX::PCB_RENDER_SETTINGS*>( aView->GetPainter()->GetSettings() );
+    KIGFX::GAL* gal = aView->GetGAL();
+    gal->SetIsStroke( true );
+    gal->SetIsFill( false );
+    gal->SetLineWidth( cfg->m_Display.m_RatsnestThickness / gal->GetWorldScale() );
 
     std::set<int>        highlightedNets = rs->GetHighlightNetCodes();
     const std::set<int>& hiddenNets      = rs->GetHiddenNets();
@@ -108,12 +110,12 @@ void RATSNEST_VIEW_ITEM::ViewDraw( int aLayer, KIGFX::VIEW* aView ) const
     }
 
     auto adjustColor =
-            [&]( COLOR4D& color, double brightnessDelta, double alpha )
+            [&]( COLOR4D& color_base, double brightnessDelta, double alpha )
             {
                 if( rs->GetColor( nullptr, LAYER_PCB_BACKGROUND ).GetBrightness() < 0.5 )
-                    return color.Brightened( brightnessDelta ).WithAlpha( std::min( alpha, 1.0 ) );
+                    return color_base.Brightened( brightnessDelta ).WithAlpha( std::min( alpha, 1.0 ) );
                 else
-                    return color.Darkened( brightnessDelta ).WithAlpha( std::min( alpha, 1.0 ) );
+                    return color_base.Darkened( brightnessDelta ).WithAlpha( std::min( alpha, 1.0 ) );
             };
 
     const bool curved_ratsnest = cfg->m_Display.m_DisplayRatsnestLinesCurved;
@@ -196,12 +198,13 @@ void RATSNEST_VIEW_ITEM::ViewDraw( int aLayer, KIGFX::VIEW* aView ) const
 
             const std::shared_ptr<const CN_ANCHOR>& sourceNode = edge.GetSourceNode();
             const std::shared_ptr<const CN_ANCHOR>& targetNode = edge.GetTargetNode();
-            const VECTOR2I                          source( sourceNode->Pos() );
-            const VECTOR2I                          target( targetNode->Pos() );
 
-            if( !sourceNode->Valid() || !targetNode->Valid() )
+            if( !sourceNode || sourceNode->Dirty() || !targetNode || targetNode->Dirty() )
                 continue;
 
+            const VECTOR2I source( sourceNode->Pos() );
+            const VECTOR2I target( targetNode->Pos() );
+
             bool enable =  !sourceNode->GetNoLine() && !targetNode->GetNoLine();
             bool show;
 
