@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013  CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -23,6 +23,7 @@
 #include <functional>
 using namespace std::placeholders;
 
+#include <gal/graphics_abstraction_layer.h>
 #include <pcb_painter.h>
 #include <pcbnew_settings.h>
 
@@ -95,10 +96,11 @@ void TOOL_BASE::Reset( RESET_REASON aReason )
 }
 
 
-ITEM* TOOL_BASE::pickSingleItem( const VECTOR2I& aWhere, int aNet, int aLayer, bool aIgnorePads,
-								 const std::vector<ITEM*> aAvoidItems )
+ITEM* TOOL_BASE::pickSingleItem( const VECTOR2I& aWhere, NET_HANDLE aNet, int aLayer,
+                                 bool aIgnorePads, const std::vector<ITEM*> aAvoidItems )
 {
     int tl = aLayer > 0 ? aLayer : getView()->GetTopLayer();
+    int maxSlopRadius = std::max( m_gridHelper->GetGrid().x, m_gridHelper->GetGrid().y );
 
     static const int candidateCount = 5;
     ITEM* prioritized[candidateCount];
@@ -122,9 +124,9 @@ ITEM* TOOL_BASE::pickSingleItem( const VECTOR2I& aWhere, int aNet, int aLayer, b
                 return false;
             };
 
-    for( bool useClearance : { false, true } )
+    for( int slopRadius : { 0, maxSlopRadius } )
     {
-        ITEM_SET candidates = m_router->QueryHoverItems( aWhere, useClearance );
+        ITEM_SET candidates = m_router->QueryHoverItems( aWhere, slopRadius );
 
         for( ITEM* item : candidates.Items() )
         {
@@ -148,7 +150,7 @@ ITEM* TOOL_BASE::pickSingleItem( const VECTOR2I& aWhere, int aNet, int aLayer, b
             {
                 continue;
             }
-            else if( aNet <= 0 || item->Net() == aNet )
+            else if( m_router->GetInterface()->GetNetCode( aNet) <= 0 || item->Net() == aNet )
             {
                 if( item->OfKind( ITEM::VIA_T | ITEM::SOLID_T ) )
                 {
@@ -234,11 +236,15 @@ ITEM* TOOL_BASE::pickSingleItem( const VECTOR2I& aWhere, int aNet, int aLayer, b
 }
 
 
-void TOOL_BASE::highlightNets( bool aEnabled, std::set<int> aNetcodes )
+void TOOL_BASE::highlightNets( bool aEnabled, std::set<NET_HANDLE> aNets )
 {
     RENDER_SETTINGS* rs = getView()->GetPainter()->GetSettings();
+    std::set<int>    netcodes;
 
-    if( aNetcodes.size() > 0 && aEnabled )
+    for( const NET_HANDLE& net : aNets )
+        netcodes.insert( m_router->GetInterface()->GetNetCode( net ) );
+
+    if( netcodes.size() > 0 && aEnabled )
     {
         // If the user has previously set some of the routed nets to be highlighted,
         // we assume they want to keep them highlighted after routing
@@ -246,7 +252,7 @@ void TOOL_BASE::highlightNets( bool aEnabled, std::set<int> aNetcodes )
         const std::set<int>& currentNetCodes = rs->GetHighlightNetCodes();
         bool                 keep = false;
 
-        for( const int& netcode : aNetcodes )
+        for( const int& netcode : netcodes )
         {
             if( currentNetCodes.find( netcode ) != currentNetCodes.end() )
             {
@@ -260,7 +266,7 @@ void TOOL_BASE::highlightNets( bool aEnabled, std::set<int> aNetcodes )
         else
             m_startHighlightNetcodes.clear();
 
-        rs->SetHighlight( aNetcodes, true );
+        rs->SetHighlight( netcodes, true );
     }
     else
     {
@@ -276,28 +282,26 @@ void TOOL_BASE::highlightNets( bool aEnabled, std::set<int> aNetcodes )
 bool TOOL_BASE::checkSnap( ITEM *aItem )
 {
     // Sync PNS engine settings with the general PCB editor options.
-    auto& pnss = m_router->Settings();
+    ROUTING_SETTINGS& pnss = m_router->Settings();
 
     // If we're dragging a track segment, don't try to snap to items that are part of the original line.
     if( m_startItem && aItem && m_router->GetState() == ROUTER::DRAG_SEGMENT
         && m_router->GetDragger() )
     {
         DRAGGER*     dragger = dynamic_cast<DRAGGER*>( m_router->GetDragger() );
-        LINKED_ITEM* liItem = dynamic_cast<LINKED_ITEM*>( aItem );
+        LINKED_ITEM* linkedItem = dynamic_cast<LINKED_ITEM*>( aItem );
 
-        if( dragger && liItem && dragger->GetOriginalLine().ContainsLink( liItem ) )
-        {
+        if( dragger && linkedItem && dragger->GetOriginalLine().ContainsLink( linkedItem ) )
             return false;
-        }
     }
 
-    pnss.SetSnapToPads(
-            frame()->GetMagneticItemsSettings()->pads == MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL ||
-            frame()->GetMagneticItemsSettings()->pads == MAGNETIC_OPTIONS::CAPTURE_ALWAYS );
+    MAGNETIC_SETTINGS* magSettings = frame()->GetMagneticItemsSettings();
 
-    pnss.SetSnapToTracks(
-            frame()->GetMagneticItemsSettings()->tracks == MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL
-            || frame()->GetMagneticItemsSettings()->tracks == MAGNETIC_OPTIONS::CAPTURE_ALWAYS );
+    pnss.SetSnapToPads( magSettings->pads == MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL
+                        || magSettings->pads == MAGNETIC_OPTIONS::CAPTURE_ALWAYS );
+
+    pnss.SetSnapToTracks( magSettings->tracks == MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL
+                          || magSettings->tracks == MAGNETIC_OPTIONS::CAPTURE_ALWAYS );
 
     if( aItem )
     {
@@ -314,26 +318,19 @@ bool TOOL_BASE::checkSnap( ITEM *aItem )
 void TOOL_BASE::updateStartItem( const TOOL_EVENT& aEvent, bool aIgnorePads )
 {
     int      tl = getView()->GetTopLayer();
-    VECTOR2I cp = aEvent.IsPrime() ? aEvent.Position()
-                                   : controls()->GetCursorPosition( !aEvent.Modifier( MD_SHIFT ) );
-    VECTOR2I p;
     GAL*     gal = m_toolMgr->GetView()->GetGAL();
+    VECTOR2I pos = aEvent.HasPosition() ? (VECTOR2I) aEvent.Position() : m_startSnapPoint;
 
     controls()->ForceCursorPosition( false );
     m_gridHelper->SetUseGrid( gal->GetGridSnapping() && !aEvent.DisableGridSnapping()  );
     m_gridHelper->SetSnap( !aEvent.Modifier( MD_SHIFT ) );
 
-    if( aEvent.IsMotion() || aEvent.IsClick() )
-        p = aEvent.Position();
-    else
-        p = cp;
-
-    m_startItem = pickSingleItem( aEvent.IsClick() ? cp : p, -1, -1, aIgnorePads );
+    m_startItem = pickSingleItem( pos, nullptr, -1, aIgnorePads );
 
     if( !m_gridHelper->GetUseGrid() && m_startItem && !m_startItem->Layers().Overlaps( tl ) )
         m_startItem = nullptr;
 
-    m_startSnapPoint = snapToItem( m_startItem, p );
+    m_startSnapPoint = snapToItem( m_startItem, pos );
     controls()->ForceCursorPosition( true, m_startSnapPoint );
 }
 
@@ -359,7 +356,7 @@ void TOOL_BASE::updateEndItem( const TOOL_EVENT& aEvent )
     }
 
     if( m_router->Settings().Mode() != RM_MarkObstacles &&
-        ( m_router->GetCurrentNets().empty() || m_router->GetCurrentNets().front() < 0 ) )
+        ( m_router->GetCurrentNets().empty() || m_router->GetCurrentNets().front() == nullptr ) )
     {
         m_endSnapPoint = snapToItem( nullptr, mousePos );
         controls()->ForceCursorPosition( true, m_endSnapPoint );
@@ -375,9 +372,9 @@ void TOOL_BASE::updateEndItem( const TOOL_EVENT& aEvent )
 
     ITEM* endItem = nullptr;
 
-    std::vector<int> nets = m_router->GetCurrentNets();
+    std::vector<NET_HANDLE> nets = m_router->GetCurrentNets();
 
-    for( int net : nets )
+    for( NET_HANDLE net : nets )
     {
         endItem = pickSingleItem( mousePos, net, layer, false, { m_startItem } );
 
@@ -393,7 +390,8 @@ void TOOL_BASE::updateEndItem( const TOOL_EVENT& aEvent )
     else
     {
         m_endItem = nullptr;
-        m_endSnapPoint = m_gridHelper->Align( mousePos );
+        m_endSnapPoint = m_gridHelper->Align( mousePos, m_router->IsPlacingVia() ? GRID_VIAS
+                                                                                 : GRID_WIRES );
     }
 
     controls()->ForceCursorPosition( true, m_endSnapPoint );
@@ -413,17 +411,44 @@ ROUTER *TOOL_BASE::Router() const
 }
 
 
+PNS_KICAD_IFACE* TOOL_BASE::GetInterface() const
+{
+    return m_iface;
+}
+
+
 const VECTOR2I TOOL_BASE::snapToItem( ITEM* aItem, const VECTOR2I& aP )
 {
     if( !aItem || !m_iface->IsItemVisible( aItem ) )
     {
-        return m_gridHelper->Align( aP );
+        return m_gridHelper->Align( aP, m_router->IsPlacingVia() ? GRID_VIAS : GRID_WIRES );
     }
 
     switch( aItem->Kind() )
     {
     case ITEM::SOLID_T:
-        return static_cast<SOLID*>( aItem )->Pos();
+    {
+        SOLID* solid = static_cast<SOLID*>( aItem );
+
+        if( solid->AnchorPoints().empty() )
+            return solid->Anchor( 0 );
+
+        VECTOR2I anchor;
+        SEG::ecoord minDist = std::numeric_limits<SEG::ecoord>::max();
+
+        for( VECTOR2I anchorCandidate : solid->AnchorPoints() )
+        {
+            SEG::ecoord distSq = ( aP - anchorCandidate ).SquaredEuclideanNorm();
+
+            if( distSq < minDist )
+            {
+                minDist = distSq;
+                anchor  = anchorCandidate;
+            }
+        }
+
+        return anchor;
+    }
 
     case ITEM::VIA_T:
         return static_cast<VIA*>( aItem )->Pos();
@@ -461,7 +486,7 @@ const VECTOR2I TOOL_BASE::snapToItem( ITEM* aItem, const VECTOR2I& aP )
         break;
     }
 
-    return m_gridHelper->Align( aP );
+    return m_gridHelper->Align( aP, m_router->IsPlacingVia() ? GRID_VIAS : GRID_WIRES );
 }
 
 }
