@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2015 CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -21,6 +21,7 @@
 
 #include "pns_line.h"
 #include "pns_segment.h"
+#include "pns_arc.h"
 #include "pns_node.h"
 #include "pns_joint.h"
 #include "pns_solid.h"
@@ -59,26 +60,26 @@ bool TOPOLOGY::SimplifyLine( LINE* aLine )
 }
 
 
-const TOPOLOGY::JOINT_SET TOPOLOGY::ConnectedJoints( JOINT* aStart )
+const TOPOLOGY::JOINT_SET TOPOLOGY::ConnectedJoints( const JOINT* aStart )
 {
-    std::deque<JOINT*> searchQueue;
-    JOINT_SET processed;
+    std::deque<const JOINT*> searchQueue;
+    JOINT_SET                processed;
 
     searchQueue.push_back( aStart );
     processed.insert( aStart );
 
     while( !searchQueue.empty() )
     {
-        JOINT* current = searchQueue.front();
+        const JOINT* current = searchQueue.front();
         searchQueue.pop_front();
 
         for( ITEM* item : current->LinkList() )
         {
-            if( item->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T ) )
+            if( item->OfKind( ITEM::SEGMENT_T ) )
             {
-                JOINT* a = m_world->FindJoint( item->Anchor( 0 ), item );;
-                JOINT* b = m_world->FindJoint( item->Anchor( 1 ), item );;
-                JOINT* next = ( *a == *current ) ? b : a;
+                const JOINT* a = m_world->FindJoint( item->Anchor( 0 ), item );;
+                const JOINT* b = m_world->FindJoint( item->Anchor( 1 ), item );;
+                const JOINT* next = ( *a == *current ) ? b : a;
 
                 if( processed.find( next ) == processed.end() )
                 {
@@ -94,7 +95,7 @@ const TOPOLOGY::JOINT_SET TOPOLOGY::ConnectedJoints( JOINT* aStart )
 
 
 bool TOPOLOGY::NearestUnconnectedAnchorPoint( const LINE* aTrack, VECTOR2I& aPoint,
-                                              LAYER_RANGE& aLayers )
+                                              LAYER_RANGE& aLayers, ITEM*& aItem )
 {
     LINE track( *aTrack );
     VECTOR2I end;
@@ -105,9 +106,9 @@ bool TOPOLOGY::NearestUnconnectedAnchorPoint( const LINE* aTrack, VECTOR2I& aPoi
     std::unique_ptr<NODE> tmpNode( m_world->Branch() );
     tmpNode->Add( track );
 
-    JOINT* jt = tmpNode->FindJoint( track.CPoint( -1 ), &track );
+    const JOINT* jt = tmpNode->FindJoint( track.CPoint( -1 ), &track );
 
-    if( !jt || jt->Net() <= 0 )
+    if( !jt || m_world->GetRuleResolver()->NetCode( jt->Net() ) <= 0 )
        return false;
 
     if( ( !track.EndsWithVia() && jt->LinkCount() >= 2 )
@@ -115,6 +116,7 @@ bool TOPOLOGY::NearestUnconnectedAnchorPoint( const LINE* aTrack, VECTOR2I& aPoi
     {
         end = jt->Pos();
         aLayers = jt->Layers();
+        aItem = jt->LinkList()[0];
     }
     else
     {
@@ -128,6 +130,7 @@ bool TOPOLOGY::NearestUnconnectedAnchorPoint( const LINE* aTrack, VECTOR2I& aPoi
 
         end = it->Anchor( anchor );
         aLayers = it->Layers();
+        aItem = it;
     }
 
     aPoint = end;
@@ -140,8 +143,9 @@ bool TOPOLOGY::LeadingRatLine( const LINE* aTrack, SHAPE_LINE_CHAIN& aRatLine )
     VECTOR2I end;
     // Ratline doesn't care about the layer
     LAYER_RANGE layers;
+    ITEM*       unusedItem;
 
-    if( !NearestUnconnectedAnchorPoint( aTrack, end, layers ) )
+    if( !NearestUnconnectedAnchorPoint( aTrack, end, layers, unusedItem ) )
         return false;
 
     aRatLine.Clear();
@@ -151,7 +155,7 @@ bool TOPOLOGY::LeadingRatLine( const LINE* aTrack, SHAPE_LINE_CHAIN& aRatLine )
 }
 
 
-ITEM* TOPOLOGY::NearestUnconnectedItem( JOINT* aStart, int* aAnchor, int aKindMask )
+ITEM* TOPOLOGY::NearestUnconnectedItem( const JOINT* aStart, int* aAnchor, int aKindMask )
 {
     std::set<ITEM*> disconnected;
 
@@ -194,29 +198,40 @@ ITEM* TOPOLOGY::NearestUnconnectedItem( JOINT* aStart, int* aAnchor, int aKindMa
 }
 
 
-bool TOPOLOGY::followTrivialPath( LINE* aLine, bool aLeft, ITEM_SET& aSet,
-                                  std::set<ITEM*>& aVisited, JOINT** aTerminalJoint )
+bool TOPOLOGY::followTrivialPath( LINE* aLine2, bool aLeft, ITEM_SET& aSet,
+                                  const JOINT** aTerminalJoint, bool aFollowLockedSegments )
 {
-    assert( aLine->IsLinked() );
+    assert( aLine2->IsLinked() );
+    LINE*           curr_line = aLine2;
+    std::set<ITEM*> visited;
 
-    VECTOR2I     anchor = aLeft ? aLine->CPoint( 0 ) : aLine->CPoint( -1 );
-    LINKED_ITEM* last   = aLeft ? aLine->Links().front() : aLine->Links().back();
-    JOINT*       jt     = m_world->FindJoint( anchor, aLine );
-
-    assert( jt != nullptr );
-
-    aVisited.insert( last );
-
-    if( jt->IsNonFanoutVia() || jt->IsTraceWidthChange() )
+    while( true )
     {
-        ITEM* via = nullptr;
+        VECTOR2I     anchor = aLeft ? curr_line->CPoint( 0 ) : curr_line->CPoint( -1 );
+        LINKED_ITEM* last = aLeft ? curr_line->Links().front() : curr_line->Links().back();
+        const JOINT* jt = m_world->FindJoint( anchor, curr_line );
+
+        assert( jt != nullptr );
+
+        if( !visited.insert( last ).second
+            || ( !jt->IsNonFanoutVia() && !jt->IsTraceWidthChange() ) )
+        {
+            if( aTerminalJoint )
+                *aTerminalJoint = jt;
+
+            return false;
+        }
+
+        ITEM*    via = nullptr;
         SEGMENT* next_seg = nullptr;
 
-        for( ITEM* link : jt->Links().Items() )
+        ITEM_SET links( jt->CLinks() );
+
+        for( ITEM* link : links )
         {
             if( link->OfKind( ITEM::VIA_T ) )
                 via = link;
-            else if( aVisited.find( link ) == aVisited.end() )
+            else if( visited.insert( link ).second )
                 next_seg = static_cast<SEGMENT*>( link );
         }
 
@@ -228,8 +243,7 @@ bool TOPOLOGY::followTrivialPath( LINE* aLine, bool aLeft, ITEM_SET& aSet,
             return false;
         }
 
-        LINE l = m_world->AssembleLine( next_seg );
-
+        LINE     l = m_world->AssembleLine( next_seg, nullptr, false, aFollowLockedSegments );
         VECTOR2I nextAnchor = ( aLeft ? l.CLine().CPoint( -1 ) : l.CLine().CPoint( 0 ) );
 
         if( nextAnchor != anchor )
@@ -243,6 +257,7 @@ bool TOPOLOGY::followTrivialPath( LINE* aLine, bool aLeft, ITEM_SET& aSet,
                 aSet.Prepend( via );
 
             aSet.Prepend( l );
+            curr_line = static_cast<PNS::LINE*>( aSet[0] );
         }
         else
         {
@@ -250,39 +265,36 @@ bool TOPOLOGY::followTrivialPath( LINE* aLine, bool aLeft, ITEM_SET& aSet,
                 aSet.Add( via );
 
             aSet.Add( l );
+            curr_line = static_cast<PNS::LINE*>( aSet[aSet.Size() - 1] );
         }
 
-        return followTrivialPath( &l, aLeft, aSet, aVisited, aTerminalJoint );
+        continue;
     }
-
-    if( aTerminalJoint )
-        *aTerminalJoint = jt;
-
-    return false;
 }
 
 
 const ITEM_SET TOPOLOGY::AssembleTrivialPath( ITEM* aStart,
-                                              std::pair<JOINT*, JOINT*>* aTerminalJoints,
+                                              std::pair<const JOINT*, const JOINT*>* aTerminalJoints,
                                               bool aFollowLockedSegments )
 {
     ITEM_SET        path;
-    std::set<ITEM*> visited;
     LINKED_ITEM*    seg = nullptr;
 
     if( aStart->Kind() == ITEM::VIA_T )
     {
-        VIA*   via = static_cast<VIA*>( aStart );
-        JOINT* jt  = m_world->FindJoint( via->Pos(), via );
+        VIA*         via = static_cast<VIA*>( aStart );
+        const JOINT* jt  = m_world->FindJoint( via->Pos(), via );
 
         if( !jt->IsNonFanoutVia() )
             return ITEM_SET();
 
-        for( const ITEM_SET::ENTRY& entry : jt->Links().Items() )
+        ITEM_SET links( jt->CLinks() );
+
+        for( ITEM* item : links )
         {
-            if( entry.item->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T ) )
+            if( item->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T ) )
             {
-                seg = static_cast<LINKED_ITEM*>( entry.item );
+                seg = static_cast<LINKED_ITEM*>( item );
                 break;
             }
         }
@@ -297,15 +309,15 @@ const ITEM_SET TOPOLOGY::AssembleTrivialPath( ITEM* aStart,
 
     // Assemble a line following through locked segments
     // TODO: consider if we want to allow tuning lines with different widths in the future
-    LINE l = m_world->AssembleLine( seg, nullptr, false, true );
+    LINE l = m_world->AssembleLine( seg, nullptr, false, aFollowLockedSegments );
 
     path.Add( l );
 
-    JOINT* jointA = nullptr;
-    JOINT* jointB = nullptr;
+    const JOINT* jointA = nullptr;
+    const JOINT* jointB = nullptr;
 
-    followTrivialPath( &l, false, path, visited, &jointB );
-    followTrivialPath( &l, true, path, visited, &jointA );
+    followTrivialPath( &l, false, path, &jointB, aFollowLockedSegments );
+    followTrivialPath( &l, true, path, &jointA, aFollowLockedSegments );
 
     if( aTerminalJoints )
     {
@@ -319,14 +331,14 @@ const ITEM_SET TOPOLOGY::AssembleTrivialPath( ITEM* aStart,
 
 const ITEM_SET TOPOLOGY::AssembleTuningPath( ITEM* aStart, SOLID** aStartPad, SOLID** aEndPad )
 {
-    std::pair<JOINT*, JOINT*> joints;
+    std::pair<const JOINT*, const JOINT*> joints;
     ITEM_SET initialPath = AssembleTrivialPath( aStart, &joints, true );
 
     PAD* padA = nullptr;
     PAD* padB = nullptr;
 
     auto getPadFromJoint =
-            []( JOINT* aJoint, PAD** aTargetPad, SOLID** aTargetSolid )
+            []( const JOINT* aJoint, PAD** aTargetPad, SOLID** aTargetSolid )
             {
                 for( ITEM* item : aJoint->LinkList() )
                 {
@@ -359,7 +371,7 @@ const ITEM_SET TOPOLOGY::AssembleTuningPath( ITEM* aStart, SOLID** aStartPad, SO
     auto clipLineToPad =
             []( SHAPE_LINE_CHAIN& aLine, PAD* aPad, bool aForward = true )
             {
-                const std::shared_ptr<SHAPE_POLY_SET>& shape = aPad->GetEffectivePolygon();
+                const auto& shape = aPad->GetEffectivePolygon( ERROR_INSIDE );
 
                 int start = aForward ? 0 : aLine.PointCount() - 1;
                 int delta = aForward ? 1 : -1;
@@ -403,9 +415,9 @@ const ITEM_SET TOPOLOGY::AssembleTuningPath( ITEM* aStart, SOLID** aStartPad, SO
             };
 
     auto processPad =
-            [&]( JOINT* aJoint, PAD* aPad )
+            [&]( const JOINT* aJoint, PAD* aPad )
             {
-                const std::shared_ptr<SHAPE_POLY_SET>& shape = aPad->GetEffectivePolygon();
+                const auto& shape = aPad->GetEffectivePolygon( ERROR_INSIDE );
 
                 for( int idx = 0; idx < initialPath.Size(); idx++ )
                 {
@@ -441,7 +453,7 @@ const ITEM_SET TOPOLOGY::AssembleTuningPath( ITEM* aStart, SOLID** aStartPad, SO
 }
 
 
-const ITEM_SET TOPOLOGY::ConnectedItems( JOINT* aStart, int aKindMask )
+const ITEM_SET TOPOLOGY::ConnectedItems( const JOINT* aStart, int aKindMask )
 {
     return ITEM_SET();
 }
@@ -458,69 +470,146 @@ bool commonParallelProjection( SEG p, SEG n, SEG &pClip, SEG& nClip );
 
 bool TOPOLOGY::AssembleDiffPair( ITEM* aStart, DIFF_PAIR& aPair )
 {
-    int refNet = aStart->Net();
-    int coupledNet = m_world->GetRuleResolver()->DpCoupledNet( refNet );
+    NET_HANDLE   refNet = aStart->Net();
+    NET_HANDLE   coupledNet = m_world->GetRuleResolver()->DpCoupledNet( refNet );
+    LINKED_ITEM* startItem = dynamic_cast<LINKED_ITEM*>( aStart );
 
-    if( coupledNet < 0 )
+    if( !coupledNet || !startItem )
         return false;
 
-    std::set<ITEM*> coupledItems;
+    LINE lp = m_world->AssembleLine( startItem );
 
+    std::vector<ITEM*> pItems;
+    std::vector<ITEM*> nItems;
+
+    for( ITEM* item : lp.Links() )
+    {
+        if( item->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T ) && item->Layers() == startItem->Layers() )
+            pItems.push_back( item );
+    }
+
+    std::set<ITEM*> coupledItems;
     m_world->AllItemsInNet( coupledNet, coupledItems );
 
-    SEGMENT* coupledSeg = nullptr, *refSeg;
-    int minDist = std::numeric_limits<int>::max();
-
-    if( ( refSeg = dyn_cast<SEGMENT*>( aStart ) ) != nullptr )
+    for( ITEM* item : coupledItems )
     {
-        for( ITEM* item : coupledItems )
+        if( item->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T ) && item->Layers() == startItem->Layers() )
+            nItems.push_back( item );
+    }
+
+    LINKED_ITEM* refItem = nullptr;
+    LINKED_ITEM* coupledItem = nullptr;
+    SEG::ecoord  minDist_sq = std::numeric_limits<SEG::ecoord>::max();
+    SEG::ecoord  minDistTarget_sq = std::numeric_limits<SEG::ecoord>::max();
+    VECTOR2I     targetPoint = aStart->Shape()->Centre();
+
+    auto findNItem = [&]( ITEM* p_item )
+    {
+        for( ITEM* n_item : nItems )
         {
-            if( SEGMENT* s = dyn_cast<SEGMENT*>( item ) )
+            SEG::ecoord dist_sq = std::numeric_limits<SEG::ecoord>::max();
+
+            if( n_item->Kind() != p_item->Kind() )
+                continue;
+
+            if( p_item->Kind() == ITEM::SEGMENT_T )
             {
-                if( s->Layers().Start() == refSeg->Layers().Start() &&
-                    s->Width() == refSeg->Width() )
+                const SEGMENT* p_seg = static_cast<const SEGMENT*>( p_item );
+                const SEGMENT* n_seg = static_cast<const SEGMENT*>( n_item );
+
+                if( n_seg->Width() != p_seg->Width() )
+                    continue;
+
+                if( !p_seg->Seg().ApproxParallel( n_seg->Seg(), DP_PARALLELITY_THRESHOLD ) )
+                    continue;
+
+                SEG p_clip, n_clip;
+
+                if( !commonParallelProjection( p_seg->Seg(), n_seg->Seg(), p_clip, n_clip ) )
+                    continue;
+
+                dist_sq = n_seg->Seg().SquaredDistance( p_seg->Seg() );
+            }
+            else if( p_item->Kind() == ITEM::ARC_T )
+            {
+                const ARC* p_arc = static_cast<const ARC*>( p_item );
+                const ARC* n_arc = static_cast<const ARC*>( n_item );
+
+                if( n_arc->Width() != p_arc->Width() )
+                    continue;
+
+                VECTOR2I    centerDiff = n_arc->CArc().GetCenter() - p_arc->CArc().GetCenter();
+                SEG::ecoord centerDist_sq = centerDiff.SquaredEuclideanNorm();
+
+                if( centerDist_sq > SEG::Square( DP_PARALLELITY_THRESHOLD ) )
+                    continue;
+
+                dist_sq = SEG::Square( p_arc->CArc().GetRadius() - n_arc->CArc().GetRadius() );
+            }
+
+            if( dist_sq <= minDist_sq )
+            {
+                SEG::ecoord distTarget_sq = n_item->Shape()->SquaredDistance( targetPoint );
+                if( distTarget_sq < minDistTarget_sq )
                 {
-                    int dist = s->Seg().Distance( refSeg->Seg() );
-                    bool isParallel = refSeg->Seg().ApproxParallel( s->Seg(), DP_PARALLELITY_THRESHOLD );
-                    SEG p_clip, n_clip;
+                    minDistTarget_sq = distTarget_sq;
+                    minDist_sq = dist_sq;
 
-                    bool isCoupled = commonParallelProjection( refSeg->Seg(), s->Seg(), p_clip,
-                                                               n_clip );
-
-                    if( isParallel && isCoupled && dist < minDist )
-                    {
-                        minDist = dist;
-                        coupledSeg = s;
-                    }
+                    refItem = static_cast<LINKED_ITEM*>( p_item );
+                    coupledItem = static_cast<LINKED_ITEM*>( n_item );
                 }
             }
         }
-    }
-    else
+    };
+
+    findNItem( startItem );
+
+    if( !coupledItem )
     {
-        return false;
+        LINKED_ITEM*    linked = static_cast<LINKED_ITEM*>( startItem );
+        std::set<ITEM*> linksToTest;
+
+        for( int i = 0; i < linked->AnchorCount(); i++ )
+        {
+            const JOINT* jt = m_world->FindJoint( linked->Anchor( i ), linked );
+
+            if( !jt )
+                continue;
+
+            for( ITEM* link : jt->LinkList() )
+            {
+                if( link != linked )
+                    linksToTest.emplace( link );
+            }
+        }
+
+        for( ITEM* link : linksToTest )
+            findNItem( link );
     }
 
-    if( !coupledSeg )
+    if( !coupledItem )
         return false;
 
-    LINE lp = m_world->AssembleLine( refSeg );
-    LINE ln = m_world->AssembleLine( coupledSeg );
+    LINE ln = m_world->AssembleLine( coupledItem );
 
     if( m_world->GetRuleResolver()->DpNetPolarity( refNet ) < 0 )
-    {
         std::swap( lp, ln );
-    }
 
     int gap = -1;
 
-    if( refSeg->Seg().ApproxParallel( coupledSeg->Seg(), DP_PARALLELITY_THRESHOLD ) )
+    if( refItem && refItem->Kind() == ITEM::SEGMENT_T )
     {
         // Segments are parallel -> compute pair gap
-        const VECTOR2I refDir       = refSeg->Anchor( 1 ) - refSeg->Anchor( 0 );
-        const VECTOR2I displacement = refSeg->Anchor( 1 ) - coupledSeg->Anchor( 1 );
+        const VECTOR2I refDir       = refItem->Anchor( 1 ) - refItem->Anchor( 0 );
+        const VECTOR2I displacement = refItem->Anchor( 1 ) - coupledItem->Anchor( 1 );
         gap = (int) std::abs( refDir.Cross( displacement ) / refDir.EuclideanNorm() ) - lp.Width();
     }
+    else if( refItem && refItem->Kind() == ITEM::ARC_T )
+    {
+        const ARC* refArc = static_cast<ARC*>( refItem );
+        const ARC* coupledArc = static_cast<ARC*>( coupledItem );
+        gap = (int) std::abs( refArc->CArc().GetRadius() - coupledArc->CArc().GetRadius() ) - lp.Width();
+    }
 
     aPair = DIFF_PAIR( lp, ln );
     aPair.SetWidth( lp.Width() );
@@ -535,6 +624,11 @@ const std::set<ITEM*> TOPOLOGY::AssembleCluster( ITEM* aStart, int aLayer )
     std::set<ITEM*> visited;
     std::deque<ITEM*> pending;
 
+    COLLISION_SEARCH_OPTIONS opts;
+
+    opts.m_differentNetsOnly = false;
+    opts.m_overrideClearance = 0;
+
     pending.push_back( aStart );
 
     while( !pending.empty() )
@@ -546,9 +640,9 @@ const std::set<ITEM*> TOPOLOGY::AssembleCluster( ITEM* aStart, int aLayer )
 
         visited.insert( top );
 
-        m_world->QueryColliding( top, obstacles, ITEM::ANY_T, -1, false, 0 ); // only query touching objects
+        m_world->QueryColliding( top, obstacles, opts ); // only query touching objects
 
-        for( OBSTACLE& obs : obstacles )
+        for( const OBSTACLE& obs : obstacles )
         {
             bool trackOnTrack = ( obs.m_item->Net() != top->Net() ) &&  obs.m_item->OfKind( ITEM::SEGMENT_T ) && top->OfKind( ITEM::SEGMENT_T );
 
