@@ -1,10 +1,10 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2012 Jean-Pierre Charras, jean-pierre.charras@ujf-grenoble.fr
+ * Copyright (C) 2012 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2012 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,7 +52,7 @@
 #include <widgets/wx_aui_utils.h>
 #include <wx/wupdlock.h>
 #include <wx/dcmemory.h>
-#include <wx/choice.h>
+#include <wx/combobox.h>
 
 #include "../scripting/python_scripting.h"
 
@@ -103,7 +103,7 @@ void PCB_EDIT_FRAME::PrepareLayerIndicator( bool aForceRebuild )
     COLOR4D    top_color, bottom_color, background_color;
     bool       change = aForceRebuild;
 
-    int requested_scale = Pgm().GetCommonSettings()->m_Appearance.icon_scale;
+    int requested_scale = KiIconScale( this );
 
     if( m_prevIconVal.previous_requested_scale != requested_scale )
     {
@@ -188,10 +188,10 @@ void PCB_EDIT_FRAME::PrepareLayerIndicator( bool aForceRebuild )
 
         LayerPairBitmap = std::make_unique<wxBitmap>( image );
 
-        if( m_mainToolBar )
+        if( m_auxiliaryToolBar )
         {
-            m_mainToolBar->SetToolBitmap( PCB_ACTIONS::selectLayerPair, *LayerPairBitmap );
-            m_mainToolBar->Refresh();
+            m_auxiliaryToolBar->SetToolBitmap( PCB_ACTIONS::selectLayerPair, *LayerPairBitmap );
+            m_auxiliaryToolBar->Refresh();
         }
     }
 }
@@ -218,16 +218,6 @@ void PCB_EDIT_FRAME::ReCreateHToolbar()
                                             KICAD_AUI_TB_STYLE | wxAUI_TB_HORZ_LAYOUT |
                                             wxAUI_TB_HORIZONTAL );
         m_mainToolBar->SetAuiManager( &m_auimgr );
-
-        // The layer indicator is special, so we register a callback directly that will
-        // regenerate the bitmap instead of using the conditions system.
-        auto layerIndicatorUpdate =
-            [this] ( wxUpdateUIEvent& )
-            {
-                PrepareLayerIndicator();
-            };
-
-        Bind( wxEVT_UPDATE_UI, layerIndicatorUpdate, PCB_ACTIONS::selectLayerPair.GetUIId() );
     }
 
     // Set up toolbar
@@ -275,6 +265,7 @@ void PCB_EDIT_FRAME::ReCreateHToolbar()
     m_mainToolBar->AddScaledSeparator( this );
     m_mainToolBar->Add( ACTIONS::showFootprintEditor );
     m_mainToolBar->Add( ACTIONS::showFootprintBrowser );
+    m_mainToolBar->Add( ACTIONS::show3DViewer );
 
     m_mainToolBar->AddScaledSeparator( this );
 
@@ -285,20 +276,6 @@ void PCB_EDIT_FRAME::ReCreateHToolbar()
 
     m_mainToolBar->Add( PCB_ACTIONS::runDRC );
 
-    m_mainToolBar->AddScaledSeparator( this );
-
-    if( m_SelLayerBox == nullptr )
-    {
-        m_SelLayerBox = new PCB_LAYER_BOX_SELECTOR( m_mainToolBar, ID_TOOLBARH_PCB_SELECT_LAYER );
-        m_SelLayerBox->SetBoardFrame( this );
-    }
-
-    ReCreateLayerBox( false );
-    m_mainToolBar->AddControl( m_SelLayerBox );
-
-    m_mainToolBar->Add( PCB_ACTIONS::selectLayerPair );
-    PrepareLayerIndicator( true );    // Force rebuild of the bitmap with the active layer colors
-
     m_mainToolBar->AddScaledSeparator( this );
     m_mainToolBar->Add( PCB_ACTIONS::showEeschema );
 
@@ -310,10 +287,6 @@ void PCB_EDIT_FRAME::ReCreateHToolbar()
         AddActionPluginTools();
     }
 
-    // Go through and ensure the comboboxes are the correct size, since the strings in the
-    // box could have changed widths.
-    m_mainToolBar->UpdateControlWidth( ID_TOOLBARH_PCB_SELECT_LAYER );
-
     // after adding the buttons to the toolbar, must call Realize() to reflect the changes
     m_mainToolBar->KiRealize();
 }
@@ -343,6 +316,7 @@ void PCB_EDIT_FRAME::ReCreateOptToolbar()
     }
 
     m_optionsToolBar->Add( ACTIONS::toggleGrid,               ACTION_TOOLBAR::TOGGLE );
+    m_optionsToolBar->Add( ACTIONS::toggleGridOverrides,      ACTION_TOOLBAR::TOGGLE );
 
     m_optionsToolBar->Add( PCB_ACTIONS::togglePolarCoords,    ACTION_TOOLBAR::TOGGLE );
     m_optionsToolBar->Add( ACTIONS::inchesUnits,              ACTION_TOOLBAR::TOGGLE );
@@ -387,6 +361,7 @@ void PCB_EDIT_FRAME::ReCreateOptToolbar()
     PCB_SELECTION_TOOL*          selTool = m_toolManager->GetTool<PCB_SELECTION_TOOL>();
     std::unique_ptr<ACTION_MENU> gridMenu = std::make_unique<ACTION_MENU>( false, selTool );
     gridMenu->Add( ACTIONS::gridProperties );
+    gridMenu->Add( ACTIONS::gridOrigin );
     m_optionsToolBar->AddToolContextMenu( ACTIONS::toggleGrid, std::move( gridMenu ) );
 
     m_optionsToolBar->KiRealize();
@@ -427,8 +402,8 @@ void PCB_EDIT_FRAME::ReCreateVToolbar()
     if( !originGroup )
     {
         originGroup = new ACTION_GROUP( "group.pcbOrigins",
-                                        { &PCB_ACTIONS::drillOrigin,
-                                          &PCB_ACTIONS::gridSetOrigin } );
+                                        { &PCB_ACTIONS::gridSetOrigin,
+                                          &PCB_ACTIONS::drillOrigin } );
     }
 
     if( !routingGroup )
@@ -441,9 +416,9 @@ void PCB_EDIT_FRAME::ReCreateVToolbar()
     if( !tuneGroup )
     {
         tuneGroup = new ACTION_GROUP( "group.pcbTune",
-                                      { &PCB_ACTIONS::routerTuneSingleTrace,
-                                        &PCB_ACTIONS::routerTuneDiffPair,
-                                        &PCB_ACTIONS::routerTuneDiffPairSkew } );
+                                      { &PCB_ACTIONS::tuneSingleTrack,
+                                        &PCB_ACTIONS::tuneDiffPair,
+                                        &PCB_ACTIONS::tuneSkew } );
     }
 
     m_drawToolBar->Add( ACTIONS::selectionTool,            ACTION_TOOLBAR::TOGGLE );
@@ -463,7 +438,7 @@ void PCB_EDIT_FRAME::ReCreateVToolbar()
     m_drawToolBar->Add( PCB_ACTIONS::drawRectangle,        ACTION_TOOLBAR::TOGGLE );
     m_drawToolBar->Add( PCB_ACTIONS::drawCircle,           ACTION_TOOLBAR::TOGGLE );
     m_drawToolBar->Add( PCB_ACTIONS::drawPolygon,          ACTION_TOOLBAR::TOGGLE );
-    m_drawToolBar->Add( PCB_ACTIONS::placeImage,           ACTION_TOOLBAR::TOGGLE );
+    m_drawToolBar->Add( PCB_ACTIONS::placeReferenceImage,  ACTION_TOOLBAR::TOGGLE );
     m_drawToolBar->Add( PCB_ACTIONS::placeText,            ACTION_TOOLBAR::TOGGLE );
     m_drawToolBar->Add( PCB_ACTIONS::drawTextBox,          ACTION_TOOLBAR::TOGGLE );
     m_drawToolBar->AddGroup( dimensionGroup,               ACTION_TOOLBAR::TOGGLE );
@@ -504,18 +479,6 @@ void PCB_EDIT_FRAME::ReCreateVToolbar()
     m_drawToolBar->AddToolContextMenu( PCB_ACTIONS::routeSingleTrack, makeRouteMenu() );
     m_drawToolBar->AddToolContextMenu( PCB_ACTIONS::routeDiffPair, makeRouteMenu() );
 
-    auto makeTuneMenu =
-        [&]()
-        {
-            std::unique_ptr<ACTION_MENU> tuneMenu = std::make_unique<ACTION_MENU>( false, selTool );
-            tuneMenu->Add( PCB_ACTIONS::lengthTunerSettingsDialog );
-            return tuneMenu;
-        };
-
-    m_drawToolBar->AddToolContextMenu( PCB_ACTIONS::routerTuneSingleTrace, makeTuneMenu() );
-    m_drawToolBar->AddToolContextMenu( PCB_ACTIONS::routerTuneDiffPair, makeTuneMenu() );
-    m_drawToolBar->AddToolContextMenu( PCB_ACTIONS::routerTuneDiffPairSkew, makeTuneMenu() );
-
     std::unique_ptr<ACTION_MENU> zoneMenu = std::make_unique<ACTION_MENU>( false, selTool );
     zoneMenu->Add( PCB_ACTIONS::zoneFillAll );
     zoneMenu->Add( PCB_ACTIONS::zoneUnfillAll );
@@ -541,6 +504,17 @@ void PCB_EDIT_FRAME::ReCreateAuxiliaryToolbar()
         m_auxiliaryToolBar = new ACTION_TOOLBAR( this, ID_AUX_TOOLBAR, wxDefaultPosition,
                                                  wxDefaultSize,
                                                  KICAD_AUI_TB_STYLE | wxAUI_TB_HORZ_LAYOUT );
+
+        // The layer indicator is special, so we register a callback directly that will
+        // regenerate the bitmap instead of using the conditions system.
+        auto layerIndicatorUpdate =
+            [this] ( wxUpdateUIEvent& )
+            {
+                PrepareLayerIndicator();
+            };
+
+        Bind( wxEVT_UPDATE_UI, layerIndicatorUpdate, PCB_ACTIONS::selectLayerPair.GetUIId() );
+
         m_auxiliaryToolBar->SetAuiManager( &m_auimgr );
     }
 
@@ -551,7 +525,7 @@ void PCB_EDIT_FRAME::ReCreateAuxiliaryToolbar()
         m_SelTrackWidthBox = new wxChoice( m_auxiliaryToolBar, ID_AUX_TOOLBAR_PCB_TRACK_WIDTH,
                                            wxDefaultPosition, wxDefaultSize, 0, nullptr );
 
-    UpdateTrackWidthSelectBox( m_SelTrackWidthBox );
+    UpdateTrackWidthSelectBox( m_SelTrackWidthBox, true, true );
     m_auxiliaryToolBar->AddControl( m_SelTrackWidthBox );
     m_SelTrackWidthBox->SetToolTip( _( "Select the default width for new tracks. Note that this "
                                        "width can be overridden by the board minimum width, or by "
@@ -572,9 +546,23 @@ void PCB_EDIT_FRAME::ReCreateAuxiliaryToolbar()
         m_SelViaSizeBox = new wxChoice( m_auxiliaryToolBar, ID_AUX_TOOLBAR_PCB_VIA_SIZE,
                                         wxDefaultPosition, wxDefaultSize, 0, nullptr );
 
-    UpdateViaSizeSelectBox( m_SelViaSizeBox );
+    UpdateViaSizeSelectBox( m_SelViaSizeBox, true, true );
     m_auxiliaryToolBar->AddControl( m_SelViaSizeBox );
 
+    m_auxiliaryToolBar->AddScaledSeparator( this );
+
+    if( m_SelLayerBox == nullptr )
+    {
+        m_SelLayerBox = new PCB_LAYER_BOX_SELECTOR( m_auxiliaryToolBar, ID_TOOLBARH_PCB_SELECT_LAYER );
+        m_SelLayerBox->SetBoardFrame( this );
+    }
+
+    ReCreateLayerBox( false );
+    m_auxiliaryToolBar->AddControl( m_SelLayerBox );
+
+    m_auxiliaryToolBar->Add( PCB_ACTIONS::selectLayerPair );
+    PrepareLayerIndicator( true );    // Force rebuild of the bitmap with the active layer colors
+
     // Add the box to display and select the current grid size:
     m_auxiliaryToolBar->AddScaledSeparator( this );
 
@@ -602,6 +590,7 @@ void PCB_EDIT_FRAME::ReCreateAuxiliaryToolbar()
     m_auxiliaryToolBar->UpdateControlWidth( ID_AUX_TOOLBAR_PCB_VIA_SIZE );
     m_auxiliaryToolBar->UpdateControlWidth( ID_ON_ZOOM_SELECT );
     m_auxiliaryToolBar->UpdateControlWidth( ID_ON_GRID_SELECT );
+    m_auxiliaryToolBar->UpdateControlWidth( ID_TOOLBARH_PCB_SELECT_LAYER );
 
     // after adding the buttons to the toolbar, must call Realize()
     m_auxiliaryToolBar->KiRealize();
@@ -613,7 +602,6 @@ void PCB_EDIT_FRAME::UpdateToolbarControlSizes()
     if( m_mainToolBar )
     {
         // Update the item widths
-        m_mainToolBar->UpdateControlWidth( ID_TOOLBARH_PCB_SELECT_LAYER );
     }
 
     if( m_auxiliaryToolBar )
@@ -623,6 +611,7 @@ void PCB_EDIT_FRAME::UpdateToolbarControlSizes()
         m_auxiliaryToolBar->UpdateControlWidth( ID_AUX_TOOLBAR_PCB_VIA_SIZE );
         m_auxiliaryToolBar->UpdateControlWidth( ID_ON_ZOOM_SELECT );
         m_auxiliaryToolBar->UpdateControlWidth( ID_ON_GRID_SELECT );
+        m_auxiliaryToolBar->UpdateControlWidth( ID_TOOLBARH_PCB_SELECT_LAYER );
     }
 }
 
@@ -652,7 +641,8 @@ static wxString ComboBoxUnits( EDA_UNITS aUnits, double aValue, bool aIncludeLab
 }
 
 
-void PCB_EDIT_FRAME::UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox, bool aEdit )
+void PCB_EDIT_FRAME::UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox, bool aShowNetclass,
+                                                bool aShowEdit )
 {
     if( aTrackWidthSelectBox == nullptr )
         return;
@@ -666,7 +656,8 @@ void PCB_EDIT_FRAME::UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox,
 
     aTrackWidthSelectBox->Clear();
 
-    aTrackWidthSelectBox->Append( _( "Track: use netclass width" ) );
+    if( aShowNetclass )
+        aTrackWidthSelectBox->Append( _( "Track: use netclass width" ) );
 
     for( unsigned ii = 1; ii < GetDesignSettings().m_TrackWidthList.size(); ii++ )
     {
@@ -678,7 +669,7 @@ void PCB_EDIT_FRAME::UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox,
         aTrackWidthSelectBox->Append( msg );
     }
 
-    if( aEdit )
+    if( aShowEdit )
     {
         aTrackWidthSelectBox->Append( wxT( "---" ) );
         aTrackWidthSelectBox->Append( _( "Edit Pre-defined Sizes..." ) );
@@ -691,7 +682,8 @@ void PCB_EDIT_FRAME::UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox,
 }
 
 
-void PCB_EDIT_FRAME::UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool aEdit )
+void PCB_EDIT_FRAME::UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool aShowNetclass,
+                                             bool aShowEdit )
 {
     if( aViaSizeSelectBox == nullptr )
         return;
@@ -718,7 +710,8 @@ void PCB_EDIT_FRAME::UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool a
             secondaryUnit = EDA_UNITS::MILS;
     }
 
-    aViaSizeSelectBox->Append( _( "Via: use netclass sizes" ) );
+    if( aShowNetclass )
+        aViaSizeSelectBox->Append( _( "Via: use netclass sizes" ) );
 
     for( unsigned ii = 1; ii < GetDesignSettings().m_ViasDimensionsList.size(); ii++ )
     {
@@ -746,7 +739,7 @@ void PCB_EDIT_FRAME::UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool a
         aViaSizeSelectBox->Append( msg );
     }
 
-    if( aEdit )
+    if( aShowEdit )
     {
         aViaSizeSelectBox->Append( wxT( "---" ) );
         aViaSizeSelectBox->Append( _( "Edit Pre-defined Sizes..." ) );
@@ -761,7 +754,7 @@ void PCB_EDIT_FRAME::UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool a
 
 void PCB_EDIT_FRAME::ReCreateLayerBox( bool aForceResizeToolbar )
 {
-    if( m_SelLayerBox == nullptr || m_mainToolBar == nullptr )
+    if( m_SelLayerBox == nullptr || m_auxiliaryToolBar == nullptr )
         return;
 
     m_SelLayerBox->SetToolTip( _( "+/- to switch" ) );
@@ -796,31 +789,6 @@ void PCB_EDIT_FRAME::ToggleLayersManager()
 }
 
 
-void PCB_EDIT_FRAME::ToggleProperties()
-{
-    if( !m_propertiesPanel )
-        return;
-
-    PCBNEW_SETTINGS* settings = GetPcbNewSettings();
-
-    m_show_properties = !m_show_properties;
-
-    wxAuiPaneInfo& propertiesPaneInfo = m_auimgr.GetPane( "PropertiesManager" );
-    propertiesPaneInfo.Show( m_show_properties );
-
-    if( m_show_properties )
-    {
-        SetAuiPaneSize( m_auimgr, propertiesPaneInfo,
-                        settings->m_AuiPanels.properties_panel_width, -1 );
-    }
-    else
-    {
-        settings->m_AuiPanels.properties_panel_width = m_propertiesPanel->GetSize().x;
-        m_auimgr.Update();
-    }
-}
-
-
 void PCB_EDIT_FRAME::ToggleSearch()
 {
     PCBNEW_SETTINGS* settings = GetPcbNewSettings();
@@ -832,16 +800,32 @@ void PCB_EDIT_FRAME::ToggleSearch()
 
     wxAuiPaneInfo& searchPaneInfo = m_auimgr.GetPane( SearchPaneName() );
     searchPaneInfo.Show( m_show_search );
-
+    
     if( m_show_search )
     {
-        SetAuiPaneSize( m_auimgr, searchPaneInfo, -1, settings->m_AuiPanels.search_panel_height );
+        searchPaneInfo.Direction( settings->m_AuiPanels.search_panel_dock_direction );
+
+        if( settings->m_AuiPanels.search_panel_dock_direction == wxAUI_DOCK_TOP
+            || settings->m_AuiPanels.search_panel_dock_direction == wxAUI_DOCK_BOTTOM )
+        {
+            SetAuiPaneSize( m_auimgr, searchPaneInfo,
+                            -1, settings->m_AuiPanels.search_panel_height );
+        }
+        else if( settings->m_AuiPanels.search_panel_dock_direction == wxAUI_DOCK_LEFT
+                 || settings->m_AuiPanels.search_panel_dock_direction == wxAUI_DOCK_RIGHT )
+        {
+            SetAuiPaneSize( m_auimgr, searchPaneInfo,
+                            settings->m_AuiPanels.search_panel_width, -1 );
+        }
         m_searchPane->FocusSearch();
     }
     else
-    {
+    {  
         settings->m_AuiPanels.search_panel_height = m_searchPane->GetSize().y;
+        settings->m_AuiPanels.search_panel_width = m_searchPane->GetSize().x;
+        settings->m_AuiPanels.search_panel_dock_direction = searchPaneInfo.dock_direction;
         m_auimgr.Update();
+        GetCanvas()->SetFocus();
     }
 }
 
