@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014-2016 CERN
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -35,9 +35,6 @@
 #include <board_commit.h>
 #include <bitmaps.h>
 
-#include <confirm.h>
-#include <menus_helpers.h>
-
 
 ALIGN_DISTRIBUTE_TOOL::ALIGN_DISTRIBUTE_TOOL() :
     TOOL_INTERACTIVE( "pcbnew.Placement" ),
@@ -79,30 +76,33 @@ bool ALIGN_DISTRIBUTE_TOOL::Init()
     m_placementMenu->Add( PCB_ACTIONS::distributeVertically );
 
     CONDITIONAL_MENU& selToolMenu = m_selectionTool->GetToolMenu().GetMenu();
-    selToolMenu.AddMenu( m_placementMenu, SELECTION_CONDITIONS::MoreThan( 1 ) );
+    selToolMenu.AddMenu( m_placementMenu, SELECTION_CONDITIONS::MoreThan( 1 ), 100 );
 
     return true;
 }
 
 
 template <class T>
-ALIGNMENT_RECTS GetBoundingBoxes( const T& aItems )
+std::vector<std::pair<BOARD_ITEM*, BOX2I>> GetBoundingBoxes( const T& aItems )
 {
-    ALIGNMENT_RECTS rects;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> rects;
 
     for( EDA_ITEM* item : aItems )
     {
+        if( !item->IsBOARD_ITEM() )
+            continue;
+
         BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( item );
 
-        if( item->Type() == PCB_FOOTPRINT_T )
+        if( boardItem->Type() == PCB_FOOTPRINT_T )
         {
-            FOOTPRINT* footprint = static_cast<FOOTPRINT*>( item );
-            rects.emplace_back( std::make_pair( boardItem,
+            FOOTPRINT* footprint = static_cast<FOOTPRINT*>( boardItem );
+            rects.emplace_back( std::make_pair( footprint,
                                                 footprint->GetBoundingBox( false, false ) ) );
         }
         else
         {
-            rects.emplace_back( std::make_pair( boardItem, item->GetBoundingBox() ) );
+            rects.emplace_back( std::make_pair( boardItem, boardItem->GetBoundingBox() ) );
         }
     }
 
@@ -111,17 +111,18 @@ ALIGNMENT_RECTS GetBoundingBoxes( const T& aItems )
 
 
 template< typename T >
-int ALIGN_DISTRIBUTE_TOOL::selectTarget( ALIGNMENT_RECTS& aItems, ALIGNMENT_RECTS& aLocked,
+int ALIGN_DISTRIBUTE_TOOL::selectTarget( std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aItems,
+                                         std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aLocked,
                                          T aGetValue )
 {
-    wxPoint curPos = (wxPoint) getViewControls()->GetCursorPosition();
+    VECTOR2I curPos = getViewControls()->GetCursorPosition();
 
     // Prefer locked items to unlocked items.
     // Secondly, prefer items under the cursor to other items.
 
     if( aLocked.size() >= 1 )
     {
-        for( const ALIGNMENT_RECT& item : aLocked )
+        for( const std::pair<BOARD_ITEM*, BOX2I>& item : aLocked )
         {
             if( item.second.Contains( curPos ) )
                 return aGetValue( item );
@@ -130,7 +131,7 @@ int ALIGN_DISTRIBUTE_TOOL::selectTarget( ALIGNMENT_RECTS& aItems, ALIGNMENT_RECT
         return aGetValue( aLocked.front() );
     }
 
-    for( const ALIGNMENT_RECT& item : aItems )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& item : aItems )
     {
         if( item.second.Contains( curPos ) )
             return aGetValue( item );
@@ -141,8 +142,9 @@ int ALIGN_DISTRIBUTE_TOOL::selectTarget( ALIGNMENT_RECTS& aItems, ALIGNMENT_RECT
 
 
 template< typename T >
-size_t ALIGN_DISTRIBUTE_TOOL::GetSelections( ALIGNMENT_RECTS& aItemsToAlign,
-                                             ALIGNMENT_RECTS& aLockedItems, T aCompare )
+size_t ALIGN_DISTRIBUTE_TOOL::GetSelections( std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aItemsToAlign,
+                                             std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aLockedItems,
+                                             T aCompare )
 {
     PCB_SELECTION& selection = m_selectionTool->RequestSelection(
             []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
@@ -162,6 +164,9 @@ size_t ALIGN_DISTRIBUTE_TOOL::GetSelections( ALIGNMENT_RECTS& aItemsToAlign,
 
     for( EDA_ITEM* item : selection )
     {
+        if( !item->IsBOARD_ITEM() )
+            continue;
+
         BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( item );
 
         // We do not lock items in the footprint editor
@@ -170,16 +175,14 @@ size_t ALIGN_DISTRIBUTE_TOOL::GetSelections( ALIGNMENT_RECTS& aItemsToAlign,
             // Locking a pad but not the footprint means that we align the footprint using
             // the pad position.  So we test for footprint locking here
             if( boardItem->Type() == PCB_PAD_T && !boardItem->GetParent()->IsLocked() )
-            {
                 itemsToAlign.push_back( boardItem );
-            }
             else
-            {
                 lockedItems.push_back( boardItem );
-            }
         }
         else
+        {
             itemsToAlign.push_back( boardItem );
+        }
     }
 
     aItemsToAlign = GetBoundingBoxes( itemsToAlign );
@@ -193,13 +196,13 @@ size_t ALIGN_DISTRIBUTE_TOOL::GetSelections( ALIGNMENT_RECTS& aItemsToAlign,
 
 int ALIGN_DISTRIBUTE_TOOL::AlignTop( const TOOL_EVENT& aEvent )
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetTop() < right.second.GetTop() );
+                return ( lhs.second.GetTop() < rhs.second.GetTop() );
             } ) )
     {
         return 0;
@@ -208,13 +211,13 @@ int ALIGN_DISTRIBUTE_TOOL::AlignTop( const TOOL_EVENT& aEvent )
     BOARD_COMMIT commit( m_frame );
 
     int targetTop = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
                 return aVal.second.GetTop();
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
         BOARD_ITEM* item = i.first;
         int difference = targetTop - i.second.GetTop();
@@ -227,24 +230,23 @@ int ALIGN_DISTRIBUTE_TOOL::AlignTop( const TOOL_EVENT& aEvent )
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( 0, difference ) );
+        item->Move( VECTOR2I( 0, difference ) );
     }
 
-    commit.Push( _( "Align to top" ) );
-
+    commit.Push( _( "Align to Top" ) );
     return 0;
 }
 
 
 int ALIGN_DISTRIBUTE_TOOL::AlignBottom( const TOOL_EVENT& aEvent )
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs)
             {
-                return ( left.second.GetBottom() > right.second.GetBottom() );
+                return ( lhs.second.GetBottom() > rhs.second.GetBottom() );
             } ) )
     {
         return 0;
@@ -253,13 +255,13 @@ int ALIGN_DISTRIBUTE_TOOL::AlignBottom( const TOOL_EVENT& aEvent )
     BOARD_COMMIT commit( m_frame );
 
     int targetBottom = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
                 return aVal.second.GetBottom();
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
         int difference = targetBottom - i.second.GetBottom();
         BOARD_ITEM* item = i.first;
@@ -272,11 +274,10 @@ int ALIGN_DISTRIBUTE_TOOL::AlignBottom( const TOOL_EVENT& aEvent )
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( 0, difference ) );
+        item->Move( VECTOR2I( 0, difference ) );
     }
 
-    commit.Push( _( "Align to bottom" ) );
-
+    commit.Push( _( "Align to Bottom" ) );
     return 0;
 }
 
@@ -286,25 +287,21 @@ int ALIGN_DISTRIBUTE_TOOL::AlignLeft( const TOOL_EVENT& aEvent )
     // Because this tool uses bounding boxes and they aren't mirrored even when
     // the view is mirrored, we need to call the other one if mirrored.
     if( getView()->IsMirroredX() )
-    {
         return doAlignRight();
-    }
     else
-    {
         return doAlignLeft();
-    }
 }
 
 
 int ALIGN_DISTRIBUTE_TOOL::doAlignLeft()
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetLeft() < right.second.GetLeft() );
+                return ( lhs.second.GetLeft() < rhs.second.GetLeft() );
             } ) )
     {
         return 0;
@@ -313,13 +310,13 @@ int ALIGN_DISTRIBUTE_TOOL::doAlignLeft()
     BOARD_COMMIT commit( m_frame );
 
     int targetLeft = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
                 return aVal.second.GetLeft();
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
         int difference = targetLeft - i.second.GetLeft();
         BOARD_ITEM* item = i.first;
@@ -332,11 +329,10 @@ int ALIGN_DISTRIBUTE_TOOL::doAlignLeft()
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( difference, 0 ) );
+        item->Move( VECTOR2I( difference, 0 ) );
     }
 
-    commit.Push( _( "Align to left" ) );
-
+    commit.Push( _( "Align to Left" ) );
     return 0;
 }
 
@@ -346,25 +342,21 @@ int ALIGN_DISTRIBUTE_TOOL::AlignRight( const TOOL_EVENT& aEvent )
     // Because this tool uses bounding boxes and they aren't mirrored even when
     // the view is mirrored, we need to call the other one if mirrored.
     if( getView()->IsMirroredX() )
-    {
         return doAlignLeft();
-    }
     else
-    {
         return doAlignRight();
-    }
 }
 
 
 int ALIGN_DISTRIBUTE_TOOL::doAlignRight()
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetRight() > right.second.GetRight() );
+                return ( lhs.second.GetRight() > rhs.second.GetRight() );
             } ) )
     {
         return 0;
@@ -373,13 +365,13 @@ int ALIGN_DISTRIBUTE_TOOL::doAlignRight()
     BOARD_COMMIT commit( m_frame );
 
     int targetRight = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
                 return aVal.second.GetRight();
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
         int difference = targetRight - i.second.GetRight();
         BOARD_ITEM* item = i.first;
@@ -392,24 +384,23 @@ int ALIGN_DISTRIBUTE_TOOL::doAlignRight()
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( difference, 0 ) );
+        item->Move( VECTOR2I( difference, 0 ) );
     }
 
-    commit.Push( _( "Align to right" ) );
-
+    commit.Push( _( "Align to Right" ) );
     return 0;
 }
 
 
 int ALIGN_DISTRIBUTE_TOOL::AlignCenterX( const TOOL_EVENT& aEvent )
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetCenter().x < right.second.GetCenter().x );
+                return ( lhs.second.Centre().x < rhs.second.Centre().x );
             } ) )
     {
         return 0;
@@ -418,15 +409,15 @@ int ALIGN_DISTRIBUTE_TOOL::AlignCenterX( const TOOL_EVENT& aEvent )
     BOARD_COMMIT commit( m_frame );
 
     int targetX = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
-                return aVal.second.GetCenter().x;
+                return aVal.second.Centre().x;
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
-        int difference = targetX - i.second.GetCenter().x;
+        int difference = targetX - i.second.Centre().x;
         BOARD_ITEM* item = i.first;
 
         if( item->GetParent() && item->GetParent()->IsSelected() )
@@ -437,24 +428,23 @@ int ALIGN_DISTRIBUTE_TOOL::AlignCenterX( const TOOL_EVENT& aEvent )
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( difference, 0 ) );
+        item->Move( VECTOR2I( difference, 0 ) );
     }
 
-    commit.Push( _( "Align to middle" ) );
-
+    commit.Push( _( "Align to Middle" ) );
     return 0;
 }
 
 
 int ALIGN_DISTRIBUTE_TOOL::AlignCenterY( const TOOL_EVENT& aEvent )
 {
-    ALIGNMENT_RECTS itemsToAlign;
-    ALIGNMENT_RECTS locked_items;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToAlign;
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> locked_items;
 
     if( !GetSelections( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetCenter().y < right.second.GetCenter().y );
+                return ( lhs.second.Centre().y < rhs.second.Centre().y );
             } ) )
     {
         return 0;
@@ -463,15 +453,15 @@ int ALIGN_DISTRIBUTE_TOOL::AlignCenterY( const TOOL_EVENT& aEvent )
     BOARD_COMMIT commit( m_frame );
 
     int targetY = selectTarget( itemsToAlign, locked_items,
-            []( const ALIGNMENT_RECT& aVal )
+            []( const std::pair<BOARD_ITEM*, BOX2I>& aVal )
             {
-                return aVal.second.GetCenter().y;
+                return aVal.second.Centre().y;
             } );
 
     // Move the selected items
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToAlign )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToAlign )
     {
-        int difference = targetY - i.second.GetCenter().y;
+        int difference = targetY - i.second.Centre().y;
         BOARD_ITEM* item = i.first;
 
         if( item->GetParent() && item->GetParent()->IsSelected() )
@@ -482,11 +472,10 @@ int ALIGN_DISTRIBUTE_TOOL::AlignCenterY( const TOOL_EVENT& aEvent )
             item = item->GetParent();
 
         commit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( 0, difference ) );
+        item->Move( VECTOR2I( 0, difference ) );
     }
 
-    commit.Push( _( "Align to center" ) );
-
+    commit.Push( _( "Align to Center" ) );
     return 0;
 }
 
@@ -510,14 +499,14 @@ int ALIGN_DISTRIBUTE_TOOL::DistributeHorizontally( const TOOL_EVENT& aEvent )
     if( selection.Size() <= 1 )
         return 0;
 
-    BOARD_COMMIT    commit( m_frame );
-    ALIGNMENT_RECTS itemsToDistribute = GetBoundingBoxes( selection );
+    BOARD_COMMIT                               commit( m_frame );
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToDistribute = GetBoundingBoxes( selection );
 
     // find the last item by reverse sorting
     std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-            [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetRight() > right.second.GetRight() );
+                return ( lhs.second.GetRight() > rhs.second.GetRight() );
             } );
 
     BOARD_ITEM* lastItem = itemsToDistribute.begin()->first;
@@ -525,17 +514,17 @@ int ALIGN_DISTRIBUTE_TOOL::DistributeHorizontally( const TOOL_EVENT& aEvent )
 
     // sort to get starting order
     std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-            [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetX() < right.second.GetX() );
+                return ( lhs.second.GetX() < rhs.second.GetX() );
             } );
 
     const int minX = itemsToDistribute.begin()->second.GetX();
     int       totalGap = maxRight - minX;
     int       totalWidth = 0;
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
-        totalWidth += i.second.GetWidth();
+    for( const auto& [ item, rect ] : itemsToDistribute )
+        totalWidth += rect.GetWidth();
 
     if( totalGap < totalWidth )
     {
@@ -548,21 +537,20 @@ int ALIGN_DISTRIBUTE_TOOL::DistributeHorizontally( const TOOL_EVENT& aEvent )
         doDistributeGapsHorizontally( itemsToDistribute, commit, lastItem, totalGap );
     }
 
-    commit.Push( _( "Distribute horizontally" ) );
-
+    commit.Push( _( "Distribute Horizontally" ) );
     return 0;
 }
 
 
-void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsHorizontally( ALIGNMENT_RECTS& itemsToDistribute,
+void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsHorizontally( std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aItems,
                                                           BOARD_COMMIT& aCommit,
                                                           const BOARD_ITEM* lastItem,
                                                           int totalGap ) const
 {
-    const int itemGap = totalGap / ( itemsToDistribute.size() - 1 );
-    int       targetX = itemsToDistribute.begin()->second.GetX();
+    const int itemGap = totalGap / ( aItems.size() - 1 );
+    int       targetX = aItems.begin()->second.GetX();
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : aItems )
     {
         BOARD_ITEM* item = i.first;
 
@@ -579,27 +567,26 @@ void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsHorizontally( ALIGNMENT_RECTS& items
 
         int difference = targetX - i.second.GetX();
         aCommit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( difference, 0 ) );
+        item->Move( VECTOR2I( difference, 0 ) );
         targetX += ( i.second.GetWidth() + itemGap );
     }
 }
 
 
-void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersHorizontally( ALIGNMENT_RECTS &itemsToDistribute,
+void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersHorizontally( std::vector<std::pair<BOARD_ITEM*, BOX2I>> &aItems,
                                                              BOARD_COMMIT& aCommit ) const
 {
-    std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-            [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+    std::sort( aItems.begin(), aItems.end(),
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
             {
-                return ( left.second.GetCenter().x < right.second.GetCenter().x );
+                return ( lhs.second.Centre().x < rhs.second.Centre().x );
             } );
 
-    const int totalGap = ( itemsToDistribute.end()-1 )->second.GetCenter().x
-                          - itemsToDistribute.begin()->second.GetCenter().x;
-    const int itemGap = totalGap / ( itemsToDistribute.size() - 1 );
-    int       targetX = itemsToDistribute.begin()->second.GetCenter().x;
+    const int totalGap = ( aItems.end()-1 )->second.Centre().x - aItems.begin()->second.Centre().x;
+    const int itemGap = totalGap / ( aItems.size() - 1 );
+    int       targetX = aItems.begin()->second.Centre().x;
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : aItems )
     {
         BOARD_ITEM* item = i.first;
 
@@ -610,9 +597,9 @@ void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersHorizontally( ALIGNMENT_RECTS &it
         if( item->Type() == PCB_PAD_T && m_frame->IsType( FRAME_PCB_EDITOR ) )
             item = item->GetParent();
 
-        int difference = targetX - i.second.GetCenter().x;
+        int difference = targetX - i.second.Centre().x;
         aCommit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( difference, 0 ) );
+        item->Move( VECTOR2I( difference, 0 ) );
         targetX += ( itemGap );
     }
 }
@@ -637,31 +624,31 @@ int ALIGN_DISTRIBUTE_TOOL::DistributeVertically( const TOOL_EVENT& aEvent )
     if( selection.Size() <= 1 )
         return 0;
 
-    BOARD_COMMIT    commit( m_frame );
-    ALIGNMENT_RECTS itemsToDistribute = GetBoundingBoxes( selection );
+    BOARD_COMMIT                               commit( m_frame );
+    std::vector<std::pair<BOARD_ITEM*, BOX2I>> itemsToDistribute = GetBoundingBoxes( selection );
 
     // find the last item by reverse sorting
     std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-        [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
-        {
-            return ( left.second.GetBottom() > right.second.GetBottom() );
-        } );
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
+            {
+                return ( lhs.second.GetBottom() > rhs.second.GetBottom() );
+            } );
 
     BOARD_ITEM* lastItem = itemsToDistribute.begin()->first;
     const int   maxBottom = itemsToDistribute.begin()->second.GetBottom();
 
     // sort to get starting order
     std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-        [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
-        {
-            return ( left.second.GetCenter().y < right.second.GetCenter().y );
-        } );
+            []( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs )
+            {
+                return ( lhs.second.Centre().y < rhs.second.Centre().y );
+            } );
 
     int minY = itemsToDistribute.begin()->second.GetY();
     int totalGap = maxBottom - minY;
     int totalHeight = 0;
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : itemsToDistribute )
         totalHeight += i.second.GetHeight();
 
     if( totalGap < totalHeight )
@@ -675,21 +662,20 @@ int ALIGN_DISTRIBUTE_TOOL::DistributeVertically( const TOOL_EVENT& aEvent )
         doDistributeGapsVertically( itemsToDistribute, commit, lastItem, totalGap );
     }
 
-    commit.Push( _( "Distribute vertically" ) );
-
+    commit.Push( _( "Distribute Vertically" ) );
     return 0;
 }
 
 
-void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsVertically( ALIGNMENT_RECTS& itemsToDistribute,
+void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsVertically( std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aItems,
                                                         BOARD_COMMIT& aCommit,
                                                         const BOARD_ITEM* lastItem,
                                                         int totalGap ) const
 {
-    const int itemGap = totalGap / ( itemsToDistribute.size() - 1 );
-    int       targetY = itemsToDistribute.begin()->second.GetY();
+    const int itemGap = totalGap / ( aItems.size() - 1 );
+    int       targetY = aItems.begin()->second.GetY();
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
+    for( std::pair<BOARD_ITEM*, BOX2I>& i : aItems )
     {
         BOARD_ITEM* item = i.first;
 
@@ -706,27 +692,27 @@ void ALIGN_DISTRIBUTE_TOOL::doDistributeGapsVertically( ALIGNMENT_RECTS& itemsTo
 
         int difference = targetY - i.second.GetY();
         aCommit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( 0, difference ) );
+        item->Move( VECTOR2I( 0, difference ) );
         targetY += ( i.second.GetHeight() + itemGap );
     }
 }
 
 
-void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersVertically( ALIGNMENT_RECTS& itemsToDistribute,
+void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersVertically( std::vector<std::pair<BOARD_ITEM*, BOX2I>>& aItems,
                                                            BOARD_COMMIT& aCommit ) const
 {
-    std::sort( itemsToDistribute.begin(), itemsToDistribute.end(),
-        [] ( const ALIGNMENT_RECT left, const ALIGNMENT_RECT right)
+    std::sort( aItems.begin(), aItems.end(),
+        [] ( const std::pair<BOARD_ITEM*, BOX2I>& lhs, const std::pair<BOARD_ITEM*, BOX2I>& rhs)
         {
-            return ( left.second.GetCenter().y < right.second.GetCenter().y );
+            return ( lhs.second.Centre().y < rhs.second.Centre().y );
         } );
 
-    const int totalGap = ( itemsToDistribute.end()-1 )->second.GetCenter().y
-                          - itemsToDistribute.begin()->second.GetCenter().y;
-    const int itemGap  = totalGap / ( itemsToDistribute.size() - 1 );
-    int       targetY  = itemsToDistribute.begin()->second.GetCenter().y;
+    const int totalGap = ( aItems.end() - 1 )->second.Centre().y
+                         - aItems.begin()->second.Centre().y;
+    const int itemGap  = totalGap / ( aItems.size() - 1 );
+    int       targetY  = aItems.begin()->second.Centre().y;
 
-    for( std::pair<BOARD_ITEM*, EDA_RECT>& i : itemsToDistribute )
+    for( const std::pair<BOARD_ITEM*, BOX2I>& i : aItems )
     {
         BOARD_ITEM* item = i.first;
 
@@ -737,9 +723,9 @@ void ALIGN_DISTRIBUTE_TOOL::doDistributeCentersVertically( ALIGNMENT_RECTS& item
         if( item->Type() == PCB_PAD_T && m_frame->IsType( FRAME_PCB_EDITOR ) )
             item = item->GetParent();
 
-        int difference = targetY - i.second.GetCenter().y;
+        int difference = targetY - i.second.Centre().y;
         aCommit.Stage( item, CHT_MODIFY );
-        item->Move( wxPoint( 0, difference ) );
+        item->Move( VECTOR2I( 0, difference ) );
         targetY += ( itemGap );
     }
 }
