@@ -4,7 +4,7 @@
  * Copyright (C) 2012 Jean-Pierre Charras, jean-pierre.charras@ujf-grenoble.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2016 CERN
- * Copyright (C) 2012-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2012-2023 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,9 +29,9 @@
 using namespace std::placeholders;
 #include <macros.h>
 #include <pcb_edit_frame.h>
-#include <board.h>
 #include <pcb_track.h>
 #include <pcb_group.h>
+#include <pcb_generator.h>
 #include <pcb_target.h>
 #include <footprint.h>
 #include <pad.h>
@@ -42,6 +42,7 @@ using namespace std::placeholders;
 #include <tools/pcb_selection_tool.h>
 #include <tools/pcb_control.h>
 #include <tools/board_editor_control.h>
+#include <board_commit.h>
 #include <drawing_sheet/ds_proxy_undo_item.h>
 #include <wx/msgdlg.h>
 
@@ -95,128 +96,14 @@ using namespace std::placeholders;
  */
 
 
-/**
- * Test if aItem exists somewhere in undo/redo lists of items.  Used by PutDataInPreviousState
- * to be sure an item was not deleted since an undo or redo.
- *
- * This could be possible:
- *   - if a call to SaveCopyInUndoList was forgotten in Pcbnew
- *   - in zones outlines, when a change in one zone merges this zone with an other
- * Before using this function to test existence of items, it must be called with aItem = NULL to
- * prepare the list.
- *
- * @param aPcb is the board to test.
- * @param aItem is the item to find or NULL to build the list of existing items.
- */
-static bool TestForExistingItem( BOARD* aPcb, BOARD_ITEM* aItem )
-{
-    for( PCB_TRACK* item : aPcb->Tracks() )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( item ) )
-            return true;
-    }
-
-    for( FOOTPRINT* item : aPcb->Footprints() )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( item ) )
-            return true;
-    }
-
-    for( BOARD_ITEM* item : aPcb->Drawings() )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( item ) )
-            return true;
-    }
-
-    for( ZONE* item : aPcb->Zones() )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( item ) )
-            return true;
-    }
-
-    NETINFO_LIST& netInfo = aPcb->GetNetInfo();
-
-    for( NETINFO_LIST::iterator i = netInfo.begin(); i != netInfo.end(); ++i )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( *i ) )
-            return true;
-    }
-
-    for( PCB_GROUP* item : aPcb->Groups() )
-    {
-        if( aItem == static_cast<BOARD_ITEM*>( item ) )
-            return true;
-    }
-
-    return false;
-}
-
-
 void PCB_BASE_EDIT_FRAME::saveCopyInUndoList( PICKED_ITEMS_LIST* commandToUndo,
                                               const PICKED_ITEMS_LIST& aItemsList,
                                               UNDO_REDO aCommandType )
 {
     int preExisting = commandToUndo->GetCount();
 
-    // First, filter unnecessary stuff from the list (i.e. for multiple pads / labels modified),
-    // take the first occurrence of the footprint (we save copies of footprints when one of its
-    // subitems is changed).
     for( unsigned ii = 0; ii < aItemsList.GetCount(); ii++ )
-    {
-        ITEM_PICKER curr_picker = aItemsList.GetItemWrapper(ii);
-        BOARD_ITEM* item        = dynamic_cast<BOARD_ITEM*>( aItemsList.GetPickedItem( ii ) );
-
-        // For items belonging to footprints, we need to save state of the parent footprint
-        if( item && item->GetParent() && item->GetParent()->Type() == PCB_FOOTPRINT_T )
-        {
-            item = item->GetParent();
-
-            // Check if the parent footprint has already been saved in another entry
-            bool found = false;
-
-            for( unsigned j = 0; j < commandToUndo->GetCount(); j++ )
-            {
-                if( commandToUndo->GetPickedItem( j ) == item
-                        && commandToUndo->GetPickedItemStatus( j ) == UNDO_REDO::CHANGED )
-                {
-                    found = true;
-                    break;
-                }
-            }
-
-            if( !found )
-            {
-                // Create a clean copy of the parent footprint
-                FOOTPRINT* orig = static_cast<FOOTPRINT*>( item );
-                FOOTPRINT* clone = new FOOTPRINT( *orig );
-                clone->SetParent( GetBoard() );
-                clone->SetParentGroup( nullptr );
-
-                // Clear current flags (which can be temporary set by a current edit command)
-                for( BOARD_ITEM* child : clone->GraphicalItems() )
-                    child->ClearEditFlags();
-
-                for( PAD* pad : clone->Pads() )
-                    pad->ClearEditFlags();
-
-                clone->Reference().ClearEditFlags();
-                clone->Value().ClearEditFlags();
-
-                ITEM_PICKER picker( nullptr, item, UNDO_REDO::CHANGED );
-                picker.SetLink( clone );
-                commandToUndo->PushItem( picker );
-            }
-            else
-            {
-                continue;
-            }
-        }
-        else
-        {
-            // Normal case: all other BOARD_ITEMs, are simply copied to the new list
-            commandToUndo->PushItem( curr_picker );
-        }
-    }
+        commandToUndo->PushItem( aItemsList.GetItemWrapper(ii) );
 
     for( unsigned ii = preExisting; ii < commandToUndo->GetCount(); ii++ )
     {
@@ -236,15 +123,9 @@ void PCB_BASE_EDIT_FRAME::saveCopyInUndoList( PICKED_ITEMS_LIST* commandToUndo,
         case UNDO_REDO::CHANGED:
         case UNDO_REDO::DRILLORIGIN:
         case UNDO_REDO::GRIDORIGIN:
-
             // If we don't yet have a copy in the link, set one up
             if( !commandToUndo->GetPickedItemLink( ii ) )
-            {
-                BOARD_ITEM* clone = static_cast<BOARD_ITEM*>( item->Clone() );
-                clone->SetParentGroup( nullptr );
-
-                commandToUndo->SetPickedItemLink( clone, ii );
-            }
+                commandToUndo->SetPickedItemLink( BOARD_COMMIT::MakeImage( item ), ii );
 
             break;
 
@@ -256,8 +137,7 @@ void PCB_BASE_EDIT_FRAME::saveCopyInUndoList( PICKED_ITEMS_LIST* commandToUndo,
             break;
 
         default:
-            wxFAIL_MSG( wxString::Format( wxT( "SaveCopyInUndoList() error (unknown code %X)" ),
-                                          command ) );
+            wxFAIL_MSG( wxString::Format( wxT( "Unrecognized undo command: %X" ), command ) );
             break;
         }
     }
@@ -293,6 +173,7 @@ void PCB_BASE_EDIT_FRAME::SaveCopyInUndoList( const PICKED_ITEMS_LIST& aItemsLis
                                               UNDO_REDO aCommandType )
 {
     PICKED_ITEMS_LIST* commandToUndo = new PICKED_ITEMS_LIST();
+    commandToUndo->SetDescription( aItemsList.GetDescription() );
 
     saveCopyInUndoList( commandToUndo, aItemsList, aCommandType );
 }
@@ -304,7 +185,10 @@ void PCB_BASE_EDIT_FRAME::AppendCopyToUndoList( const PICKED_ITEMS_LIST& aItemsL
     PICKED_ITEMS_LIST* commandToUndo = PopCommandFromUndoList();
 
     if( !commandToUndo )
+    {
         commandToUndo = new PICKED_ITEMS_LIST();
+        commandToUndo->SetDescription( aItemsList.GetDescription() );
+    }
 
     saveCopyInUndoList( commandToUndo, aItemsList, aCommandType );
 }
@@ -349,7 +233,7 @@ void PCB_BASE_EDIT_FRAME::RestoreCopyFromRedoList( wxCommandEvent& aEvent )
         return;
 
     // Inform tools that redo command was issued
-    m_toolManager->ProcessEvent( { TC_MESSAGE, TA_UNDO_REDO_PRE, AS_GLOBAL } );
+    m_toolManager->ProcessEvent( EVENTS::UndoRedoPreEvent );
 
     // Get the old list
     PICKED_ITEMS_LIST* list = PopCommandFromRedoList();
@@ -363,7 +247,7 @@ void PCB_BASE_EDIT_FRAME::RestoreCopyFromRedoList( wxCommandEvent& aEvent )
 
     OnModify();
 
-    m_toolManager->ProcessEvent( { TC_MESSAGE, TA_UNDO_REDO_POST, AS_GLOBAL } );
+    m_toolManager->ProcessEvent( EVENTS::UndoRedoPostEvent );
     m_toolManager->PostEvent( EVENTS::SelectedItemsModified );
 
     GetCanvas()->Refresh();
@@ -380,15 +264,83 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
     auto view = GetCanvas()->GetView();
     auto connectivity = GetBoard()->GetConnectivity();
 
-    PCB_GROUP* group = nullptr;
-
     GetBoard()->IncrementTimeStamp();   // clear caches
 
+    // Enum to track the modification type of items. Used to enable bulk BOARD_LISTENER
+    // callbacks at the end of the undo / redo operation
+    enum ITEM_CHANGE_TYPE
+    {
+        ADDED,
+        DELETED,
+        CHANGED
+    };
+
+    std::unordered_map<EDA_ITEM*, ITEM_CHANGE_TYPE> item_changes;
+
+    auto update_item_change_state = [&]( EDA_ITEM* item, ITEM_CHANGE_TYPE change_type )
+    {
+        auto item_itr = item_changes.find( item );
+
+        if( item_itr == item_changes.end() )
+        {
+            // First time we've seen this item - tag the current change type
+            item_changes.insert( { item, change_type } );
+        }
+        else
+        {
+            // Update the item state based on the current and next change type
+            switch( item_itr->second )
+            {
+            case ITEM_CHANGE_TYPE::ADDED:
+            {
+                if( change_type == ITEM_CHANGE_TYPE::DELETED )
+                {
+                    // The item was previously added, now deleted - as far as bulk callbacks
+                    // are concerned, the item has never existed
+                    item_changes.erase( item_itr );
+                }
+                else if( change_type == ITEM_CHANGE_TYPE::ADDED )
+                {
+                    // Error condition - added an already added item
+                    wxASSERT_MSG( false, wxT( "Undo / Redo - should not add already added item" ) );
+                }
+
+                // For all other cases, the item remains as ADDED as seen by the bulk callbacks
+                break;
+            }
+            case ITEM_CHANGE_TYPE::DELETED:
+            {
+                // This is an error condition - item has already been deleted so should not
+                // be operated on further
+                wxASSERT_MSG( false, wxT( "Undo / Redo - should not alter already deleted item" ) );
+                break;
+            }
+            case ITEM_CHANGE_TYPE::CHANGED:
+            {
+                if( change_type == ITEM_CHANGE_TYPE::DELETED )
+                {
+                    item_itr->second = ITEM_CHANGE_TYPE::DELETED;
+                }
+                else if( change_type == ITEM_CHANGE_TYPE::ADDED )
+                {
+                    // This is an error condition - item has already been changed so should not
+                    // be added
+                    wxASSERT_MSG( false,
+                                  wxT( "Undo / Redo - should not add already changed item" ) );
+                }
+
+                // Otherwise, item remains CHANGED
+                break;
+            }
+            }
+        }
+    };
+
     // Undo in the reverse order of list creation: (this can allow stacked changes
     // like the same item can be changes and deleted in the same complex command
 
     // Restore changes in reverse order
-    for( int ii = aList->GetCount() - 1; ii >= 0 ; ii-- )
+    for( int ii = (int) aList->GetCount() - 1; ii >= 0 ; ii-- )
     {
         EDA_ITEM* eda_item = aList->GetPickedItem( (unsigned) ii );
 
@@ -402,11 +354,13 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
         UNDO_REDO status = aList->GetPickedItemStatus( ii );
 
         if( status != UNDO_REDO::DELETED
+                && status != UNDO_REDO::REGROUP
+                && status != UNDO_REDO::UNGROUP
                 && status != UNDO_REDO::DRILLORIGIN     // origin markers never on board
                 && status != UNDO_REDO::GRIDORIGIN      // origin markers never on board
                 && status != UNDO_REDO::PAGESETTINGS )  // nor are page settings proxy items
         {
-            if( !TestForExistingItem( GetBoard(), (BOARD_ITEM*) eda_item ) )
+            if( GetBoard()->GetItem( eda_item->m_Uuid ) == DELETED_BOARD_ITEM::GetInstance() )
             {
                 // Checking if it ever happens
                 wxASSERT_MSG( false, wxT( "Item in the undo buffer does not exist" ) );
@@ -479,19 +433,26 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
         case UNDO_REDO::CHANGED:    /* Exchange old and new data for each item */
         {
             BOARD_ITEM* item = (BOARD_ITEM*) eda_item;
+
+            if( item->GetParentFootprint() )
+            {
+                // We need the current item, which may be different from what was stored if
+                // we're multiple frames up the undo stack.
+                item = GetBoard()->GetItem( item->m_Uuid );
+            }
+
             BOARD_ITEM* image = (BOARD_ITEM*) aList->GetPickedItemLink( ii );
 
-            // Remove all pads/drawings/texts, as they become invalid
-            // for the VIEW after SwapItemData() called for footprints
             view->Remove( item );
             connectivity->Remove( item );
 
             item->SwapItemData( image );
 
-            if( item->Type() == PCB_GROUP_T )
-            {
-                group = static_cast<PCB_GROUP*>( item );
+            item->ClearFlags( UR_TRANSIENT );
+            image->SetFlags( UR_TRANSIENT );
 
+            if( PCB_GROUP* group = dynamic_cast<PCB_GROUP*>( item ) )
+            {
                 group->RunOnChildren( [&]( BOARD_ITEM* child )
                                       {
                                           child->SetParentGroup( group );
@@ -501,56 +462,81 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
             view->Add( item );
             view->Hide( item, false );
             connectivity->Add( item );
-            item->GetBoard()->OnItemChanged( item );
+            update_item_change_state( item, ITEM_CHANGE_TYPE::CHANGED );
             break;
         }
 
         case UNDO_REDO::NEWITEM:        /* new items are deleted */
             aList->SetPickedItemStatus( UNDO_REDO::DELETED, ii );
-            GetModel()->Remove( (BOARD_ITEM*) eda_item );
+            GetModel()->Remove( (BOARD_ITEM*) eda_item, REMOVE_MODE::BULK );
+            update_item_change_state( eda_item, ITEM_CHANGE_TYPE::DELETED );
 
             if( eda_item->Type() != PCB_NETINFO_T )
                 view->Remove( eda_item );
 
+            eda_item->SetFlags( UR_TRANSIENT );
+
             break;
 
         case UNDO_REDO::DELETED:    /* deleted items are put in List, as new items */
             aList->SetPickedItemStatus( UNDO_REDO::NEWITEM, ii );
-            GetModel()->Add( (BOARD_ITEM*) eda_item );
+
+            eda_item->ClearFlags( UR_TRANSIENT );
+
+            GetModel()->Add( (BOARD_ITEM*) eda_item, ADD_MODE::BULK_APPEND );
+            update_item_change_state( eda_item, ITEM_CHANGE_TYPE::ADDED );
 
             if( eda_item->Type() != PCB_NETINFO_T )
                 view->Add( eda_item );
 
-            if( eda_item->Type() == PCB_GROUP_T )
-                group = static_cast<PCB_GROUP*>( eda_item );
-
             break;
 
-        case UNDO_REDO::REGROUP:
+        case UNDO_REDO::REGROUP:    /* grouped items are ungrouped */
             aList->SetPickedItemStatus( UNDO_REDO::UNGROUP, ii );
-            static_cast<BOARD_ITEM*>( eda_item )->SetParentGroup( nullptr );
+
+            if( eda_item->IsBOARD_ITEM() )
+            {
+                BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( eda_item );
+
+                if( PCB_GROUP* group = boardItem->GetParentGroup() )
+                {
+                    aList->SetPickedItemGroupId( group->m_Uuid, ii );
+
+                    group->RemoveItem( boardItem );
+                }
+            }
+
             break;
 
-        case UNDO_REDO::UNGROUP:
+        case UNDO_REDO::UNGROUP:    /* ungrouped items are re-added to their previuos groups */
             aList->SetPickedItemStatus( UNDO_REDO::REGROUP, ii );
 
-            if( group )
-                group->AddItem( static_cast<BOARD_ITEM*>( eda_item ) );
+            if( eda_item->IsBOARD_ITEM() )
+            {
+                BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( eda_item );
+
+                PCB_GROUP* group = dynamic_cast<PCB_GROUP*>(
+                        GetBoard()->GetItem( aList->GetPickedItemGroupId( ii ) ) );
+
+                if( group )
+                    group->AddItem( boardItem );
+            }
 
             break;
 
         case UNDO_REDO::DRILLORIGIN:
         case UNDO_REDO::GRIDORIGIN:
         {
-            BOARD_ITEM* item = (BOARD_ITEM*) eda_item;
-            BOARD_ITEM* image = (BOARD_ITEM*) aList->GetPickedItemLink( ii );
+            // Warning: DRILLORIGIN and GRIDORIGIN undo/redo command create EDA_ITEMs
+            // that cannot be casted to BOARD_ITEMs
+            EDA_ITEM* image = aList->GetPickedItemLink( ii );
             VECTOR2D origin = image->GetPosition();
             image->SetPosition( eda_item->GetPosition() );
 
             if( aList->GetPickedItemStatus( ii ) == UNDO_REDO::DRILLORIGIN )
-                BOARD_EDITOR_CONTROL::DoSetDrillOrigin( view, this, item, origin );
+                BOARD_EDITOR_CONTROL::DoSetDrillOrigin( view, this, eda_item, origin );
             else
-                PCB_CONTROL::DoSetGridOrigin( view, this, item, origin );
+                PCB_CONTROL::DoSetGridOrigin( view, this, eda_item, origin );
 
             break;
         }
@@ -561,7 +547,7 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
             DS_PROXY_UNDO_ITEM  alt_item( this );
             DS_PROXY_UNDO_ITEM* item = static_cast<DS_PROXY_UNDO_ITEM*>( eda_item );
             item->Restore( this );
-            *item = alt_item;
+            *item = std::move( alt_item );
             break;
         }
 
@@ -578,7 +564,11 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
     if( IsType( FRAME_PCB_EDITOR ) )
     {
         if( reBuild_ratsnest || deep_reBuild_ratsnest )
+        {
+            // Connectivity may have changed; rebuild internal caches to remove stale items
+            GetBoard()->BuildConnectivity();
             Compile_Ratsnest( false );
+        }
 
         if( solder_mask_dirty )
             HideSolderMask();
@@ -588,6 +578,40 @@ void PCB_BASE_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
     selTool->RebuildSelection();
 
     GetBoard()->SanitizeNetcodes();
+
+    // Invoke bulk BOARD_LISTENER callbacks
+    std::vector<BOARD_ITEM*> added_items, deleted_items, changed_items;
+
+    for( auto& item_itr : item_changes )
+    {
+        switch( item_itr.second )
+        {
+        case ITEM_CHANGE_TYPE::ADDED:
+        {
+            added_items.push_back( static_cast<BOARD_ITEM*>( item_itr.first ) );
+            break;
+        }
+        case ITEM_CHANGE_TYPE::DELETED:
+        {
+            deleted_items.push_back( static_cast<BOARD_ITEM*>( item_itr.first ) );
+            break;
+        }
+        case ITEM_CHANGE_TYPE::CHANGED:
+        {
+            changed_items.push_back( static_cast<BOARD_ITEM*>( item_itr.first ) );
+            break;
+        }
+        }
+    }
+
+    if( added_items.size() > 0 )
+        GetBoard()->FinalizeBulkAdd( added_items );
+
+    if( deleted_items.size() > 0 )
+        GetBoard()->FinalizeBulkRemove( deleted_items );
+
+    if( changed_items.size() > 0 )
+        GetBoard()->OnItemsChanged( changed_items );
 }
 
 
@@ -596,32 +620,41 @@ void PCB_BASE_EDIT_FRAME::ClearUndoORRedoList( UNDO_REDO_LIST whichList, int aIt
     if( aItemCount == 0 )
         return;
 
-    UNDO_REDO_CONTAINER& list = whichList == UNDO_LIST ? m_undoList : m_redoList;
-    unsigned             icnt = list.m_CommandsList.size();
+    UNDO_REDO_CONTAINER& list = ( whichList == UNDO_LIST ) ? m_undoList : m_redoList;
 
-    if( aItemCount > 0 )
-        icnt = aItemCount;
-
-    for( unsigned ii = 0; ii < icnt; ii++ )
+    if( aItemCount < 0 )
     {
-        if( list.m_CommandsList.size() == 0 )
-            break;
+        list.ClearCommandList();
+    }
+    else
+    {
+        for( int ii = 0; ii < aItemCount; ii++ )
+        {
+            if( list.m_CommandsList.size() == 0 )
+                break;
 
-        PICKED_ITEMS_LIST* curr_cmd = list.m_CommandsList[0];
-        list.m_CommandsList.erase( list.m_CommandsList.begin() );
-        ClearListAndDeleteItems( curr_cmd );
-        delete curr_cmd;    // Delete command
+            PICKED_ITEMS_LIST* curr_cmd = list.m_CommandsList[0];
+            list.m_CommandsList.erase( list.m_CommandsList.begin() );
+            ClearListAndDeleteItems( curr_cmd );
+            delete curr_cmd;    // Delete command
+        }
     }
 }
 
 
 void PCB_BASE_EDIT_FRAME::ClearListAndDeleteItems( PICKED_ITEMS_LIST* aList )
 {
-    aList->ClearListAndDeleteItems( []( EDA_ITEM* item )
-                                    {
-                                        static_cast<BOARD_ITEM*>( item )->SetParentGroup( nullptr );
-                                        delete item;
-                                    } );
+    aList->ClearListAndDeleteItems(
+            []( EDA_ITEM* item )
+            {
+                wxASSERT_MSG( item->HasFlag( UR_TRANSIENT ),
+                              "Item on undo/redo list not owned by undo/redo!" );
+
+                if( item->IsBOARD_ITEM() )
+                    static_cast<BOARD_ITEM*>( item )->SetParentGroup( nullptr );
+
+                delete item;
+            } );
 }
 
 
