@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -67,22 +67,11 @@ struct ISOLATED_ISLANDS
  *
  * A zone is described by a main polygon, a time stamp, a layer or a layer set, and a net name.
  * Other polygons inside the main polygon are holes in the zone.
- *
- * a item ZONE is living in a board
- * a variant FP_ZONE is living in a footprint
  */
 class ZONE : public BOARD_CONNECTED_ITEM
 {
 public:
-
-    /**
-     * The ctor to build ZONE, but compatible with FP_ZONE requirement.
-     * if aInFP is true, a FP_ZONE is actually built
-     * (same item, but with a specific type id:
-     * The type is PCB_ZONE_T for a ZONE
-     * The type is PCB_FP_ZONE_T for a FP_ZONE
-     */
-    ZONE( BOARD_ITEM_CONTAINER* parent, bool aInFP = false );
+    ZONE( BOARD_ITEM_CONTAINER* parent );
 
     ZONE( const ZONE& aZone );
     ZONE& operator=( const ZONE &aOther );
@@ -139,7 +128,14 @@ public:
     void SetLayerSet( LSET aLayerSet ) override;
     virtual LSET GetLayerSet() const override { return m_layerSet; }
 
-    wxString GetZoneName() const { return m_zoneName; }
+    /**
+     * Set the zone to be on the aLayerSet layers and only remove the fill polygons
+     * from the unused layers, while keeping the fills on the layers in both the old
+     * and new layer sets.
+     */
+    void SetLayerSetAndRemoveUnusedFills( LSET aLayerSet );
+
+    const wxString& GetZoneName() const { return m_zoneName; }
     void SetZoneName( const wxString& aName ) { m_zoneName = aName; }
 
     bool Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const override
@@ -604,17 +600,6 @@ public:
     ZONE_BORDER_DISPLAY_STYLE GetHatchStyle() const { return m_borderStyle; }
     void SetHatchStyle( ZONE_BORDER_DISPLAY_STYLE aStyle ) { m_borderStyle = aStyle; }
 
-    /**
-     * Test if 2 zones are equivalent.
-     *
-     * Zones are equivalent if they have same parameters and same outline info.
-     *
-     * @note Filling is not taken into account.
-     *
-     * @param aZoneToCompare is the zone to compare with "this"
-     */
-    bool IsSame( const ZONE &aZoneToCompare );
-
     bool HasFilledPolysForLayer( PCB_LAYER_ID aLayer ) const
     {
         return m_FilledPolysList.count( aLayer ) > 0;
@@ -732,7 +717,7 @@ public:
     void SetDoNotAllowPads( bool aEnable )       { m_doNotAllowPads = aEnable; }
     void SetDoNotAllowFootprints( bool aEnable ) { m_doNotAllowFootprints = aEnable; }
 
-    const ISLAND_REMOVAL_MODE GetIslandRemovalMode() const   { return m_islandRemovalMode; }
+    ISLAND_REMOVAL_MODE GetIslandRemovalMode() const   { return m_islandRemovalMode; }
     void SetIslandRemovalMode( ISLAND_REMOVAL_MODE aRemove ) { m_islandRemovalMode = aRemove; }
 
     long long int GetMinIslandArea() const       { return m_minIslandArea; }
@@ -795,6 +780,10 @@ public:
      */
     MD5_HASH GetHashValue( PCB_LAYER_ID aLayer );
 
+    double Similarity( const BOARD_ITEM& aOther ) const override;
+
+    bool operator==( const BOARD_ITEM& aOther ) const override;
+
 #if defined(DEBUG)
     virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
 
@@ -927,24 +916,10 @@ protected:
 };
 
 
-/**
- * A specialization of ZONE for use in footprints.
- */
-class FP_ZONE : public ZONE
-{
-public:
-    FP_ZONE( BOARD_ITEM_CONTAINER* aParent );
-    FP_ZONE( const FP_ZONE& aZone );
-    FP_ZONE& operator=( const FP_ZONE &aOther );
-
-    EDA_ITEM* Clone() const override;
-
-    double ViewGetLOD( int aLayer, KIGFX::VIEW* aView ) const override;
-};
-
 #ifndef SWIG
 DECLARE_ENUM_TO_WXANY( ZONE_CONNECTION )
 DECLARE_ENUM_TO_WXANY( ZONE_FILL_MODE )
+DECLARE_ENUM_TO_WXANY( ISLAND_REMOVAL_MODE )
 #endif
 
 #endif  // ZONE_H
