@@ -35,8 +35,6 @@
 #include <pcb_track.h>
 #include <pcb_text.h>
 #include <pcb_textbox.h>
-#include <fp_text.h>
-#include <fp_textbox.h>
 #include <connectivity/connectivity_data.h>
 #include <convert_basic_shapes_to_polygon.h>
 #include <board_commit.h>
@@ -45,9 +43,10 @@
 #include <geometry/convex_hull.h>
 #include <geometry/geometry_utils.h>
 #include <confirm.h>
-#include <thread_pool.h>
+#include <core/thread_pool.h>
 #include <math/util.h>      // for KiROUND
 #include "zone_filler.h"
+#include "pcb_dimension.h"
 
 
 ZONE_FILLER::ZONE_FILLER(  BOARD* aBoard, COMMIT* aCommit ) :
@@ -100,8 +99,6 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
     connectivity->ClearRatsnest();
     connectivity->Build( m_board, m_progressReporter );
 
-    BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
-
     m_worstClearance = m_board->GetMaxClearanceValue();
 
     if( m_progressReporter )
@@ -138,6 +135,7 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
 
         // Rules may depend on insideCourtyard() or other expressions
         footprint->BuildCourtyardCaches();
+        footprint->BuildNetTieCache();
     }
 
     LSET boardCuMask = m_board->GetEnabledLayers() & LSET::AllCuMask();
@@ -611,12 +609,15 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
 
     for( ZONE* zone : aZones )
     {
-        LSET   zoneCopperLayers = zone->GetLayerSet() & LSET::AllCuMask( MAX_CU_LAYERS );
+        // Don't check for connections on layers that only exist in the zone but
+        // were disabled in the board
+        BOARD* board = zone->GetBoard();
+        LSET zoneCopperLayers = zone->GetLayerSet() & LSET::AllCuMask() & board->GetEnabledLayers();
 
         // Min-thickness is the web thickness.  On the other hand, a blob min-thickness by
         // min-thickness is not useful.  Since there's no obvious definition of web vs. blob, we
-        // arbitrarily choose "at least 2X the area".
-        double minArea = (double) zone->GetMinThickness() * zone->GetMinThickness() * 2;
+        // arbitrarily choose "at least 3X the area".
+        double minArea = (double) zone->GetMinThickness() * zone->GetMinThickness() * 3;
 
         for( PCB_LAYER_ID layer : zoneCopperLayers.Seq() )
         {
@@ -809,28 +810,7 @@ void ZONE_FILLER::addKnockout( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer, int aGap,
 {
     switch( aItem->Type() )
     {
-    case PCB_FP_TEXT_T:
-    {
-        FP_TEXT* text = static_cast<FP_TEXT*>( aItem );
-
-        if( text->IsVisible() )
-        {
-            if( text->IsKnockout() )
-            {
-                int antiGap = -m_maxError * 2;   // Don't leave gaps around knockout text
-                text->TransformShapeToPolygon( aHoles, aLayer, antiGap, m_maxError, ERROR_OUTSIDE,
-                                               aIgnoreLineWidth );
-            }
-            else
-            {
-                text->TransformShapeToPolygon( aHoles, aLayer, aGap, m_maxError, ERROR_OUTSIDE,
-                                               aIgnoreLineWidth );
-            }
-        }
-
-        break;
-    }
-
+    case PCB_FIELD_T:
     case PCB_TEXT_T:
     {
         PCB_TEXT* text = static_cast<PCB_TEXT*>( aItem );
@@ -839,8 +819,11 @@ void ZONE_FILLER::addKnockout( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer, int aGap,
         {
             if( text->IsKnockout() )
             {
-                int antiGap = -m_maxError * 2;   // Don't leave gaps around knockout text
-                text->TransformShapeToPolygon( aHoles, aLayer, antiGap, m_maxError, ERROR_OUTSIDE );
+                // Knockout text should only leave holes where the text is, not where the copper fill
+                // around it would be.
+                PCB_TEXT textCopy = *text;
+                textCopy.SetIsKnockout( false );
+                textCopy.TransformShapeToPolygon( aHoles, aLayer, 0, m_maxError, ERROR_OUTSIDE );
             }
             else
             {
@@ -851,16 +834,6 @@ void ZONE_FILLER::addKnockout( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer, int aGap,
         break;
     }
 
-    case PCB_FP_TEXTBOX_T:
-    {
-        FP_TEXTBOX* textbox = static_cast<FP_TEXTBOX*>( aItem );
-
-        if( textbox->IsVisible() )
-            textbox->TransformShapeToPolygon( aHoles, aLayer, aGap, m_maxError, ERROR_OUTSIDE );
-
-        break;
-    }
-
     case PCB_TEXTBOX_T:
     {
         PCB_TEXTBOX* textbox = static_cast<PCB_TEXTBOX*>( aItem );
@@ -871,10 +844,27 @@ void ZONE_FILLER::addKnockout( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer, int aGap,
         break;
     }
 
-    default:
+    case PCB_SHAPE_T:
+    case PCB_TARGET_T:
         aItem->TransformShapeToPolygon( aHoles, aLayer, aGap, m_maxError, ERROR_OUTSIDE,
                                         aIgnoreLineWidth );
         break;
+
+    case PCB_DIM_ALIGNED_T:
+    case PCB_DIM_LEADER_T:
+    case PCB_DIM_CENTER_T:
+    case PCB_DIM_RADIAL_T:
+    case PCB_DIM_ORTHOGONAL_T:
+    {
+        PCB_DIMENSION_BASE* dim = static_cast<PCB_DIMENSION_BASE*>( aItem );
+
+        dim->TransformShapeToPolygon( aHoles, aLayer, aGap, m_maxError, ERROR_OUTSIDE, false );
+        dim->PCB_TEXT::TransformShapeToPolygon( aHoles, aLayer, aGap, m_maxError, ERROR_OUTSIDE );
+        break;
+    }
+
+    default:
+        break;
     }
 }
 
@@ -892,6 +882,7 @@ void ZONE_FILLER::knockoutThermalReliefs( const ZONE* aZone, PCB_LAYER_ID aLayer
     ZONE_CONNECTION        connection;
     DRC_CONSTRAINT         constraint;
     int                    padClearance;
+    std::shared_ptr<SHAPE> padShape;
     int                    holeClearance;
     SHAPE_POLY_SET         holes;
 
@@ -905,9 +896,18 @@ void ZONE_FILLER::knockoutThermalReliefs( const ZONE* aZone, PCB_LAYER_ID aLayer
             if( !padBBox.Intersects( aZone->GetBoundingBox() ) )
                 continue;
 
-            if( pad->GetNetCode() != aZone->GetNetCode()
-                || pad->GetNetCode() <= 0
-                || pad->GetZoneLayerOverride( aLayer ) == ZLO_FORCE_NO_ZONE_CONNECTION )
+            bool noConnection = pad->GetNetCode() != aZone->GetNetCode();
+
+            if( !aZone->IsTeardropArea() )
+            {
+                if( aZone->GetNetCode() == 0
+                    || pad->GetZoneLayerOverride( aLayer ) == ZLO_FORCE_NO_ZONE_CONNECTION )
+                {
+                    noConnection = true;
+                }
+            }
+
+            if( noConnection )
             {
                 // collect these for knockout in buildCopperItemClearances()
                 aNoConnectionPads.push_back( pad );
@@ -924,22 +924,23 @@ void ZONE_FILLER::knockoutThermalReliefs( const ZONE* aZone, PCB_LAYER_ID aLayer
                 connection = constraint.m_ZoneConnection;
             }
 
+            if( connection == ZONE_CONNECTION::THERMAL && !pad->CanFlashLayer( aLayer ) )
+                connection = ZONE_CONNECTION::NONE;
+
             switch( connection )
             {
             case ZONE_CONNECTION::THERMAL:
-                constraint = bds.m_DRCEngine->EvalRules( THERMAL_RELIEF_GAP_CONSTRAINT, pad, aZone,
-                                                         aLayer );
-                padClearance = constraint.GetValue().Min();
+                padShape = pad->GetEffectiveShape( aLayer, FLASHING::ALWAYS_FLASHED );
 
-                if( pad->CanFlashLayer( aLayer ) )
+                if( aFill.Collide( padShape.get(), 0 ) )
                 {
+                    constraint = bds.m_DRCEngine->EvalRules( THERMAL_RELIEF_GAP_CONSTRAINT, pad,
+                                                             aZone, aLayer );
+                    padClearance = constraint.GetValue().Min();
+
                     aThermalConnectionPads.push_back( pad );
                     addKnockout( pad, aLayer, padClearance, holes );
                 }
-                else if( pad->GetDrillSize().x > 0 )
-                {
-                    pad->TransformHoleToPolygon( holes, padClearance, m_maxError, ERROR_OUTSIDE );
-                }
 
                 break;
 
@@ -1022,7 +1023,8 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
     auto knockoutPadClearance =
             [&]( PAD* aPad )
             {
-                int  gap = evalRulesForItems( PHYSICAL_CLEARANCE_CONSTRAINT, aZone, aPad, aLayer );
+                int  init_gap = evalRulesForItems( PHYSICAL_CLEARANCE_CONSTRAINT, aZone, aPad, aLayer );
+                int  gap = init_gap;
                 bool hasHole = aPad->GetDrillSize().x > 0;
                 bool flashLayer = aPad->FlashLayer( aLayer );
                 bool platedHole = hasHole && aPad->GetAttribute() == PAD_ATTRIB::PTH;
@@ -1038,6 +1040,10 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
 
                 if( hasHole )
                 {
+                    // NPTH do not need copper clearance gaps to their holes
+                    if( aPad->GetAttribute() == PAD_ATTRIB::NPTH )
+                        gap = init_gap;
+
                     gap = std::max( gap, evalRulesForItems( PHYSICAL_HOLE_CLEARANCE_CONSTRAINT,
                                                             aZone, aPad, aLayer ) );
 
@@ -1064,8 +1070,10 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
             {
                 if( aTrack->GetBoundingBox().Intersects( zone_boundingbox ) )
                 {
-                    bool sameNet = aTrack->GetNetCode() == aZone->GetNetCode()
-                                        && aZone->GetNetCode() != 0;
+                    bool sameNet = aTrack->GetNetCode() == aZone->GetNetCode();
+
+                    if( !aZone->IsTeardropArea() && aZone->GetNetCode() == 0 )
+                        sameNet = false;
 
                     int  gap = evalRulesForItems( PHYSICAL_CLEARANCE_CONSTRAINT,
                                                   aZone, aTrack, aLayer );
@@ -1134,12 +1142,21 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
         knockoutTrackClearance( track );
     }
 
-    // Add graphic item clearances.  They are by definition unconnected, and have no clearance
-    // definitions of their own.
+    // Add graphic item clearances.
     //
     auto knockoutGraphicClearance =
             [&]( BOARD_ITEM* aItem )
             {
+                int shapeNet = -1;
+
+                if( aItem->Type() == PCB_SHAPE_T )
+                    shapeNet = static_cast<PCB_SHAPE*>( aItem )->GetNetCode();
+
+                bool sameNet = shapeNet == aZone->GetNetCode();
+
+                if( !aZone->IsTeardropArea() && aZone->GetNetCode() == 0 )
+                    sameNet = false;
+
                 // A item on the Edge_Cuts or Margin is always seen as on any layer:
                 if( aItem->IsOnLayer( aLayer )
                         || aItem->IsOnLayer( Edge_Cuts )
@@ -1151,7 +1168,7 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                         int  gap = evalRulesForItems( PHYSICAL_CLEARANCE_CONSTRAINT,
                                                       aZone, aItem, aLayer );
 
-                        if( aItem->IsOnLayer( aLayer ) )
+                        if( aItem->IsOnLayer( aLayer ) && !sameNet )
                         {
                             gap = std::max( gap, evalRulesForItems( CLEARANCE_CONSTRAINT,
                                                                     aZone, aItem, aLayer ) );
@@ -1159,16 +1176,17 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                         else if( aItem->IsOnLayer( Edge_Cuts ) )
                         {
                             gap = std::max( gap, evalRulesForItems( EDGE_CLEARANCE_CONSTRAINT,
-                                                                    aZone, aItem, Edge_Cuts ) );
+                                                                    aZone, aItem, aLayer ) );
                             ignoreLineWidths = true;
                         }
                         else if( aItem->IsOnLayer( Margin ) )
                         {
                             gap = std::max( gap, evalRulesForItems( EDGE_CLEARANCE_CONSTRAINT,
-                                                                    aZone, aItem, Margin ) );
+                                                                    aZone, aItem, aLayer ) );
                         }
 
-                        addKnockout( aItem, aLayer, gap + extra_margin, ignoreLineWidths, aHoles );
+                        if( gap > 0 )
+                            addKnockout( aItem, aLayer, gap + extra_margin, ignoreLineWidths, aHoles );
                     }
                 }
             };
@@ -1186,7 +1204,12 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
         {
             for( PAD* pad : footprint->Pads() )
             {
-                if( pad->GetNetCode() == aZone->GetNetCode() )
+                bool sameNet = pad->GetNetCode() == aZone->GetNetCode();
+
+                if( !aZone->IsTeardropArea() && aZone->GetNetCode() == 0 )
+                    sameNet = false;
+
+                if( sameNet )
                 {
                     if( pad->IsOnLayer( aLayer ) )
                         allowedNetTiePads.insert( pad );
@@ -1408,7 +1431,6 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
     // deflating/inflating.
     int half_min_width = aZone->GetMinThickness() / 2;
     int epsilon = pcbIUScale.mmToIU( 0.001 );
-    int numSegs = GetArcToSegmentCount( half_min_width, m_maxError, FULL_CIRCLE );
 
     // Solid polygons are deflated and inflated during calculations.  Deflating doesn't cause
     // issues, but inflate is tricky as it can create excessively long and narrow spikes for
@@ -1420,8 +1442,8 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
     // as a "less-safe" option.
     // ROUND_ALL_CORNERS produces the uniformly nicest shapes, but also a lot of segments.
     // CHAMFER_ALL_CORNERS improves the segment count.
-    SHAPE_POLY_SET::CORNER_STRATEGY fastCornerStrategy = SHAPE_POLY_SET::CHAMFER_ALL_CORNERS;
-    SHAPE_POLY_SET::CORNER_STRATEGY cornerStrategy = SHAPE_POLY_SET::ROUND_ALL_CORNERS;
+    CORNER_STRATEGY fastCornerStrategy = CORNER_STRATEGY::CHAMFER_ALL_CORNERS;
+    CORNER_STRATEGY cornerStrategy = CORNER_STRATEGY::ROUND_ALL_CORNERS;
 
     std::vector<PAD*>            thermalConnectionPads;
     std::vector<PAD*>            noConnectionPads;
@@ -1473,10 +1495,10 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
     // Prune features that don't meet minimum-width criteria
     if( half_min_width - epsilon > epsilon )
     {
-        testAreas.Deflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
+        testAreas.Deflate( half_min_width - epsilon, fastCornerStrategy, m_maxError );
         DUMP_POLYS_TO_COPPER_LAYER( testAreas, In5_Cu, wxT( "spoke-test-deflated" ) );
 
-        testAreas.Inflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
+        testAreas.Inflate( half_min_width - epsilon, fastCornerStrategy, m_maxError );
         DUMP_POLYS_TO_COPPER_LAYER( testAreas, In6_Cu, wxT( "spoke-test-reinflated" ) );
     }
 
@@ -1543,17 +1565,26 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
      */
 
     if( half_min_width - epsilon > epsilon )
-        aFillPolys.Deflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
+        aFillPolys.Deflate( half_min_width - epsilon, fastCornerStrategy, m_maxError );
 
     // Min-thickness is the web thickness.  On the other hand, a blob min-thickness by
     // min-thickness is not useful.  Since there's no obvious definition of web vs. blob, we
-    // arbitrarily choose "at least 1/2 min-thickness on one axis".
+    // arbitrarily choose "at least 2X min-thickness on one axis".  (Since we're doing this
+    // during the deflated state, that means we test for "at least min-thickness".)
     for( int ii = aFillPolys.OutlineCount() - 1; ii >= 0; ii-- )
     {
         std::vector<SHAPE_LINE_CHAIN>& island = aFillPolys.Polygon( ii );
-        BOX2I                          islandExtents = island.front().BBox();
+        BOX2I                          islandExtents;
 
-        if( islandExtents.GetSizeMax() < half_min_width )
+        for( const VECTOR2I& pt : island.front().CPoints() )
+        {
+            islandExtents.Merge( pt );
+
+            if( islandExtents.GetSizeMax() > aZone->GetMinThickness() )
+                break;
+        }
+
+        if( islandExtents.GetSizeMax() < aZone->GetMinThickness() )
             aFillPolys.DeletePolygon( ii );
     }
 
@@ -1580,7 +1611,7 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
      */
 
     if( half_min_width - epsilon > epsilon )
-        aFillPolys.Inflate( half_min_width - epsilon, numSegs, cornerStrategy, true );
+        aFillPolys.Inflate( half_min_width - epsilon, cornerStrategy, m_maxError, true );
 
     DUMP_POLYS_TO_COPPER_LAYER( aFillPolys, In15_Cu, wxT( "after-reinflating" ) );
 
@@ -1661,14 +1692,25 @@ bool ZONE_FILLER::fillNonCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer,
     aFillPolys = aSmoothedOutline;
     aFillPolys.BooleanSubtract( clearanceHoles, SHAPE_POLY_SET::PM_FAST );
 
+    for( ZONE* keepout : m_board->Zones() )
+    {
+        if( !keepout->GetIsRuleArea() )
+            continue;
+
+        if( keepout->GetDoNotAllowCopperPour() && keepout->IsOnLayer( aLayer ) )
+        {
+            if( keepout->GetBoundingBox().Intersects( zone_boundingbox ) )
+                aFillPolys.BooleanSubtract( *keepout->Outline(), SHAPE_POLY_SET::PM_FAST );
+        }
+    }
+
     // Features which are min_width should survive pruning; features that are *less* than
     // min_width should not.  Therefore we subtract epsilon from the min_width when
     // deflating/inflating.
     int half_min_width = aZone->GetMinThickness() / 2;
     int epsilon = pcbIUScale.mmToIU( 0.001 );
-    int numSegs = GetArcToSegmentCount( half_min_width, m_maxError, FULL_CIRCLE );
 
-    aFillPolys.Deflate( half_min_width - epsilon, numSegs );
+    aFillPolys.Deflate( half_min_width - epsilon, CORNER_STRATEGY::CHAMFER_ALL_CORNERS, m_maxError );
 
     // Remove the non filled areas due to the hatch pattern
     if( aZone->GetFillMode() == ZONE_FILL_MODE::HATCH_PATTERN )
@@ -1679,7 +1721,7 @@ bool ZONE_FILLER::fillNonCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer,
 
     // Re-inflate after pruning of areas that don't meet minimum-width criteria
     if( half_min_width - epsilon > epsilon )
-        aFillPolys.Inflate( half_min_width - epsilon, numSegs );
+        aFillPolys.Inflate( half_min_width - epsilon, CORNER_STRATEGY::ROUND_ALL_CORNERS, m_maxError );
 
     aFillPolys.Fracture( SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
     return true;
@@ -1735,12 +1777,13 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
     BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
     BOX2I                  zoneBB = aZone->GetBoundingBox();
     DRC_CONSTRAINT         constraint;
+    int                    zone_half_width = aZone->GetMinThickness() / 2;
 
     zoneBB.Inflate( std::max( bds.GetBiggestClearanceValue(), aZone->GetLocalClearance() ) );
 
-    // Is a point on the boundary of the polygon inside or outside?  This small epsilon lets
-    // us avoid the question.
-    int epsilon = KiROUND( pcbIUScale.IU_PER_MM * 0.04 ); // about 1.5 mil
+    // Is a point on the boundary of the polygon inside or outside?
+    // The boundary may be off by MaxError
+    int epsilon = bds.m_MaxError;
 
     for( PAD* pad : aSpokedPadsList )
     {
@@ -1759,8 +1802,7 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
         // and the algo to count the actual number of spokes can fail
         int spoke_max_allowed_w = std::min( pad->GetSize().x, pad->GetSize().y );
 
-        spoke_w = std::max( spoke_w, constraint.Value().Min() );
-        spoke_w = std::min( spoke_w, constraint.Value().Max() );
+        spoke_w = alg::clamp( constraint.Value().Min(), spoke_w, constraint.Value().Max() );
 
         // ensure the spoke width is smaller than the pad minor size
         spoke_w = std::min( spoke_w, spoke_max_allowed_w );
@@ -1778,110 +1820,135 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
         if( !( itemBB.Intersects( zoneBB ) ) )
             continue;
 
+        bool customSpokes = false;
+
+        if( pad->GetShape() == PAD_SHAPE::CUSTOM )
+        {
+            for( const std::shared_ptr<PCB_SHAPE>& primitive : pad->GetPrimitives() )
+            {
+                if( primitive->IsProxyItem() && primitive->GetShape() == SHAPE_T::SEGMENT )
+                {
+                    customSpokes = true;
+                    break;
+                }
+            }
+        }
+
         // Thermal spokes consist of square-ended segments from the pad center to points just
         // outside the thermal relief.  The outside end has an extra center point (which must be
         // at idx 3) which is used for testing whether or not the spoke connects to copper in the
         // parent zone.
 
         auto buildSpokesFromOrigin =
-                [&]( const BOX2I& box )
+                [&]( const BOX2I& box, EDA_ANGLE angle )
                 {
-                    for( int i = 0; i < 4; i++ )
-                    {
-                        SHAPE_LINE_CHAIN spoke;
+                    VECTOR2I center = box.GetCenter();
+                    VECTOR2I half_size( box.GetWidth() / 2, box.GetHeight() / 2 );
 
-                        switch( i )
+                    // Function to find intersection of line with box edge
+                    auto intersectLineBox = [&](const VECTOR2D& direction) -> VECTOR2I {
+                        double dx = direction.x;
+                        double dy = direction.y;
+
+                        // Shortcircuit the axis cases because they will be degenerate in the
+                        // intersection test
+                        if( direction.x == 0 )
                         {
-                        case 0:       // lower stub
-                            spoke.Append( +spoke_half_w, -spoke_half_w );
-                            spoke.Append( -spoke_half_w, -spoke_half_w );
-                            spoke.Append( -spoke_half_w, box.GetBottom() );
-                            spoke.Append( 0,             box.GetBottom() );  // test pt
-                            spoke.Append( +spoke_half_w, box.GetBottom() );
-                            break;
-
-                        case 1:       // upper stub
-                            spoke.Append( +spoke_half_w, +spoke_half_w );
-                            spoke.Append( -spoke_half_w, +spoke_half_w );
-                            spoke.Append( -spoke_half_w, box.GetTop() );
-                            spoke.Append( 0,             box.GetTop() );     // test pt
-                            spoke.Append( +spoke_half_w, box.GetTop() );
-                            break;
-
-                        case 2:       // right stub
-                            spoke.Append( -spoke_half_w,  +spoke_half_w );
-                            spoke.Append( -spoke_half_w,  -spoke_half_w );
-                            spoke.Append( box.GetRight(), -spoke_half_w );
-                            spoke.Append( box.GetRight(), 0             );   // test pt
-                            spoke.Append( box.GetRight(), +spoke_half_w );
-                            break;
-
-                        case 3:       // left stub
-                            spoke.Append( +spoke_half_w, +spoke_half_w );
-                            spoke.Append( +spoke_half_w, -spoke_half_w );
-                            spoke.Append( box.GetLeft(), -spoke_half_w );
-                            spoke.Append( box.GetLeft(), 0             );    // test pt
-                            spoke.Append( box.GetLeft(), +spoke_half_w );
-                            break;
+                            return VECTOR2I(0, dy * half_size.y );
+                        }
+                        else if( direction.y == 0 )
+                        {
+                            return VECTOR2I(dx * half_size.x, 0);
                         }
 
+                        // We are going to intersect with one side or the other.  Whichever
+                        // we hit first is the fraction of the spoke length we keep
+                        double tx = std::min( half_size.x / std::abs( dx ),
+                                              half_size.y / std::abs( dy ) );
+                        return VECTOR2I( dx * tx, dy * tx );
+                    };
+
+                    // Precalculate angles for four cardinal directions
+                    const EDA_ANGLE angles[4] = {
+                        EDA_ANGLE(  0.0, DEGREES_T ) + angle,  // Right
+                        EDA_ANGLE( 90.0, DEGREES_T ) + angle,  // Up
+                        EDA_ANGLE( 180.0, DEGREES_T ) + angle, // Left
+                        EDA_ANGLE( 270.0, DEGREES_T ) + angle  // Down
+                    };
+
+                    // Generate four spokes in cardinal directions
+                    for( const EDA_ANGLE& spokeAngle : angles )
+                    {
+                        VECTOR2D direction( spokeAngle.Cos(), spokeAngle.Sin() );
+                        VECTOR2D perpendicular = direction.Perpendicular();
+
+                        VECTOR2I intersection = intersectLineBox( direction );
+                        VECTOR2I spoke_side = perpendicular.Resize( spoke_half_w );
+
+                        SHAPE_LINE_CHAIN spoke;
+                        spoke.Append( center + spoke_side );
+                        spoke.Append( center - spoke_side );
+                        spoke.Append( center + intersection - spoke_side );
+                        spoke.Append( center + intersection ); // test pt
+                        spoke.Append( center + intersection + spoke_side );
                         spoke.SetClosed( true );
                         aSpokesList.push_back( std::move( spoke ) );
                     }
                 };
 
-        // If the spokes are at a cardinal angle then we can generate them from a bounding box
-        // without trig.
-        if( ( pad->GetOrientation() + pad->GetThermalSpokeAngle() ).IsCardinal() )
+        if( customSpokes )
         {
-            BOX2I spokesBox = pad->GetBoundingBox();
-            spokesBox.Inflate( thermalReliefGap + epsilon );
+            SHAPE_POLY_SET   thermalPoly;
+            SHAPE_LINE_CHAIN thermalOutline;
 
-            // Spokes are from center of pad shape, not from hole.
-            spokesBox.Offset( - pad->ShapePos() );
+            pad->TransformShapeToPolygon( thermalPoly, aLayer, thermalReliefGap + epsilon,
+                                          m_maxError, ERROR_OUTSIDE );
 
-            buildSpokesFromOrigin( spokesBox );
+            if( thermalPoly.OutlineCount() )
+                thermalOutline = thermalPoly.Outline( 0 );
 
-            auto spokeIter = aSpokesList.rbegin();
-
-            for( int ii = 0; ii < 4; ++ii, ++spokeIter )
-                spokeIter->Move( pad->ShapePos() );
-        }
-        // Even if the spokes are rotated, we can fudge it for round and square pads by rotating
-        // the bounding box to match the spokes.
-        else if( pad->GetSizeX() == pad->GetSizeY() && pad->GetShape() != PAD_SHAPE::CUSTOM )
-        {
-            // Since the bounding-box needs to be correclty rotated we use a dummy pad to keep
-            // from dirtying the real pad's cached shapes.
-            PAD dummy_pad( *pad );
-            dummy_pad.SetOrientation( pad->GetThermalSpokeAngle() );
-
-            // Spokes are from center of pad shape, not from hole. So the dummy pad has no shape
-            // offset and is at position 0,0
-            dummy_pad.SetPosition( VECTOR2I( 0, 0 ) );
-            dummy_pad.SetOffset( VECTOR2I( 0, 0 ) );
-
-            BOX2I spokesBox = dummy_pad.GetBoundingBox();
-            spokesBox.Inflate( thermalReliefGap + epsilon );
-
-            buildSpokesFromOrigin( spokesBox );
-
-            auto spokeIter = aSpokesList.rbegin();
-
-            for( int ii = 0; ii < 4; ++ii, ++spokeIter )
+            for( const std::shared_ptr<PCB_SHAPE>& primitive : pad->GetPrimitives() )
             {
-                spokeIter->Rotate( pad->GetOrientation() + pad->GetThermalSpokeAngle() );
-                spokeIter->Move( pad->ShapePos() );
+                if( primitive->IsProxyItem() && primitive->GetShape() == SHAPE_T::SEGMENT )
+                {
+                    SEG seg( primitive->GetStart(), primitive->GetEnd() );
+                    SHAPE_LINE_CHAIN::INTERSECTIONS intersections;
+
+                    RotatePoint( seg.A, pad->GetOrientation() );
+                    RotatePoint( seg.B, pad->GetOrientation() );
+                    seg.A += pad->ShapePos();
+                    seg.B += pad->ShapePos();
+
+                    // Make sure seg.A is the origin
+                    if( !pad->GetEffectivePolygon( ERROR_OUTSIDE )->Contains( seg.A ) )
+                        seg.Reverse();
+
+                    // Trim seg.B to the thermal outline
+                    if( thermalOutline.Intersect( seg, intersections ) )
+                    {
+                        seg.B = intersections.front().p;
+
+                        VECTOR2I offset = ( seg.B - seg.A ).Perpendicular().Resize( spoke_half_w );
+                        SHAPE_LINE_CHAIN spoke;
+
+                        spoke.Append( seg.A + offset );
+                        spoke.Append( seg.A - offset );
+                        spoke.Append( seg.B - offset );
+                        spoke.Append( seg.B          );  // test pt
+                        spoke.Append( seg.B + offset );
+
+                        spoke.SetClosed( true );
+                        aSpokesList.push_back( std::move( spoke ) );
+                    }
+                }
             }
         }
-        // And lastly, even when we have to resort to trig, we can use it only in a post-process
-        // after the rotated-bounding-box trick from above.
         else
         {
             // Since the bounding-box needs to be correclty rotated we use a dummy pad to keep
             // from dirtying the real pad's cached shapes.
             PAD dummy_pad( *pad );
-            dummy_pad.SetOrientation( pad->GetThermalSpokeAngle() );
+            dummy_pad.SetOrientation( ANGLE_0 );
 
             // Spokes are from center of pad shape, not from hole. So the dummy pad has no shape
             // offset and is at position 0,0
@@ -1890,36 +1957,40 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
 
             BOX2I spokesBox = dummy_pad.GetBoundingBox();
 
-            // In this case make the box -big-; we're going to clip to the "real" bbox later.
-            spokesBox.Inflate( thermalReliefGap + spokesBox.GetWidth() + spokesBox.GetHeight() );
+            //Add the half width of the zone mininum width to the inflate amount to account for the fact that
+            //the deflation procedure will shrink the results by half the half the zone min width
+            spokesBox.Inflate( thermalReliefGap + epsilon + zone_half_width );
 
-            buildSpokesFromOrigin( spokesBox );
+            // This is a touchy case because the bounding box for circles overshoots the mark
+            // when rotated at 45 degrees.  So we just build spokes at 0 degrees and rotate
+            // them later.
+            if( pad->GetShape() == PAD_SHAPE::CIRCLE
+                || ( pad->GetShape() == PAD_SHAPE::OVAL && pad->GetSizeX() == pad->GetSizeY() ) )
+            {
+                buildSpokesFromOrigin( spokesBox, ANGLE_0 );
 
-            BOX2I realBBox = pad->GetBoundingBox();
-            realBBox.Inflate( thermalReliefGap + epsilon );
+                if( pad->GetThermalSpokeAngle() != ANGLE_0 )
+                {
+                    //Rotate the last four elements of aspokeslist
+                    for( auto it = aSpokesList.rbegin(); it != aSpokesList.rbegin() + 4; ++it )
+                        it->Rotate( pad->GetThermalSpokeAngle() );
+                }
+            }
+            else
+            {
+                buildSpokesFromOrigin( spokesBox, pad->GetThermalSpokeAngle() );
+            }
 
             auto spokeIter = aSpokesList.rbegin();
 
             for( int ii = 0; ii < 4; ++ii, ++spokeIter )
             {
-                spokeIter->Rotate( pad->GetOrientation() + pad->GetThermalSpokeAngle() );
+                spokeIter->Rotate( pad->GetOrientation() );
                 spokeIter->Move( pad->ShapePos() );
-
-                VECTOR2I origin_p = spokeIter->GetPoint( 0 );
-                VECTOR2I origin_m = spokeIter->GetPoint( 1 );
-                VECTOR2I origin = ( origin_p + origin_m ) / 2;
-                VECTOR2I end_m = spokeIter->GetPoint( 2 );
-                VECTOR2I end = spokeIter->GetPoint( 3 );
-                VECTOR2I end_p = spokeIter->GetPoint( 4 );
-
-                ClipLine( &realBBox, origin_p.x, origin_p.y, end_p.x, end_p.y );
-                ClipLine( &realBBox, origin_m.x, origin_m.y, end_m.x, end_m.y );
-                ClipLine( &realBBox, origin.x, origin.y, end.x, end.y );
-
-                spokeIter->SetPoint( 2, end_m );
-                spokeIter->SetPoint( 3, end );
-                spokeIter->SetPoint( 4, end_p );
             }
+
+            // Remove group membership from dummy item before deleting
+            dummy_pad.SetParentGroup( nullptr );
         }
     }
 
@@ -1944,6 +2015,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
 
     int linethickness = thickness - aZone->GetMinThickness();
     int gridsize = thickness + aZone->GetHatchGap();
+    int maxError = m_board->GetDesignSettings().m_MaxError;
 
     SHAPE_POLY_SET filledPolys = aFillPolys.CloneDropTriangulation();
     // Use a area that contains the rotated bbox by orientation, and after rotate the result
@@ -2005,7 +2077,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
             smooth_value = std::min( smooth_value, aZone->GetHatchGap() / 2 );
 
             // the error to approximate a circle by segments when smoothing corners by a arc
-            int error_max = std::max( pcbIUScale.mmToIU( 0.01 ), smooth_value / 20 );
+            maxError = std::max( maxError * 2, smooth_value / 20 );
 
             switch( smooth_level )
             {
@@ -2017,9 +2089,9 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
 
             default:
                 if( aZone->GetHatchSmoothingLevel() > 2 )
-                    error_max /= 2;    // Force better smoothing
+                    maxError /= 2;    // Force better smoothing
 
-                hole_base = smooth_hole.Fillet( smooth_value, error_max ).Outline( 0 );
+                hole_base = smooth_hole.Fillet( smooth_value, maxError ).Outline( 0 );
                 break;
 
             case 0:
@@ -2068,12 +2140,13 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
     // The fill has already been deflated to ensure GetMinThickness() so we just have to
     // account for anything beyond that.
     SHAPE_POLY_SET deflatedFilledPolys = aFillPolys.CloneDropTriangulation();
-    deflatedFilledPolys.Deflate( outline_margin - aZone->GetMinThickness(), 16 );
+    deflatedFilledPolys.Deflate( outline_margin - aZone->GetMinThickness(),
+                                 CORNER_STRATEGY::CHAMFER_ALL_CORNERS, maxError );
     holes.BooleanIntersection( deflatedFilledPolys, SHAPE_POLY_SET::PM_FAST );
     DUMP_POLYS_TO_COPPER_LAYER( holes, In11_Cu, wxT( "fill-clipped-hatch-holes" ) );
 
     SHAPE_POLY_SET deflatedOutline = aZone->Outline()->CloneDropTriangulation();
-    deflatedOutline.Deflate( outline_margin, 16 );
+    deflatedOutline.Deflate( outline_margin, CORNER_STRATEGY::CHAMFER_ALL_CORNERS, maxError );
     holes.BooleanIntersection( deflatedOutline, SHAPE_POLY_SET::PM_FAST );
     DUMP_POLYS_TO_COPPER_LAYER( holes, In12_Cu, wxT( "outline-clipped-hatch-holes" ) );
 
@@ -2100,7 +2173,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
                     int r = std::max( min_apron_radius,
                                       via->GetDrillValue() / 2 + outline_margin );
 
-                    TransformCircleToPolygon( aprons, via->GetPosition(), r, ARC_HIGH_DEF,
+                    TransformCircleToPolygon( aprons, via->GetPosition(), r, maxError,
                                               ERROR_OUTSIDE );
                 }
             }
@@ -2126,7 +2199,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
                                               outline_margin - min_annular_ring_width );
 
                     clearance = std::max( 0, clearance - linethickness / 2 );
-                    pad->TransformShapeToPolygon( aprons, aLayer, clearance, ARC_HIGH_DEF,
+                    pad->TransformShapeToPolygon( aprons, aLayer, clearance, maxError,
                                                   ERROR_OUTSIDE );
                 }
             }
